Package: UCSCXenaShiny
Title: Interactive Analysis of UCSC Xena Data
Version: 2.2.1
Authors@R: c(
    person("Shixiang", "Wang", , "w_shixiang@163.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-9855-7357")),
    person("Shensuo", "Li", , "lishensuo@163.com", role = "aut"),
    person("Yi", "Xiong", , "neuroxiong@openbiox.com", role = "aut",
           comment = c(ORCID = "0000-0002-4370-9824")),
    person("Longfei", "Zhao", , "longfei8533@126.com", role = "aut",
           comment = c(ORCID = "0000-0002-6277-0137")),
    person("Kai", "Gu", , "gukai1212@163.com", role = "aut",
           comment = c(ORCID = "0000-0002-0177-0774")),
    person("Yin", "Li", , "yinli18@fudan.edu.cn", role = "aut"),
    person("Fei", "Zhao", , "zhaofei415@mails.ucas.edu.cn", role = "aut")
  )
Maintainer: Shixiang Wang <w_shixiang@163.com>
Description: Provides functions and a Shiny application for downloading,
    analyzing and visualizing datasets from UCSC Xena
    (<http://xena.ucsc.edu/>), which is a collection of UCSC-hosted public
    databases such as TCGA, ICGC, TARGET, GTEx, CCLE, and others.
License: GPL (>= 3)
URL: https://github.com/openbiox/UCSCXenaShiny,
        https://openbiox.github.io/UCSCXenaShiny/
BugReports: https://github.com/openbiox/UCSCXenaShiny/issues
Depends: R (>= 4.1)
Imports: digest, dplyr (>= 0.8.3), ezcox, forcats, ggplot2 (>= 3.2.0),
        ggpubr (>= 0.2), httr, magrittr (>= 1.5), ppcor, psych, purrr,
        rlang, shiny (>= 1.3.2), stats, stringr, tibble (>= 2.1.3),
        tidyr, UCSCXenaTools, utils
Suggests: covr (>= 3.2.1), cowplot, DT (>= 0.5), funkyheatmap, furrr,
        future, ggrepel, ggstatsplot, glue, knitr, pacman, plotly,
        plyr, RColorBrewer (>= 1.1.2), readr, rmarkdown, Rtsne, scales,
        survival, survminer, testthat (>= 2.0.1), umap
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
NeedsCompilation: no
RoxygenNote: 7.3.2
Packaged: 2026-05-05 12:09:52 UTC; wsx
Author: Shixiang Wang [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9855-7357>),
  Shensuo Li [aut],
  Yi Xiong [aut] (ORCID: <https://orcid.org/0000-0002-4370-9824>),
  Longfei Zhao [aut] (ORCID: <https://orcid.org/0000-0002-6277-0137>),
  Kai Gu [aut] (ORCID: <https://orcid.org/0000-0002-0177-0774>),
  Yin Li [aut],
  Fei Zhao [aut]
Repository: CRAN
Date/Publication: 2026-05-05 14:14:38 UTC
Built: R 4.6.0; ; 2026-05-05 17:34:55 UTC; unix

# Generated by roxygen2: do not edit by hand

export("%>%")
export(.opt_pancan)
export(analyze_gene_drug_response_asso)
export(analyze_gene_drug_response_diff)
export(app_run)
export(app_run2)
export(available_hosts)
export(calc_pw_mut_cnv)
export(ezcor)
export(ezcor_batch)
export(ezcor_partial_cor)
export(get_ccle_cn_value)
export(get_ccle_gene_value)
export(get_ccle_mutation_status)
export(get_ccle_protein_value)
export(get_nonomics_value)
export(get_pancan_cn_value)
export(get_pancan_gene_value)
export(get_pancan_methylation_value)
export(get_pancan_miRNA_value)
export(get_pancan_mutation_status)
export(get_pancan_protein_value)
export(get_pancan_transcript_value)
export(get_pancan_value)
export(get_pcawg_APOBEC_mutagenesis_value)
export(get_pcawg_fusion_value)
export(get_pcawg_gene_value)
export(get_pcawg_miRNA_value)
export(get_pcawg_promoter_value)
export(keep_cat_cols)
export(load_data)
export(mol_quick_analysis)
export(query_molecule_value)
export(query_pancan_value)
export(query_tcga_group)
export(query_toil_value_df)
export(tcga_surv_get)
export(tcga_surv_plot)
export(vis_ccle_gene_cor)
export(vis_ccle_tpm)
export(vis_dim_dist)
export(vis_gene_TIL_cor)
export(vis_gene_cor)
export(vis_gene_cor_cancer)
export(vis_gene_cross_omics)
export(vis_gene_drug_response_asso)
export(vis_gene_drug_response_diff)
export(vis_gene_immune_cor)
export(vis_gene_msi_cor)
export(vis_gene_pw_cor)
export(vis_gene_stemness_cor)
export(vis_gene_tmb_cor)
export(vis_identifier_cor)
export(vis_identifier_dim_dist)
export(vis_identifier_grp_comparison)
export(vis_identifier_grp_surv)
export(vis_identifier_multi_cor)
export(vis_pancan_anatomy)
export(vis_pathway_cross_omics)
export(vis_pcawg_dist)
export(vis_pcawg_gene_cor)
export(vis_pcawg_unicox_tree)
export(vis_toil_Mut)
export(vis_toil_Mut_cancer)
export(vis_toil_TvsN)
export(vis_toil_TvsN_cancer)
export(vis_unicox_tree)
import(dplyr)
import(forcats)
import(ggplot2)
import(ppcor)
import(purrr)
import(tibble)
importFrom(magrittr,"%>%")
importFrom(psych,corr.test)
importFrom(shiny,shinyAppFile)
importFrom(stats,complete.cases)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(utils,data)
importFrom(utils,download.file)
importFrom(utils,head)
importFrom(utils,install.packages)
importFrom(utils,tail)

# UCSCXenaShiny <img src="https://raw.githubusercontent.com/lishensuo/images2/main/img01ucsc.png" align="right" width="130" height="150" />

[![CRAN status](https://www.r-pkg.org/badges/version/UCSCXenaShiny)](https://cran.r-project.org/package=UCSCXenaShiny)
[![](https://cranlogs.r-pkg.org/badges/grand-total/UCSCXenaShiny?color=orange)](https://cran.r-project.org/package=UCSCXenaShiny)
[![lifecycle](https://img.shields.io/badge/lifecycle-stable-blue.svg)](https://lifecycle.r-lib.org/articles/stages.html) 
[![R-CMD-check](https://github.com/openbiox/UCSCXenaShiny/actions/workflows/r-check.yml/badge.svg?branch=master)](https://github.com/openbiox/UCSCXenaShiny/actions/workflows/r-check.yml)
[![check in Biotreasury](https://img.shields.io/badge/Biotreasury-collected-brightgreen)](https://biotreasury.rjmart.cn/#/tool?id=61144)

![](https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/Github_readme.png)

**UCSCXenaShiny** is an R package for interactively exploring UCSC Xena.
It is mainly designed to provide a web app (built on the top of [`{shiny}`](https://shiny.posit.co/) framework and [`{UCSCXenaTools}`](https://github.com/ropensci/UCSCXenaTools/) package) for downloading,
analyzing and visualizing datasets from [UCSC
Xena](https://xenabrowser.net/datapages/).



![Alt](https://repobeats.axiom.co/api/embed/2f1f0c9a7a5c3c710febb3ee3d1e26f249d5a3ed.svg "Repobeats analytics image")

Please cite any of the following articles when you used **UCSCXenaShiny** in your study:

---------------

**V2**

- Shensuo Li, Yuzhong Peng, Minjun Chen, Yankun Zhao, Yi Xiong, Jianfeng Li, Peng Luo, Haitao Wang, Fei Zhao, Qi Zhao, Yanru Cui, Sujun Chen, Jian-Guo Zhou, Shixiang Wang,  Facilitating integrative and personalized oncology omics analysis with UCSCXenaShiny, Communications Biology, 1200 (2024),  https://doi.org/10.1038/s42003-024-06891-2


**V1**

- Shixiang Wang<sup>\#</sup>, Yi Xiong<sup>\#</sup>, Longfei Zhao<sup>\#</sup>, Kai Gu<sup>\#</sup>, Yin Li, Fei Zhao, Jianfeng Li, Mingjie Wang, Haitao Wang, Ziyu Tao, Tao Wu, Yichao Zheng, Xuejun Li, Xue-Song Liu, UCSCXenaShiny: An R/CRAN Package for Interactive Analysis of UCSC Xena Data, Bioinformatics, 2021;, btab561, https://doi.org/10.1093/bioinformatics/btab561.

---------------

## :cloud: Use on cloud

If you don't want to install R and packages locally, or you have no programming experience, try using this tool on `Oncoharmony Network` (`http://shiny.zhoulab.ac.cn/UCSCXenaShiny`) or `Hiplot ORG` platform (`https://shiny.hiplot.cn/ucsc-xena-shiny`).

We have deployed two additional apps: a basic version of V2 (`https://shiny.zhoulab.ac.cn/UCSCXenaShiny_basic/`) and an archive version of V1 (`https://shiny.zhoulab.ac.cn/UCSCXenaShinyV1/`, code please see v1 branch).

## :snake: Use with Conda

| Name | Downloads | Version | Platforms |
| --- | --- | --- | --- |
| [![Conda Recipe](https://img.shields.io/badge/recipe-r--ucscxenashiny-green.svg)](https://github.com/conda-forge/r-ucscxenashiny-feedstock) | [![Conda Downloads](https://img.shields.io/conda/dn/conda-forge/r-ucscxenashiny.svg)](https://github.com/conda-forge/r-ucscxenashiny-feedstock) | [![Conda Version](https://img.shields.io/conda/vn/conda-forge/r-ucscxenashiny.svg)](https://github.com/conda-forge/r-ucscxenashiny-feedstock) | [![Conda Platforms](https://img.shields.io/conda/pn/conda-forge/r-ucscxenashiny.svg)](https://github.com/conda-forge/r-ucscxenashiny-feedstock) |

Install from `conda-forge` channel with:

```bash
conda install -c conda-forge r-ucscxenashiny
```

It is possible to list all of the versions of `r-ucscxenashiny` available on your platform with:

```bash
conda search r-ucscxenashiny --channel conda-forge
```

## :package: Use with Docker

<img alt="Docker Image Version (latest by date)" src="https://img.shields.io/docker/v/shixiangwang/ucscxenashiny?color=blue">
<img alt="Docker Image Size (latest by date)" src="https://img.shields.io/docker/image-size/shixiangwang/ucscxenashiny">
<img alt="Docker Pulls" src="https://img.shields.io/docker/pulls/shixiangwang/ucscxenashiny">

UCSCXenaShiny has corresponding docker image at <https://hub.docker.com/r/shixiangwang/ucscxenashiny/>, you can install the latest version with:

```bash
docker pull shixiangwang/ucscxenashiny
```
> From v2, docker pull from GitHub `ghcr.io` registry is also supported, e.g., `docker pull ghcr.io/openbiox/ucscxenashiny:master`.

All versions can be found at <https://hub.docker.com/r/shixiangwang/ucscxenashiny/tags/>.
To use a specified version (e.g., `2.0.0`; latest code commit will auto-build a tag `master`), run the following command to install:

```bash
docker pull shixiangwang/ucscxenashiny:2.0.0
# docker pull shixiangwang/ucscxenashiny:master  # For latest code, unstable
```

Run the latest stable docker image and keep it at background with:

```bash
docker run -d --name xenashiny -p 3838:3838 shixiangwang/ucscxenashiny
```

Now you should find the Shiny when you open URL `http://127.0.0.1:3838` with your web browser.
If you deploy the docker in a public (cloud) Linux server, change `127.0.0.1` to the host IP.

> For the first time, it may return a failure about 'take a long time to respond', please
> refresh the web page.

If the application failed to start. Check if the container has installed all dependencies.

```bash
docker exec xenashiny R -e 'source(system.file("shinyapp/utils_pkgs.R", package = "UCSCXenaShiny"))'
```

Or you can interactively check the container:

```bash
docker exec -it xenashiny /bin/bash
```

You can manage the deployed container with the following commands:

```bash
# Stop the container
docker stop xenashiny
# Start the container
docker start xenashiny
```

## :arrow_double_down: Manual installation

You can install stable release of **UCSCXenaShiny** from CRAN with:

```r
install.packages("UCSCXenaShiny")
```

You can install the development version of **UCSCXenaShiny** from Github
with:

```r
remotes::install_github("openbiox/UCSCXenaShiny")
```

Or r-universe:

```r
install.packages("UCSCXenaShiny", repos = c("https://openbiox.r-universe.dev", "https://cran.r-project.org"))
```

Other dependent R packages specific to the Shiny application will be automatically installed when you start with `app_run()` command. If you failed to install **UCSCXenaShiny**, please check if the following system dependencies have been properly installed or see [**Troubleshooting**](#hammer_and_wrench-troubleshooting) section for specific installation issues.

### System dependencies installation

When you use Windows/MacOS, please skip reading this sub-section.

As Linux distributions are very diverse,  here we only test the installation of **UCSCXenaShiny** on common used Ubuntu/CentOS. If you are using other Linux distributions, you need to solve the system dependencies installation problems yourself when you encounter R package installation errors. However, the installation of system dependencies on Ubuntu/CentOS could be very good references.

> Please note all commands below are execuated with `root`.

**Ubuntu**:

```bash
apt update -y && apt install -y libcurl4-openssl-dev libssl-dev libxml2-dev \
	libgmp3-dev libmpfr-dev
```

**CentOS**:

```bash
yum update -y && yum install -y libcurl-devel openssl-devel libxml2-devel \
	gmp-devel mpfr-devel libjpeg-devel cairo-devel
```



## :beginner: Usage

First load package:

```r
library(UCSCXenaShiny)
```

Start Shiny in your R console (ignore this if you just want to use functions in this package):

```r
app_run()
# At default, the Shiny is running under client mode
# It means the data queried from remote UCSC Xena server will
# be saved to temporary directory determined by R
# If you frequently use this tool or deploy this tool as a web service for multiple users
# It is recommended to run it with 'server' mode
# i.e.,
#
# app_run("server")
```

If you want deploy UCSC Xena Shiny with Shiny Server, please copy `App.R` and `www/` directory under [`shinyapp`](https://github.com/openbiox/UCSCXenaShiny/tree/master/inst/shinyapp).
`xena.runMode` on the top of `App.R` is recommended to set as `"server"` instead of `"client"` (default).

For advanced users, examples for illustrating useful functions to obtain and analyze data are described in vignette.

All exported data and functions are organized at [here](https://openbiox.github.io/UCSCXenaShiny/reference/index.html).

`xena.cacheDir` and `xena.zenodoDir` are two options to control where to store data.

e.g.,

```r
xena.cacheDir = "/xena"
xena.zenodoDir = "/xena/datasets"

options(xena.cacheDir = xena.cacheDir, xena.zenodoDir = xena.zenodoDir)

options(xena.runMode = "server")
```

Option `xena.runMode` can be used to control the way how the Shiny works.
It can be 'client' or 'server'. You can directly set it in `app_run()`.

## :open_book: Book tutorials

We have prepared the book tutorials to introduce the detailed usage of UCSCXenaShiny based on R Markdown and bookdown. 

- https://lishensuo.github.io/UCSCXenaShiny_Book


## :movie_camera: Video tutorials

Users can also watch the video tutorials and follow the step-by-step operations to quickly learn how to use UCSCXenaShiny.

#### UCSCXenaShiny v2
**Chinese website**
- Home page: <https://www.bilibili.com/video/BV1rT421k7Zv>
- T•P•C Modules: <https://www.bilibili.com/video/BV1E14218779> 
- T•P•C Pipelines: <https://www.bilibili.com/video/BV141421t71g>
- Repository/General Analysis/Download: <https://www.bilibili.com/video/BV1TZ421K7VH>

**English website**: <https://www.youtube.com/watch?v=VnIlzcVB1Ow&list=PLN1jxcOQONG6pH8zvHr-qhI3MMtvluITf>

#### UCSCXenaShiny v1

- Home page: <https://www.bilibili.com/video/bv1R84y1F7WT>.
- Repository page: <https://www.bilibili.com/video/bv1Mq4y1J73o>.
- General Analysis page: <https://www.bilibili.com/video/bv1p64y1U7iU>.
- Quick PanCan Analysis page: <https://www.bilibili.com/video/bv1o64y1y7jW>.
- Features:
  - Genomic Signature: <https://www.bilibili.com/video/bv1zB4y1c7eB>.
  - Upload custom data for general analysis: <https://www.bilibili.com/video/BV1pf4y1j7eP>.
  Please note that upload files must in the format of csv/tsv (gz compression is supported) and
  files must have extension tsv/txt/csv/gz.

## :hammer_and_wrench: Troubleshooting

1. `ERROR: dependencies ‘gmp’, ‘Rmpfr’ are not available for package ‘PMCMRplus’` or `ERROR: dependency ‘pairwiseComparisons’ is not available for package ‘ggstatsplot’`.

   **Your operating system lacks `gmp` and `Rmpfr` development libraries.**
   
   ```bash
   # Ubuntu
   apt install -y libgmp3-dev libmpfr-dev
   # CentOS
   yum install -y gmp-devel mpfr-devel
   ```
   
2. `installation of package ‘gridtext’ had non-zero exit status` with error info `grid-renderer.h:61:94: error: no matching function for call to ‘Rcpp::Vector<10, Rcpp::PreserveStorage>::Vector(int, bool&, const GraphicsContext&)’`.
   
    **You have an older C++ version which cannot support C++11 features. This error seems only happen on CentOS. Install a newer C++ and set it as default compiler for R would fix this problem.** 
    
    ```bash
    yum install centos-release-scl
    yum install devtoolset-9
    # If you use your non-root account to install packages, 
    # change /root to /home/your_id in the following command
    mkdir -p /root/.R
    vi /root/.R/Makevars 
    ```
    
     **Append content to the openning file.**

    ```bash
    CXX11=/opt/rh/devtoolset-9/root/usr/bin/g++ -std=c++11
    ```


3. `installation of package ‘nloptr’ had non-zero exit status` with error info `libtool: link: ERROR: no information for variable 'AR' cru`.

   **The latest version of `nloptr` can only support R>=4.0. When you are using R3.6 or below would have this issue. So install an older version in R console can fix this.**

   ```R
   packageurl <- "https://cran.r-project.org/src/contrib/Archive/nloptr/nloptr_1.2.1.tar.gz"
   
   install.packages(packageurl, repos=NULL, type="source")
   ```

   > Reference: https://stackoverflow.com/questions/62900525/install-lme4-from-cran-on-ubuntu

4. `package ‘pacman’ is not available` or similar.

   **Install it by hand in R console.**

   ```R
   install.packages("pacman")
   ```

5. `there is no package called ‘shinythemes’` or similar.

   **Install it by hand in R cosole.**

   ```R
   install.packages("shinythemes")
   ```

6. Install package `gganatogram` failed or similar.

   **Install it by hand in R cosole.**

   ```R
   if (!requireNamespace("gganatogram")) {
      library(remotes)
      tryCatch(
         remotes::install_github("jespermaag/gganatogram"),
         error = function(e) {
            remotes::install_git("https://gitee.com/XenaShiny/gganatogram")
         }
      )
   }
   ```

7. Install package `ggradar` failed or similar.

   **Install it by hand in R cosole.**

   ```R
   if (!requireNamespace("ggradar")) {
      library(remotes)
      tryCatch(
         remotes::install_github("ricardo-bion/ggradar"),
         error = function(e) {
            remotes::install_git("https://gitee.com/XenaShiny/ggradar")
         }
      )
   }
   ```


## :writing_hand: Author

  - [Shixiang Wang](https://github.com/ShixiangWang)
  - [Yi Xiong](https://github.com/Byronxy)
  - [Shensuo Li](https://github.com/lishensuo)
  - [Longfei Zhao](https://github.com/longfei8533)
  - [Kai Gu](https://github.com/kaigu1990)
  - [Yuzhong Peng](https://github.com/mugpeng)
  - [Yin Li](https://github.com/yinlisssss)
  - [Fei Zhao](https://github.com/fei0810)

## :page_with_curl: LICENSE

GPLv3 © [Openbiox](https://github.com/openbiox)

uiDrugOmicPair <- function(id){
  ns <- NS(id)
  fluidPage(
    fluidRow(
      column(4,
             selectInput(inputId = ns("select_omics"), 
                         "Please select the molecular type:", 
                         choices = c("Copy Number Data" = "cnv",
                                     "DNA Methylation" = "meth",
                                     "Gene Fusion" = "fusion",
                                     "Gene Mutation" = "mutation_gene",
                                     "Gene Site Mutation" = "mutation_site",
                                     "mRNA Expression" = "mRNA",
                                     "Protein Expression" = "protein"
                         ), selected = "mRNA"
             )),
      # Select omics ----
      column(4,
             selectizeInput(
               ns("select_specific_omic"), "Molecule selection:", choices = NULL,
               options = list(
                 placeholder = 'Please select a molecular feature',
                 onInitialize = I('function() { this.setValue(""); }'), selected = "ABCC3"
               )),
             # DOPselectOmicsUI("DOPselectOmics"),
      ),
      # Select drugs ----
      column(4,
             selectizeInput(
               ns("select_specific_drug"), "Drug selection:", choices = NULL,
               options = list(
                 placeholder = 'Please select a drug',
                 onInitialize = I('function() { this.setValue(""); }'), selected = "YM-155"
               ))
      ),
    ),
    # Plot results ----
    wellPanel(
      # textOutput("total")
      column(12, plotOutput(ns("p_search"), height="20cm")),
      h5("."),
    ),
    h4(strong("NOTEs:")),
    h5(strong("Given that there are overlapping cells in different drug and omics datasets, we have utilized the common data to assess correlations, thereby maximizing the utilization of existing information. For instance, the designation 'gdsc_ctrp1' indicates that the omics data is sourced from the GDSC project, while the drug sensitivity data is derived from the CTRP1 project.")),
    h5("The y axis is the drug sensitivity metric."),
    h5("The x-axis represents the molecular state or numerical data, including the boolean state for gene mutations or mRNA expression.")
    # Download plot ----
    # column(3,
    #        downloadLink(ns("dl_p_search"), "Save as PDF")
    # )
  )
}

serverDrugOmicPair <- function(input, output, session){
  ns <- session$ns
  # Select ----
  ## Omics ----
  omics_search_sel <- reactiveValues()
  observeEvent(input$select_omics, {
    omics_search_sel$omics <- switch(input$select_omics, 
                                     "mRNA" = omics_search[omics_search$type %in% "mRNA",]$omics,
                                     "meth" = omics_search[omics_search$type %in% "meth",]$omics,
                                     "protein" = omics_search[omics_search$type %in% "protein",]$omics,
                                     "cnv" = omics_search[omics_search$type %in% "cnv",]$omics,
                                     "mutation_gene" = omics_search[omics_search$type %in% "mutation_gene",]$omics,
                                     "mutation_site" = omics_search[omics_search$type %in% "mutation_site",]$omics,
                                     "fusion" = omics_search[omics_search$type %in% "fusion",]$omics)
    updateSelectizeInput(session = session, inputId = 'select_specific_omic',
                         label = 'Molecule selection:', choices = omics_search_sel$omics, server = TRUE,
                         options = list(placeholder = 'Please select a molecular feature', onInitialize = I('function() { this.setValue(""); }')),
                         selected = "ABCC3"
    )
  })
  
  ## Drugs ----
  updateSelectizeInput(session = session, inputId = 'select_specific_drug',
                       label = 'Drug selection:', choices = drugs_search$drugs, server = TRUE,
                       options = list(placeholder = 'Please select a drug', onInitialize = I('function() { this.setValue(""); }')),
                       selected = "YM-155"
  )
  # Plot ----
  # Select omics
  omics_search_list1_sel <- reactive({omics_search_list1[[input$select_omics]]}) # cor
  omics_search_list2_sel <- reactive({omics_search_list2[[input$select_omics]]}) # discrete
  output$p_search <- renderPlot({
    ## Cor plot ----
    if(input$select_omics %in% c("cnv",
                                 "protein",
                                 "meth",
                                 "mRNA")){
      p_list <- lapply(1:length(omics_search_list1_sel()), function(x){
        # TSPAN6 YM-155
        omics_sel <- omics_search_list1_sel()[[x]][[1]]
        drugs_sel <- omics_search_list1_sel()[[x]][[2]]
        sel_omics <- omics_sel[rownames(omics_sel) %in% input$select_specific_omic,] %>% as.numeric()
        sel_drugs <- drugs_sel[rownames(drugs_sel) %in% input$select_specific_drug,] %>% as.numeric()
        if(length(na.omit(sel_omics)) == 0 | length(na.omit(sel_drugs)) == 0){ return(NULL) }
        cor_df <- data.frame(
          genes = sel_omics,
          drugs = sel_drugs
        )
        p <- ggscatter(cor_df, x = "genes", y = "drugs",
                       alpha = 0.2) + 
          stat_cor(size = 6, method = "spearman") + stat_smooth(formula = y ~ x,method = "lm") + theme_bw() + 
          theme(
            axis.title = element_blank(),
            title = element_text(size = 15, face = "bold"),
            axis.text = element_text(size = 12)
          ) + ggtitle(names(omics_search_list1_sel())[[x]])
      })
      # Box plot ----
    } else{
      p_list <- lapply(1:length(omics_search_list2_sel()), function(x){
        omics_sel <- omics_search_list2_sel()[[x]][[1]]
        drugs_sel <- omics_search_list2_sel()[[x]][[2]]
        sel_omics <- omics_sel$cells[omics_sel[[1]] %in% input$select_specific_omic] %>% unique()
        sel_drugs <- drugs_sel[rownames(drugs_sel) %in% input$select_specific_drug,] %>% as.numeric()
        yes_drugs <- sel_drugs[colnames(drugs_sel) %in% sel_omics] %>% na.omit()
        no_drugs <- sel_drugs[!colnames(drugs_sel) %in% sel_omics] %>% na.omit()
        if(length(yes_drugs) == 0 | length(no_drugs) == 0){ return(NULL) }
        box_df <- data.frame(
          drugs = c(no_drugs, yes_drugs),
          events = rep(c("no","yes"), times = c(length(no_drugs), length(yes_drugs))))
        p <- ggboxplot(data = box_df, x = "events", y = "drugs",
                       fill = "events", palette = c("#BEBADAFF", "#FB8072FF"),
                       add = "jitter", add.params = list(alpha = 0.2)) + 
          stat_compare_means(size = 6, label.x = 0.8) + theme_bw() + 
          theme(
            axis.title = element_blank(),
            title = element_text(size = 20, face = "bold"),
            axis.text = element_text(size = 18),
            # axis.text.x = element_text(size = 17),
            legend.position = "none"
          ) + 
          coord_cartesian(ylim = c(0, max(box_df$drugs) + 
                                     max(box_df$drugs)/20)) + 
          ggtitle(names(omics_search_list2_sel())[[x]])
      })
    }
    p_list <- p_list[!sapply(p_list, is.null)]
    # Warning 
    shiny::validate(
      need(length(p_list) > 0, "You have not chosen yet, or there is no result for this drug-omic pair.")
    )
    p <- wrap_plots(p_list, ncol = 3)
    return(p)
  })
}

ui.modules_ga_surv_analysis <- function(id) {
  ns <- NS(id)
  fluidPage(
    fluidRow(
      column(
        3,
        wellPanel(
          h4("Analysis Controls"),
          uiOutput(ns("ga_data1_id")),
          virtualSelectInput(
            inputId = ns("ga_data1_mid"), # molecule identifier
            label = "Dataset 1 molecule identifier:",
            choices = NULL,
            search = TRUE,
            allowNewOption = TRUE,
            dropboxWidth = "200%"
          ),
          shinyBS::bsPopover(ns("ga_data1_mid"),
            title = "Note",
            content = "Ignore this option when you select a phenotype dataset",
            placement = "right", options = list(container = "body")
          ),
          uiOutput(ns("ga_data2_id")),
          selectInput(ns("color_palette"), "Color palette:",
            choices = c("npg", "aaas", "lancet", "jco", "ucscgb", "uchicago", "simpsons", "rickandmorty"),
            selected = "aaas"
          ),
          actionBttn(
            inputId = ns("ga_preprocess_data"),
            label = "Preprocess",
            style = "gradient",
            icon = icon("check"),
            color = "default",
            block = TRUE,
            size = "sm"
          ),
          tags$br(),
          actionBttn(
            inputId = ns("ga_go"),
            label = "Submit",
            style = "gradient",
            icon = icon("check"),
            color = "default",
            block = TRUE,
            size = "sm"
          )
        )
      ),
      column(
        6,
        plotOutput(ns("ga_output")),
        DT::dataTableOutput(ns("ga_output_data"))
      ),
      column(
        3,
        wellPanel(
          h4("Sample Filters"),
          uiOutput(ns("ga_data_filter1_id")),
          actionBttn(
            inputId = ns("ga_filter_button"),
            label = "Click to filter!",
            color = "primary",
            style = "bordered",
            size = "sm"
          ),
          tags$br(),
          tags$br(),
          numericInput(inputId = ns("height"), label = "Height", value = 25),
          numericInput(inputId = ns("width"), label = "Width", value = 20),
          column(
            width = 12, align = "center",
            prettyRadioButtons(
              inputId = ns("device"),
              label = "Choose plot format",
              choices = c("png", "pdf"),
              selected = "png",
              inline = TRUE,
              icon = icon("check"),
              animation = "jelly",
              fill = TRUE
            )
          ),
          downloadBttn(
            outputId = ns("download"),
            style = "gradient",
            color = "default",
            block = TRUE,
            size = "sm"
          )
        )
      )
    )
  )
}


server.modules_ga_surv_analysis <- function(
  input, output, session,
  selected_database_rm_phenotype, selected_database_add_url_and_phenotype,
  custom_file) {
  ns <- session$ns

  output$ga_data1_id <- renderUI({
    show_table <- selected_database_add_url_and_phenotype()
    selectInput(
      inputId = ns("ga_data1_id"),
      label = "Select dataset for grouping:",
      choices = c("NONE", unique(show_table$XenaDatasets)),
      selected = "NONE",
      multiple = FALSE
    )
  })

  id1_choices <- eventReactive(input$ga_data1_id, {
    if (!identical(input$ga_data1_id, "NONE")) {
      if (input$ga_data1_id %in% phenotype_datasets || input$ga_data1_id == "custom_phenotype_dataset") {
        # A phenotype data is selected
        return(list(
          all = "NONE",
          selected = "NONE"
        ))
      } else {
        # !!Assume a dense matrix dataset is selected
        return(list(
          all = if (is.null(custom_file$fData)) all_preload_identifiers else
            unique(c(custom_file$fData[[1]], all_preload_identifiers)),
          selected = "TP53"
        ))
      }
    } else {
      return(list(
        all = "NONE",
        selected = "NONE"
      ))
    }
  })

  observe({
    updateVirtualSelect(
      "ga_data1_mid",
      choices = id1_choices()$all,
      selected = id1_choices()$selected
    )
  })

  output$ga_data2_id <- renderUI({
    show_table <- selected_database_add_url_and_phenotype()
    selectInput(
      inputId = ns("ga_data2_id"),
      label = "Select survival dataset:",
      choices = c("NONE", unique(show_table$XenaDatasets[show_table$XenaDatasets %in% c(phenotype_datasets, "custom_phenotype_dataset")])),
      selected = "NONE",
      multiple = FALSE
    )
  })

  # Preprocessing
  group_col_status <- reactiveValues(status = "Off") # Off, CO (连续值), CA（离散值）

  observeEvent(input$ga_preprocess_data, {
    message("Preprocess button is clicked by user.")

    # dataset1 is the value data
    dataset1 <- setdiff(input$ga_data1_id, "NONE")
    dataset1_id <- setdiff(input$ga_data1_mid, "NONE")
    dataset1_phenotype <- if (dataset1 %in% phenotype_datasets) "YES" else "NO"

    if (dataset1_phenotype == "YES") {
      data1 <- XenaGenerate(subset = XenaDatasets == dataset1) %>%
        XenaQuery() %>%
        XenaDownload(destdir = XENA_DEST) %>%
        XenaPrepare() %>%
        as.data.frame()
    } else {
      if (length(dataset1_id) || dataset1 == "custom_phenotype_dataset") {
        data1 <- get_data_df(dataset1, dataset1_id)
      } else {
        sendSweetAlert(session,
          title = "Warning",
          text = "Please input a ID (e.g., gene symbol) for axis X when you select a non-phenotype dataset"
        )
      }
    }
    print(head(data1))

    # dataset2 is the survival data
    dataset2 <- setdiff(input$ga_data2_id, "NONE")
    dataset2_id <- setdiff(input$ga_data2_mid, "NONE")

    if (dataset2 == "custom_phenotype_dataset") {
      data2 <- readRDS(file.path(tempdir(), "custom_phenotype_data.rds"))
    } else {
      data2 <- XenaGenerate(subset = XenaDatasets == dataset2) %>%
        XenaQuery() %>%
        XenaDownload(destdir = XENA_DEST) %>%
        XenaPrepare() %>%
        as.data.frame()
    }
    print(head(data2))

    if (length(dataset1) && length(dataset2)) {
      showModal(
        modalDialog(
          title = "Preprocessing",
          size = "l",
          fluidPage(
            wellPanel(
              h3("1. Process dataset 1 to determine the survival groups"),
              DT::dataTableOutput(ns("data1_table")),
              fluidRow(
                column(4, selectInput(ns("data1_sample_col"), "Sample column", choices = colnames(data1), selected = colnames(data1)[1])),
                shinyBS::bsPopover(ns("data1_sample_col"),
                  title = "Note",
                  content = "The sample IDs in this column must match with the second table",
                  placement = "top", options = list(container = "body")
                ),
                column(4, selectInput(ns("data1_group_col"), "Group column", choices = colnames(data1), selected = colnames(data1)[2]))
              ),
              shinyjs::hidden(
                prettyRadioButtons(
                  inputId = ns("group_col_cutoff_mode"),
                  label = "Cutoff mode",
                  icon = icon("check"),
                  choices = c("Auto", "Custom"),
                  selected = "Auto",
                  animation = "jelly",
                  status = "default",
                  inline = TRUE
                ),
                sliderInput(
                  inputId = ns("group_col_cutpoint"), label = "Select (min and max) percent cutoff (%) to generate (Low and High) groups:",
                  min = 10, max = 90, value = c(50, 50)
                )
              ),
              shinyjs::hidden(
                uiOutput(ns("group_col_groups"))
              )
            ),
            wellPanel(
              h3("2. Process dataset 2 to determine the time and status data"),
              DT::dataTableOutput(ns("data2_table")),
              fluidRow(
                column(3, selectInput(ns("data2_sample_col"), "Sample column", choices = colnames(data2), selected = colnames(data2)[1])),
                shinyBS::bsPopover(ns("data2_sample_col"),
                  title = "Note",
                  content = "The sample IDs in this column must match with the first table",
                  placement = "top", options = list(container = "body")
                ),
                column(3, selectInput(ns("data2_time_col"), "Time column", choices = c("NONE", colnames(data2)), selected = "NONE")),
                column(3, selectInput(ns("data2_status_col"), "Status column", choices = c("NONE", colnames(data2)), selected = "NONE")),
                column(3, selectInput(
                  inputId = ns("status_label"),
                  label = "Event label",
                  choices = NULL
                ))
              )
            ),
            wellPanel(
              h3("3. Join selected columns in the two tables"),
              actionBttn(
                inputId = ns("join_table_button"),
                label = "Click to join the data",
                color = "primary",
                style = "bordered", size = "sm",
                block = F
              ),
              h4("If the table below is right, dismiss this page and click submit button to plot"),
              markdown("- The first column refers to sample ID.
                       - The second column refers to groups (numeric type is accepted, it will be classified when plotting).
                       - The third column refers to survival time.
                       - The fourth column refers survival status."),
              DT::dataTableOutput(ns("joined_table"))
            )
          )
        )
      )

      observe({
        group_col <- setdiff(input$data1_group_col, "NONE")
        if (length(group_col) && group_col %in% colnames(data1)) {
          # A column has been selected by user
          data <- na.omit(data1[[group_col]])
          if (length(data)) {
            if (is.numeric(data) && length(unique(data)) > 5) {
              shinyjs::hide(id = "group_col_groups")
              shinyjs::show(id = "group_col_cutoff_mode")
              if (input$group_col_cutoff_mode == "Custom") {
                shinyjs::show(id = "group_col_cutpoint")
              } else {
                shinyjs::hide(id = "group_col_cutpoint")
              }
              group_col_status$status <- "CO"
            } else {
              choices <- unique(data)
              group_col_status$status <- "CA"
              output$group_col_groups <- renderUI({
                prettyCheckboxGroup(
                  inputId = ns("group_col_groups"),
                  label = "Select groups:",
                  choices = choices,
                  selected = choices,
                  animation = "jelly",
                  status = "info"
                )
              })
              shinyjs::hide(id = "group_col_cutoff_mode")
              shinyjs::hide(id = "group_col_cutpoint")
              shinyjs::show(id = "group_col_groups")
            }
            message("Observe:")
            print(group_col_status$status)
          } else {
            sendSweetAlert(session,
              title = "Warning",
              text = "The group column you selected seems have no valid data!",
              type = "warning"
            )
          }
        } else {
          shinyjs::hide(id = "group_col_cutoff_mode")
          shinyjs::hide(id = "group_col_cutpoint")
          shinyjs::hide(id = "group_col_groups")
          group_col_status$status <- "Off"
        }
      })

      output$data1_table <- DT::renderDataTable(server = TRUE, {
        DT::datatable(
          data1,
          rownames = FALSE,
          options = list(
            scrollY = 350,
            scrollX = 300,
            initComplete = JS(
              "function(settings, json) {",
              "$(this.api().table().header()).css({'font-size': '90%'});",
              "$(this.api().table().body()).css({'font-size': '85%'});",
              "}"
            )
          )
        )
      })

      output$data2_table <- DT::renderDataTable(server = TRUE, {
        DT::datatable(
          data2,
          rownames = FALSE,
          options = list(
            scrollY = 350,
            scrollX = 300
          )
        )
      })

      event_choices <- reactive({
        data2_status_col <- setdiff(input$data2_status_col, "NONE")
        if (length(data2_status_col)) {
          names(table(data2[[data2_status_col]]))
        } else {
          NULL
        }
      })

      observe({
        if (length(event_choices())) {
          updateSelectInput(
            session,
            "status_label",
            choices = event_choices(),
            selected = if (length(event_choices) > 1) event_choices()[2] else event_choices()[2]
          )
        }
      })

      observeEvent(input$join_table_button, {
        data1_sample_col <- isolate(input$data1_sample_col)
        data2_sample_col <- isolate(input$data2_sample_col)
        data1_group_col <- isolate(input$data1_group_col)
        data2_time_col <- setdiff(isolate(input$data2_time_col), "NONE")
        data2_status_col <- setdiff(isolate(input$data2_status_col), "NONE")
        status_label <- isolate(input$status_label)

        if (length(data1_group_col) > 0 && length(data2_time_col) > 0 && length(data2_status_col) > 0 && length(status_label) > 0) {
          data1_cols <- c(data1_sample_col, data1_group_col)
          data2_cols <- c(data2_sample_col, data2_time_col, data2_status_col)
          col_order <- c("sample", data1_group_col, data2_time_col, data2_status_col)
          message("column order:")
          print(col_order)

          if (all(data1_cols %in% colnames(data1)) && all(data2_cols %in% colnames(data2))) {
            data1_copy <- dplyr::select(data1, dplyr::all_of(data1_cols))
            colnames(data1_copy)[1] <- "sample"
            message("data1 to join:")
            print(head(data1_copy))
            message("data1 columns:")
            print(colnames(data1_copy))

            data2_copy <- dplyr::select(data2, dplyr::all_of(data2_cols))
            colnames(data2_copy)[1] <- "sample"
            message("data2 to join:")
            print(head(data2_copy))
            message("data2 columns:")
            print(colnames(data2_copy))

            message("Joining:")
            joined_data <- tryCatch(
              {
                dplyr::inner_join(
                  data1_copy, data2_copy,
                  by = "sample"
                ) %>%
                  dplyr::select(dplyr::all_of(col_order)) %>%
                  as.data.frame() %>%
                  setNames(c("sample", "group", "time", "status"))
              },
              error = function(e) {
                message("Joining failed for group comparison analysis.")
                NULL
              }
            )
            print(head(joined_data))

            print(group_col_status$status)
            if (group_col_status$status == "CA") {
              # 作为离散值处理
              message(length(input$group_col_groups), " groups remained.")
              joined_data <- joined_data[joined_data[[2]] %in% isolate(input$group_col_groups), ]
            }

            # Make sure the status is 0 or 1
            message("Handling survival status...")
            joined_data$status <- as.character(joined_data$status)
            joined_data$status <- ifelse(joined_data$status == status_label, 1L, 0L)

            grp_df$data <- joined_data

            if (!is.null(joined_data)) {
              message("Displaying joining table")
              output$joined_table <- DT::renderDataTable(server = TRUE, {
                DT::datatable(
                  joined_data,
                  rownames = FALSE,
                  options = list(
                    scrollY = 350,
                    scrollX = 300
                  )
                )
              })
            } else {
              sendSweetAlert(session,
                title = "Warning",
                text = "Joining failed! Please check your selection, especially the sample ID column.", type = "warning"
              )
            }
          }
        } else {
          print(str(data2_time_col))
          print(str(data2_status_col))
          sendSweetAlert(session,
            title = "Warning",
            text = "Some options (e.g., time, status) have not been selected!", type = "warning"
          )
        }
      })
    } else {
      sendSweetAlert(session,
        title = "Warning",
        text = "Two datasets must be selected.",
        type = "warning"
      )
    }
  })

  selected_samps <- reactiveValues(id = NULL)
  grp_df <- reactiveValues(data = NULL)

  p_surv <- eventReactive(input$ga_go, {
    if (is.null(selected_samps$id)) {
      message("All samples selected for analysis.")
    } else {
      message(length(selected_samps$id), " samples selected for analysis.")
    }
    tryCatch(
      vis_identifier_grp_surv(
        surv_df = grp_df$data,
        samples = selected_samps$id,
        cutoff_mode = if (group_col_status$status == "CA") {
          "None"
        } else {
          input$group_col_cutoff_mode
        },
        cutpoint = input$group_col_cutpoint,
        palette = input$color_palette
      ),
      error = function(e) {
        message("General analysis survival plot error:")
        print(e$message)
        "Error"
      }
    )
  })

  observeEvent(input$ga_go, {
    # Analyze correlation with 2 input datasets and identifiers
    output$ga_output <- renderPlot(
      if (inherits(p_surv(), "ggsurvplot")) {
        print(p_surv())
      } else {
        sendSweetAlert(
          session,
          title = "Error",
          text = "Error to query data and plot. Please make sure you have go through the 'preprocess' step.",
          type = "error"
        )
      }
    )
    output$download <- downloadHandler(
      filename = function() {
        paste0(Sys.Date(), "_surv_plot.", input$device)
      },
      content = function(file) {
        if(input$device == "png"){
          png(filename = file, units = "cm", width = input$width, height = input$height, res = 600)
          print(p_surv())
          dev.off()
        }
        if(input$device == "pdf"){
          pdf(file = file, onefile = FALSE)
          print(p_surv())
          dev.off()
        }
        # ggplot2::ggsave(
        #   filename = file, plot = print(p_surv(), newpage = F), device = input$device,
        #   units = "cm", width = input$width, height = input$height, dpi = 600
        # )
      }
    )

    output$ga_output_data <- DT::renderDataTable(server = FALSE, {
      if (inherits(p_surv(), "ggsurvplot")) {
        DT::datatable(
          attr(p_surv(), "data"),
          rownames = FALSE,
          extensions = c("Buttons"),
          options = list(
            pageLength = 5,
            dom = "Bfrtip",
            buttons = list(
              list(
                extend = "csv", text = "Download Current Page", filename = "page",
                exportOptions = list(
                  modifier = list(page = "current")
                )
              ),
              list(
                extend = "csv", text = "Download Full Results", filename = "data",
                exportOptions = list(
                  modifier = list(page = "all")
                )
              )
            )
          )
        )
      }
    })
  })

  output$ga_data_filter1_id <- renderUI({
    show_table <- selected_database_add_url_and_phenotype()
    selectInput(
      inputId = ns("ga_data_filter1_id"),
      label = "Select phenotype dataset:",
      choices = c("NONE", unique(show_table$XenaDatasets[show_table$XenaDatasets %in% c(phenotype_datasets, "custom_phenotype_dataset")])),
      selected = "NONE",
      multiple = FALSE
    )
  })

  observeEvent(input$ga_data_filter1_id, {
    # If phenotype dataset has been reset, we use all samples for plotting
    if (length(input$ga_data_filter1_id)) {
      if (input$ga_data_filter1_id == "NONE") {
        selected_samps$id <- NULL
      }
    }
  })

  observeEvent(input$ga_filter_button, {
    message("Sample filter button is clicked by user.")
    pdataset <- setdiff(input$ga_data_filter1_id, "NONE")

    if (length(pdataset)) {
      showModal(
        modalDialog(
          title = "Filter samples for analysis",
          size = "l",
          fluidPage(
            wellPanel(
              h4("1. Select available columns on the left to right"),
              uiOutput(ns("ga_col_chooser")),
              tags$br(),
              actionBttn(
                inputId = ns("show_or_update_ptable"),
                label = "Show/Update Phenotype Table",
                color = "primary",
                style = "bordered",
                size = "sm",
                block = F
              )
            ),
            wellPanel(
              h4("2. Filter rows by SearchPanels"),
              DT::dataTableOutput(ns("ga_phenotype_data"))
            ),
            wellPanel(
              h4("3. Specify sample column and hit button"),
              uiOutput(ns("ga_select_samp_col")),
              actionBttn(
                inputId = ns("ga_filter_submit_button"),
                label = "Submit to filter",
                color = "primary",
                style = "bordered", size = "sm",
                block = F
              )
            ),
            wellPanel(
              h4("4. After hitting button, dismiss this page and click submit button")
            )
          )
        )
      )

      phenotype_table <- XenaGenerate(subset = XenaDatasets == pdataset) %>%
        XenaQuery() %>%
        XenaDownload(destdir = XENA_DEST) %>%
        XenaPrepare()

      output$ga_col_chooser <- renderUI({
        all_cols <- colnames(phenotype_table)
        sel_idx <- seq_len(min(length(all_cols), 5))
        chooserInput(ns("ga_col_chooser"), "Available columns", "Selected columns",
          if (length(all_cols) == length(sel_idx)) c() else all_cols[-sel_idx],
          all_cols[sel_idx],
          size = 5, multiple = TRUE
        )
      })

      observeEvent(input$show_or_update_ptable, {
        selected_cols <- isolate(input$ga_col_chooser$right)
        if (length(selected_cols)) {
          message("Following columns selected by users from sample filter window.")
          print(selected_cols)

          output$ga_phenotype_data <- DT::renderDataTable(server = FALSE, {
            DT::datatable(
              phenotype_table %>%
                dplyr::select(all_of(selected_cols)),
              rownames = FALSE,
              extensions = c("Buttons", "Select", "SearchPanes"), # "Scroller" causes bug in searchPanel
              options = list(
                dom = "Bfrtip", # P
                buttons = list(
                  "searchPanes",
                  list(
                    extend = "csv", text = "Download Current Page", filename = "page",
                    exportOptions = list(
                      modifier = list(page = "current")
                    )
                  ),
                  list(
                    extend = "csv", text = "Download Full Results", filename = "data",
                    exportOptions = list(
                      modifier = list(page = "all")
                    )
                  )
                ),
                scrollY = 350,
                scrollX = 300,
                deferRender = TRUE,
                # scroller = TRUE,
                stateSave = FALSE
              ),
              selection = "none"
            )
          })
        } else {
          sendSweetAlert(session, title = "Warning", type = "warn", text = "Please select at least 1 column!")
        }
      })

      output$ga_select_samp_col <- renderUI({
        selectInput(
          inputId = ns("ga_select_samp_col"),
          label = "Select sample column:",
          choices = c("NONE", input$ga_col_chooser$right),
          selected = "NONE",
          multiple = FALSE
        )
      })
      observeEvent(input$ga_filter_submit_button, {
        rows_filtered <- input$ga_phenotype_data_rows_all
        col_sample <- input$ga_select_samp_col
        keep_samples <- phenotype_table[rows_filtered, , drop = FALSE][[col_sample]]
        message(length(keep_samples), " samples left after filtering.") # 为什么该代码块会被多次调用呢？
        if (length(keep_samples) < 1) {
          sendSweetAlert(session, title = "Error", text = "No samples left!", type = "error")
        } else {
          selected_samps$id <- isolate(keep_samples)
        }
      })
    } else {
      sendSweetAlert(session, title = "Warning", type = "warn", text = "Please select a dataset!")
    }
  })
}

ui.modules_ccle_cor_m2o = function(id) {
	ns = NS(id)
	fluidPage(
		fluidRow(
			# 初始设置
			column(
				3,
				wellPanel(
					style = "height:1100px",
					h2("S1: Preset", align = "center"),

					h4(strong("S1.1 Modify datasets"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Modify datasets", 
					                   content = "data_origin"),
					mol_origin_UI(ns("mol_origin2cor"), database = "ccle"),

					h4(strong("S1.2 Choose sites")),
					pickerInput(
						ns("choose_cancer"),NULL,
						choices = sort(unique(ccle_info_fine$Site_Primary)),
						multiple = TRUE,
						selected = sort(unique(ccle_info_fine$Site_Primary)),
						options = list(`actions-box` = TRUE)
					),
					br(),

					h4(strong("S1.3 Filter samples"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Filter samples", 
					                   content = "choose_samples"),
					h5("Exact filter:"),
					filter_samples_UI(ns("filter_samples2cor"), database = "ccle"),
					br(),
					verbatimTextOutput(ns("filter_phe_id_info")),
					br(),

					h4(strong("S1.4 Upload metadata"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Upload metadata", 
					                   content = "custom_metadata"),
					shinyFeedback::useShinyFeedback(),
					custom_meta_UI(ns("custom_meta2cor")),
					br(),

					h4(strong("S1.5 Add signature"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Add signature", 
					                   content = "add_signature"),
					add_signature_UI(ns("add_signature2cor"), database = "ccle"),
				)
			),
			column(
				4,
				wellPanel(
					style = "height:1100px",
					h2("S2: Get data", align = "center"),
					# 批量数据下载
					h4(strong("S2.1 Get batch data for X-axis")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Get batch data", 
					                   content = "get_batch_data"),
					multi_upload_UI(ns("multi_upload2cor"),
						button_name = "Query", database = "ccle"),
					# br(),br(),
					# 单项数据下载
					h4(strong("S2.2 Get data for Y-axis")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Get one data", 
					                   content = "get_one_data"), 
					download_feat_UI(ns("download_y_axis"), 
						button_name="Query", database = "ccle")
				)
			),
			column(
				5,
				wellPanel(
					style = "height:1100px",
					h2("S3: Analyze", align = "center") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Analyze", 
					                   content = "analyze_cor_3"),  
					# br(),br(),
					h4(strong("S3.1 Set analysis parameters")), 
					selectInput(ns("cor_method"), "Correlation method:",choices = c("pearson", "spearman")),

					shinyWidgets::actionBttn(
						ns("cal_batch_cor"), "Run",
				        style = "gradient",
				        icon = icon("table"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					),
					br(),br(),
					fluidRow(
						column(10, offset = 1,
							   div(uiOutput(ns("cor_stat_tb.ui")),style = "height:600px"),
							   )
					),
					h4(strong("S3.2 Download results")), 
					download_res_UI(ns("download_res2cor"))
				)
			)
		)
	)
}



server.modules_ccle_cor_m2o = function(input, output, session) {
	ns <- session$ns

	# 记录选择癌症
	cancer_choose <- reactiveValues(name = "lung", phe_primary="",
		filter_phe_id=query_tcga_group(database = "ccle", cancer = "lung", return_all = T))
	observe({
		cancer_choose$name = input$choose_cancer
		cancer_choose$phe_primary <- query_tcga_group(database = "ccle",
			cancer = cancer_choose$name, return_all = T)
	})

	# 数据源设置
	opt_pancan = callModule(mol_origin_Server, "mol_origin2cor", database = "ccle")


	# 自定义上传metadata数据
	custom_meta = callModule(custom_meta_Server, "custom_meta2cor", database = "ccle")
	# signature
	sig_dat = callModule(add_signature_Server, "add_signature2cor", database = "ccle")
	custom_meta_sig = reactive({
		if(is.null(custom_meta())){
			return(sig_dat())
		} else {
			if(is.null(sig_dat())){
				return(custom_meta())
			} else {
				custom_meta_sig = dplyr::inner_join(custom_meta(),sig_dat())
				return(custom_meta_sig)
			}
		}
	})

	## 过滤样本
	# exact filter module
	filter_phe_id = callModule(filter_samples_Server, "filter_samples2cor",
					   database = "ccle",
					   cancers=reactive(cancer_choose$name),
					   custom_metadata=reactive(custom_meta_sig()),
					   opt_pancan = reactive(opt_pancan()))
	observe({
		# exact filter
		if(is.null(filter_phe_id())){
			cancer_choose$filter_phe_id = cancer_choose$phe_primary$Sample
		} else {
			cancer_choose$filter_phe_id = filter_phe_id()
		}

		output$filter_phe_id_info = renderPrint({
			cat(paste0("Tip: ", length(cancer_choose$filter_phe_id), " samples are retained"))
		})
	})



	# 批量下载数据
	L3s_x_data =  callModule(multi_upload_Server, "multi_upload2cor", 
							 database = "ccle",
							 samples=reactive(cancer_choose$filter_phe_id),
							 custom_metadata=reactive(custom_meta_sig()),
						     opt_pancan = reactive(opt_pancan())
							 )
	L3s_x = reactive({
		unique(L3s_x_data()$id)
	})

	output$tmp123 = renderPrint({head(L3s_x_data())})
	output$tmp456 = renderPrint({head(L3_y_data())})


	L3_y_data = callModule(download_feat_Server, "download_y_axis", 
							 database = "ccle",
							 samples=reactive(cancer_choose$filter_phe_id),
							 custom_metadata=reactive(custom_meta_sig()),
						     opt_pancan = reactive(opt_pancan()),
						     check_numeric=TRUE
							 )
	# 相关性分析
	cor_stat = eventReactive(input$cal_batch_cor,{
		shinyjs::disable("cal_batch_cor")
		x_datas = L3s_x_data()[,c("id","Sample","value")]
		colnames(x_datas)[c(1,3)] = paste0("x_",colnames(x_datas)[c(1,3)])
		y_data = L3_y_data()[,c("id","Sample","value")]
		colnames(y_data)[c(1,3)] = paste0("y_",colnames(y_data)[c(1,3)])

		withProgress(message = "Please wait for a while.",{
			cor_stat = lapply(seq(L3s_x()), function(i) {
			    incProgress(1 / length(L3s_x()), detail = paste0("(Finished ",i,"/",length(L3s_x()),")"))
				
				L3_x = L3s_x()[i]
				xy_data = x_datas %>%
					dplyr::filter(x_id == L3_x) %>%
					dplyr::inner_join(y_data) %>% as.data.frame()
				if(nrow(na.omit(xy_data))==0){return(c(NaN, NaN))}
			    cor_obj = cor.test(xy_data[,"x_value"],xy_data[,"y_value"],
			                       method = input$cor_method)
			   	return(c(cor_obj$p.value, cor_obj$estimate))
			}) %>% do.call(rbind, .) %>% as.data.frame()
			colnames(cor_stat) = c("p.value","R")
			cor_stat2 = cor_stat %>% 
			  dplyr::select(R, p.value) %>% 
			  dplyr::mutate(id.x = L3s_x(), .before = 1) %>% 
			  dplyr::mutate(id.y = input$L3_y, .before = 2) %>%
			  dplyr::arrange(p.value)
			cor_stat2
		})
		shinyjs::enable("cal_batch_cor")
		cor_stat2
	})

	output$cor_stat_tb.ui = renderUI({
		output$cor_stat_tb = renderDataTable({
			cor_stat_ = cor_stat() %>%
				dplyr::rename("Batch identifiers"="id.x")
			cor_stat_$p.value = format(cor_stat_$p.value, scientific=T, digits = 3)
			dt = datatable(cor_stat_,
				# class = "nowrap row-border",
				options = list(pageLength = 10, 
					columnDefs = list(
						list(className = 'dt-center', targets="_all"),
						list(orderable=TRUE, targets = 0)))
			) %>%
				formatRound(columns = c("R"), digits = 3)
			dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
			dt
		}) 
	dataTableOutput(ns("cor_stat_tb"))
	})

	# Download results
	observeEvent(input$cal_batch_cor,{
		res1 = NULL
		x_datas = L3s_x_data() %>%
			dplyr::mutate(axis = "X")
		y_data = L3_y_data() %>%
			dplyr::select(id, Sample, value) %>%
			dplyr::mutate(axis = "Y")
		res2 = rbind(x_datas, y_data)
		res3 = cor_stat()
		callModule(download_res_Server, "download_res2cor", res1, res2, res3)
	})
}

ui.page_developers <- function() {
  tabPanel(
    title = "Developers",
    icon = icon("user-friends"),
    fluidPage(
      fluidRow(
        column(
          12,
          tags$div(
            class = "card-deck text-center block-center",
            tags$div(
              class = "card bg-info",
              tags$div(
                class = "card-body",
                tags$img(
                  src = "https://avatars3.githubusercontent.com/u/25057508?s=400&v=4.png",
                  class = "img-circle",
                  alt = "logo", style = "width: 30% ; margin: 0 auto "
                ),
                tags$h4(class = "card-title", "Shixiang Wang"),
                tags$p(class = "card-text", "Leader")
              ),
              tags$div(
                class = "card-footer",
                tags$a(href = "https://github.com/ShixiangWang", class = "card-link", "See Profile")
              )
            ),
            tags$div(
              class = "card bg-warning",
              tags$div(
                class = "card-body",
                tags$img(
                  src = "https://avatars3.githubusercontent.com/u/80585985?s=400&v=4.png",
                  class = "img-circle",
                  alt = "logo", style = "width: 30% ; margin: 0 auto "
                ),
                tags$h4(class = "card-title", "Shensuo Li"),
                tags$p(class = "card-text", "Developer of XenaShiny v2")
              ),
              tags$div(
                class = "card-footer",
                tags$a(href = "https://github.com/lishensuo", class = "card-link", "See Profile")
              )
            ),
            tags$div(
              class = "card bg-info",
              tags$div(
                class = "card-body",
                tags$img(
                  src = "https://avatars.githubusercontent.com/u/52995448?v=4",
                  class = "img-circle",
                  alt = "logo", style = "width: 30% ; margin: 0 auto "
                ),
                tags$h4(class = "card-title", "Yuzhong Peng"),
                tags$p(class = "card-text", "Developer of XenaShiny v2 (PharmacoGenomics)")
              ),
              tags$div(
                class = "card-footer",
                tags$a(href = "https://github.com/mugpeng", class = "card-link", "See Profile")
              )
            ),   
            tags$div(
              class = "card bg-warning",
              tags$div(
                class = "card-body",
                tags$img(
                  src = "https://avatars3.githubusercontent.com/u/17489298?s=400&v=4",
                  class = "img-circle",
                  alt = "logo", style = "width: 30% ; margin: 0 auto "
                ),
                tags$h4(class = "card-title", "Fei Zhao"),
                tags$p(class = "card-text", "Counselor")
              ),
              tags$div(
                class = "card-footer",
                tags$a(href = "https://github.com/fei0810", class = "card-link", "See Profile")
              )
            ),
          ),
          tags$br(),
          tags$div(
            class = "card-deck text-center block-center",
            tags$div(
              class = "card bg-info",
              tags$div(
                class = "card-body",
                tags$img(
                  src = "https://avatars0.githubusercontent.com/u/28949856?s=400&v=4",
                  class = "img-circle",
                  alt = "logo", style = "width: 30% ; margin: 0 auto "
                ),
                tags$h4(class = "card-title", "Yi Xiong"),
                tags$p(class = "card-text", "Key developer")
              ),
              tags$div(
                class = "card-footer",
                tags$a(href = "https://github.com/Byronxy", class = "card-link", "See Profile")
              )
            ),
            tags$div(
              class = "card bg-warning",
              tags$div(
                class = "card-body",
                tags$img(
                  src = "https://avatars2.githubusercontent.com/u/37660840?s=400&v=4",
                  class = "img-circle",
                  alt = "logo", style = "width: 30% ; margin: 0 auto "
                ),
                tags$h4(class = "card-title", "Longfei Zhao"),
                tags$p(class = "card-text", "Key developer")
              ),
              tags$div(
                class = "card-footer",
                tags$a(href = "https://github.com/longfei8533", class = "card-link", "See Profile")
              )
            ),
            tags$div(
              class = "card bg-info",
              tags$div(
                class = "card-body",
                tags$img(
                  src = "https://avatars0.githubusercontent.com/u/38618580?s=400&v=4",
                  class = "img-circle",
                  alt = "logo", style = "width: 30% ; margin: 0 auto "
                ),
                tags$h4(class = "card-title", "Yin Li"),
                tags$p(class = "card-text", " Developer")
              ),
              tags$div(
                class = "card-footer",
                tags$a(href = "https://github.com/yinlisssss", class = "card-link", "See Profile")
              )
            ),
            tags$div(
              class = "card bg-warning",
              tags$div(
                class = "card-body",
                tags$img(
                  src = "https://avatars2.githubusercontent.com/u/22772592?s=400&v=4",
                  class = "img-circle",
                  alt = "logo", style = "width: 30% ; margin: 0 auto "
                ),
                tags$h4(class = "card-title", "Kai Gu"),
                tags$p(class = "card-text", "Key developer")
              ),
              tags$div(
                class = "card-footer",
                tags$a(href = "https://github.com/kaigu1990", class = "card-link", "See Profile")
              )
            )
          )
        )
      )
    )
  )
}

custom_meta_UI = function(id, button_name="Multi-conditions filters"){
	ns = NS(id)
	tagList(
		fluidRow(
			column(8,fileInput(ns("upload_sp_info"),"", accept = ".csv")),
			column(3,br(),downloadButton(ns("example_sp_info"), "e.g."))
		)
	)
}



custom_meta_Server = function(input, output, session, database = "toil"){
	ns <- session$ns
	custom_meta = reactive({
		file = input$upload_sp_info
		if(is.null(file$datapath)){
			NULL
		} else {
			csv_format = tools::file_ext(file$name)=="csv"
			shinyFeedback::feedbackDanger("upload_sp_info", !csv_format, "Non .csv format file")
			req(csv_format,cancelOutput = TRUE)
			read.csv(file$datapath)
		} 
	})
	output$example_sp_info = downloadHandler(
		filename = function(){
			"example_sample_info.csv"
		},
		content = function(file){
			sp_info = query_tcga_group(database = database)$data[,"Sample"]
			set.seed(42)
			scores = matrix(rnorm(nrow(sp_info)*5,mean = 1, sd = 1), ncol = 5) %>% as.data.frame()
			colnames(scores) = paste0("TF",1:5)
			sp_info = cbind(sp_info, scores)
			write.csv(sp_info, file, row.names = FALSE)
		}
	)
	return(custom_meta)
}

# Obtain path to individual server code parts ----------------------------
server_file <- function(x) {
  server_path <- system.file("shinyapp", "server",
    package = "UCSCXenaShiny", mustWork = TRUE
  )
  file.path(server_path, x)
}


# Set utility functions ---------------------------------------------------
QUERY_CACHE <- dplyr::tibble()
xe_query_url <- function(data, use_cache = TRUE) {
  if (use_cache) {
    if (nrow(QUERY_CACHE) == 0) {
      non_exist_idx <- !data$XenaDatasets %in% NULL
    } else {
      non_exist_idx <- !data$XenaDatasets %in% QUERY_CACHE$datasets
    }
    if (any(non_exist_idx)) {
      non_exist_query <- xe_query_url(data[non_exist_idx, , drop = FALSE], use_cache = FALSE)
      QUERY_CACHE <<- dplyr::bind_rows(
        QUERY_CACHE,
        non_exist_query
      )
    }

    xe_query <- dplyr::filter(QUERY_CACHE, QUERY_CACHE$datasets %in% data$XenaDatasets)
  } else {
    xe <-
      UCSCXenaTools::XenaGenerate(subset = XenaDatasets %in% data$XenaDatasets)

    xe_query <- UCSCXenaTools::XenaQuery(xe)
    xe_query$browse <- purrr::map2(
      xe_query$datasets, xe_query$hosts,
      ~ utils::URLencode(
        paste0(
          "https://xenabrowser.net/datapages/?",
          "dataset=", .x, "&host=", .y
        )
      )
    ) %>% unlist()
  }

  return(xe_query)
}

get_data_df <- function(dataset, id) {
  if (dataset == "custom_phenotype_dataset") {
    message("Loading custom phenotype data.")
    df <- readRDS(file.path(tempdir(), "custom_phenotype_data.rds"))
  } else {
    message("Querying data of identifier ", id, " from dataset ", dataset)
    id_value <- if (dataset == "custom_feature_dataset") {
      UCSCXenaShiny:::query_custom_feature_value(id)
    } else {
      UCSCXenaShiny::query_molecule_value(dataset, id)
    }
    df <- dplyr::tibble(
      sample = names(id_value),
      X = as.numeric(id_value)
    )
    colnames(df)[2] <- id 
  }
  df
}




useShinydashboard2 <- function() {
  if (!requireNamespace(package = "shinydashboard"))
    message("Package 'shinydashboard' is required to run this function")
  deps <- htmltools::findDependencies(shinydashboard::dashboardPage(
    header = shinydashboard::dashboardHeader(),
    sidebar = shinydashboard::dashboardSidebar(),
    body = shinydashboard::dashboardBody()
  ))
  htmltools::attachDependencies(tags$div(class = "main-sidebar", style = "display: none;"), value = deps)
}




# vis_gene_cross_omics = function(gene="TP53", 
#                                 tumor_samples = NULL,
#                                 n_trans = 5, n_methy = 5,seed = 42,
#                                 return_list = FALSE){
#   if(is.null(tumor_samples)){
#     tcga_gtex2 = tcga_gtex
#     tcga_clinical_fine2 = tcga_clinical_fine
#   } else {
#     tcga_gtex2 = tcga_gtex %>% 
#       dplyr::filter(type2=="normal" | sample %in% tumor_samples)
#     tcga_clinical_fine2 = tcga_clinical_fine %>% 
#       dplyr::filter(Code=="NT" | Sample %in% tumor_samples)
#   }
#   #### Omics--mRNA
#   gene_mrna = query_pancan_value(gene, "mRNA") %>% 
#     .[["expression"]] %>% as.data.frame() %>% 
#     dplyr::rename('Exp'='.') %>% 
#     tibble::rownames_to_column("sample") %>% 
#     dplyr::inner_join(tcga_gtex2)
#   if(all(is.na(gene_mrna$Exp))) stop("Please input a valid gne!")
  
#   #median value, normal samples include GTEx
#   gene_mrna_md = gene_mrna %>% 
#     dplyr::group_by(tissue, type2) %>% 
#     dplyr::summarise(Exp = median(Exp)) %>% as.data.frame() %>% 
#     dplyr::mutate(Exp = (Exp-min(Exp))/(max(Exp)-min(Exp))) %>%  # 0~1 scale
#     tidyr::pivot_wider(id_cols = tissue, names_from = type2, values_from = Exp)
  
#   gene_mrna_comp = compare_means(Exp ~ type2, data = gene_mrna,
#                                  group.by = "tissue", method = "wilcox.test") 
  
#   gene_mrna_plot = dplyr::left_join(gene_mrna_md, gene_mrna_comp[,c("tissue","p","p.adj")]) %>% 
#     as.data.frame() %>% 
#     dplyr::rename("id" = "tissue") %>% 
#     dplyr::mutate(Label = case_when(
#       p < 0.01 & tumor>normal ~ "Up-Reg.",
#       p < 0.01 & tumor<normal ~ "Down-Reg.",
#       TRUE~"Non-Sig."
#     )) %>% 
#     dplyr::select(id, normal, tumor, Label)
  
#   #### Omics--mutation
#   gene_mut = query_pancan_value(gene, "mutation") %>% 
#     as.data.frame() %>% 
#     dplyr::rename("Mut"=".") %>% 
#     tibble::rownames_to_column("sample") %>% 
#     dplyr::inner_join(tcga_clinical_fine2, by=c("sample"="Sample"))
  
#   gene_mut_plot = gene_mut %>% 
#     dplyr::group_by(Cancer) %>% 
#     dplyr::summarise(Mut = mean(Mut)) %>% 
#     as.data.frame() %>% 
#     dplyr::mutate(Ratio = NA)
  
#   for(i in seq(nrow(gene_mut_plot))){
#     # i = 1
#     gene_mut_plot$Ratio[i] = list(c(Mut=gene_mut_plot$Mut[i], Wild=1-gene_mut_plot$Mut[i]))
#   }
  
#   gene_mut_plot = gene_mut_plot %>% 
#     dplyr::mutate(Mut_str = scales::percent(Mut,accuracy = 0.1)) %>% 
#     dplyr::select(Cancer, Ratio, Mut, Mut_str)
  
#   #### Omics--CNV
#   opt_pancan = .opt_pancan
#   opt_pancan$toil_cnv$use_thresholded_data=T
#   gene_cnv = query_pancan_value(
#     gene, "cnv", opt_pancan = opt_pancan
#   ) %>% as.data.frame() %>% 
#     dplyr::select(data) %>% 
#     dplyr::rename('CNV'='data') %>% 
#     tibble::rownames_to_column("sample") %>% 
#     dplyr::inner_join(tcga_clinical_fine2, by=c("sample"="Sample"))
  
#   gene_cnv_tb = table(gene_cnv$Cancer, gene_cnv$CNV)
#   gene_cnv_plot = gene_cnv %>% 
#     dplyr::count(Cancer, name = "CNV_total") %>% 
#     dplyr::mutate(CNV_Pie=NA, CNV_Del = NA, CNV_Amp = NA)
  
#   for (i in seq(nrow(gene_cnv_plot))){
#     # i = 1
#     Cancer = gene_cnv_plot$Cancer[i]
#     gene_cnv_plot$CNV_Pie[i] = list(c(
#       ' -2' = gene_cnv_tb[Cancer,"-2"],
#       ' -1' = gene_cnv_tb[Cancer,"-1"],
#       ' 0' = gene_cnv_tb[Cancer,"0"],
#       ' 1' = gene_cnv_tb[Cancer,"1"],
#       ' 2' = gene_cnv_tb[Cancer,"2"]
#     ))
#     gene_cnv_plot$CNV_Del[i] = gene_cnv_tb[Cancer,"-2"] + gene_cnv_tb[Cancer,"-1"]
#     gene_cnv_plot$CNV_Amp[i] = gene_cnv_tb[Cancer,"2"] + gene_cnv_tb[Cancer,"1"]
#   }
  
#   gene_cnv_plot = gene_cnv_plot %>%
#     dplyr::mutate(CNV_Amp = CNV_Amp/CNV_total) %>% 
#     dplyr::mutate(CNV_Amp_str = scales::percent(CNV_Amp, accuracy = 0.1)) %>% 
#     dplyr::mutate(CNV_Del = CNV_Del/CNV_total) %>% 
#     dplyr::mutate(CNV_Del_str = scales::percent(CNV_Del, accuracy = 0.1)) %>% 
#     dplyr::select(Cancer, CNV_Pie, CNV_Amp, CNV_Amp_str, CNV_Del, CNV_Del_str)
  
#   #### Omics--Transcript
#   Trans_sub = load_data("v2_tpc_id_help")$tcga$id_trans %>% 
#     dplyr::filter(Symbol == gene) %>% 
#     dplyr::pull(Level3)
#   set.seed(seed)
#   if (class(n_trans)=="character"){
#     if(length(n_trans)>10) stop("Less than 10 CpG sites are supported!")
#     trans_sle = n_trans[n_trans %in% Trans_sub]
#   } else if (class(n_trans)=="numeric"){
#     if(n_trans>10) stop("Less than 10 transcripts are supported!")
#     trans_sle = unique(sample(Trans_sub, n_trans, replace = n_trans>length(Trans_sub)))
#   }
  
#   if(all(is.na(n_trans)) | length(trans_sle)==0){ 
#     gene_trans_n = 0 
#   } else {
#     gene_trans = lapply(seq(trans_sle), function(i){
#       # i = 1
#       gene_trans_tmp = query_pancan_value(
#         trans_sle[i], "transcript"
#       ) %>% as.data.frame() %>% 
#         dplyr::select(expression) %>% 
#         dplyr::rename("Trans"="expression") %>% 
#         tibble::rownames_to_column("sample") %>% 
#         dplyr::inner_join(tcga_gtex2) %>% 
#         dplyr::filter(type2=="tumor") %>%
#         dplyr::group_by(tissue) %>% 
#         dplyr::summarise(Trans = median(Trans))
#       colnames(gene_trans_tmp)[2] = trans_sle[i]
#       gene_trans_tmp %>% tibble::column_to_rownames("tissue") 
#     }) %>% do.call(cbind, .)
#     gene_trans_plot = gene_trans[,apply(gene_trans,2,sd)!=0,drop=FALSE] %>% 
#       as.data.frame() %>% 
#       tibble::rownames_to_column("tissue")
#     gene_trans_n = ncol(gene_trans_plot)-1
#   }
  
  
#   #### Omics--Methylation
#   Methy450_sub = load_data("v2_tpc_id_help")$tcga$id_M450 %>% 
#     dplyr::filter(Level3 == gene) %>% 
#     dplyr::pull(CpG)
#   set.seed(seed)
#   if (class(n_methy)=="character"){
#     if(length(n_methy)>10) stop("Less than 10 CpG sites are supported!")
#     cpg_sites = n_methy[n_methy %in% Methy450_sub]
#   } else if (class(n_methy)=="numeric"){
#     if(n_methy>10) stop("Less than 10 CpG sites are supported!")
#     cpg_sites = unique(sample(Methy450_sub, n_methy, replace = n_methy>length(Methy450_sub)))
#   }
#   if(all(is.na(n_methy)) | length(cpg_sites)==0){ 
#     gene_methy_n = 0 
#   } else {
#     gene_methy_cpgs = lapply(seq(cpg_sites), function(i){
#       # i = 1
#       opt_pancan = .opt_pancan
#       opt_pancan$toil_methylation$rule_out = setdiff(Methy450_sub, cpg_sites[i])
#       opt_pancan$toil_methylation$aggr = "mean"
#       gene_methy_tmp = query_pancan_value(
#         gene, "methylation", opt_pancan = opt_pancan
#       ) %>% as.data.frame() %>% 
#         dplyr::select(data) %>% 
#         dplyr::rename("Methy"='data') %>% 
#         tibble::rownames_to_column("sample") %>% 
#         # dplyr::inner_join(tcga_gtex2) %>% 
#         dplyr::inner_join(tcga_clinical_fine2, by=c("sample"="Sample")) %>% 
#         dplyr::filter(Code!="NT") %>% 
#         dplyr::group_by(Cancer) %>% 
#         dplyr::summarise(Methy = median(Methy))
#       colnames(gene_methy_tmp)[2] = cpg_sites[i]
#       gene_methy_tmp %>% tibble::column_to_rownames("Cancer") 
#     }) %>% do.call(cbind, .) %>% 
#       tibble::rownames_to_column("Cancer")
#     gene_methy_plot = gene_methy_cpgs
#     gene_methy_n = ncol(gene_methy_plot)-1
#   }
  
#   #### Ready for Plot
#   gene_cross_dat = data.frame(id = as.character(sort(unique(tcga_gtex$tissue)))) %>% 
#     dplyr::left_join(gene_mrna_plot) %>% 
#     dplyr::left_join(gene_mut_plot, by=c("id"="Cancer")) %>% 
#     dplyr::left_join(gene_cnv_plot, by=c("id"="Cancer"))
    
#   idx_1 = which(sapply(gene_cross_dat$Ratio, is.null))
#   for(i in idx_1){
#     gene_cross_dat$Ratio[[i]] = c(Mut=NA, Wild=NA)
#   }
#   idx_2 = which(sapply(gene_cross_dat$CNV_Pie, is.null))
#   for(i in idx_2){
#     gene_cross_dat$CNV_Pie[[i]] = c(` -2`=NA, ` -1`=NA, ` 0`=NA, ` 1`=NA, ` 2`=NA)
#   }  
  
#   if(gene_trans_n>0){
#     gene_cross_dat = dplyr::left_join(gene_cross_dat, gene_trans_plot, by=c("id"="tissue"))
#   }
#   if(gene_methy_n>0){
#     gene_cross_dat = dplyr::left_join(gene_cross_dat, gene_methy_plot, by=c("id"="Cancer"))
#   }
  
  
  
#   label_name = colnames(gene_cross_dat)
#   label_name[1:12] = c("TCGA","Normal Exp","Tumor Exp","Tumor vs. Normal", 
#                        "Mutation Dist", "Mutation PCT", "",
#                        "CNV Dist", "CNV Amp", "","CNV Del", "")
  
#   column_info = data.frame(
#     # columns to vis
#     id = colnames(gene_cross_dat),
#     # column name
#     name = label_name, 
#     # group name
#     group = c(NA, rep("Gene Expression", 3), rep("Gene Mutation", 3), rep("Gene CNV", 5), 
#               rep("Gene Transcript", gene_trans_n), rep("Gene Methylation", gene_methy_n)),
#     # geom type
#     geom = c("text","bar","bar","text",   "pie","bar","text",
#              "pie","bar","text","bar","text",
#              rep("funkyrect", gene_trans_n), rep("funkyrect", gene_methy_n)),
#     # palette type
#     palette = c(NA,"Tumor-Exp","Tumor-Exp",NA ,"Mut-Pie","Mut-Ratio",NA, 
#                 "CNV-Pie","CNV-Amp",NA,"CNV-Del",NA, 
#                 rep("Scale.Transcript", gene_trans_n), rep("Scale.Methylation",gene_methy_n)), 
#     # column width
#     width = c(4, 4, 4, 4,   2, 4, 4, 
#               2, 4, 4, 4, 4, rep(1.1, gene_trans_n), rep(1.1, gene_methy_n))
#   )
  
#   column_info$options = NA
#   for(i in seq(nrow(column_info))){
#     column_info$options[i] = list(list())
#   }
#   column_info$options[[2]] = list(scale=F,hjust=1)
#   column_info$options[[3]] = list(scale=F)
#   column_info$options[[6]] = list(scale=F)
#   column_info$options[[7]] = list(label="Mut_str",overlay=T)
#   column_info$options[[9]] = list(scale=F)
#   column_info$options[[10]] = list(label="CNV_Amp_str",overlay=T)
#   column_info$options[[11]] = list(scale=F)
#   column_info$options[[12]] = list(label="CNV_Del_str",overlay=T)
  
#   column_groups = data.frame(
#     group = c("Gene Expression", "Gene Mutation", "Gene CNV", "Gene Methylation", 'Gene Transcript'),
#     palette = c("palette1", "palette2", "palette3","palette4", "palette5")
#   )
  
#   palettes = list(
#     palette1 = c("#66c2a5","#66c2a5"),
#     palette2 = c("#fc8d62","#fc8d62"),
#     palette3 = c("#8da0cb","#8da0cb"),
#     palette4 = c("#e78ac3","#e78ac3"),
#     palette5 = c("#a6d854","#a6d854"),
#     `Tumor-Exp` = RColorBrewer::brewer.pal(n = 9, name = "Greys")[1:6],
#     `Mut-Ratio` = RColorBrewer::brewer.pal(n = 9, name = "YlOrRd"),
#     `Mut-Pie` = c(Mut="red", Wild="grey"),
#     `CNV-Pie` = c(' -2'='#5e3c99',' -1'='#b2abd2',' 0'='#f7f7f7',' 1'='#fdb863',' 2'='#e66101'),
#     `CNV-Amp` = RColorBrewer::brewer.pal(n = 9, name = "Oranges"),
#     `CNV-Del` = RColorBrewer::brewer.pal(n = 9, name = "Purples"),
#     Scale.Methylation = RColorBrewer::brewer.pal(n = 9, name = "BuPu"),
#     Scale.Transcript = RColorBrewer::brewer.pal(n = 9, name = "Greens")
#   )
  
#   position_arguments = position_arguments(expand_xmax  = 6, 
#                                           col_annot_angle = 35, 
#                                           col_width = 1.1,
#                                           col_space = 0.2,
#                                           col_annot_offset = 4)
  
#   g = funky_heatmap(gene_cross_dat, 
#                     column_info = column_info,
#                     column_groups = column_groups,
#                     palettes = palettes,
#                     position_args = position_arguments)
  
  
#   if(return_list){
#     return(list(plot = g, data = gene_cross_dat))
#   } else {
#     return(g)
#   }
# }

ui.file_upload <- function(id, test = FALSE) {
  ns <- NS(id)

  wPanel <- wellPanel(
    # Input: Select a file ----
    fileInput(ns("upload_file"), "Choose CSV File",
      multiple = FALSE,
      accept = c(
        "text/csv",
        "text/comma-separated-values,text/plain",
        ".csv"
      )
    ),

    # Horizontal line ----
    tags$hr(),

    # Input: Checkbox if file has header ----
    checkboxInput(ns("header"), "Header", TRUE),

    # Input: Select separator ----
    radioButtons(ns("sep"), "Separator",
      choices = c(
        Comma = ",",
        Semicolon = ";",
        Tab = "\t"
      ),
      selected = ","
    ),

    # Input: Select quotes ----
    radioButtons(ns("quote"), "Quote",
      choices = c(
        None = "",
        "Double Quote" = '"',
        "Single Quote" = "'"
      ),
      selected = '"'
    ),

    # Horizontal line ----
    tags$hr(),

    # Input: Select number of rows to display ----
    radioButtons(ns("disp"), "Display",
      choices = c(
        Head = "head",
        All = "all"
      ),
      selected = "head"
    )
  )

  if (test) {
    fluidPage(
      titlePanel("Module: Upload File (here for testing only)"),
      sidebarLayout(
        sidebarPanel = sidebarPanel(
          wPanel
        ),
        mainPanel = mainPanel(
          # Output: Data file ----
          tableOutput(ns("contents"))
        )
      )
    )
  } else {
    wPanel
  }
}

server.file_upload <- function(input, output, session) {
  ## https://shiny.rstudio.com/articles/modules.html
  # ns <- session$ns

  userFile <- reactive({
    # If no file is selected, don't do anything
    validate(need(input$upload_file, message = FALSE))
    input$upload_file
  })

  output$contents <- renderTable({

    # input$file1 will be NULL initially. After the user selects
    # and uploads a file, head of that data file by default,
    # or all rows if selected, will be shown.
    message("rendering table")

    # when reading semicolon separated files,
    # having a comma separator causes `read.csv` to error
    tryCatch(
      {
        df <- read.csv(userFile()$datapath,
          header = input$header,
          sep = input$sep,
          quote = input$quote,
          stringsAsFactors = FALSE
        )
      },
      error = function(e) {
        # return a safeError if a parsing error occurs
        stop(safeError(e))
      }
    )

    if (input$disp == "head") {
      return(head(df))
    }
    else {
      return(df)
    }
  })

  # We can run observers in here if we want to
  observe({
    msg <- sprintf("File %s was uploaded", userFile()$name)
    cat(msg, "\n")
  })
}

ui.modules_1_tcga_11 = function(id){
    ns = NS(id)

    main_ui = tagList(
        h4("1. Select omics type"),
        selectInput(
            ns("profile"), NULL,
            choices = c("mRNA Expression", "Transcript Expression", "DNA Methylation", 
                        "Protein Expression", "miRNA Expression", "Copy Number Variation"),
            selected = "mRNA Expression"),

        h4("2. Input the molecule ids (>=3) by ?"),
        shinyWidgets::prettyRadioButtons(
            inputId = ns("input_ways"), label = NULL,
            choiceValues = c("Selection", "Pathway", "File"),
            choiceNames = c("Selection", "Pathway", "File"),
            animation = "jelly",
            inline = TRUE
        ),
        tabsetPanel(
            id = ns("input_params"),
            type = "hidden",
            tabPanel("Selection",
                virtualSelectInput(
                inputId = ns("ids_ways_1"),
                label = NULL,
                choice = NULL,
                width = "100%",
                multiple = TRUE, 
                search = TRUE,
                allowNewOption = TRUE,
                dropboxWidth = "200%")
            ),
            tabPanel("Pathway",
                fluidRow(
                    column(4,
                        virtualSelectInput(
                            inputId = ns("msigdbr_cat"),
                            label = NULL,
                            choices = msigdbr_types$gs_subcat_label, 
                            selected =  msigdbr_types$gs_subcat_label[1],
                            dropboxWidth = "200%")
                    ),
                    column(8,
                        virtualSelectInput(
                            inputId = ns("msigdbr_pw"),
                            label = NULL,
                            choices = NULL, 
                            selected =  NULL,
                            search = TRUE,
                            dropboxWidth = "200%")
                    )
                ),
                uiOutput(ns("msigdb_note.ui"))
            ),
            tabPanel("File",
                fluidRow(
                    column(8, fileInput(ns("ids_ways_3"),NULL, accept = ".txt")),
                    column(4, downloadButton(ns("dw_L3_x"), "e.g."))
                ),
                # p("Keys: (1) TXT format; (2) One column without colname")
            )
        ),  
        shinyWidgets::actionBttn(
            inputId = ns("query_data"),
            label = "Cache data",
            style = "gradient",
            color = "primary",
            block = TRUE,
            size = "sm"
        ),
		verbatimTextOutput(ns("tip_s1")),
        h4("3. Select TCGA sample range and grouping method"),
        shinyWidgets::prettyRadioButtons(
            inputId = ns("group_ways"), label = NULL,
            choiceValues = c("Preset Group", "Custom Group"),
            choiceNames = c("Preset Group", "Custom Group"),
            animation = "jelly",
            inline = TRUE
        ),
        tabsetPanel(
            id = ns("group_params"),
            type = "hidden",
            tabPanel(
                "Preset Group",
                fluidRow(
                    column(6,
                        virtualSelectInput(
                            ns("choose_cancer"), "Choose cancer(s)",
                            choices = sort(tcga_names),
                            multiple = TRUE,
                            selected = "BRCA"
                        )
                    ),
                    column(6,
                        virtualSelectInput(
                            ns("choose_group"), "Choose grouping variable",
                            choices = colnames(tcga_clinical_fine)[-1],
                            selected = "Code",
                            optionsCount = 4
                        )
                    )
                )
            ),
            tabPanel(
                "Custom Group",
                # br(),
                fluidRow(
                    column(
                        8,
                        fileInput(ns("upload_file"), NULL, accept = ".csv"),
                    ),
                    column(
                        3,
                        downloadButton(ns("eg_file"), "e.g.")
                    ),
                ),
            )
        ),
        h4("4. Select Dimension-Reduction method"),
        awesomeRadio(
            inputId = ns("method"),
            label = NULL, 
            choices = c("PCA", "UMAP", "tSNE"),
            selected = "PCA",
            inline = TRUE, checkbox = TRUE
        ),
        shinyWidgets::actionBttn(
            inputId = ns("search_bttn"),
            label = "Go!",
            style = "gradient",
            icon = icon("search"),
            color = "primary",
            block = TRUE,
            size = "sm"
        ),
		verbatimTextOutput(ns("tip_s2"))
    )
    out_ui = tagList(
        fluidRow(
            uiOutput(ns("dim_plot"))
        )
    )
    side_ui = tagList(
        fluidRow(
            column(6,
                h4("1. Visualization parameters"),
                h5("(1) Marginal plot:"),
                selectInput(ns("margin"), "Marginal plot",
                    c('NONE', "density" ,"boxplot")),
                h5("(2) Color palette:"),
                selectInput(ns("palette"), "Color setting",
                    c("Set1", "Set2", "Set3", "Paired", "Pastel1", "Pastel2", "Accent", "Dark2"))
            ),
            column(6,
                h4("2. Download options"),
                h5("(1) Figure:"),
                numericInput(inputId = ns("height"), label = "Height", value = 8),
                numericInput(inputId = ns("width"), label = "Width", value = 8),
                awesomeRadio(ns("device"), label = "Format", 
                    choices = c("pdf", "png"), selected = "pdf", inline = TRUE),
                downloadBttn(
                  outputId = ns("download_1"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                ),
                h5("(2) Data table:"),
                downloadBttn(
                  outputId = ns("download_2"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                )
            )
        )
    )
    fluidPage(
        wellPanel(style = "height:725px",
        box(main_ui,
            width = 5,
            solidHeader = TRUE,
            title = "Quick TCGA Analysis: Dimensionality reduction analysis", 
            status = "warning",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Click the bottom button to execute/update the analysis."
        ),
        box(out_ui,
            width = 7,
            solidHeader = TRUE,
            title = "Analytical results:", 
            status = "warning",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Pull the sidebar to adjust plot parameters or download results through the top-right widget.",
            sidebar = boxSidebar(
                        id = ns("sidebar"),
                        width = 50,
                        side_ui
            )
        )
        )
    )
}

server.modules_1_tcga_11 = function(input, output, session){
    ns = session$ns

	profile = reactive({
	  switch(input$profile,
	    `mRNA Expression`="mRNA",
	    `DNA Methylation` = "methylation",
	    `Protein Expression` = "protein",
	    `Transcript Expression` = "transcript",
	    `miRNA Expression` = "miRNA",
	    `Copy Number Variation` = "cnv",
	    list(all = "NONE", default = "NONE")
	  )
	})

	profile_choices <- reactive({
	  switch(profile(),
	    mRNA = list(all = tcga_id.list[["Gene"]], default = c("TP53", "KRAS", "PTEN")),
	    methylation = list(all = tcga_id.list[["Gene"]], default = c("TP53", "KRAS", "PTEN")),
	    protein = list(all = tcga_id.list[["Protein"]], default = c("P53", "GATA3", "PTEN")),
	    transcript = list(all = tcga_id.list[["Transcript"]], default = c("ENST00000269305","ENST00000311936","ENST00000371953")),
	    miRNA = list(all = tcga_id.list[["miRNA"]], default = c("hsa-miR-522-3p","hsa-miR-1271-5p","hsa-miR-518e-3p")),
	    cnv = list(all = tcga_id.list[["Gene"]],default = c("TP53", "KRAS", "PTEN")),
	    list(all = "NONE", default = "NONE")
	  )
	})

	observe({
	  updateVirtualSelect(
	    "ids_ways_1",
	    choices = profile_choices()$all,
	    selected = profile_choices()$default
	  )
	})

	observeEvent(input$input_ways, {
	  updateTabsetPanel(inputId = "input_params", selected = input$input_ways)
	}) 
	observeEvent(input$group_ways, {
	  updateTabsetPanel(inputId = "group_params", selected = input$group_ways)
	}) 


    ## MsigDB通路数据库
	msigdbr_query = reactive({
		category = msigdbr_types$gs_cat[msigdbr_types$gs_subcat_label==input$msigdbr_cat]
		subcategory = msigdbr_types$gs_subcat[msigdbr_types$gs_subcat_label==input$msigdbr_cat]
		if(length(category)!=0){
			msigdbr_query = msigdbr(species = "Homo sapiens", 
			                      category = category, 
			                      subcategory = subcategory)
			msigdbr_query
		}
	})
	observe({
		if(!is.null(msigdbr_query())){
			msigdbr_term_stat = as.data.frame(table(msigdbr_query()$gs_name)) %>% 
			  dplyr::rename(term=Var1, size=Freq) %>% 
			  dplyr::arrange(term) %>% 
			  dplyr::mutate(term_size = paste0(term," (", size,")"))
			updateVirtualSelect(
			  "msigdbr_pw",
			  choices = msigdbr_term_stat$term_size,
			  selected = msigdbr_term_stat$term_size[1]
			)
		}
	})
	output$msigdb_note.ui = renderUI({
		# pw_sle = ifelse(is.null(input$msigdbr_pw),"HALLMARK_ADIPOGENESIS",input$msigdbr_pw)
		pw_sle = str_split(input$msigdbr_pw," ")[[1]][1]
		term_link = sprintf("https://www.gsea-msigdb.org/gsea/msigdb/human/%s.html",
                    		ifelse(is.null(pw_sle),"HALLMARK_ADIPOGENESIS",pw_sle))
		msigdb_link = "https://www.gsea-msigdb.org/gsea/msigdb/human/collection_details.jsp"

		p("Link to ",a("the pathway", href = term_link)," from ", a("MSigDB database", href = msigdb_link), ":")

	})

    # 示例id数据下载
	output$dw_L3_x = downloadHandler(
		filename = function(){
			"sample_multiple_ids.txt"
		},
		content = function(file){
			set.seed(42)
			all_ids = tcga_id_option[["Molecular profile"]][[input$profile]]$all
			sample_ids = sample(all_ids,ifelse(length(all_ids)>10,10,length(all_ids)))
			write.table(sample_ids, file,
				quote = F, row.names = F, col.names = F)
		}
	)

    # 多个id的选择
    ids = eventReactive(input$query_data, {
    	if(input$input_ways=="Selection"){
    		ids = input$ids_ways_1
    	} else if (input$input_ways=="File") {
            req(input$ids_ways_3)
			file = input$ids_ways_3
			if(is.null(file$datapath)){  # 如果空文件
				ids = NULL
			} else {
				ids = read.table(file$datapath)[,1]
				ids = ids[ids %in% tcga_id_option[["Molecular profile"]][[input$profile]]$all]
				if(length(ids)>1000){
					L3s_x = L3s_x[1:1000]
				} 
			}
    	} else if (input$input_ways=="Pathway") {
    		ids = tcga_id_option[["Molecular profile"]][[input$profile]]$all
			pw_genes = msigdbr_query() %>% 
			  dplyr::filter(gs_name %in% str_split(input$msigdbr_pw," ")[[1]][1]) %>% 
			  dplyr::pull(gene_symbol)
			if(input$profile %in% 
				c("mRNA Expression","DNA Methylation","Mutation status","Copy Number Variation")){
				ids = ids[ids %in% pw_genes]
			} else if(L2_x() %in% c("Transcript Expression")){
				if(!exists("tcga_id_referrence")){
					message("Loading \"pancan_identifier_help\"")
					tcga_id_referrence = load_data("pancan_identifier_help")
				}
				ids = ids[ids %in% tcga_id_referrence[[1]][[5]]$Level3[tcga_id_referrence[[1]][[5]]$Symbol %in% pw_genes]]
			}
    	}
	    unique(ids)
	})

    # 缓存id数据
	cache_dat = eventReactive(input$query_data, {
        if(length(ids()))
		withProgress(message=paste0("Query ", length(ids()), " molecules one by one"),{
		  data = purrr::map(ids(), function(x) {
		    # x = ids[1]
		    data <- query_pancan_value(x, data_type=profile())
		    data = data[[1]]
		    data <- dplyr::tibble(sample = names(data), y = as.numeric(data))
		    colnames(data)[2] <- x
		    incProgress(1 / length(ids()))
		    data
		  }) %>% purrr::reduce(dplyr::full_join, by = "sample")
	    })
	    data = data[, apply(data, 2, function(m) {!all(is.na(m))})]
	    return(data)
	})

	output$tip_s1 = renderPrint({
		cat(paste0("Tips: ", ncol(cache_dat())-1, " valid identifiers are cached."))
	})



	## Group分组信息
	output$eg_file = downloadHandler(
		filename = function(){
			"example_group.csv"
		},
		content = function(file){
			group_info = tcga_clinical_fine %>% 
			  dplyr::filter(Cancer == "BRCA") %>% 
			  dplyr::filter(Code %in% c("TP","NT")) %>% 
			  dplyr::select(Sample, Code)
			colnames(group_info)[2] = "Group"
			write.csv(group_info, file, quote = F, row.names = F)
		}
	)

	group_info = eventReactive(input$search_bttn,{
		if(input$group_params=="Preset Group"){
			group_info = tcga_clinical_fine %>% 
			  dplyr::filter(Code %in% c("NT", "TP")) %>%
			  dplyr::mutate(Code = ifelse(Code=="TP","tumor","normal")) %>%
			  dplyr::mutate(Age = ifelse(Age>60, "Old(>60)", "Young(<60)")) %>%
			  dplyr::filter(Cancer == input$choose_cancer)
			# only tumor samples when non-code group
			if(input$choose_group!="Code"){
				group_info = group_info %>%
					dplyr::filter(Code=="tumor")
			}
			group_info = group_info[,c("Sample", input$choose_group)] %>% na.omit()
			colnames(group_info)[2] = "Group"


		} else if (inparams=="Custom Group"){
			if(!is.null(input$upload_file$datapath)){
				group_info = read.csv(input$upload_file$datapath) %>% na.omit()
			} else {
				group_info = data.frame("Sample"=NULL, "Group"=NULL)
			}
		}
		shiny::validate(
			need(try(nrow(group_info)>0), 
				"Error: No group infomation is detected.")
		)
		groups = length(unique(group_info$Group))
		shiny::validate(
			need(try(groups>=2 & groups<=10), 
				"Error: Too few (<2) or much (>10) groups are detected.")
		)
		group_info
	})

	output$tip_s2 = renderPrint({
		cat(paste0("Tips: ", nrow(group_info())," samples with ", 
			length(unique(group_info()$Group))," groups are detected."))
	})

	plot_func <- eventReactive(input$search_bttn, {
      id <- showNotification(h3("The task is running..."), duration = NULL, closeButton = FALSE, type = "message")
      on.exit(removeNotification(id), add = TRUE)  #reactive语句执行完毕时，运行remove命令
      print(head(group_info()))
	  p = vis_dim_dist(
		ids=ids(),
		data_type=profile(), 
		group_info = group_info(),
	  	DR_method = input$method,
	  	palette=input$palette,
	  	add_margin=if (input$margin == "NONE") NULL else input$margin
	  )
	  return(p)
	})

    # Show waiter for plot
    w <- waiter::Waiter$new(id = ns("dim_plot"), html = waiter::spin_hexdots(), color = "black")
    observeEvent(input$search_bttn,{
        shinyjs::disable("search_bttn")
        output$dim_plot <- renderUI({
            w$show()
            output$plot = renderPlot(plot_func())
            fluidRow(
                column(10, offset = 1,
                    plotOutput(ns("plot"), height = "620"),
                )
            )
        })    
        shinyjs::enable("search_bttn")
    })

    output$download_1 <- downloadHandler(
        filename = function() {
            paste0(profile(),"_",input$method,".", input$device)
        },
        content = function(file) {
            p <- plot_func()
            if (input$device == "pdf") {
                pdf(file, width = input$width, height = input$height)
                print(p)
                dev.off()
            } else {
                png(file, width = input$width, height = input$height, res = 600, units = "in")
                print(p)
                dev.off()
            }
        }
    )

    output$download_2 <- downloadHandler(
        filename = function() {
            paste0(profile(),"_",input$method,"_DR.csv")
        },
        content = function(file) {
            data = plot_func()$data[,1:4]
            write.csv(data, file, row.names = FALSE)
        }
    )
}

ui.modules_pcawg_comp_m2o = function(id) {
	ns = NS(id)
	fluidPage(
		fluidRow(
			# 初始设置
			column(
				3,
				wellPanel(
					style = "height:1100px",
					h2("S1: Preset", align = "center"),
					h4(strong("S1.1 Modify datasets"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Modify datasets", 
					                   content = "data_origin"),
					mol_origin_UI(ns("mol_origin2comp"), database = "pcawg"),

					h4(strong("S1.2 Choose project")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "PCAWG projects", 
					                   content = "pcawg_projects"),
					pickerInput(
						ns("choose_cancer"),NULL,
						choices = pcawg_names),
					br(),

					h4(strong("S1.3 Filter samples"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Filter samples", 
					                   content = "choose_samples"),
					h5("Quick filter:"),
					pickerInput(
						ns("filter_by_code"), NULL,
						choices = NULL, selected =  NULL,
						multiple = TRUE, options = list(`actions-box` = TRUE)
					),
					h5("Exact filter:"),
					filter_samples_UI(ns("filter_samples2comp"), database = "pcawg"),
					br(),
					verbatimTextOutput(ns("filter_phe_id_info")),
					br(),

					h4(strong("S1.4 Upload metadata"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Upload metadata", 
					                   content = "custom_metadata"),
					shinyFeedback::useShinyFeedback(),
					custom_meta_UI(ns("custom_meta2comp")),
					br(),

					h4(strong("S1.5 Add signature"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Add signature", 
					                   content = "add_signature"),
					add_signature_UI(ns("add_signature2comp"), database = "pcawg"),
				)
			),
			# 分组设置
			column(
				4,
				wellPanel(
					style = "height:1100px",
					h2("S2: Get data", align = "center"),
					# 调用分组模块UI
					h4(strong("S2.1 Divide 2 groups by one condition")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Divide 2 groups", 
					                   content = "set_groups"),
					group_samples_UI(ns("group_samples2comp"),database = "pcawg"),
					# 批量数据下载
					h4(strong("S2.2 Get batch data for comparison")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Get batch data", 
					                   content = "get_batch_data"),  
					multi_upload_UI(ns("multi_upload2comp"),
						button_name="Query",database = "pcawg"),
				)
			),
			column(
				5,
				wellPanel(
					style = "height:1100px",
					h2("S3: Analyze", align = "center") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Analyze", 
					                   content = "analyze_comp_3"), 
					h4(strong("S3.1 Set analysis parameters")), 
					# br(),br(),
					selectInput(ns("comp_method"), "Correlation method:",choices = c("t.test", "wilcox.test")),
					shinyWidgets::actionBttn(
						ns("cal_batch_comp"), "Run",
				        style = "gradient",
				        icon = icon("table"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					),
					br(),
					fluidRow(
						column(10, offset = 1,
							   div(uiOutput(ns("comp_stat_tb.ui")),style = "height:600px"),
							   )
					),
					h4(strong("S3.2 Download results")), 
					download_res_UI(ns("download_res2comp"))
				)
			)
		)
	)
}


server.modules_pcawg_comp_m2o = function(input, output, session) {
	ns <- session$ns

	# 记录选择癌症
	cancer_choose <- reactiveValues(name = "BLCA-US", phe_primary="",
		filter_phe_id=query_tcga_group(database = "pcawg", cancer = "BLCA-US", return_all = T))
	observe({
		cancer_choose$name = input$choose_cancer
		cancer_choose$phe_primary <- query_tcga_group(database = "pcawg",
			cancer = cancer_choose$name, return_all = T)
	})

	# 数据源设置
	opt_pancan = callModule(mol_origin_Server, "mol_origin2comp", database = "pcawg")

	# 自定义上传metadata数据
	custom_meta = callModule(custom_meta_Server, "custom_meta2comp", database = "pcawg")
	# signature
	sig_dat = callModule(add_signature_Server, "add_signature2comp", database = "pcawg")
	custom_meta_sig = reactive({
		if(is.null(custom_meta())){
			return(sig_dat())
		} else {
			if(is.null(sig_dat())){
				return(custom_meta())
			} else {
				custom_meta_sig = dplyr::inner_join(custom_meta(),sig_dat())
				return(custom_meta_sig)
			}
		}
	})

	## 过滤样本
	# quick filter widget
	observe({
		code_types_valid = unique(cancer_choose$phe_primary$Type)
		updatePickerInput(
			session,
			"filter_by_code",
			choices = code_types_valid,
			selected =  code_types_valid
		)
	})
	# exact filter module
	filter_phe_id = callModule(filter_samples_Server, "filter_samples2comp",
					   database = "pcawg",
					   cancers=reactive(cancer_choose$name),
					   custom_metadata=reactive(custom_meta_sig()),
					   opt_pancan = reactive(opt_pancan()))

	# 综合上述二者
	observe({
		# quick filter
		filter_phe_id2 = cancer_choose$phe_primary %>%
			dplyr::filter(Type %in% input$filter_by_code) %>%
			dplyr::pull("Sample")

		# exact filter
		if(is.null(filter_phe_id())){
			cancer_choose$filter_phe_id = filter_phe_id2
		} else {
			cancer_choose$filter_phe_id = intersect(filter_phe_id2,filter_phe_id())
		}

		output$filter_phe_id_info = renderPrint({
			cat(paste0("Tip: ", length(cancer_choose$filter_phe_id), " samples are retained"))
		})
	})

	# 设置分组
	group_final = callModule(group_samples_Server, "group_samples2comp",
						   database = "pcawg",
						   cancers=reactive(cancer_choose$name),
						   samples=reactive(cancer_choose$filter_phe_id),
						   custom_metadata=reactive(custom_meta_sig()),
						   opt_pancan = reactive(opt_pancan())
						   )
	# 批量下载数据
	L3s_x_data =  callModule(multi_upload_Server, "multi_upload2comp", 
							 database = "pcawg",
							 samples=reactive(cancer_choose$filter_phe_id),
							 custom_metadata=reactive(custom_meta_sig()),
						     opt_pancan = reactive(opt_pancan()),
						     table.ui = FALSE
							 )
	L3s_x = reactive({
		unique(L3s_x_data()$id)
	})

	## 比较分析
	comp_stat = eventReactive(input$cal_batch_comp, {
		shiny::validate(
			need(try(nrow(group_final())>0), 
				"Please inspect whether to set groups in S2 step."),
		)
		shinyjs::disable("cal_batch_comp")
		withProgress(message = "Please wait for a while.",{
			comp_stat = lapply(seq(L3s_x()), function(i){
				# i = 1
			    incProgress(1 / length(L3s_x()), detail = paste0("(Finished ",i,"/",length(L3s_x()),")"))

				L3_x = L3s_x()[i]

				y_data = L3s_x_data() %>%
					dplyr::filter(id == L3_x) %>% 
					dplyr::select(Sample, value)

				group_data = group_final()[,c(1,3,4)]
				colnames(group_data) = c("Sample","group","phenotype")
				data = dplyr::inner_join(y_data, group_data) %>%
					dplyr::select(Sample, value, group, everything()) %>% na.omit()
				# 检查数据是否合理
				if(nrow(data)==0 | sd(data$value)==0 | length(unique(data$group))==1) return(c(NaN, NaN, NaN))
				
				if(input$comp_method == "t.test"){
					comp_obj = t.test(value ~ group, data)
				} else if (input$comp_method == "wilcox.test"){
					comp_obj = wilcox.test(value ~ group, data)
				}
				means = data %>%
				  dplyr::group_by(group) %>% 
				  dplyr::summarise(value=mean(value)) %>% 
				  dplyr::pull(value)

				comp_res = c(means, comp_obj$p.value)
				comp_res
			}) %>% do.call(rbind, .) %>% as.data.frame()

			colnames(comp_stat) = c(levels(group_final()$group),"p.value")
			comp_stat = comp_stat %>% 
			  dplyr::mutate(id = L3s_x(), .before = 1) %>%
			  dplyr::arrange(p.value)
			comp_stat
		})
		shinyjs::enable("cal_batch_comp")
		comp_stat
	})

	output$comp_stat_tb.ui = renderUI({
		output$comp_stat_tb = renderDataTable({
			# comp_stat()
			comp_stat_ = comp_stat() %>%
				dplyr::rename("Batch identifiers"="id")
			comp_stat_$p.value = format(comp_stat_$p.value, scientific=T, digits = 3)
			dt = datatable(comp_stat_,
				# class = "nowrap row-border",
				options = list(pageLength = 10, 
					columnDefs = list(
						list(className = 'dt-center', targets="_all"),
						list(orderable=TRUE, targets = 0)))
			) %>%
				formatRound(columns = levels(group_final()$group), digits = 3)
			dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
			dt
		}) 
	dataTableOutput(ns("comp_stat_tb"))
	})

	# Download results
	observeEvent(input$cal_batch_comp,{
		res1 = NULL
		group_data = group_final()[,c(1,3,4)]
		colnames(group_data) = c("Sample","group","phenotype")
		res2 = L3s_x_data() %>%
			dplyr::inner_join(group_data) %>% na.omit()
		res3 = comp_stat()
		callModule(download_res_Server, "download_res2comp", res1, res2, res3)
	})
}

ui.page_repository <- function() {
  tabPanel(
    title = "Repository",
    icon = icon("database"),
    sidebarLayout(
      sidebarPanel(
        width = 3,
        shinyWidgets::actionBttn(
          inputId = "use_repository",
          label = "How to use repository",
          icon = icon("question-circle"),
          style = "bordered",
          color = "primary",
          size = "sm"
        ),
        shinyWidgets::prettyCheckboxGroup("hubs_text", "Active Data Hub:",
          choiceNames = c(
            "UCSC Public", "TCGA",
            "GDC", "GDC V8", "ICGC",
            "Pan-Cancer Atlas", "TOIL",
            "Treehouse", "PCAWG",
            "ATAC-seq", "Single Cell"
          ),
          choiceValues = c(
            "publicHub", "tcgaHub", "gdcHub", "gdcHubV18", "icgcHub", "pancanAtlasHub",
            "toilHub", "treehouseHub", "pcawgHub", "atacseqHub", "singlecellHub"
          ),
          selected = "gdcHub",
          shape = "round",
          status = "success",
          animation = "tada"
        ),
        shinyBS::bsPopover("hubs_text",
          title = "Tips",
          content = "Data hub/host is an individual database for storing genomic data",
          placement = "right", options = list(container = "body")
        ),
        uiOutput("cohorts_text"),
        shinyBS::bsPopover("cohorts_text",
          title = "Tips",
          content = "Cohort is an independent study, it contains datasets with same patients",
          placement = "right", options = list(container = "body")
        ),
        shinyWidgets::prettyCheckboxGroup("type_text", "Data Type:",
          choiceNames = c(
            "Phenotype", "Genomic matrix\n(e.g., expression)",
            "Genomic segments", "Mutations"
          ),
          choiceValues = c("clinicalMatrix", "genomicMatrix", "genomicSegment", "mutationVector"),
          selected = c("clinicalMatrix", "genomicMatrix", "genomicSegment", "mutationVector"),
          status = "success",
          animation = "tada"
        ),
        shinyBS::bsPopover("type_text",
          title = "Tips",
          content = "Data type divide datasets into 4 basic categories: Phenotype for clinical or other phenotype data; Genomic matrix is a feature by sample matrix representing gene/probe expression matrix; Genomic segments for copy number records; Mutations for mutation annotations",
          placement = "right", options = list(container = "body")
        ),
        uiOutput("subtype_text"),
        shinyBS::bsPopover("subtype_text",
          title = "Tips",
          content = "Subtype are categories based on biological meaning instead of storing format like Type",
          placement = "right", options = list(container = "body")
        )
      ),
      mainPanel(
        DT::dataTableOutput("xena_table"),
        hr(),
        fluidRow(
          column(
            offset = 1,
            4,
            actionBttn(
              inputId = "show_met",
              label = "Show Metadata",
              style = "gradient",
              # color = "danger",
              icon = icon("database"),
              size = "sm"
            )
          ),
          column(
            4,
            actionBttn(
              inputId = "req_data",
              label = "Request Data",
              style = "gradient",
              icon = icon("file"),
              size = "sm"
            )
          ),
          column(
            3,
            actionBttn(
              inputId = "analyze_data",
              label = "Analyze Data",
              style = "gradient",
              icon = icon("chart-bar"),
              size = "sm"
            )
          )
        ),
        hr(),
        shinyjs::hidden(
          tags$div(
            id = "show_data",
            h4("Selected Data:", style = "font-size: 1.2em;font-weight: bold;margin-bottom: 5px;margin-top: 0;"),
            h6("Wait until a table shows..."),
            tableOutput(
              "table"
            )
          )
        )
      )
    )
  )
}

if (!requireNamespace("pacman")) {
  install.packages("pacman")
}
library(pacman)

if (!requireNamespace("gganatogram")) {
  pacman::p_load(remotes)
  tryCatch(
    remotes::install_github("jespermaag/gganatogram"),
    error = function(e) {
      remotes::install_git("https://gitee.com/XenaShiny/gganatogram")
    }
  )
}


pacman::p_load(
  purrr,
  tidyr,
  stringr,
  magrittr,
  R.utils,
  data.table,
  dplyr,
  ggplot2,
  cowplot,
  patchwork,
  ggpubr,
  plotly,
  UpSetR,
  UCSCXenaTools,
  UCSCXenaShiny,
  shiny,
  shinyBS,
  shinyjs,
  shinyWidgets,
  shinyalert,
  shinyFiles,
  shinyFeedback,
  shinythemes,
  shinyhelper,
  shinycssloaders,
  shinydashboard,
  shinydashboardPlus,
  # bs4Dash,
  # bslib,
  survival,
  survminer,
  ezcox,
  waiter,
  colourpicker,
  DT,
  fs,
  RColorBrewer,
  gganatogram,
  ggcorrplot,
  ggstatsplot,
  ggradar,
  zip,
  msigdbr,
  slickR,
  funkyheatmap
)

if (!requireNamespace("ggradar")) {
  pacman::p_load(remotes)
  tryCatch(
    remotes::install_github("ricardo-bion/ggradar"),
    error = function(e) {
      remotes::install_git("https://gitee.com/XenaShiny/ggradar")
    }
  )
}

if (!requireNamespace("slickR")) {
  pacman::p_load(remotes)
  tryCatch(
    remotes::install_github("yonicd/slickR"),
    error = function(e) {
      message("please install slickR manually")
    }
  )
}

if (packageVersion("UCSCXenaTools") < "1.4.4") {
  tryCatch(
    install.packages("UCSCXenaTools"),
    error = function(e) {
      warning("UCSCXenaTools <1.4.4, this shiny has a known issue (the download button cannot be used) to work with it. Please upate this package!",
        immediate. = TRUE
      )
    }
  )
}

library(shiny)

data(iris)

# load separate module and function scripts
source("xx_modules.R")

ui <- fluidPage(
  headerPanel("Iris k-means clustering"),
  xx_mod_ui("input1")
)

server <- function(input, output, session) {
  # res <- callModule(xx_mod, "input1")

  res <- callModule(xx_mod_server, "input1")
}

shinyApp(ui, server)

multi_upload_UI = function(id, button_name = "Query data(x-axis)", database = "toil"){
	ns = NS(id)
	id_option = switch(database, 
			"toil"=tcga_id_option,
			"pcawg"=pcawg_id_option,
			"ccle"=ccle_id_option)

	tagList(
		# shinyWidgets::actionBttn(
		# 	ns("inspect_data_x"), button_name,
	 #        style = "gradient",
	 #        icon = icon("search"),
	 #        color = "primary",
	 #        block = TRUE,
	 #        size = "sm"
		# ),

		# 选择major/minor type
	    fluidRow(
	    	column(
	    		6,
			    selectInput(
			    	ns("data_L1"), label = "Data type:",
			    	choices = names(id_option),
			    	selected = "Molecular profile"
			    )
	    	),
	    	column(
	    		6,
			    tabsetPanel(
				    id = ns("data_L2_tab"),
				    type = "hidden",
					tabPanel("Molecular profile", 
						selectInput(
							ns("genomic_profile"), "Data subtype:",
							choices = names(id_option[["Molecular profile"]]),
							selected = "mRNA Expression")
					),
					tabPanel("Tumor index",
						selectInput(
							ns("tumor_index"), "Data subtype:",
							choices = names(id_option[["Tumor index"]]),
							selected = "Tumor Purity")
					),
					tabPanel("Immune Infiltration",
						selectInput(
							ns("immune_infiltration"), "Data subtype:",
							choices = names(id_option[["Immune Infiltration"]]),
							selected = "CIBERSORT")
					),
					tabPanel("Pathway activity",
						selectInput(
							ns("pathway_activity"), "Data subtype:",
							choices = names(id_option[["Pathway activity"]]),
							selected = "HALLMARK")
					),
					tabPanel("Phenotype data",
						selectInput(
							ns("phenotype_data"), "Data subtype:",
							choices = names(id_option[["Phenotype data"]]),
							selected = "Clinical Phenotye")
					)
				)
	    	)
	    ),

		prettyRadioButtons(ns("L3_x_type"),"Choose multi-ids by",
			choices = c("Selection","All","File"), selected = "Selection", inline=TRUE),
		tabsetPanel(id = ns("L3_x_type_tab"),
			type = "hidden",
			tabPanel("Selection",
			    tabsetPanel(
				    id = ns("data_L3_tab"),
				    type = "hidden",
					tabPanel("Molecular profile",
			            virtualSelectInput(
			              inputId = ns("genomic_profile_id"),
			              label = NULL,
			              choices = NULL, multiple = TRUE,
			              search = TRUE,
			              allowNewOption = TRUE,
			              dropboxWidth = "200%")
					),
					tabPanel("Tumor index",
			            virtualSelectInput(
			              inputId = ns("tumor_index_id"),
			              label = NULL,
			              choices = NULL, multiple = TRUE,
			              search = TRUE,
			              allowNewOption = FALSE,
			              dropboxWidth = "200%")
					),
					tabPanel("Immune Infiltration",
			            virtualSelectInput(
			              inputId = ns("immune_infiltration_id"),
			              label = NULL,
			              choices = NULL, multiple = TRUE,
			              search = TRUE,
			              allowNewOption = FALSE,
			              dropboxWidth = "200%")
					),
					tabPanel("Pathway activity",
			            virtualSelectInput(
			              inputId = ns("pathway_activity_id"),
			              label = NULL,
			              choices = NULL, multiple = TRUE,
			              search = TRUE,
			              allowNewOption = FALSE,
			              dropboxWidth = "200%")
					),
					tabPanel("Phenotype data",
			            virtualSelectInput(
			              inputId = ns("phenotype_data_id"),
			              label = NULL,
			              choices = NULL, multiple = TRUE,
			              search = TRUE,
			              allowNewOption = FALSE,
			              dropboxWidth = "200%")
					)
				),	
			),
			tabPanel("All",
				fluidRow(
					uiOutput(ns("msigdb_note.ui")), 
					uiOutput(ns("msigdb_sle.ui1")),
					uiOutput(ns("msigdb_sle.ui2")), 
				)
			),
			tabPanel("File",
				fluidRow(
					column(8, fileInput(ns("fl_L3_x"),NULL, accept = ".txt")),
					column(3, downloadButton(ns("dw_L3_x"), "e.g."))
				)
			)
		),
	    div(verbatimTextOutput(ns("L3s_x_tip")),
	      style = "margin-top: 0px; margin-bottom: 1px;"
	    ),
		shinyWidgets::actionBttn(
			ns("inspect_data_x"), button_name,
	        style = "gradient",
	        icon = icon("search"),
	        color = "primary",
	        block = TRUE,
	        size = "sm"
		),
		div(uiOutput(ns("L3s_x_data.ui")),
			style = "margin-top: 1px; margin-bottom: 0px;"
		)
	)
}


multi_upload_Server = function(input, output, session, database = "toil", #id_option = tcga_id_option,
							   samples=NULL,custom_metadata=NULL, opt_pancan=NULL, table.ui=TRUE){
	ns <- session$ns

	id_option = switch(database, 
			"toil"=tcga_id_option,
			"pcawg"=pcawg_id_option,
			"ccle"=ccle_id_option)
	id_category = lapply(id_option, names)
	
	observe({
	  updateTabsetPanel(inputId = "data_L2_tab", selected = input$data_L1)
	  updateTabsetPanel(inputId = "data_L3_tab", selected = input$data_L1)
	  updateTabsetPanel(inputId = "L3_x_type_tab", selected = input$L3_x_type)
	}) 

	genomic_profile_choices <- reactive({
	  id_option[["Molecular profile"]][[input$genomic_profile]]
	})

	tumor_index_choices <- reactive({
	  id_option[["Tumor index"]][[input$tumor_index]]
	})

	immune_infiltration_choices <- reactive({
	  id_option[["Immune Infiltration"]][[input$immune_infiltration]]
	})

	pathway_activity_choices <- reactive({
	  id_option[["Pathway activity"]][[input$pathway_activity]]
	})

	phenotype_data_choices <- reactive({
	    id_tmp = id_option[["Phenotype data"]]
		if(!is.null(custom_metadata)){
			id_tmp[["Custom metadata"]]$all = sort(colnames(custom_metadata()[-1]))
			id_tmp[["Custom metadata"]]$default = sort(colnames(custom_metadata()[-1]))[1]
		}
		id_tmp[[input$phenotype_data]]
	})


	observe({
	  updateVirtualSelect(
	    "genomic_profile_id",
	    choices = genomic_profile_choices()$all,
	    selected = genomic_profile_choices()$default
	  )
	  updateVirtualSelect(
	    "tumor_index_id",
	    choices = tumor_index_choices()$all,
	    selected = tumor_index_choices()$default
	  )
	  updateVirtualSelect(
	    "immune_infiltration_id",
	    choices = immune_infiltration_choices()$all,
	    selected = immune_infiltration_choices()$default
	  )
	  updateVirtualSelect(
	    "pathway_activity_id",
	    choices = pathway_activity_choices()$all,
	    selected = pathway_activity_choices()$default
	  )
	  updateVirtualSelect(
	    "phenotype_data_id",
	    choices = phenotype_data_choices()$all,
	    selected = phenotype_data_choices()$default
	  )
	})


	L2_x = reactive({
	  switch(input$data_L1,
	    `Molecular profile` = input$genomic_profile,
	    `Tumor index` = input$tumor_index,
	    `Immune Infiltration` = input$immune_infiltration,
	    `Pathway activity` = input$pathway_activity,
	    `Phenotype data` = input$phenotype_data
	  )
	})

	msigdbr_query = reactive({
		category = msigdbr_types$gs_cat[msigdbr_types$gs_subcat_label==msigdbr_var$msigdbr_cat]
		subcategory = msigdbr_types$gs_subcat[msigdbr_types$gs_subcat_label==msigdbr_var$msigdbr_cat]
		if(length(category)!=0){
			msigdbr_query = msigdbr(species = "Homo sapiens", 
			                      category = category, 
			                      subcategory = subcategory)
			msigdbr_query
		}
	})


	msigdbr_var = reactiveValues(msigdbr_cat="H--H",
		msigdbr_pw = "HALLMARK_ADIPOGENESIS (210)")

	observe({
		if(!is.null(input$msigdbr_cat)){
			msigdbr_var$msigdbr_cat = input$msigdbr_cat
		}
		if(!is.null(input$msigdbr_pw)){
			msigdbr_var$msigdbr_pw = input$msigdbr_pw
		}
	})

	output$msigdb_note.ui = renderUI({
		if(L2_x() %in% 
			c("mRNA Expression","DNA Methylation","Mutation status","Copy Number Variation","Transcript Expression")){
			pw_sle = str_split(msigdbr_var$msigdbr_pw," ")[[1]][1]
			term_link = sprintf("https://www.gsea-msigdb.org/gsea/msigdb/human/%s.html",
	                    		ifelse(is.null(pw_sle),"HALLMARK_ADIPOGENESIS",pw_sle))
			msigdb_link = "https://www.gsea-msigdb.org/gsea/msigdb/human/collection_details.jsp"

			tagList(
				h5(strong(HTML('&nbsp;'),HTML('&nbsp;'),HTML('&nbsp;'),"Note: For this molecular type, select ids in ",
					  a("one pathway", href = term_link)," from ",
					  a("MSigDB database", href = msigdb_link), ":"))
			)
		}
	})

	output$msigdb_sle.ui1 = renderUI({
		if(L2_x() %in% 
			c("mRNA Expression","DNA Methylation","Mutation status","Copy Number Variation","Transcript Expression")){
			tagList(
				column(
					4,
		            virtualSelectInput(
		              inputId = ns("msigdbr_cat"),
		              label = NULL,
		              choices = isolate(msigdbr_types$gs_subcat_label), 
		              # selected =  msigdbr_types$gs_subcat_label[1],
		              dropboxWidth = "200%")
				),
			)
		}
	})


	output$msigdb_sle.ui2 = renderUI({
		if(!is.null(msigdbr_query())){
			msigdbr_term_stat = as.data.frame(table(msigdbr_query()$gs_name)) %>% 
			  dplyr::rename(term=Var1, size=Freq) %>% 
			  dplyr::arrange(term) %>% 
			  dplyr::mutate(term_size = paste0(term," (", size,")"))
		}
		if(L2_x() %in% 
			c("mRNA Expression","DNA Methylation","Mutation status","Copy Number Variation","Transcript Expression")){
			tagList(
				column(
					8,
		            virtualSelectInput(
		              inputId = ns("msigdbr_pw"),
		              label = NULL,
		              choices = isolate(msigdbr_term_stat$term_size), 
		              # selected =  msigdbr_term_stat$term_size[1],
		              search = TRUE,
		              dropboxWidth = "200%")
				)
			)
		}
	})


	output$dw_L3_x = downloadHandler(
		filename = function(){
			"sample_multiple_ids.txt"
		},
		content = function(file){
			set.seed(42)
			all_ids = id_option[[input$data_L1]][[L2_x()]]$all 
			sample_ids = sample(all_ids,ifelse(length(all_ids)>10,10,length(all_ids)))
			if(L2_x()=="Custom metadata"){
				if(is.null(custom_metadata)){
					sample_ids = NULL
				} else {
					sample_ids = sample(colnames(custom_metadata()[-1]),
						ifelse(length(colnames(custom_metadata()[-1]))>10,10,
							   length(colnames(custom_metadata()[-1]))))
				}
			}
			write.table(sample_ids, file,
				quote = F, row.names = F, col.names = F)
		}
	)

	L3s_x = reactive({
		if(input$L3_x_type=="Selection"){
			L3s_x = switch(input$data_L1,
				    `Molecular profile` = input$genomic_profile_id,
				    `Tumor index` = input$tumor_index_id,
				    `Immune Infiltration` = input$immune_infiltration_id,
				    `Pathway activity` = input$pathway_activity_id,
				    `Phenotype data` = input$phenotype_data_id
				  )
		} else if (input$L3_x_type=="File"){
			file = input$fl_L3_x
			if(is.null(file$datapath)){  # 如果空文件
				L3s_x = NULL
			} else {
				L3s_x = read.table(file$datapath)[,1]
				L3s_x = L3s_x[L3s_x %in% id_option[[input$data_L1]][[L2_x()]]$all]
				if(length(L3s_x)>1000 & L2_x() %in% id_category[["Molecular profile"]]){
					L3s_x = L3s_x[1:1000]
				} 
			}
		} else if (input$L3_x_type=="All"){
			pw_genes = msigdbr_query() %>% 
			  dplyr::filter(gs_name %in% str_split(msigdbr_var$msigdbr_pw," ")[[1]][1]) %>% 
			  dplyr::pull(gene_symbol)
			L3s_x = id_option[[input$data_L1]][[L2_x()]]$all #!!! 2w候选基因

			if(!exists("tcga_id_referrence")){
				message("Loading \"pancan_identifier_help\"")
				tcga_id_referrence = load_data("pancan_identifier_help")
			}
			
			if(L2_x() %in% 
				c("mRNA Expression","DNA Methylation","Mutation status","Copy Number Variation")){
				gene_full = tcga_id_referrence$id_molecule$id_gene$Level3
				L3s_x = gene_full[gene_full %in% pw_genes]
			} else if(L2_x() %in% c("Transcript Expression")){
				L3s_x = L3s_x[L3s_x %in% tcga_id_referrence[[1]][[5]]$Level3[tcga_id_referrence[[1]][[5]]$Symbol %in% pw_genes]]
			}
			if(L2_x()=="Custom metadata" & !is.null(custom_metadata)){
				L3s_x = colnames(custom_metadata()[-1])
			}
		}
		L3s_x
	})
	output$L3s_x_tip = renderPrint({
		cat(paste0("Tip: ",length(L3s_x())," ids are selected.\n"))
	})

	L3s_x_data = eventReactive(input$inspect_data_x, {
		shinyjs::disable("inspect_data_x")
		L1_x = names(id_category)[sapply(id_category, function(x){any(x %in% L2_x())})]
		digest_code = digest::digest(c(database, L1_x, L2_x(), sort(L3s_x())))
		digest_file = paste0(UCSCXenaShiny:::get_cache_dir(), "/", digest::digest(digest_code), ".rds")
		if(file.exists(digest_file)){
			print("Load existed data.")
			x_data_merge = readRDS(digest_file)
		} else {
			withProgress(message = "Please wait for a while.",{
				x_data_merge = lapply(seq(L3s_x()), function(i){
					# 进度提醒
				    incProgress(1 / length(L3s_x()), detail = paste0("(Run ",i,"/",length(L3s_x()),")"))

					L3_x = L3s_x()[i]
					L2_x = L2_x()

					if(is.null(opt_pancan)){
						opt_pancan = .opt_pancan
					} else {
						opt_pancan = opt_pancan()
					}

					if(database=="toil"){
						if(!exists("tcga_value_nonomics")){
							tcga_value_nonomics = load_data("v2_tcga_value_nonomics")
						}
						clinical_phe = tcga_clinical_fine
						x_data = UCSCXenaShiny:::query_general_value(L1_x, L2_x, L3_x, database,
										tcga_value_nonomics, opt_pancan,custom_metadata())
					} else if(database=="pcawg"){
						if(!exists("pcawg_value_nonomics")){
							pcawg_value_nonomics = load_data("v2_pcawg_value_nonomics")
						}
						clinical_phe = pcawg_info_fine
						x_data = UCSCXenaShiny:::query_general_value(L1_x, L2_x, L3_x, database,
										pcawg_value_nonomics, opt_pancan,custom_metadata())
					} else if (database=="ccle"){
						if(!exists("ccle_value_nonomics")){
							ccle_value_nonomics = load_data("v2_ccle_value_nonomics")
						}
						clinical_phe = ccle_info_fine
						x_data = UCSCXenaShiny:::query_general_value(L1_x, L2_x, L3_x, database,
										ccle_value_nonomics, opt_pancan,custom_metadata())
					}
					x_data = x_data %>%
						# dplyr::filter(Sample %in% samples()) %>%
					    dplyr::select(id, Sample, value)
					# 默认批量下载的应为数值型变量，若不是则剔除
					if(class(x_data$value)=="character"){  
						return(NULL)
					} else {
						return(x_data)
					}
				}) %>% do.call(rbind, .)
			})
			saveRDS(x_data_merge, file = digest_file)
		}
		x_data_merge =  x_data_merge %>%
			dplyr::filter(Sample %in% samples()) %>%
			dplyr::arrange(id,Sample)
		shinyjs::enable("inspect_data_x")
		x_data_merge
	})



	observeEvent(input$inspect_data_x,{
		output$L3s_x_data.ui = renderUI({
			shiny::validate(
				need(try(nrow(L3s_x_data())>0), 
					"No sample data were available. Please inspect operations in Preset step."),
			)
			if(table.ui){
				L3s_x_data_ = L3s_x_data()[,c("id","Sample","value")]
				if(class(L3s_x_data_[,"value"])=="numeric"){
					L3s_x_data_[,"value"] = round(L3s_x_data_[,"value"], digits = 3)
				}
				output$L3s_x_data = renderDataTable({
					datatable(L3s_x_data_,
						# class = "nowrap row-border",
						options = list(pageLength = 3, 
							columnDefs = list(list(className = 'dt-center', targets="_all")))
					)
				})
				dataTableOutput(ns("L3s_x_data"))
			} else {
				output$L3s_x_data = renderPrint({
					ids_num = length(unique(L3s_x_data()$id))
					cat(paste0("Tip: ", ids_num, " ids are prepared."))
				})
				verbatimTextOutput(ns("L3s_x_data"))
			}
		})

	})


	return(L3s_x_data)
}

ui.modules_pcawg_sur_m2o = function(id) {
	ns = NS(id)
	fluidPage(
		fluidRow(
			# 初始设置
			column(
				3,
				wellPanel(
					style = "height:1100px",
					h2("S1: Preset", align = "center"),

					h4(strong("S1.1 Modify datasets"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Modify datasets", 
					                   content = "data_origin"),
					mol_origin_UI(ns("mol_origin2sur"), database = "pcawg"),

					h4(strong("S1.2 Choose project")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "PCAWG projects", 
					                   content = "pcawg_projects"),
					pickerInput(
						ns("choose_cancer"),NULL,
						choices = pcawg_names),
					br(),

					h4(strong("S1.3 Filter samples"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Filter samples", 
					                   content = "choose_samples"),
					h5("Quick filter:"),
					pickerInput(
						ns("filter_by_code"), NULL,
						choices = NULL, selected =  NULL,
						multiple = TRUE, options = list(`actions-box` = TRUE)
					),
					h5("Exact filter:"),
					filter_samples_UI(ns("filter_samples2sur"), database = "pcawg"),
					br(),
					verbatimTextOutput(ns("filter_phe_id_info")),
					br(),

					h4(strong("S1.4 Upload metadata"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Upload metadata", 
					                   content = "custom_metadata"),
					shinyFeedback::useShinyFeedback(),
					custom_meta_UI(ns("custom_meta2sur")),
					br(),

					h4(strong("S1.5 Add signature"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Add signature", 
					                   content = "add_signature"),
					add_signature_UI(ns("add_signature2sur"), database = "pcawg")
				)
			),	
			column(
				4,
				wellPanel(
					style = "height:1100px",

					h2("S2: Get data", align = "center"),
					h4(strong("S2.1 Select survival endpoint")), 
					p("Only OS (Overall Survial) is supported."),
				    br(),

					h4(strong("S2.2 Divide 2 groups by batch conditions")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Divide 2 groups by batch conditions", 
					                   content = "get_batch_data_set_groups"), 
					multi_upload_UI(ns("multi_upload2sur"),	button_name="Observe",database = "pcawg"),
				    # uiOutput(ns("L3s_x_data_sur.ui")),


					# br(),
					uiOutput(ns("set_quantile.ui")),
					uiOutput(ns("set_group1.ui")),
					uiOutput(ns("set_group2.ui")),
					shinyWidgets::actionBttn(
						ns("set_group"), "Group",
				        style = "gradient",
				        icon = icon("search"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					),
					# uiOutput(ns("L3s_x_data_sur_group.ui"))
				    div(uiOutput(ns("L3s_x_data_sur_group.ui")),
				      style = "margin-top: 5px; margin-bottom: 0px;"
				    ),
				)
			),	
			column(
				5,
				wellPanel(
					style = "height:1100px",
					h2("S3: Analyze", align = "center") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Analyze", 
					                   content = "analyze_sur_3"),
					# br(),
					h4(strong("S3.1 Set analysis parameters")), 
					# br(),br(),
					selectInput(ns("sur_method"), "Survival method:",c("Log-rank test","Cox regression")),
				    materialSwitch(ns("use_origin"), 
					    	"Whether use initial data before grouping?"),
					br(),br(),
					shinyWidgets::actionBttn(
						ns("cal_batch_sur"), "Run",
				        style = "gradient",
				        icon = icon("table"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					),
					br(),br(),
					fluidRow(
						column(10, offset = 1,
							   div(uiOutput(ns("sur_stat_tb.ui")),style = "height:600px"),
							   )
					),
					br(),
					h4(strong("S3.2 Download results")), 
					download_res_UI(ns("download_res2sur"))
				)
			)
		)
	)
}


server.modules_pcawg_sur_m2o = function(input, output, session) {
	ns <- session$ns

	# 记录选择癌症
	cancer_choose <- reactiveValues(name = "BLCA-US", phe_primary="",
		filter_phe_id=query_tcga_group(database = "pcawg", cancer = "BLCA-US", return_all = T))
	observe({
		cancer_choose$name = input$choose_cancer
		cancer_choose$phe_primary <- query_tcga_group(database = "pcawg",
			cancer = cancer_choose$name, return_all = T)
	})

	# 数据源设置
	opt_pancan = callModule(mol_origin_Server, "mol_origin2sur", database = "pcawg")

	# 自定义上传metadata数据
	custom_meta = callModule(custom_meta_Server, "custom_meta2sur", database = "pcawg")
	# signature
	sig_dat = callModule(add_signature_Server, "add_signature2sur", database = "pcawg")
	custom_meta_sig = reactive({
		if(is.null(custom_meta())){
			return(sig_dat())
		} else {
			if(is.null(sig_dat())){
				return(custom_meta())
			} else {
				custom_meta_sig = dplyr::inner_join(custom_meta(),sig_dat())
				return(custom_meta_sig)
			}
		}
	})

	## 过滤样本
	# quick filter widget
	observe({
		code_types_valid = unique(cancer_choose$phe_primary$Type)
		updatePickerInput(
			session,
			"filter_by_code",
			choices = code_types_valid,
			selected =  code_types_valid
		)
	})
	# exact filter module
	filter_phe_id = callModule(filter_samples_Server, "filter_samples2sur",
					   database = "pcawg",
					   cancers=reactive(cancer_choose$name),
					   custom_metadata=reactive(custom_meta_sig()),
					   opt_pancan = reactive(opt_pancan()))

	# 综合上述二者
	observe({
		# quick filter
		filter_phe_id2 = cancer_choose$phe_primary %>%
			dplyr::filter(Type %in% input$filter_by_code) %>%
			dplyr::pull("Sample")

		# exact filter
		if(is.null(filter_phe_id())){
			cancer_choose$filter_phe_id = filter_phe_id2
		} else {
			cancer_choose$filter_phe_id = intersect(filter_phe_id2,filter_phe_id())
		}

		output$filter_phe_id_info = renderPrint({
			cat(paste0("Tip: ", length(cancer_choose$filter_phe_id), " samples are retained"))
		})
	})


	# 批量下载数据
	L3s_x_data =  callModule(multi_upload_Server, "multi_upload2sur", 
							 database = "pcawg",
							 samples=reactive(cancer_choose$filter_phe_id),
							 custom_metadata=reactive(custom_meta_sig()),
						     opt_pancan = reactive(opt_pancan()),
						     table.ui = FALSE
							 )
	L3s_x = reactive({
		unique(L3s_x_data()$id)
	})

	L3s_x_data_sur = reactive({
		sur_dat_raw = pcawg_info[,c("icgc_specimen_id","OS","OS.time")]
		colnames(sur_dat_raw) = c("Sample","status","time")
		sur_dat_sub = sur_dat_raw %>%  dplyr::distinct() %>% na.omit()

		x_data_merge = L3s_x_data() %>%
			dplyr::inner_join(sur_dat_sub)
		x_data_merge
	})

	# 分组方式
	output$set_group1.ui = renderUI({
		choice_chrs = ifelse(input$inspect_data_x==0,0,L3s_x_data_sur()$value)
		fluidRow(
			if(class(choice_chrs)=="character"){
				column(
					8,
					selectInput(ns("group1_range"),"Group1 Range",
						sort(unique(choice_chrs), ,na.last = T), multiple=T)
				)
			} else {
				column(8,
					fluidRow(
						column(6,
							numericInput(ns("group1_min"),"Group1 [min",value=NA)),
						column(6,
							numericInput(ns("group1_max"),"max)",
								value=ifelse(input$set_quantile,0.5,median(choice_chrs, na.rm=T))))
					)
				)
			}
			,
			column(
				4,
				textInput(ns("group1_name"),"Group Name", "Group1")
			)
		)
	})
	output$set_group2.ui = renderUI({
		choice_chrs = ifelse(input$inspect_data_x==0,0,L3s_x_data_sur()$value)
		fluidRow(
			if(class(choice_chrs)=="character"){
				column(
					8,
					selectInput(ns("group2_range"),"Group2 Range",
						sort(unique(choice_chrs), ,na.last = T), multiple=T)
				)
			} else {
				column(8,
					fluidRow(
						column(6,
							numericInput(ns("group2_min"),"Group2 [min",
								value=ifelse(input$set_quantile,0.5,median(choice_chrs, na.rm=T)))),
						column(6,
							numericInput(ns("group2_max"),"max]",value=NA))
					)
				)
			}
			,
			column(
				4,
				textInput(ns("group2_name"),"Group Name", "Group2")
			)
		)
	})
	output$set_quantile.ui = renderUI({
		choice_chrs = ifelse(input$inspect_data_x==0,0,L3s_x_data_sur()$value)
		if(length(choice_chrs)==0) choice_chrs = 1:100
		if(class(choice_chrs)!="character"){
			materialSwitch(ns("set_quantile"),"Whether group by percentile?",value = TRUE)
		}
	})

	L3s_x_data_sur_group = eventReactive(input$set_group,{
		datas = L3s_x_data_sur()
		choice_chrs = L3s_x_data_sur()$value

		merge_by = list(
			c(input$group1_min, input$group1_max),
			c(input$group2_min, input$group2_max)
		)
		names(merge_by) = c(input$group1_name, input$group2_name)

		if(class(datas$value)=="character"){
		    shiny::validate(
		      need(length(intersect(input$group1_range,input$group2_range))>0, 
		        "Please inspect whether to input independent group range.")
		    )
			datas$group = NA
			datas$group[choice_chrs %in% input$group1_range] = input$group1_name
			datas$group[choice_chrs %in% input$group2_range] = input$group2_name
			datas = na.omit(datas)
		} else if (input$set_quantile) {
			if(is.na(merge_by[[1]][1])) merge_by[[1]][1] = 0
			if(is.na(merge_by[[2]][1])) merge_by[[1]][1] = 0
			if(is.na(merge_by[[1]][2])) merge_by[[1]][1] = 1
			if(is.na(merge_by[[2]][2])) merge_by[[2]][2] = 1
			datas_split = split(datas, datas$id)
			shinyjs::disable("set_group")
			withProgress(message = "Please wait for a while.",{			
				datas = lapply(seq(datas_split), function(i){
					# 进度提醒
				    incProgress(1 / length(datas_split), detail = paste0("(Run ",i,"/",length(datas_split),")"))

					data_id = datas_split[[i]]
					choice_chrs_id = data_id$value
					merge_by_one = merge_by
					for (i in seq(merge_by_one)){
					merge_by_one[[i]] = quantile(choice_chrs_id, merge_by_one[[i]], na.rm = T)
					}
					data_id$group = NA
					data_id$group[which(findInterval(choice_chrs_id, merge_by_one[[1]])==1)] = names(merge_by_one[1])
					data_id$group[which(findInterval(choice_chrs_id, merge_by_one[[2]], rightmost.closed = TRUE)==1)] = names(merge_by_one[2])
					data_id = na.omit(data_id)
				}) %>% do.call(rbind, .) %>% tibble::remove_rownames()
			})
			shinyjs::enable("set_group")

		} else {
			if(is.na(merge_by[[1]][1])) merge_by[[1]][1] = min(choice_chrs, na.rm=T)
			if(is.na(merge_by[[2]][1])) merge_by[[2]][1] = min(choice_chrs, na.rm=T)
			if(is.na(merge_by[[1]][2])) merge_by[[1]][2] = max(choice_chrs, na.rm=T)
			if(is.na(merge_by[[2]][2])) merge_by[[2]][2] = max(choice_chrs, na.rm=T)
			datas$group = NA
			datas$group[which(findInterval(choice_chrs, merge_by[[1]])==1)] = names(merge_by[1])
			datas$group[which(findInterval(choice_chrs, merge_by[[2]], rightmost.closed = TRUE)==1)] = names(merge_by[2])
			datas = na.omit(datas)
		}
		## 去除分组结果里只有一组的id
		valid_ids = datas %>%
			dplyr::distinct(id, group) %>%
			dplyr::count(id) %>%
			dplyr::filter(n==2) %>% dplyr::pull("id")
		datas_sub = datas %>% 
			dplyr::filter(id %in% valid_ids)
		datas_sub
	})


	w <- waiter::Waiter$new(id = ns("L3s_x_data_sur_group"), html = waiter::spin_hexdots(), color = "black")

	output$L3s_x_data_sur_group.ui = renderUI({
		output$L3s_x_data_sur_group = renderDataTable({
			w$show()
			group_stat = L3s_x_data_sur_group()
			group_stat = group_stat %>%
				dplyr::count(id, group) %>% 

			datatable(group_stat,
				# class = "nowrap row-border",
				options = list(pageLength = 2, 
					columnDefs = list(list(className = 'dt-center', targets="_all")))
			)
		})
	dataTableOutput(ns("L3s_x_data_sur_group"))
	})


	sur_stat = eventReactive(input$cal_batch_sur,{
		datas = L3s_x_data_sur_group()
		valid_ids = unique(datas$id)
		shinyjs::disable("cal_batch_sur")
		withProgress(message = "Please wait for a while.",{
			sur_stat = lapply(seq(valid_ids), function(i){
			    incProgress(1 / length(valid_ids), detail = paste0("(Finished ",i,"/",length(valid_ids),")"))
				valid_id = valid_ids[i]
				datas_sub = subset(datas, id %in% valid_id)
				if(input$sur_method=="Log-rank test"){
					if(!input$use_origin){
						datas_sub$Group = datas_sub$group
					} else {
						datas_sub$Group = datas_sub$value
						if(class(datas_sub$value)!="character"){
							res.cut <- surv_cutpoint(datas_sub, 
								time = "time", event = "status", variables = "value")
							groups_1_2 = datas_sub %>% 
								  dplyr::group_by(group) %>% 
								  dplyr::summarise(mean = mean(value)) %>% 
								  dplyr::arrange(mean) %>% 
								  dplyr::pull(group) %>% as.character()
							datas_sub$Group = ifelse(surv_categorize(res.cut)$value=="low", groups_1_2[1], groups_1_2[2])
							datas_sub$Group = factor(datas_sub$Group, levels=groups_1_2)
						}
					}
					surv_diff <- survdiff(Surv(time, status) ~ Group, data = datas_sub)
					pval = 1 - pchisq(surv_diff$chisq, length(surv_diff$n) - 1)
					sur_res = summary(survfit(Surv(time, status) ~ Group, data = datas_sub))$table %>% 
						    as.data.frame() %>% dplyr::pull(median)
					sur_res = c(sur_res,pval)
					names(sur_res) = c(input$group1_name,input$group2_name,"p.value")
					names(sur_res)[1:2] = paste0(names(sur_res)[1:2],"\nmedian.time")
				} else if (input$sur_method=="Cox regression"){
					if(!input$use_origin){
						datas_sub$Group = datas_sub$group
					} else {
						datas_sub$Group = datas_sub$value
					}
					fit <- coxph(Surv(time, status) ~ Group, data = datas_sub)
					sur_res = summary(fit)$coefficients %>% as.data.frame() %>%
						tibble::rownames_to_column("Group.obs")
					sur_res = sur_res[,c(1,3,6)]
					sur_res[,1] = gsub("^Group","",sur_res[,1])
					sur_res[,2] = round(sur_res[,2], 3)
					colnames(sur_res)[2:3] = c("HR","p.value")
				}
				sur_res
			}) %>% do.call(rbind, .)
			sur_stat = sur_stat %>% as.data.frame() %>%
				dplyr::mutate(id = valid_ids, .before = 1) %>%
				dplyr::arrange(p.value)
			sur_stat
		})
		shinyjs::enable("cal_batch_sur")
		sur_stat
	})

	output$sur_stat_tb.ui = renderUI({
		output$sur_stat_tb = renderDataTable({
			sur_stat_ = sur_stat() %>%
				dplyr::rename("Batch identifiers"="id")
			sur_stat_$p.value = format(sur_stat_$p.value, scientific=T, digits = 3)

			dt = datatable(sur_stat_,
				# class = "nowrap row-border",
				options = list(pageLength = 10, 
					columnDefs = list(
						list(className = 'dt-center', targets="_all"),
						list(orderable=TRUE, targets = 0)))
			)
			dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
			dt
		}) 
		dataTableOutput(ns("sur_stat_tb"))
	})

	# Download results
	observeEvent(input$cal_batch_sur,{
		res1 = NULL
		res2 = L3s_x_data_sur_group()
		res3 = sur_stat()
		callModule(download_res_Server, "download_res2sur", res1, res2, res3)
	})
}

download_feat_UI = function(id, button_name="Query data", database = "toil"){
	ns = NS(id)

	id_option = switch(database, 
			"toil"=tcga_id_option,
			"pcawg"=pcawg_id_option,
			"ccle"=ccle_id_option)

	tagList(
		# shinyWidgets::actionBttn(
		# 	ns("query_data"), button_name,
	 #        style = "gradient",
	 #        icon = icon("search"),
	 #        color = "primary",
	 #        block = TRUE,
	 #        size = "sm"
		# ),
		# fluidRow(
		# 	column(10, offset = 1,
			    fluidRow(
			    	column(
			    		6,
					    selectInput(
					    	ns("data_L1"), label = "Data type:",
					    	choices = names(id_option),
					    	selected = "Molecular profile"
					    )
			    	),
			    	column(
			    		6,
					    tabsetPanel(
						    id = ns("data_L2_tab"),
						    type = "hidden",
							tabPanel("Molecular profile", 
								selectInput(
									ns("genomic_profile"), "Data subtype:",
									choices = names(id_option[["Molecular profile"]]),
									selected = "mRNA Expression")
							),
							tabPanel("Tumor index",
								selectInput(
									ns("tumor_index"), "Data subtype:",
									choices = names(id_option[["Tumor index"]]),
									selected = "Tumor Purity")
							),
							tabPanel("Immune Infiltration",
								selectInput(
									ns("immune_infiltration"), "Data subtype:",
									choices = names(id_option[["Immune Infiltration"]]),
									selected = "CIBERSORT")
							),
							tabPanel("Pathway activity",
								selectInput(
									ns("pathway_activity"), "Data subtype:",
									choices = names(id_option[["Pathway activity"]]),
									selected = "HALLMARK")
							),
							tabPanel("Phenotype data",
								selectInput(
									ns("phenotype_data"), "Data subtype:",
									choices = names(id_option[["Phenotype data"]]),
									selected = "Clinical Phenotye")
							)
						)
			    	)
			    ),
			    tabsetPanel(
				    id = ns("data_L3_tab"),
				    type = "hidden",
					tabPanel("Molecular profile",
			            virtualSelectInput(
			              inputId = ns("genomic_profile_id"),
			              label = "Identifier:",
			              choices = NULL,
			              search = TRUE,
			              allowNewOption = TRUE,
			              dropboxWidth = "200%")
					),
					tabPanel("Tumor index",
			            virtualSelectInput(
			              inputId = ns("tumor_index_id"),
			              label = "Identifier:",
			              choices = NULL,
			              search = TRUE,
			              allowNewOption = FALSE,
			              dropboxWidth = "200%")
					),
					tabPanel("Immune Infiltration",
			            virtualSelectInput(
			              inputId = ns("immune_infiltration_id"),
			              label = "Identifier:",
			              choices = NULL,
			              search = TRUE,
			              allowNewOption = FALSE,
			              dropboxWidth = "200%")
					),
					tabPanel("Pathway activity",
			            virtualSelectInput(
			              inputId = ns("pathway_activity_id"),
			              label = "Identifier:",
			              choices = NULL,
			              search = TRUE,
			              allowNewOption = FALSE,
			              dropboxWidth = "200%")
					),
					tabPanel("Phenotype data",
			            virtualSelectInput(
			              inputId = ns("phenotype_data_id"),
			              label = "Identifier:",
			              choices = NULL,
			              search = TRUE,
			              allowNewOption = FALSE,
			              dropboxWidth = "200%")
					)
				),
				shinyWidgets::actionBttn(
					ns("query_data"), button_name,
			        style = "gradient",
			        icon = icon("search"),
			        color = "primary",
			        block = TRUE,
			        size = "sm"
				),
		# 	),
		# ),

		# br(),
		div(uiOutput(ns("x_axis_data_table")),
			style = "margin-top: 5px; margin-bottom: 0px;"
		)
	)
}



download_feat_Server = function(input, output, session, database = "toil",#id_option=tcga_id_option,
								samples=NULL, custom_metadata=NULL, opt_pancan=NULL, check_numeric=FALSE, table.ui=TRUE){
	ns <- session$ns
	# id_option = tcga_id_option
	id_option = switch(database, 
			"toil"=tcga_id_option,
			"pcawg"=pcawg_id_option,
			"ccle"=ccle_id_option)
	id_category = lapply(id_option, names)

	observe({
	  updateTabsetPanel(inputId = "data_L2_tab", selected = input$data_L1)
	  updateTabsetPanel(inputId = "data_L3_tab", selected = input$data_L1)
	}) 

	genomic_profile_choices <- reactive({
	  id_option[["Molecular profile"]][[input$genomic_profile]]
	})

	tumor_index_choices <- reactive({
	  id_option[["Tumor index"]][[input$tumor_index]]
	})

	immune_infiltration_choices <- reactive({
	  id_option[["Immune Infiltration"]][[input$immune_infiltration]]
	})

	pathway_activity_choices <- reactive({
	  id_option[["Pathway activity"]][[input$pathway_activity]]
	})


	phenotype_data_choices <- reactive({
	    id_tmp = id_option[["Phenotype data"]]
		if(!is.null(custom_metadata)){
			id_tmp[["Custom metadata"]]$all = sort(colnames(custom_metadata()[-1]))
			id_tmp[["Custom metadata"]]$default = sort(colnames(custom_metadata()[-1]))[1]
		}
		id_tmp[[input$phenotype_data]]
	})


	observe({
	  updateVirtualSelect(
	    "genomic_profile_id",
	    choices = genomic_profile_choices()$all,
	    selected = genomic_profile_choices()$default
	  )
	  updateVirtualSelect(
	    "tumor_index_id",
	    choices = tumor_index_choices()$all,
	    selected = tumor_index_choices()$default
	  )
	  updateVirtualSelect(
	    "immune_infiltration_id",
	    choices = immune_infiltration_choices()$all,
	    selected = immune_infiltration_choices()$default
	  )
	  updateVirtualSelect(
	    "pathway_activity_id",
	    choices = pathway_activity_choices()$all,
	    selected = pathway_activity_choices()$default
	  )
	  updateVirtualSelect(
	    "phenotype_data_id",
	    choices = phenotype_data_choices()$all,
	    selected = phenotype_data_choices()$default
	  )
	})


	download_data = eventReactive(input$query_data, {
		shinyjs::disable("query_data")
		L2_x = switch(input$data_L1,
		    `Molecular profile` = input$genomic_profile,
		    `Tumor index` = input$tumor_index,
		    `Immune Infiltration` = input$immune_infiltration,
		    `Pathway activity` = input$pathway_activity,
		    `Phenotype data` = input$phenotype_data
		)
		L3_x = switch(input$data_L1,
		    `Molecular profile` = input$genomic_profile_id,
		    `Tumor index` = input$tumor_index_id,
		    `Immune Infiltration` = input$immune_infiltration_id,
		    `Pathway activity` = input$pathway_activity_id,
		    `Phenotype data` = input$phenotype_data_id
		)
		L1_x = names(id_category)[sapply(id_category, function(x){any(x %in% L2_x)})]
		if(is.null(opt_pancan)){
			opt_pancan = .opt_pancan
		} else {
			opt_pancan = opt_pancan()
		}
		## 利用内部自定义下载函数获取数据
		if(database=="toil"){
			if(!exists("tcga_value_nonomics")){
				tcga_value_nonomics = load_data("v2_tcga_value_nonomics")
			}
			clinical_phe = tcga_clinical_fine
			x_data = UCSCXenaShiny:::query_general_value(L1_x, L2_x, L3_x, database,
							tcga_value_nonomics, opt_pancan,custom_metadata())
		} else if(database=="pcawg"){
			if(!exists("pcawg_value_nonomics")){
				pcawg_value_nonomics = load_data("v2_pcawg_value_nonomics")
			}
			clinical_phe = pcawg_info_fine
			x_data = UCSCXenaShiny:::query_general_value(L1_x, L2_x, L3_x, database,
							pcawg_value_nonomics, opt_pancan,custom_metadata())
		} else if (database=="ccle"){
			if(!exists("ccle_value_nonomics")){
				ccle_value_nonomics = load_data("v2_ccle_value_nonomics")
			}
			clinical_phe = ccle_info_fine
			x_data = UCSCXenaShiny:::query_general_value(L1_x, L2_x, L3_x, database,
							ccle_value_nonomics, opt_pancan,custom_metadata())
		}
		print(head(x_data))
		
		if(!is.null(samples)){
			if(!is.null(samples())){
				x_data = x_data %>%
					dplyr::filter(Sample %in% samples())
			}
		}
		if(nrow(x_data)==0) {return(NULL)}

		x_data$level1 = L1_x
		## 这里PCAWG应该是Project，但为了统一起见，先都叫cancer，包括后面的ccle
		x_data$cancer = clinical_phe[,2,drop=T][match(x_data$Sample, clinical_phe$Sample)]
		x_data = x_data[,c("id","level1","level2","Sample","value","cancer")] %>%
			dplyr::arrange(cancer,Sample)
		shinyjs::enable("query_data")
		x_data
	})

	w <- waiter::Waiter$new(id = ns("x_axis_data_table"), html = waiter::spin_hexdots(), color = "black")

	# observeEvent(input$query_data,{
	observe({
		if (input$query_data>0) {
			w$show()
			output$x_axis_data_table = renderUI({
				if(table.ui){
					output$x_tmp_table = renderDataTable({
						# shinyjs::enable("query_data")
						shiny::validate(
							need(try(nrow(download_data())>0), 
								"No sample data  available. Please inspect operations in Preset step."),
						)
						if(check_numeric){
							shiny::validate(
								need(try(class(download_data()$value)!="character"), 
									"Please select a numeric variable."),
							)	
						}
						x_axis_data_ = download_data()[,c("Sample","value","cancer")]

						if(class(x_axis_data_[,"value"])=="numeric"){
							x_axis_data_[,"value"] = round(x_axis_data_[,"value"], digits = 3)
						}
						datatable(x_axis_data_, 
							options = list(pageLength = 3,
								columnDefs = list(list(className = 'dt-center', targets="_all")))
						)
					}) 
					dataTableOutput(ns("x_tmp_table"))
				} else {
					output$x_tmp_table = renderPrint({
						# ids_num = nrow()
						cat(paste0("Tip: identifier values of ", nrow(download_data()), " samples are prepared."))
					})
					verbatimTextOutput(ns("x_tmp_table"))
				}
			})
		}		
	})

	# observeEvent(input$query_data,{
	# 	if(input$query_data==1){
	# 	  shinyalert(
	# 	    title = "Clicked",
	# 	    text = paste("You query has been uploaded successfully.\n",
	# 	    	"Please wait a moment for Molecular profile type due to network.\n",
	# 	    	"The notion will only appear once."),
	# 	    size = "s", 
	# 	    closeOnEsc = TRUE,
	# 	    closeOnClickOutside = FALSE,
	# 	    html = FALSE,
	# 	    type = "success",
	# 	    showConfirmButton = TRUE,
	# 	    showCancelButton = FALSE,
	# 	    confirmButtonText = "OK",
	# 	    confirmButtonCol = "#AEDEF4",
	# 	    timer = 0,
	# 	    imageUrl = "",
	# 	    animation = TRUE
	# 	  )
	# 	}
	# })
	return(download_data)
}


#     id            level1          level2          Sample value cancer
# 1 TP53 Molecular profile mRNA Expression TCGA-3C-AAAU-01 5.445   BRCA
# 2 TP53 Molecular profile mRNA Expression TCGA-3C-AALI-01 3.446   BRCA
# 3 TP53 Molecular profile mRNA Expression TCGA-3C-AALJ-01 4.946   BRCA
# 4 TP53 Molecular profile mRNA Expression TCGA-3C-AALK-01 5.107   BRCA
# 5 TP53 Molecular profile mRNA Expression TCGA-4H-AAAK-01 5.440   BRCA
# 6 TP53 Molecular profile mRNA Expression TCGA-5L-AAT0-01 5.239   BRCA

ui.page_pancan_quick <- function() {
  navbarMenu(
    title = "T·P·C Modules",
    icon = icon("buromobelexperte"),

    tabPanel(
      "Introduction",
      fluidPage(
        includeMarkdown(set_md_path("intro_quick_mods.md"))
      )
    ),

    tabPanel("TCGA (GTEx): Molecular comparison",
      tabsetPanel(
        tabPanel(
          "Tumor VS Normal (Box plot)",
          ui.modules_1_tcga_01("modules_1_tcga_01")
        ),
        tabPanel(
          "Tumor VS Normal (Anatomy plot)",
          ui.modules_1_tcga_02("modules_1_tcga_02")
        ),
        tabPanel(
          "Mutation VS Wild",
          ui.modules_1_tcga_08("modules_1_tcga_08")
        ),
      )
    ),

    tabPanel("TCGA: Molecular correlation",
      tabsetPanel(
        tabPanel(
          "Molecule-Molecule",
          ui.modules_1_tcga_03("modules_1_tcga_03")
        ),
        tabPanel(
          "Molecule-Tumor Immune Infiltration",
          ui.modules_1_tcga_04("modules_1_tcga_04")
        ),
        tabPanel(
          "Molecule-Immune Signature",
          ui.modules_1_tcga_05("modules_1_tcga_05")
        ),
        tabPanel(
          "Molecule-TMB/Stemness/MSI",
          ui.modules_1_tcga_06("modules_1_tcga_06")
        ),
        tabPanel(
          "Molecule-Pathway",
          ui.modules_1_tcga_07("modules_1_tcga_07")
        ),
      )
    ),

    tabPanel("TCGA: Survival analysis",
      tabsetPanel(
        tabPanel(
          "Kaplan-Meier",
          ui.modules_1_tcga_09("modules_1_tcga_09")
        ),
        tabPanel(
          "Cox regression",
          ui.modules_1_tcga_10("modules_1_tcga_10")
        )
      )
    ),

    tabPanel("TCGA: Dimensionality reduction",
      tabsetPanel(
        tabPanel(
          "Dimensionality reduction",
          ui.modules_1_tcga_11("modules_1_tcga_11")
        )
      )
    ),

    tabPanel("PCAWG: Molecular comparison",
      tabsetPanel(
        tabPanel(
          "Tumor VS Normal",
          ui.modules_2_pcawg_01("modules_2_pcawg_01")
        )
      )
    ),

    tabPanel("PCAWG: Molecular correlation",
      tabsetPanel(
        tabPanel(
          "Molecule-Molecule",
          ui.modules_2_pcawg_02("modules_2_pcawg_02")
        )
      )
    ),

    tabPanel("PCAWG: Survival analysis",
      tabsetPanel(
        tabPanel(
          "Kaplan-Meier",
          ui.modules_2_pcawg_03("modules_2_pcawg_03")
        ),
        tabPanel(
          "Cox regression",
          ui.modules_2_pcawg_04("modules_2_pcawg_04")
        )
      )
    ),

    tabPanel("CCLE: Molecular comparison",
      tabsetPanel(
        tabPanel(
          "Cancer Primary Sites",
          ui.modules_3_ccle_01("modules_3_ccle_01")
        )
      )
    ),

    tabPanel("CCLE: Molecular correlation",
      tabsetPanel(
        tabPanel(
          "Molecule-Molecule",
          ui.modules_3_ccle_02("modules_3_ccle_02")
        )
      )
    ),

    tabPanel("CCLE: Drug analysis",
      tabsetPanel(
        tabPanel(
          "Drug Response Association",
          ui.modules_3_ccle_03("modules_3_ccle_03")
        ),
        tabPanel(
          "Drug Response Difference",
          ui.modules_3_ccle_04("modules_3_ccle_04")
        )
      )
    )
  )
}

ui.home_daily_gene <- function(id) {
  ns <- NS(id)
  tagList(
    # h3("Daily Gene", icon = "dice", align = "center"),
    fluidRow(
      column(6, #offset = 1,
        uiOutput(ns("gene_tip")),

        fluidRow(
          column(12, align = "center",
            actionBttn(
               inputId = ns("change_gene"),
               label = "Sample another one", 
               style = "bordered",
               color = "primary",
               icon = icon("dice"),
               size = "sm"
            )
          )
        )
      ),
      column(6, #offset = 2,
        uiOutput(ns("gene_cancer"))
      )
    ),
    # fluidRow(
    #   column(8, offset = 2,
    #     uiOutput(ns("gene_cancer"))
    #   )
    # ),
  )
}





server.home_daily_gene <- function(input, output, session) {
  ns <- session$ns


  gene_sp = reactiveValues(gene="TP53")


  gene_sp = reactive({
    tcga_gene = tcga_id.list[["Gene"]]

    if(input$change_gene==0){
      current_date <- Sys.Date()
      formatted_date <- as.integer(format(current_date, "%Y%m%d"))
      set.seed(formatted_date)
      gene_sp = sample(tcga_gene,size = 1)
    } else {
      current_time <- Sys.time()
      total_seconds <- as.integer(difftime(current_time, as.POSIXct("1970-01-01"), units = "secs"))
      set.seed(total_seconds)
      gene_sp = sample(tcga_gene,size = 1)
    } 
    gene_sp
  })

  gene_data = reactive({
    data.list = get_pancan_gene_value(gene_sp(), norm = "tpm")
    data = data.list$expression
    shiny::validate(
      need(try(!all(is.na(data))), 
        "Sorry, no valid value for the sampling gene. Please click the button again."),
    )   
    data = data.frame(value=data) %>% 
      tibble::rownames_to_column("sample") %>% 
      dplyr::inner_join(tcga_gtex) %>% 
      dplyr::mutate(tissue = as.character(tissue))
    data
  })



  stat_res = reactive({
    data = gene_data()
    stat_res = compare_means(value ~ type2, data = data,  
                             group.by = "tissue") %>% 
      dplyr::arrange(p)
    stat_res
  })


  output$gene_cancer = renderUI({
    output$box_plot = renderPlotly({
      data = gene_data()
      p = data %>% 
        dplyr::filter(tissue == stat_res()$tissue[1]) %>% 
        dplyr::filter(value > -9.966) %>% 
        ggboxplot(x = "type2", y = "value", 
                  fill = "type2", 
                  color = "black",
                  width = 0.5,
                  palette = "aaas") + 
        xlab(paste0("TCGA-",stat_res()$tissue[1])) + 
        ylab("TPM value (TCGA+GTEx)") +
        ggtitle(gene_sp(),
                subtitle = paste0("Wilcoxon, p=", 
                                  formatC(stat_res()$p[1], format = "e", digits = 2))) +
        theme(legend.position = "none",
              text = element_text(size = 12),
              plot.title = element_text(hjust = 0.5),
              plot.subtitle = element_text(size = 12, hjust = 0.5))
        subtitle_txt = paste0("Wilcoxon, p=", formatC(stat_res()$p[1], format = "e", digits = 2))
        title_text = 
      p = ggplotly(p) %>%
        layout(title = list(text = paste0(gene_sp(),'<br>','<sup>', subtitle_txt,'</sup>'),
                            font = list(size = 18)),
               xaxis = list(font=list(size = 15)),
               yaxis = list(font=list(size = 10)))
      p
    })
    plotlyOutput(ns("box_plot"), height = "220px")
  })


  output$gene_tip = renderUI({

    tcga_link = "https://gdc.cancer.gov/resources-tcga-users/tcga-code-tables/tcga-study-abbreviations"
    gc_link = paste0("https://www.genecards.org/cgi-bin/carddisp.pl?gene=", gene_sp())
    pm_link = paste0("https://pubmed.ncbi.nlm.nih.gov/?term=%28",gene_sp(),"%29+AND+%28cancer%29")
    tagList(
      tags$ul(
        tags$li(gene_sp()," is most differentially expressed in ",
                a(paste0("TCGA-",stat_res()$tissue[1]), href = tcga_link), ".",
                style = "font-size: 16px;"),
        tags$li("Explore its pan-cancer feature in right panel.",style = "font-size: 16px;"),
        tags$li("Search more about the gene in ",
                a("GeneCards", href = gc_link), " or ",
                a("PubMed", href = pm_link), ".",
                style = "font-size: 16px;")
      )
    )
  })
}

ui.modules_pancan_cross_gene_o2m = function(id) {
	ns = NS(id)
	fluidPage(
		fluidRow(
			# 初始设置
			column(
				3,
				wellPanel(
					style = "height:1100px",
					h2("S1: Preset", align = "center"),

					h4(strong("S1.1 Modify datasets"),"[Only for S1.3]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Modify datasets", 
					                   content = "data_origin"),
					mol_origin_UI(ns("mol_origin2cor"), database = "toil"),

					h4(strong("S1.2 Choose cancers")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Cancer types", 
					                   content = "tcga_types"),
					pickerInput(
						ns("choose_cancers"), NULL,
						choices = sort(tcga_names),
						multiple = TRUE,
						selected = sort(tcga_names),
						options = list(`actions-box` = TRUE)
					),

				    br(),

					h4(strong("S1.3 Filter samples"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Filter samples", 
					                   content = "choose_samples"),
					h5("Quick filter:"),
					pickerInput(
						ns("filter_by_code"), NULL,
						choices = NULL, selected =  NULL,
						multiple = TRUE, options = list(`actions-box` = TRUE)
					),
					h5("Exact filter:"),
					filter_samples_UI(ns("filter_samples2cor"), database = "toil"),
					br(),
					verbatimTextOutput(ns("filter_phe_id_info")),
					br()
				)
			),
			# 下载X轴数据
			column(
				3,
				wellPanel(
					style = "height:1100px",
					h2("S2: Get data", align = "center"),
					# 调用下载模块UI
					h4(strong("S2.1 Select one gene")),
					virtualSelectInput(
						inputId = ns("gene_id"),
						label = NULL,
						choices = NULL,
						search = TRUE),
					br(),
					h4(strong("S2.2 Load mRNA/Mutation/CNV data")),
					shinyWidgets::actionBttn(
						ns("step2_2_load"), "Load",
				        style = "gradient",
				        icon = icon("box"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					), 
					br(),
					verbatimTextOutput(ns("step2_2_text")),
					br(),
					h4(strong("S2.3 Load transcript data")),
					checkboxInput(ns("overall_trans"), label = "Add overall(mean) values for all transcripts", value = TRUE),
					virtualSelectInput(
						inputId = ns("trans_id"),
						multiple = TRUE,
						label = NULL,
						choices = NULL,
						search = TRUE),
					shinyWidgets::actionBttn(
						ns("step2_3_load"), "Load",
				        style = "gradient",
				        icon = icon("box"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					), 
					br(),
					verbatimTextOutput(ns("step2_3_text")),
					br(),
					h4(strong("S2.4 Load methylation (450K) data")),
					checkboxInput(ns("overall_methy"), label = "Add overall(mean) values for all cpg sites", value = TRUE),
					virtualSelectInput(
						inputId = ns("methy_id"),
						multiple = TRUE,
						label = NULL,
						choices = NULL,
						search = TRUE),
					shinyWidgets::actionBttn(
						ns("step2_4_load"), "Load",
				        style = "gradient",
				        icon = icon("box"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					), 
					br(),
					verbatimTextOutput(ns("step2_4_text")),
				)
			),
			# 分析/绘图/下载
			column(
				6,
				wellPanel(
					h2("S3: Analyze & Visualize", align = "center") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Analyze & Visualize", 
					                   content = "cross_gene"), 
					style = "height:1100px",
					# h4(strong("S3.1 Set analysis parameters")), 
					# h4(strong("S3.2 Set visualization parameters")), 
					shinyWidgets::actionBttn(
						ns("step3_plot"), "Run (Visualize)",
				        style = "gradient",
				        icon = icon("chart-line"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					),
					br(),
					fluidRow(
						column(12, offset = 0,
							   plotOutput({ns("funky_plot")}, height = "700px") 
						)
					),
					br(),
					fluidRow(
						column(3,
							div(shinyjs::hidden(downloadButton(ns("save_plot_bt"), "Figure")), style="display: inline-block;vertical-align:top;"),
							div(style="display: inline-block;vertical-align:top",
								dropMenu(  
									actionBttn(ns("plot_opt"),label = NULL,style = "material-circle", 
												color = "success",icon = icon("gear")),
									div(h3("1. Height:"),style="width:400px;"),
									numericInput(ns("save_plot_H"), NULL ,min = 1, max = 20, value = NULL, step = 0.1),
									div(h3("2. Width:"),style="width:400px;"),
									numericInput(ns("save_plot_W"), NULL ,min = 1, max = 20, value = NULL, step = 0.1),	
									div(h3("3. Format:"),style="width:400px;"),		
									radioGroupButtons(
										inputId = ns("save_plot_F"),
										label = NULL,
										status = "primary",
										choices = c("PDF", "PNG"),
										justified = TRUE
									),
									placement = "top"
								)
							)
						),
						column(3, offset = 0, shinyjs::hidden(downloadButton(ns("save_data_res"), "Statistical data(.csv)")))
					)
				)
			)
		)
	)
}


server.modules_pancan_cross_gene_o2m = function(input, output, session) {
	ns <- session$ns

	# 记录选择癌症
	cancer_choose <- reactiveValues(name = "ACC", phe_primary="",
		filter_phe_id=query_tcga_group(database = "toil", cancer = "BRCA", return_all = T))
	observe({
		cancer_choose$name = input$choose_cancers
		cancer_choose$phe_primary <- query_tcga_group(database = "toil", cancer = cancer_choose$name, return_all = T)
	})
	
	# 数据源设置
	opt_pancan = callModule(mol_origin_Server, "mol_origin2cor", database = "toil")

    custom_meta_sig = reactive(NULL)
    sig_dat = reactive(NULL)

	## 过滤样本
	# exact filter module
	filter_phe_id = callModule(filter_samples_Server, "filter_samples2cor",
					   database = "toil",
					   cancers=reactive(cancer_choose$name),
					   custom_metadata=reactive(custom_meta_sig()),
					   opt_pancan = reactive(opt_pancan()))
	# quick filter widget
	observe({
		code_types_valid = code_types[names(code_types) %in% 
							unique(cancer_choose$phe_primary$Code)]
		code_types_valid = setdiff(code_types_valid, "NT (normal tissue)")
		updatePickerInput(
			session,
			"filter_by_code",
			choices = unlist(code_types_valid,use.names = F),
			selected =  unlist(code_types_valid,use.names = F)
		)
	})

	# 综合上述二者
	observe({
		# quick filter
		choose_codes = names(code_types)[unlist(code_types) %in% input$filter_by_code]
		filter_phe_id2 = cancer_choose$phe_primary %>%
			dplyr::filter(Code %in% choose_codes) %>%
			dplyr::pull("Sample")

		# exact filter
		if(is.null(filter_phe_id())){
			cancer_choose$filter_phe_id = filter_phe_id2
		} else {
			cancer_choose$filter_phe_id = intersect(filter_phe_id2,filter_phe_id())
		}

		output$filter_phe_id_info = renderPrint({
			cat(paste0("Tip: ", length(cancer_choose$filter_phe_id), " samples are retained"))
		})
	})


	check_omics = reactiveValues(mRNA=TRUE,mutation=TRUE,cnv=TRUE)

	updateVirtualSelect(
		"gene_id",
		choices = tcga_id_option[["Molecular profile"]][["mRNA Expression"]]$all,
		selected = tcga_id_option[["Molecular profile"]][["mRNA Expression"]]$default
	)

	notify <- function(msg, id = NULL) {
		showNotification(msg, id = id, duration = NULL, closeButton = FALSE, type = "message")
	}
	observeEvent(input$step2_2_load, {
		id <- notify(h3("[1/3] Caching mRNA data..."))
		on.exit(removeNotification(id), add = TRUE)

		dat_tmp = query_pancan_value(input$gene_id, "mRNA")
		if(all(is.na(dat_tmp$expression))){check_omics$mRNA=FALSE}
		Sys.sleep(0.5)

		notify(h3("[2/3] Caching mutation data..."), id = id)
		dat_tmp = query_pancan_value(input$gene_id, "mutation")
		if(all(is.na(dat_tmp))){check_omics$mutation=FALSE}
		Sys.sleep(0.5)

		notify(h3("[3/3] Caching CNV data..."), id = id)
		opt_pancan = .opt_pancan
		opt_pancan$toil_cnv$use_thresholded_data=T
		dat_tmp = query_pancan_value(input$gene_id, "cnv",opt_pancan = opt_pancan)
		if(all(is.na(dat_tmp$data))){check_omics$cnv=FALSE}
		Sys.sleep(0.5)

		output$step2_2_text = renderPrint({
			cat(paste0("Tips: \n(1) mRNA is ", 
					   ifelse(check_omics$mRNA,"OK; ","missing; "),
					   "\n(2) Mutation is ",
					   ifelse(check_omics$mutation,"OK; ","missing; "),
					   "\n(3) CNV is ",
					   ifelse(check_omics$cnv,"OK.","missing.")
			))
		})
	})


	candi_trans = reactive({
		# req(input$gene_id)
		load_data("v2_tpc_id_help")$tcga$id_trans %>% 
			dplyr::filter(Symbol == input$gene_id) %>% 
			dplyr::pull(Level3) %>% sort()
	})
	observe({
		updateVirtualSelect(
			"trans_id",
			choices = candi_trans(),
			selected = candi_trans()[1:5]
		)
	})
	observeEvent(input$step2_3_load, {
		withProgress(message = "Caching transcript... ", {
			valid_trans = rep(TRUE, length(input$trans_id))
			for (i in seq(input$trans_id)) {
				incProgress(1 / length(input$trans_id), detail = paste0("Now: ",i))
				# gene_trans_tmp = query_pancan_value(input$trans_id[i], "transcript")
				gene_trans_tmp = query_pancan_value(
					input$trans_id[i], "transcript"
				) %>% as.data.frame() %>% 
					dplyr::select(expression) %>% 
					dplyr::rename("Trans"="expression") %>% 
					tibble::rownames_to_column("sample") %>% 
					dplyr::inner_join(tcga_gtex) %>% 
					dplyr::filter(type2=="tumor") %>%
					dplyr::group_by(tissue) %>% 
					dplyr::summarise(Trans = median(Trans))
				if(sd(gene_trans_tmp$Trans)==0){
					valid_trans[i] = FALSE
				}
			}
			check_omics$valid_trans = input$trans_id[valid_trans]
		})
		output$step2_3_text = renderPrint({
			cat(paste0("Tip: ",length(check_omics$valid_trans)," valid transcript(s) is cached."))
		})
	})


	candi_methy = reactive({
		load_data("v2_tpc_id_help")$tcga$id_M450 %>% 
			dplyr::filter(Level3 == input$gene_id) %>% 
			dplyr::pull(CpG) %>% sort()
	})
	observe({
		updateVirtualSelect(
			"methy_id",
			choices = candi_methy(),
			selected = candi_methy()[1:5]
		)
	})
	observeEvent(input$step2_4_load, {
		withProgress(message = "Caching methylation... ", {
			for (i in seq(input$methy_id)) {
				incProgress(1 / length(input$methy_id), detail = paste0("Now: ",i))
				opt_pancan = .opt_pancan
				opt_pancan$toil_methylation$rule_out = setdiff(candi_methy(), input$methy_id[i])
				opt_pancan$toil_methylation$aggr = "mean"
				gene_methy_tmp = query_pancan_value(input$gene_id, "methylation", opt_pancan=opt_pancan)
			}
		})
		check_omics$valid_methy = input$methy_id
		output$step2_4_text = renderPrint({
			# cat(paste0("Tip: Methylation is OK."))
			cat(paste0("Tip: ",length(check_omics$valid_methy)," valid cpg site(s) is cached."))
		})
	})

	plot_func = eventReactive(input$step3_plot,{
		shiny::validate(
			need(all(check_omics$mRNA,check_omics$mRNA,check_omics$mRNA), 
				"Please load valid mRNA/Mutation/CNV data in Step2.2"),
			# need(try(length(check_omics$valid_trans)>0), 
			# 	"Please load valid transcript data in Step2.3"),
			# need(try(length(check_omics$valid_methy)>0), 
			# 	"Please load valid methylation data in Step2.4"),
			need(try(length(check_omics$valid_trans)<15), 
				"Please select less than 15 transcript in Step2.3"),
			need(try(length(check_omics$valid_methy)<15), 
				"Please select less than 15 methylation CpG sites in Step2.4"),
		)
		shinyjs::disable("step3_plot")
		res = vis_gene_cross_omics(input$gene_id,
							 tumor_projects = cancer_choose$name,
							 tumor_samples = cancer_choose$filter_phe_id,
							 n_trans = check_omics$valid_trans,
							 n_methy = check_omics$valid_methy,
							 add_mean_trans = input$overall_trans,
							 add_mean_methy = input$overall_methy,
							 return_list = TRUE)
        shinyjs::enable("step3_plot")
		res
	})

	output$funky_plot = renderPlot({plot_func()$plot})

	# three download buttons
    observeEvent(input$step3_plot, {
        shinyjs::show("save_plot_bt")
        shinyjs::show("save_data_res")
		updateNumericInput(session, "save_plot_H", value = plot_func()$plot$height)
		updateNumericInput(session, "save_plot_W", value = plot_func()$plot$width)
    })

	output$save_plot_bt = downloadHandler(
		filename = function(){
			paste0("Plot", "_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".",tolower(input$save_plot_F))
		},
		content = function(file){
            p = plot_func()$plot
		    if (input$save_plot_F == "PDF") {
		      pdf(file, width = input$save_plot_W, height = input$save_plot_H, onefile = FALSE)
		      print(p)
		      dev.off()
		    } else if (input$save_plot_F == "PNG"){
		      png(file, width = input$save_plot_W, height = input$save_plot_H, res = 600, units = "in")
		      print(p)
		      dev.off()
		    }
		}
	)
	output$save_data_res = downloadHandler(
		filename = function(){
			paste0("Statdata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv")
		},
		content = function(file){
            p_stat = plot_func()$data
			p_stat = apply(p_stat,2,as.character) #list column
			write.csv(p_stat, file, row.names = FALSE)
		}
	)

}

ui.modules_pancan_comp_o2o = function(id) {
	ns = NS(id)
	fluidPage(
		fluidRow(
			# 初始设置
			column(
				3,
				wellPanel(
					style = "height:1100px",
					h2("S1: Preset", align = "center"),

					h4(strong("S1.1 Modify datasets"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Modify datasets", 
					                   content = "data_origin"),
					mol_origin_UI(ns("mol_origin2comp"), database = "toil"),

					h4(strong("S1.2 Choose cancer")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Cancer types", 
					                   content = "tcga_types"),
					pickerInput(
						ns("choose_cancer"), NULL,
						choices = sort(tcga_names),
						selected = "BRCA"),
				    br(),

					h4(strong("S1.3 Filter samples"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Filter samples", 
					                   content = "choose_samples"),
					h5("Quick filter:"),
					pickerInput(
						ns("filter_by_code"), NULL,
						choices = NULL, selected =  NULL,
						multiple = TRUE, options = list(`actions-box` = TRUE)
					),
					h5("Exact filter:"),
					filter_samples_UI(ns("filter_samples2comp"), database = "toil"),
					br(),
					verbatimTextOutput(ns("filter_phe_id_info")),
					br(),

					h4(strong("S1.4 Upload metadata"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Upload metadata", 
					                   content = "custom_metadata"),
					shinyFeedback::useShinyFeedback(),
					custom_meta_UI(ns("custom_meta2comp")),
					br(),
					
					h4(strong("S1.5 Add signature"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Add signature", 
					                   content = "add_signature"),
					add_signature_UI(ns("add_signature2comp"), database = "toil"),
				)
			),
			# 分组设置
			column(
				4,
				wellPanel(
					style = "height:1100px",
					h2("S2: Get data", align = "center"),
					h4(strong("S2.1 Divide 2 groups by one condition")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Divide 2 groups", 
					                   content = "set_groups"),
					# 调用分组模块UI
					group_samples_UI(ns("group_samples2comp"), database = "toil"),
					h4(strong("S2.2 Get data for comparison")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Get one data", 
					                   content = "get_one_data"), 
					# 下载待比较数据
					download_feat_UI(ns("download_y_axis"), button_name="Query", database = "toil"),

				)
			),
			# 分析/绘图/下载
			column(
				5,
				wellPanel(
					h2("S3: Analyze & Visualize", align = "center") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Analyze & Visualize", 
					                   content = "analyze_comp_1"),  
					style = "height:1100px",
					h4(strong("S3.1 Set analysis parameters")), 
					selectInput(ns("comp_method"), "Comparison method:",choices = c("wilcoxon","t-test"),selected="wilcoxon"),

					h4(strong("S3.2 Set visualization parameters")), 
					fluidRow(
						column(3, colourpicker::colourInput(inputId = ns("group_1_color"), "Color (Group 1):", "#E69F00")),
						column(3, colourpicker::colourInput(inputId = ns("group_2_color"), "Color (Group 2):", "#56B4E9")),
					),
					dropMenu(
						actionBttn(ns("more_visu"), label = "Other options", style = "bordered",color = "success",icon = icon("bars")),
						div(h3("1. Select ggplot theme:"),style="width:400px;"),
						fluidRow(
							column(6,
								selectInput(inputId = ns("theme"), label = NULL, 
											choices = names(themes_list), selected = "ggstatplot")
							)
						),
						div(h3("2. Adjust points:"),style="width:400px;"),
						fluidRow(
							column(3, numericInput(inputId = ns("point_size"), label = "Point size:", value = 3, step = 0.5)),
							column(3, numericInput(inputId = ns("point_alpha"), label = "Point alpha:", value = 0.4, step = 0.1, min = 0, max = 1)),
						),
						div(h3("3. Adjust text size:"),style="width:400px;"),
						fluidRow(
							column(4, numericInput(inputId = ns("axis_size"), label = "Text size:", value = 18, step = 0.5)),
							column(4, numericInput(inputId = ns("title_size"), label = "Title size:", value = 20, step = 0.5))
						),				
						div(h3("4. Adjust lab and title name:"),style="width:400px;"),
						fluidRow(
							column(4, textInput(inputId = ns("x_name"), label = "X-axis name:")),
							column(4, textInput(inputId = ns("y_name"), label = "Y-axis name:")),
							column(4, textInput(inputId = ns("title_name"), label = "Title name:"))
						),	
						div(h5("Note: You can download the raw data and plot in local R environment for more detailed adjustment.")),

					),
					br(),
					# verbatimTextOutput(ns("tmp123")),
					shinyWidgets::actionBttn(
						ns("step3_plot_box"), "Run",
				        style = "gradient",
				        icon = icon("chart-line"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					),
					br(),
					fluidRow(
						column(10, offset = 1,
							   plotOutput({ns("comp_plot_box")}, height = "480px") 
						)
					),
					br(),
					h4(strong("S3.3 Download results")), 
					download_res_UI(ns("download_res2comp"))
				)
			)
		)
	)
}


server.modules_pancan_comp_o2o = function(input, output, session) {
	ns <- session$ns

	# 记录选择癌症
	cancer_choose <- reactiveValues(name = "BRCA", 
		phe_primary=query_tcga_group(database = "toil",cancer = "BRCA", return_all = T),
		filter_phe_id=NULL, single_cancer_ok = TRUE, multi_cancer_ok=FALSE)
	observe({
		cancer_choose$name = input$choose_cancer
		cancer_choose$phe_primary <- query_tcga_group(database = "toil",cancer = cancer_choose$name, return_all = T)
	})

	# 自定义上传metadata数据
	custom_meta = callModule(custom_meta_Server, "custom_meta2comp", database = "toil")
	# signature
	sig_dat = callModule(add_signature_Server, "add_signature2comp", database = "toil")

	custom_meta_sig = reactive({
		if(is.null(custom_meta())){
			return(sig_dat())
		} else {
			if(is.null(sig_dat())){
				return(custom_meta())
			} else {
				custom_meta_sig = dplyr::inner_join(custom_meta(),sig_dat())
				return(custom_meta_sig)
			}
		}
	})

	# 数据源设置
	opt_pancan = callModule(mol_origin_Server, "mol_origin2comp", database = "toil")

	## 过滤样本
	# exact filter module
	filter_phe_id = callModule(filter_samples_Server, "filter_samples2comp",
					   database = "toil",
					   cancers=reactive(cancer_choose$name),
					   custom_metadata=reactive(custom_meta_sig()),
					   opt_pancan = reactive(opt_pancan()))
	# quick filter widget
	observe({
		code_types_valid = code_types[names(code_types) %in% 
							unique(cancer_choose$phe_primary$Code)]
		updatePickerInput(
			session,
			"filter_by_code",
			choices = unlist(code_types_valid,use.names = F),
			selected =  unlist(code_types_valid,use.names = F)
		)
	})
	# 综合上述二者
	observe({
		# quick filter
		choose_codes = names(code_types)[unlist(code_types) %in% input$filter_by_code]
		filter_phe_id2 = cancer_choose$phe_primary %>%
			dplyr::filter(Code %in% choose_codes) %>%
			dplyr::pull("Sample")

		# exact filter
		if(is.null(filter_phe_id())){
			cancer_choose$filter_phe_id = filter_phe_id2
		} else {
			cancer_choose$filter_phe_id = intersect(filter_phe_id2,filter_phe_id())
		}

		output$filter_phe_id_info = renderPrint({
			cat(paste0("Tip: ", length(cancer_choose$filter_phe_id), " samples are retained"))
		})
	})


	# 设置分组
	group_final = callModule(group_samples_Server, "group_samples2comp",
					   	   database = "toil",
						   cancers=reactive(cancer_choose$name),
						   samples=reactive(cancer_choose$filter_phe_id),
						   custom_metadata=reactive(custom_meta_sig()),
						   opt_pancan = reactive(opt_pancan())
						   )

	# 下载待比较数据
	y_axis_data = callModule(download_feat_Server, "download_y_axis", 
							 database = "toil",
							 samples=reactive(cancer_choose$filter_phe_id),
							 custom_metadata=reactive(custom_meta_sig()),
						     opt_pancan = reactive(opt_pancan()),
						     check_numeric=TRUE,
						     table.ui=FALSE
							 )


	# 合并分析
	# boxviolin逻辑：先绘图，再提取相关性结果
	merge_data_box = eventReactive(input$step3_plot_box, {
		group_data = group_final()[,c(1,3,4)]
		colnames(group_data) = c("Sample","group","phenotype")
		y_axis_data = y_axis_data()
		data = dplyr::inner_join(y_axis_data, group_data) %>%
			dplyr::select(cancer, Sample, value, group, everything())
		data
	})

	# 检查数据
	observe({
		cancer_choose$single_cancer_ok = min(table(merge_data_box()$group))>=3
	})


	observe({
		updateTextInput(session, "x_name", value = "group")
		updateTextInput(session, "y_name", value = unique(y_axis_data()$id))
		updateTextInput(session, "title_name", value = cancer_choose$name)
	})


	comp_plot_box = eventReactive(input$step3_plot_box, {
		shiny::validate(
			need(try(nrow(merge_data_box())>0), 
				"Please inspect whether to set groups or download variable data in S2 or S3 step."),
			need(try(length(unique(merge_data_box()$group))==2), 
				"No two groups for comparing, please check your input."),
			need(try(min(table(merge_data_box()$group))>2),   
				"No enough samples (n≤3) for one group for comparing, please check your input."),	
		)
		merge_data_box = merge_data_box()

		if(!cancer_choose$single_cancer_ok){
			return("No enough samples for comparing, please check your input.")
		} else {
			p = plot_comb_o2o(
				data = merge_data_box(), xlab=input$x_name, ylab=input$y_name, title=input$title_name,
				comp_method=input$comp_method, point_size=input$point_size, point_alpha=input$point_alpha,
				group_1_color=input$group_1_color, group_2_color=input$group_2_color,
				axis_size=input$axis_size, title_size=input$title_size,
				custom_theme=themes_list[[input$theme]]
			)
			return(p)
		}
	})
	output$comp_plot_box = renderPlot({comp_plot_box()})

	# Download results
	observeEvent(input$step3_plot_box,{
		res1 = comp_plot_box()
		res2 = merge_data_box()
		p_comp = extract_stats(comp_plot_box())$subtitle_data
		p_comp = p_comp[-which(colnames(p_comp)=="expression")]
		p_comp$identifier = unique(merge_data_box()$id)
		p_comp$phenotype = unique(merge_data_box()$phenotype)
		p_comp$group_1 = levels(merge_data_box()$group)[1]
		p_comp$group_2 = levels(merge_data_box()$group)[2]
		res3 = p_comp
		callModule(download_res_Server, "download_res2comp", res1, res2, res3)
	})
}

ui.home_search_box <- function(id) {
  ns <- NS(id)

  # ref: https://shiny.rstudio.com/articles/selectize.html
  # https://stackoverflow.com/questions/51343552/dynamic-selectizeinput-in-shiny
  tagList(
    fluidRow(
      column(10, offset = 1,
      #   5,
        selectInput(
          inputId = ns("Pancan_search_type"),
          # label = NULL,
          label = "1. Select one molecule:",
          choices = c("mRNA", "transcript", "protein", "mutation",
                      "cnv",  "methylation", "miRNA"),
          selected = "mRNA"
        )
      )
    ),
    fluidRow(
      column(10, offset = 1,
        virtualSelectInput(
          inputId = ns("Pancan_search"),
          label = NULL,
          choices = NULL,
          width = "100%",
          search = TRUE,
          allowNewOption = TRUE,
          dropboxWidth = "200%"
        )
      )
    ),
    br(),
    fluidRow(
      column(11, offset = 1,
        h5(strong("2.Run two explorations:"))
      )
    ),
    fluidRow(
      column(
        5, offset = 1,
        # align="center",
        actionBttn(
          inputId = ns("search_1"),
          label = "Tumor vs. Normal",
          color = "primary",
          style = "bordered",
          size = "sm"
        )
      ),
      column(
        5,
        # align="center",
        actionBttn(
          inputId = ns("search_2"),
          label = "Generate Report",
          color = "primary",
          style = "bordered",
          size = "sm"
        )
      )
    )
  )

}

server.home_search_box <- function(input, output, session) {
  ns <- session$ns

  observe({

    mol_choices = switch(ifelse(is.null(input$Pancan_search_type),"mRNA",input$Pancan_search_type),
      "mRNA" = tcga_id.list[["Gene"]],
      "transcript" = tcga_id_option$`Molecular profile`$`Transcript Expression`$all,
      "protein" = tcga_id_option$`Molecular profile`$`Protein Expression`$all,
      "mutation" = tcga_id_option$`Molecular profile`$`Mutation status`$all,
      "cnv" = tcga_id_option$`Molecular profile`$`Copy Number Variation`$all,
      "methylation" = tcga_id_option$`Molecular profile`$`DNA Methylation`$all,
      "miRNA" = tcga_id_option$`Molecular profile`$`miRNA Expression`$all
    )
    mol_selected = switch(ifelse(is.null(input$Pancan_search_type),"mRNA",input$Pancan_search_type),
      "mRNA" = "TP53",
      "transcript" = tcga_id_option$`Molecular profile`$`Transcript Expression`$default,
      "protein" = tcga_id_option$`Molecular profile`$`Protein Expression`$default,
      "mutation" = tcga_id_option$`Molecular profile`$`Mutation status`$default,
      "cnv" = tcga_id_option$`Molecular profile`$`Copy Number Variation`$default,
      "methylation" = tcga_id_option$`Molecular profile`$`DNA Methylation`$default,
      "miRNA" = tcga_id_option$`Molecular profile`$`miRNA Expression`$default
    )
    updateVirtualSelect(
      "Pancan_search",
      choices = mol_choices,
      selected = mol_selected
    )
  })

  observeEvent(input$search_1, {
    message(input$Pancan_search, " is queried by user from home search box.")
    if (nchar(input$Pancan_search) >= 1) {
      showModal(
        modalDialog(
          title = paste("Pancan distribution of gene", input$Pancan_search),
          size = "l",
          fluidPage(
            fluidRow(
              materialSwitch(ns("pdist_mode"), "Show violin plot", inline = TRUE),
              materialSwitch(ns("pdist_show_p_value"), "Show P value", inline = TRUE),
              materialSwitch(ns("pdist_show_p_label"), "Show P label", inline = TRUE)
            ),
            column(
              12,
              plotOutput(ns("gene_pancan_dist"))
            ),

            br(),
            
            column(
              12,
              h4("NOTEs:"),
              h5("1. The data query may take some time based on your network. Wait until a plot shows"),
              h5("2. You have to turn on both 'Show P value' and 'Show P label' to show significant labels"),
              h5("3. If a void plot shows, please check your input"),
              h5("4. You can get more features for this plot in page 'Quick PanCan Analysis'"),
              h5("")
            )
          )
        )
      )
      output$gene_pancan_dist <- renderPlot({
        p <- vis_toil_TvsN(
          Gene = input$Pancan_search,
          data_type = input$Pancan_search_type,
          Mode = ifelse(input$pdist_mode, "Violinplot", "Boxplot"),
          Show.P.value = input$pdist_show_p_value,
          Show.P.label = input$pdist_show_p_label
        )
        shiny::validate(
          need(try(inherits(p,'gg')), 
            "Error: Please select a valid molecule.")
        )
        p + cowplot::theme_cowplot() + ggpubr::rotate_x_text(45)
      })
    }
  })


  observeEvent(input$search_2, {
    message("Generate an analysis for report for ",input$Pancan_search)
    if (nchar(input$Pancan_search) >= 1) {
      showModal(
        modalDialog(
          title = paste("Generate an analysis report for ",
                        input$Pancan_search_type," ",input$Pancan_search),
          size = "l",
          fluidPage(
            fluidRow(
              column(
                4,
                wellPanel(
                  style = "height:180px",
                  h4("1. Run the analysis:"),
                  br(),
                  column(
                    12, align="center", 
                    actionBttn(
                      inputId = ns("report_1"),
                      label = "Go!",
                      color = "primary",
                      style = "pill",
                      size = "md",
                      icon = shiny::icon("gear")
                    )
                  ),
                  br(),
                  fluidRow(textOutput(ns("tip1"))),
                )
              ),
              column(
                4,
                wellPanel(
                  style = "height:180px",
                  h4("2. Knit the report:"),
                  br(),
                  column(
                    12, align="center", 
                    downloadBttn(
                      outputId = ns("report_2"),
                      label = "Download",
                      color = "primary",
                      style = "pill",
                      size = "md"
                    ),
                  ),
                  br(),
                  fluidRow(textOutput(ns("tip2"))),
                )
              ),
              column(
                4,
                wellPanel(
                  style = "height:180px",
                  h4("3. Obtain analyzed result:"),
                  br(),
                  column(
                    12, align="center", 
                    downloadBttn(
                      outputId = ns("report_3"),
                      label = "Download",
                      color = "primary",
                      style = "pill",
                      size = "md"
                    )
                  )
                )
              ),              
            ),
            fluidRow(
              br(),
              h3("NOTEs:"),
              h4("1. It will take about ",strong("one minute",style="color:red")," for the general analysis which includes the relationships between queried molecule and ",
                "(1) clinical phenotypes, (2) survival influence, (3) tumor index (4) immune infiltration, (5) pathway activity.",
                "You can see the ",a("example report", href = "https://lishensuo.github.io/UCSCXenaShiny_Book/example_report.html")," while waiting."),
              h4("2. It will take about ", strong("10 seconds",style="color:red"), " for the generation of organized report in html format."),
              h4("3. Final, the analyzed result under the report can be directly downloaded in zip format.")
            )
          )
        )
      )
    }

  })



  w <- waiter::Waiter$new(id = ns("tip1"), html = waiter::spin_hexdots(), color = "grey95")
  observeEvent(input$report_1, {
    w$show()
    output$tip1 = renderText({
      if(report_1_OK()) {
        paste0("Tip: You now can run step2 or step3. (",
          format(Sys.time(), "%H:%M:%S"), ")")
      } else {
        "Error: Please exit and select a valid molecule."
      }
    })
  })

  report_1_OK <- reactiveVal(TRUE)
  observeEvent(input$report_1, {    
    time_stamp = format(Sys.time(), "%Y-%m-%d_%H-%M-%S")
    res_dat = mol_quick_analysis(molecule = input$Pancan_search, 
                       data_type = input$Pancan_search_type, 
                       out_dir = tempdir(), out_report = FALSE)

    if(is.null(res_dat)) {
      report_1_OK(FALSE)
    } 
    shiny::validate(
      need(try(!is.null(res_dat)), 
        "Error: Please exit and select a valid molecule.")
    )

    output$report_2 = downloadHandler(   
      filename = paste0(time_stamp,"_report.html"),
      content = function(file) {
        tempReport <- file.path(tempdir(), paste0(time_stamp,"_report.Rmd"))
        file.copy(system.file("rmd","report_template.Rmd", package = "UCSCXenaShiny"), 
                  tempReport, overwrite = TRUE)
        params = list("sur_res" = res_dat$sur_res,
                      "cor_res" = res_dat$cor_res,
                      "phe_res" = res_dat$phe_res,
                      "id_name" = input$Pancan_search,
                      "id_type" = input$Pancan_search_type)
        rmarkdown::render(tempReport, output_file = file,
          params = params,
          envir = new.env(parent = globalenv()))
        
      }
    )

    output$report_3 = downloadHandler(
      filename = paste0(time_stamp,"_rawdata.zip"),
      content = function(file) {

        fs1 = file.path(tempdir(), paste0(time_stamp,"_molecule_clinical_result.csv"))
        fs2 = file.path(tempdir(), paste0(time_stamp,"_molecule_correlation_result.csv"))
        fs3 = file.path(tempdir(), paste0(time_stamp,"_molecule_survival_result.csv"))

        write.csv(res_dat$phe_res, row.names = FALSE, file=fs1)
        write.csv(res_dat$cor_res, row.names = FALSE, file=fs2)
        write.csv(res_dat$sur_res, row.names = FALSE, file=fs3)

        fs = c(fs1, fs2, fs3)

        zip(zipfile=file, files=fs, mode = "cherry-pick")
      },
      contentType = "application/zip"
    )

  })


}

ui.modules_pcawg_cor_m2o = function(id) {
	ns = NS(id)
	fluidPage(
		fluidRow(
			# 初始设置
			column(
				3,
				wellPanel(
					style = "height:1100px",
					h2("S1: Preset", align = "center"),

					h4(strong("S1.1 Modify datasets"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Modify datasets", 
					                   content = "data_origin"),
					mol_origin_UI(ns("mol_origin2cor"), database = "pcawg"),

					h4(strong("S1.2 Choose project")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "PCAWG projects", 
					                   content = "pcawg_projects"),
					pickerInput(
						ns("choose_cancer"),NULL,
						choices = pcawg_names),
					br(),

					h4(strong("S1.3 Filter samples"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Filter samples", 
					                   content = "choose_samples"),
					h5("Quick filter:"),
					pickerInput(
						ns("filter_by_code"), NULL,
						choices = NULL, selected =  NULL,
						multiple = TRUE, options = list(`actions-box` = TRUE)
					),
					h5("Exact filter:"),
					filter_samples_UI(ns("filter_samples2cor"), database = "pcawg"),
					br(),
					verbatimTextOutput(ns("filter_phe_id_info")),
					br(),

					h4(strong("S1.4 Upload metadata"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Upload metadata", 
					                   content = "custom_metadata"),
					shinyFeedback::useShinyFeedback(),
					custom_meta_UI(ns("custom_meta2cor")),
					br(),

					h4(strong("S1.5 Add signature"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Add signature", 
					                   content = "add_signature"),
					add_signature_UI(ns("add_signature2cor"), database = "pcawg"),

				)
			),
			column(
				4,
				wellPanel(
					style = "height:1100px",
					h2("S2: Get data", align = "center"),
					# 批量数据下载
					h4(strong("S2.1 Get batch data for X-axis")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Get batch data", 
					                   content = "get_batch_data"), 
					multi_upload_UI(ns("multi_upload2cor"),
						button_name = "Query", database = "pcawg"),
					# br(),br(),
					# 单项数据下载
					h4(strong("S2.2 Get data for Y-axis")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Get one data", 
					                   content = "get_one_data"),  
					download_feat_UI(ns("download_y_axis"), 
						button_name="Query", database = "pcawg")
				)
			),
			column(
				5,
				wellPanel(
					style = "height:1100px",
					h2("S3: Analyze", align = "center") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Analyze", 
					                   content = "analyze_cor_3"), 

					br(),br(),
					h4(strong("S3.1 Set analysis parameters")), 
					selectInput(ns("cor_method"), "Correlation method:",choices = c("pearson", "spearman")),

					# br(),br(),
					shinyWidgets::actionBttn(
						ns("cal_batch_cor"), "Run",
				        style = "gradient",
				        icon = icon("table"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					),
					br(),
					fluidRow(
						column(10, offset = 1,
							   div(uiOutput(ns("cor_stat_tb.ui")),style = "height:600px"),
							   )
					),
					h4(strong("S3.2 Download results")), 
					download_res_UI(ns("download_res2cor"))
				)
			)
		)
	)
}


server.modules_pcawg_cor_m2o = function(input, output, session) {
	ns <- session$ns
	# 记录选择癌症
	cancer_choose <- reactiveValues(name = "BLCA-US", phe_primary="",
		filter_phe_id=query_tcga_group(database = "pcawg", cancer = "BLCA-US", return_all = T))
	observe({
		cancer_choose$name = input$choose_cancer
		cancer_choose$phe_primary <- query_tcga_group(database = "pcawg",
			cancer = cancer_choose$name, return_all = T)
	})

	# 数据源设置
	opt_pancan = callModule(mol_origin_Server, "mol_origin2cor", database = "pcawg")

	# 自定义上传metadata数据
	custom_meta = callModule(custom_meta_Server, "custom_meta2cor", database = "pcawg")
	# signature
	sig_dat = callModule(add_signature_Server, "add_signature2cor", database = "pcawg")
	custom_meta_sig = reactive({
		if(is.null(custom_meta())){
			return(sig_dat())
		} else {
			if(is.null(sig_dat())){
				return(custom_meta())
			} else {
				custom_meta_sig = dplyr::inner_join(custom_meta(),sig_dat())
				return(custom_meta_sig)
			}
		}
	})

	## 过滤样本
	# quick filter widget
	observe({
		code_types_valid = unique(cancer_choose$phe_primary$Type)
		updatePickerInput(
			session,
			"filter_by_code",
			choices = code_types_valid,
			selected =  code_types_valid
		)
	})
	# exact filter module
	filter_phe_id = callModule(filter_samples_Server, "filter_samples2cor",
					   database = "pcawg",
					   cancers=reactive(cancer_choose$name),
					   custom_metadata=reactive(custom_meta_sig()),
					   opt_pancan = reactive(opt_pancan()))

	# 综合上述二者
	observe({
		# quick filter
		filter_phe_id2 = cancer_choose$phe_primary %>%
			dplyr::filter(Type %in% input$filter_by_code) %>%
			dplyr::pull("Sample")

		# exact filter
		if(is.null(filter_phe_id())){
			cancer_choose$filter_phe_id = filter_phe_id2
		} else {
			cancer_choose$filter_phe_id = intersect(filter_phe_id2,filter_phe_id())
		}

		output$filter_phe_id_info = renderPrint({
			cat(paste0("Tip: ", length(cancer_choose$filter_phe_id), " samples are retained"))
		})
	})

	# 批量下载数据
	L3s_x_data =  callModule(multi_upload_Server, "multi_upload2cor", 
							 database = "pcawg",
							 samples=reactive(cancer_choose$filter_phe_id),
							 custom_metadata=reactive(custom_meta_sig()),
						     opt_pancan = reactive(opt_pancan())
							 )
	L3s_x = reactive({
		unique(L3s_x_data()$id)
	})

	output$tmp123 = renderPrint({head(L3s_x_data())})
	output$tmp456 = renderPrint({head(L3_y_data())})


	L3_y_data = callModule(download_feat_Server, "download_y_axis", 
							 database = "pcawg",
							 samples=reactive(cancer_choose$filter_phe_id),
							 custom_metadata=reactive(custom_meta_sig()),
						     opt_pancan = reactive(opt_pancan()),
						     check_numeric=TRUE
							 )

	# 相关性分析
	cor_stat = eventReactive(input$cal_batch_cor,{
		shinyjs::disable("cal_batch_cor")
		x_datas = L3s_x_data()[,c("id","Sample","value")]
		colnames(x_datas)[c(1,3)] = paste0("x_",colnames(x_datas)[c(1,3)])
		y_data = L3_y_data()[,c("id","Sample","value")]
		colnames(y_data)[c(1,3)] = paste0("y_",colnames(y_data)[c(1,3)])

		withProgress(message = "Please wait for a while.",{
			cor_stat = lapply(seq(L3s_x()), function(i) {
			    incProgress(1 / length(L3s_x()), detail = paste0("(Finished ",i,"/",length(L3s_x()),")"))
				
				L3_x = L3s_x()[i]
				xy_data = x_datas %>%
					dplyr::filter(x_id == L3_x) %>%
					dplyr::inner_join(y_data) %>% as.data.frame()
				if(nrow(na.omit(xy_data))==0){return(c(NaN, NaN))}
			    cor_obj = cor.test(xy_data[,"x_value"],xy_data[,"y_value"],
			                       method = input$cor_method)
			   	return(c(cor_obj$p.value, cor_obj$estimate))
			}) %>% do.call(rbind, .) %>% as.data.frame()
			colnames(cor_stat) = c("p.value","R")
			cor_stat2 = cor_stat %>% 
			  dplyr::select(R, p.value) %>% 
			  dplyr::mutate(id.x = L3s_x(), .before = 1) %>% 
			  dplyr::mutate(id.y = input$L3_y, .before = 2) %>%
			  dplyr::arrange(p.value)
			shinyjs::enable("cal_batch_cor")
			cor_stat2
		})
	})

	output$cor_stat_tb.ui = renderUI({
		output$cor_stat_tb = renderDataTable({
			cor_stat_ = cor_stat() %>%
				dplyr::rename("Batch identifiers"="id.x")
			cor_stat_$p.value = format(cor_stat_$p.value, scientific=T, digits = 3)
			dt = datatable(cor_stat_,
				# class = "nowrap row-border",
				options = list(pageLength = 10, 
					columnDefs = list(
						list(className = 'dt-center', targets="_all"),
						list(orderable=TRUE, targets = 0)))
			) %>%
				formatRound(columns = c("R"), digits = 3)
			dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
			dt
		}) 
	dataTableOutput(ns("cor_stat_tb"))
	})

	# Download results
	observeEvent(input$cal_batch_cor,{
		res1 = NULL
		x_datas = L3s_x_data() %>%
			dplyr::mutate(axis = "X")
		y_data = L3_y_data() %>%
			dplyr::select(id, Sample, value) %>%
			dplyr::mutate(axis = "Y")
		res2 = rbind(x_datas, y_data)
		res3 = cor_stat()
		callModule(download_res_Server, "download_res2cor", res1, res2, res3)
	})
}

uiFeatureDatabaseSig <- function(id){
  ns <- NS(id)
  fluidPage(
    useWaiter(),
    fluidRow(
      # Select Features ----
      column(3,
             selectInput(inputId = ns("select_features1"), 
                         "Please select target feature type:", 
                         choices = c("Copy Number Data" = "cnv",
                                     "DNA Methylation" = "meth",
                                     "Gene Fusion" = "fusion",
                                     "Gene Mutation" = "mutation_gene",
                                     "Gene Site Mutation" = "mutation_site",
                                     "mRNA Expression" = "mRNA",
                                     "Protein Expression" = "protein",
                                     "Drug Sensitivity" = "drug"
                         )
             )),
      # Select specific feature ----
      column(3,
             selectizeInput(
               ns("select_specific_feature"), "Target feature:", choices = NULL,
               options = list(
                 placeholder = 'Please select a feature',
                 onInitialize = I('function() { this.setValue(""); }'), selected = ""
               ))
      ),
      # Select Feature to compare ----
      column(3,
             selectInput(inputId = ns("select_features2"), 
                         "Feature type to scan:", 
                         choices = c("Copy Number Data" = "cnv",
                                     "DNA Methylation" = "meth",
                                     "Gene Fusion" = "fusion",
                                     "Gene Mutation" = "mutation_gene",
                                     "Gene Site Mutation" = "mutation_site",
                                     "mRNA Expression" = "mRNA",
                                     "Protein Expression" = "protein",
                                     "Drug Sensitivity" = "drug"
                         )
             )),
      # Set the threshold value for setting significance ----
      column(3, 
             uiOutput(ns("threshold_value"))
      )  
    ),
    # Output results ----
    wellPanel(
      # plotOutput(ns("p_search2"), height="20cm"),
      column(12,
             tabsetPanel(
               tabPanel("Frequency Table",
                        DT::dataTableOutput(ns("freq_table"))),
               tabPanel("Result Table",
                        DT::dataTableOutput(ns("re_table"))),
             )),
      h5("."),
    ),
    # Download results ----
    downloadButton(ns("download_sig_re"), 'Download Result Table'),
    h5("."),
    h4(strong("NOTEs:")),
    h5("This analysis can examine connections between a target feature and all features within all datasets."),
    h5("the determination of effect size and p-value varies based on the type of the features being compared:"),
    h5("1. For ", strong("continuous features"), "against ", strong("continuous datasets"),", the Spearman correlation coefficient R, which ranges from 0 to 1, is utilized;"),
    h5("2. When comparing", strong("discrete features"), "against ", strong("continuous datasets"), ", the log2 fold change (events/wildtype) serves as the measure of effect, with p-values obtained from the Wilcoxon test;"),
    h5("3. In the case of", strong(" discrete features"), " against ", strong("discrete datasets"), ", the effect size is quantified using the log2 odds ratio, with p-values calculated via the Chi-squared test;"),
    h5("The continuous features comprise Copy Number Data, DNA Methylation, mRNA Expression, Protein Expression, and drug sensitivity. On the other hand, the discrete features encompass Gene Fusion, Gene Mutation, and Gene Site Mutation."),
    h5("The Frequency table contains two columns: the count column, which records the number of significant pairs, and the proportion column, representing the ratio of significant pairs to total pairs."),
    h5("You can change the threshold to filter the effect size. The result table indicates a 'yes' in the Significance column when the p-value is below 0.05 and the effect size falls above the threshold defined by the user for the feature pair.")
  )
}

serverFeatureDatabaseSig <- function(input, output, session){
  ns <- session$ns
  # Select ----
  features_search_sel <- reactiveValues()
  observeEvent(input$select_features1, {
    features_search_sel$features <- switch(input$select_features1,
                                           "drug" = drugs_search$drugs, 
                                           "mRNA" = omics_search[omics_search$type %in% "mRNA",]$omics,
                                           "meth" = omics_search[omics_search$type %in% "meth",]$omics,
                                           "protein" = omics_search[omics_search$type %in% "protein",]$omics,
                                           "cnv" = omics_search[omics_search$type %in% "cnv",]$omics,
                                           "mutation_gene" = omics_search[omics_search$type %in% "mutation_gene",]$omics,
                                           "mutation_site" = omics_search[omics_search$type %in% "mutation_site",]$omics,
                                           "fusion" = omics_search[omics_search$type %in% "fusion",]$omics)
    updateSelectizeInput(session = session, inputId = 'select_specific_feature',
                         label = 'Target feature:', choices = features_search_sel$features, server = TRUE,
                         options = list(placeholder = 'Please select a feature', onInitialize = I('function() { this.setValue(""); }')),
                         selected = ""
    )
  })

  # Set the threshold value for setting significance ----
  output$threshold_value <- renderUI({
    # Spearman correlation
    if (input$select_features1 %in% c("drug", "cnv",
                                      "protein",
                                      "meth",
                                      "mRNA") & 
        input$select_features2 %in% c("drug", "cnv",
                                      "protein",
                                      "meth",
                                      "mRNA")) {
      numericInput(inputId = ns("dynamic_numeric"), 
                   "Please set the threshold for effect size:", 
                   value = 0.2, min = 0)
      # Chi-squared test
    } else if(!input$select_features1 %in% c("drug", "cnv",
                                             "protein",
                                             "meth",
                                             "mRNA") & 
              !input$select_features2 %in% c("drug", "cnv",
                                             "protein",
                                             "meth",
                                             "mRNA")){
      numericInput(inputId = ns("dynamic_numeric"), 
                   "Please set the threshold for size effect:", 
                   value = 4, min = 0) 
    } else{
      # Wilcoxon test
      numericInput(inputId = ns("dynamic_numeric"), 
                   "Please set the threshold for size effect:", 
                   value = 4, min = 0)
    }
  })
  # Preparation ----
  re_list <- reactive({
    profile_vec1 <- profile_vec_list[[input$select_features1]]
    profile_vec2 <- profile_vec_list[[input$select_features2]]
    profile_comb <- expand.grid(profile_vec1, profile_vec2)
    re_list <- list()
    withProgress(message = "Running (please be patient)", value = 0, {
      waiter_show( # show the waiter
        html = spin_3(), # use a spinner
        color = transparent(0.1)
      )
      for(index in 1:nrow(profile_comb)){
        # w$show()
        # index = 1
        # Prepare
        select_features1_2 <- input$select_features1
        if(select_features1_2 == "mRNA") select_features1_2 <- "exp"
        profile1 <- base::get(paste0(profile_comb[index,1], "_", select_features1_2), envir = parent.env(environment()))
        select_features2_2 <- input$select_features2
        if(select_features2_2 == "mRNA") select_features2_2 <- "exp"
        profile2 <- base::get(paste0(profile_comb[index,2], "_", select_features2_2), envir = parent.env(environment()))
        # Select specific feature and all features data
        # con vs con ----
        if(input$select_features1 %in% c("drug", "cnv",
                                         "protein",
                                         "meth",
                                         "mRNA") & 
           input$select_features2 %in% c("drug", "cnv",
                                         "protein",
                                         "meth",
                                         "mRNA")){
          intersected_cells <- intersect(colnames(profile1), colnames(profile2))
          fea <- profile1[rownames(profile1) %in% input$select_specific_feature,
                          match(intersected_cells, colnames(profile1))] %>% as.numeric()
          db <- profile2[,match(intersected_cells, colnames(profile2))]
          db <- db[!rownames(db) %in% input$select_specific_feature,]
          fea_nrow <- profile1[rownames(profile1) %in% input$select_specific_feature,match(intersected_cells, colnames(profile1))] %>% nrow()
          if(fea_nrow == 0 | length(intersected_cells) == 0){next}
          # Calculate
          re <- lapply(1:nrow(db), function(x){
            re2 <- tryCatch(cor.test(fea, as.numeric(db[x,]),
                                     method = "spearman"),
                            error = function(x){NA})
            if(all(is.na(re2))){
              re3 <- data.frame(
                p = NA,
                effect = NA
              )
            } else {
              re3 <- data.frame(
                p = re2$p.value,
                effect = re2$estimate)
            }
          })
          re <- do.call(rbind, re)
          re$fea <- rownames(db)
          re <- na.omit(re)
          # re <- re[order(re$R),]
          # quantiles_80 <- quantile(re$R, probs = c(0.1, 0.9))[2]
          # quantiles_20 <- quantile(re$R, probs = c(0.1, 0.9))[1]
          # dis vs dis ----
        } else if(!input$select_features1 %in% c("drug", "cnv",
                                                 "protein",
                                                 "meth",
                                                 "mRNA") & 
                  !input$select_features2 %in% c("drug", "cnv",
                                                 "protein",
                                                 "meth",
                                                 "mRNA")){
          intersected_cells <- intersect(profile1[[2]], profile2[[2]]) %>% unique()
          fea <- profile1[profile1[[1]] %in% input$select_specific_feature,]
          db <- profile2[profile2[[2]] %in% intersected_cells,]
          db <- db[!db[[1]] %in% input$select_specific_feature,]
          db_feas <- unique(db[[1]])
          if(nrow(fea) == 0 | length(intersected_cells) == 0){next}
          re <- lapply(1:length(db_feas), function(x){
            fea_cells <- unique(as.data.frame(fea)[,2])
            sel_cells <- unique(as.data.frame(db[db[[1]] %in% db_feas[x],])[,2])
            yes_yes <- length(intersected_cells[intersected_cells %in% intersect(fea_cells, sel_cells)])
            yes_no <- length(intersected_cells[intersected_cells %in% fea_cells & !(intersected_cells %in% sel_cells)])
            no_yes <- length(intersected_cells[intersected_cells %in% sel_cells & !(intersected_cells %in% fea_cells)])
            no_no <- length(intersected_cells[!intersected_cells %in% c(fea_cells, sel_cells)])
            chi_df <- t(data.frame(
              yes = c(yes_yes, yes_no),
              no = c(no_yes, no_no)
            ))
            re2 <- tryCatch(
              chisq.test(chi_df),
              error = function(x){NA}
            )
            if(all(is.na(re2))){
              re3 <- data.frame(
                p = NA,
                effect = NA
              )
            } else {
              re3 <- data.frame(
                p = re2$p.value,
                effect = log2(re2$statistic)
              )
            }
            re3
          })
          re <- do.call(rbind, re)
          re$fea <- db_feas
          re <- na.omit(re)
          # con vs dis ----
        } else if(input$select_features1 %in% c("drug", "cnv",
                                                "protein",
                                                "meth",
                                                "mRNA") & 
                  !input$select_features2 %in% c("drug", "cnv",
                                                 "protein",
                                                 "meth",
                                                 "mRNA")){
          intersected_cells <- intersect(colnames(profile1), profile2[[2]]) %>% unique()
          fea <- profile1[rownames(profile1) %in% input$select_specific_feature,
                          match(intersected_cells, colnames(profile1))]
          db <- profile2[profile2[[2]] %in% intersected_cells,]
          db_feas <- unique(db[[1]])
          if(nrow(fea) == 0 | length(intersected_cells) == 0){next}
          re <- lapply(1:length(db_feas), function(x){
            sel_cells <- as.data.frame(db[db[[1]] %in% db_feas[x],2])
            sel_cells <- sel_cells[,1]
            yes_drugs <- na.omit(as.numeric(fea[,colnames(fea) %in% sel_cells]))
            no_drugs <- na.omit(as.numeric(fea[,!colnames(fea) %in% sel_cells]))
            re2 <- tryCatch(
              wilcox.test(yes_drugs, no_drugs),
              error = function(x){NA}
            )
            if(all(is.na(re2))){
              re3 <- data.frame(
                p = NA,
                effect = NA
              )
            } else {
              re3 <- data.frame(
                p = re2$p.value,
                effect = log2(mean(yes_drugs)/mean(no_drugs))
              )
            }
            re3
          })
          re <- do.call(rbind, re)
          re$fea <- db_feas
          re <- na.omit(re)
          # dis vs con ----
        } else if(!input$select_features1 %in% c("drug", "cnv",
                                                 "protein",
                                                 "meth",
                                                 "mRNA") & 
                  input$select_features2 %in% c("drug", "cnv",
                                                "protein",
                                                "meth",
                                                "mRNA")){
          intersected_cells <- intersect(profile1[[2]], colnames(profile2)) %>% unique()
          db <- profile2[,colnames(profile2) %in% intersected_cells]
          sel_omics <- profile1$cells[profile1[[1]] %in% input$select_specific_feature] %>% unique()
          if(length(intersected_cells) == 0 | length(sel_omics) == 0){next}
          re <- lapply(1:nrow(db), function(x){
            # x = 1
            yes_drugs <- na.omit(as.numeric(db[x,colnames(db) %in% sel_omics]))
            no_drugs <- na.omit(as.numeric(db[x,!colnames(db) %in% sel_omics]))
            re2 <- tryCatch(
              wilcox.test(yes_drugs, no_drugs),
              error = function(x){NA}
            )
            if(all(is.na(re2))){
              re3 <- data.frame(
                p = NA,
                effect = NA
              )
            } else {
              re3 <- data.frame(
                p = re2$p.value,
                effect = log2(mean(yes_drugs)/mean(no_drugs))
              )
            }
          })
          re <- do.call(rbind, re)
          re$fea <- rownames(db)
          re <- na.omit(re)
        }
        if(nrow(re) == 0){ next }
        re$sig <- "no"
        # add db name
        re$database <- paste0(profile_comb$Var1[index], "_", profile_comb$Var2[index])
        rownames(re) <- NULL
        re_list[[index]] <- re
        incProgress(1/nrow(profile_comb), detail = paste0("Doing part ", index, "(Total ", nrow(profile_comb), ")"))
        # Warning 
        shiny::validate(
          need(length(profile_comb) > 0, "You have not chosen yet, or there is no result for this feature-database pair.")
        )
      }
      waiter_hide() # hide the waiter
    }
    )
    re_list <- re_list[!sapply(re_list, is.null)] 
    re_list
  })
  re_list2 <- reactive({
    lapply(re_list(), function(re){
      re$sig[abs(re$effect) > input$dynamic_numeric & re$p < .05] <- "yes"
      re
    })
  })
  # Freq Table ----
  output$freq_table <- DT::renderDataTable({ 
    # add freq
    re_name_list1 <- lapply(re_list2(), function(x){
      x$fea
    })
    # Warning 
    shiny::validate(
      need(length(re_name_list1) > 0, "You have not chosen yet, or there is no result for this feature-database pair.")
    )
    # message(class(re_list()))
    test1 <- unlist(re_name_list1, use.names = F)
    test1 <- as.data.frame(table(test1))
    colnames(test1)[1] <- "Name"
    # sig freq
    re_name_list2 <- lapply(re_list2(), function(x){
      x$fea[x$sig %in% "yes"]
    })
    test2 <- unlist(re_name_list2, use.names = F)
    test2 <- as.data.frame(table(test2))
    colnames(test2)[1] <- "Name"
    test2_2 <- data.frame(
      Name = test1$Name[!test1$Name %in% test2$Name],
      Freq = 0
    )
    test2 <- rbind(test2, test2_2)
    test2 <- test2[match(test1$Name,test2$Name),]
    freq_df <- test2
    freq_df$Prop <- test2$Freq/test1$Freq
    freq_df <- freq_df[order(freq_df$Prop, freq_df$Freq, decreasing = T),]
    colnames(freq_df) <- c("Feature", "Counts", "Proportion")
    freq_df
  }, options = list(scrollX = TRUE), selection = 'single')
  # Result Table ----
  output$re_table <- DT::renderDataTable({
    re_df <- do.call(rbind, re_list2())
    shiny::validate(
      need(length(re_df) > 0, "You have not chosen yet, or there is no result for this feature-database pair.")
    )
    colnames(re_df) <- c("P value", "Effect size", "Feature",
                         "Significance", "Dataset")
    rownames(re_df) <- NULL
    return(re_df)
  })
  # Download ----
  # input_list <- reactive({
  #   list(
  #     select_features1 = input$select_features1,
  #     select_specific_feature = input$select_specific_feature,
  #     select_features2 = input$select_features2
  #   )
  # })
  output$download_sig_re <- downloadHandler(
    filename =  function(){
      paste0(input$select_features1, "_", 
             input$select_specific_feature, "-",
             input$select_features2, "-",
             "Sig_result",
             ".csv")
    },
    content = function(filename) {
      re_df <- do.call(rbind, re_list2())
      data.table::fwrite(re_df, 
                         file = filename)
    })
}

ui.modules_ga_group_comparison <- function(id) {
  ns <- NS(id)
  fluidPage(
    fluidRow(
      column(
        3,
        wellPanel(
          h4("Analysis Controls"),
          uiOutput(ns("ga_data1_id")),
          virtualSelectInput(
            inputId = ns("ga_data1_mid"), # molecule identifier
            label = "Molecule identifier",
            choices = NULL,
            search = TRUE,
            allowNewOption = TRUE,
            dropboxWidth = "200%"
          ),
          shinyBS::bsPopover(ns("ga_data1_mid"),
            title = "Note",
            content = "Ignore this option when you select a phenotype dataset",
            placement = "right", options = list(container = "body")
          ),
          uiOutput(ns("ga_data2_id")),
          virtualSelectInput(
            inputId = ns("ga_data2_mid"), # molecule identifier
            label = "Molecule identifier",
            choices = NULL,
            search = TRUE,
            allowNewOption = TRUE,
            dropboxWidth = "200%"
          ),
          shinyBS::bsPopover(ns("ga_data2_mid"),
            title = "Note",
            content = "Ignore this option when you select a phenotype dataset",
            placement = "right", options = list(container = "body")
          ),
          selectInput(ns("ga_test_type"), "Test Type",
            choices = c("parametric", "nonparametric", "robust", "bayes"),
            selected = "parametric", multiple = FALSE
          ),
          selectInput(ns("ga_test_adjust"), "Adjust Method",
            choices = c(
              "holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr",
              "none"
            ),
            selected = "holm", multiple = FALSE
          ),
          materialSwitch(
            inputId = ns("ga_use_pairwise"),
            label = "Display pairwise comparisons?",
            value = TRUE,
            status = "primary"
          ),
          actionBttn(
            inputId = ns("ga_preprocess_data"),
            label = "Preprocess",
            style = "gradient",
            icon = icon("check"),
            color = "default",
            block = TRUE,
            size = "sm"
          ),
          tags$br(),
          actionBttn(
            inputId = ns("ga_go"),
            label = "Submit",
            style = "gradient",
            icon = icon("check"),
            color = "default",
            block = TRUE,
            size = "sm"
          )
        )
      ),
      column(
        6,
        plotOutput(ns("ga_output")),
        DT::dataTableOutput(ns("ga_output_data"))
      ),
      column(
        3,
        wellPanel(
          h4("Sample Filters"),
          uiOutput(ns("ga_data_filter1_id")),
          actionBttn(
            inputId = ns("ga_filter_button"),
            label = "Click to filter!",
            color = "primary",
            style = "bordered",
            size = "sm"
          ),
          tags$br(),
          tags$br(),
          numericInput(inputId = ns("height"), label = "Height", value = 8),
          numericInput(inputId = ns("width"), label = "Width", value = 10),
          column(
            width = 12, align = "center",
            prettyRadioButtons(
              inputId = ns("device"),
              label = "Choose plot format",
              choices = c("png", "pdf"),
              selected = "png",
              inline = TRUE,
              icon = icon("check"),
              animation = "jelly",
              fill = TRUE
            )
          ),
          downloadBttn(
            outputId = ns("download"),
            style = "gradient",
            color = "default",
            block = TRUE,
            size = "sm"
          )
        )
      )
    )
  )
}


server.modules_ga_group_comparison <- function(
  input, output, session,
  selected_database_rm_phenotype, selected_database_add_url_and_phenotype,
  custom_file) {
  ns <- session$ns

  output$ga_data1_id <- renderUI({
    show_table <- selected_database_add_url_and_phenotype()
    selectInput(
      inputId = ns("ga_data1_id"),
      label = "Select dataset for groups:",
      choices = c("NONE", unique(show_table$XenaDatasets)),
      selected = "NONE",
      multiple = FALSE
    )
  })

  id1_choices <- eventReactive(input$ga_data1_id, {
    if (!identical(input$ga_data1_id, "NONE")) {
      if (input$ga_data1_id %in% phenotype_datasets || input$ga_data1_id == "custom_phenotype_dataset") {
        # A phenotype data is selected
        return(list(
          all = "NONE",
          selected = "NONE"
        ))
      } else {
        # !!Assume a dense matrix dataset is selected
        return(list(
          all = if (is.null(custom_file$fData)) all_preload_identifiers else
            unique(c(custom_file$fData[[1]], all_preload_identifiers)),
          selected = "TP53"
        ))
      }
    } else {
      return(list(
        all = "NONE",
        selected = "NONE"
      ))
    }
  })

  observe({
    updateVirtualSelect(
      "ga_data1_mid",
      choices = id1_choices()$all,
      selected = id1_choices()$selected
    )
  })

  output$ga_data2_id <- renderUI({
    show_table <- selected_database_add_url_and_phenotype()
    selectInput(
      inputId = ns("ga_data2_id"),
      label = "Select dataset for axis Y:",
      choices = c("NONE", unique(show_table$XenaDatasets)),
      selected = "NONE",
      multiple = FALSE
    )
  })

  id2_choices <- eventReactive(input$ga_data2_id, {
    if (!identical(input$ga_data2_id, "NONE")) {
      if (input$ga_data2_id %in% phenotype_datasets || input$ga_data2_id == "custom_phenotype_dataset") {
        # A phenotype data is selected
        return(list(
          all = "NONE",
          selected = "NONE"
        ))
      } else {
        # !!Assume a dense matrix dataset is selected
        return(list(
          all = if (is.null(custom_file$fData)) all_preload_identifiers else
            unique(c(custom_file$fData[[1]], all_preload_identifiers)),
          selected = "TP53"
        ))
      }
    } else {
      return(list(
        all = "NONE",
        selected = "NONE"
      ))
    }
  })

  observe({
    updateVirtualSelect(
      "ga_data2_mid",
      choices = id2_choices()$all,
      selected = id2_choices()$selected
    )
  })

  # Preprocessing
  group_col_status <- reactiveValues(status = "Off") # Off, CO (连续值), CA（离散值）

  observeEvent(input$ga_preprocess_data, {
    message("Preprocess button is clicked by user.")

    dataset1 <- setdiff(input$ga_data1_id, "NONE")
    dataset1_id <- setdiff(input$ga_data1_mid, "NONE")
    dataset1_phenotype <- if (dataset1 %in% phenotype_datasets) "YES" else "NO"

    if (dataset1_phenotype == "YES") {
      data1 <- XenaGenerate(subset = XenaDatasets == dataset1) %>%
        XenaQuery() %>%
        XenaDownload(destdir = XENA_DEST) %>%
        XenaPrepare() %>%
        as.data.frame()
    } else {
      if (length(dataset1_id) || dataset1 == "custom_phenotype_dataset") {
        data1 <- get_data_df(dataset1, dataset1_id)
      } else {
        sendSweetAlert(session,
          title = "Warning",
          text = "Please input a ID (e.g., gene symbol) for axis X when you select a non-phenotype dataset"
        )
      }
    }
    print(head(data1))

    dataset2 <- setdiff(input$ga_data2_id, "NONE")
    dataset2_id <- setdiff(input$ga_data2_mid, "NONE")
    dataset2_phenotype <- if (dataset2 %in% phenotype_datasets) "YES" else "NO"
    if (dataset2_phenotype == "YES") {
      data2 <- XenaGenerate(subset = XenaDatasets == dataset2) %>%
        XenaQuery() %>%
        XenaDownload(destdir = XENA_DEST) %>%
        XenaPrepare() %>%
        as.data.frame()
    } else {
      if (length(dataset2_id) || dataset2 == "custom_phenotype_dataset") {
        data2 <- get_data_df(dataset2, dataset2_id)
      } else {
        sendSweetAlert(session,
          title = "Warning",
          text = "Please input a ID (e.g., gene symbol) for axis Y when you select a non-phenotype dataset"
        )
      }
    }
    print(head(data2))

    if (length(dataset1) && length(dataset2)) {
      showModal(
        modalDialog(
          title = "Preprocessing",
          size = "l",
          fluidPage(
            wellPanel(
              h3("1. Process dataset 1 to determine the groups indicated in axis X"),
              DT::dataTableOutput(ns("data1_table")),
              fluidRow(
                column(4, selectInput(ns("data1_sample_col"), "Sample column", choices = colnames(data1), selected = colnames(data2)[1])),
                shinyBS::bsPopover(ns("data1_sample_col"),
                  title = "Note",
                  content = "The sample IDs in this column must match with the second table",
                  placement = "top", options = list(container = "body")
                ),
                column(4, selectInput(ns("data1_group_col"), "Group column", choices = c("NONE", colnames(data1)), selected = "NONE")),
                column(4, selectInput(ns("data1_facet_col"), "Facet column (optional)", choices = c("NONE", colnames(data1)), selected = "NONE")),
                shinyBS::bsPopover(ns("data1_facet_col"),
                  title = "Note",
                  content = "Two tables can only select 1 column for generating multiple sub-plots",
                  placement = "top", options = list(container = "body")
                )
              ),
              shinyjs::hidden(
                sliderInput(
                  inputId = ns("group_col_cutpoint"), label = "Select (min and max) percent cutoff (%) to generate (Low and High) groups:",
                  min = 10, max = 90, value = c(50, 50)
                )
              ),
              shinyjs::hidden(
                uiOutput(ns("group_col_groups"))
              )
            ),
            wellPanel(
              h3("2. Process dataset 2 to determine the values indicated in axis Y"),
              DT::dataTableOutput(ns("data2_table")),
              fluidRow(
                column(4, selectInput(ns("data2_sample_col"), "Sample column", choices = colnames(data2), selected = colnames(data2)[1])),
                shinyBS::bsPopover(ns("data2_sample_col"),
                  title = "Note",
                  content = "The sample IDs in this column must match with the first table",
                  placement = "top", options = list(container = "body")
                ),
                column(4, selectInput(ns("data2_value_col"), "Value column", choices = c("NONE", colnames(data2)), selected = colnames(data2)[2])),
                column(4, selectInput(ns("data2_facet_col"), "Facet column (optional)", choices = c("NONE", colnames(data2)), selected = "NONE")),
                shinyBS::bsPopover(ns("data2_facet_col"),
                  title = "Note",
                  content = "Two tables can only select 1 column for generating multiple sub-plots",
                  placement = "top", options = list(container = "body")
                )
              )
            ),
            wellPanel(
              h3("3. Join selected columns in the two tables"),
              actionBttn(
                inputId = ns("join_table_button"),
                label = "Click to join the data",
                color = "primary",
                style = "bordered", size = "sm",
                block = F
              ),
              h4("If the table below is right, dismiss this page and click submit button to plot"),
              markdown("- The first column refers to sample ID.
                       - The second column refers to values indicated in axis Y.
                       - The third column refers to groups indicated in axis X.
                       - The fourth column is optional, which indicates facet variable."),
              DT::dataTableOutput(ns("joined_table"))
            )
          )
        )
      )

      observe({
        group_col <- setdiff(input$data1_group_col, "NONE")
        if (length(group_col) && group_col %in% colnames(data1)) {
          # A column has been selected by user
          data <- na.omit(data1[[group_col]])
          if (length(data)) {
            if (is.numeric(data) && length(unique(data)) > 5) {
              shinyjs::hide(id = "group_col_groups")
              shinyjs::show(id = "group_col_cutpoint")
              group_col_status$status <- "CO"
            } else {
              choices <- unique(data)
              group_col_status$status <- "CA"
              output$group_col_groups <- renderUI({
                prettyCheckboxGroup(
                  inputId = ns("group_col_groups"),
                  label = "Select groups shown in X axis:",
                  choices = choices,
                  selected = choices,
                  animation = "jelly",
                  status = "info"
                )
              })
              shinyjs::hide(id = "group_col_cutpoint")
              shinyjs::show(id = "group_col_groups")
            }
            message("Observe:")
            print(group_col_status$status)
          } else {
            sendSweetAlert(session,
              title = "Warning",
              text = "The group column you selected seems have no valid data!",
              type = "warning"
            )
          }
        } else {
          shinyjs::hide(id = "group_col_cutpoint")
          shinyjs::hide(id = "group_col_groups")
          group_col_status$status <- "Off"
        }
      })

      output$data1_table <- DT::renderDataTable(server = TRUE, {
        DT::datatable(
          data1,
          rownames = FALSE,
          options = list(
            scrollY = 350,
            scrollX = 300,
            initComplete = JS(
              "function(settings, json) {",
              "$(this.api().table().header()).css({'font-size': '90%'});",
              "$(this.api().table().body()).css({'font-size': '85%'});",
              "}"
            )
          )
        )
      })

      output$data2_table <- DT::renderDataTable(server = TRUE, {
        DT::datatable(
          data2,
          rownames = FALSE,
          options = list(
            scrollY = 350,
            scrollX = 300
          )
        )
      })

      observeEvent(input$join_table_button, {
        data1_sample_col <- input$data1_sample_col
        data2_sample_col <- input$data2_sample_col
        data1_group_col <- setdiff(input$data1_group_col, "NONE")
        data2_value_col <- setdiff(input$data2_value_col, "NONE")
        data1_facet_col <- setdiff(input$data1_facet_col, "NONE")
        data2_facet_col <- setdiff(input$data2_facet_col, "NONE")

        if (length(data1_facet_col) && length(data2_facet_col)) {
          sendSweetAlert(session,
            title = "Warning",
            text = "Two tables can only select 1 column for generating multiple sub-plots!", type = "warning"
          )
        } else if (length(data1_group_col) > 0 && length(data2_value_col) > 0) {
          data1_cols <- c(data1_sample_col, data1_group_col, data1_facet_col)
          data2_cols <- c(data2_sample_col, data2_value_col, data2_facet_col)
          col_order <- c("sample", data2_value_col, data1_group_col, data1_facet_col, data2_facet_col)
          message("column order:")
          print(col_order)

          if (all(data1_cols %in% colnames(data1)) && all(data2_cols %in% colnames(data2))) {
            data1_copy <- dplyr::select(data1, dplyr::all_of(data1_cols))
            colnames(data1_copy)[1] <- "sample"
            data2_copy <- dplyr::select(data2, dplyr::all_of(data2_cols))
            colnames(data2_copy)[1] <- "sample"

            message("Joining:")
            joined_data <- tryCatch(
              {
                dplyr::inner_join(
                  data1_copy, data2_copy,
                  by = "sample"
                ) %>%
                  dplyr::select(dplyr::all_of(col_order)) %>%
                  as.data.frame()
              },
              error = function(e) {
                message("Joining failed for group comparison analysis.")
                NULL
              }
            )

            print(group_col_status$status)
            if (group_col_status$status != "Off") {
              if (group_col_status$status == "CO") {
                # 作为连续值处理，读取分割点
                data <- joined_data
                data$.group <- joined_data[[3]]
                data <- data %>%
                  dplyr::arrange(.data$.group) %>%
                  dplyr::mutate(per_rank = 100 / nrow(.) * (1:nrow(.))) %>%
                  dplyr::mutate(.group = dplyr::case_when(
                    .data$per_rank > !!input$group_col_cutpoint[2] ~ "High",
                    .data$per_rank <= !!input$group_col_cutpoint[1] ~ "Low",
                    TRUE ~ NA_character_
                  ))
                joined_data[[3]] <- data$.group
                joined_data <- na.omit(joined_data)
              } else {
                # 作为离散值处理
                message(length(input$group_col_groups), " groups remained.")
                joined_data <- joined_data[joined_data[[3]] %in% input$group_col_groups, ]
                print(nrow(joined_data))
              }
            }

            grp_df$data <- joined_data

            if (!is.null(joined_data)) {
              output$joined_table <- DT::renderDataTable(server = TRUE, {
                DT::datatable(
                  joined_data,
                  rownames = FALSE,
                  options = list(
                    scrollY = 350,
                    scrollX = 300
                  )
                )
              })
            } else {
              sendSweetAlert(session,
                title = "Warning",
                text = "Joining failed! Please check your selection, especially the sample ID column.", type = "warning"
              )
            }
          }
        } else {
          print(str(data1_group_col))
          print(str(data2_value_col))
          sendSweetAlert(session,
            title = "Warning",
            text = "Some options have not been selected!", type = "warning"
          )
        }
      })
    } else {
      sendSweetAlert(session,
        title = "Warning",
        text = "Two datasets must be selected.",
        type = "warning"
      )
    }
  })

  selected_samps <- reactiveValues(id = NULL)
  grp_df <- reactiveValues(data = NULL)

  p_grp_comp <- eventReactive(input$ga_go, {
    if (is.null(selected_samps$id)) {
      message("All samples selected for analysis.")
    } else {
      message(length(selected_samps$id), " samples selected for analysis.")
    }
    tryCatch(
      vis_identifier_grp_comparison(
        grp_df = grp_df$data,
        type = isolate(input$ga_test_type),
        pairwise.comparisons = isolate(input$ga_use_pairwise),
        p.adjust.method = isolate(input$ga_test_adjust),
        samples = isolate(selected_samps$id)
      ),
      error = function(e) {
        message("General analysis group comparison plot error:")
        print(e$message)
        "Error"
      }
    )
  })

  observeEvent(input$ga_go, {
    # Analyze correlation with 2 input datasets and identifiers
    output$ga_output <- renderPlot(
      if (inherits(p_grp_comp(), c("ggplot", "grob"))) {
        print(p_grp_comp())
      } else {
        sendSweetAlert(
          session,
          title = "Error",
          text = "Error to plot. Please make sure you have go through the 'preprocess' step.",
          type = "error"
        )
      }
    )
    output$download <- downloadHandler(
      filename = function() {
        paste0(Sys.Date(), "_group_comparison_plot.", input$device)
      },
      content = function(file) {
        ggplot2::ggsave(
          filename = file, plot = p_grp_comp(), newpage = F, device = input$device,
          units = "cm", width = input$width, height = input$height, dpi = 600
        )
      }
    )

    output$ga_output_data <- DT::renderDataTable(server = FALSE, {
      if (inherits(p_grp_comp(), "ggplot")) {
        DT::datatable(
          p_grp_comp()$data,
          rownames = FALSE,
          extensions = c("Buttons"),
          options = list(
            pageLength = 5,
            dom = "Bfrtip",
            buttons = list(
              list(
                extend = "csv", text = "Download Current Page", filename = "page",
                exportOptions = list(
                  modifier = list(page = "current")
                )
              ),
              list(
                extend = "csv", text = "Download Full Results", filename = "data",
                exportOptions = list(
                  modifier = list(page = "all")
                )
              )
            )
          )
        )
      }
    })
  })

  output$ga_data_filter1_id <- renderUI({
    show_table <- selected_database_add_url_and_phenotype()
    selectInput(
      inputId = ns("ga_data_filter1_id"),
      label = "Select phenotype dataset:",
      choices = c("NONE", unique(show_table$XenaDatasets[show_table$XenaDatasets %in% c(phenotype_datasets, "custom_phenotype_dataset")])),
      selected = "NONE",
      multiple = FALSE
    )
  })

  observeEvent(input$ga_data_filter1_id, {
    # If phenotype dataset has been reset, we use all samples for plotting
    if (length(input$ga_data_filter1_id)) {
      if (input$ga_data_filter1_id == "NONE") {
        selected_samps$id <- NULL
      }
    }
  })

  observeEvent(input$ga_filter_button, {
    message("Sample filter button is clicked by user.")
    pdataset <- setdiff(input$ga_data_filter1_id, "NONE")

    if (length(pdataset)) {
      showModal(
        modalDialog(
          title = "Filter samples for analysis",
          size = "l",
          fluidPage(
            wellPanel(
              h4("1. Select available columns on the left to right"),
              uiOutput(ns("ga_col_chooser")),
              tags$br(),
              actionBttn(
                inputId = ns("show_or_update_ptable"),
                label = "Show/Update Phenotype Table",
                color = "primary",
                style = "bordered",
                size = "sm",
                block = F
              )
            ),
            wellPanel(
              h4("2. Filter rows by SearchPanels"),
              DT::dataTableOutput(ns("ga_phenotype_data"))
            ),
            wellPanel(
              h4("3. Specify sample column and hit button"),
              uiOutput(ns("ga_select_samp_col")),
              actionBttn(
                inputId = ns("ga_filter_submit_button"),
                label = "Submit to filter",
                color = "primary",
                style = "bordered", size = "sm",
                block = F
              )
            ),
            wellPanel(
              h4("4. After hitting button, dismiss this page and click submit button")
            )
          )
        )
      )

      phenotype_table <- XenaGenerate(subset = XenaDatasets == pdataset) %>%
        XenaQuery() %>%
        XenaDownload(destdir = XENA_DEST) %>%
        XenaPrepare()

      output$ga_col_chooser <- renderUI({
        all_cols <- colnames(phenotype_table)
        sel_idx <- seq_len(min(length(all_cols), 5))
        chooserInput(ns("ga_col_chooser"), "Available columns", "Selected columns",
          if (length(all_cols) == length(sel_idx)) c() else all_cols[-sel_idx],
          all_cols[sel_idx],
          size = 5, multiple = TRUE
        )
      })

      observeEvent(input$show_or_update_ptable, {
        selected_cols <- isolate(input$ga_col_chooser$right)
        if (length(selected_cols)) {
          message("Following columns selected by users from sample filter window.")
          print(selected_cols)

          output$ga_phenotype_data <- DT::renderDataTable(server = FALSE, {
            DT::datatable(
              phenotype_table %>%
                dplyr::select(all_of(selected_cols)),
              rownames = FALSE,
              extensions = c("Buttons", "Select", "SearchPanes"), # "Scroller" causes bug in searchPanel
              options = list(
                dom = "Bfrtip", # P
                buttons = list(
                  "searchPanes",
                  list(
                    extend = "csv", text = "Download Current Page", filename = "page",
                    exportOptions = list(
                      modifier = list(page = "current")
                    )
                  ),
                  list(
                    extend = "csv", text = "Download Full Results", filename = "data",
                    exportOptions = list(
                      modifier = list(page = "all")
                    )
                  )
                ),
                scrollY = 350,
                scrollX = 300,
                deferRender = TRUE,
                # scroller = TRUE,
                stateSave = FALSE
              ),
              selection = "none"
            )
          })
        } else {
          sendSweetAlert(session, title = "Warning", type = "warn", text = "Please select at least 1 column!")
        }
      })

      output$ga_select_samp_col <- renderUI({
        selectInput(
          inputId = ns("ga_select_samp_col"),
          label = "Select sample column:",
          choices = c("NONE", input$ga_col_chooser$right),
          selected = "NONE",
          multiple = FALSE
        )
      })
      observeEvent(input$ga_filter_submit_button, {
        rows_filtered <- input$ga_phenotype_data_rows_all
        col_sample <- input$ga_select_samp_col
        keep_samples <- phenotype_table[rows_filtered, , drop = FALSE][[col_sample]]
        message(length(keep_samples), " samples left after filtering.") # 为什么该代码块会被多次调用呢？
        if (length(keep_samples) < 1) {
          sendSweetAlert(session, title = "Error", text = "No samples left!", type = "error")
        } else {
          selected_samps$id <- isolate(keep_samples)
        }
      })
    } else {
      sendSweetAlert(session, title = "Warning", type = "warn", text = "Please select a dataset!")
    }
  })
}

ui.modules_id_reference = function(id) {
	ns = NS(id)
	fluidPage(
		navlistPanel(
			"Omics molecule identifiers",
			tabPanel(
				"1. TCGA Molecular profile",
				tabsetPanel(
					tabPanel("Gene", dataTableOutput(ns("dt_1_1_gene"))),
					tabPanel("Protein", dataTableOutput(ns("dt_1_2_pro"))),
					tabPanel("Mutation", dataTableOutput(ns("dt_1_3_mut"))),
					tabPanel("CNV", dataTableOutput(ns("dt_1_4_cn"))),
					tabPanel("Transcript", dataTableOutput(ns("dt_1_5_trans"))),
					tabPanel("Methylation(450K)", dataTableOutput(ns("dt_1_6_450"))),
					tabPanel("Methylation(27K)", dataTableOutput(ns("dt_1_7_27k"))),
					tabPanel("miRNA", dataTableOutput(ns("dt_1_8_mi")))
				)  %>%
					helper(type = "inline", size = "m", fade = TRUE, title="Tip:" , 
						content = "The Chromosomal coordinates is based on GENCODE V23(hg38)"),
			),
			tabPanel(
				"2. PCAWG Molecular profile",
				tabsetPanel(
					tabPanel("Gene", dataTableOutput(ns("dt_5_1_gene"))),
					tabPanel("Promoter", dataTableOutput(ns("dt_5_2_pro"))),
					tabPanel("Fusion", dataTableOutput(ns("dt_5_3_fu"))),
					tabPanel("miRNA", dataTableOutput(ns("dt_5_4_mi"))),
					tabPanel("APOBEC", dataTableOutput(ns("dt_5_5_apo")))
				)  %>%
					helper(type = "inline", size = "m", fade = TRUE, title="Tip:" , 
						content = "The Chromosomal coordinates is based on GENCODE 19(hg19)"),
			),
			tabPanel(
				"3. CCLE Molecular profile",
				tabsetPanel(
					tabPanel("Gene", dataTableOutput(ns("dt_6_1_gene"))),
					tabPanel("Protein", dataTableOutput(ns("dt_6_2_pro"))),
					tabPanel("CNV", dataTableOutput(ns("dt_6_3_cn"))),
					tabPanel("Mutation", dataTableOutput(ns("dt_6_4_mu")))
				)  %>%
					helper(type = "inline", size = "m", fade = TRUE, title="Tip:" , 
						content = "The Chromosomal coordinates is based on GENCODE 19(hg19)"),
			),
			"Other non-omics identifiers",
			tabPanel(
				"4. Tumor index",
				tabsetPanel(
					tabPanel("Purity", dataTableOutput(ns("dt_2_1_purity"))),
					tabPanel("Stemness", dataTableOutput(ns("dt_2_2_stemness"))),
					tabPanel("TMB", dataTableOutput(ns("dt_2_3_tmb"))),
					tabPanel("MSI", dataTableOutput(ns("dt_2_4_msi"))),
					tabPanel("Genome instability", dataTableOutput(ns("dt_2_5_instability"))),
				)
			),
			tabPanel(
				"5. Immune Infiltration",
				tabsetPanel(
					tabPanel("CIBERSORT", dataTableOutput(ns("dt_3_1_ciber"))),
					tabPanel("CIBERSORT-ABS", dataTableOutput(ns("dt_3_2_ciberABS"))),
					tabPanel("EPIC", dataTableOutput(ns("dt_3_3_epic"))),
					tabPanel("MCPCOUNTER", dataTableOutput(ns("dt_3_4_mcp"))),
					tabPanel("QUANTISEQ", dataTableOutput(ns("dt_3_5_quan"))),
					tabPanel("TIMER", dataTableOutput(ns("dt_3_6_timer"))),
					tabPanel("XCELL", dataTableOutput(ns("dt_3_7_xcell"))),
				)
			),
			tabPanel(
				"6. Pathway activity",
				tabsetPanel(
					tabPanel("HALLMARK", dataTableOutput(ns("dt_4_1_hm"))),
					tabPanel("KEGG", dataTableOutput(ns("dt_4_2_kegg"))),
					tabPanel("IOBR", dataTableOutput(ns("dt_4_3_iobr")))
				)
			),
			widths = c(3, 8)
		)
	)
}


server.modules_id_reference = function(input, output, session){
	ns = session$ns

	# load id data
	
	id_referrence = load_data("v2_tpc_id_help")
	tcga_id_referrence = id_referrence[["tcga"]]
	pcawg_id_referrence = id_referrence[["pcawg"]]
	ccle_id_referrence = id_referrence[["ccle"]]
	
	## Molecular profile(TCGA)
	output$dt_1_1_gene = renderDataTable({
		dt = datatable(tcga_id_referrence$id_gene,
			options=list(columnDefs = list(list(orderable=TRUE, targets=0)),
						search = list(regex = TRUE))) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_1_2_pro = renderDataTable({
		dt = datatable(tcga_id_referrence$id_pro,
			options=list(columnDefs = list(list(orderable=TRUE, targets=0)),
						search = list(regex = TRUE))) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_1_3_mut = renderDataTable({
		dt = datatable(tcga_id_referrence$id_mut,
			options=list(columnDefs = list(list(orderable=TRUE, targets=0)),
						search = list(regex = TRUE))) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_1_4_cn = renderDataTable({
		dt = datatable(tcga_id_referrence$id_cn,
			options=list(columnDefs = list(list(orderable=TRUE, targets=0)),
						search = list(regex = TRUE)))  
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_1_5_trans = renderDataTable({
		dt = datatable(tcga_id_referrence$id_trans,
			options=list(columnDefs = list(list(orderable=TRUE, targets=0)),
						search = list(regex = TRUE))) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_1_6_450 = renderDataTable({
		dt = datatable(tcga_id_referrence$id_M450,
			options=list(columnDefs = list(list(orderable=TRUE, targets=0)),
						search = list(regex = TRUE))) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_1_7_27k = renderDataTable({
		dt = datatable(tcga_id_referrence$id_M27K,
			options=list(columnDefs = list(list(orderable=TRUE, targets=0)),
						search = list(regex = TRUE))) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_1_8_mi = renderDataTable({
		dt = datatable(tcga_id_referrence$id_mi,
			options=list(columnDefs = list(list(orderable=TRUE, targets=0)),
						search = list(regex = TRUE))) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})

	## Molecular profile(PCAWG)
	output$dt_5_1_gene = renderDataTable({
		dt = datatable(pcawg_id_referrence$id_gene,
			options=list(columnDefs = list(list(orderable=TRUE, targets=0)),
						search = list(regex = TRUE))) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_5_2_pro = renderDataTable({
		dt = datatable(pcawg_id_referrence$id_pro,
			options=list(columnDefs = list(list(orderable=TRUE, targets=0)),
						search = list(regex = TRUE))) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_5_3_fu = renderDataTable({
		dt = datatable(pcawg_id_referrence$id_fusion,
			options=list(columnDefs = list(list(orderable=TRUE, targets=0)),
						search = list(regex = TRUE))) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_5_4_mi = renderDataTable({
		dt = datatable(pcawg_id_referrence$id_mi,
			options=list(columnDefs = list(list(orderable=TRUE, targets=0)),
						search = list(regex = TRUE))) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_5_5_apo = renderDataTable({
		dt = datatable(pcawg_id_referrence$id_maf,
			options=list(columnDefs = list(list(orderable=TRUE, targets=0)),
						search = list(regex = TRUE))) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	## Molecular profile(CCLE)
	output$dt_6_1_gene = renderDataTable({
		dt = datatable(ccle_id_referrence$id_gene,
			options=list(columnDefs = list(list(orderable=TRUE, targets=0)),
						search = list(regex = TRUE))) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_6_2_pro = renderDataTable({
		dt = datatable(ccle_id_referrence$id_pro,
			options=list(columnDefs = list(list(orderable=TRUE, targets=0)),
						search = list(regex = TRUE))) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_6_3_cn = renderDataTable({
		dt = datatable(ccle_id_referrence$id_cnv,
			options=list(columnDefs = list(list(orderable=TRUE, targets=0)),
						search = list(regex = TRUE))) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_6_4_mu = renderDataTable({
		dt = datatable(ccle_id_referrence$id_mut,
			options=list(columnDefs = list(list(orderable=TRUE, targets=0)),
						search = list(regex = TRUE))) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	
	## Tumor index
	output$dt_2_1_purity = renderDataTable({
		dt = datatable(
				data.frame(
					Level2 = "Tumor Purity",
					Level3 = tcga_id.list[["Purity"]]
				), options=list(columnDefs = list(list(orderable=TRUE, targets=0)))
			) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_2_2_stemness = renderDataTable({
		dt = datatable(
				data.frame(
					Level2 = "Tumor Stemness",
					Level3 = tcga_id.list[["Stem"]]
				), options=list(columnDefs = list(list(orderable=TRUE, targets=0)))
			) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_2_3_tmb = renderDataTable({
		dt = datatable(
				data.frame(
					Level2 = "Tumor Mutation Burden",
					Level3 = tcga_id.list[["TMB"]]
				), options=list(columnDefs = list(list(orderable=TRUE, targets=0)))
			) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_2_4_msi = renderDataTable({
		dt = datatable(
				data.frame(
					Level2 = "Microsatellite Instability",
					Level3 = tcga_id.list[["MSI"]]
				), options=list(columnDefs = list(list(orderable=TRUE, targets=0)))
			) 
	})
	output$dt_2_5_instability = renderDataTable({
		dt = datatable(
				data.frame(
					Level2 = "Genome Instability",
					Level3 = tcga_id.list[["GI"]]
				), options=list(columnDefs = list(list(orderable=TRUE, targets=0)))
			)  
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})

	## Immune infiltration
	output$dt_3_1_ciber = renderDataTable({
		dt = datatable(
				data.frame(
					Level2 = "CIBERSORT",
					Level3 = tcga_id.list[["CIB"]]
				), options=list(columnDefs = list(list(orderable=TRUE, targets=0)))
			) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_3_2_ciberABS = renderDataTable({
		dt = datatable(
				data.frame(
					Level2 = "CIBERSORT-ABS",
					Level3 = tcga_id.list[["CIB.ABS"]]
				), options=list(columnDefs = list(list(orderable=TRUE, targets=0)))
			) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_3_3_epic = renderDataTable({
		dt = datatable(
				data.frame(
					Level2 = "EPIC",
					Level3 = tcga_id.list[["EPIC"]]
				), options=list(columnDefs = list(list(orderable=TRUE, targets=0)))
			) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_3_4_mcp = renderDataTable({
		dt = datatable(
				data.frame(
					Level2 = "MCPCOUNTER",
					Level3 = tcga_id.list[["MCP"]]
				), options=list(columnDefs = list(list(orderable=TRUE, targets=0)))
			) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_3_5_quan = renderDataTable({
		dt = datatable(
				data.frame(
					Level2 = "QUANTISEQ",
					Level3 = tcga_id.list[["Quant"]]
				), options=list(columnDefs = list(list(orderable=TRUE, targets=0)))
			) 
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_3_6_timer = renderDataTable({
		dt = datatable(
				data.frame(
					Level2 = "TIMER",
					Level3 = tcga_id.list[["TIMER"]]
				), options=list(columnDefs = list(list(orderable=TRUE, targets=0)))
			)
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_3_7_xcell = renderDataTable({
		dt = datatable(
				data.frame(
					Level2 = "XCELL",
					Level3 = tcga_id.list[["XCELL"]]
				), options=list(columnDefs = list(list(orderable=TRUE, targets=0)))
			)
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})

	## Pathway activity
	output$dt_4_1_hm = renderDataTable({
		dt = datatable(
				data.frame(
					Level2 = "HALLMARK",
					Level3 = tcga_id.list[["HM"]]
				), options=list(columnDefs = list(list(orderable=TRUE, targets=0)))
			)
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_4_2_kegg = renderDataTable({
		dt = datatable(
				data.frame(
					Level2 = "KEGG",
					Level3 = tcga_id.list[["KEGG"]]
				), options=list(columnDefs = list(list(orderable=TRUE, targets=0)))
			)
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
	output$dt_4_3_iobr = renderDataTable({
		dt = datatable(
				data.frame(
					Level2 = "IOBR",
					Level3 = tcga_id.list[["IOBR"]]
				), options=list(columnDefs = list(list(orderable=TRUE, targets=0)))
			)
		dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
		dt
	})
}

mol_origin_UI = function(id, database = "toil"){
  ns = NS(id)
  if (!is.null(database)) {
    if (database == "toil") {
      choices = c("mRNA Expression","Transcript Expression","DNA Methylation",
      						"Protein Expression","miRNA Expression","Mutation status","Copy Number Variation")
    } else if (database == "pcawg") {
      choices = c("mRNA Expression","Gene Fusion", "Promoter Activity",
      						"miRNA Expression","APOBEC Mutagenesis")
    } else if (database == "ccle") {
      choices = c("mRNA Expression","Protein Expression",
      						"Copy Number Variation","Mutation status")
    }
  }
  selected = choices[1]

	wellPanel(
		style = "background: #f7f7f7",
		div(
			selectInput(
				ns("data_origin"),NULL,
				choices = choices,
				selected = selected
			), style = "margin-top: 0px;"
		),
		tabsetPanel(id = ns("data_origin_type"),
			type = "hidden",
			tabPanel("mRNA Expression",
				if(database == "toil"){
			    shinyWidgets::prettyRadioButtons(
			        inputId = ns("toil_gene_norm"), label = "(1)Normalization method",
			        choiceValues = c("tpm", "fpkm", "nc"),
			        choiceNames = c("RSEM TPM", "RSEM FPKM", "RSEM Norm_Count"),
			        selected = "tpm")
		    } else if(database == "ccle"){
			    shinyWidgets::prettyRadioButtons(
			        inputId = ns("ccle_gene_norm"), label = "(1)Type",
			        choiceValues = c("rpkm", "nc"),
			        choiceNames = c("RPKM", "log2(count+1)"),
			        selected = "rpkm"
			    )
		    } else if(database == "pcawg"){
			    p(strong(span("No alternative datasets can be chosen.", style="color:grey")))
		    }
			),
			tabPanel("Transcript Expression",
				if(database == "toil"){
			    shinyWidgets::prettyRadioButtons(
			        inputId = ns("toil_trans_norm"), label = "(1)Normalization method",
			        choiceValues = c("tpm", "fpkm", "isopct"),
			        choiceNames = c("RSEM TPM", "RSEM FPKM", "RSEM Isoform Percentage"),
			        selected = "tpm"
			    )
				}
			),
			tabPanel("DNA Methylation",
				if(database == "toil"){
					tagList(
						fluidRow(
							column(
								6,
								selectInput(ns("toil_L2_3_methy_1"),"(1)Platform",
									choices = c("450K","27K"), selected = TRUE)
							),
							column(
								6,
								virtualSelectInput(ns("toil_L2_3_methy_3_gene"),"(2)Gene",
									choices = NULL, 
									search = TRUE,
									allowNewOption = TRUE,
									dropboxWidth = "200%"),
							)
						),
						fluidRow(
							column(6,numericInput(ns("chr_min"),"Min coord",value = NULL)),
							column(6,numericInput(ns("chr_max"),"Max coord",value = NULL))
						),
						fluidRow(
							column(
								8,
				        virtualSelectInput(
				          inputId = ns("toil_L2_3_methy_3_cpg"),
				          label = "(4)CpG sites",
				          choices = NULL,
				          multiple = TRUE,
				          allowNewOption = TRUE)
							),
							column(
								4,
								selectInput(ns("toil_L2_3_methy_2"),"(5)Aggregation",
									choices = c("NA", "mean", "Q0", "Q25", "Q50", "Q75", "Q100"), 
									selected = "mean")
							)
						),
						verbatimTextOutput(ns("cpgs_sle_text"))
					)
				}
			),
			tabPanel("Copy Number Variation",
				if(database == "toil"){
					tagList(
						selectInput(ns("toil_L2_7_cnv_1"),"(1)GISTIC2 data",
							choices = c(TRUE, FALSE), selected = TRUE),
						selectInput(ns("toil_L2_7_cnv_2"),"(2)Thresholded data",
							choices = c(TRUE, FALSE), selected = FALSE)	
					)
				} else if (database == "ccle"){
					p(strong(span("No alternative datasets can be chosen.", style="color:grey")))
				}
			),
			tabPanel("miRNA Expression",
				if(database == "pcawg"){
			    shinyWidgets::prettyRadioButtons(
			        inputId = ns("pcawg_mi_norm"), label = "(1)Normalization method",
			        choiceValues = c("TMM", "UQ"),
			        choiceNames = c("TMM", "UQ"),
			        selected = "TMM"
			    )
				} else if (database=="toil"){
					p(strong(span("No alternative datasets can be chosen.", style="color:grey")))
				}
			),
			tabPanel("Promoter Activity",
				# only PCAWG
		    shinyWidgets::prettyRadioButtons(
		        inputId = ns("pcawg_pro_type"), label = "(1)Type",
		        choiceValues = c("raw", "relative", "outlier"),
		        choiceNames = c("Raw", "Relative", "Outlier"),
		        selected = "raw"
		    )
			),
			tabPanel("APOBEC Mutagenesis",
					p(strong(span("No alternative datasets can be chosen.", style="color:grey")))
			),
			tabPanel("Protein Expression",
					p(strong(span("No alternative datasets can be chosen.", style="color:grey")))
			),
			tabPanel("Mutation status",
					p(strong(span("No alternative datasets can be chosen.", style="color:grey")))
			)
		),
		# hr(style = "border-top: 1px solid #000000;"),
		div(style = "border-top: 1px dashed #000; margin-top: 5px; margin-bottom: 5px;"),
		# p(strong("Active Data Hub: "), a("toilHub", href = "http://baidu.com"), style = "margin-top: 0px; margin-bottom: 0px;"),
		# p(strong("Active Data Set: "), a("tcga_RSEM_gene_fpkm", href = "http://baidu.com"), style = "margin-top: 0px; margin-bottom: 0px;"),
		uiOutput(ns("hub_set.ui")),
		# p("See Repository for more information."),
		div(style = "border-top: 1px solid #000; margin-top: 5px; margin-bottom: 5px;"),

		# verbatimTextOutput(ns("tmp123"))
	)
}


mol_origin_Server = function(input, output, session, database = "toil"){
	ns <- session$ns

	observeEvent(input$data_origin, {
	  updateTabsetPanel(inputId = "data_origin_type", 
	  	selected = input$data_origin)
	}) 

  updateVirtualSelect(
    inputId = "toil_L2_3_methy_3_gene",
    choices = tcga_id.list[["Gene"]],
    selected = "TP53"
  )



  # 特定基因下所包含的所有CpG位点
	candi_cpg = reactive({
			# 仅当选中甲基化类型时，才加载该数据
			req(input$data_origin=="DNA Methylation")
			if(!exists("tcga_id_referrence")){
				message("Loading \"pancan_identifier_help\"")
				tcga_id_referrence = load_data("pancan_identifier_help")
			}
			if(is.null(input$toil_L2_3_methy_1)){
				toil_L2_3_methy_1 = "450K"
			} else {
				toil_L2_3_methy_1 = input$toil_L2_3_methy_1
			}
			candi_cpg = switch(toil_L2_3_methy_1,
				`450K` = tcga_id_referrence$id_molecule$id_M450,
				`27K` = tcga_id_referrence$id_molecule$id_M27K
			) %>%
				dplyr::filter(Level3 %in% input$toil_L2_3_methy_3_gene)
	})

	observe({
		cpg_ids = candi_cpg() %>% 
			dplyr::filter(chromStart >= input$chr_min) %>% 
			dplyr::filter(chromEnd  <= input$chr_max) %>% 
			dplyr::pull(CpG)
    updateVirtualSelect(
      "toil_L2_3_methy_3_cpg",
      choices = cpg_ids,
      selected = cpg_ids
      # selected = cpg_ids
    )
	})

	observe({
		if(nrow(candi_cpg())>0){
			updateNumericInput(
				session,
				"chr_min",
				label = paste("(3)Min coord ","(",unique(candi_cpg()$chrom),")"),
				value = min(candi_cpg()$chromStart)
			)
			updateNumericInput(
				session,
				"chr_max",
				label = paste("Max coord ","(",unique(candi_cpg()$chrom),")"),
				value = max(candi_cpg()$chromEnd)
			)
		}
	})

	cpg_ids_retain = reactive({
		if (is.data.frame(candi_cpg()) && nrow(candi_cpg()) > 0) {
		  shiny::validate(
		    need(try(input$chr_min>=min(candi_cpg()$chromStart)), 
		         paste0("Min coord cannot be less than ",min(candi_cpg()$chromStart),".")),
		    need(try(input$chr_max<=max(candi_cpg()$chromEnd)), 
		         paste0("Max coord cannot be greater than ",max(candi_cpg()$chromEnd),"."))
		  )
		  cpg_ids = candi_cpg() %>% 
		    dplyr::filter(chromStart >= input$chr_min) %>% 
		    dplyr::filter(chromEnd  <= input$chr_max) %>% 
		    dplyr::pull(CpG)
		  if(is.null(input$toil_L2_3_methy_3_cpg)){
		    cpg_ids_retain = cpg_ids
		  } else {
		    cpg_ids_retain = input$toil_L2_3_methy_3_cpg
		  }
		  cpg_ids_retain
		} else NULL
	})
	# 提示包含多少个CpG位点
	output$cpgs_sle_text = renderPrint({
		cat(paste0("Tip: ",length(cpg_ids_retain())," CpG sites are included."))
	})

	opt_pancan = reactive({
		opt_pancan = .opt_pancan
		if(database=="toil" & input$data_origin!="DNA Methylation"){
			opt_pancan$toil_mRNA = list(norm = input$toil_gene_norm)
			opt_pancan$toil_transcript = list(norm = input$toil_trans_norm)
			opt_pancan$toil_cnv = list(gistic2 = ifelse(is.null(input$toil_L2_7_cnv_1),TRUE,as.logical(input$toil_L2_7_cnv_1)),
											use_thresholded_data = ifelse(is.null(input$toil_L2_7_cnv_2),TRUE,as.logical(input$toil_L2_7_cnv_2)))
		} else if (database=="toil" & input$data_origin=="DNA Methylation"){
			opt_pancan$toil_methylation = list(type = ifelse(is.null(input$toil_L2_3_methy_1),"450K",input$toil_L2_3_methy_1), 
									aggr = ifelse(is.null(input$toil_L2_3_methy_2),"NA",input$toil_L2_3_methy_2),
									rule_out = setdiff(candi_cpg()$CpG, cpg_ids_retain()))
		} else if (database=="pcawg"){
		  opt_pancan$pcawg_miRNA = list(norm = input$pcawg_mi_norm)
		  opt_pancan$pcawg_promoter = list(type = input$pcawg_pro_type)
		} else if (database=="ccle"){
		  opt_pancan$ccle_mRNA = list(norm = input$ccle_gene_norm)
		}
		opt_pancan
	})


	output$hub_set.ui = renderUI({

		active_hub = "toilHub"
		active_set = "tcga_RSEM_gene_fpkm"
		link_hub = "https://toil.xenahubs.net"
		link_set = "https://xenabrowser.net/datapages/?dataset=tcga_RSEM_gene_fpkm&host=https://toil.xenahubs.net"

		if(database=="toil"){
			active_hub = "toilHub"
			link_hub = "https://toil.xenahubs.net"
			if(input$data_origin == "mRNA Expression"){
				active_set = switch(input$toil_gene_norm,
					"tpm" = "TcgaTargetGtex_rsem_gene_tpm",
					"fpkm" = "TcgaTargetGtex_rsem_gene_fpkm",
					"nc" = "TcgaTargetGtex_RSEM_Hugo_norm_count"		
				)
				link_set = switch(input$toil_gene_norm,
					"tpm" = "https://xenabrowser.net/datapages/?dataset=TcgaTargetGtex_rsem_gene_tpm&host=https://toil.xenahubs.net",
					"fpkm" = "https://xenabrowser.net/datapages/?dataset=TcgaTargetGtex_rsem_gene_fpkm&host=https://toil.xenahubs.net",
					"nc" = "https://xenabrowser.net/datapages/?dataset=TcgaTargetGtex_RSEM_Hugo_norm_count&host=https://toil.xenahubs.net"		
				)
			} else if(input$data_origin == "Transcript Expression"){
				active_set = switch(input$toil_trans_norm,
					"tpm" = "TcgaTargetGtex_rsem_isoform_tpm",
					"fpkm" = "TcgaTargetGtex_RSEM_isoform_fpkm",
					"isopct" = "TcgaTargetGtex_rsem_isopct"		
				)
				link_set = switch(input$toil_trans_norm,
					"tpm" = "https://xenabrowser.net/datapages/?dataset=TcgaTargetGtex_rsem_isoform_tpm&host=https://toil.xenahubs.net",
					"fpkm" = "https://xenabrowser.net/datapages/?dataset=TcgaTargetGtex_RSEM_isoform_fpkm&host=https://toil.xenahubs.net",
					"isopct" = "https://xenabrowser.net/datapages/?dataset=TcgaTargetGtex_rsem_isopct&host=https://toil.xenahubs.net"		
				)
			} else if(input$data_origin == "DNA Methylation"){
				# Not toil hub!
				active_hub = "gdcHub"
				link_hub = "https://gdc.xenahubs.net"
				active_set = switch(input$toil_L2_3_methy_1,
					"450K" = "GDC-PANCAN.methylation450.tsv",
					"27K" = "GDC-PANCAN.methylation27.tsv"	
				)
				link_set = switch(input$toil_L2_3_methy_1,
					"450K" = "https://xenabrowser.net/datapages/?dataset=GDC-PANCAN.methylation450.tsv&host=https://gdc.xenahubs.net",
					"27K" = "https://xenabrowser.net/datapages/?dataset=GDC-PANCAN.methylation27.tsv&host=https://gdc.xenahubs.net"
				)
			} else if(input$data_origin == "Protein Expression"){
				# Not toil hub!
				active_hub = "pancanAtlasHub"
				active_set = "TCGA-RPPA-pancan-clean.xena"
				link_hub = "https://pancanatlas.xenahubs.net"
				link_set = "https://xenabrowser.net/datapages/?dataset=TCGA-RPPA-pancan-clean.xena&host=https://pancanatlas.xenahubs.net"
			} else if(input$data_origin == "miRNA Expression"){
				# Not toil hub!
				active_hub = "pancanAtlasHub"
				active_set = "pancanMiRs_EBadjOnProtocolPlatformWithoutRepsWithUnCorrectMiRs_08_04_16.xena"
				link_hub = "https://pancanatlas.xenahubs.net"
				link_set = "https://xenabrowser.net/datapages/?dataset=pancanMiRs_EBadjOnProtocolPlatformWithoutRepsWithUnCorrectMiRs_08_04_16.xena&host=https://pancanatlas.xenahubs.net"
			} else if(input$data_origin == "Mutation status"){
				# Not toil hub!
				active_hub = "pancanAtlasHub"
				active_set = "mc3.v0.2.8.PUBLIC.nonsilentGene.xena"
				link_hub = "https://pancanatlas.xenahubs.net"
				link_set = "https://xenabrowser.net/datapages/?dataset=mc3.v0.2.8.PUBLIC.nonsilentGene.xena&host=https://pancanatlas.xenahubs.net"
			} else if(input$data_origin == "Copy Number Variation"){
				if(!as.logical(input$toil_L2_7_cnv_1)){
					# Not toil hub!
					active_hub = "pancanAtlasHub"
					active_set = "broad.mit.edu_PANCAN_Genome_Wide_SNP_6_whitelisted.gene.xena"
					link_hub = "https://pancanatlas.xenahubs.net"
					link_set = "https://xenabrowser.net/datapages/?dataset=broad.mit.edu_PANCAN_Genome_Wide_SNP_6_whitelisted.gene.xena&host=https://pancanatlas.xenahubs.net"
				} else if (as.logical(input$toil_L2_7_cnv_2)){
					active_hub = "tcgaHub"
					active_set = "TCGA.PANCAN.sampleMap/Gistic2_CopyNumber_Gistic2_all_thresholded.by_genes"
					link_hub = "https://tcga.xenahubs.net"
					link_set = "https://xenabrowser.net/datapages/?dataset=TCGA.PANCAN.sampleMap/Gistic2_CopyNumber_Gistic2_all_thresholded.by_genes&host=https://tcga.xenahubs.net"					
				} else if (!as.logical(input$toil_L2_7_cnv_2)){
					active_hub = "tcgaHub"
					active_set = "TCGA.PANCAN.sampleMap/Gistic2_CopyNumber_Gistic2_all_data_by_genes"
					link_hub = "https://tcga.xenahubs.net"
					link_set = "https://xenabrowser.net/datapages/?dataset=TCGA.PANCAN.sampleMap/Gistic2_CopyNumber_Gistic2_all_data_by_genes&host=https://tcga.xenahubs.net"					
				}
			} 
		} else if (database=="pcawg"){
			active_hub = "pcawgHub"
			link_hub = "https://pcawg.xenahubs.net"
			if(input$data_origin == "mRNA Expression"){
				active_set = "tophat_star_fpkm_uq.v2_aliquot_gl.sp.log"
				link_set = "https://xenabrowser.net/datapages/?dataset=tophat_star_fpkm_uq.v2_aliquot_gl.sp.log&host=https://pcawg.xenahubs.net"
			} else if(input$data_origin == "Gene Fusion"){
				active_set = "pcawg3_fusions_PKU_EBI.gene_centric.sp.xena"
				link_set = "https://xenabrowser.net/datapages/?dataset=pcawg3_fusions_PKU_EBI.gene_centric.sp.xena&host=https://pcawg.xenahubs.net"
			} else if(input$data_origin == "miRNA Expression"){
				active_set = switch(input$pcawg_mi_norm,
					"TMM" = "x3t2m1.mature.TMM.mirna.matrix.log",
					"UQ" = "x3t2m1.mature.UQ.mirna.matrix.log"
				)
				link_set = switch(input$pcawg_mi_norm,
					"TMM" = "https://xenabrowser.net/datapages/?dataset=x3t2m1.mature.UQ.mirna.matrix.log&host=https://pcawg.xenahubs.net",
					"UQ" = "https://xenabrowser.net/datapages/?dataset=promoterCentricTable_0.2_1.0.sp&host=https://pcawg.xenahubs.net"
				)
			} else if(input$data_origin == "Promoter Activity"){
				active_set = switch(input$pcawg_pro_type,
					"raw" = "rawPromoterActivity.sp",
					"relative" = "relativePromoterActivity.sp",
					"outlier" = "promoterCentricTable_0.2_1.0.sp"
				)
				link_set = switch(input$pcawg_pro_type,
					"raw" = "https://xenabrowser.net/datapages/?dataset=rawPromoterActivity.sp&host=https://pcawg.xenahubs.net",
					"relative" = "https://xenabrowser.net/datapages/?dataset=relativePromoterActivity.sp&host=https://pcawg.xenahubs.net",
					"outlier" = "https://xenabrowser.net/datapages/?dataset=promoterCentricTable_0.2_1.0.sp&host=https://pcawg.xenahubs.net"
				)
			} else if(input$data_origin == "APOBEC Mutagenesis"){
				active_set = "MAF_Aug31_2016_sorted_A3A_A3B_comparePlus.sp"
				link_set = "https://xenabrowser.net/datapages/?dataset=MAF_Aug31_2016_sorted_A3A_A3B_comparePlus.sp&host=https://pcawg.xenahubs.net"
			}  
		} else if (database=="ccle"){
			active_hub = "publicHub"
			link_hub = "https://ucscpublic.xenahubs.net"
			if(input$data_origin == "mRNA Expression"){
				active_set = switch(input$ccle_gene_norm,
					"rpkm" = "ccle/CCLE_DepMap_18Q2_RNAseq_RPKM_20180502",
					"nc" = "ccle/CCLE_DepMap_18Q2_RNAseq_reads_20180502.log2"		
				)
				link_set = switch(input$ccle_gene_norm,
					"rpkm" = "https://xenabrowser.net/datapages/?dataset=ccle/CCLE_DepMap_18Q2_RNAseq_RPKM_20180502&host=https://ucscpublic.xenahubs.net",
					"nc" = "https://xenabrowser.net/datapages/?dataset=ccle/CCLE_DepMap_18Q2_RNAseq_reads_20180502.log2&host=https://ucscpublic.xenahubs.net"		
				)
			} else if(input$data_origin == "Protein Expression"){
				active_set = "ccle/CCLE_RPPA_20180123"
				link_set = "https://xenabrowser.net/datapages/?dataset=ccle/CCLE_RPPA_20180123&host=https://ucscpublic.xenahubs.net"
			} else if(input$data_origin == "Mutation status"){
				active_set = "ccle/CCLE_DepMap_18Q2_maf_20180502"
				link_set = "https://xenabrowser.net/datapages/?dataset=ccle/CCLE_DepMap_18Q2_maf_20180502&host=https://ucscpublic.xenahubs.net"
			} else if(input$data_origin == "Copy Number Variation"){
				active_set = "ccle/CCLE_copynumber_byGene_2013-12-03"
				link_set = "https://xenabrowser.net/datapages/?dataset=ccle/CCLE_copynumber_byGene_2013-12-03&host=https://ucscpublic.xenahubs.net"
			}  
		}

		tagList(
			p(strong("Active Data Hub : "), a(active_hub, href = link_hub), 
					style = "margin-top: 0px; margin-bottom: 0px;"),
			p(strong("Active Data Set : "), a(active_set, href = link_set), 
					style = "margin-top: 0px; margin-bottom: 0px;"),
		)
	})



	# output$tmp123 = renderPrint({
	# 	opt_pancan()
	# })

	return(opt_pancan)

}

tcga_id = load_data("v2_tcga_id")
pcawg_id = load_data("v2_pcawg_id")
ccle_id = load_data("v2_ccle_id")

tcga_id.list = split(tcga_id$L3, tcga_id$L2)
tcga_id.list[["IOBR"]] = sort(tcga_id.list[["IOBR"]])

pcawg_id.list = split(pcawg_id$L3, pcawg_id$L2)
ccle_id.list = split(ccle_id$L3, ccle_id$L2)

names(tcga_id.list)




## General Analysis
all_preload_identifiers <- c("NONE", unlist(tcga_id.list[c("Gene","Protein","miRNA")]))

tcga_id_option = list(
  "Molecular profile" = list(
    "mRNA Expression" = list(
      "all" = tcga_id.list[["Gene"]],
      "default" = "TP53"
    ),
    "Transcript Expression" = list(
      "all" = tcga_id.list[["Transcript"]],
      "default" = "ENST00000000233"
    ),
    "DNA Methylation" = list(
      "all" = tcga_id.list[["Gene"]],
      "default" = "TP53"
    ),
    "Protein Expression" = list(
      "all" = tcga_id.list[["Protein"]],
      "default" = "P53"
    ),
    "miRNA Expression" = list(
      "all" = tcga_id.list[["miRNA"]],
      "default" = "hsa-miR-769-3p"
    ),
    "Mutation status" = list(
      "all" = tcga_id.list[["Gene"]],
      "default" = "TP53"
    ),
    "Copy Number Variation" = list(
      "all" = tcga_id.list[["Gene"]],
      "default" = "TP53"
    )
  ),
  "Tumor index" = list(
    "Tumor Purity" = list(
      "all" = tcga_id.list[["Purity"]],
      "default" = "ESTIMATE"
    ),
    "Tumor Stemness" = list(
      "all" = tcga_id.list[["Stem"]],
      "default" = "RNAss"
    ),
    "Tumor Mutation Burden" = list(
      "all" = tcga_id.list[["TMB"]],
      "default" = "Non_silent_per_Mb"
    ),
    "Microsatellite Instability" = list(
      "all" = tcga_id.list[["MSI"]],
      "default" = "Total_nb_MSI_events"
    ),
    "Genome Instability" = list(
      "all" = tcga_id.list[["GI"]],
      "default" = "ploidy"
    )
  ),
  "Immune Infiltration" = list(
    "CIBERSORT" = list(
      "all" = tcga_id.list[["CIB"]],
      "default" = "Monocyte"
    ),
    "CIBERSORT-ABS" = list(
      "all" = tcga_id.list[["CIB.ABS"]],
      "default" = "Monocyte"
    ),
    "EPIC" = list(
      "all" = tcga_id.list[["EPIC"]],
      "default" = "Macrophage"
    ),
    "MCPCOUNTER" = list(
      "all" = tcga_id.list[["MCP"]],
      "default" = "Macrophage"
    ),
    "QUANTISEQ" = list(
      "all" = tcga_id.list[["Quant"]],
      "default" = "Monocyte"
    ),
    "TIMER" = list(
      "all" = tcga_id.list[["TIMER"]],
      "default" = "Macrophage"
    ),
    "XCELL" = list(
      "all" = tcga_id.list[["XCELL"]],
      "default" = "Monocyte"
    )
  ),
  "Pathway activity" = list(
    "HALLMARK" = list(
      "all" = tcga_id.list[["HM"]],
      "default" = "APOPTOSIS"
    ),
    "KEGG" = list(
      "all" = tcga_id.list[["KEGG"]],
      "default" = "CELL_CYCLE"
    ),
    "IOBR" = list(
      "all" = tcga_id.list[["IOBR"]],
      "default" = "Biotin_Metabolism"
    )
  ),
  "Phenotype data" = list(
    "Clinical Phenotype" = list(
      "all" = tcga_id.list[["Clinical"]],
      "default" = "Code"
    ),
    "Custom metadata" = list(
      "all" = NULL,
      "default" = NULL
    )
  )
)

pcawg_id_option = list(
  "Molecular profile" = list(
    "mRNA Expression" = list(
      "all" = pcawg_id.list[["Gene"]],
      "default" = "TP53"
    ),
    "Promoter Activity" = list(
      "all" = pcawg_id.list[["Promoter"]],
      "default" = "prmtr.1"
    ),
    "Gene Fusion" = list(
      "all" = pcawg_id.list[["Fusion"]],
      "default" = "SAMD11"
    ),
    "miRNA Expression" = list(
      "all" = pcawg_id.list[["miRNA"]],
      "default" = "hsa-let-7a-2-3p"
    ),
    "APOBEC Mutagenesis" = list(
      "all" = pcawg_id.list[["Muta"]],
      "default" = "A3A_or_A3B"
    )
  ),
  "Tumor index" = list(
    "Tumor Purity" = list(
      "all" = pcawg_id.list[["Purity"]],
      "default" = "purity"
    )
  ),
  "Immune Infiltration" = list(
    "CIBERSORT" = list(
      "all" = pcawg_id.list[["CIB"]],
      "default" = "Monocyte"
    ),
    "CIBERSORT-ABS" = list(
      "all" = pcawg_id.list[["CIB.ABS"]],
      "default" = "Monocyte"
    ),
    "EPIC" = list(
      "all" = pcawg_id.list[["EPIC"]],
      "default" = "Macrophage"
    ),
    "MCPCOUNTER" = list(
      "all" = pcawg_id.list[["MCP"]],
      "default" = "Macrophage"
    ),
    "QUANTISEQ" = list(
      "all" = pcawg_id.list[["Quant"]],
      "default" = "Monocyte"
    ),
    "TIMER" = list(
      "all" = pcawg_id.list[["XCELL"]],
      "default" = "Monocyte"
    ),
    "XCELL" = list(
      "all" = pcawg_id.list[["TIMER"]],
      "default" = "Monocyte"
    )
  ),
  "Pathway activity" = list(
    "HALLMARK" = list(
      "all" = pcawg_id.list[["HM"]],
      "default" = "APOPTOSIS"
    ),
    "KEGG" = list(
      "all" = pcawg_id.list[["KEGG"]],
      "default" = "CELL_CYCLE"
    ),
    "IOBR" = list(
      "all" = pcawg_id.list[["IOBR"]],
      "default" = "Biotin_Metabolism"
    )
  ),
  "Phenotype data" = list(
    "Clinical Phenotype" = list(
      "all" = pcawg_id.list[["Clinical"]],
      "default" = "Age"
    ),
    "Custom metadata" = list(
      "all" = NULL,
      "default" = NULL
    )
  )
)

ccle_id_option = list(
  "Molecular profile" = list(
    "mRNA Expression" = list(
      "all" = ccle_id.list[["Gene"]],
      "default" = "TP53"
    ),
    "Protein Expression" = list(
      "all" = ccle_id.list[["Protein"]],
      "default" = "14-3-3_beta"
    ),
    "Copy Number Variation" = list(
      "all" = ccle_id.list[["Gene"]],
      "default" = "TP53"
    ),
    "Mutation status" = list(
      "all" = ccle_id.list[["Gene"]],
      "default" = "TP53"
    )
  ),
  "Tumor index" = list(
    "Tumor Purity" = list(
      "all" = ccle_id.list[["Purity"]],
      "default" = "Purity"
    )
  ),
  "Immune Infiltration" = NULL,
  "Pathway activity" = NULL,
  "Phenotype data" = list(
    "Clinical Phenotype" = list(
      "all" = ccle_id.list[["Clinical"]],
      "default" = "Gender"
    ),
    "Custom metadata" = list(
      "all" = NULL,
      "default" = NULL
    )
  )
)


## 33 TCGAs
tcga_names = sort(unique(tcga_clinical_fine$Cancer))
## 30 PCAWGs
pcawg_names = sort(unique(pcawg_info_fine$Project))

code_types = list("NT"= "NT (normal tissue)",
                  "TP"= "TP (primary tumor)",
                  "TR"= "TR (recurrent tumor)",
                  "TB"= "TB (blood derived tumor)",
                  "TAP"="TAP (additional primary)",
                  "TM"= "TM (metastatic tumor)",
                  "TAM"="TAM (additional metastatic)")

choices_primary_site <- c(
  "prostate", "stomach",
  "urinary_tract", "central_nervous_system",
  "ovary", "haematopoietic_and_lymphoid_tissue",
  "kidney", "thyroid",
  "skin", "soft_tissue",
  "salivary_gland", "lung",
  "bone", "pleura",
  "endometrium", "pancreas",
  "breast", "upper_aerodigestive_tract",
  "large_intestine", "autonomic_ganglia",
  "oesophagus", "liver",
  "biliary_tract", "small_intestine"
)


# Global theme
themes_list <- list(
  "BW" = ggplot2::theme_bw(),
  "Cowplot" = cowplot::theme_cowplot(),
  "Light" = ggplot2::theme_light(),
  "Minimal" = ggplot2::theme_minimal(),
  "Classic" = ggplot2::theme_classic(),
  "Gray" = ggplot2::theme_gray(),
  "Half_open" = cowplot::theme_half_open(),
  "Minimal_grid" = cowplot::theme_minimal_grid(),
  "ggstatplot" = ggstatsplot::theme_ggstatsplot()
)
# Global color
mycolor <- c(RColorBrewer::brewer.pal(12, "Paired"))


msigdbr_types <- data.frame(
  gs_cat = c("H","C1", "C2", "C2", "C2", "C2", "C2", "C2", "C2", 
             "C3", "C3", "C3", "C3", "C4", "C4", "C5", "C5", 
             "C5", "C5", "C6", "C7", "C7", "C8"),
  gs_subcat = c("","", "CGP", "CP", "CP:BIOCARTA", "CP:KEGG", 
                "CP:PID", "CP:REACTOME", "CP:WIKIPATHWAYS", 
                "MIR:MIRDB", "MIR:MIR_Legacy", "TFT:GTRD", 
                "TFT:TFT_Legacy", "CGN", "CM", "GO:BP", 
                "GO:CC", "GO:MF", "HPO", "", "IMMUNESIGDB", 
                "VAX", "")
)
msigdbr_types = msigdbr_types %>% 
  dplyr::mutate(gs_subcat2 = ifelse(gs_subcat=="",gs_cat,gs_subcat)) %>% 
  dplyr::mutate(gs_subcat_label = paste0(gs_cat, "--",gs_subcat2))

#   gs_cat   gs_subcat  gs_subcat2 gs_subcat_label
# 1      H                       H            H--H
# 2     C1                      C1          C1--C1
# 3     C2         CGP         CGP         C2--CGP
# 4     C2          CP          CP          C2--CP



## PharmacoGenomics ----
#source(system.file("shinyapp/PGdata.R", package = "UCSCXenaShiny"), local = PGdata <- new.env(), echo = FALSE)

# appdata_path = path.expand(file.path(getOption("xena.cacheDir"), "appdata.RData"))
# message("Saving data to ", appdata_path)
# save.image(file = appdata_path)

  md_prefix <- system.file("shinyapp/shiny-doc", package = "UCSCXenaShiny")
  set_md_path <- function(x) {
    file.path(md_prefix, x)
  }

ui.modules_pancan_sur_m2o = function(id) {
	ns = NS(id)
	fluidPage(
		fluidRow(
			column(
				3,
				wellPanel(
					style = "height:1100px",
					h2("S1: Preset", align = "center"),
					h4(strong("S1.1 Modify datasets"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Modify datasets", 
					                   content = "data_origin"),
					mol_origin_UI(ns("mol_origin2sur"), database = "toil"),

					h4(strong("S1.2 Choose cancer")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Cancer types", 
					                   content = "tcga_types"),
					pickerInput(
						ns("choose_cancer"), NULL,
						choices = sort(tcga_names),
						selected = "BRCA"),
					br(),

					h4(strong("S1.3 Filter samples"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Filter samples", 
					                   content = "choose_samples"),
					h5("Quick filter:"),
					pickerInput(
						ns("filter_by_code"), NULL,
						choices = NULL, selected =  NULL,
						multiple = TRUE, options = list(`actions-box` = TRUE)
					),
					h5("Exact filter:"),
					filter_samples_UI(ns("filter_samples2sur"), database = "toil"),
					br(),
					verbatimTextOutput(ns("filter_phe_id_info")),
					br(),
					
					h4(strong("S1.4 Upload metadata"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Upload metadata", 
					                   content = "custom_metadata"),
					shinyFeedback::useShinyFeedback(),
					custom_meta_UI(ns("custom_meta2sur")),
					br(),

					h4(strong("S1.5 Add signature"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Add signature", 
					                   content = "add_signature"),
					add_signature_UI(ns("add_signature2sur"), database = "toil"),
				)
			),
			column(
				4,
				wellPanel(
					style = "height:1100px",

					h2("S2: Get data", align = "center"),
					h4(strong("S2.1 Select survival endpoint")), 
				    shinyWidgets::prettyRadioButtons(
				        inputId = ns("endpoint_type"), label = NULL,
				        choiceValues = c("OS", "DSS", "DFI", "PFI"),
				        choiceNames = c("OS (Overall Survial)", "DSS (Disease-Specific Survival)", 
				        				"DFI (Disease-Free Interval)", "PFI (Progression-Free Interval)"),
				        selected = "OS",inline = FALSE
				    ),
					h4(strong("S2.2 Divide 2 groups by batch conditions")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Divide 2 groups by batch conditions", 
					                   content = "get_batch_data_set_groups"), 
					multi_upload_UI(ns("multi_upload2sur"),"Observe", database = "toil"),
				    # uiOutput(ns("L3s_x_data_sur.ui")),
					uiOutput(ns("set_quantile.ui")),
					uiOutput(ns("set_group1.ui")),
					uiOutput(ns("set_group2.ui")),
					# br(),
					shinyWidgets::actionBttn(
						ns("set_group"), "Group",
				        style = "gradient",
				        icon = icon("search"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					),
				    div(uiOutput(ns("L3s_x_data_sur_group.ui")),
				      style = "margin-top: 5px; margin-bottom: 0px;"
				    ),
					# uiOutput(ns("L3s_x_data_sur_group.ui"))
				)
			),	
			column(
				5,
				wellPanel(
					style = "height:1100px",
					h2("S3: Analyze", align = "center") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Analyze", 
					                   content = "analyze_sur_3"), 
					# br(),
					h4(strong("S3.1 Set analysis parameters")), 

					# br(),br(),
					selectInput(ns("sur_method"), "Survival method:",c("Log-rank test","Cox regression")),
				    materialSwitch(ns("use_origin"), 
					    	"Whether use initial data before grouping?"),
					br(),br(),
					shinyWidgets::actionBttn(
						ns("cal_batch_sur"), "Run",
				        style = "gradient",
				        icon = icon("table"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					),
					br(),
					fluidRow(
						column(10, offset = 1,
							   div(uiOutput(ns("sur_stat_tb.ui")),style = "height:600px"),
							   )
					),
					br(),
					h4(strong("S3.2 Download results")), 
					download_res_UI(ns("download_res2sur"))
				)
			)
		)
	)
}


server.modules_pancan_sur_m2o = function(input, output, session) {
	ns <- session$ns

	# 记录选择癌症
	cancer_choose <- reactiveValues(name = "BRCA", filter_phe_id=NULL,
		phe_primary=query_tcga_group(database = "toil",cancer = "BRCA", return_all = T))
	observe({
		cancer_choose$name = input$choose_cancer
		cancer_choose$phe_primary <- query_tcga_group(database = "toil",cancer = cancer_choose$name, return_all = T)
	})

	# 自定义上传metadata数据
	custom_meta = callModule(custom_meta_Server, "custom_meta2sur", database = "toil")
	# signature
	sig_dat = callModule(add_signature_Server, "add_signature2sur", database = "toil")

	custom_meta_sig = reactive({
		if(is.null(custom_meta())){
			return(sig_dat())
		} else {
			if(is.null(sig_dat())){
				return(custom_meta())
			} else {
				custom_meta_sig = dplyr::inner_join(custom_meta(),sig_dat())
				return(custom_meta_sig)
			}
		}
	})


	# 数据源设置
	opt_pancan = callModule(mol_origin_Server, "mol_origin2sur", database = "toil")

	## 过滤样本
	# exact filter module
	filter_phe_id = callModule(filter_samples_Server, "filter_samples2sur",
					   database = "toil",
					   cancers=reactive(cancer_choose$name),
					   custom_metadata=reactive(custom_meta_sig()),
					   opt_pancan = reactive(opt_pancan()))
	# quick filter widget
	observe({
		code_types_valid = code_types[names(code_types) %in% 
							unique(cancer_choose$phe_primary$Code)]
		updatePickerInput(
			session,
			"filter_by_code",
			choices = unlist(code_types_valid,use.names = F),
			selected =  unlist(code_types_valid,use.names = F)
		)
	})
	# 综合上述二者
	observe({
		# quick filter
		choose_codes = names(code_types)[unlist(code_types) %in% input$filter_by_code]
		filter_phe_id2 = cancer_choose$phe_primary %>%
			dplyr::filter(Code %in% choose_codes) %>%
			dplyr::pull("Sample")

		# exact filter
		if(is.null(filter_phe_id())){
			cancer_choose$filter_phe_id = filter_phe_id2
		} else {
			cancer_choose$filter_phe_id = intersect(filter_phe_id2,filter_phe_id())
		}

		output$filter_phe_id_info = renderPrint({
			cat(paste0("Tip: ", length(cancer_choose$filter_phe_id), " samples are retained"))
		})
	})


	# 批量下载数据
	L3s_x_data =  callModule(multi_upload_Server, "multi_upload2sur", 
							 samples=reactive(cancer_choose$filter_phe_id),
							 custom_metadata=reactive(custom_meta_sig()),
						     opt_pancan = reactive(opt_pancan()),
						     table.ui = FALSE
							 )
	L3s_x = reactive({
		unique(L3s_x_data()$id)
	})


	L3s_x_data_sur = reactive({
		sur_dat_raw = load_data("tcga_surv") %>%
			dplyr::rename("Sample"="sample") %>%
			dplyr::select("Sample",contains(input$endpoint_type)) %>%
			na.omit()
		colnames(sur_dat_raw)[2:3] = c("status","time")
		x_data_merge = L3s_x_data() %>%
			dplyr::inner_join(sur_dat_raw)
		x_data_merge

	})


	# 分组方式
	output$set_group1.ui = renderUI({
		choice_chrs = ifelse(input$inspect_data_x==0,0,L3s_x_data_sur()$value)
		fluidRow(
			if(class(choice_chrs)=="character"){
				column(
					8,
					selectInput(ns("group1_range"),"Group1 Range",
						sort(unique(choice_chrs), ,na.last = T), multiple=T)
				)
			} else {
				column(8,
					fluidRow(
						column(6,
							numericInput(ns("group1_min"),"Group1 [min",value=NA)),
						column(6,
							numericInput(ns("group1_max"),"max)",
								value=ifelse(input$set_quantile,0.5,median(choice_chrs, na.rm=T))))
					)
				)
			}
			,
			column(
				4,
				textInput(ns("group1_name"),"Group Name", "Group1")
			)
		)
	})
	output$set_group2.ui = renderUI({
		choice_chrs = ifelse(input$inspect_data_x==0,0,L3s_x_data_sur()$value)
		fluidRow(
			if(class(choice_chrs)=="character"){
				column(
					8,
					selectInput(ns("group2_range"),"Group2 Range",
						sort(unique(choice_chrs), ,na.last = T), multiple=T)
				)
			} else {
				column(8,
					fluidRow(
						column(6,
							numericInput(ns("group2_min"),"Group2 [min",
								value=ifelse(input$set_quantile,0.5,median(choice_chrs, na.rm=T)))),
						column(6,
							numericInput(ns("group2_max"),"max]",value=NA))
					)
				)
			}
			,
			column(
				4,
				textInput(ns("group2_name"),"Group Name", "Group2")
			)
		)
	})
	output$set_quantile.ui = renderUI({
		choice_chrs = ifelse(input$inspect_data_x==0,0,L3s_x_data_sur()$value)
		if(length(choice_chrs)==0) choice_chrs = 1:100
		if(class(choice_chrs)!="character"){
			materialSwitch(ns("set_quantile"),"Whether group by percentile?",value = TRUE)
		}
	})

	L3s_x_data_sur_group = eventReactive(input$set_group,{
		datas = L3s_x_data_sur()
		choice_chrs = L3s_x_data_sur()$value

		merge_by = list(
			c(input$group1_min, input$group1_max),
			c(input$group2_min, input$group2_max)
		)
		names(merge_by) = c(input$group1_name, input$group2_name)

		if(class(datas$value)=="character"){
		    shiny::validate(
		      need(length(intersect(input$group1_range,input$group2_range))>0, 
		        "Please inspect whether to input independent group range.")
		    )
			datas$group = NA
			datas$group[choice_chrs %in% input$group1_range] = input$group1_name
			datas$group[choice_chrs %in% input$group2_range] = input$group2_name
			datas = na.omit(datas)
		} else if (input$set_quantile) {
		    shiny::validate(
		      need(try(merge_by[[1]][2]<=merge_by[[2]][1]), 
		        "Please inspect whether to input independent group range.")
		    )
			if(is.na(merge_by[[1]][1])) merge_by[[1]][1] = 0
			if(is.na(merge_by[[2]][1])) merge_by[[1]][1] = 0
			if(is.na(merge_by[[1]][2])) merge_by[[1]][1] = 1
			if(is.na(merge_by[[2]][2])) merge_by[[2]][2] = 1
			datas_split = split(datas, datas$id)
			shinyjs::disable("set_group")
			withProgress(message = "Please wait for a while.",{
				datas = lapply(seq(datas_split), function(i){
					# 进度提醒
				    incProgress(1 / length(datas_split), detail = paste0("(Run ",i,"/",length(datas_split),")"))

					data_id = datas_split[[i]]
					choice_chrs_id = data_id$value
					merge_by_one = merge_by
			        for (i in seq(merge_by_one)){
			          merge_by_one[[i]] = quantile(choice_chrs_id, merge_by_one[[i]], na.rm = T)
			        }
					data_id$group = NA
					data_id$group[which(findInterval(choice_chrs_id, merge_by_one[[1]])==1)] = names(merge_by_one[1])
					data_id$group[which(findInterval(choice_chrs_id, merge_by_one[[2]], rightmost.closed = TRUE)==1)] = names(merge_by_one[2])
					data_id = na.omit(data_id)
				}) %>% do.call(rbind, .) %>% tibble::remove_rownames()
			})
			shinyjs::enable("set_group")

		} else {
		    shiny::validate(
		      need(merge_by[[1]][2] >= min(choice_chrs, na.rm=T), 
		        "Please inspect whether to input valid group range for Group1."),
		      need(merge_by[[2]][1] <= max(choice_chrs, na.rm=T), 
		        "Please inspect whether to input valid group range for Group2."),		      
		    )
		    shiny::validate(
		      need(try(merge_by[[1]][2]<=merge_by[[2]][1]), 
		        "Please inspect whether to input independent group range.")
		    )
			if(is.na(merge_by[[1]][1])) merge_by[[1]][1] = min(choice_chrs, na.rm=T)
			if(is.na(merge_by[[2]][1])) merge_by[[2]][1] = min(choice_chrs, na.rm=T)
			if(is.na(merge_by[[1]][2])) merge_by[[1]][2] = max(choice_chrs, na.rm=T)
			if(is.na(merge_by[[2]][2])) merge_by[[2]][2] = max(choice_chrs, na.rm=T)
			datas$group = NA
			datas$group[which(findInterval(choice_chrs, merge_by[[1]])==1)] = names(merge_by[1])
			datas$group[which(findInterval(choice_chrs, merge_by[[2]], rightmost.closed = TRUE)==1)] = names(merge_by[2])
			datas = na.omit(datas)
		}
		## 去除分组结果里只有一组的id
		valid_ids = datas %>%
			dplyr::distinct(id, group) %>%
			dplyr::count(id) %>%
			dplyr::filter(n==2) %>% dplyr::pull("id")
		datas_sub = datas %>% 
			dplyr::filter(id %in% valid_ids)
		datas_sub
	})

	output$L3s_x_data_sur_group.ui = renderUI({
		# group_stat = L3s_x_data_sur_group()
		output$L3s_x_data_sur_group = renderDataTable({
			group_stat = L3s_x_data_sur_group() 
			group_stat = group_stat %>%
				dplyr::count(id, group) %>% 
			datatable(group_stat,
				# class = "nowrap row-border",
				options = list(pageLength = 2, 
					columnDefs = list(list(className = 'dt-center', targets="_all")))
			)
		})
		output$valid_group = renderPrint({
			ids_num = length(unique(L3s_x_data_sur_group()$id))
			cat(paste0("Tip: ", ids_num, " ids have valid binary groups."))
		})		
		tagList(
			dataTableOutput(ns("L3s_x_data_sur_group")),
			verbatimTextOutput(ns("valid_group"))
		)
	})


	sur_stat = eventReactive(input$cal_batch_sur,{
		datas = L3s_x_data_sur_group()
		valid_ids = unique(datas$id)
		shinyjs::disable("cal_batch_sur")
		withProgress(message = "Please wait for a while.",{
			sur_stat = lapply(seq(valid_ids), function(i){
			    incProgress(1 / length(valid_ids), detail = paste0("(Finished ",i,"/",length(valid_ids),")"))
				valid_id = valid_ids[i]
				datas_sub = subset(datas, id %in% valid_id)
				print(head(datas_sub))
				if(input$sur_method=="Log-rank test"){
					if(!input$use_origin){
						datas_sub$Group = datas_sub$group
					} else {
						# datas_sub$Group = datas_sub$value
						if(class(datas_sub$value)!="character"){
							res.cut <- surv_cutpoint(datas_sub, 
								time = "time", event = "status", variables = "value")
							groups_1_2 = datas_sub %>% 
								  dplyr::group_by(group) %>% 
								  dplyr::summarise(mean = mean(value)) %>% 
								  dplyr::arrange(mean) %>% 
								  dplyr::pull(group) %>% as.character()
							datas_sub$Group = ifelse(surv_categorize(res.cut)$value=="low", groups_1_2[1], groups_1_2[2])
							datas_sub$Group = factor(datas_sub$Group, levels=groups_1_2)
						}
					}
					surv_diff <- survdiff(Surv(time, status) ~ Group, data = datas_sub)
					pval = 1 - pchisq(surv_diff$chisq, length(surv_diff$n) - 1)
					# sur_res = summary(survfit(Surv(time, status) ~ Group, data = datas_sub))$table %>% 
					# 	    as.data.frame() %>% dplyr::pull(median)

					sur_tab = summary(survfit(Surv(time, status) ~ Group, data = datas_sub))$table %>% 
						    as.data.frame()

					sur_res = c(sur_tab[paste0("Group=",input$group1_name),"median"],
								sur_tab[paste0("Group=",input$group2_name),"median"], pval)
					# sur_res = c(sur_res,pval)
					names(sur_res) = c(input$group1_name,input$group2_name,"p.value")
					names(sur_res)[1:2] = paste0(names(sur_res)[1:2],"\nmedian.time")
				} else if (input$sur_method=="Cox regression"){
					if(!input$use_origin){
						datas_sub$Group = datas_sub$group
					} else {
						if(class(datas_sub$value)!="character"){
							datas_sub$Group = datas_sub$value
						} else {
							datas_sub$Group = datas_sub$group
						}
					}
					fit <- coxph(Surv(time, status) ~ Group, data = datas_sub)
					sur_res = summary(fit)$coefficients %>% as.data.frame() %>%
						tibble::rownames_to_column("Group.obs")
					sur_res = sur_res[,c(1,3,6)]
					sur_res[,1] = gsub("^Group","",sur_res[,1])
					sur_res[,2] = round(sur_res[,2], 3)
					colnames(sur_res)[2:3] = c("HR","p.value")
				}
				sur_res
			}) %>% do.call(rbind, .)
			sur_stat = sur_stat %>% as.data.frame() %>%
				dplyr::mutate(id = valid_ids, .before = 1) %>%
				dplyr::arrange(p.value)
			sur_stat
		})
		shinyjs::enable("cal_batch_sur")
		sur_stat

	})

	output$sur_stat_tb.ui = renderUI({
		output$sur_stat_tb = renderDataTable({
			sur_stat_ = sur_stat() %>%
				dplyr::rename("Batch identifiers"="id")
			sur_stat_$p.value = format(sur_stat_$p.value, scientific=T, digits = 3)

			dt = datatable(sur_stat_,
				# class = "nowrap row-border",
				options = list(pageLength = 10, 
					columnDefs = list(
						list(className = 'dt-center', targets="_all"),
						list(orderable=TRUE, targets = 0)))
			)
			dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
			dt
		}) 
		dataTableOutput(ns("sur_stat_tb"))
	})
	# Download results
	observeEvent(input$cal_batch_sur,{
		res1 = NULL
		res2 = L3s_x_data_sur_group()
		res3 = sur_stat()
		callModule(download_res_Server, "download_res2sur", res1, res2, res3)
	})
}

# https://github.com/rstudio/shiny-examples/blob/master/036-custom-input-control/chooser.R
chooserInput <- function(inputId, leftLabel, rightLabel, leftChoices, rightChoices,
                         size = 5, multiple = FALSE) {
  leftChoices <- lapply(leftChoices, tags$option)
  rightChoices <- lapply(rightChoices, tags$option)

  if (multiple) {
    multiple <- "multiple"
  } else {
    multiple <- NULL
  }

  tagList(
    singleton(tags$head(
      tags$script(src = "chooser-binding.js"),
      tags$style(
        type = "text/css",
        HTML(".chooser-container { display: inline-block; }")
      )
    )),
    div(
      id = inputId, class = "chooser",
      div(
        class = "chooser-container chooser-left-container",
        tags$select(class = "left", size = size, multiple = multiple, leftChoices)
      ),
      div(
        class = "chooser-container chooser-center-container",
        icon("arrow-circle-o-right", "right-arrow fa-3x"),
        tags$br(),
        icon("arrow-circle-o-left", "left-arrow fa-3x")
      ),
      div(
        class = "chooser-container chooser-right-container",
        tags$select(class = "right", size = size, multiple = multiple, rightChoices)
      )
    )
  )
}

registerInputHandler("shinyjsexamples.chooser", function(data, ...) {
  if (is.null(data)) {
    NULL
  } else {
    list(left = as.character(data$left), right = as.character(data$right))
  }
}, force = TRUE)

ui.modules_1_tcga_06 = function(id){
    ns = NS(id)

    main_ui = tagList(
        mol_quick_select_UI(ns("id"), "tcga", 
            c("mRNA","transcript","methylation","miRNA","protein", "cnv")),

        h4("3. Select fearture type"),
        selectInput(
            ns("Type"), NULL,
            choices = c("Stemness", "TMB", "MSI"),
            selected = "Stemness"
        ),
        h4("4. Select correlation method"),
        awesomeRadio(
            inputId = ns("cor_method"),
            label = NULL, 
            choices = c("spearman", "pearson"),
            selected = "spearman",
            inline = TRUE, checkbox = TRUE
        ),
        shinyWidgets::actionBttn(
            inputId = ns("search_bttn"),
            label = "Go!",
            style = "gradient",
            icon = icon("search"),
            color = "primary",
            block = TRUE,
            size = "sm"
        )
    )
    out_ui = tagList(
        fluidRow(
            uiOutput(ns("gene_pancan_radar")),
        )
    )
    side_ui = tagList(
        fluidRow(
            column(6,
                h4("1. Visualization parameters"),
                h5("(1) Line color:"),
                colourpicker::colourInput(
                    inputId = ns("color_line"), NULL, "#00AFBB")
            ),
            column(6,
                h4("2. Download options"),
                h5("(1) Figure:"),
                numericInput(inputId = ns("height"), label = "Height", value = 6),
                numericInput(inputId = ns("width"), label = "Width", value = 8),
                awesomeRadio(ns("device"), label = "Format", 
                    choices = c("pdf", "png"), selected = "pdf", inline = TRUE),
                downloadBttn(
                  outputId = ns("download_1"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                ),
                h5("(2) Data table:"),
                downloadBttn(
                  outputId = ns("download_2"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                )
            )
        )
    )
    fluidPage(
        style = "height:600px",
        wellPanel(style = "height:725px",
        box(main_ui,
            width = 5,
            solidHeader = TRUE,
            title = "Quick TCGA Analysis: Correlation in tumor samples",
            status = "success",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Click the bottom button to execute/update the analysis."
        ),
        box(out_ui,
            width = 7,
            solidHeader = TRUE,
            title = "Analytical results:", 
            status = "success",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Pull the sidebar to adjust plot parameters or download results through the top-right widget.",
            sidebar = boxSidebar(
                        id = ns("sidebar"),
                        width = 50,
                        side_ui
            )
        )
        )
    )



}


server.modules_1_tcga_06 = function(input, output, session){
    ns = session$ns

    mol_info = callModule(mol_quick_select_Server, "id", "tcga")

    plot_func <- eventReactive(input$search_bttn, {
        id <- showNotification(h3("The task is running..."), duration = NULL, closeButton = FALSE, type = "message")
        on.exit(removeNotification(id), add = TRUE)  #reactive语句执行完毕时，运行remove命令
        vis_fun <- switch(input$Type,
            Stemness = vis_gene_stemness_cor,
            TMB = vis_gene_tmb_cor,
            MSI = vis_gene_msi_cor
        )
        p <- vis_fun(Gene = mol_info$molecule(), 
                    data_type = mol_info$profile(),
                    cor_method = input$cor_method)
        
        if (is.null(p)) {
            sendSweetAlert(session, title = "Warning", text = "No data could be queried!")
            return(NULL)
        }

        pdata <- p$data %>%
            dplyr::mutate(cor = round(cor, digits = 3), p.value = round(p.value, digits = 3))
        df <- pdata %>%
            select(cor, cancer) %>%
            pivot_wider(names_from = cancer, values_from = cor) %>%
            tibble::rownames_to_column()
        
        plot <- ggradar::ggradar(
            df[1, ],
            font.radar = "sans",
            values.radar = c("-1", "0", "1"),
            grid.min = -1, grid.mid = 0, grid.max = 1,
            # Background and grid lines
            background.circle.colour = "white",
            gridline.mid.colour = "grey",
            # Polygons
            group.line.width = 1,
            group.point.size = 3,
            group.colours = input$color_line,
            plot.title = paste0(mol_info$molecule(), " ", mol_info$profile(), "——", input$Type)
        ) + theme(plot.title = element_text(hjust = .5))
        return(list(plot = plot, data = pdata))
    })



    # Show waiter for plot
    w <- waiter::Waiter$new(id = ns("gene_pancan_radar"), html = waiter::spin_hexdots(), color = "black")
    observeEvent(input$search_bttn,{
        shinyjs::disable("search_bttn")
        # check whether valid out plot
        chect_plot = is.null(plot_func()) 
        if(chect_plot){
            sendSweetAlert(session, title = "Warning", type = "error", text = "Please select a valid molecule.")
            req(chect_plot)
        }
        output$gene_pancan_radar <- renderUI({
            w$show()
            output$plot = renderPlot(plot_func()$plot)
            fluidRow(
                column(8, offset = 2,
                    plotOutput(ns("plot"), height = "580px"),
                )
            )
        })  
        shinyjs::enable("search_bttn")
    })

    output$download_1 <- downloadHandler(
        filename = function() {
            paste0(mol_info$molecule(), "_", mol_info$profile(), "_pancan_index.", input$device)
        },
        content = function(file) {
            p <- plot_func()
            if (input$device == "pdf") {
                pdf(file, width = input$width, height = input$height)
                print(p)
                dev.off()
            } else {
                png(file, width = input$width, height = input$height, res = 600, units = "in")
                print(p)
                dev.off()
            }
        }
    )

    output$download_2 <- downloadHandler(
        filename = function() {
            paste0(mol_info$molecule(), "_", mol_info$profile(), "_pancan_index.csv")
        },
        content = function(file) {
            data = plot_func()$data %>%
                dplyr::rename('Cancer'='cancer',
                'Cor'='cor', 'P.value'='p.value') %>%
                dplyr::select(Cancer, Cor, P.value) %>%
                tibble::remove_rownames()
            write.csv(data, file, row.names = FALSE)
        }
    )
}

group_samples_UI = function(id, button_name="Filter by multi-conditions", database = "toil"){
  ns = NS(id)
  id_option = switch(database, 
      "toil"=tcga_id_option,
      "pcawg"=pcawg_id_option,
      "ccle"=ccle_id_option)

  tagList(
    # h4("1. Select one condition"),
    
    fluidRow(
      column(
        6,
        selectInput(
          ns("data_L1"), "Data type:",
          choices = names(id_option),
          selected = "Phenotype data"
        )
      ),
      column(
          6,
          tabsetPanel(
            id = ns("data_L2_tab"),
            type = "hidden",
          tabPanel("Molecular profile", 
            selectInput(
              ns("genomic_profile"), "Data subtype:",
              choices = names(id_option[["Molecular profile"]]),
              selected = "mRNA Expression")
          ),
          tabPanel("Tumor index",
            selectInput(
              ns("tumor_index"), "Data subtype:",
              choices = names(id_option[["Tumor index"]]),
              selected = "Tumor Purity")
          ),
          tabPanel("Immune Infiltration",
            selectInput(
              ns("immune_infiltration"), "Data subtype:",
              choices = names(id_option[["Immune Infiltration"]]),
              selected = "CIBERSORT")
          ),
          tabPanel("Pathway activity",
            selectInput(
              ns("pathway_activity"), "Data subtype:",
              choices = names(id_option[["Pathway activity"]]),
              selected = "HALLMARK")
          ),
          tabPanel("Phenotype data",
            selectInput(
              ns("phenotype_data"), "Data subtype:",
              choices = names(id_option[["Phenotype data"]]),
              selected = "Clinical Phenotye")
          )
        )
      )
    ),
    tabsetPanel(
      id = ns("data_L3_tab"),
      type = "hidden",
      # selected = "Molecular_profile",
      tabPanel("Molecular profile",
               virtualSelectInput(
                 inputId = ns("genomic_profile_id"),
                 label = "Identifier:",
                 choices = NULL,
                 search = TRUE,
                 allowNewOption = TRUE,
                 dropboxWidth = "200%"
               )
      ),
      tabPanel("Tumor index",
               virtualSelectInput(
                 inputId = ns("tumor_index_id"),
                 label = "Identifier:",
                 choices = NULL,
                 search = TRUE,
                 allowNewOption = FALSE,
                 dropboxWidth = "200%")
      ),
      tabPanel("Immune Infiltration",
               virtualSelectInput(
                 inputId = ns("immune_infiltration_id"),
                 label = "Identifier:",
                 choices = NULL,
                 search = TRUE,
                 allowNewOption = FALSE,
                 dropboxWidth = "200%")
      ),
      tabPanel("Pathway activity",
               virtualSelectInput(
                 inputId = ns("pathway_activity_id"),
                 label = "Identifier:",
                 choices = NULL,
                 search = TRUE,
                 allowNewOption = FALSE,
                 dropboxWidth = "200%")	
      ),
      tabPanel("Phenotype data",
               virtualSelectInput(
                 inputId = ns("phenotype_data_id"),
                 label = "Identifier:",
                 choices = NULL,
                 search = TRUE,
                 allowNewOption = FALSE,
                 dropboxWidth = "200%")  
      )
    ),
    shinyWidgets::actionBttn(
      ns("query_dist"), "Observe",
          style = "gradient",
          icon = icon("chart-simple"),
          color = "primary",
          block = TRUE,
          size = "sm"
    ),
    div(verbatimTextOutput(ns("condi_dist")),
      style = "margin-top: 5px; margin-bottom: 0px;"
    ),
    
    # h4("2. Group by two range"),
    # shinyWidgets::actionBttn(
    #   ns("group_range"), "Group",
    #       style = "gradient",
    #       icon = icon("list"),
    #       color = "primary",
    #       block = TRUE,
    #       size = "sm"
    # ),

		uiOutput(ns("set_quantile.ui")),
    materialSwitch(ns("reverse_level"), "Whether reverse levels?"),
     # %>%
     #    helper(type = "markdown", size = "m", fade = TRUE, 
     #               title = "Split above phenotype into 2 groups", 
     #               content = "set_groups"),
		uiOutput(ns("set_group1.ui")),
		uiOutput(ns("set_group2.ui")),
    shinyWidgets::actionBttn(
      ns("group_range"), "Group",
          style = "gradient",
          icon = icon("list"),
          color = "primary",
          block = TRUE,
          size = "sm"
    ),

    div(verbatimTextOutput(ns("merge_out_tip")),
      style = "margin-top: 5px; margin-bottom: 0px;"
    )
    # verbatimTextOutput(ns("tmp123"))
    
  )
}


group_samples_Server = function(input, output, session, database = "toil", 
                                cancers=NULL, samples=NULL, custom_metadata=NULL, opt_pancan=NULL){
  ns <- session$ns

  id_option = switch(database, 
      "toil"=tcga_id_option,
      "pcawg"=pcawg_id_option,
      "ccle"=ccle_id_option)
  id_category = lapply(id_option, names)

  observe({
    updateTabsetPanel(inputId = "data_L2_tab", selected = input$data_L1)
    updateTabsetPanel(inputId = "data_L3_tab", selected = input$data_L1)
  }) 

  genomic_profile_choices <- reactive({
    id_option[["Molecular profile"]][[input$genomic_profile]]
  })

  tumor_index_choices <- reactive({
    id_option[["Tumor index"]][[input$tumor_index]]
  })

  immune_infiltration_choices <- reactive({
    id_option[["Immune Infiltration"]][[input$immune_infiltration]]
  })

  pathway_activity_choices <- reactive({
    id_option[["Pathway activity"]][[input$pathway_activity]]
  })

  phenotype_data_choices <- reactive({
      id_tmp = id_option[["Phenotype data"]]
    if(!is.null(custom_metadata)){
      id_tmp[["Custom metadata"]]$all = sort(colnames(custom_metadata()[-1]))
      id_tmp[["Custom metadata"]]$default = sort(colnames(custom_metadata()[-1]))[1]
    }
    id_tmp[[input$phenotype_data]]
  })

  observe({
    updateVirtualSelect(
      "genomic_profile_id",
      choices = genomic_profile_choices()$all,
      selected = genomic_profile_choices()$default
    )
    updateVirtualSelect(
      "tumor_index_id",
      choices = tumor_index_choices()$all,
      selected = tumor_index_choices()$default
    )
    updateVirtualSelect(
      "immune_infiltration_id",
      choices = immune_infiltration_choices()$all,
      selected = immune_infiltration_choices()$default
    )
    updateVirtualSelect(
      "pathway_activity_id",
      choices = pathway_activity_choices()$all,
      selected = pathway_activity_choices()$default
    )
    updateVirtualSelect(
      "phenotype_data_id",
      choices = phenotype_data_choices()$all,
      selected = phenotype_data_choices()$default
    )
  })
  
  ## query conditon & observe distribution
  condi_data = eventReactive(input$query_dist,{
    shinyjs::disable("query_dist")
    L2_x = switch(input$data_L1,
        `Molecular profile` = input$genomic_profile,
        `Tumor index` = input$tumor_index,
        `Immune Infiltration` = input$immune_infiltration,
        `Pathway activity` = input$pathway_activity,
        `Phenotype data` = input$phenotype_data
    )
    L3_x = switch(input$data_L1,
        `Molecular profile` = input$genomic_profile_id,
        `Tumor index` = input$tumor_index_id,
        `Immune Infiltration` = input$immune_infiltration_id,
        `Pathway activity` = input$pathway_activity_id,
        `Phenotype data` = input$phenotype_data_id
    )
    L1_x = names(id_category)[sapply(id_category, function(x){any(x %in% L2_x)})]
    if(is.null(opt_pancan)){
      opt_pancan = .opt_pancan
    } else {
      opt_pancan = opt_pancan()
    }
    ## 利用内部自定义下载函数获取数据
		if(database=="toil"){
			if(!exists("tcga_value_nonomics")){
				tcga_value_nonomics = load_data("v2_tcga_value_nonomics")
			}
			clinical_phe = tcga_clinical_fine
			x_data = UCSCXenaShiny:::query_general_value(L1_x, L2_x, L3_x, database,
							tcga_value_nonomics, opt_pancan,custom_metadata())
		} else if(database=="pcawg"){
			if(!exists("pcawg_value_nonomics")){
				pcawg_value_nonomics = load_data("v2_pcawg_value_nonomics")
			}
			clinical_phe = pcawg_info_fine
			x_data = UCSCXenaShiny:::query_general_value(L1_x, L2_x, L3_x, database,
							pcawg_value_nonomics, opt_pancan,custom_metadata())
		} else if (database=="ccle"){
			if(!exists("ccle_value_nonomics")){
				ccle_value_nonomics = load_data("v2_ccle_value_nonomics")
			}
			clinical_phe = ccle_info_fine
			x_data = UCSCXenaShiny:::query_general_value(L1_x, L2_x, L3_x, database,
							ccle_value_nonomics, opt_pancan,custom_metadata())
		}
    if(!is.null(samples)){
      if(!is.null(samples())){
        x_data = x_data %>%
          dplyr::filter(Sample %in% samples())
      }
    }
    # req(nrow(x_data)>0)
    shinyjs::enable("query_dist")
    if(nrow(x_data)>0){
      x_data$level1 = L1_x
      x_data$cancer = clinical_phe[,2,drop=T][match(x_data$Sample, clinical_phe$Sample)]
      x_data = x_data[,c("id","level1","level2","Sample","value","cancer")] %>%
        dplyr::arrange(cancer, Sample)
      x_data
    } else {
      NULL
    }
  })
  # output$condi_dist = renderPrint({head(condi_data())})

  output$condi_dist = renderPrint({
    if(is.null(condi_data())) return("No valid sammple for the identifier.")
    if(class(condi_data()[,"value",drop=TRUE])=="numeric"){
      summary(condi_data()[,"value",drop=TRUE])
    } else {
      table(condi_data()[,"value",drop=TRUE])
    }
  })


  output$set_quantile.ui = renderUI({
    req(!is.null(condi_data()))
    choice_chrs = condi_data()$value
    if(class(choice_chrs)!="character"){
      materialSwitch(ns("set_quantile"),"Whether group by percentile?",value = TRUE)
    }
  })


  # 创建两分组

  output$set_group1.ui = renderUI({
    req(!is.null(condi_data()))
    choice_chrs = condi_data()$value
    fluidRow(
      if(class(choice_chrs)=="character"){
        column(
          8,
          selectInput(ns("group1_range"),"Group1 Range",
            sort(unique(choice_chrs), ,na.last = T), multiple=T)
        )
      } else {
        column(8,
          fluidRow(
            column(6,
              numericInput(ns("group1_min"),"Group1 [min",value=NA)),
            column(6,
              numericInput(ns("group1_max"),"max)",
                value=ifelse(input$set_quantile,0.5,median(choice_chrs, na.rm=T))))
          )
        )
      }
      ,
      column(
        4,
        textInput(ns("group1_name"),"Group1 Name", "Group1")
      )
    )
  })

  output$set_group2.ui = renderUI({
    req(!is.null(condi_data()))
    choice_chrs = condi_data()$value
    fluidRow(
      if(class(choice_chrs)=="character"){
        column(
          8,
          selectInput(ns("group2_range"),"Group2 Range",
            sort(unique(choice_chrs), ,na.last = T), multiple=T)
        )
      } else {
        column(8,
          fluidRow(
            column(6,
              numericInput(ns("group2_min"),"Group2 [min",
                value=ifelse(input$set_quantile,0.5,median(choice_chrs, na.rm=T)))),
            column(6,
              numericInput(ns("group2_max"),"max]",value=NA))
          )
        )
      }
      ,
      column(
        4,
        textInput(ns("group2_name"),"Group2 Name", "Group2")
      )
    )
  })




  # 提取分组条件
  merge_by = eventReactive(input$group_range, {
    merge_by.list = list(NULL, NULL)
    names(merge_by.list) = c(input$group1_name, input$group2_name)

    choice_chrs = condi_data()$value
    if(class(choice_chrs)=="character"){
      merge_by.list[[1]] = input$group1_range
      merge_by.list[[2]] = input$group2_range
    } else {
      merge_by.list[[1]] = 
        c(ifelse(is.null(input$group1_min),NA,input$group1_min),
          ifelse(is.null(input$group1_max),NA,input$group1_max))
      merge_by.list[[2]] = 
        c(ifelse(is.null(input$group2_min),NA,input$group2_min),
          ifelse(is.null(input$group2_max),NA,input$group2_max))
    }
    merge_by.list
  })

  # # 查看分组条件

    
  # 根据提供条件进行分组


  merge_out = eventReactive(input$group_range,{
    merge_tmp = query_tcga_group(database = database,
                                 cancer=cancers(), 
                                 custom=condi_data()[,c("Sample","value")],
                                 group="value", filter_id = samples(),
                                 merge_by = merge_by(),
                                 merge_quantile = input$set_quantile)[['data']]
    merge_tmp

    merge_tmp$phenotype = unique(condi_data()$id)
    colnames(merge_tmp)[3] = "group"

    if(input$reverse_level){
      group_levels = c(input$group2_name, input$group1_name)
    } else {
      group_levels = c(input$group1_name, input$group2_name)
    }
    merge_tmp[,"group"] = factor(merge_tmp[,"group",drop=TRUE], levels = group_levels)  
    #分组前数据
    merge_tmp$origin = condi_data()$value[match(merge_tmp$Sample, condi_data()$Sample)]

    merge_tmp
  })

  # 分组结果提示
  observeEvent(input$group_range,{
    output$merge_out_tip = renderPrint({
      shiny::validate(
        need(try(nrow(merge_out())>0), 
          "Please inspect whether to input valid grouping set."),
      )
      tb = table(merge_out()$group)
      cat(paste0(levels(merge_out()$group)[1],": ", tb[1], ", ",
                     levels(merge_out()$group)[2],": ", tb[2]))
    })
  })


  # output$tmp123 = renderPrint({head(merge_out())})

  # 检查是否有效分组
  # 主要针对多癌症分组情况
  merge_out_check = reactive({
    valid_cancers = merge_out() %>%
      dplyr::mutate(group = as.character(group)) %>%
      dplyr::distinct(Cancer, group) %>%
      dplyr::count(Cancer) %>%
      dplyr::filter(n==2) %>% dplyr::pull("Cancer")
    valid_group = merge_out() %>% 
      dplyr::filter(Cancer %in% valid_cancers)
    valid_group
  })

  return(merge_out_check)
}


# # A tibble: 6 × 6
#   Sample          Patient      Cancer group phenotype origin
#   <chr>           <chr>        <chr>  <fct> <chr>     <chr> 
# 1 TCGA-A7-A0CE-11 TCGA-A7-A0CE BRCA   NT    Code      NT    
# 2 TCGA-A7-A0CH-11 TCGA-A7-A0CH BRCA   NT    Code      NT    
# 3 TCGA-A7-A0D9-11 TCGA-A7-A0D9 BRCA   NT    Code      NT    
# 4 TCGA-A7-A0DB-11 TCGA-A7-A0DB BRCA   NT    Code      NT    
# 5 TCGA-A7-A0DC-11 TCGA-A7-A0DC BRCA   NT    Code      NT    
# 6 TCGA-A7-A13E-11 TCGA-A7-A13E BRCA   NT    Code      NT 

ui.modules_3_ccle_03 = function(id){
    ns = NS(id)
    main_ui = tagList(
        h4("1. Select omics type"),
        selectInput(
            inputId = ns("profile"), label = NULL,
            choices  = c("mRNA Expression"="mRNA"),
            selected = "mRNA"
        ),
        h4(id = ns("id_tip"), "2. Select one gene list"),
        bsTooltip(ns("id_tip"), "When two or more genes are selected, their geometric mean is calculated as the signature value.", 
                    placement = "right", trigger = "hover", options = list(container = "body")),
        virtualSelectInput(
            inputId = ns("Pancan_search_1"),
            label = NULL, choices = NULL, multiple = TRUE,
            width = "100%", search = TRUE,
            allowNewOption = FALSE, dropboxWidth = "100%"
        ),
        h4("3. Select X-axis type"),
        selectInput(
            inputId = ns("x_axis_type"),
            label = NULL,
            choices = c("mean.diff", "median.diff"),
            selected = "mean.diff"
        ),
        shinyWidgets::actionBttn(
            inputId = ns("search_bttn"),
            label = "Go!",
            style = "gradient",
            icon = icon("search"),
            color = "primary",
            block = TRUE,
            size = "sm"
        )
    )
    out_ui = tagList(
        fluidRow(
            uiOutput(ns("gene_ccle_drug_target")),
        )
    )
    side_ui = tagList(
        fluidRow(
            column(6,
                h4("1. Visualization parameters"),
                selectInput(
                    inputId = ns("output_form"),
                    label = "(1) Plot output form:",
                    choices = c("plotly", "ggplot2"),
                    selected = "plotly"
                ),
            ),
            column(6,
                h4("2. Download options"),
                h5("(1) Figure:"),
                numericInput(inputId = ns("height"), label = "Height", value = 6),
                numericInput(inputId = ns("width"), label = "Width", value = 6),
                awesomeRadio(ns("device"), label = "Format", 
                    choices = c("pdf", "png"), selected = "pdf", inline = TRUE),
                downloadBttn(
                  outputId = ns("download_1"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                ),
                h5("(2) Data table:"),
                downloadBttn(
                  outputId = ns("download_2"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                )
            )
        )
    )
    fluidPage(
        wellPanel(style = "height:725px",
        box(main_ui,
            width = 5,
            solidHeader = TRUE,
            title = "Quick Analysis: Analyze Association between Gene (Signature) and Drug Response", 
            status = "primary",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Click the bottom button to execute/update the analysis."
        ),
        box(out_ui,
            width = 7,
            solidHeader = TRUE,
            title = "Analytical results:", 
            status = "primary",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Pull the sidebar to adjust plot parameters or download results through the top-right widget.",
            sidebar = boxSidebar(
                        id = ns("sidebar"),
                        width = 50,
                        side_ui
            )
        )
        )
    )
}



server.modules_3_ccle_03 = function(input, output, session){
    ns = session$ns

    gene_ref = sort(rownames(load_data("ccle_expr_and_drug_response")$expr))

    profile_choices <- reactive({
        list(all = ccle_id.list[["Gene"]], default = "TP53")
    })

    observe({
        updateVirtualSelect(
        "Pancan_search_1",
        # choices = profile_choices()$all,
        choices = gene_ref,
        selected = profile_choices()$default
        )
    })

    plot_func <- eventReactive(input$search_bttn, {
        shiny::validate(
            need(try(length(input$Pancan_search_1)<10), 
                "Error: Less than 10-gene signature is supported.")
        )

        id <- showNotification(h3("The task is running..."), duration = NULL, closeButton = FALSE, type = "message")

        on.exit(removeNotification(id), add = TRUE)  #reactive语句执行完毕时，运行remove命令

        p <- vis_gene_drug_response_asso(
            Gene = input$Pancan_search_1,
            output_form = input$output_form,
            x_axis_type = input$x_axis_type
        )
        print(class(p))
        return(p)
    })

    w <- waiter::Waiter$new(id = ns("gene_ccle_drug_target"), html = waiter::spin_hexdots(), color = "black")
 
    observeEvent(input$search_bttn,{
        shinyjs::disable("search_bttn")
        if (input$output_form == "ggplot2") {
            output$gene_ccle_drug_target <- renderUI({
                w$show()
                output$plot = renderPlot(plot_func())
                fluidRow(
                    column(10, offset = 1,
                        plotOutput(ns("plot"), height = "580px"),
                    )
                )
            }) 
        } else if (input$output_form == "plotly"){
            output$gene_ccle_drug_target <- renderUI({
                w$show()
                output$plot = plotly::renderPlotly(plot_func())
                fluidRow(
                    column(10, offset = 1,
                        plotly::plotlyOutput(ns("plot"), height = "580px"),
                    )
                )
            }) 
        }
        shinyjs::enable("search_bttn")
    })

    output$download_1 <- downloadHandler(
        filename = function() {
            paste0("ccle_drug_target.", input$device)
        },
        content = function(file) {
            p <- plot_func()
            if (input$device == "pdf") {
                pdf(file, width = input$width, height = input$height)
                print(p)
                dev.off()
            } else {
                png(file, width = input$width, height = input$height, res = 600, units = "in")
                print(p)
                dev.off()
            }
        }
    )

    output$download_2 <- downloadHandler(
        filename = function() {
            "ccle_drug_target.csv"
        },
        content = function(file) {
            data = data <- analyze_gene_drug_response_asso(input$Pancan_search_1, combine = TRUE)
            write.csv(data, file, row.names = FALSE)
        }
    )
} 

ui.modules_download_pancan = function(id){
	ns = NS(id)
	fluidPage(
		fluidRow(
			column(
				8,
				wellPanel(
					style = "height:1100px",
					fluidRow(
						column(6,
							h2("Part1: Download molecular data", align = "center"),
							h3("1. Select one database"),
							radioGroupButtons(
							   inputId = ns("L0"),
							   label = NULL,
							   choiceNames = c("TCGA(TOIL)", "PCAWG", "CCLE"),
							   choiceValues = c("toil", "pcawg", "ccle"),
							   selected = "toil",
							   justified = TRUE,
							   checkIcon = list(
							      yes = icon("ok", 
							    lib = "glyphicon"))
							),
							tabsetPanel(id = ns("L0_datasets_tab"),
								type = "hidden",
								tabPanel("toil",
									mol_origin_UI(ns("mol_origin2toil"), database = "toil"),
								),
								tabPanel("pcawg",
									mol_origin_UI(ns("mol_origin2pcawg"), database = "pcawg"),
								),
								tabPanel("ccle",
									mol_origin_UI(ns("mol_origin2ccle"), database = "ccle"),
								),
							),

							h3("2. Select samples"),
							h5("Quick filter:"),
							fluidRow(
								column(6,
									pickerInput(
										ns("filter_by_cancer"), NULL,
										choices = NULL, selected =  NULL,
										multiple = TRUE, options = list(`actions-box` = TRUE)
									)
								),
								column(6,
									pickerInput(
										ns("filter_by_code"), NULL,
										choices = NULL, selected =  NULL,
										multiple = TRUE, options = list(`actions-box` = TRUE)
									)
								)
							),
							h5("Exact filter:"),
							tabsetPanel(id = ns("filter_samples2dw_tab"),
								type = "hidden",
								tabPanel("toil",
									filter_samples_UI(ns("filter_samples2dw_1"), database = "toil"),
								),
								tabPanel("pcawg",
									filter_samples_UI(ns("filter_samples2dw_2"), database = "pcawg"),
								),
								tabPanel("ccle",
									filter_samples_UI(ns("filter_samples2dw_3"), database = "ccle"),
								),
							),
							verbatimTextOutput(ns("filter_id_info")),
							
							h3("3. Select identifiers"),	
							# 选择major/minor type
						    fluidRow(
						    	column(
						    		6,
								    selectInput(
								    	ns("data_L1"), label = "Data type:",
								    	# choices = c("Molecular profile","Tumor index","Immune Infiltration","Pathway activity","Phenotype data"),
								    	choices = c("Molecular profile"),
								    	selected = "Molecular profile"
								    )
						    	),
						    	column(
						    		6,
								    tabsetPanel(
									    id = ns("data_L2_tab"),
									    type = "hidden",
										tabPanel("Molecular profile", 
											selectInput(
												ns("genomic_profile"), "Data subtype:",
												choices = NULL,
												selected = "mRNA Expression")
										)
									)
						    	)
						    ),
							prettyRadioButtons(ns("L3_x_type"),"Choose multi-ids by", 
								choices = c("Selection","All","File"), selected = "Selection",
								inline=TRUE
							),
							tabsetPanel(id = ns("L3_x_type_tab"),
								type = "hidden",
								tabPanel("Selection",
								    tabsetPanel(
									    id = ns("data_L3_tab"),
									    type = "hidden",
										tabPanel("Molecular profile",
								            virtualSelectInput(
								              inputId = ns("genomic_profile_id"),
								              label = NULL,
								              choices = NULL, multiple = TRUE,
								              search = TRUE,
								              allowNewOption = TRUE,
								              dropboxWidth = "200%")
										)
									),	
								),
								tabPanel("All",
									# uiOutput(ns("tab_All"))
									fluidRow(
										uiOutput(ns("msigdb_note.ui")), 
										column(
											4,
								            virtualSelectInput(
								              inputId = ns("msigdbr_cat"),
								              label = NULL,
								              choices = msigdbr_types$gs_subcat_label, 
								              selected =  msigdbr_types$gs_subcat_label[1],
								              dropboxWidth = "200%")
										),
										column(
											8,
								            virtualSelectInput(
								              inputId = ns("msigdbr_pw"),
								              label = NULL,
								              choices = NULL, 
								              selected =  NULL,
								              search = TRUE,
								              dropboxWidth = "200%")
										)
									)
								),
								tabPanel("Pathway",
									uiOutput(ns("tab_Pathway"))
								),
								tabPanel("File",
									fluidRow(
										column(8, fileInput(ns("fl_L3_x"),NULL, accept = ".txt")),
										column(3, downloadButton(ns("dw_L3_x"), "e.g."))
									)
								)
							),
							# verbatimTextOutput(ns("L3s_x_tip")),
							verbatimTextOutput(ns("L3s_x_tip")),
							shinyWidgets::actionBttn(
								ns("inspect_data_x"), "Query",
						        style = "gradient",
						        icon = icon("search"),
						        color = "primary",
						        block = TRUE,
						        size = "sm"
							),
							# br(),
							verbatimTextOutput(ns("L3s_x_tip2")),
							# br(),

						),
						column(6,
							style = "height:1100px",
							br(),br(),br(),br(),
							dataTableOutput(ns("L3s_x_data")),
							br(),
							h3("4. Download results"),
						    fluidRow(
						    	column(3, downloadButton(ns("save_csv"), "Download table(.csv)")),
						    	column(3, offset = 2, downloadButton(ns("save_rda"), "Download table(.rda)"))
						    ),
							br(),
							strong(h3("NOTEs:")),
							h5("1. To get the whole dataset, please click 'Respository' page and download derictly from UCSC website."),
							h5("2. Queried data in long format is for easy display and it is downloaded as the wide format. "),	
						)
					)
				)
			),
			column(
				4,
				wellPanel(
					style = "height:1100px",
					h2("Part2: Download other data", align = "center"),
					h3("1. TCGA database"),
					selectInput(ns("tcga_other_type"),NULL,
						choices = c("Basic Phenotype data","Survival data", "Tumor index", "Immune Infiltration", "Pathway activity"),
						selected = c("Basic Phenotype data")
					),
					tabsetPanel(
						id = ns("tcga_other_type_tab"),
						type = "hidden",
						tabPanel("Basic Phenotype data",
							wellPanel(downloadButton(ns("save_tcga_phe"), "Download Basic Phenotype(.csv)",style="width:300px;"))
						),
						tabPanel("Survival data",
							wellPanel(downloadButton(ns("save_tcga_sur"), "Download Survival data(.csv)",style="width:300px;"))
						),
						tabPanel("Tumor index",
							wellPanel(
								fluidRow(downloadButton(ns("save_tcga_idx_purity"), "Download Tumor Purity(.csv)",style="width:300px;")),
								fluidRow(downloadButton(ns("save_tcga_idx_stemness"), "Download Tumor Stemness(.csv)",style="width:300px;"),
									style = "margin-top: 5px; margin-bottom: 0px;width:300px;"),
								fluidRow(downloadButton(ns("save_tcga_idx_tmb"), "Download Tumor Mutation Burden(.csv)",style="width:300px;"),
									style = "margin-top: 5px; margin-bottom: 0px;width:300px;"),
								fluidRow(downloadButton(ns("save_tcga_idx_msi"), "Download Microsatellite Instability(.csv)",style="width:300px;"),
									style = "margin-top: 5px; margin-bottom: 0px;width:300px;"),
								fluidRow(downloadButton(ns("save_tcga_idx_gi"), "Download Genome Instability(.csv)",style="width:300px;"),
									style = "margin-top: 5px; margin-bottom: 0px;width:300px;")
							)

						),
						tabPanel("Immune Infiltration",
							wellPanel(
								fluidRow(downloadButton(ns("save_tcga_til_cib"), "Download CIBERSORT(.csv)",style="width:300px;")),
								fluidRow(downloadButton(ns("save_tcga_til_cib_abs"), "Download CIBERSORT-ABS(.csv)",style="width:300px;"),
									style = "margin-top: 5px; margin-bottom: 0px;"),
								fluidRow(downloadButton(ns("save_tcga_til_epic"), "Download EPIC(.csv)",style="width:300px;"),
									style = "margin-top: 5px; margin-bottom: 0px;width:300px;"),
								fluidRow(downloadButton(ns("save_tcga_til_mcp"), "Download MCPCOUNTER(.csv)",style="width:300px;"),
									style = "margin-top: 5px; margin-bottom: 0px;width:300px;"),
								fluidRow(downloadButton(ns("save_tcga_til_quan"), "Download QUANTISEQ(.csv)",style="width:300px;"),
									style = "margin-top: 5px; margin-bottom: 0px;width:300px;"),
								fluidRow(downloadButton(ns("save_tcga_til_tim"), "Download TIMER(.csv)",style="width:300px;"),
									style = "margin-top: 5px; margin-bottom: 0px;width:300px;"),
								fluidRow(downloadButton(ns("save_tcga_til_xce"), "Download XCELL(.csv)",style="width:300px;"),
									style = "margin-top: 5px; margin-bottom: 0px;width:300px;")
							)

						),
						tabPanel("Pathway activity",
							wellPanel(
								fluidRow(downloadButton(ns("save_tcga_pw_hm"), "Download HALLMARK(.csv)",style="width:300px;")),
								fluidRow(downloadButton(ns("save_tcga_pw_kegg"), "Download KEGG(.csv)",style="width:300px;"),
									style = "margin-top: 5px; margin-bottom: 0px;"),
								fluidRow(downloadButton(ns("save_tcga_pw_iobr"), "Download IOBR(.csv)",style="width:300px;"),
									style = "margin-top: 5px; margin-bottom: 0px;width:300px;")							)
						),
					),

					h3("2. PCAWG database"),
					selectInput(ns("pcawg_other_type"),NULL,
						choices = c("Basic Phenotype data","Survival data", "Tumor index", "Immune Infiltration", "Pathway activity"),
						selected = c("Basic Phenotype data")
					),
					tabsetPanel(
						id = ns("pcawg_other_type_tab"),
						type = "hidden",
						tabPanel("Basic Phenotype data",
							wellPanel(downloadButton(ns("save_pcawg_phe"), "Download Basic Phenotype(.csv)",style="width:300px;"))
						),
						tabPanel("Survival data",
							wellPanel(downloadButton(ns("save_pcawg_sur"), "Download Survival data(.csv)",style="width:300px;"))
						),
						tabPanel("Tumor index",
							wellPanel(
								fluidRow(downloadButton(ns("save_pcawg_idx_purity"), "Download Tumor Purity(.csv)",style="width:300px;"))
							)
						),
						tabPanel("Immune Infiltration",
							wellPanel(
								fluidRow(downloadButton(ns("save_pcawg_til_cib"), "Download CIBERSORT(.csv)",style="width:300px;")),
								fluidRow(downloadButton(ns("save_pcawg_til_cib_abs"), "Download CIBERSORT-ABS(.csv)",style="width:300px;"),
									style = "margin-top: 5px; margin-bottom: 0px;"),
								fluidRow(downloadButton(ns("save_pcawg_til_epic"), "Download EPIC(.csv)",style="width:300px;"),
									style = "margin-top: 5px; margin-bottom: 0px;width:300px;"),
								fluidRow(downloadButton(ns("save_pcawg_til_mcp"), "Download MCPCOUNTER(.csv)",style="width:300px;"),
									style = "margin-top: 5px; margin-bottom: 0px;width:300px;"),
								fluidRow(downloadButton(ns("save_pcawg_til_quan"), "Download QUANTISEQ(.csv)",style="width:300px;"),
									style = "margin-top: 5px; margin-bottom: 0px;width:300px;"),
								fluidRow(downloadButton(ns("save_pcawg_til_tim"), "Download TIMER(.csv)",style="width:300px;"),
									style = "margin-top: 5px; margin-bottom: 0px;width:300px;"),
								fluidRow(downloadButton(ns("save_pcawg_til_xce"), "Download XCELL(.csv)",style="width:300px;"),
									style = "margin-top: 5px; margin-bottom: 0px;width:300px;")
							)

						),
						tabPanel("Pathway activity",
							wellPanel(
								fluidRow(downloadButton(ns("save_pcawg_pw_hm"), "Download HALLMARK(.csv)",style="width:300px;")),
								fluidRow(downloadButton(ns("save_pcawg_pw_kegg"), "Download KEGG(.csv)",style="width:300px;"),
									style = "margin-top: 5px; margin-bottom: 0px;"),
								fluidRow(downloadButton(ns("save_pcawg_pw_iobr"), "Download IOBR(.csv)",style="width:300px;"),
									style = "margin-top: 5px; margin-bottom: 0px;width:300px;")							)
						),
					),
					h3("3. CCLE database"),
					selectInput(ns("ccle_other_type"),NULL,
						choices = c("Basic Phenotype data","Tumor index"),
						selected = c("Basic Phenotype data")
					),
					tabsetPanel(
						id = ns("ccle_other_type_tab"),
						type = "hidden",
						tabPanel("Basic Phenotype data",
							wellPanel(downloadButton(ns("save_ccle_phe"), "Download Basic Phenotype(.csv)",style="width:300px;"))
						),
						tabPanel("Tumor index",
							wellPanel(
								fluidRow(downloadButton(ns("save_ccle_idx_purity"), "Download Tumor Purity(.csv)",style="width:300px;"))
							)
						),
					),


				)

			)
		)
	)
}


server.modules_download_pancan = function(input, output, session, custom_metadata = NULL, opt_pancan = NULL){
	ns = session$ns

	tcga_value_nonomics = load_data("v2_tcga_value_nonomics")
	pcawg_value_nonomics = load_data("v2_pcawg_value_nonomics")
	ccle_value_nonomics = load_data("v2_ccle_value_nonomics")

	id_option = reactive({switch(input$L0, 
					"toil"=tcga_id_option,
					"pcawg"=pcawg_id_option,
					"ccle"=ccle_id_option)})
	id_category = reactive({lapply(id_option(), names)})

	observe({
	  updateTabsetPanel(inputId = "L0_datasets_tab", selected = input$L0)
	  updateTabsetPanel(inputId = "filter_samples2dw_tab", selected = input$L0)
	  updateTabsetPanel(inputId = "data_L2_tab", selected = input$data_L1)
	  updateTabsetPanel(inputId = "data_L3_tab", selected = input$data_L1)
	  updateTabsetPanel(inputId = "L3_x_type_tab", selected = input$L3_x_type)
	  updateTabsetPanel(inputId = "tcga_other_type_tab", selected = input$tcga_other_type)
	  updateTabsetPanel(inputId = "pcawg_other_type_tab", selected = input$pcawg_other_type)
	  updateTabsetPanel(inputId = "ccle_other_type_tab", selected = input$ccle_other_type)
	}) 

	opt_pancan_toil = callModule(mol_origin_Server, "mol_origin2toil", database = "toil")
	opt_pancan_pcawg = callModule(mol_origin_Server, "mol_origin2pcawg", database = "pcawg")
	opt_pancan_ccle = callModule(mol_origin_Server, "mol_origin2ccle", database = "ccle")

	opt_pancan = reactive({
		switch(input$L0,
			`toil`=opt_pancan_toil(),
			`pcawg`=opt_pancan_pcawg(),
			`ccle`=opt_pancan_ccle(),
		)
	})

	sp_exact_toil = callModule(filter_samples_Server, "filter_samples2dw_1",
					   database = "toil",
					   cancers=reactive(input$filter_by_cancer),
					   opt_pancan = reactive(opt_pancan()))
	sp_exact_pcawg = callModule(filter_samples_Server, "filter_samples2dw_2",
					   database = "pcawg",
					   cancers=reactive(input$filter_by_cancer),
					   opt_pancan = reactive(opt_pancan()))
	sp_exact_ccle = callModule(filter_samples_Server, "filter_samples2dw_3",
					   database = "ccle",
					   cancers=reactive(input$filter_by_cancer),
					   opt_pancan = reactive(opt_pancan()))

	filter_samples = reactiveValues(sp=NULL)
	observe({
		# quick filter
		if(input$L0=="toil"){
			sps = tcga_clinical_fine %>%
				dplyr::filter(Cancer %in% input$filter_by_cancer) %>%
				dplyr::filter(Code %in% input$filter_by_code) %>%
				dplyr::pull(Sample) %>% unique() %>% sort()
			if(!is.null(sp_exact_toil())){
				sps = intersect(sps, sp_exact_toil())
			}
		} else if (input$L0=="ccle"){
			sps = pcawg_info_fine %>%
				dplyr::filter(Project %in% input$filter_by_cancer) %>%
				dplyr::filter(Type %in% input$filter_by_code) %>%
				dplyr::pull(Sample) %>% unique() %>% sort()
			if(!is.null(sp_exact_pcawg())){
				sps = intersect(sps, sp_exact_pcawg())
			}
		} else {
			sps = ccle_info_fine %>%
				dplyr::filter(Site_Primary %in% input$filter_by_code)
			if(!is.null(sp_exact_ccle())){
				sps = intersect(sps, sp_exact_ccle())
			}
		} 

		filter_samples$sp = sps
		output$filter_id_info = renderPrint({
			cat(paste0("Tip: ", length(sps), " samples are retained"))
		})
	})


	## select samples
	observe({
		cancer_types = switch(input$L0,
			`toil` = sort(unique(tcga_clinical_fine$Cancer)),
			`pcawg` = sort(unique(pcawg_info_fine$Project))
			# ,
			# `ccle` = sort(unique(ccle_info_fine$Site_Primary))
			)
		updatePickerInput(
			session,
			"filter_by_cancer",
			choices = cancer_types,
			selected =  cancer_types
		)
	})
	observe({
		if(input$L0=="toil"){
			code_types_valid = tcga_clinical_fine %>%
				dplyr::filter(Cancer %in% input$filter_by_cancer) %>%
				dplyr::pull(Code) %>% unique() %>% sort()
		} else if (input$L0=="pcawg"){
			code_types_valid = pcawg_info_fine %>%
				dplyr::filter(Project %in% input$filter_by_cancer) %>%
				dplyr::pull(Type) %>% unique() %>% sort()
		} else (
			code_types_valid = sort(unique(ccle_info_fine$Site_Primary))
		)
		updatePickerInput(
			session,
			"filter_by_code",
			choices = code_types_valid,
			selected =  code_types_valid
		)
	})


	## select ids
	genomic_profile_choices <- reactive({
	  id_option()[["Molecular profile"]][[input$genomic_profile]]
	})

	# update L2 choice
	observe({
	  #L2
	  updateSelectInput(
	  	session,
	    "genomic_profile",
	    choices = names(id_option()[["Molecular profile"]])
	  )
	})

	# update L3 choice
	observe({
	  #L3
	  updateVirtualSelect(
	    "genomic_profile_id",
	    choices = genomic_profile_choices()$all,
	    selected = genomic_profile_choices()$default
	  )
	})

	L2_x = reactive({
	  switch(input$data_L1,
	    `Molecular profile` = input$genomic_profile,
	    `Tumor index` = input$tumor_index,
	    `Immune Infiltration` = input$immune_infiltration,
	    `Pathway activity` = input$pathway_activity,
	    `Phenotype data` = input$phenotype_data
	  )
	})

	msigdbr_query = reactive({
		category = msigdbr_types$gs_cat[msigdbr_types$gs_subcat_label==input$msigdbr_cat]
		subcategory = msigdbr_types$gs_subcat[msigdbr_types$gs_subcat_label==input$msigdbr_cat]
		if(length(category)!=0){
			msigdbr_query = msigdbr(species = "Homo sapiens", 
			                      category = category, 
			                      subcategory = subcategory)
			msigdbr_query
		}
	})

	observe({
		if(!is.null(msigdbr_query())){
			msigdbr_term_stat = as.data.frame(table(msigdbr_query()$gs_name)) %>% 
			  dplyr::rename(term=Var1, size=Freq) %>% 
			  dplyr::arrange(term) %>% 
			  dplyr::mutate(term_size = paste0(term," (", size,")"))
			updateVirtualSelect(
			  "msigdbr_pw",
			  choices = msigdbr_term_stat$term_size,
			  selected = msigdbr_term_stat$term_size[1]
			)
		}
	})

	output$msigdb_note.ui = renderUI({
		# pw_sle = ifelse(is.null(input$msigdbr_pw),"HALLMARK_ADIPOGENESIS",input$msigdbr_pw)
		pw_sle = str_split(input$msigdbr_pw," ")[[1]][1]
		term_link = sprintf("https://www.gsea-msigdb.org/gsea/msigdb/human/%s.html",
                    		ifelse(is.null(pw_sle),"HALLMARK_ADIPOGENESIS",pw_sle))
		msigdb_link = "https://www.gsea-msigdb.org/gsea/msigdb/human/collection_details.jsp"

		h5(strong(HTML('&nbsp;'),HTML('&nbsp;'),HTML('&nbsp;'),"Note: For Molecular profile, select ids in ",
			  a("one pathway", href = term_link)," from ",
			  a("MSigDB database", href = msigdb_link), ":"))

	})

	output$dw_L3_x = downloadHandler(
		filename = function(){
			"sample_multiple_ids.txt"
		},
		content = function(file){
			set.seed(42)
			all_ids = id_option()[[input$data_L1]][[L2_x()]]$all 
			sample_ids = sample(all_ids,ifelse(length(all_ids)>10,10,length(all_ids)))
			write.table(sample_ids, file,
				quote = F, row.names = F, col.names = F)
		}
	)

	L3s_x = reactive({
		if(input$L3_x_type=="Selection"){
			L3s_x = switch(input$data_L1,
				    `Molecular profile` = input$genomic_profile_id,
				    `Tumor index` = input$tumor_index_id,
				    `Immune Infiltration` = input$immune_infiltration_id,
				    `Pathway activity` = input$pathway_activity_id,
				    `Phenotype data` = input$phenotype_data_id
				  )
		} else if (input$L3_x_type=="File"){
			file = input$upload_sp_info
			if(is.null(file$datapath)){  # 如果空文件
				L3s_x = NULL
			} else {
				L3s_x = read.table(file$datapath)[,1]
				L3s_x = L3s_x[L3s_x %in% all_ids]
				if(length(L3s_x)>500 & L2_x() %in% id_category()[["Molecular profile"]]){
					L3s_x = L3s_x[1:500]
				}
			}
		} else if (input$L3_x_type=="All"){
			L3s_x = id_option()[[input$data_L1]][[L2_x()]]$all
			pw_genes = msigdbr_query() %>% 
			  dplyr::filter(gs_name %in% str_split(input$msigdbr_pw," ")[[1]][1]) %>% 
			  dplyr::pull(gene_symbol)
			if(L2_x() %in% 
				c("mRNA Expression","DNA Methylation","Mutation status","Copy Number Variation")){
				L3s_x = L3s_x[L3s_x %in% pw_genes]
			} else if(L2_x() %in% c("Transcript Expression")){
				if(!exists("tcga_id_referrence")){
					message("Loading \"pancan_identifier_help\"")
					tcga_id_referrence = load_data("pancan_identifier_help")
				}
				L3s_x = L3s_x[L3s_x %in% tcga_id_referrence[[1]][[5]]$Level3[tcga_id_referrence[[1]][[5]]$Symbol %in% pw_genes]]
			}
			if(L2_x()=="Custom metadata" & !is.null(custom_metadata)){
				L3s_x = colnames(custom_metadata()[-1])
			}
		} 
		L3s_x
	})
	output$L3s_x_tip = renderPrint({
		cat(paste0("Tip: ",length(L3s_x())," unique ids are selected.\n"))
		# str(L3s_x())
	})
	L3s_x_data = eventReactive(input$inspect_data_x, {
		L1_x = names(id_category())[sapply(id_category(), function(x){any(x %in% L2_x())})]
		withProgress(message = "Please wait for a while...",{
			x_data_merge = lapply(seq(L3s_x()), function(i){
				# 进度提醒
			    incProgress(1 / length(L3s_x()), detail = paste0("(Run ",i,"/",length(L3s_x()),")"))

				L3_x = L3s_x()[i]
				L2_x = L2_x()
				database = input$L0

				if(is.null(opt_pancan)){
					opt_pancan = .opt_pancan
				} else {
					opt_pancan = opt_pancan()
				}

				if(database=="toil"){
					x_data = UCSCXenaShiny:::query_general_value(L1_x, L2_x, L3_x, database,
								   tcga_value_nonomics, opt_pancan,custom_metadata())
				} else if(database=="pcawg"){
					x_data = UCSCXenaShiny:::query_general_value(L1_x, L2_x, L3_x, database,
								   pcawg_value_nonomics, opt_pancan,custom_metadata())
				} else if (database=="ccle"){
					x_data = UCSCXenaShiny:::query_general_value(L1_x, L2_x, L3_x, database,
								   ccle_value_nonomics, opt_pancan,custom_metadata())
				}
				x_data = x_data %>%
					dplyr::arrange(Sample) %>%
					dplyr::filter(Sample %in% filter_samples$sp) %>%
				    dplyr::select(id, Sample, value)
				# 默认批量下载的应为数值型变量，若不是则剔除
				# if(class(x_data$value)=="character"){  
				# 	return(NULL)
				# } else {
				# 	return(x_data)
				# }
				## 提醒用户注意
				return(x_data)
			}) %>% do.call(rbind, .)
			# x_data_merge =  x_data_merge %>%
			# 	tibble::rownames_to_column("Sample")
			x_data_merge
		})
	})
	observeEvent(input$inspect_data_x,{
		shiny::validate(
			need(try(nrow(L3s_x_data())>0), 
				"No sample data were available. Please inspect operations in Preset step."),
		)
		output$L3s_x_tip2 = renderPrint({
			ids_num = length(unique(L3s_x_data()$id))
			cat(paste0("Tip: ", ids_num, " ids are queried successfully!"))
		})
		verbatimTextOutput(ns("L3s_x_tip2"))

		L3s_x_data_ = L3s_x_data()



		if(class(L3s_x_data_[,"value"])=="numeric"){
			L3s_x_data_[,"value"] = round(L3s_x_data_[,"value"], digits = 3)
		}
		output$L3s_x_data = renderDataTable({
			datatable(L3s_x_data_,
				# class = "nowrap row-border",
				options = list(pageLength = 10, 
					columnDefs = list(list(className = 'dt-center', targets="_all")))
			)
		})

	})
	output$save_csv = downloadHandler(
		filename = function(){
			paste0("Batch_query_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv")
		},
		content = function(file){
			L3s_x_data_ = L3s_x_data()
			L3s_x_data_wide = reshape2::dcast(L3s_x_data_, Sample~id, value.var = "value")
			write.csv(L3s_x_data_wide, file, row.names = FALSE)
		}
	)
	output$save_rda = downloadHandler(
		filename = function(){
			paste0("Batch_query_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".rda")
		},
		content = function(file){
			L3s_x_data_ = L3s_x_data()
			L3s_x_data_wide = reshape2::dcast(L3s_x_data_, Sample~id, value.var = "value")
			query_data = L3s_x_data_wide
			save(query_data, file = file)
		}
	)

	## Part two

	output$save_tcga_phe = downloadHandler(
		filename = function(){ paste0("TCGA_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ write.csv(tcga_clinical_fine, file, row.names = FALSE) }
	)
	output$save_tcga_sur = downloadHandler(
		filename = function(){ paste0("TCGA_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ write.csv(tcga_surv, file, row.names = FALSE) }
	)
	output$save_tcga_idx_purity = downloadHandler(
		filename = function(){ paste0("TCGA_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = tcga_value_nonomics %>%
				dplyr::select("Sample", starts_with("Index+Purity"))
			colnames(dat_sub) = gsub("Index\\+Purity\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_tcga_idx_stemness = downloadHandler(
		filename = function(){ paste0("TCGA_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = tcga_value_nonomics %>%
				dplyr::select("Sample", starts_with("Index+Stem"))
			colnames(dat_sub) = gsub("Index\\+Stem\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_tcga_idx_tmb = downloadHandler(
		filename = function(){ paste0("TCGA_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = tcga_value_nonomics %>%
				dplyr::select("Sample", starts_with("Index+TMB"))
			colnames(dat_sub) = gsub("Index\\+TMB\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_tcga_idx_msi = downloadHandler(
		filename = function(){ paste0("TCGA_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = tcga_value_nonomics %>%
				dplyr::select("Sample", starts_with("Index+MSI"))
			colnames(dat_sub) = gsub("Index\\+MSI\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_tcga_idx_gi = downloadHandler(
		filename = function(){ paste0("TCGA_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = tcga_value_nonomics %>%
				dplyr::select("Sample", starts_with("Index+GI"))
			colnames(dat_sub) = gsub("Index\\+GI\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_tcga_til_cib = downloadHandler(
		filename = function(){ paste0("TCGA_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = tcga_value_nonomics %>%
				dplyr::select("Sample", starts_with("Immune+CIB"))
			colnames(dat_sub) = gsub("Immune\\+CIB\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_tcga_til_cib_abs = downloadHandler(
		filename = function(){ paste0("TCGA_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = tcga_value_nonomics %>%
				dplyr::select("Sample", starts_with("Immune+EPIC"))
			colnames(dat_sub) = gsub("Immune\\+CIB\\.ABS\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_tcga_til_epic = downloadHandler(
		filename = function(){ paste0("TCGA_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = tcga_value_nonomics %>%
				dplyr::select("Sample", starts_with("Immune+EPIC"))
			colnames(dat_sub) = gsub("Immune\\+EPIC\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_tcga_til_mcp = downloadHandler(
		filename = function(){ paste0("TCGA_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = tcga_value_nonomics %>%
				dplyr::select("Sample", starts_with("Immune+MCP"))
			colnames(dat_sub) = gsub("Immune\\+MCP\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_tcga_til_quan = downloadHandler(
		filename = function(){ paste0("TCGA_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = tcga_value_nonomics %>%
				dplyr::select("Sample", starts_with("Immune+Quant"))
			colnames(dat_sub) = gsub("Immune\\+Quant\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_tcga_til_tim = downloadHandler(
		filename = function(){ paste0("TCGA_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = tcga_value_nonomics %>%
				dplyr::select("Sample", starts_with("Immune+TIMER"))
			colnames(dat_sub) = gsub("Immune\\+TIMER\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_tcga_til_xce = downloadHandler(
		filename = function(){ paste0("TCGA_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = tcga_value_nonomics %>%
				dplyr::select("Sample", starts_with("Immune+XCELL"))
			colnames(dat_sub) = gsub("Immune\\+XCELL\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_tcga_pw_hm = downloadHandler(
		filename = function(){ paste0("TCGA_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = tcga_value_nonomics %>%
				dplyr::select("Sample", starts_with("Pathway+HM"))
			colnames(dat_sub) = gsub("Pathway\\+HM\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)	
	output$save_tcga_pw_kegg = downloadHandler(
		filename = function(){ paste0("TCGA_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = tcga_value_nonomics %>%
				dplyr::select("Sample", starts_with("Pathway+KEGG"))
			colnames(dat_sub) = gsub("Pathway\\+KEGG\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)	
	output$save_tcga_pw_iobr = downloadHandler(
		filename = function(){ paste0("TCGA_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = tcga_value_nonomics %>%
				dplyr::select("Sample", starts_with("Pathway+IOBR"))
			colnames(dat_sub) = gsub("Pathway\\+IOBR\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)	

	### PCAWG
	output$save_pcawg_phe = downloadHandler(
		filename = function(){ paste0("PCAWG_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ write.csv(pcawg_info_fine, file, row.names = FALSE) }
	)
	output$save_pcawg_sur = downloadHandler(
		filename = function(){ paste0("PCAWG_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			sur_dat_raw = pcawg_info[,c("dcc_project_code","icgc_specimen_id","OS","OS.time")]
			colnames(sur_dat_raw) = c("cancer","Sample","status","time")
			sur_dat_sub = sur_dat_raw %>% dplyr::distinct() 
			write.csv(sur_dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_pcawg_idx_purity = downloadHandler(
		filename = function(){ paste0("PCAWG_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = pcawg_value_nonomics %>%
				dplyr::select("Sample", starts_with("Index+Purity"))
			colnames(dat_sub) = gsub("Index\\+Purity\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_pcawg_til_cib = downloadHandler(
		filename = function(){ paste0("PCAWG_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = pcawg_value_nonomics %>%
				dplyr::select("Sample", starts_with("Immune+CIB"))
			colnames(dat_sub) = gsub("Immune\\+CIB\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_pcawg_til_cib_abs = downloadHandler(
		filename = function(){ paste0("PCAWG_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = pcawg_value_nonomics %>%
				dplyr::select("Sample", starts_with("Immune+CIB.ABS"))
			colnames(dat_sub) = gsub("Immune\\+CIB\\.ABS\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_pcawg_til_epic = downloadHandler(
		filename = function(){ paste0("PCAWG_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = pcawg_value_nonomics %>%
				dplyr::select("Sample", starts_with("Immune+EPIC"))
			colnames(dat_sub) = gsub("Immune\\+EPIC\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_pcawg_til_mcp = downloadHandler(
		filename = function(){ paste0("PCAWG_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = pcawg_value_nonomics %>%
				dplyr::select("Sample", starts_with("Immune+MCP"))
			colnames(dat_sub) = gsub("Immune\\+MCP\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_pcawg_til_quan = downloadHandler(
		filename = function(){ paste0("PCAWG_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = pcawg_value_nonomics %>%
				dplyr::select("Sample", starts_with("Immune+Quant"))
			colnames(dat_sub) = gsub("Immune\\+Quant\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_pcawg_til_tim = downloadHandler(
		filename = function(){ paste0("PCAWG_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = pcawg_value_nonomics %>%
				dplyr::select("Sample", starts_with("Immune+TIMER"))
			colnames(dat_sub) = gsub("Immune\\+TIMER\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_pcawg_til_xce = downloadHandler(
		filename = function(){ paste0("PCAWG_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = pcawg_value_nonomics %>%
				dplyr::select("Sample", starts_with("Immune+XCELL"))
			colnames(dat_sub) = gsub("Immune\\+XCELL\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
	output$save_pcawg_pw_hm = downloadHandler(
		filename = function(){ paste0("PCAWG_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = pcawg_value_nonomics %>%
				dplyr::select("Sample", starts_with("Immune+HM"))
			colnames(dat_sub) = gsub("Immune\\+HM\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)	
	output$save_pcawg_pw_kegg = downloadHandler(
		filename = function(){ paste0("PCAWG_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = pcawg_value_nonomics %>%
				dplyr::select("Sample", starts_with("Immune+KEGG"))
			colnames(dat_sub) = gsub("Immune\\+KEGG\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)	
	output$save_pcawg_pw_iobr = downloadHandler(
		filename = function(){ paste0("PCAWG_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = pcawg_value_nonomics %>%
				dplyr::select("Sample", starts_with("Immune+IOBR"))
			colnames(dat_sub) = gsub("Immune\\+IOBR\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)	

	### CCLE
	output$save_ccle_phe = downloadHandler(
		filename = function(){ paste0("CCLE_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ write.csv(ccle_info_fine, file, row.names = FALSE) }
	)
	output$save_ccle_idx_purity = downloadHandler(
		filename = function(){ paste0("CCLE_metadata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv") },
		content = function(file){ 
			dat_sub = ccle_value_nonomics %>%
				dplyr::select("Sample", starts_with("Index+Purity"))
			colnames(dat_sub) = gsub("Index\\+Purity\\+","",colnames(dat_sub))
			write.csv(dat_sub, file, row.names = FALSE) 
		}
	)
}

ui.page_download <- function() {
	navbarMenu(
	    title = "Download",
	    icon = icon("cloud-arrow-down"),
	    tabPanel(
			"The integrated TPC data",
			ui.modules_download_pancan("modules_download_pancan")
	    ),
	    tabPanel(
			"The Repository Dataset",
			ui.modules_download_dataset("modules_download_dataset")
	    ),
	)
}

ui.modules_1_tcga_09 = function(id){
    ns = NS(id)

    main_ui = tagList(
        mol_quick_select_UI(ns("id"), "tcga", 
            c("mRNA","transcript","methylation","miRNA","protein", "cnv", "mutation")),
        
        h4("3. Select TCGA cancer type and endpoint type"),
        fluidRow(
            column(6,
                selectInput(ns("Cancer"), NULL,sort(tcga_names)),
            ),
            column(6,
                selectInput(ns("endpoint"), NULL,c("OS", "DSS", "DFI", "PFI"))
            )
        ),
        
        h4("4. Filter by clinical features"),
        fluidRow(
            column(6,
                virtualSelectInput(ns("sex"), "Gender:",c("FEMALE", "MALE", "Unknown"),
                            multiple=TRUE, selected = c("FEMALE", "MALE", "Unknown")),
            ),
            column(6,
                virtualSelectInput(ns("stage"), "Tumor stage:",c("I", "II", "III", "IV", "Unknown"),
                            multiple=TRUE, selected = c("I", "II", "III", "IV", "Unknown")),
            ),
        ),
        sliderInput(ns("age"), "Age", min = 0, max = 100, value = c(0, 100)),

        h4("5. Grouping by"),

        awesomeRadio(
            inputId = ns("groupby"),
            label = NULL, 
            choices = c("Median cutoff", "Optimal cutoff"),
            selected = "Median cutoff",
            inline = TRUE, checkbox = TRUE
        ),
        bsTooltip(ns("groupby"), "Only valid for continuous variable, other than molecules from Copy Number Variation/Mutation status", 
                    placement = "right", trigger = "hover", options = list(container = "body")),
        shinyWidgets::actionBttn(
          inputId = ns("submit_bt"), label = "Go!",
          style = "gradient",
          icon = icon("check"),
          color = "primary",
          block = TRUE,
          size = "sm"
        ),
        verbatimTextOutput(ns("msg"))
    )
    out_ui = tagList(
        fluidRow(
            uiOutput(ns("surplot"))
        )
    )
    side_ui = tagList(
        fluidRow(
            column(6,
                h4("1. Visualization parameters"),
                h5("(1) Color palette:"),
                selectInput(ns("palette"), NULL, c("hue", "grey", "RdBu", "Blues", "npg", "aaas"),
                            selected = "aaas")
            ),
            column(6,
                h4("2. Download options"),
                h5("(1) Figure:"),
                numericInput(inputId = ns("height"), label = "Height", value = 8),
                numericInput(inputId = ns("width"), label = "Width", value = 8),
                awesomeRadio(ns("device"), label = "Format", 
                    choices = c("pdf", "png"), selected = "pdf", inline = TRUE),
                downloadBttn(
                  outputId = ns("download_1"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                ),
                h5("(2) Data table:"),
                downloadBttn(
                  outputId = ns("download_2"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                )
            )
        )
    )
    fluidPage(
        wellPanel(style = "height:795px",
        box(main_ui,
            width = 5,
            solidHeader = TRUE,
            title = "Quick TCGA Analysis: Kaplan-Meier survival analysis(Log-rank)", 
            status = "info",
            background = "gray",
            collapsible = FALSE,
            style = "height:670px",
            footer = "TIPs: Click the bottom button to execute/update the analysis."
        ),
        box(out_ui,
            width = 7,
            solidHeader = TRUE,
            title = "Analytical results:", 
            status = "info",
            background = "gray",
            collapsible = FALSE,
            style = "height:670px",
            footer = "TIPs: Pull the sidebar to adjust plot parameters or download results through the top-right widget.",
            sidebar = boxSidebar(
                        id = ns("sidebar"),
                        width = 50,
                        side_ui
            )
        )
        )
    )
}

server.modules_1_tcga_09 = function(input, output, session){
    ns = session$ns

    ## Data filter
    dat_filter <- function(data, age, gender, stage, endpoint) {
        endpoint.time <- paste0(endpoint, ".time")
        dat <- data %>%
            dplyr::rename(time = !!endpoint.time, status = !!endpoint) %>%
            dplyr::filter(
            age >= !!age[1],
            age <= !!age[2],
            gender %in% !!gender,
            stage %in% !!stage,
            !is.na(time),
            !is.na(status)
            )
        message("cases after filtering: ", nrow(dat))
        return(dat)
    }

    mol_info = callModule(mol_quick_select_Server, "id", "tcga")

    sur_dat = eventReactive(input$submit_bt,{
        TCGA_cli_merged <- dplyr::full_join(
            load_data("tcga_clinical"),
            load_data("tcga_surv"),
            by = "sample"
        )

        dat1 = tcga_surv_get(
            TCGA_cohort = input$Cancer, item = mol_info$molecule(),
            profile = mol_info$profile(), TCGA_cli_data = TCGA_cli_merged
        )
        if(is.null(dat1)){dat1 = data.frame(
                                    sampleID=NA, value=NA, 
                                    OS=NA, OS.time=NA, 
                                    DSS=NA, DSS.time=NA, 
                                    DFI=NA, DFI.time=NA, 
                                    PFI=NA, PFI.time=NA, 
                                    gender=NA, age=NA, stage=NA)
        } 
        # filter
        dat2 = dat_filter(data = dat1, age = input$age,
                          gender = input$sex, stage = input$stage, 
                          endpoint = input$endpoint)
        dat2
    })


    tips = eventReactive(input$submit_bt, {
		shiny::validate(
			need(try(nrow(sur_dat())>0), 
				"Error: Please select a valid molecule.")
		)
        if(mol_info$profile() == "mutation"){
            sur_dat2 = sur_dat() %>%
                    dplyr::mutate(group = value)
            t1 = "variation status"
        } else if (mol_info$profile() == "cnv") {
            sur_dat2 = sur_dat() %>%
                    dplyr::mutate(group = dplyr::case_when(
                    .data$value > 0 ~ "Duplicated",
                    .data$value < 0 ~ "Deleted",
                    TRUE ~ "Normal"
                ))
            t1 = "variation status"
        } else {
            sur_dat2 = sur_dat() %>%
                dplyr::arrange(.data$value) %>%
                dplyr::mutate(per_rank = 100 / nrow(.) * (1:nrow(.))) %>%
                dplyr::mutate(group = dplyr::case_when(
                    .data$per_rank > !!50 ~ "High",
                    .data$per_rank <= !!50 ~ "Low",
                    TRUE ~ NA_character_
                ))
            t1 = ifelse(input$groupby=="Optimal cutoff","optimal cutoff","median value")   
        }
        chech_dat = sur_dat2  %>% 
                dplyr::count(group) %>% 
                dplyr::filter(n>3) # the number of groups > 3 samples
        if(nrow(chech_dat)<=1){
            msg = "Warning: Please adjust above input for valid sample grouping."
            sendSweetAlert(session, title = "Warning", text = "No more than two groups (> 3 samples) are available!")
        } else {
            msg = paste0("Note: ", nrow(sur_dat()), " samples are grouped by ", 
                         t1, " of ", mol_info$molecule(), " ", mol_info$profile(), ".")
        }
        msg
    })

    output$msg = renderPrint({cat(tips())})

    plot_func = eventReactive(input$submit_bt, {
        req(grep("Note", tips()))
        id <- showNotification(h3("The task is running..."), duration = NULL, closeButton = FALSE, type = "message")
        on.exit(removeNotification(id), add = TRUE)  #reactive语句执行完毕时，运行remove命令
        cutoff_mode = ifelse(input$groupby=="Optimal cutoff","Auto","Custom")

        p <- tcga_surv_plot(sur_dat(),
                            cutoff_mode = cutoff_mode, #"Custom",
                            cutpoint = c(50, 50),
                            profile = mol_info$profile(),
                            palette = input$palette #"aaas"
        )
        p
    })

    # Show waiter for plot
    w <- waiter::Waiter$new(id = ns("surplot"), html = waiter::spin_hexdots(), color = "black")
    observeEvent(input$submit_bt,{
        shinyjs::disable("submit_bt")

        output$surplot <- renderUI({
            w$show()
            output$plot = renderPlot(plot_func())
            fluidRow(
                column(10, offset = 1,
                    plotOutput(ns("plot"), height = "650px"),
                )
            )
        })   
        shinyjs::enable("submit_bt")
    })

    output$download_1 <- downloadHandler(
        filename = function() {
            paste0(mol_info$molecule(), "_", mol_info$profile(), "_tcga_surplot.", input$device)
        },
        content = function(file) {
            p <- plot_func()
            if (input$device == "pdf") {
                pdf(file, width = input$width, height = input$height, onefile = FALSE)
                print(p)
                dev.off()
            } else {
                png(file, width = input$width, height = input$height, res = 600, units = "in")
                print(p)
                dev.off()
            }
        }
    )

    output$download_2 <- downloadHandler(
        filename = function() {
            paste0(mol_info$molecule(), "_", mol_info$profile(), "_tcga_surplot.csv")
        },
        content = function(file) {
            data = sur_dat() %>%
                dplyr::rename('Sample'='sampleID','Value'='value',
                'Status'='status', 'Time'='time') %>%
                dplyr::mutate(Cancer = input$Cancer,Event = input$endpoint) %>%
                dplyr::select(Cancer, Sample, Event, Status, Time, Value)
            write.csv(data, file, row.names = FALSE)
        }
    )
}

xx_mod_ui <- function(id) {
  ns <- NS(id)

  tagList(
    sidebarPanel(
      selectInput(ns("xcol"), "X Variable", names(iris)),
      selectInput(ns("ycol"), "Y Variable", names(iris),
        selected = names(iris)[[2]]
      ),
      numericInput(ns("clusters"), "Cluster count", 3,
        min = 1, max = 9
      )
    ),
    mainPanel(
      plotOutput(ns("plot1"))
    )
  )
}

# xx_mod <- function(input, output, session){
#   output$plot1 <- renderPlot({
#     plot(iris)
#   })
# }

xx_mod_server <- function(input, output, session) {
  # Combine the selected variables into a new data frame
  selectedData <- reactive({
    iris[, c(input$xcol, input$ycol)]
  })

  clusters <- reactive({
    kmeans(selectedData(), input$clusters)
  })

  output$plot1 <- renderPlot({
    palette(c(
      "#E41A1C", "#377EB8", "#4DAF4A", "#984EA3",
      "#FF7F00", "#FFFF33", "#A65628", "#F781BF", "#999999"
    ))

    par(mar = c(5.1, 4.1, 0, 1))
    plot(selectedData(),
      col = clusters()$cluster,
      pch = 20, cex = 3
    )
    points(clusters()$centers, pch = 4, cex = 4, lwd = 4)
  })
}

ui.modules_pancan_cor_o2o = function(id) {
	ns = NS(id)
	fluidPage(
		fluidRow(
			# 初始设置
			column(
				3,
				wellPanel(
					style = "height:1100px",
					h2("S1: Preset", align = "center"),

					h4(strong("S1.1 Modify datasets"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Modify datasets", 
					                   content = "data_origin"),
					mol_origin_UI(ns("mol_origin2cor"), database = "toil"),

					h4(strong("S1.2 Choose cancer")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Cancer types", 
					                   content = "tcga_types"),
					pickerInput(
						ns("choose_cancer"), NULL,
						choices = sort(tcga_names)),
				    br(),

					h4(strong("S1.3 Filter samples"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Filter samples", 
					                   content = "choose_samples"),
					h5("Quick filter:"),
					pickerInput(
						ns("filter_by_code"), NULL,
						choices = NULL, selected =  NULL,
						multiple = TRUE, options = list(`actions-box` = TRUE)
					),
					h5("Exact filter:"),
					filter_samples_UI(ns("filter_samples2cor"), database = "toil"),
					br(),
					verbatimTextOutput(ns("filter_phe_id_info")),
					br(),

					h4(strong("S1.4 Upload metadata"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Upload metadata", 
					                   content = "custom_metadata"),
					shinyFeedback::useShinyFeedback(),
					custom_meta_UI(ns("custom_meta2cor")),
					br(),

					h4(strong("S1.5 Add signature"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Add signature", 
					                   content = "add_signature"),
					add_signature_UI(ns("add_signature2cor"), database = "toil"),
					br(),

				)
			),
			# 下载X/Y轴数据
			column(
				4,
				wellPanel(
					style = "height:1100px",
					h2("S2: Get data", align = "center"),
					# X
					h4(strong("S2.1 Get data for X-axis")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Get one data", 
					                   content = "get_one_data"), 
					download_feat_UI(ns("download_x_axis"), 
						button_name="Query", database = "toil"),
		            # br(),
		            # Y
		            h4(strong("S2.2 Get data for Y-axis")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Get one data", 
					                   content = "get_one_data"),  
					download_feat_UI(ns("download_y_axis"), 
						button_name="Query", database = "toil")
				)
			),
			# 分析/绘图/下载
			column(
				5,
				wellPanel(
					h2("S3: Analyze & Visualize", align = "center") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Analyze & Visualize", 
					                   content = "analyze_cor_1"),  
					style = "height:1100px",
					h4(strong("S3.1 Set analysis parameters")), 
					selectInput(ns("cor_method"), "Correlation method:",choices = c("Spearman", "Pearson"),selected="Spearman"),
					h4(strong("S3.2 Set visualization parameters")), 
					fluidRow(
						column(3, colourpicker::colourInput(inputId = ns("line_color"), "Line color:", "#0000FF")),
						column(3, colourpicker::colourInput(inputId = ns("x_hist_color"), "Hist color(x):", "#009E73")),
						column(3, colourpicker::colourInput(inputId = ns("y_hist_color"), "Hist color(y):", "#D55E00"))
					),
					dropMenu(
						actionBttn(ns("more_visu"), label = "Other options", style = "bordered",color = "success",icon = icon("bars")),
						div(h3("1. Select ggplot theme:"),style="width:400px;"),
						fluidRow(
							column(6,
								selectInput(inputId = ns("theme"), label = NULL, 
											choices = names(themes_list), selected = "ggstatplot")
							)
						),
						div(h3("2. Adjust points:"),style="width:400px;"),
						fluidRow(
							column(4, numericInput(inputId = ns("point_size"), label = "Point size:", value = 3, step = 0.5)),
							column(3, numericInput(inputId = ns("point_alpha"), label = "Point alpha:", value = 0.4, step = 0.1, min = 0, max = 1))
						),
						div(h3("3. Adjust text size:"),style="width:400px;"),
						fluidRow(
							column(4, numericInput(inputId = ns("axis_size"), label = "Text size:", value = 18, step = 0.5)),
							column(4, numericInput(inputId = ns("title_size"), label = "Title size:", value = 20, step = 0.5))
						),				
						div(h3("4. Adjust lab and title name:"),style="width:400px;"),
						fluidRow(
							column(4, textInput(inputId = ns("x_name"), label = "X-axis name:")),
							column(4, textInput(inputId = ns("y_name"), label = "Y-axis name:")),
							column(4, textInput(inputId = ns("title_name"), label = "Title name:"))
						),	
						div(h3("5. Display the histogram or not:"),style="width:400px;"),
						fluidRow(
							column(6, radioButtons(inputId = ns("side_hist"), label = NULL, 
								choices = c("NO", "YES"), selected="YES",inline = TRUE)),
						),	
						div(h5("Note: You can download the raw data and plot in local R environment for more detailed adjustment.")),
					),
					br(),
					shinyWidgets::actionBttn(
						ns("step3_plot_sct"), "Run",
				        style = "gradient",
				        icon = icon("chart-line"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					),
					br(),
					fluidRow(
						column(10, offset = 1,
							   plotOutput({ns("cor_plot_sct")}, height = "480px") 
						)
					),
					br(),
					h4(strong("S3.3 Download results")), 
					download_res_UI(ns("download_res2cor"))
				)
			)
		)
	)
}


server.modules_pancan_cor_o2o = function(input, output, session) {
	ns <- session$ns

	# 记录选择癌症
	cancer_choose <- reactiveValues(name = "ACC", phe_primary="",
		filter_phe_id=query_tcga_group(database = "toil",cancer = "BRCA", return_all = T))
	observe({
		cancer_choose$name = input$choose_cancer
		cancer_choose$phe_primary <- query_tcga_group(database = "toil",cancer = cancer_choose$name, return_all = T)
	})

	# 自定义上传metadata数据
	custom_meta = callModule(custom_meta_Server, "custom_meta2cor", database = "toil")
	# # signature
	sig_dat = callModule(add_signature_Server, "add_signature2cor", database = "toil")
	# sig_dat = reactive({NULL})
	custom_meta_sig = reactive({
		if(is.null(custom_meta())){
			return(sig_dat())
		} else {
			if(is.null(sig_dat())){
				return(custom_meta())
			} else {
				custom_meta_sig = dplyr::inner_join(custom_meta(),sig_dat())
				return(custom_meta_sig)
			}
		}
	})

	# 数据源设置
	opt_pancan = callModule(mol_origin_Server, "mol_origin2cor", database = "toil")

	## 过滤样本
	# exact filter module
	filter_phe_id = callModule(filter_samples_Server, "filter_samples2cor",
					   database = "toil",
					   cancers=reactive(cancer_choose$name),
					   custom_metadata=reactive(custom_meta_sig()),
					   opt_pancan = reactive(opt_pancan()))

	# filter_phe_id = reactive({NULL})
	# quick filter widget
	observe({
		code_types_valid = code_types[names(code_types) %in% 
							unique(cancer_choose$phe_primary$Code)]
		updatePickerInput(
			session,
			"filter_by_code",
			choices = unlist(code_types_valid,use.names = F),
			selected =  unlist(code_types_valid,use.names = F)
		)
	})

	# 综合上述二者
	observe({
		# quick filter
		choose_codes = names(code_types)[unlist(code_types) %in% input$filter_by_code]
		filter_phe_id2 = cancer_choose$phe_primary %>%
			dplyr::filter(Code %in% choose_codes) %>%
			dplyr::pull("Sample")

		# exact filter
		if(is.null(filter_phe_id())){
			cancer_choose$filter_phe_id = filter_phe_id2
		} else {
			cancer_choose$filter_phe_id = intersect(filter_phe_id2,filter_phe_id())
		}

		output$filter_phe_id_info = renderPrint({
			cat(paste0("Tip: ", length(cancer_choose$filter_phe_id), " samples are retained"))
		})
	})


	## x-axis panel
	x_axis_data = callModule(download_feat_Server, "download_x_axis", 
							 database = "toil",
							 samples=reactive(cancer_choose$filter_phe_id),
							 custom_metadata=reactive(custom_meta_sig()),
						     opt_pancan = reactive(opt_pancan()),
						     check_numeric=TRUE
							 )

	## y-axis panel
	y_axis_data = callModule(download_feat_Server, "download_y_axis", 
							 database = "toil",
							 samples=reactive(cancer_choose$filter_phe_id),
							 custom_metadata=reactive(custom_meta_sig()),
						     opt_pancan = reactive(opt_pancan()),
						     check_numeric=TRUE
							 )

	# 合并分析
	# scatterplot逻辑：先绘图，再提取相关性结果
	merge_data_sct = eventReactive(input$step3_plot_sct, {
		x_axis_data = x_axis_data()
		colnames(x_axis_data)[c(1:3,5)] = paste0("x_",colnames(x_axis_data)[c(1:3,5)])
		y_axis_data = y_axis_data()
		colnames(y_axis_data)[c(1:3,5)] = paste0("y_",colnames(y_axis_data)[c(1:3,5)])

		data = dplyr::inner_join(x_axis_data, y_axis_data) %>%
			dplyr::select(cancer, Sample, everything())
		data
	})

	observe({
		updateTextInput(session, "x_name", value = unique(x_axis_data()$id))
		updateTextInput(session, "y_name", value = unique(y_axis_data()$id))
		updateTextInput(session, "title_name", value = cancer_choose$name)
	})

	cor_plot_sct = eventReactive(input$step3_plot_sct, {
		shiny::validate(
			need(try(nrow(merge_data_sct())>0), 
				"Please inspect whether to get valid data in Step2."),
			need(try(nrow(merge_data_sct())>2), 
				"Please adjust to ensure that enough samples (n≥3) are included for analysis.")
		)
		p = plot_cor_o2o(
			data = merge_data_sct(), cor_method = input$cor_method,
			xlab = input$x_name, ylab = input$y_name, title = input$title_name, 
			point.args = list(size = input$point_size, alpha = input$point_alpha),
			smooth.line.args = list(color = input$line_color,linewidth = 1.5,method = "lm",formula = y ~ x),
			xsidehistogram.args = list(fill = input$x_hist_color, color = "black", na.rm = TRUE),
			ysidehistogram.args = list(fill = input$y_hist_color, color = "black", na.rm = TRUE),
			axis_size = input$axis_size, title_size = input$title_size, side_hist = input$side_hist,
			custom_theme = themes_list[[input$theme]]
		)
		return(p)
	})
	output$cor_plot_sct = renderPlot({cor_plot_sct()})

	# Download results
	observeEvent(input$step3_plot_sct,{
		res1 = cor_plot_sct()
		res2 = merge_data_sct()
		p_cor = extract_stats(cor_plot_sct())$subtitle_data
		p_cor = p_cor[-which(colnames(p_cor)=="expression")]
		p_cor$parameter1 = unique(merge_data_sct()$x_axis)
		p_cor$parameter2 = unique(merge_data_sct()$y_axis)
		p_cor = p_cor %>% dplyr::mutate(cancer = cancer_choose$name, .before=1)
		res3 = p_cor

		callModule(download_res_Server, "download_res2cor", res1, res2, res3)
	})
}

ui.footer <- function() {
  tags$footer(tags$hr(),
    HTML("Copyright &copy; 2019 - 2024 @"),
    tags$a(href = "https://github.com/openbiox", "Openbiox"),
    tags$br(),
    HTML("A community-driven bioinformatics innovation collaboration group in China |"),
    tags$a(href = "https://github.com/openbiox/XenaShiny/blob/master/LICENSE", "GPLv3 License"),
    HTML("<br><a href='https://clustrmaps.com/site/1c2kd'  title='Visit tracker'><img src='//clustrmaps.com/map_v2.png?cl=ffffff&w=a&t=tt&d=09ckOL1huqv48licSGJ9K7oM8wHMiOO6WxNZDLesR10'/></a>"),
    align = "center", style = "
                           position:relative;
                           bottom:0;
                           width:100%;
                           height:50px;   /* Height of the footer */
                           padding: 10px;
                           z-index: 1000;"
  )
}

ui.modules_download_dataset = function(id){
	ns = NS(id)
	fluidPage(
		fluidRow(
			column(
				4,
				wellPanel(
					style = "height:1000px",
					h3("1. Select one data hub"),
					selectInput(ns("one_datahub"),NULL,
						choices = sort(unique(XenaData$XenaHostNames)),
						selected = "toilHub"),
					uiOutput(ns("hub_link")),
					# br(),
					h3("2. Select one dataset"),
					fluidRow(
						column(
							6,
							h4("(1) Format type"),
							selectInput(ns("format_type"), NULL,
								choices = NULL, selected = NULL)
						),
						column(
							6,
							h4("(2) Profile type"),
							selectInput(ns("profile_type"), NULL,
								choices = NULL, selected = NULL)
						),
					),
					virtualSelectInput(ns("one_dataset"), NULL,choices=NULL,selected=NULL,
					                   search = TRUE,
					                   allowNewOption = FALSE,
					                   dropboxWidth = "200%"),
					uiOutput(ns("dataset_link")),
					# br(),
					h3("3. Select multiple ids"),

					# h4("(1) Load ids"),
					fluidRow(
						column(
							4,
							actionButton(ns("load_id"), "Load", icon = icon("circle-down"),
								style="color: #fff; background-color: #337ab7; border-color: #2e6da4")
						),
						column(
							8,
							radioGroupButtons(
							   inputId = ns("id_type"),
							   label = NULL,
							   choices = "NULL",
							   justified = TRUE
							)
						)
					),

					tabsetPanel(
						id = ns("upload_type"),
						tabPanel("Selection",
							uiOutput(ns("multi_ids.ui")) %>% withSpinner(),
							# virtualSelectInput(
							# 	inputId = ns("multi_ids"),
					  #           label = "Identifier:",
					  #           choices = NULL,
					  #           multiple = TRUE,
					  #           search = TRUE,
					  #           allowNewOption = FALSE,
					  #           dropboxWidth = "200%")
						),
						tabPanel("File",
							fluidRow(
								column(9, fileInput(ns("fl_ids"),NULL, accept = ".txt",
									placeholder = "Please upload an id list(.txt)")),
								column(3, downloadButton(ns("fl_eg"), "e.g."))
							),
						),
					),

					verbatimTextOutput(ns("query_tip1")),
					shinyWidgets::actionBttn(
						ns("inspect_data_x"), "Query",
				        style = "gradient",
				        icon = icon("search"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					),
					br(),
					verbatimTextOutput(ns("query_tip2")),
					h3("3. Download results"),
				    fluidRow(
				    	column(3, downloadButton(ns("save_csv"), "Download table(.csv)")),
				    	column(3, offset = 2, downloadButton(ns("save_rda"), "Download table(.rda)"))
				    ),
				)
			),
			column(
				5,
				offset = 1,
				br(),br(),br(),br(),
				dataTableOutput(ns("query_data_df")),
				br(),br(),
				strong(h3("NOTEs:")),
				h5("1. To get the whole dataset, please click 'Respository' page and download derictly from UCSC website."),
				h5("2. Queried data in long format is for easy display and it is downloaded as the wide format. "),
			)

		)

	)

}



server.modules_download_dataset = function(input, output, session){
	ns = session$ns


	output$hub_link = renderUI({
		link = unique(XenaData$XenaHosts[XenaData$XenaHostNames==input$one_datahub])
		p("See more info about the hub from Respository page or",
			a("Xena site.",href = link))

	})
	output$dataset_link = renderUI({
		hub_link = unique(XenaData$XenaHosts[XenaData$XenaHostNames==input$one_datahub])
		set_link = paste0("https://xenabrowser.net/datapages/?dataset=",
						  input$one_dataset, "&host=", hub_link)
		p("See more info about the dataset from Respository page or",
			a("Xena site.",href = set_link))

	})

	observe({
		format_types = sort(unique(XenaData$Type[XenaData$XenaHostNames==input$one_datahub]))
	    updateSelectInput(
	      session,
	      "format_type",
	      choices = format_types,
	      selected = "genomicMatrix"
	    )
	})

	observe({
	    profile_types = sort(unique(XenaData$DataSubtype[
	    							XenaData$XenaHostNames==input$one_datahub & 
	    							XenaData$Type==input$format_type]))
	    updateSelectInput(
	      session,
	      "profile_type",
	      choices = profile_types,
	      selected = profile_types[1]
	    )
	})

	observe({
	    datasets = sort(unique(XenaData$XenaDatasets[
	    							XenaData$XenaHostNames==input$one_datahub & 
	    							XenaData$Type==input$format_type &
	    							XenaData$DataSubtype==input$profile_type]))
	    updateVirtualSelect(
	      "one_dataset",
	      choices = datasets,
	      selected = datasets[1]
	    )
	})


	observe({
		hub_link = unique(XenaData$XenaHosts[XenaData$XenaHostNames==input$one_datahub])
		url <- has_probeMap(host = hub_link,dataset = input$one_dataset)
		if(length(url)!=0){
			if(url){
				choices = c("Primary ID", "ProbeMap ID")
			} else {
				choices = c("Primary ID")
			}
			updateRadioGroupButtons(
				session,
				"id_type",
				choices = choices,
				selected = "Primary ID"
			)
		}

	})

	candi_ids = eventReactive(input$load_id,{
		hub_link = unique(XenaData$XenaHosts[XenaData$XenaHostNames==input$one_datahub])
		url <- has_probeMap(host = hub_link,dataset = input$one_dataset, return_url = T)
		if(input$id_type=="Primary ID"){
			candi_ids = fetch_dataset_identifiers(host = hub_link,dataset = input$one_dataset)
		} else {
			candi_ids <- UCSCXenaTools:::use_cache(url, op = "readr::read_tsv(url,\n            col_types = readr::cols()\n        )[[2]]")
		}
		candi_ids
	})


	output$multi_ids.ui = renderUI({
		input$load_id
		Sys.sleep(1.5)
		virtualSelectInput(
			inputId = ns("multi_ids"),
            label = "Identifier:",
            choices = NULL,
            multiple = TRUE,
            search = TRUE,
            allowNewOption = FALSE,
            dropboxWidth = "200%")

	})


	observe({
	    updateVirtualSelect(
	      "multi_ids",
	      choices = sort(candi_ids()),
	      selected = sort(candi_ids())[1]
	    )	
	})

	output$fl_eg = downloadHandler(
		filename = function(){
			"sample_multiple_ids.txt"
		},
		content = function(file){
			set.seed(42)
			sample_ids = sample(candi_ids(),ifelse(length(candi_ids())>10,10,length(candi_ids())))
			write.table(sample_ids, file,
				quote = F, row.names = F, col.names = F)
		}
	)


	query_ids = reactive({
		shiny::validate(
			need(try(length(candi_ids())>0), 
				"Please load ids firstly!")
		)
		if(input$upload_type=="Selection"){
			query_ids = unique(input$multi_ids)
		} else {
			file = input$fl_ids
			if(is.null(file$datapath)){  # 如果空文件
				query_ids = NULL
			} else {
				query_ids = read.table(file$datapath)[,1]
				query_ids = query_ids[query_ids %in% candi_ids()]
				if(length(query_ids)>500){
					query_ids = query_ids[1:500]
				}
			}
		}
		query_ids
		
	})


	output$query_tip1 = renderPrint({
		ids_num = length(query_ids())
		cat(paste0("Tip: ", ids_num, " unique ids are selected."))
	})
	query_data = eventReactive(input$inspect_data_x, {
		withProgress(message = "Please wait for a while.",{
			data = lapply(seq(query_ids()), function(i){
				id = query_ids()[i]
				data_tmp = UCSCXenaShiny:::get_data(dataset = input$one_dataset,
				                    identifier = id,
				                    host = input$one_datahub)
				# 进度提醒
			    incProgress(1 / length(query_ids()), detail = paste0("(Run ",i,"/",length(query_ids()),")"))

				return(data_tmp)
			}) %>% do.call(cbind, .) %>% as.data.frame()
			colnames(data) = query_ids()
			data = data[,apply(data, 2, function(x){!all(is.na(x))}),drop=FALSE]
			data = data %>%
				tibble::rownames_to_column("Sample")
			data
		})
	})

	observeEvent(input$inspect_data_x,{
		shiny::validate(
			need(try(nrow(query_data())>0), 
				"No sample data were available. Please inspect operations in Preset step."),
		)
		output$query_tip2 = renderPrint({
			ids_num = ncol(query_data())-1
			cat(paste0("Tip: ", ids_num, " ids are queried successfully!"))
		})
		verbatimTextOutput(ns("query_tip2"))

		query_data_ = query_data() %>%
						reshape2::melt("Sample") %>% 
						dplyr::rename("id"="variable")

		output$query_data_df = renderDataTable({
			datatable(query_data_[,c("id","Sample","value")],
				# class = "nowrap row-border",
				options = list(pageLength = 10, 
					columnDefs = list(list(className = 'dt-center', targets="_all")))
			)
		})
	})

	output$save_csv = downloadHandler(
		filename = function(){
			paste0("Batch_query_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv")
		},
		content = function(file){
			write.csv(query_data(), file, row.names = FALSE)
		}
	)
	output$save_rda = downloadHandler(
		filename = function(){
			paste0("Batch_query_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".rda")
		},
		content = function(file){
			query_data = query_data()
			save(query_data, file = file)
		}
	)

}

ui.page_global <- function() {
  tabPanel(
    title = "Global Setting",
    icon = icon("cogs"),
    fluidPage(
      h3("Global setting:"),
      switchInput("hiplot_switch", label = "Use hiplot mirror to query data?", labelWidth = "200px", inline = T),
      p(style = "height:500px"),
    )
  )
}

html_spin = tagList(spin_1(), br(),
                    h2("Loading independent modules (Only needed for the first time)"))

# # quick module
observeEvent(req(input$navbar=="TCGA (GTEx): Molecular comparison"),{
  callModule(server.modules_1_tcga_01, "modules_1_tcga_01")
  callModule(server.modules_1_tcga_02, "modules_1_tcga_02")
  callModule(server.modules_1_tcga_08, "modules_1_tcga_08")
}, once = TRUE)  


observeEvent(req(input$navbar=="TCGA: Molecular correlation"),{
  callModule(server.modules_1_tcga_03, "modules_1_tcga_03")
  callModule(server.modules_1_tcga_04, "modules_1_tcga_04")
  callModule(server.modules_1_tcga_05, "modules_1_tcga_05")
  callModule(server.modules_1_tcga_06, "modules_1_tcga_06")
  callModule(server.modules_1_tcga_07, "modules_1_tcga_07")
}, once = TRUE)  

observeEvent(req(input$navbar=="TCGA: Survival analysis"),{
  callModule(server.modules_1_tcga_09, "modules_1_tcga_09")
  callModule(server.modules_1_tcga_10, "modules_1_tcga_10")
}, once = TRUE)  

observeEvent(req(input$navbar=="TCGA: Dimensionality reduction"),{
  callModule(server.modules_1_tcga_11, "modules_1_tcga_11")
}, once = TRUE)  

# observeEvent(req(input$navbar=="TCGA: Pathway cross analysis"),{
#   callModule(server.modules_1_tcga_12, "modules_1_tcga_12")
# }, once = TRUE)  

observeEvent(req(input$navbar=="PCAWG: Molecular comparison"),{
  callModule(server.modules_2_pcawg_01, "modules_2_pcawg_01")
}, once = TRUE)  

observeEvent(req(input$navbar=="PCAWG: Molecular correlation"),{
  callModule(server.modules_2_pcawg_02, "modules_2_pcawg_02")
}, once = TRUE)  

observeEvent(req(input$navbar=="PCAWG: Survival analysis"),{
  callModule(server.modules_2_pcawg_03, "modules_2_pcawg_03")
  callModule(server.modules_2_pcawg_04, "modules_2_pcawg_04")
}, once = TRUE)  


observeEvent(req(input$navbar=="CCLE: Molecular comparison"),{
  callModule(server.modules_3_ccle_01, "modules_3_ccle_01")
}, once = TRUE)  

observeEvent(req(input$navbar=="CCLE: Molecular correlation"),{
  callModule(server.modules_3_ccle_02, "modules_3_ccle_02")
}, once = TRUE)   

observeEvent(req(input$navbar=="CCLE: Drug analysis"),{
  waiter <- waiter::Waiter$new(color = "grey", fadeout = TRUE, html = html_spin)
  waiter$show()
  callModule(server.modules_3_ccle_03, "modules_3_ccle_03")
  callModule(server.modules_3_ccle_04, "modules_3_ccle_04")
  on.exit(waiter$hide())
}, once = TRUE)  



## TPC 

# TCGA pancan
observeEvent(req(input$navbar=="TCGA: Correlation Analysis"),{
    waiter <- waiter::Waiter$new(color = "grey", fadeout = TRUE, html = html_spin)
    waiter$show()
		callModule(server.modules_pancan_cor_o2o, "modules_pancan_cor_o2o")
		callModule(server.modules_pancan_cor_o2m, "modules_pancan_cor_o2m")
		callModule(server.modules_pancan_cor_m2o, "modules_pancan_cor_m2o")
    on.exit(waiter$hide())
}, once = TRUE)  

observeEvent(req(input$navbar=="TCGA: Comparison Analysis"),{
    waiter <- waiter::Waiter$new(color = "grey", fadeout = TRUE, html = html_spin)
    waiter$show()
		callModule(server.modules_pancan_comp_o2o, "modules_pancan_comp_o2o")
		callModule(server.modules_pancan_comp_o2m, "modules_pancan_comp_o2m")
		callModule(server.modules_pancan_comp_m2o, "modules_pancan_comp_m2o")
    on.exit(waiter$hide())
}, once = TRUE)  

observeEvent(req(input$navbar=="TCGA: Survival Analysis"),{
    waiter <- waiter::Waiter$new(color = "grey", fadeout = TRUE, html = html_spin)
    waiter$show()
		callModule(server.modules_pancan_sur_o2o, "modules_pancan_sur_o2o")
		callModule(server.modules_pancan_sur_o2m, "modules_pancan_sur_o2m")
		callModule(server.modules_pancan_sur_m2o, "modules_pancan_sur_m2o")
    on.exit(waiter$hide())
}, once = TRUE)  

observeEvent(req(input$navbar=="TCGA: Cross-Omics Analysis"),{
    waiter <- waiter::Waiter$new(color = "grey", fadeout = TRUE, html = html_spin)
    waiter$show()
		callModule(server.modules_pancan_cross_gene_o2m, "modules_pancan_cross_gene_o2m")
		callModule(server.modules_pancan_cross_pw_o2m, "modules_pancan_cross_pw_o2m")
    on.exit(waiter$hide())
}, once = TRUE)  

# PCAWG 
observeEvent(req(input$navbar=="PCAWG: Correlation Analysis"),{
    waiter <- waiter::Waiter$new(color = "grey", fadeout = TRUE, html = html_spin)
    waiter$show()
    callModule(server.modules_pcawg_cor_o2o, "modules_pcawg_cor_o2o")
    callModule(server.modules_pcawg_cor_o2m, "modules_pcawg_cor_o2m")
    callModule(server.modules_pcawg_cor_m2o, "modules_pcawg_cor_m2o")
    on.exit(waiter$hide())
}, once = TRUE)  
observeEvent(req(input$navbar=="PCAWG: Comparison Analysis"),{
    waiter <- waiter::Waiter$new(color = "grey", fadeout = TRUE, html = html_spin)
    waiter$show()
    callModule(server.modules_pcawg_comp_o2o, "modules_pcawg_comp_o2o")
    callModule(server.modules_pcawg_comp_o2m, "modules_pcawg_comp_o2m")
    callModule(server.modules_pcawg_comp_m2o, "modules_pcawg_comp_m2o")
    on.exit(waiter$hide())
}, once = TRUE)  
observeEvent(req(input$navbar=="PCAWG: Survival Analysis"),{
    waiter <- waiter::Waiter$new(color = "grey", fadeout = TRUE, html = html_spin)
    waiter$show()
    callModule(server.modules_pcawg_sur_o2o, "modules_pcawg_sur_o2o")
    callModule(server.modules_pcawg_sur_o2m, "modules_pcawg_sur_o2m")
    callModule(server.modules_pcawg_sur_m2o, "modules_pcawg_sur_m2o")
    on.exit(waiter$hide())
}, once = TRUE)  


# CCLE
observeEvent(req(input$navbar=="CCLE: Correlation Analysis"),{
    waiter <- waiter::Waiter$new(color = "grey", fadeout = TRUE, html = html_spin)
    waiter$show()
    callModule(server.modules_ccle_cor_o2o, "modules_ccle_cor_o2o")
    callModule(server.modules_ccle_cor_m2o, "modules_ccle_cor_m2o")
    on.exit(waiter$hide())
}, once = TRUE) 

observeEvent(req(input$navbar=="CCLE: Comparison Analysis"),{
    waiter <- waiter::Waiter$new(color = "grey", fadeout = TRUE, html = html_spin)
    waiter$show()
    callModule(server.modules_ccle_comp_o2o, "modules_ccle_comp_o2o")
    callModule(server.modules_ccle_comp_m2o, "modules_ccle_comp_m2o")
    on.exit(waiter$hide())
}, once = TRUE) 




# # identifier help
observeEvent(req(input$navbar=="TPC ID Query"),{
  waiter <- waiter::Waiter$new(color = "grey", fadeout = TRUE, html = html_spin)
  waiter$show()
	callModule(server.modules_id_reference, "modules_id_reference")
  on.exit(waiter$hide())
}, once = TRUE) 




# # download data
observeEvent(req(input$navbar=="The integrated TPC data"),{
    waiter <- waiter::Waiter$new(color = "grey", fadeout = TRUE, html = html_spin)
    waiter$show()
	  callModule(server.modules_download_pancan, "modules_download_pancan")
    on.exit(waiter$hide())
}, once = TRUE) 

observeEvent(req(input$navbar=="The Repository Dataset"),{
    waiter <- waiter::Waiter$new(color = "grey", fadeout = TRUE, html = html_spin)
    waiter$show()
	  callModule(server.modules_download_dataset, "modules_download_dataset")
    on.exit(waiter$hide())
}, once = TRUE) 


# PharmacoGenomics
observeEvent(req(input$navbar %in% c(
  "Drug-Omics Correlation Analysis",
  "Feature Abundance Profile in Databases",
  "Dimension Reduction Profile of Cell Drug Sensitivity",
  "Feature Scaling Association Analysis",
  "Statistics and Annotations"
)),{

    waiter <- waiter::Waiter$new(color = "grey", fadeout = TRUE, html = html_spin)
    waiter$show()
    message("Preprocessing drug omics data...")
    source(system.file("shinyapp/PGdata.R", package = "UCSCXenaShiny"))
    callModule(serverDrugOmicPair, "DrugOmicPair")
    callModule(serverFeatureAcrossType, "FeatureAcrossType")
    callModule(serverProfileDrugSens, "ProfileDrugSens")
    callModule(serverFeatureDatabaseSig, "FeatureDatabaseSig")
    callModule(serverStatAnno, "StatAnno")
    message("Done for loading data and modules.")
    on.exit(waiter$hide())
}, once = TRUE) 


callModule(server.home_search_box, "homepage_pancan_search")
callModule(server.home_daily_gene, "homepage_daily_gene")

ui.modules_pancan_sur_o2o = function(id) {
	ns = NS(id)
	fluidPage(
		fluidRow(
			# 初始设置
			column(
				3,
				wellPanel(
					style = "height:1100px",
					h2("S1: Preset", align = "center"),
					h4(strong("S1.1 Modify datasets"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Modify datasets", 
					                   content = "data_origin"),
					mol_origin_UI(ns("mol_origin2sur"), database = "toil"),

					h4(strong("S1.2 Choose cancer")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Cancer types", 
					                   content = "tcga_types"),
					pickerInput(
						ns("choose_cancer"), NULL,
						choices = sort(tcga_names)),
				    br(),

					h4(strong("S1.3 Filter samples"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Filter samples", 
					                   content = "choose_samples"),
					h5("Quick filter:"),
					pickerInput(
						ns("filter_by_code"), NULL,
						choices = NULL, selected =  NULL,
						multiple = TRUE, options = list(`actions-box` = TRUE)
					),
					h5("Exact filter:"),
					filter_samples_UI(ns("filter_samples2sur"), database = "toil"),
					br(),
					verbatimTextOutput(ns("filter_phe_id_info")),
					br(),

					h4(strong("S1.4 Upload metadata"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Upload metadata", 
					                   content = "custom_metadata"),
					shinyFeedback::useShinyFeedback(),
					custom_meta_UI(ns("custom_meta2sur")),
					br(),

					h4(strong("S1.5 Add signature"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Add signature", 
					                   content = "add_signature"),
					add_signature_UI(ns("add_signature2sur"), database = "toil"),

				)
			),
			# 选择生存资料并设置分组
			column(
				4,
				wellPanel(
					style = "height:1100px",
					h2("S2: Get data", align = "center"),
					h4(strong("S2.1 Select survival endpoint")), 

				    shinyWidgets::prettyRadioButtons(
				        inputId = ns("endpoint_type"), label = NULL,
				        choiceValues = c("OS", "DSS", "DFI", "PFI"),
				        choiceNames = c("OS (Overall Survial)", "DSS (Disease-Specific Survival)", 
				        				"DFI (Disease-Free Interval)", "PFI (Progression-Free Interval)"),
				        selected = "OS"
				    ),
				    br(),br(),
					h4(strong("S2.2 Divide 2 groups by one condition")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Divide 2 groups", 
					                   content = "set_groups"),
				    group_samples_UI(ns("group_samples2sur"), database = "toil")  
				)
			),
			# 分析/可视化/下载
			column(
				5,
				wellPanel(
					style = "height:1100px",
					h2("S3: Analyze & Visualize", align = "center") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Analyze & Visualize", 
					                   content = "analyze_sur_1"),  
					h4(strong("S3.1 Set analysis parameters")), 
					selectInput(ns("sur_method"), "Survival method:",
						choices = c("Log-rank test", "Univariate Cox regression")),

				    materialSwitch(ns("use_origin"), 
				    	"Whether use initial data before grouping?"),
					h4(strong("S3.2 Set visualization parameters")), 
			      	uiOutput(ns("one_params.ui")),
					dropMenu(
						actionBttn(ns("more_visu"), label = "Other options", style = "bordered",color = "success",icon = icon("bars")),
						div(h3(strong("Params for Log-rank test")),style="width:500px;"),
						div(h4("1. Wheather to dislpay risk.table:"),style="width:500px;"),
						fluidRow(
							column(6, radioButtons(inputId = ns("plot_table"), label = NULL, 
								choices = c("NO", "YES"), selected="NO",inline = TRUE)),
						),
						div(h4("2. Wheather to dislpay ncensor.plot:"),style="width:500px;"),
						fluidRow(
							column(6, radioButtons(inputId = ns("plot_ncensor"), label = NULL, 
								choices = c("NO", "YES"), selected="NO",inline = TRUE)),
						),
						div(h4("3. Wheather to dislpay confidence interval:"),style="width:500px;"),
						fluidRow(
							column(12, radioButtons(inputId = ns("plot_CI"), label = NULL, 
								choices = c("NO", "YES(ribbon)", "YES(step)"), selected="NO",inline = TRUE)),
						),	
						div(h4("2. Adjust text size:"),style="width:400px;"),
						fluidRow(
							column(4, numericInput(inputId = ns("axis_size"), label = "Text size:", value = 14, step = 0.5)),
							column(4, numericInput(inputId = ns("title_size"), label = "Title size:", value = 18, step = 0.5))
						),	
						div(h4("4. Adjust lab and title name:"),style="width:500px;"),
						fluidRow(
							column(4, textInput(inputId = ns("x_name"), label = "X-axis name:")),
							column(4, textInput(inputId = ns("title_name"), label = "Title name:"))
						),	
						div(h3(strong("Params for Cox regression")),style="width:500px;"),
						div(h4("1. Adjust text size:"),style="width:400px;"),
						fluidRow(
							column(4, numericInput(inputId = ns("axis_size_2"), label = "Font size:", value = 0.7, step = 0.1)),
						),	
						div(h4("2. Adjust lab and title name:"),style="width:500px;"),
						fluidRow(
							column(4, textInput(inputId = ns("title_name_2"), label = "Title name:", value = "Hazard ratio"))
						),	
						div(h5("Note: You can download the raw data and plot in local R environment for more detailed adjustment.")),
					),
					br(),
					shinyWidgets::actionBttn(
						ns("sur_analysis_bt_single"), "Run",
				        style = "gradient",
				        icon = icon("chart-line"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					),
					br(),
					fluidRow(
						column(10, offset = 1,
							   plotOutput({ns("sur_plot_one")}, height = "500px") 
						)
					),
					br(),
					h4(strong("S3.3 Download results")), 
					download_res_UI(ns("download_res2sur"))
				)
			)
		)
	)
}

server.modules_pancan_sur_o2o = function(input, output, session) {
	ns <- session$ns

	# 记录选择癌症
	cancer_choose <- reactiveValues(name = "BRCA", filter_phe_id=NULL,
		phe_primary=query_tcga_group(database = "toil",cancer = "BRCA", return_all = T))
	observe({
		cancer_choose$name = input$choose_cancer
		cancer_choose$phe_primary <- query_tcga_group(database = "toil",cancer = cancer_choose$name, return_all = T)
	})

	# 自定义上传metadata数据
	custom_meta = callModule(custom_meta_Server, "custom_meta2sur",database = "toil")
	# signature
	sig_dat = callModule(add_signature_Server, "add_signature2sur",database = "toil")

	custom_meta_sig = reactive({
		if(is.null(custom_meta())){
			return(sig_dat())
		} else {
			if(is.null(sig_dat())){
				return(custom_meta())
			} else {
				custom_meta_sig = dplyr::inner_join(custom_meta(),sig_dat())
				return(custom_meta_sig)
			}
		}
	})

	# 数据源设置
	opt_pancan = callModule(mol_origin_Server, "mol_origin2sur", database = "toil")


	## 过滤样本
	# exact filter module
	filter_phe_id = callModule(filter_samples_Server, "filter_samples2sur",
					   database = "toil",
					   cancers=reactive(cancer_choose$name),
					   custom_metadata=reactive(custom_meta_sig()),
					   opt_pancan = reactive(opt_pancan()))
	# quick filter widget
	observe({
		code_types_valid = code_types[names(code_types) %in% 
							unique(cancer_choose$phe_primary$Code)]
		updatePickerInput(
			session,
			"filter_by_code",
			choices = unlist(code_types_valid,use.names = F),
			selected =  unlist(code_types_valid,use.names = F)
		)
	})
	# 综合上述二者
	observe({
		# quick filter
		choose_codes = names(code_types)[unlist(code_types) %in% input$filter_by_code]
		filter_phe_id2 = cancer_choose$phe_primary %>%
			dplyr::filter(Code %in% choose_codes) %>%
			dplyr::pull("Sample")

		# exact filter
		if(is.null(filter_phe_id())){
			cancer_choose$filter_phe_id = filter_phe_id2
		} else {
			cancer_choose$filter_phe_id = intersect(filter_phe_id2,filter_phe_id())
		}

		output$filter_phe_id_info = renderPrint({
			cat(paste0("Tip: ", length(cancer_choose$filter_phe_id), " samples are retained"))
		})
	})


	# 生存资料
	sur_dat_v1 = reactive({
		sur_dat_raw = load_data("tcga_surv") %>% dplyr::rename("Sample"="sample")
		cli_dat_raw = load_data("tcga_clinical") %>% dplyr::rename("Sample"="sample")
		sur_dat_sub = sur_dat_raw %>%
			dplyr::filter(Sample %in% cancer_choose$filter_phe_id) %>%
			dplyr::select("Sample",contains(input$endpoint_type)) %>%
			dplyr::mutate(cancer = cli_dat_raw$type[match(Sample,cli_dat_raw$Sample)],.before = 1) %>%
			na.omit()
		colnames(sur_dat_sub)[3:4] = c("status","time")
		sur_dat_sub
	})

	# 设置分组
	group_final = callModule(group_samples_Server, "group_samples2sur",
					   	   database = "toil",
						   cancers=reactive(cancer_choose$name),
						   samples=reactive(sur_dat_v1()$Sample),
						   custom_metadata=reactive(custom_meta_sig()),
						   opt_pancan = reactive(opt_pancan())
						   )

	# 合并分组与生存
	sur_res_one = reactiveValues(sur_dat = NULL, cutoff=NULL, sur_res = NULL)

	group_sur_final = reactive({
		dat = dplyr::inner_join(group_final(),sur_dat_v1()[,-1],by=c("Sample"="Sample"))
		## 验证是否只有一组分组的癌症
		dat = dat %>%
			dplyr::filter(Cancer %in% sort(unique(dat$Cancer))[
				apply(table(dat$Cancer,dat$group),1,function(x) {min(x)>=1})])
		dat
	})

	output$one_params.ui = renderUI(
		if(input$sur_method=="Log-rank test"){
		  	fluidRow(
		  		column(4,colourpicker::colourInput(ns("one_log_color1"), "Color (Group 1):", "#E7B800")),
		  		column(4,colourpicker::colourInput(ns("one_log_color2"), "Color (Group 2):", "#2E9FDF")),
		  	)
		} else if(input$sur_method=="Univariate Cox regression") {
			fluidRow(
				column(4,numericInput(ns("text_c1"), "Position of text col-1", 0.02, step = 0.01)),
				column(4,numericInput(ns("text_c2"), "text col-2", 0.22, step = 0.01)),
				column(4,numericInput(ns("text_c3"), "text col-3", 0.4, step = 0.01))
			)
		}
	)

	# 生存分析的输入（供绘图）与输出结果（供下载）
	observeEvent(input$sur_analysis_bt_single, {
		sur_res_one$sur_dat = group_sur_final()

		if(input$sur_method=="Log-rank test"){
			if(!input$use_origin){ #是否使用分组前的原始值
				sur_res_one$sur_dat$Group = sur_res_one$sur_dat$group
			} else {
				if(class(group_sur_final()$origin) != "character"){ #若原始值为数值型，则寻找最佳阈值
					res.cut <- surv_cutpoint(sur_res_one$sur_dat, time = "time", event = "status", variables = "origin")
					groups_1_2 = sur_res_one$sur_dat %>% 
						  dplyr::group_by(group) %>% 
						  dplyr::summarise(mean = mean(origin)) %>% 
						  dplyr::arrange(mean) %>% 
						  dplyr::pull(group) %>% as.character()
					sur_res_one$sur_dat$Group = ifelse(surv_categorize(res.cut)$origin=="low", groups_1_2[1], groups_1_2[2])
					sur_res_one$sur_dat$Group = factor(sur_res_one$sur_dat$Group, levels=groups_1_2)
				} else {
					sur_res_one$sur_dat$Group = sur_res_one$sur_dat$group # 若不是，则仍使用提供的分组结果
				}
			}
			# print(head(sur_res_one$sur_dat))
			surv_diff <- survdiff(Surv(time, status) ~ Group, data = sur_res_one$sur_dat)
			pval = 1 - pchisq(surv_diff$chisq, length(surv_diff$n) - 1)
			sur_res_one$sur_res = summary(survfit(Surv(time, status) ~ Group, data = sur_res_one$sur_dat))$table %>% 
				    as.data.frame() %>% tibble::rownames_to_column("Group") %>% 
				    dplyr::mutate(Cancer = cancer_choose$name, .before = 1) %>% 
				    dplyr::mutate(p.value = pval)

		} else if (input$sur_method=="Univariate Cox regression"){
			if(!input$use_origin){
				sur_res_one$sur_dat$Group = sur_res_one$sur_dat$group
			} else {
				if(class(group_sur_final()$origin) != "character"){
					sur_res_one$sur_dat$Group = sur_res_one$sur_dat$origin
				} else {
					sur_res_one$sur_dat$Group = sur_res_one$sur_dat$group
				}
			}
			fit <- coxph(Surv(time, status) ~ Group, data = sur_res_one$sur_dat)
			# sur_res_one$pval = summary(fit)$coefficients[1,5]
			sur_res_one$sur_res = summary(fit)$coefficients %>% as.data.frame()
		}
	})


 	observe({
		updateTextInput(session, "x_name", value = paste(input$endpoint_type, "(days)"))
		updateTextInput(session, "title_name", value = NULL)
 	})


	sur_plot_one = eventReactive(input$sur_analysis_bt_single,{
		shiny::validate(
			need(try(nrow(sur_res_one$sur_dat)>0), 
				"Please inspect whether to set valid groups in S3 step."),
		)
		p = plot_sur_o20(
			sur_res_one$sur_dat, plot_CI=input$plot_CI, plot_table=input$plot_table, 
			plot_ncensor=input$plot_ncensor, sur_method=input$sur_method,
        	one_log_color1=input$one_log_color1, one_log_color2=input$one_log_color2, 
			axis_size=input$axis_size, x_name=input$x_name, 
			title_name=input$title_name, title_size=input$title_size,
        	text_c1=input$text_c1, text_c2=input$text_c2, text_c3=input$text_c3,
			axis_size_2=input$axis_size_2, title_name_2=input$title_name_2)
		p
	})

	output$sur_plot_one = renderPlot({sur_plot_one()})

	# Download results
	observeEvent(input$sur_analysis_bt_single,{
		res1 = sur_plot_one()
		res2 = sur_res_one$sur_dat
		res3 = sur_res_one$sur_res
		callModule(download_res_Server, "download_res2sur", res1, res2, res3)
	})
}

ui.modules_3_ccle_02 = function(id){
    ns = NS(id)
    main_ui = tagList(
        fluidRow(
            column(6,
                style = 'border-right: 1px solid; border-color: white',
                h4(strong("X-axis:")),
                mol_quick_select_UI(ns("id_1"), "ccle", c("mRNA","protein","cnv"))
            ),
            column(6,
                h4(strong("Y-axis:")),
                mol_quick_select_UI(ns("id_2"), "ccle", c("mRNA","protein","cnv"))
            )
        ),
        h4("3. Select primary site(s)"),
        virtualSelectInput(
            ns("SitePrimary"), NULL,
            choices = choices_primary_site,
            multiple = TRUE,
            selected = "prostate"
        ),
        h4("4. Select correlation method"),
        awesomeRadio(
            inputId = ns("cor_method"),
            label = NULL, 
            choices = c("spearman", "pearson"),
            selected = "spearman",
            inline = TRUE, checkbox = TRUE
        ),
        shinyWidgets::actionBttn(
            inputId = ns("search_bttn"),
            label = "Go!",
            style = "gradient",
            icon = icon("search"),
            color = "primary",
            block = TRUE,
            size = "sm"
        )
    )
    out_ui = tagList(
        fluidRow(
            uiOutput(ns("gene_ccle_gene_cor")),
        )
    )
    side_ui = tagList(
        fluidRow(
            column(6,
                h4("1. Visualization parameters"),
                h5("(1) Regression line:"),
                switchInput(
                    inputId = ns("use_regline"),
                    value = TRUE,
                    onLabel = "Yes",
                    offLabel = "No"
                ),
                h5("(2) Point transparent:"),
                sliderTextInput(
                    inputId = ns("alpha"),
                    label = NULL,
                    choices = seq(from = 0, to = 1, by = 0.1),
                    selected = "0.5", grid = TRUE
                ),
                h5("(3) Point color:"),
                colourpicker::colourInput(
                    inputId = ns("color"), NULL, "#000000"),
                h5("(4) ggplot theme:"),
                selectInput(inputId = ns("theme"), label = NULL, 
                            choices = names(themes_list), selected = "Cowplot"),
            ),
            column(6,
                h4("2. Download options"),
                h5("(1) Figure:"),
                numericInput(inputId = ns("height"), label = "Height", value = 6),
                numericInput(inputId = ns("width"), label = "Width", value = 6),
                awesomeRadio(ns("device"), label = "Format", 
                    choices = c("pdf", "png"), selected = "pdf", inline = TRUE),
                downloadBttn(
                  outputId = ns("download_1"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                ),
                h5("(2) Data table:"),
                downloadBttn(
                  outputId = ns("download_2"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                )
            )
        )
    )
    fluidPage(
        wellPanel(style = "height:725px",
        box(main_ui,
            width = 5,
            solidHeader = TRUE,
            title = "Quick CCLE Analysis: Correlation in cancer cell lines", 
            status = "success",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Click the bottom button to execute/update the analysis."
        ),
        box(out_ui,
            width = 7,
            solidHeader = TRUE,
            title = "Analytical results:", 
            status = "success",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Pull the sidebar to adjust plot parameters or download results through the top-right widget.",
            sidebar = boxSidebar(
                        id = ns("sidebar"),
                        width = 50,
                        side_ui
            )
        )
        )
    )
}



server.modules_3_ccle_02 = function(input, output, session){
    ns = session$ns

    mol_info_1 = callModule(mol_quick_select_Server, "id_1", "ccle")
    mol_info_2 = callModule(mol_quick_select_Server, "id_2", "ccle")

    plot_func <- eventReactive(input$search_bttn, {
        id <- showNotification(h3("The task is running..."), duration = NULL, closeButton = FALSE, type = "message")
        on.exit(removeNotification(id), add = TRUE)  #reactive语句执行完毕时，运行remove命令
        # check whether valid out plot
        # chect_plot = is.null(plot_func()) 
        # if(chect_plot){
        #     sendSweetAlert(session, title = "Warning", type = "error", text = "Please select a valid molecule.")
        #     req(chect_plot)
        # }
        p <- vis_ccle_gene_cor(
            Gene1 = mol_info_1$molecule(),
            Gene2 = mol_info_2$molecule(),
            data_type1 = mol_info_1$profile(),
            data_type2 = mol_info_2$profile(),
            cor_method = input$cor_method,
            # use_log_x = input$use_log_x,
            # use_log_y = input$use_log_y,
            use_regline = input$use_regline,
            color = input$color,
            alpha = input$alpha,
            SitePrimary = input$SitePrimary
        )
        p <- p + themes_list[[input$theme]] +
                theme(text = element_text(size = 20),
                    legend.position = "none")
        return(p)
    })

    w <- waiter::Waiter$new(id = ns("gene_ccle_gene_cor"), html = waiter::spin_hexdots(), color = "white")
    observeEvent(input$search_bttn,{
        shinyjs::disable("search_bttn")
        output$gene_ccle_gene_cor <- renderUI({
            w$show()
            output$plot = renderPlot(plot_func())
            fluidRow(
                column(8, offset = 2,
                    plotOutput(ns("plot"), height = "580px"),
                )
            )
        })    
        shinyjs::enable("search_bttn")
    })

    output$download_1 <- downloadHandler(
        filename = function() {
            paste0(mol_info_1$molecule(), "_", mol_info_1$profile(), "_",  
                   mol_info_2$molecule(), "_", mol_info_2$profile(), "_cor.", input$device)
        },
        content = function(file) {
            p <- plot_func()
            if (input$device == "pdf") {
                pdf(file, width = input$width, height = input$height)
                print(p)
                dev.off()
            } else {
                png(file, width = input$width, height = input$height, res = 600, units = "in")
                print(p)
                dev.off()
            }
        }
    )

    output$download_2 <- downloadHandler(
        filename = function() {
            paste0(mol_info_1$molecule(), "_", mol_info_1$profile(), "_", 
                   mol_info_2$molecule(), "_", mol_info_2$profile(), "_cor.csv")
        },
        content = function(file) {
        data = plot_func()$data %>%
            dplyr::rename('Cell_line'='sample',
            'Molecule1'='gene1', 'Molecule2'='gene2') %>%
            dplyr::select(Cell_line,Site_Primary,Molecule1,Molecule2)
        write.csv(data, file, row.names = FALSE)
        }
    )
}

# Render cohorts_text and subtype_text from server
xena_table <- UCSCXenaTools::XenaData[, c(
  "XenaDatasets", "XenaHostNames", "XenaCohorts",
  "SampleCount", "DataSubtype", "Label", "Unit"
)]
xena_table$SampleCount <- as.integer(xena_table$SampleCount)
colnames(xena_table)[c(1:3)] <- c("Dataset ID", "Hub", "Cohort")


update_repo_data <- function(input) {
  res <- XenaData

  if (!is.null(input$hubs_text)) {
    res <- dplyr::filter(res, XenaHostNames %in% input$hubs_text)
  }

  if (!is.null(input$cohorts_text) && input$cohorts_text != "") {
    if (!"ALL" %in% input$cohorts_text) {
      res <- dplyr::filter(res, XenaCohorts %in% input$cohorts_text)
    }
  }

  if (!is.null(input$subtype_text) && input$subtype_text != "") {
    if (!"ALL" %in% input$subtype_text) {
      res <- dplyr::filter(res, DataSubtype %in% input$subtype_text)
    }
  }

  if (!is.null(input$type_text)) {
    res <- dplyr::filter(res, Type %in% input$type_text)
  }
  return(res)
}

repo_filters <- reactive({
  list(
    hub = input$hubs_text,
    cohort = input$cohorts_text,
    type = input$type_text,
    subtype = input$subtype_text
  )
})
dataset <- eventReactive(repo_filters(), update_repo_data(input))
# df <- dataset

output$cohorts_text <- renderUI({
  cohorts_text <- isolate(input$cohorts_text)
  selectInput(
    inputId = "cohorts_text",
    label = "Cohort Name:",
    choices = c("ALL", unique(xena_table$Cohort[xena_table$Hub %in% input$hubs_text])),
    selected = cohorts_text,
    multiple = TRUE
  )
})

output$subtype_text <- renderUI({
  subtype_text <- isolate(input$subtype_text)
  selectInput(
    inputId = "subtype_text",
    label = "Data Subtype:",
    choices = c("ALL", unique(xena_table$DataSubtype[xena_table$Hub %in% input$hubs_text])),
    selected = subtype_text,
    multiple = TRUE
  )
})

output$xena_table <- DT::renderDataTable({
  dataset()[, c(
    "XenaDatasets",
    "XenaHostNames",
    "XenaCohorts",
    "SampleCount",
    "DataSubtype",
    "Label",
    "Unit"
  )] %>%
    DT::datatable(
      rownames = FALSE,
      colnames = c(
        "Dataset",
        "Hub",
        "Cohort",
        "Samples",
        "Subtype",
        "Label",
        "Unit"
      ),
      options = list(
        language = list(search = "Filter with keyword:"),
        initComplete = JS(
          "function(settings, json) {",
          "$(this.api().table().header()).css({'background-color': '#4a5a6a', 'color': '#fff'});",
          "$(this.api().table().header()).css({'font-size': '95%'});",
          "$(this.api().table().body()).css({'font-size': '90%'});",
          "}"
        )
      )
    )
})


# Keep selected database
selected_database <- reactive({
  s <- input$xena_table_rows_selected
  if (length(s)) {
    return(dataset()[s, ])
  }
})

selected_database_add_url <- reactive({
  data <- selected_database()
  if (!is.null(data)) {
    data$download <- unlist(lapply(query_url()$url, function(x) {
      as.character(tags$a(href = x, "download link"))
    }))
    data$browse <- unlist(lapply(query_url()$browse, function(x) {
      as.character(tags$a(href = x, "browse Xena dataset page"))
    }))
  }
  data
})


query_url <- reactive({
  s <- input$xena_table_rows_selected
  if (!is.null(s)) {
    xe_query <- xe_query_url(selected_database())
    return(xe_query)
  }
})

# Show download url for selected database
w <- waiter::Waiter$new(id = "table", html = waiter::spin_wobblebar(), color = "white")

observe({
  s <- input$xena_table_rows_selected

  if (length(s) > 0) {
    data <- selected_database_add_url()

    w$show() # Waiter add-ins
    Sys.sleep(0.2)

    if (length(data$download) > 0) {
      output$table <- renderTable(
        {
          tryCatch(
            data.frame(
              No = seq_along(s),
              CohortName = data$XenaCohorts,
              Label = data$Label,
              Download = data$download,
              Browse = data$browse
            ),
            error = function(e) {
              message("Detect error from data.frame construction, no pain.")
            }
          )
        },
        sanitize.text.function = function(x) x
      )
    }
    shinyjs::show("show_data")
  } else {
    shinyjs::hide("show_data")
  }
})


# Button to show metadata -------------------------------------------------

# Dialog for showing selected data information
observeEvent(input$show_met, {
  s <- input$xena_table_rows_selected
  if (length(s)) {
    showModal(modalDialog(
      title = "Detail information...",
      size = "l",
      DT::DTOutput("detail_info")
    ))
  } else {
    sendSweetAlert(
      session,
      "Alert",
      "Please select at least one dataset by clicking the rows in the data table firstly.",
      type = "info"
    )
  }
})

# Show metadata
observeEvent(input$show_met, {
  s <- input$xena_table_rows_selected
  if (length(s)) {
    m <-
      purrr::map2(dataset()$XenaHosts[s], dataset()$XenaDatasets[s], function(x, y) {
        temp <- .p_dataset_metadata(x, y)
        json_data <- jsonlite::parse_json(temp$text)
        message("Metadata for ", y, " queried.")
        json_data <-
          purrr::map(json_data, ~ ifelse(length(.) > 0, paste(., collapse = ","), .))
        json_data <- tibble::enframe(json_data)
        json_data$value <- unlist(json_data$value)
        json_data
      })
    m <- purrr::reduce(m, dplyr::full_join, by = "name")
    message("Metadata are loaded!")
    colnames(m) <- c("Metadata", paste0("dataset", 1:(ncol(m) - 1L)))
  }
  output$detail_info <-
    DT::renderDT(m, options = list(dom = "t", scrollX = TRUE))
})


# Button to request (download) data ---------------------------------------

observeEvent(input$req_data, {
  s <- input$xena_table_rows_selected
  if (length(s)) {
    showModal(
      modalDialog(
        title = "Submitted datasets:",
        size = "l",
        DT::DTOutput("table_query"),
        hr(),
        if (xena.runMode == "client") {
          shinyDirButton(
            id = "download",
            label = "Download data directly",
            title = "Please select a folder",
            icon = icon("download"),
            style = "margin-bottom: 10px;"
          )
        } else {
          downloadButton(
            outputId = "download",
            label = "Download data directly",
            icon = icon("download"),
            style = "margin-bottom: 10px;"
          )
        },
        downloadButton(
          outputId = "total_url",
          label = "Batch download in terminal",
          icon = icon("download"),
          style = "margin-bottom: 10px; margin-left: 40px;"
        ),
        shinyBS::bsPopover(
          "total_url",
          title = "Tips",
          content = "Download wget commands to download requested datasets.",
          placement = "bottom",
          options = list(container = "body")
        ),
        actionButton(
          "show_R_code",
          label = "Copy R download code",
          icon = icon("file"),
          style = "margin-bottom: 10px; margin-left: 65px;"
        ),
        hr(),
        verbatimTextOutput("R_download_code")
      )
    )

    output$table_query <-
      DT::renderDT(query_url(), options = list(dom = "t", scrollX = TRUE))
  } else {
    sendSweetAlert(
      session,
      "Alert",
      "Please select at least one dataset by clicking the rows in the data table firstly.",
      type = "info"
    )
  }
})

if (xena.runMode == "client") {
  observeEvent(input$download, {
    volumes <- c(home = fs::path_home(), root = "/")
    shinyDirChoose(input, "download", roots = volumes, session = session)

    if (is.integer(input$download)) {
      message("No directory has been selected.")
    } else {
      message("Download datasets from client mode.")
      withProgress(
        message = "Download data directly from remote server with R",
        detail = "This may take a while...",
        value = 0,
        {
          for (i in seq_len(nrow(query_url()))) {
            UCSCXenaTools::XenaDownload(
              query_url()[i, c("hosts", "datasets", "url")],
              destdir = parseDirPath(volumes, input$download),
              download_probeMap = TRUE,
              trans_slash = TRUE
            )
            Sys.sleep(0.05)
            incProgress(1 / nrow(query_url()))
          }
        }
      )
    }
  })
} else {
  # Download buttom of request data with zip compress
  message("Download datasets from server mode.")

  output$download <- downloadHandler(
    filename = paste(format(Sys.time(), "%Y-%m-%d-%H-%M-%S"), "xena-datasets.zip", sep = "-"),
    contentType = "application/zip",
    content = function(file) {
      withProgress(
        message = "Query data from remote server",
        detail = "This may take a while...",
        value = 0,
        {
          xe_download <- dplyr::tibble()
          for (i in seq_len(nrow(query_url()))) {
            xe_download <- dplyr::bind_rows(
              xe_download,
              UCSCXenaTools::XenaDownload(
                query_url()[i, c("hosts", "datasets", "url")],
                destdir = XENA_DEST,
                download_probeMap = TRUE,
                trans_slash = TRUE
              )
            )
            incProgress(1 / (2 * nrow(query_url())))
            Sys.sleep(0.05)
          }
          zip::zipr(
            zipfile = file.path(XENA_DEST, "xena-datasets.zip"),
            files = xe_download$destfiles, recurse = FALSE
          )
          incProgress(1 / 4)
          file.copy(file.path(XENA_DEST, "xena-datasets.zip"), file)
          file.remove(file.path(XENA_DEST, "xena-datasets.zip"))
          incProgress(1 / 4)
        }
      )
    }
  )
}

# Download list of urls
output$total_url <- downloadHandler(
  filename = paste0(Sys.Date(), "-commands.sh"),
  contentType = "text/txt",
  content = function(file) {
    write.table(
      file = file,
      c(
        "#!/usr/bin/env bash",
        paste(
          "#Usage: run bash",
          paste0(Sys.Date(), "-commands.sh"),
          "in your terminal under a desired directory"
        ),
        paste0("wget -c ", query_url()$url)
      ),
      row.names = F,
      col.names = F,
      quote = F
    )
  }
)

## Show download code for reproducible research
observeEvent(input$show_R_code, {
  f <- tempfile()
  dput(query_url(), f)
  R_code <- paste(
    "# Load R package",
    "library('UCSCXenaTools')",
    "",
    "# Generate dataset(s) information",
    paste0("dataset_query <- ", paste(readLines(f), collapse = "")),
    "",
    "# Download dataset(s)",
    "dl <- XenaDownload(dataset_query,
                        destdir = './', # At default, download to working directory
                        download_probeMap = TRUE,
                        trans_slash = TRUE)",
    "",
    "# Load dataset(s) into R",
    "datasets <- XenaPrepare(dl)",
    "# Check data",
    "datasets",
    sep = "\n"
  )
  file.remove(f)

  output$R_download_code <- shiny::renderText(R_code)
})


# Button to Analysis ------------------------------------------------------

# ref: https://stackoverflow.com/questions/38706965/is-there-any-way-for-an-actionbutton-to-navigate-to-another-tab-within-a-r-shi
observeEvent(input$analyze_data, {
  updateNavbarPage(session = session, inputId = "navbar", selected = "General Analysis")
})

# Show alert info when select rows from table -----------------------------

observeEvent(input$use_repository, {
  # Show a modal when the button is pressed
  shinyalert(
    title = "Repository Usage",
    text = paste(
      "Firstly, filter dataset table based on left filters or right search bar.",
      "Secondly, select datasets by clicking corresponding rows in dataset table.",
      "Lastly, click the button under the dataset table to check metadata or download datasets.",
      sep = "\n\n"
    ),
    type = "info",
    timer = 0,
    confirmButtonCol = "#202324"
  )
})

ui.modules_3_ccle_04 = function(id){
    ns = NS(id)
    main_ui = tagList(
        h4("1. Select omics type"),
        selectInput(
            inputId = ns("profile"), label = NULL,
            choices  = c("mRNA Expression"="mRNA"),
            selected = "mRNA"
        ),
        h4(id = ns("id_tip"), "2. Select one gene list"),
        bsTooltip(ns("id_tip"), "When two or more genes are selected, their geometric mean is calculated as the signature value.", 
                    placement = "right", trigger = "hover", options = list(container = "body")),
        virtualSelectInput(
            inputId = ns("Pancan_search_1"),
            label = NULL, choices = NULL, multiple = TRUE,
            width = "100%", search = TRUE,
            allowNewOption = TRUE, dropboxWidth = "100%"
        ),
        h4("3. Select primary site(s)"),
        virtualSelectInput(
            ns("SitePrimary"), NULL,
            choices = choices_primary_site,
            multiple = TRUE,
            selected = "prostate"
        ),
        shinyWidgets::actionBttn(
            inputId = ns("search_bttn"),
            label = "Go!",
            style = "gradient",
            icon = icon("search"),
            color = "primary",
            block = TRUE,
            size = "sm"
        )
    )
    out_ui = tagList(
        fluidRow(
            uiOutput(ns("gene_ccle_drug_response_diff")),
        )
    )
    side_ui = tagList(
        fluidRow(
            column(6,
                h4("1. Visualization parameters"),
                h5("(1) Show P value:"),
                switchInput(
                    inputId = ns("pdist_show_p_value"),
                    value = FALSE,
                    onLabel = "Yes",
                    offLabel = "No"
                ),
                h5("(2) Colors:"),
                colourpicker::colourInput(inputId = ns("high_col"), "High group color", "#DF2020"),
                colourpicker::colourInput(inputId = ns("low_col"), "Low group color", "#DDDF21"),
                h5("(3) Point transparent:"),
                sliderTextInput(
                    inputId = ns("alpha"), label = NULL,
                    choices = seq(from = 0, to = 1, by = 0.1),
                    selected = "0.5", grid = TRUE
                ),
            ),
            column(6,
                h4("2. Download options"),
                h5("(1) Figure:"),
                numericInput(inputId = ns("height"), label = "Height", value = 8),
                numericInput(inputId = ns("width"), label = "Width", value = 6),
                awesomeRadio(ns("device"), label = "Format", 
                    choices = c("pdf", "png"), selected = "pdf", inline = TRUE),
                downloadBttn(
                  outputId = ns("download_1"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                ),
                h5("(2) Data table:"),
                downloadBttn(
                  outputId = ns("download_2"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                )
            )
        )
    )
    fluidPage(
        wellPanel(style = "height:725px",
        box(main_ui,
            width = 5,
            solidHeader = TRUE,
            title = "Quick Analysis: Analyze Difference of Drug Response between Gene (Signature) High and Low Expression", 
            status = "primary",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Click the bottom button to execute/update the analysis."
        ),
        box(out_ui,
            width = 7,
            solidHeader = TRUE,
            title = "Analytical results:", 
            status = "primary",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Pull the sidebar to adjust plot parameters or download results through the top-right widget.",
            sidebar = boxSidebar(
                        id = ns("sidebar"),
                        width = 50,
                        side_ui
            )
        )
        )
    )
}



server.modules_3_ccle_04 = function(input, output, session){
    ns = session$ns

    gene_ref = sort(rownames(load_data("ccle_expr_and_drug_response")$expr))

    profile_choices <- reactive({
        list(all = ccle_id.list[["Gene"]], default = "TP53")
    })

    observe({
        updateVirtualSelect(
        "Pancan_search_1",
        # choices = profile_choices()$all,
        choices = gene_ref,
        selected = profile_choices()$default
        )
    })

    plot_func <- eventReactive(input$search_bttn, {
        shiny::validate(
            need(try(length(input$Pancan_search_1)<10), 
                "Error: Less than 10-gene signature is supported.")
        )
        id <- showNotification(h3("The task is running..."), duration = NULL, closeButton = FALSE, type = "message")
        on.exit(removeNotification(id), add = TRUE)  #reactive语句执行完毕时，运行remove命令
        p <- vis_gene_drug_response_diff(
            Gene = input$Pancan_search_1,
            values = c(input$high_col, input$low_col),
            tissue = input$SitePrimary,
            Method = "wilcox.test",
            Show.P.label = input$pdist_show_p_value,
            alpha = input$alpha
        )
        print(class(p))
        return(p)
    })

    w <- waiter::Waiter$new(id = ns("gene_ccle_drug_response_diff"), html = waiter::spin_hexdots(), color = "black")
    observeEvent(input$search_bttn,{
        shinyjs::disable("search_bttn")
        # # check whether valid out plot
        # chect_plot = inherits(plot_func(), "try-error")
        # if(chect_plot){
        #     sendSweetAlert(session, title = "Warning", type = "error", text = "Please select a valid molecule.")
        #     req(chect_plot)
        # }
        output$gene_ccle_drug_response_diff <- renderUI({
            w$show()
            output$plot = renderPlot(plot_func())
            fluidRow(
                column(12, offset = 0,
                    plotOutput(ns("plot"), height = "580px"),
                )
            )
        }) 
        shinyjs::enable("search_bttn")
    })

    output$download_1 <- downloadHandler(
        filename = function() {
            paste0("ccle_target_response_diff.", input$device)
        },
        content = function(file) {
            p <- plot_func()
            if (input$device == "pdf") {
                pdf(file, width = input$width, height = input$height)
                print(p)
                dev.off()
            } else {
                png(file, width = input$width, height = input$height, res = 600, units = "in")
                print(p)
                dev.off()
            }
        }
    )

    output$download_2 <- downloadHandler(
        filename = function() {
            "ccle_target_response_diff.csv"
        },
        content = function(file) {
            data = data <- analyze_gene_drug_response_asso(mol_info$molecule(), combine = TRUE)
            write.csv(data, file, row.names = FALSE)
        }
    )

} 

ui.modules_pcawg_sur_o2m = function(id) {
	ns = NS(id)
	fluidPage(
		fluidRow(
			# 初始设置
			column(
				3,
				wellPanel(
					style = "height:1100px",
					h2("S1: Preset", align = "center"),

					h4(strong("S1.1 Modify datasets"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Modify datasets", 
					                   content = "data_origin"),
					mol_origin_UI(ns("mol_origin2sur"), database = "pcawg"),

					h4(strong("S1.2 Choose projects")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "PCAWG projects", 
					                   content = "pcawg_projects"),
					pickerInput(
						ns("choose_cancers"),NULL,
						choices = pcawg_names,
						multiple = TRUE,
						selected = pcawg_names,
						options = list(`actions-box` = TRUE)
					),
					br(),

					h4(strong("S1.3 Filter samples"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Filter samples", 
					                   content = "choose_samples"),
					h5("Quick filter:"),
					pickerInput(
						ns("filter_by_code"), NULL,
						choices = NULL, selected =  NULL,
						multiple = TRUE, options = list(`actions-box` = TRUE)
					),
					h5("Exact filter:"),
					filter_samples_UI(ns("filter_samples2sur"), database = "pcawg"),
					br(),
					verbatimTextOutput(ns("filter_phe_id_info")),
					br(),

					h4(strong("S1.4 Upload metadata"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Upload metadata", 
					                   content = "custom_metadata"),
					shinyFeedback::useShinyFeedback(),
					custom_meta_UI(ns("custom_meta2sur")),
					br(),

					h4(strong("S1.5 Add signature"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Add signature", 
					                   content = "add_signature"),
					add_signature_UI(ns("add_signature2sur"), database = "pcawg")
				)
			),
			# 选择生存资料
			column(
				4,
				wellPanel(
					style = "height:1100px",
					h2("S2: Get data", align = "center"),
					h4(strong("S2.1 Select survival endpoint")), 
					p("Only OS (Overall Survial) is supported."),
					br(),br(),
					h4(strong("S2.2 Divide 2 groups by one condition")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Divide 2 groups", 
					                   content = "set_groups"),
				    group_samples_UI(ns("group_samples2sur"),database = "pcawg") 
				)
			),
			# 分析/可视化/下载
			column(
				5,
				wellPanel(
					style = "height:1100px",
					h2("S3: Analyze & Visualize", align = "center") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Analyze & Visualize", 
					                   content = "analyze_sur_2"),  
					h4(strong("S3.1 Set analysis parameters")), 
					selectInput(ns("sur_method"), "Survival method:",
						choices = c("Log-rank test", "Univariate Cox regression")),
				    materialSwitch(ns("use_origin"), 
					    	"Whether use initial data before grouping?"),
					h4(strong("S3.2 Set visualization parameters")), 
			      	uiOutput(ns("multi_params.ui")),

			      	dropMenu(
						actionBttn(ns("more_visu"), label = "Other options", style = "bordered",color = "success",icon = icon("bars")),		
						placement = "left",
						div(h3("Select ggplot theme:"),style="width:400px;"),
						fluidRow(
							column(6,
								selectInput(inputId = ns("theme"), label = NULL, 
											choices = names(themes_list), selected = "Minimal")
							)
						),
						div(h3(strong("Params for Log-rank test")),style="width:500px;"),
						div(h4("1. Add one vertical line:"),style="width:400px;"),
				      	fluidRow(
					  		column(6, numericInput(ns("multi_log_line"), "Add line(P):", 0.05)),
				      	),
						div(h4("2. Significance display:"),style="width:400px;"),
				      	fluidRow(
					  		column(6, selectInput(ns("multi_log_label"),"Add text:",
					  			choices = c("Signif.(symbol)", "Signif.(value)"),selected = "Signif.(symbol)"))	
				      	),
						div(h4("3. Adjust text size:"),style="width:400px;"),
						fluidRow(
							column(4, numericInput(inputId = ns("axis_size"), label = "Text size:", value = 18, step = 0.5)),
							column(4, numericInput(inputId = ns("title_size"), label = "Title size:", value = 20, step = 0.5)),
							column(4, numericInput(inputId = ns("label_size"), label = "Label size:", value = 5, step = 0.5)),
						),				
						div(h4("4. Adjust lab and title name:"),style="width:400px;"),
						fluidRow(
							column(4, textInput(inputId = ns("x_name"), label = "X-axis name:", value = "-log10(P-value)")),
							column(4, textInput(inputId = ns("title_name"), label = "Title name:",
								value = NULL))
						),

						div(h3(strong("Params for Cox regression")),style="width:500px;"),
						div(h4("1. Add one vertical line:"),style="width:400px;"),
				      	fluidRow(
					  		column(6, numericInput(ns("multi_cox_line"), "Add line(P):", 0.05)),
				      	),
						div(h4("2. Significance display:"),style="width:400px;"),
				      	fluidRow(
					  		column(6, selectInput(ns("multi_cox_label"),"Add text:",
					  			choices = c("Signif.(symbol)", "Signif.(value)", "HR value"),selected = "HR value"))	
				      	),
						div(h4("3. Adjust text size:"),style="width:400px;"),
						fluidRow(
							column(4, numericInput(inputId = ns("axis_size_2"), label = "Text size:", value = 18, step = 0.5)),
							column(4, numericInput(inputId = ns("title_size_2"), label = "Title size:", value = 20, step = 0.5)),
							column(4, numericInput(inputId = ns("label_size_2"), label = "Label size:", value = 5, step = 0.5)),
						),				
						div(h4("4. Adjust lab and title name:"),style="width:400px;"),
						fluidRow(
							column(4, textInput(inputId = ns("x_name_2"), label = "X-axis name:", value = "-log10(P-value)")),
							column(4, textInput(inputId = ns("title_name_2"), label = "Title name:",
								value = NULL))
						),
						div(h5("Note: You can download the raw data and plot in local R environment for more detailed adjustment."))
			      	),
			      	br(),

					shinyWidgets::actionBttn(
						ns("sur_analysis_bt_multi"), "Run",
				        style = "gradient",
				        icon = icon("chart-line"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					),
					br(),
					fluidRow(
						column(10, offset = 1,
							   plotOutput({ns("sur_plot_multi")}, height = "500px") 
						)
					),
					br(),
					h4(strong("S3.3 Download results")), 
				    download_res_UI(ns("download_res2sur"))
				)
			)

		)

	)

}


server.modules_pcawg_sur_o2m = function(input, output, session) {
	ns <- session$ns

	# 记录选择癌症
	cancer_choose <- reactiveValues(name = "BLCA-US", phe_primary="",
		filter_phe_id=query_tcga_group(database = "pcawg", cancer = "BLCA-US", return_all = T))
	observe({
		cancer_choose$name = input$choose_cancers
		cancer_choose$phe_primary <- query_tcga_group(database = "pcawg",
			cancer = cancer_choose$name, return_all = T)
	})


	# 数据源设置
	opt_pancan = callModule(mol_origin_Server, "mol_origin2sur", database = "pcawg")

	# 自定义上传metadata数据
	custom_meta = callModule(custom_meta_Server, "custom_meta2sur", database = "pcawg")
	# signature
	sig_dat = callModule(add_signature_Server, "add_signature2sur", database = "pcawg")
	custom_meta_sig = reactive({
		if(is.null(custom_meta())){
			return(sig_dat())
		} else {
			if(is.null(sig_dat())){
				return(custom_meta())
			} else {
				custom_meta_sig = dplyr::inner_join(custom_meta(),sig_dat())
				return(custom_meta_sig)
			}
		}
	})


	## 过滤样本
	# quick filter widget
	observe({
		code_types_valid = unique(cancer_choose$phe_primary$Type)
		updatePickerInput(
			session,
			"filter_by_code",
			choices = code_types_valid,
			selected =  code_types_valid
		)
	})
	# exact filter module
	filter_phe_id = callModule(filter_samples_Server, "filter_samples2sur",
					   database = "pcawg",
					   cancers=reactive(cancer_choose$name),
					   custom_metadata=reactive(custom_meta_sig()),
					   opt_pancan = reactive(opt_pancan()))

	# 综合上述二者
	observe({
		# quick filter
		filter_phe_id2 = cancer_choose$phe_primary %>%
			dplyr::filter(Type %in% input$filter_by_code) %>%
			dplyr::pull("Sample")

		# exact filter
		if(is.null(filter_phe_id())){
			cancer_choose$filter_phe_id = filter_phe_id2
		} else {
			cancer_choose$filter_phe_id = intersect(filter_phe_id2,filter_phe_id())
		}

		output$filter_phe_id_info = renderPrint({
			cat(paste0("Tip: ", length(cancer_choose$filter_phe_id), " samples are retained"))
		})
	})
	# 生存资料
	sur_dat_v1 = reactive({
		sur_dat_raw = pcawg_info[,c("dcc_project_code","icgc_specimen_id","OS","OS.time")]
		colnames(sur_dat_raw) = c("cancer","Sample","status","time")
		sur_dat_sub = sur_dat_raw %>% 
		  dplyr::distinct()  %>% na.omit() %>% 
		  dplyr::filter(Sample %in% cancer_choose$filter_phe_id)
		sur_dat_sub
	})

	# 设置分组
	group_final = callModule(group_samples_Server, "group_samples2sur",
						   database = "pcawg", 
						   cancers=reactive(cancer_choose$name),
						   samples=reactive(sur_dat_v1()$Sample),
						   custom_metadata=reactive(custom_meta_sig()),
						   opt_pancan = reactive(opt_pancan())
						   )
	# 合并分组与生存
	sur_res_multi = reactiveValues(sur_dat = NULL, cutoff=NULL, sur_res = NULL)	

	group_sur_final = reactive({
		dat = dplyr::inner_join(group_final(),sur_dat_v1()[,-1],by=c("Sample"="Sample"))
		## 验证是否只有一组分组的癌症
		dat = dat %>%
			dplyr::filter(Cancer %in% sort(unique(dat$Cancer))[
				apply(table(dat$Cancer,dat$group),1,function(x) {min(x)>=2})])
		dat
	})

	# 根据选项，动态更新绘图参数
	output$multi_params.ui = renderUI(
		if(input$sur_method=="Log-rank test"){
		  	fluidRow(
		  		column(3,colourpicker::colourInput(ns("multi_log_color1"), "Color (Group 1):", "#d53e4f")),
		  		column(3,colourpicker::colourInput(ns("multi_log_color2"), "Color (Group 2):", "#3288bd")),
		  	)
		} else if(input$sur_method=="Univariate Cox regression") {
		  	fluidRow(
		  		# column(4,colourpicker::colourInput(ns("multi_cox_color"), "Color:", "grey")),
		  		column(5,colourpicker::colourInput(ns("cox_h_g1_color"), "Color(HR>1):", "#d53e4f")),
		  		column(5,colourpicker::colourInput(ns("cox_h_l1_color"), "Color(HR<1):", "#3288bd")),

		  	)
		}
	)

	observeEvent(input$sur_analysis_bt_multi, {
		sur_res_multi$sur_dat = group_sur_final()	

		if(input$sur_method=="Log-rank test"){
			if(!input$use_origin){ #是否使用分组前的原始值
				sur_res_multi$sur_dat$Group = sur_res_multi$sur_dat$group
			} else {
				if(class(sur_res_multi$sur_dat$origin) != "character"){ #若原始值为数值型，则寻找最佳阈值
					# res.cut <- surv_cutpoint(sur_res_multi$sur_dat, time = "time", event = "status", variables = "Group")
					# res.cat <- surv_categorize(res.cut)
					sur_res_multi$sur_dat = lapply(sort(unique(sur_res_multi$sur_dat$Cancer)), function(x) {
						sur_dat_sub = subset(sur_res_multi$sur_dat, Cancer==x)
						groups_1_2 = sur_dat_sub %>% 
							  dplyr::group_by(group) %>% 
							  dplyr::summarise(mean = mean(origin)) %>% 
							  dplyr::arrange(mean) %>% 
							  dplyr::pull(group) %>% as.character()
						res.cut <- surv_cutpoint(sur_dat_sub, time = "time", event = "status", variables = "origin")
						sur_dat_sub$Group = ifelse(surv_categorize(res.cut)$origin=="low", groups_1_2[1], groups_1_2[2])
						if(min(table(sur_dat_sub$Group))==0) return(NULL)
						sur_dat_sub$Group = factor(sur_dat_sub$Group, levels=groups_1_2)
						sur_dat_sub
					}) %>% do.call(rbind, .) %>% as.data.frame()
				} else {
					sur_res_multi$sur_dat$Group = sur_res_multi$sur_dat$group
				}

			}
			sur_res_multi$sur_res = lapply(sort(unique(sur_res_multi$sur_dat$Cancer)), function(x){
			  sur_dat_sub = subset(sur_res_multi$sur_dat, Cancer==x)
			  
			  #如果只有一个有效分组
			  if(length(unique(sur_dat_sub$Group))==1) return(NULL)

			  surv_diff <- survdiff(Surv(time, status) ~ Group, data = sur_dat_sub)
			  pval = 1 - pchisq(surv_diff$chisq, length(surv_diff$n) - 1)
			  
			  summary(survfit(Surv(time, status) ~ Group, data = sur_dat_sub))$table %>% 
			    as.data.frame() %>% tibble::rownames_to_column("Group") %>% 
			    dplyr::mutate(Cancer = x, .before = 1) %>% 
			    dplyr::mutate(p.value = pval)
			}) %>% do.call(rbind, .) %>% as.data.frame()

		} else if (input$sur_method=="Univariate Cox regression"){
			if(!input$use_origin){
				sur_res_multi$sur_dat$Group = sur_res_multi$sur_dat$group
			} else {
				sur_res_multi$sur_dat$Group = sur_res_multi$sur_dat$origin
			}
			sur_res_multi$sur_res = lapply(sort(unique(sur_res_multi$sur_dat$Cancer)), function(x){
			  sur_dat_sub = subset(sur_res_multi$sur_dat, Cancer==x)
			  fit = coxph(Surv(time, status) ~ Group , data = sur_dat_sub)
			  summary(fit)$coefficients %>% as.data.frame() %>% 
			    tibble::rownames_to_column("Group") %>% 
			    dplyr::mutate(Cancer = x, .before = 1)
			}) %>% do.call(rbind, .) %>% as.data.frame()
		}
	})

	sur_plot_multi = eventReactive(input$sur_analysis_bt_multi,{
		shiny::validate(
			need(try(nrow(sur_res_multi$sur_dat)>0), 
				"Please inspect whether to set valid groups in S2 step."),
		)
		p = plot_sur_02m(
			sur_res_multi, sur_method=input$sur_method, 
			multi_log_color1=input$multi_log_color1, multi_log_color2=input$multi_log_color2, 
			x_name=input$x_name, title_name=input$title_name, multi_log_line=input$multi_log_line, 
			axis_size=input$axis_size, title_size=input$title_size, multi_log_label=input$multi_log_label,
        	label_size=input$label_size, cox_h_g1_color=input$cox_h_g1_color, cox_h_l1_color=input$cox_h_l1_color, 
			x_name_2=input$x_name_2, title_name_2=input$title_name_2, multi_cox_line=input$multi_cox_line, 
			axis_size_2=input$axis_size_2, title_size_2=input$title_size_2, 
			multi_cox_label=input$multi_cox_label, label_size_2=input$label_size_2,
			custom_theme=themes_list[[input$theme]]
		)
		p
	})

	output$sur_plot_multi = renderPlot({sur_plot_multi()})

	# 3个下载按钮
	observeEvent(input$sur_analysis_bt_multi,{
		res1 = sur_plot_multi()
		res2 = sur_res_multi$sur_dat
		res3 = sur_res_multi$sur_res
		callModule(download_res_Server, "download_res2sur", res1, res2, res3)
	})
}

ui.modules_3_ccle_04 = function(id){
    ns = NS(id)
    main_ui = tagList(
        h4("1. Select omics type"),
        selectInput(
            inputId = ns("profile"), label = NULL,
            choices  = c("mRNA Expression"="mRNA"),
            selected = "mRNA"
        ),
        h4(id = ns("id_tip"), "2. Select one gene list"),
        bsTooltip(ns("id_tip"), "When two or more genes are selected, their geometric mean is calculated as the signature value.", 
                    placement = "right", trigger = "hover", options = list(container = "body")),
        virtualSelectInput(
            inputId = ns("Pancan_search_1"),
            label = NULL, choices = NULL, multiple = TRUE,
            width = "100%", search = TRUE,
            allowNewOption = TRUE, dropboxWidth = "100%"
        ),
        h4("3. Select primary site(s)"),
        virtualSelectInput(
            ns("SitePrimary"), NULL,
            choices = choices_primary_site,
            multiple = TRUE,
            selected = "prostate"
        ),
        shinyWidgets::actionBttn(
            inputId = ns("search_bttn"),
            label = "Go!",
            style = "gradient",
            icon = icon("search"),
            color = "primary",
            block = TRUE,
            size = "sm"
        )
    )
    out_ui = tagList(
        fluidRow(
            uiOutput(ns("gene_ccle_drug_response_diff")),
        )
    )
    side_ui = tagList(
        fluidRow(
            column(6,
                h4("1. Visualization parameters"),
                h5("(1) Show P value:"),
                switchInput(
                    inputId = ns("pdist_show_p_value"),
                    value = FALSE,
                    onLabel = "Yes",
                    offLabel = "No"
                ),
                h5("(2) Colors:"),
                colourpicker::colourInput(inputId = ns("high_col"), "High group color", "#DF2020"),
                colourpicker::colourInput(inputId = ns("low_col"), "Low group color", "#DDDF21"),
                h5("(3) Point transparent:"),
                sliderTextInput(
                    inputId = ns("alpha"), label = NULL,
                    choices = seq(from = 0, to = 1, by = 0.1),
                    selected = "0.5", grid = TRUE
                ),
            ),
            column(6,
                h4("2. Download options"),
                h5("(1) Figure:"),
                numericInput(inputId = ns("height"), label = "Height", value = 8),
                numericInput(inputId = ns("width"), label = "Width", value = 6),
                awesomeRadio(ns("device"), label = "Format", 
                    choices = c("pdf", "png"), selected = "pdf", inline = TRUE),
                downloadBttn(
                  outputId = ns("download_1"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                ),
                h5("(2) Data table:"),
                downloadBttn(
                  outputId = ns("download_2"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                )
            )
        )
    )
    fluidPage(
        wellPanel(style = "height:725px",
        box(main_ui,
            width = 5,
            solidHeader = TRUE,
            title = "Quick Analysis: Compare between tumor and normal", 
            status = "primary",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Click the bottom button to execute/update the analysis."
        ),
        box(out_ui,
            width = 7,
            solidHeader = TRUE,
            title = "Analytical results:", 
            status = "primary",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Pull the sidebar to adjust plot parameters or download results through the top-right widget.",
            sidebar = boxSidebar(
                        id = ns("sidebar"),
                        width = 50,
                        side_ui
            )
        )
        )
    )
}



server.modules_3_ccle_04 = function(input, output, session){
    ns = session$ns

    gene_ref = sort(rownames(load_data("ccle_expr_and_drug_response")$expr))

    profile_choices <- reactive({
        list(all = ccle_id.list[["Gene"]], default = "TP53")
    })

    observe({
        updateVirtualSelect(
        "Pancan_search_1",
        # choices = profile_choices()$all,
        choices = gene_ref,
        selected = profile_choices()$default
        )
    })

    plot_func <- eventReactive(input$search_bttn, {
        shiny::validate(
            need(try(length(input$Pancan_search_1)<10), 
                "Error: Less than 10-gene signature is supported.")
        )
        id <- showNotification(h3("The task is running..."), duration = NULL, closeButton = FALSE, type = "message")
        on.exit(removeNotification(id), add = TRUE)  #reactive语句执行完毕时，运行remove命令
        p <- vis_gene_drug_response_diff(
            Gene = input$Pancan_search_1,
            values = c(input$high_col, input$low_col),
            tissue = input$SitePrimary,
            Method = "wilcox.test",
            Show.P.label = input$pdist_show_p_value,
            alpha = input$alpha
        )
        print(class(p))
        return(p)
    })

    w <- waiter::Waiter$new(id = ns("gene_ccle_drug_response_diff"), html = waiter::spin_hexdots(), color = "black")
    observeEvent(input$search_bttn,{
        shinyjs::disable("search_bttn")
        # # check whether valid out plot
        # chect_plot = inherits(plot_func(), "try-error")
        # if(chect_plot){
        #     sendSweetAlert(session, title = "Warning", type = "error", text = "Please select a valid molecule.")
        #     req(chect_plot)
        # }
        output$gene_ccle_drug_response_diff <- renderUI({
            w$show()
            output$plot = renderPlot(plot_func())
            fluidRow(
                column(12, offset = 0,
                    plotOutput(ns("plot"), height = "580px"),
                )
            )
        }) 
        shinyjs::enable("search_bttn")
    })

    output$download_1 <- downloadHandler(
        filename = function() {
            paste0("ccle_target_response_diff.", input$device)
        },
        content = function(file) {
            p <- plot_func()
            if (input$device == "pdf") {
                pdf(file, width = input$width, height = input$height)
                print(p)
                dev.off()
            } else {
                png(file, width = input$width, height = input$height, res = 600, units = "in")
                print(p)
                dev.off()
            }
        }
    )

    output$download_2 <- downloadHandler(
        filename = function() {
            "ccle_target_response_diff.csv"
        },
        content = function(file) {
            data = data <- analyze_gene_drug_response_asso(mol_info$molecule(), combine = TRUE)
            write.csv(data, file, row.names = FALSE)
        }
    )

} 

home_text.list = list(
  intro = paste0(
    "Thank you for using UCSCXenaShiny v",packageVersion("UCSCXenaShiny")," based on ","UCSCXenaTools v", packageVersion("UCSCXenaTools"),
    ". Our web tool aims to povide a user-friendly platform to explore UCSC Xena datasets for both general and personalized cancer molecular research.",
    " If you have any questions during use, please do not hesitate to contact us via Github issue.",
    " If the tool has faciliated your research, welcome to cite our work.  :)"
  )
)


ui.page_home <- function() {
  tabPanel(
    title = "Home",
    icon = icon("home"), # create icon http://shiny.rstudio.com/reference/shiny/latest/icon.html
    fluidPage(
      # useWaiter(), 
      # waiterPreloader(html = tagList(
      #   spin_fading_circles(), 
      #   br(), br(),
      #   h1(strong("Welcome to use UCSCXenaShiny v2 application!")),
      #   br(),
      #   p("An interactive web tool with general and personalized modules to explore UCSC Xena datasets"
      #     ,style = "font-size: 25px;"),
      #   br(),br(),
      #   p("Notes:", "(1) The initiation could take about 10 seconds. (2) Please zoom in or up screen for better representation.",
      #     style = "font-size: 16px;")
      # ), color = "#2C3E50"),
      useShinydashboard2(),
      tags$head(
        tags$link(rel = "stylesheet", type = "text/css", href = "bootstrap4.css")
      ),
      fluidRow(
        column(4,
          tags$div(
            column(
              12,
              tags$h2("Welcome to UCSCXenaShiny v2!"),
              tags$p(home_text.list$intro, style = "font-size: 19px;"),
              fluidRow(
                column(4, align = "center",
                  actionBttn("bt01","Github",
                    style = "bordered", color = "primary", icon = icon("github"),
                    onclick=paste0("window.open('https://github.com/openbiox/UCSCXenaShiny','_blank')"))
                ),
                column(4, align = "center",
                  actionBttn("bt02","Tutorial",
                    style = "bordered", color = "primary", icon = icon("book"),
                    onclick=paste0("window.open('https://lishensuo.github.io/UCSCXenaShiny_Book','_blank')"))
                ),
                column(4, align = "center",
                  actionBttn("bt03","Article",
                    style = "bordered", color = "primary", icon = icon("newspaper"))
                ),
              ),
              uiOutput("citation"),
              tags$hr(),
            )
          )
        ),
        column(4,
          wellPanel(
            style = "background: #b3cde3",
            h3("Daily Gene", icon = "dice", align = "center"),
            ui.home_daily_gene("homepage_daily_gene"),
          )
        ),
        column(4,
          wellPanel(
            style = "background: #a6cee3",
            h3("Pan-Cancer Query", align = "center"),
            # br(),
            ui.home_search_box("homepage_pancan_search"),
          )
        ),
      ),
      tags$br(),
      h2(strong("※ Shiny Page Gallery")),
      tags$hr(style = "border:none; border-top:5px solid #5E81AC;"),
      fluidRow(
        column(
          12,
          slickR::slickROutput("slick_output", width = "90%", height = "700px")
        )
      ),
      br(),br(),
      h2(strong("※ Example TCGA Modules: Quick analysis with easy steps")),
      tags$hr(style = "border:none; border-top:5px solid #5E81AC;"),
      fluidRow(
        column(
          4,
          shinydashboard::box(
            title = strong("Module: Tumor and Normal Comparison",style = "font-size: 23px;"), 
            solidHeader = TRUE, status="primary",
            width = 12, height = 250,
            p("Compare one multi-omics molecular value between ",strong("tumor and normal")," (including GTEx) samples.",
              style = "font-size: 25px;"),
            actionLink("link_to_q1_n", p("Go >>>",style = "font-size: 30px;")),
          ),
        ),
        column(
          4,
          shinydashboard::box(
            title = strong("Module: Molecule-Molecule Correlation", style = "font-size: 23px;"),
            solidHeader = TRUE, status="success",
            width = 12, height = 250,
            p("Calculate the correlation of ",strong("two multi-omics molecules")," acccording to their values in tumor samples.",
              style = "font-size: 25px;"),
            actionLink("link_to_q2_n", p("Go >>>",style = "font-size: 30px;")),
          ),
        ),
        column(
          4,
          shinydashboard::box(
            title = strong("Module: Kaplan-Meier Survival Analysis", style = "font-size: 23px;"),
            solidHeader = TRUE, status="info",
            width = 12, height = 250,
            p("Perform the ",strong("log-rank test")," analysis of one multi-omics molecule in tumor samples of one cancer type.",
              style = "font-size: 25px;"),
            actionLink("link_to_q3_n", p("Go >>>",style = "font-size: 30px;")),
          )
        ),
      ),
      br(),
      h2(strong("※ Example TCGA pipelines: In-depth analysis with personalized steps")),
      tags$hr(style = "border:none; border-top:5px solid #5E81AC;"),
      fluidRow(
        column(
          4,
          shinydashboard::box(
            title = strong("Pipelines: Comparison Analysis", style = "font-size: 23px;"),
            solidHeader = TRUE, status="primary",
            width = 12, height = 250,
            p("Compare one of integrated identifiers (e.g. omis, non-omics, user-defined) based on two customizable groups of samples.",
              style = "font-size: 25px;"),
            actionLink("link_to_p1_n", p("Go >>>",style = "font-size: 30px;")),
          )
        ),
        column(
          4,
          shinydashboard::box(
            title = strong("Pipelines: Correlation Analysis", style = "font-size: 23px;"),
            solidHeader = TRUE, status="success",
            width = 12, height = 250,
            p("Calculate the correlation between any two of integrated identifiers (e.g. omis, non-omics, user-defined) among custom samples",
              style = "font-size: 25px;"),
            actionLink("link_to_p2_n", p("Go >>>",style = "font-size: 30px;")),
          )
        ),
        column(
          4,
          shinydashboard::box(
            title = strong("Pipelines: Survival Analysis", style = "font-size: 23px;"),
            solidHeader = TRUE, status="info",
            width = 12, height = 250,
            p("Perform log-rank test analysis for one of integrated identifiers based on two customizable groups of samples.",
              style = "font-size: 25px;"),
            actionLink("link_to_p3_n", p("Go >>>",style = "font-size: 30px;")),
          )
        )
      ),
      
      br(),
      h2(strong("※ Latest significant release notes")),
      tags$hr(style = "border:none; border-top:5px solid #5E81AC;"),
      fluidRow(
        column(12, #offset = 1,
          tags$ul(
            tags$li("2024-06-25: Design gene and pathway cross-omics analysis",style = "font-size: 20px;"),
            tags$li("2024-02-14: Incorporate the PharmacoGenomics modules",style = "font-size: 20px;"),
            tags$li("2024-01-21: Adjust homepge with slick gallery to show basic page help.",style = "font-size: 20px;"),
            tags$li("2024-01-16: Introduce MSigDB genesets for molecule batch analysis.",style = "font-size: 20px;"),
            tags$li("2023-12-20: Add download modules that support data requisition.",style = "font-size: 20px;"),
            tags$li("See more update logs in our", 
              a("Github", href="https://github.com/openbiox/UCSCXenaShiny"), ".",
              style = "font-size: 20px;")
          )  
        )
      )
    )
  )
}

ui.modules_pcawg_comp_o2m = function(id) {
	ns = NS(id)
	fluidPage(
		fluidRow(
			# 初始设置
			column(
				3,
				wellPanel(
					style = "height:1100px",
					h2("S1: Preset", align = "center"),

					h4(strong("S1.1 Modify datasets"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Modify datasets", 
					                   content = "data_origin"),
					mol_origin_UI(ns("mol_origin2cor"), database = "pcawg"),

					h4(strong("S1.2 Choose projects")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "PCAWG projects", 
					                   content = "pcawg_projects"),
					pickerInput(
						ns("choose_cancers"),NULL,
						choices = pcawg_names,
						multiple = TRUE,
						selected = pcawg_names,
						options = list(`actions-box` = TRUE)
					),
					br(),

					h4(strong("S1.3 Filter samples"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Filter samples", 
					                   content = "choose_samples"),
					h5("Quick filter:"),
					pickerInput(
						ns("filter_by_code"), NULL,
						choices = NULL, selected =  NULL,
						multiple = TRUE, options = list(`actions-box` = TRUE)
					),
					h5("Exact filter:"),
					filter_samples_UI(ns("filter_samples2cor"), database = "pcawg"),
					br(),
					verbatimTextOutput(ns("filter_phe_id_info")),
					br(),

					h4(strong("S1.4 Upload metadata"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Upload metadata", 
					                   content = "custom_metadata"),
					shinyFeedback::useShinyFeedback(),
					custom_meta_UI(ns("custom_meta2cor")),
					br(),

					h4(strong("S1.5 Add signature"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Add signature", 
					                   content = "add_signature"),
					add_signature_UI(ns("add_signature2comp"), database = "pcawg"),

				)
			),
			# 分组设置
			column(
				4,
				wellPanel(
					style = "height:1100px",
					h2("S2: Get data", align = "center"),
					# 调用分组模块UI
					h4(strong("S2.1 Divide 2 groups by one condition")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Divide 2 groups", 
					                   content = "set_groups"),
					group_samples_UI(ns("group_samples2comp"),database = "pcawg"),  
					# 下载待比较数据
					h4(strong("S2.2 Get data for comparison")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Get one data", 
					                   content = "get_one_data"), 
					download_feat_UI(ns("download_y_axis"), 
						button_name="Query",database = "pcawg"),
				)
			),
			# 分析/绘图/下载
			column(
				5,
				wellPanel(
					h2("S3: Analyze & Visualize", align = "center") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Analyze & Visualize", 
					                   content = "analyze_comp_2"), 
					style = "height:1100px",

					h4(strong("S3.1 Set analysis parameters")), 
					selectInput(ns("comp_method"), "Comparison method:",choices = c("t-test", "wilcoxon")),
					shinyWidgets::actionBttn(
						ns("step3_plot_line_1"), "Run (Calculate)",
				        style = "gradient",
				        icon = icon("chart-line"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					),
					verbatimTextOutput(ns("message1")),
					h4(strong("S3.2 Set visualization parameters")), 
					fluidRow(
						column(3, colourpicker::colourInput(inputId = ns("group_1_color_2"), "Color (Group 1):", "#E69F00")),
						column(3, colourpicker::colourInput(inputId = ns("group_2_color_2"), "Color (Group 2):", "#56B4E9")),
					),
					dropMenu(
						actionBttn(ns("more_visu"), label = "Other options", style = "bordered",color = "success",icon = icon("bars")),
						div(h3("1. Select ggplot theme:"),style="width:400px;"),
						fluidRow(
							column(6,
								selectInput(inputId = ns("theme"), label = NULL, 
											choices = names(themes_list), selected = "Minimal")
							)
						),
						div(h3("2. Significance display:"),style="width:400px;"),
						fluidRow(
							column(6, radioButtons(inputId = ns("significance"), label = "Significance:", 
								choices = c("Value", "Symbol"), selected="Symbol",inline = TRUE)),
						),
						div(h3("3. Adjust text size:"),style="width:400px;"),
						fluidRow(
							column(4, numericInput(inputId = ns("axis_size"), label = "Text size:", value = 18, step = 0.5)),
							column(4, numericInput(inputId = ns("title_size"), label = "Title size:", value = 20, step = 0.5)),
							column(4, numericInput(inputId = ns("label_size"), label = "Label size:", value = 5, step = 0.5)),
						),				
						div(h3("4. Adjust lab and title name:"),style="width:400px;"),
						fluidRow(
							column(4, textInput(inputId = ns("x_name"), label = "X-axis name:")),
							column(4, textInput(inputId = ns("title_name"), label = "Title name:",
								value = NULL))
						),	
						div(h5("Note: You can download the raw data and plot in local R environment for more detailed adjustment.")),
					),
					br(),
					shinyWidgets::actionBttn(
						ns("step3_plot_line_2"), "Run (Visualize)",
				        style = "gradient",
				        icon = icon("chart-line"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					),
					br(),
					fluidRow(
						column(10, offset = 1,
							   plotOutput({ns("comp_plot_line")}, height = "500px") 
						)
					),

					h4(strong("S3.3 Download results")), 
					download_res_UI(ns("download_res2comp"))
				)
			)
		)
	)
}


server.modules_pcawg_comp_o2m = function(input, output, session) {
	ns <- session$ns
	# 记录选择癌症
	cancer_choose <- reactiveValues(name = "BLCA-US", phe_primary="",
		filter_phe_id=query_tcga_group(database = "pcawg", cancer = "BLCA-US", return_all = T))
	observe({
		cancer_choose$name = input$choose_cancers
		cancer_choose$phe_primary <- query_tcga_group(database = "pcawg",
			cancer = cancer_choose$name, return_all = T)
	})

	# 数据源设置
	opt_pancan = callModule(mol_origin_Server, "mol_origin2cor", database = "pcawg")


	# 自定义上传metadata数据
	custom_meta = callModule(custom_meta_Server, "custom_meta2cor", database = "pcawg")
	# signature
	sig_dat = callModule(add_signature_Server, "add_signature2comp", database = "pcawg")
	custom_meta_sig = reactive({
		if(is.null(custom_meta())){
			return(sig_dat())
		} else {
			if(is.null(sig_dat())){
				return(custom_meta())
			} else {
				custom_meta_sig = dplyr::inner_join(custom_meta(),sig_dat())
				return(custom_meta_sig)
			}
		}
	})


	## 过滤样本
	# quick filter widget
	observe({
		code_types_valid = unique(cancer_choose$phe_primary$Type)
		updatePickerInput(
			session,
			"filter_by_code",
			choices = code_types_valid,
			selected =  code_types_valid
		)
	})
	# exact filter module
	filter_phe_id = callModule(filter_samples_Server, "filter_samples2cor",
					   database = "pcawg",
					   cancers=reactive(cancer_choose$name),
					   custom_metadata=reactive(custom_meta_sig()),
					   opt_pancan = reactive(opt_pancan()))

	# 综合上述二者
	observe({
		# quick filter
		filter_phe_id2 = cancer_choose$phe_primary %>%
			dplyr::filter(Type %in% input$filter_by_code) %>%
			dplyr::pull("Sample")

		# exact filter
		if(is.null(filter_phe_id())){
			cancer_choose$filter_phe_id = filter_phe_id2
		} else {
			cancer_choose$filter_phe_id = intersect(filter_phe_id2,filter_phe_id())
		}

		output$filter_phe_id_info = renderPrint({
			cat(paste0("Tip: ", length(cancer_choose$filter_phe_id), " samples are retained"))
		})
	})

	# 设置分组
	group_final = callModule(group_samples_Server, "group_samples2comp",
					       database = "pcawg", 
						   cancers=reactive(cancer_choose$name),
						   samples=reactive(cancer_choose$filter_phe_id),
						   custom_metadata=reactive(custom_meta_sig()),
						   opt_pancan = reactive(opt_pancan())
						   )

	# 下载待比较数据
	y_axis_data = callModule(download_feat_Server, "download_y_axis", 
							 database = "pcawg", 
							 samples=reactive(cancer_choose$filter_phe_id),
							 custom_metadata=reactive(custom_meta_sig()),
						     opt_pancan = reactive(opt_pancan()),
						     check_numeric=TRUE,
						     table.ui = FALSE
							 )
	# barplot逻辑：先批量计算相关性，再绘图
	merge_data_line = eventReactive(input$step3_plot_line_1, {
		group_data = group_final()[,c(1,3,4)]
		colnames(group_data) = c("Sample","group","phenotype")
		y_axis_data = y_axis_data()
		data = dplyr::inner_join(y_axis_data, group_data) %>%
			dplyr::select(cancer, Sample, value, group, everything())
		data
	})

	# 仍需要检查数据，因为即使二分组本身成立，但可能样本缺失variable数据导致仍然只有一个分组
	observe({
		cancer_choose$multi_cancer_ok = 
		    merge_data_line() %>%
		      dplyr::group_by(cancer, group) %>%
		      dplyr::summarise(n1=n()) %>%
		      dplyr::filter(n1>=2) %>% # 每小组的样本数大于等于2
		      dplyr::distinct(cancer, group) %>%
		      dplyr::count(cancer,name = "n2") %>%
		      dplyr::filter(n2==2) %>% dplyr::pull("cancer") # 每个肿瘤有两组
	})
	comp_data_line = eventReactive(input$step3_plot_line_1, {
		merge_data_line = merge_data_line()
		shinyjs::disable("step3_plot_line_1")
		comp_method = switch(isolate(input$comp_method),
			`t-test` = "parametric", wilcoxon = "nonparametric")
		valid_cancer_choose = sort(cancer_choose$multi_cancer_ok)
		withProgress(message = "Please wait for a while.",{
			stat_comp = lapply(seq(valid_cancer_choose), function(i){
			  tcga_type = valid_cancer_choose[i]
			  p = ggbetweenstats(
			    subset(merge_data_line, cancer==tcga_type),
			  	x = "group",
			  	y = "value",
			    type = comp_method)
			  incProgress(1 / length(valid_cancer_choose), detail = paste0("(Finished ",i,"/",length(valid_cancer_choose),")"))
			  return(extract_stats(p)$subtitle_data)
			}) %>% do.call(rbind, .) %>% 
			dplyr::select(!expression) %>% 
			dplyr::mutate(cancer = valid_cancer_choose, .before=1) %>% 
			dplyr::arrange(desc(cancer)) %>%
			dplyr::mutate(cancer = factor(cancer, levels = unique(cancer)))
			stat_comp
		})
		shinyjs::enable("step3_plot_line_1")
		stat_comp
	})
	output$message1 = renderPrint({
		req(comp_data_line())
		shiny::validate(
			need(try(nrow(comp_data_line())>0), 
				"Please inspect whether to download valid data in S2 step."),
		)
		cat(paste("The calculation has been successfully completed! (",format(Sys.time(), "%H:%M:%S"),")"))
	})
	observe({
		updateTextInput(session, "x_name", value = unique(y_axis_data()$id))
		# updateTextInput(session, "y_name", value = unique(y_axis_data()$id))
		updateTextInput(session, "title_name", value = "")
	})
	comp_plot_line = eventReactive(input$step3_plot_line_2, {
		# shiny::validate(
		# 	need(try(nrow(merge_data_line())>0), 
		# 		"Please inspect whether to set groups or download variable data in S2 or S3 step."),
		# )
		p = plot_comb_o2m(
			data1=merge_data_line(), data2=comp_data_line(),
			x_name=input$x_name, title_name=input$title_name,
			group_1_color_2=input$group_1_color_2, group_2_color_2=input$group_2_color_2,
			axis_size=input$axis_size, title_size=input$title_size,
			significance=input$significance, label_size=input$label_size,
			custom_theme=themes_list[[input$theme]]
		)
		return(p)
	})

	output$comp_plot_line = renderPlot({comp_plot_line()})

	# Download results
	observeEvent(input$step3_plot_line_2,{
		res1 = comp_plot_line()
		res2 = merge_data_line()
		p_comp = comp_data_line()
		p_comp$identifier = unique(merge_data_line()$id)
		p_comp$phenotype = unique(merge_data_line()$phenotype)	
		p_comp$group_1 = levels(merge_data_line()$group)[1]
		p_comp$group_2 = levels(merge_data_line()$group)[2]
		res3 = p_comp
		callModule(download_res_Server, "download_res2comp", res1, res2, res3)
	})
}

ui.modules_2_pcawg_03 = function(id){
    ns = NS(id)

    main_ui = tagList(
        mol_quick_select_UI(ns("id"), "pcawg", 
                            c("mRNA","miRNA","promoter","fusion","APOBEC")),

        h4("3. Select PCAWG project (Only OS event)"),
        selectInput(ns("dataset"), NULL,sort(pcawg_names)),

        h4("4. Filter by clinical features"),
        virtualSelectInput(ns("sex"), "Gender:",choices = c("Female" = "female", "Male" = "male" ),
                    multiple=TRUE, selected = c("female", "male")),
        sliderInput(ns("age"), "Age", min = 0, max = 100, value = c(0, 100)),

        h4("5. Grouping by"),

        awesomeRadio(
            inputId = ns("groupby"),
            label = NULL, 
            choices = c("Median cutoff", "Optimal cutoff"),
            selected = "Median cutoff",
            inline = TRUE, checkbox = TRUE
        ),
        bsTooltip(ns("groupby"), "Only valid for continuous variable, other than molecules from Gene Fusion", 
                    placement = "right", trigger = "hover", options = list(container = "body")),

        shinyWidgets::actionBttn(
          inputId = ns("submit_bt"), label = "Submit",
          style = "gradient",
          icon = icon("check"),
          color = "primary",
          block = TRUE,
          size = "sm"
        ),
        verbatimTextOutput(ns("msg"))
    )
    out_ui = tagList(
        fluidRow(
            uiOutput(ns("surplot"))
        )
    )
    side_ui = tagList(
        fluidRow(
            column(6,
                h4("1. Visualization parameters"),
                h5("(1) Color palette:"),
                selectInput(ns("palette"), NULL, c("hue", "grey", "RdBu", "Blues", "npg", "aaas"),
                            selected = "aaas")
            ),
            column(6,
                h4("2. Download options"),
                h5("(1) Figure:"),
                numericInput(inputId = ns("height"), label = "Height", value = 8),
                numericInput(inputId = ns("width"), label = "Width", value = 8),
                awesomeRadio(ns("device"), label = "Format", 
                    choices = c("pdf", "png"), selected = "pdf", inline = TRUE),
                downloadBttn(
                  outputId = ns("download_1"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                ),
                h5("(2) Data table:"),
                downloadBttn(
                  outputId = ns("download_2"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                )
            )
        )
    )
    fluidPage(
        wellPanel(style = "height:795px",
        box(main_ui,
            width = 5,
            solidHeader = TRUE,
            title = "Quick PCAWG Analysis: Kaplan-Meier survival analysis(Log-rank)", 
            status = "info",
            background = "gray",
            collapsible = FALSE,
            style = "height:670px",
            footer = "TIPs: Click the bottom button to execute/update the analysis."
        ),
        box(out_ui,
            width = 7,
            solidHeader = TRUE,
            title = "Analytical results:", 
            status = "info",
            background = "gray",
            collapsible = FALSE,
            style = "height:670px",
            footer = "TIPs: Pull the sidebar to adjust plot parameters or download results through the top-right widget.",
            sidebar = boxSidebar(
                        id = ns("sidebar"),
                        width = 50,
                        side_ui
            )
        )
        )
    )



}

server.modules_2_pcawg_03 = function(input, output, session){
    ns = session$ns

    mol_info = callModule(mol_quick_select_Server, "id", "pcawg")

    filter_dat <- eventReactive(input$submit_bt, {
        val <- query_pancan_value(mol_info$molecule(), mol_info$profile(), database = "pcawg")
        val <- val$data
        val <- na.omit(val)
        
        if(length(val)<10){
            sendSweetAlert(
                session = session,
                title = "Error...",
                text = "There is too little available data (<10) for this entry.",
                type = "error"
            )
            return(NULL)
        }
        
        val_dat <- data.frame("icgc_specimen_id" = names(val),"val" = as.numeric(val))
        
        dat <- dplyr::inner_join(pcawg_info,val_dat,by="icgc_specimen_id") %>% 
            dplyr::filter(.data$dcc_project_code %in% input$dataset) %>% 
            dplyr::filter(!is.na(.data$OS.time)) %>% 
            dplyr::select( sampleID = icgc_specimen_id,
                            status = OS ,
                            time = OS.time,
                            value = val,
                            gender = donor_sex,
                            age = donor_age_at_diagnosis)

        # filter samples
        dat = dplyr::filter(dat,
                    .data$age > input$age[1],
                    .data$age < input$age[2],
                    .data$gender %in% input$sex
        )
        # group fusion profile
        if(mol_info$profile() == "fusion"){
            dat <- dplyr::mutate(dat,group = case_when(
                .data$value == 1 ~ "fusion (1)",
                .data$value == 0 ~ "non-fusion (0)"
            ))
        }
        dat
    })

    tips = eventReactive(input$submit_bt, {
        if(mol_info$profile() == "fusion"){
            t1 = "fusion status"
            sur_dat2 = filter_dat()
        } else {
            sur_dat2 = filter_dat() %>%
                dplyr::arrange(.data$value) %>%
                dplyr::mutate(per_rank = 100 / nrow(.) * (1:nrow(.))) %>%
                dplyr::mutate(group = dplyr::case_when(
                    .data$per_rank > !!50 ~ "High",
                    .data$per_rank <= !!50 ~ "Low",
                    TRUE ~ NA_character_
                )) 
            t1 = ifelse(input$groupby=="Optimal cutoff","optimal cutoff","median value")   

        }
        chech_dat = sur_dat2  %>% 
                dplyr::count(group) %>% 
                dplyr::filter(n>3) # the number of groups > 3 samples
        if(nrow(chech_dat)<=1){
            msg = "Warning: Please adjust above input for valid sample grouping."
            sendSweetAlert(session, title = "Warning", text = "No more than two groups (> 3 samples) are available!")
        } else {
            msg = paste0("Note: ", nrow(filter_dat()), " samples are grouped by ", 
                         t1, " of ", mol_info$molecule(), " ", mol_info$profile(), ".")
        }
        msg
    })
    output$msg = renderPrint({cat(tips())})


    plot_func <- eventReactive(input$submit_bt, {
        req(grep("Note", tips()))
        id <- showNotification(h3("The task is running..."), duration = NULL, closeButton = FALSE, type = "message")
        on.exit(removeNotification(id), add = TRUE)  #reactive语句执行完毕时，运行remove命令
        if (!is.null(filter_dat())) {
            if (nrow(filter_dat()) >= 10) {
                if (mol_info$profile() %in% c("mRNA", "miRNA","promoter", "APOBEC")) {
                    cutoff_mode = ifelse(input$groupby=="Optimal cutoff","Auto","Custom")

                    p <- UCSCXenaShiny:::sur_plot(filter_dat(), cutoff_mode, c(50, 50), palette = input$palette)
                } else {
                    p <- UCSCXenaShiny:::p_survplot(filter_dat(), palette = input$palette) #with group column
                }
                return(p)
            } else {
                return(NULL)
            }
        } else {
            return(NULL)
        }
    })

    return_data <- eventReactive(input$submit_bt, {
        if (!is.null(filter_dat()) & nrow(filter_dat()) >= 10) {
            # shinyjs::show(id = "save_csv")
            select_data <- dplyr::select(filter_dat(), sampleID, value, status, time)
            return(select_data)
        } else {
            return(NULL)
        }
    })

    # Show waiter for plot
    w <- waiter::Waiter$new(id = ns("surplot"), html = waiter::spin_hexdots(), color = "black")
    observeEvent(input$submit_bt,{
        shinyjs::disable("submit_bt")
        output$surplot <- renderUI({
            w$show()
            output$plot = renderPlot(plot_func())
            fluidRow(
                column(10, offset = 1,
                    plotOutput(ns("plot"), height = "650px"),
                )
            )
        })   
        shinyjs::enable("submit_bt") 
    })

    output$download_1 <- downloadHandler(
        filename = function() {
            paste0(mol_info$molecule(), "_", mol_info$profile(), "_pcawg_surplot.", input$device)
        },
        content = function(file) {
            p <- plot_func()
            if (input$device == "pdf") {
                pdf(file, width = input$width, height = input$height, onefile = FALSE)
                print(p)
                dev.off()
            } else {
                png(file, width = input$width, height = input$height, res = 600, units = "in")
                print(p)
                dev.off()
            }
        }
    )

    output$download_2 <- downloadHandler(
        filename = function() {
            paste0(mol_info$molecule(), "_", mol_info$profile(), "_pcawg_surplot.csv")
        },
        content = function(file) {
            data = return_data() %>%
                dplyr::rename('Sample'='sampleID','Value'='value',
                'Status'='status', 'Time'='time') %>%
                dplyr::mutate(Cancer = input$dataset,Event = "OS") %>%
                dplyr::select(Cancer, Sample, Event, Status, Time, Value)
            write.csv(data, file, row.names = FALSE)
        }
    )
}

uiFeatureAcrossType <- function(id){
  ns <- NS(id)
  fluidPage(
    fluidRow(
      # Select Features ----
      column(4,
             selectInput(inputId = ns("select_features"), 
                         "Please select the feature type:", 
                         choices = c("Copy Number Data" = "cnv",
                                     "DNA Methylation" = "meth",
                                     "Gene Fusion" = "fusion",
                                     "Gene Mutation" = "mutation_gene",
                                     "Gene Site Mutation" = "mutation_site",
                                     "mRNA Expression" = "mRNA",
                                     "Protein Expression" = "protein",
                                     "Drug Sensitivity" = "drug"
                         ), selected = "mRNA"
             )),
      # Select specific feature ----
      column(4,
             selectizeInput(
               ns("select_specific_feature"), "Feature selection:", choices = NULL,
               options = list(
                 placeholder = 'Please select a feature',
                 onInitialize = I('function() { this.setValue(""); }'), selected = "LAPATINIB"
               ))
             # DOPselectDrugsUI("DOPselectDrugs"),
      ),
      # Select trait to profile ----
      # column(4,
      #        selectInput(inputId = ns("select_trait"), 
      #                    "Please select the trait type:", 
      #                    choices = c("Cell_Type_Source"), selected = "Cell_Type_Source"
      #        ))
    ),
    # Plot results ----
    wellPanel(
      # textOutput("total")
      plotOutput(ns("p_search2"), height="20cm"),
      h5("."),
      h4(strong("NOTEs:")),
      h5("The y axis is the drug sensitivity metric or molecular data for boxplot.")
    ),
    # Download plot ----
    # column(3,
    #        downloadLink(ns("dl_p_search"), "Save as PDF")
    # )
  )
}

serverFeatureAcrossType <- function(input, output, session){
  ns <- session$ns
  # Select ----
  features_search_sel <- reactiveValues()
  observeEvent(input$select_features, {
    features_search_sel$features <- switch(input$select_features,
                                           "drug" = drugs_search$drugs, 
                                           "mRNA" = omics_search[omics_search$type %in% "mRNA",]$omics,
                                           "meth" = omics_search[omics_search$type %in% "meth",]$omics,
                                           "protein" = omics_search[omics_search$type %in% "protein",]$omics,
                                           "cnv" = omics_search[omics_search$type %in% "cnv",]$omics,
                                           "mutation_gene" = omics_search[omics_search$type %in% "mutation_gene",]$omics,
                                           "mutation_site" = omics_search[omics_search$type %in% "mutation_site",]$omics,
                                           "fusion" = omics_search[omics_search$type %in% "fusion",]$omics)
    updateSelectizeInput(session = session, inputId = 'select_specific_feature',
                         label = 'Feature Selection:', choices = features_search_sel$features, server = TRUE,
                         options = list(placeholder = 'Please select a feature', onInitialize = I('function() { this.setValue(""); }')),
                         selected = "ABCC3"
    )
  })
  # Plot ----
  output$p_search2 <- renderPlot({
    ## Box plot ----
    if(input$select_features %in% c("cnv",
                                    "drug",
                                    "protein",
                                    "meth",
                                    "mRNA")){
      # message(
      #   profile_vec_list[[input$select_features]]
      # )
      p_list <- lapply(profile_vec_list[[input$select_features]], function(x){
        # x = tmp$profile_vec[1]
        # Preprocess
        select_features <- input$select_features
        if(select_features == "mRNA") select_features <- "exp"
        profile <- base::get(paste0(x, "_", select_features), envir = parent.env(environment()))
        intersected_cells <- intersect(cell_anno$Name, colnames(profile))
        sel_profile <- profile[rownames(profile) %in% input$select_specific_feature,
                               match(intersected_cells, colnames(profile))]
        cell_anno2 <- cell_anno[match(intersected_cells,cell_anno$Name),c(1,2)]
        sel_profile <- t(sel_profile) %>% as.numeric()
        if(length(na.omit(sel_profile)) == 0){ return(NULL) }
        # Plot
        profile_df <- data.frame(
          profile = sel_profile,
          anno = cell_anno2$Type)
        profile_df2 <- na.omit(profile_df)
        pval <- kruskal.test(split(profile_df2$profile, profile_df2$anno))$p.value %>% round(4)
        pval <- case_when(
          pval < 0.01 ~ "< 0.01",
          pval < 0.05 ~ "< 0.05",
          T ~ "> 0.05"
        )
        p <- ggboxplot(data = profile_df2, x = "anno", y = "profile",
                       add = "jitter") + 
          # stat_compare_means(size = 5,
          #                    # label.y.npc = "bottom"
          # ) 
          # geom_signif(annotations = pval) + 
          theme_bw() + 
          theme(
            axis.title = element_blank(), 
            plot.title = element_text(size = 15, face = "bold"),
            axis.text = element_text(size = 12),
            axis.text.x = element_text(angle = 60, vjust = 1, hjust = 1)
          ) + scale_x_discrete(limit =levels(factor(unique(cell_anno$Type)))) +
          labs(title = x, 
               subtitle = paste0("Kruskal-Wallis, p ", pval))
        return(p)
      })
      # Bar plot ----    
    } else{
      p_list <- lapply(profile_vec_list[[input$select_features]], function(x){
        # x = tmp$profile_vec[1]
        # Preprocess
        profile <- base::get(paste0(x, "_", input$select_features))
        sel_profile <- profile[profile[[1]] %in% input$select_specific_feature,]
        if(nrow(sel_profile) == 0){ return(NULL) }
        ## If cells in sel_profile, label "yes"
        sel_profile_df1 <- data.frame(
          cells = sel_profile$cells,
          events = "yes"
        )
        sel_profile_df2 <- data.frame(
          cells = profile$cells[!profile$cells %in% sel_profile$cells],
          events = "no"
        ) %>% unique()
        sel_profile_df <- rbind(sel_profile_df1, sel_profile_df2)
        sel_profile_df <- base::merge(sel_profile_df, cell_anno[,1:2],
                                      by.x = "cells", by.y = "Name")
        sel_profile_fq <- as.data.frame(prop.table(table(sel_profile_df$events, sel_profile_df$Type), margin = 2))
        colnames(sel_profile_fq) <- c("events", "Type", "Freq")
        # Plot
        pval <- chisq.test(table(sel_profile_df$events, sel_profile_df$Type))$p.value %>% round(4)
        pval <- case_when(
          pval < 0.01 ~ "< 0.01",
          pval < 0.05 ~ "< 0.05",
          T ~ "> 0.05"
        )
        p <- ggplot(sel_profile_fq) + 
          geom_bar(aes(x = Type, y= Freq*100, fill = events), color = "white",stat = "identity",width = 0.7,linewidth = 0.5)+ 
          theme_bw() + scale_fill_manual(values = c("#BEBADAFF", "#FB8072FF")) + 
          theme(
            axis.title.x = element_blank(), 
            axis.title.y = element_text(size = 15),
            plot.title = element_text(size = 15, face = "bold"),
            axis.text = element_text(size = 12),
            axis.text.x = element_text(angle = 60, vjust = 1, hjust = 1),
            legend.title = element_text(size = 15, face = "bold"),
            legend.text = element_text(size = 15)
          ) + scale_x_discrete(limit =levels(factor(unique(cell_anno$Type)))) + 
          labs(x='',y = 'Percentage(%)',
               title = x, 
               subtitle = paste0("Chi-Squared, p ", pval)) 
      })
    }
    p_list <- p_list[!sapply(p_list, is.null)]
    # Warning 
    shiny::validate(
      need(length(p_list) > 0, "You have not chosen yet, or there is no result for this feature.")
    )
    if(length(p_list) > 1){
      for(i in 1:(length(p_list)-1)){
        p_list[[i]] <- p_list[[i]] + theme(
          axis.text.x = element_blank(),
          axis.ticks.x = element_blank()
        )
      }
    }
    p <- wrap_plots(p_list, ncol = 1)
    return(p)
  })
}

filter_samples_UI = function(id, database="toil"){
	ns = NS(id)

	id_option =switch(database, 
			"toil"=tcga_id_option,
			"pcawg"=pcawg_id_option,
			"ccle"=ccle_id_option)


	tagList(
		dropMenu(
			shinyWidgets::actionBttn(
				ns("filter_phe"), "Enter",
		        style = "gradient",
		        icon = icon("filter"),
		        color = "primary",
		        block = TRUE,
		        size = "sm"
			),
			div(h5("Filter samples by one/multiple phenotype(s):"),style="width:800px;"),
	        wellPanel(
	        	style = "background: #f7f7f7",
		        h4("1. Add phenotypes (Optional)"),

			    fluidRow(
			    	column(
			    		6,
					    selectInput(
					    	ns("data_L1"), label = "Data type:",
					    	choices = names(id_option),
					    	selected = "Molecular profile"
					    )
			    	),
			    	column(
			    		6,
					    tabsetPanel(
						    id = ns("data_L2_tab"),
						    type = "hidden",
							tabPanel("Molecular profile", 
								selectInput(
									ns("genomic_profile"), "Data subtype:",
									choices = names(id_option[["Molecular profile"]]),
									selected = "mRNA Expression")
							),
							tabPanel("Tumor index",
								selectInput(
									ns("tumor_index"), "Data subtype:",
									choices = names(id_option[["Tumor index"]]),
									selected = "Tumor Purity")
							),
							tabPanel("Immune Infiltration",
								selectInput(
									ns("immune_infiltration"), "Data subtype:",
									choices = names(id_option[["Immune Infiltration"]]),
									selected = "CIBERSORT")
							),
							tabPanel("Pathway activity",
								selectInput(
									ns("pathway_activity"), "Data subtype:",
									choices = names(id_option[["Pathway activity"]]),
									selected = "HALLMARK")
							),
							tabPanel("Phenotype data",
								selectInput(
									ns("phenotype_data"), "Data subtype:",
									choices = names(id_option[["Phenotype data"]]),
									selected = "Clinical Phenotype")
							)
						)
			    	)
			    ),
			    tabsetPanel(
				    id = ns("data_L3_tab"),
				    type = "hidden",
					tabPanel("Molecular profile",
			            virtualSelectInput(
			              inputId = ns("genomic_profile_id"),
			              label = "Identifier:",
			              choices = NULL,
			              search = TRUE,
			              allowNewOption = TRUE,
			              dropboxWidth = "200%")
					),
					tabPanel("Tumor index",
			            virtualSelectInput(
			              inputId = ns("tumor_index_id"),
			              label = "Identifier:",
			              choices = NULL,
			              search = TRUE,
			              allowNewOption = FALSE,
			              dropboxWidth = "200%")
					),
					tabPanel("Immune Infiltration",
			            virtualSelectInput(
			              inputId = ns("immune_infiltration_id"),
			              label = "Identifier:",
			              choices = NULL,
			              search = TRUE,
			              allowNewOption = FALSE,
			              dropboxWidth = "200%")
					),
					tabPanel("Pathway activity",
			            virtualSelectInput(
			              inputId = ns("pathway_activity_id"),
			              label = "Identifier:",
			              choices = NULL,
			              search = TRUE,
			              allowNewOption = FALSE,
			              dropboxWidth = "200%")
					),
					tabPanel("Phenotype data",
			            virtualSelectInput(
			              inputId = ns("phenotype_data_id"),
			              label = "Identifier:",
			              choices = NULL,
			              search = TRUE,
			              allowNewOption = FALSE,
			              dropboxWidth = "200%")
					)
				),
			    fluidRow(
				    actionBttn(
				      inputId = ns("button_phe_add"),
				      label = "Add",
				      color = "primary",
				      style = "bordered", size = "sm",
				      block = F
				    ),
				    actionBttn(
				      inputId = ns("button_phe_add_reset"),
				      label = "Reset",
				      color = "primary",
				      style = "bordered", size = "sm",
				      block = F
				    )       
			    ),
			    verbatimTextOutput(ns("add_info"))
	        ),
	        wellPanel(
	        	style = "background: #f7f7f7",
	        	h4("2. Observe distribution"),
				uiOutput(ns("filter_phe_01_by.ui")),
				verbatimTextOutput(ns("filter_phe_01_out"))
			),
		    wellPanel(
		    	style = "background: #f7f7f7",
			    h4("3. Set conditions"),
			    fluidRow(
				    actionBttn(
				      inputId = ns("add_condi"),
				      label = "Add filter",
				      color = "primary",
				      style = "bordered", size = "sm",
				      block = F
				    ),
				    actionBttn(
				      inputId = ns("del_condi"),
				      label = "Del filter",
				      color = "primary",
				      style = "bordered", size = "sm",
				      block = F
				    )
			    ),
			    fluidRow(
			    	column(5, h5("Phenotype(s):")),
			    	column(5, h5("Direction(s):")),
			    	column(2, h5("Threshold(s):"))
			    ),
			    uiOutput(ns("multi_condi.ui")),
			    verbatimTextOutput(ns("filter_by_phe_prompt")),
			    fluidRow(
				    actionBttn(
				      inputId = ns("button_phe_filter"),
				      label = "Run",
				      color = "primary",
				      style = "bordered", size = "sm",
				      block = F
				    ),
				    actionBttn(
				      inputId = ns("button_phe_filter_reset"),
				      label = "Reset",
				      color = "primary",
				      style = "bordered", size = "sm",
				      block = F
				    )  

			    ),

				textOutput(ns("filter_phe_02_out")),
			),
			placement = "right-end",
			maxWidth = "1000px"
		)
	)
}




filter_samples_Server = function(input, output, session, database="toil", #id_option=tcga_id_option,
								 cancers=NULL, custom_metadata=NULL, opt_pancan=NULL){
	ns <- session$ns

	id_option =switch(database, 
			"toil"=tcga_id_option,
			"pcawg"=pcawg_id_option,
			"ccle"=ccle_id_option)

	observe({
	  updateTabsetPanel(inputId = "data_L2_tab", selected = input$data_L1)
  	  updateTabsetPanel(inputId = "data_L3_tab", selected = input$data_L1)
	}) 

	observeEvent(input$genomic_profile, {
		genomic_profile_choices <- reactive({
		  id_option[["Molecular profile"]][[input$genomic_profile]]
		})
	    updateVirtualSelect(
	      "genomic_profile_id",
	      choices = genomic_profile_choices()$all,
	      selected = genomic_profile_choices()$default
	    )
	})
	observeEvent(input$tumor_index, {
		tumor_index_choices <- reactive({
		  id_option[["Tumor index"]][[input$tumor_index]]
		})
	    updateVirtualSelect(
	      "tumor_index_id",
	      choices = tumor_index_choices()$all,
	      selected = tumor_index_choices()$default
	    )
	})
	observeEvent(input$immune_infiltration, {
		immune_infiltration_choices <- reactive({
		  id_option[["Immune Infiltration"]][[input$immune_infiltration]]
		})
	    updateVirtualSelect(
	      "immune_infiltration_id",
	      choices = immune_infiltration_choices()$all,
	      selected = immune_infiltration_choices()$default
	    )
	})
	observeEvent(input$pathway_activity, {
		pathway_activity_choices <- reactive({
		  id_option[["Pathway activity"]][[input$pathway_activity]]
		})
	    updateVirtualSelect(
	      "pathway_activity_id",
	      choices = pathway_activity_choices()$all,
	      selected = pathway_activity_choices()$default
	    )
	})
	observeEvent(input$phenotype_data, {
		phenotype_data_choices <- reactive({
		    id_tmp = id_option[["Phenotype data"]]
			if(!is.null(custom_metadata)){
				id_tmp[["Custom metadata"]]$all = sort(colnames(custom_metadata()[-1]))
				id_tmp[["Custom metadata"]]$default = sort(colnames(custom_metadata()[-1]))[1]
			}
			id_tmp[[input$phenotype_data]]
		})
		updateVirtualSelect(
		  "phenotype_data_id",
		  choices = phenotype_data_choices()$all,
		  selected = phenotype_data_choices()$default
		)
	})

	add_level2 = reactive({
	  switch(input$data_L1,
	    `Molecular profile` = input$genomic_profile,
	    `Tumor index` = input$tumor_index,
	    `Immune Infiltration` = input$immune_infiltration,
	    `Pathway activity` = input$pathway_activity,
	    # `Custom metadata` = input$custom_metadata
		`Phenotype data` = input$phenotype_data

	  )
	})
	add_level3 = reactive({
	  switch(input$data_L1,
	    `Molecular profile` = input$genomic_profile_id,
	    `Tumor index` = input$tumor_index_id,
	    `Immune Infiltration` = input$immune_infiltration_id,
	    `Pathway activity` = input$pathway_activity_id,
	    # `Custom metadata` = input$custom_metadata_id
		`Phenotype data` = input$phenotype_data_id

		
	  )
	})


	add_phes <- reactiveValues(cancers = "ACC",click=0, name = list(), label=list(), show="")

	observe({
		if(!is.null(cancers)){
			add_phes$cancers = cancers()
		}
	})
	observeEvent(input$button_phe_add, {
		add_phes$click = add_phes$click+1	
		add_phes$name = c(add_phes$name, 
			paste0(input$data_L1,"--",add_level2(),"--",add_level3()))
		add_phes$label = c(add_phes$label, 
			paste0("Condition_",add_phes$click))
		add_phes$show = paste0(add_phes$show,
			"Condition_",add_phes$click,": ", input$data_L1,"--",add_level2(),
			"--",add_level3(),"\n")
	}) 
	observeEvent(input$button_phe_add_reset, {
		add_phes$click = 0	
		add_phes$name = list()
		add_phes$label=list()
		add_phes$show=""
	  	dynamic_condi$sum = 0
	}) 
	output$add_info = renderPrint({cat(add_phes$show)})

	# 下载已添加的变量数据
	add_phes_dat = eventReactive(input$button_phe_add, {

		x_tmp = lapply(seq(add_phes$name), function(i){
			L1_x = str_split(add_phes$name[[i]], "--")[[1]][1]   # Level-1
			L2_x = str_split(add_phes$name[[i]], "--")[[1]][2]    # Level-2
			L3_x = str_split(add_phes$name[[i]], "--")[[1]][3] # Level-3
			print(add_phes$name[[i]])
			print(c(L1_x, L2_x, L3_x))

			if(is.null(opt_pancan)){
				opt_pancan = .opt_pancan
			} else {
				opt_pancan = opt_pancan()
			}
			if(database=="toil"){
				if(!exists("tcga_value_nonomics")){
					tcga_value_nonomics = load_data("v2_tcga_value_nonomics")
				}
				clinical_phe = tcga_clinical_fine
				x_data = UCSCXenaShiny:::query_general_value(L1_x, L2_x, L3_x, database,
								tcga_value_nonomics, opt_pancan,custom_metadata())
			} else if(database=="pcawg"){
				if(!exists("pcawg_value_nonomics")){
					pcawg_value_nonomics = load_data("v2_pcawg_value_nonomics")
				}
				clinical_phe = pcawg_info_fine
				x_data = UCSCXenaShiny:::query_general_value(L1_x, L2_x, L3_x, database,
								pcawg_value_nonomics, opt_pancan,custom_metadata())
			} else if (database=="ccle"){
				if(!exists("ccle_value_nonomics")){
					ccle_value_nonomics = load_data("v2_ccle_value_nonomics")
				}
				clinical_phe = ccle_info_fine
				x_data = UCSCXenaShiny:::query_general_value(L1_x, L2_x, L3_x, database,
								ccle_value_nonomics, opt_pancan,custom_metadata())
			}
			# x_data = x_data[,c("sample","value")]
			# 以clinical_phe作为最终的背景人群参考
			x_data = clinical_phe[,"Sample"] %>% 
			  dplyr::rename(sample=Sample) %>% 
			  dplyr::mutate(value=x_data$value[match(sample, x_data$Sample)]) 

			colnames(x_data)[2] = add_phes$label[[i]]
			x_data %>%
				tibble::column_to_rownames("sample")

		}) %>% do.call(cbind, .)
		x_tmp %>% tibble::rownames_to_column("sample")
	})


	## step2观察表型
	output$filter_phe_01_by.ui = renderUI({
		selectInput(ns("filter_phe_01_by"), NULL,
			choices = colnames(add_phes$phe_primary)[-1:-2], selected = "Gender"
		)
	})
	# 初始表型/添加表型
	observe({
		if(add_phes$click==0){
			add_phes$phe_primary = query_tcga_group(
				database = database,
				cancer = add_phes$cancers, 
				return_all = T)
		} else {
			add_phes$phe_primary = query_tcga_group(
				database = database,
				cancer = add_phes$cancers, custom = add_phes_dat(),
				return_all = T)
		}
	})
	# 观察当前表型的分布
	output$filter_phe_01_out = renderPrint({

		filter_phe_01_out_tmp = as.data.frame(add_phes$phe_primary[,input$filter_phe_01_by,drop=FALSE])

    	if(is.null(input$filter_phe_01_by)) return(NULL)

	    if(class(filter_phe_01_out_tmp[,input$filter_phe_01_by,drop=TRUE])=="numeric"){
	      summary(filter_phe_01_out_tmp[,input$filter_phe_01_by,drop=TRUE])
	    } else {
	      table(filter_phe_01_out_tmp[,input$filter_phe_01_by,drop=TRUE])
	    }

	})

	# filter--phe--step3
	dynamic_condi = reactiveValues(add = 0, del = 0, sum = 0)
	observeEvent(input$add_condi,{
		dynamic_condi$add = dynamic_condi$add + 1
		dynamic_condi$sum = dynamic_condi$add - dynamic_condi$del
	})
	observeEvent(input$del_condi,{
		dynamic_condi$del = dynamic_condi$del + 1

		if(dynamic_condi$del >= dynamic_condi$add){
			dynamic_condi$del = dynamic_condi$add
		}
		dynamic_condi$sum = dynamic_condi$add - dynamic_condi$del
	})

	# 更新：个性化添加/减少条件
	output$multi_condi.ui = renderUI({
    	if(dynamic_condi$sum == 0) return()
    	inputTagList <- tagList()
    	lapply(1:dynamic_condi$sum,function(i){
    		# item1(id) -- phenotype(label)
    		id_condi_item1 = paste0("item1_",i)
    		label_condi_item1 = paste0("Phe-",i)

    		# item2(id) -- threshold(label)
    		id_condi_item2 = paste0("item2_",i)
    		choices_condi_item2 = unique(add_phes$phe_primary[,
    			ifelse(is.null(input[[id_condi_item1]]),"Gender",input[[id_condi_item1]]),drop=T])
    		label_condi_item2 = paste0("Thres-",i)

    		# item3(id) -- direction(label)
    		id_condi_item3 = paste0("item3_",i)
    		if(class(choices_condi_item2)=="character"){
    			choices_condi_item3 = c("+", "-")
			} else {
				choices_condi_item3 = c(">", "<", "%>", "%<")
			}
    		label_condi_item3 = paste0("Direc-",i)

    		# 添加新筛选器，不重置旧数据
			new_phe <- "Gender"
			if (id_condi_item1 %in% names(input)) {
				new_phe <- input[[id_condi_item1]]
			}

			new_thres = NULL
			if (id_condi_item2 %in% names(input)) {
				new_thres <- input[[id_condi_item2]]
			}

			new_direc <- NULL
			if (id_condi_item3 %in% names(input)) {
				new_direc <- input[[id_condi_item3]]
			}

			## 设置3个输入控件
			dynamic_input = fluidRow(
				column(
					5, 
					selectInput(ns(id_condi_item1), NULL,#label_condi_item1, 
	    						colnames(add_phes$phe_primary)[-1:-2], selected=new_phe)),
				column(
					2, 
					selectInput(ns(id_condi_item3),  NULL,#label_condi_item3, 
	    						choices_condi_item3, selected=new_direc)),
				column(
					5, 
					if(class(choices_condi_item2)=="character"){
						selectInput(ns(id_condi_item2), NULL,#label_condi_item2, 
		    						sort(choices_condi_item2,na.last = T), 
		    						selected=new_thres, multiple = T)
					} else {
						numericInput(ns(id_condi_item2), NULL,# label_condi_item2, 
							value = new_thres)
					}
				)
			)

	    	# phe_Input <- selectInput(id_condi_item1, label_condi_item1, 
	    	# 						 c("Option 1", "Option 2", "Option 3"), selected=new_phe)

			inputTagList <<- tagAppendChild(inputTagList,       
			                              	dynamic_input)
			
    	})
    	inputTagList
	})


	filter_by_phe = eventReactive(input$button_phe_filter, {
		if(dynamic_condi$sum==0){
			NULL
		} else {
			lapply(1:dynamic_condi$sum,function(i){
				# 验证非空
				if(input[[paste0("item3_",i)]] %in% c("+","-")){
					logic_item2_blank = is.null(input[[paste0("item2_",i)]])
				} else {
					logic_item2_blank = is.na(input[[paste0("item2_",i)]])
				}
				shiny::validate(
					need(!logic_item2_blank, "Please input the non-null threshold."),
				)
				# 验证范围（百分位数）
				if(input[[paste0("item3_",i)]] %in% c("%>","%<")){
					logic_item2_quant = input[[paste0("item2_",i)]]>1 | input[[paste0("item2_",i)]]<0
					shiny::validate(
						need(!logic_item2_quant, "Please inspect the threshold range (0~1) when %> or %<.")
					)
				}

			    condi_item1 = input[[paste0("item1_",i)]]  #表型
			    condi_item2 = paste(input[[paste0("item2_",i)]],collapse="|") #阈值
			    condi_item3 = input[[paste0("item3_",i)]] #方向
			    c(condi_item1,condi_item2,condi_item3)
		    })
		}
	})
	output$filter_by_phe_prompt = renderPrint({
		if(inherits(filter_by_phe(),"list")){
			cat("Above conditions have been executed.")
		} else {
			filter_by_phe()
		}
	}) 

	# 重置筛选条件
	observeEvent(input$button_phe_filter_reset, {
	  dynamic_condi$add = 0
	  dynamic_condi$del = 0
	  dynamic_condi$sum = 0
	  add_phes$filter_phe_id = NULL
	})
	# 提取筛选条件



	# 获取筛选后的样本ID
	observeEvent(input$button_phe_filter, {
		shiny::validate(
			need(try(dynamic_condi$sum > 0), 
				"Please set at least one condition."),
		)
		if(add_phes$click==0){
			add_phes$filter_phe_id = query_tcga_group(database = database,cancer = add_phes$cancers,
				filter_by = filter_by_phe(),)[["data"]]$Sample
		} else {
			add_phes$filter_phe_id = query_tcga_group(database = database,cancer = add_phes$cancers,
				custom = add_phes_dat(), filter_by = filter_by_phe())[["data"]]$Sample
		}
		output$filter_phe_02_out = renderText({
			if(is.null(add_phes$filter_phe_id)){
				paste0("No sample was filtered.")
			} else {
				paste0(length(add_phes$filter_phe_id), " samples were left.")
			}		
		})
	})

	return(reactive(add_phes$filter_phe_id))

}

# options(shiny.autoreload = TRUE)
# Step1: Global setting ---------------------------------------------------------
options(shiny.fullstacktrace=TRUE)
xena.runMode <- getOption("xena.runMode", default = "client")
message("Run mode: ", xena.runMode)
# 'client' for personal user, 'server' for running on server.

# Path for storing dataset files
if (is.null(getOption("xena.cacheDir"))) {
  options(xena.cacheDir = switch(xena.runMode,
                                 client = file.path(tempdir(), "UCSCXenaShiny"), 
                                 server = "~/.xenashiny"
  ))
}
XENA_DEST <- path.expand(file.path(getOption("xena.cacheDir"), "datasets"))
if (!dir.exists(XENA_DEST)) {
  dir.create(XENA_DEST, recursive = TRUE)
}

options(shiny.maxRequestSize=1024*1024^2) #maximum file size upload

# Step2: Load necessary packages & data & functions ----------------------------------
message("Checking dependencies...")
source(system.file("shinyapp/utils_pkgs.R", package = "UCSCXenaShiny"))
source(system.file("shinyapp/utils_appdata.R", package = "UCSCXenaShiny"))
source(system.file("shinyapp/utils_func.R", package = "UCSCXenaShiny"))
source(system.file("shinyapp/utils_plot.R", package = "UCSCXenaShiny")) #TPC pipelines plot

# Put modules here --------------------------------------------------------
message("Loading modules and UIs...")
modules_path <- system.file("shinyapp", "modules", package = "UCSCXenaShiny", mustWork = TRUE)
modules_file <- dir(modules_path, pattern = "\\.R$", full.names = TRUE, recursive = TRUE)
sapply(modules_file, function(x, y) source(x, local = y), y = environment())

# Put page UIs here -----------------------------------------------------
pages_path <- system.file("shinyapp", "ui", package = "UCSCXenaShiny", mustWork = TRUE)
pages_file <- dir(pages_path, pattern = "\\.R$", full.names = TRUE, recursive = TRUE)
sapply(pages_file, function(x, y) source(x, local = y), y = environment())

source(system.file("shinyapp/app_preset.R", package = "UCSCXenaShiny"))

# Step4: Run APP
# UI part ----------------------------------------------------------------------
message("Starting...")
ui <- tagList(
  tags$head(
    tags$title("XenaShiny"),
    # tags$link(rel = "icon", type="image/svg+xml", href = "./icon.svg"),
    tags$link(rel = "icon", type="image/png", href = "./icon.png"),
    tags$link(rel = "stylesheet", type = "text/css", href = "./css/global.css")
  ),

  useWaiter(), 
  waiterPreloader(html = tagList(
    spin_fading_circles(), 
    br(), br(),
    h1(strong("Welcome to use UCSCXenaShiny v2 application!")),
    br(),
    p("An interactive web tool with general and personalized modules to explore UCSC Xena datasets"
      ,style = "font-size: 25px;"),
    br(),br(),
    p("Notes:", "(1) The initiation could take about 10 seconds. (2) Please zoom in or up screen for better representation.",
      style = "font-size: 16px;")
  ), color = "#2C3E50"),

  shinyjs::useShinyjs(),
  autoWaiter(html = spin_loader(), color = transparent(0.5)), # change style https://shiny.john-coene.com/waiter/

  navbarPage(
    id = "navbar",
    title = "UCSCXenaShiny v2",
    windowTitle = "UCSCXenaShiny",
    # inst/shinyapp/ui
    ui.page_home(),
    ui.page_repository(),
    ui.page_pancan_quick(),
    ui.page_pancan_tcga(),
    ui.page_general_analysis(),
    ui.page_download(),
    ui.page_PharmacoGenomics(),
    ui.page_help(),
    ui.page_developers(),
    footer = ui.footer(),
    collapsible = TRUE,
    theme = tryCatch(shinythemes::shinytheme("flatly"),
                     error = function(e) {
                       "Theme 'flatly' is not available, use default."
                       NULL
                     })
  )
)

# Server Part ---------------------------------------------------------------
server <- function(input, output, session) {
  message("Shiny app run successfully! Enjoy it!\n")
  message("               --  Xena shiny team\n")
  # Stop warn
  storeWarn <- getOption("warn")
  options(warn = -1) 
  # observe(print(input$navbar))
  # inst/shinyapp/server
  source(server_file("home.R"), local = TRUE)
  source(server_file("repository.R"), local = TRUE)
  source(server_file("modules.R"), local = TRUE)
  source(server_file("general-analysis.R"), local = TRUE)
  observe_helpers(help_dir = system.file("shinyapp", "helper", package = "UCSCXenaShiny"))
}

# Run web app -------------------------------------------------------------
shiny::shinyApp(
  ui = ui,
  server = server
)

download_res_UI = function(id){
    ns = NS(id)
    tagList(
        fluidRow(
            column(3, 
                div(shinyjs::hidden(downloadButton(ns("save_plot_bt"), "Figure")), style="display: inline-block;vertical-align:top;"),
                div(style="display: inline-block;vertical-align:top",
                    dropMenu(  
                        actionBttn(ns("plot_opt"),label = NULL,style = "material-circle", 
                                    color = "success",icon = icon("gear")),
                        div(h3("1. Height:"),style="width:400px;"),
                        numericInput(ns("save_plot_H"), NULL ,min = 1, max = 20, value = 10, step = 0.5),
                        div(h3("2. Width:"),style="width:400px;"),
                        numericInput(ns("save_plot_W"), NULL ,min = 1, max = 20, value = 10, step = 0.5),	
                        div(h3("3. Format:"),style="width:400px;"),		
                        radioGroupButtons(
                            inputId = ns("save_plot_F"),
                            label = NULL,
                            status = "primary",
                            choices = c("PDF", "PNG"),
                            justified = TRUE
                        ),
                        placement = "top"
                    )
                )
            ),
            column(3, offset = 0, shinyjs::hidden(downloadButton(ns("save_data_raw"), "Raw data(.csv)"))),
            column(3, offset = 0, shinyjs::hidden(downloadButton(ns("save_data_res"), "Statistical data(.csv)")))
        )
    )
}


download_res_Server = function(input, output, session,
                               res1=NULL, res2=NULL, res3=NULL){
    ns <- session$ns

	# three download buttons
    observeEvent(req(res1),{
        shinyjs::show("save_plot_bt")
    })
    observeEvent(req(res2),{
        shinyjs::show("save_data_raw")
    })
    observeEvent(req(res3),{
        shinyjs::show("save_data_res")
    })

	output$save_plot_bt = downloadHandler(
		filename = function(){
			paste0("Plot", "_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".",tolower(input$save_plot_F))
		},
		content = function(file){
			# p = cor_plot_bar()
            p = res1
		    if (input$save_plot_F == "PDF") {
		      pdf(file, width = input$save_plot_W, height = input$save_plot_H, onefile = FALSE)
		      print(p)
		      dev.off()
		    } else if (input$save_plot_F == "PNG"){
		      png(file, width = input$save_plot_W, height = input$save_plot_H, res = 600, units = "in")
		      print(p)
		      dev.off()
		    }
		}
	)

	output$save_data_raw = downloadHandler(
		filename = function(){
			paste0("Rawdata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv")
		},
		content = function(file){
			# p_raw = merge_data_bar()
            p_raw = res2
			write.csv(p_raw, file, row.names = FALSE)
		}
	)
	output$save_data_res = downloadHandler(
		filename = function(){
			paste0("Statdata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv")
		},
		content = function(file){
			# p_cor = cor_data_bar()
			# p_cor$parameter1 = unique(merge_data_bar()$x_axis)
			# p_cor$parameter2 = unique(merge_data_bar()$y_axis)	
            p_stat = res3
			write.csv(p_stat, file, row.names = FALSE)
		}
	)


}

ui.page_help <- function() {

  navbarMenu(
    title = "Help",
    icon = icon("question-circle"),
    # tabPanel(
    #   "Usage",
    #   fluidPage(
    #     includeMarkdown(set_md_path("usage.md"))
    #   )
    # ),
    tabPanel(
      "Quick Guide",
      fluidPage(
        #tags$iframe(style="height:1000px; width:100%", src="Quick_guide.pdf")
        tags$iframe(style="height:1000px; width:100%", src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Quick_guide.pdf")
      )
    ),
    tabPanel(title = a("Tutorial Book", 
               href="https://lishensuo.github.io/UCSCXenaShiny_Book/",
               target="_blank")),
    tabPanel(
      "TPC ID Query",
      ui.modules_id_reference("modules_id_reference")
    ),

    # tabPanel(a("Package doc", 
    #            href="https://openbiox.github.io/UCSCXenaShiny/index.html",
    #            target="_blank")),
    tabPanel(
      "Citation",
      fluidPage(
        includeMarkdown(set_md_path("citation2.md"))
      )
    ),
    # tabPanel(
    #   "Extra Datasets",
    #   fluidPage(
    #     includeMarkdown(set_md_path("datasets.md"))
    #   )
    # ),
    tabPanel(a("UCSCXenaShiny v1", 
               href="https://shiny.zhoulab.ac.cn/UCSCXenaShinyV1/",
               target="_blank")
    )
  )
}

### load data
## mut
OP_mut <- UCSCXenaShiny::load_data("OP_mut")
ccle_mut <- OP_mut$ccle
gdsc_mut <- OP_mut$gdsc
gCSI_mut <- OP_mut$gCSI

## cnv
OP_cnv <- UCSCXenaShiny::load_data("OP_cnv")
ccle_cnv <- OP_cnv$ccle
gdsc_cnv <- OP_cnv$gdsc
gCSI_cnv <- OP_cnv$gCSI

## mRNA,exp
OP_exp <- UCSCXenaShiny::load_data("OP_exp")
ccle_exp <- OP_exp$ccle
gdsc_exp <- OP_exp$gdsc

## meth
OP_meth <- UCSCXenaShiny::load_data("OP_meth")
ccle_meth <- OP_meth$ccle

## fusion
OP_fusion <- UCSCXenaShiny::load_data("OP_fusion")
ccle_fusion <- OP_fusion$ccle

## protein
OP_protein <- UCSCXenaShiny::load_data("OP_protein")
ccle_protein <- OP_protein$ccle

## drug,anno
OP_drug <- UCSCXenaShiny::load_data("OP_drug")
ctrp1_drug <- OP_drug$ctrp1
ctrp2_drug <- OP_drug$ctrp2
gCSI_drug <- OP_drug$gCSI
gdsc1_drug <- OP_drug$gdsc1
gdsc2_drug <- OP_drug$gdsc2
prism_drug <- OP_drug$prism

OP_anno <- UCSCXenaShiny::load_data("OP_anno")
cell_anno <- OP_anno$cell
drug_anno <- OP_anno$drug

## plot or preplot
OP_stat_plot <- UCSCXenaShiny::load_data("OP_stat_plot")
p_count_drugandcell <- OP_stat_plot$count_drugandcell
p_count_subtype <- OP_stat_plot$count_subtype
p_overlap_cell <- OP_stat_plot$overlap_cell
p_overlap_drug <- OP_stat_plot$overlap_drug

OP_drug_sens_profile <- UCSCXenaShiny::load_data("OP_drug_sens_profile")
gdsc1_ms <- OP_drug_sens_profile$ms_gdsc1
gdsc2_ms <- OP_drug_sens_profile$ms_gdsc2
prism_ms <- OP_drug_sens_profile$ms_prism
gCSI_ms <- OP_drug_sens_profile$ms_gCSI
ctrp1_ms <- OP_drug_sens_profile$ms_ctrp1
ctrp2_ms <- OP_drug_sens_profile$ms_ctrp2
gdsc1_tsne <- OP_drug_sens_profile$tsne_gdsc1
gdsc2_tsne <- OP_drug_sens_profile$tsne_gdsc2
prism_tsne <- OP_drug_sens_profile$tsne_prism
gCSI_tsne <- OP_drug_sens_profile$tsne_gCSI
ctrp1_tsne <- OP_drug_sens_profile$tsne_ctrp1
ctrp2_tsne <- OP_drug_sens_profile$tsne_ctrp2

## misc and preprocess
# source("inst/shinyapp/modules/PharmacoGenomics/Preprocess.R")
OP_misc <- UCSCXenaShiny::load_data("OP_misc")
omics_search <- OP_misc$omics_search
drugs_search <- OP_misc$drugs_search
drugs_search2 <- OP_misc$drugs_search2
profile_vec_list <- OP_misc$profile_vec_list
profile_vec_list$drug <- profile_vec_list$drug[!profile_vec_list$drug %in% c("OP_sens_plot")]

## clean OP*
rm(list = ls()[grepl("OP", ls())])

## some preprocess for Drug-Omics Correlation Analysis
tmp <- list()

tmp$omic_sel <- c("exp", "meth", "protein", "cnv")
tmp$tmp1 <- ls()[grepl("_drug$", ls())]
tmp$drug_vec <- gsub("_drug", "", tmp$tmp1[!grepl("^p_", tmp$tmp1)])
omics_search_list1 <- list()

for(i in tmp$omic_sel){
  i2 <- paste0("_", i)
  tmp$omic_vec <- gsub(i2, "", ls()[grepl(i2, ls())])
  tmp_list <- list()
  for(x in tmp$omic_vec){
    # x = tmp$omic_vec[1]
    for(y in tmp$drug_vec){
      # y = tmp$drug_vec[1]
      # select identical cells
      omic <- base::get(paste0(x, i2))
      drug <- base::get(paste0(y, "_drug"))
      intersected_cells <- intersect(colnames(omic), colnames(drug))
      omic <- omic[,match(intersected_cells, colnames(omic))]
      drug <- drug[,match(intersected_cells, colnames(drug))]
      tmp_list[[paste0(x, "_", y)]] <- list(
        "omic" = omic,
        "drug" = drug
      )
    }
  }
  omics_search_list1[[i]] <- tmp_list
}
names(omics_search_list1)[names(omics_search_list1) %in% "exp"] <- "mRNA"

### Discrete
tmp$tmp2 <- ls()[grepl("_mut$", ls())]
gCSI_mut$mutation <- NA
gCSI_mut$genes_muts <- NA
for(i in tmp$tmp2){
  omic <- base::get(i)
  omic_gene <- omic[,c(1,2)] %>% unique()
  omic_site <- omic[,c(4,2)] %>% unique()
  i2 <- paste0(gsub("mut", "", i), "mutation_gene")
  i3 <- paste0(gsub("mut", "", i), "mutation_site")
  assign(i2, omic_gene)
  assign(i3, omic_site)
}
rm(gCSI_mutation_site)

tmp$omic_sel2 <- c("mutation_gene", "mutation_site", "fusion")
tmp$tmp1 <- ls()[grepl("_drug$", ls())]
tmp$drug_vec <- gsub("_drug", "", tmp$tmp1[!grepl("^p_", tmp$tmp1)])
omics_search_list2 <- list()
for(i in tmp$omic_sel2){
  i2 <- paste0("_", i)
  tmp$omic_vec <- gsub(i2, "", ls()[grepl(i2, ls())])
  tmp_list <- list()
  for(x in tmp$omic_vec){
    # x = tmp$omic_vec[1]
    for(y in tmp$drug_vec){
      # y = tmp$drug_vec[1]
      # select identical cells
      omic <- base::get(paste0(x, i2))
      drug <- base::get(paste0(y, "_drug"))
      intersected_cells <- intersect(omic$cells, colnames(drug))
      omic <- omic[omic$cells %in% intersected_cells,]
      drug <- drug[,colnames(drug) %in% intersected_cells]
      tmp_list[[paste0(x, "_", y)]] <- list(
        "omic" = omic,
        "drug" = drug
      )
    }
  }
  omics_search_list2[[i]] <- tmp_list
}

## Function for ProfileDrugSens
plotMADandMedian <- function(ms, dataset){
  ms$Dataset <- dataset
  p <- ggplot(data = ms, 
              aes(text = Name, x = Mad,
                  y = Median, label = Target, label2 = Dataset)) +
    geom_point(alpha=0.4, size=3.5, 
               aes(color=Phase)) + theme_bw() + 
    scale_color_manual(values = paletteer::paletteer_d("ggsci::default_igv")) + 
    labs(x = "MAD", y = "Median") + 
    theme(
      axis.title = element_text(size = 15),
      title = element_text(size = 15, face = "bold"),
      axis.text = element_text(size = 12), 
      legend.text = element_text(size = 12)
    )
  return(p)
}

plotTSNE <- function(df, dataset){
  df$Dataset <- dataset
  p <- ggplot(data = df, 
              aes(text = Name, x = TSNE1,
                  y = TSNE2, label = Target, label2 = Dataset)) +
    geom_point(alpha=0.4, size=3.5, 
               aes(color=Phase)) + theme_bw() + 
    scale_color_manual(values = paletteer::paletteer_d("ggsci::default_igv")) + 
    labs(x = "TSNE1", y = "TSNE2") + 
    theme(
      axis.title = element_text(size = 15),
      title = element_text(size = 15, face = "bold"),
      axis.text = element_text(size = 12), 
      legend.text = element_text(size = 12)
    )
  return(p)
}

ui.modules_1_tcga_01 = function(id){
    ns = NS(id)
    main_ui = tagList(
        mol_quick_select_UI(ns("id"), "tcga", c("mRNA","transcript","methylation","miRNA")),

        h4("3. Select analysis mode"),
        shinyWidgets::prettyRadioButtons(
            inputId = ns("Mode"), label = NULL,
            choiceValues = c("Pan-cancer", "Single-cancer"),
            choiceNames = c("Pan-cancer", "Single-cancer"),
            animation = "jelly"
        ),
        tabsetPanel(
            id = ns("Mode_params"),
            type = "hidden",
            tabPanel("Single-cancer",
            selectInput(ns("Cancer"), "Select one cancer",sort(tcga_names))
            ),
            tabPanel("Pan-cancer")
        ),
        h4("4. Include GTEx normal samples"),
        switchInput(
            inputId = ns("pdist_dataset"),
            value = TRUE,
            onLabel = "Yes",
            offLabel = "No"
        ),
        shinyWidgets::actionBttn(
            inputId = ns("search_bttn"),
            label = "Go!",
            style = "gradient",
            icon = icon("search"),
            color = "primary",
            block = TRUE,
            size = "sm"
        )
    )
    out_ui = tagList(
        fluidRow(
            uiOutput(ns("gene_pancan_dist")),
        )
    )
    side_ui = tagList(
        fluidRow(
            column(6,
                h4("1. Visualization parameters"),
                h5("(1) Geometry type:"),
                awesomeRadio(ns("pdist_mode"), label = NULL, 
                    choices = c("Boxplot", "Violinplot"), selected = "Boxplot", inline = TRUE),
                h5("(2) Label type:"),
                awesomeRadio(ns("pdist_show_p"), label = NULL, 
                    choices = c("None", "P value", "P label"), selected = "None", inline = TRUE),
                h5("(3) Colors:"),
                colourpicker::colourInput(inputId = ns("tumor_col"), "Tumor sample color", "#DF2020"),
                colourpicker::colourInput(inputId = ns("normal_col"), "Normal sample color", "#DDDF21"),
                h5("(4) ggplot theme:"),
                selectInput(inputId = ns("theme"), label = "Select theme for plot", 
                            choices = names(themes_list), selected = "Cowplot"),
            ),
            column(6,
                h4("2. Download options"),
                h5("(1) Figure:"),
                numericInput(inputId = ns("height"), label = "Height", value = 5),
                numericInput(inputId = ns("width"), label = "Width", value = 12),
                awesomeRadio(ns("device"), label = "Format", 
                    choices = c("pdf", "png"), selected = "pdf", inline = TRUE),
                downloadBttn(
                  outputId = ns("download_1"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                ),
                h5("(2) Data table:"),
                downloadBttn(
                  outputId = ns("download_2"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                )
            )
        )
    )
    fluidPage(
        wellPanel(style = "height:725px",
        box(main_ui,
            width = 5,
            solidHeader = TRUE,
            title = "Quick TCGA Analysis: Compare between tumor and normal", 
            status = "primary",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Click the bottom button to execute/update the analysis."
        ),
        box(out_ui,
            width = 7,
            solidHeader = TRUE,
            title = "Analytical results:", 
            status = "primary",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Pull the sidebar to adjust plot parameters or download results through the top-right widget.",
            sidebar = boxSidebar(
                        id = ns("sidebar"),
                        width = 50,
                        side_ui
            )
        )
        )
    )
}



server.modules_1_tcga_01 = function(input, output, session){
    ns = session$ns

    observeEvent(input$Mode, {
        updateTabsetPanel(inputId = "Mode_params", selected = input$Mode)
    })

	
    mol_info = callModule(mol_quick_select_Server, "id", "tcga")


    mark_p = reactiveValues(Show.P.value=FALSE, Show.P.label=FALSE)
    observe({
        if(input$pdist_show_p=="P value"){
            mark_p$Show.P.value = TRUE
        } else if (input$pdist_show_p=="P label") {
           mark_p$Show.P.label = TRUE
        }
    })
    observeEvent(input$Mode, {
        if(input$Mode=="Pan-cancer"){
            updateAwesomeRadio(session, "pdist_mode", label = NULL, 
                    choices = c("Boxplot", "Violinplot"), selected = "Boxplot", inline = TRUE)
        } else {
            updateAwesomeRadio(session, "pdist_mode", label = NULL, 
                    choices = c("Dotplot", "Violinplot"), selected = "Violinplot", inline = TRUE)
        }
    })


    plot_func <- eventReactive(input$search_bttn, {
        id <- showNotification(h3("The task is running..."), duration = NULL, closeButton = FALSE, type = "message")
        on.exit(removeNotification(id), add = TRUE)  #reactive语句执行完毕时，运行remove命令
        print(input$Mode)
        p = switch(input$Mode,
        `Pan-cancer` =  vis_toil_TvsN(
                            Gene = mol_info$molecule(),
                            data_type = mol_info$profile(),
                            Mode = input$pdist_mode,
                            Show.P.value = mark_p$Show.P.value,
                            Show.P.label = mark_p$Show.P.label,
                            TCGA.only = !input$pdist_dataset,
                            values = c(input$tumor_col, input$normal_col)
                        ) + themes_list[[input$theme]] + 
                            ggplot2::theme(
                                axis.text.x = element_text(angle = 45, hjust = .5, vjust = .5),
                                text = element_text(size = 20)),                        
        `Single-cancer` = vis_toil_TvsN_cancer(
                            Gene = mol_info$molecule(),
                            data_type = mol_info$profile(),
                            Cancer = input$Cancer,
                            Mode = input$pdist_mode,
                            Show.P.value = mark_p$Show.P.value,
                            Show.P.label = mark_p$Show.P.label,
                            TCGA.only = !input$pdist_dataset,
                            values = c(input$tumor_col, input$normal_col)
                        ) + themes_list[[input$theme]] +
                            ggplot2::theme(text = element_text(size = 20),
                                           legend.position = "none")
        )
        return(p)
    })

    w <- waiter::Waiter$new(id = ns("plot"), html = waiter::spin_hexdots(), color = "black")
    observeEvent(input$search_bttn,{
        # check whether valid out plot
        shinyjs::disable("search_bttn")
        chect_plot = is.null(plot_func()) 
        if(chect_plot){
            sendSweetAlert(session, title = "Warning", type = "error", text = "Please select a valid molecule.")
            req(chect_plot)
        }
        output$gene_pancan_dist <- renderUI({
            w$show()
            if(isolate(input$Mode)=="Pan-cancer"){
                output$plot = renderPlot(plot_func())
                fluidRow(
                    column(10, offset = 1,
                        plotOutput(ns("plot"), height = "580px"),
                    )
                )
            } else {
                output$plot = renderPlot(plot_func())
                fluidRow(
                    column(6, offset = 3,
                        plotOutput(ns("plot"), height = "580px"),
                    )
                )
            }
        })    
        shinyjs::enable("search_bttn")
    })

    output$download_1 <- downloadHandler(
        filename = function() {
            paste0(mol_info$molecule(), "_", mol_info$profile(), "_tumorVSnormal.", input$device)
        },
        content = function(file) {
            p <- plot_func()
            if (input$device == "pdf") {
                pdf(file, width = input$width, height = input$height)
                print(p)
                dev.off()
            } else {
                png(file, width = input$width, height = input$height, res = 600, units = "in")
                print(p)
                dev.off()
            }
        }
    )

    output$download_2 <- downloadHandler(
        filename = function() {
            paste0(mol_info$molecule(), "_", mol_info$profile(), "_tumorVSnormal.csv")
        },
        content = function(file) {
        data = plot_func()$data %>%
            dplyr::rename('Cancer'='tissue', 'Dataset'='dataset', 'Sample'='sample',
            'Group'='type2', 'Value'="value") %>%
            dplyr::select(Cancer,Dataset,Sample,Group,Value) %>%
            dplyr::arrange(Cancer, desc(Dataset), Sample, desc(Group))
        write.csv(data, file, row.names = FALSE)
        }
    )

} 

#' Visualize Identifier-Identifier Correlation
#'
#' NOTE: the dataset must be dense matrix in UCSC Xena data hubs.
#'
#' @param id1 the first molecule identifier.
#' @param id2 the second molecule identifier.
#' @param dataset1 the dataset to obtain `id1`.
#' @param dataset2 the dataset to obtain `id2`.
#' @param samples default is `NULL`, can be common sample names for two datasets.
#' @param use_ggstats if `TRUE`, use [ggstatsplot](https://github.com/IndrajeetPatil/ggstatsplot) package for plotting.
#' @param use_simple_axis_label if `TRUE` (default), use simple axis labels.
#' Otherwise, data subtype will be labeled.
#' @param line_color set the color for regression line.
#' @param alpha set the alpha for dots.
#' @param ... other parameters passing to [ggscatter](http://rpkgs.datanovia.com/ggpubr/reference/ggscatter.html).
#' @return a (gg)plot object.
#' @export
#' @examples
#' \dontrun{
#' dataset <- "TcgaTargetGtex_rsem_isoform_tpm"
#' id1 <- "TP53"
#' id2 <- "KRAS"
#' vis_identifier_cor(dataset, id1, dataset, id2)
#'
#' samples <- c(
#'   "TCGA-D5-5538-01", "TCGA-VM-A8C8-01",
#'   "TCGA-ZN-A9VQ-01", "TCGA-EE-A17X-06",
#'   "TCGA-05-4420-01"
#' )
#' vis_identifier_cor(dataset, id1, dataset, id2, samples)
#'
#' dataset1 <- "TCGA-BLCA.htseq_counts.tsv"
#' dataset2 <- "TCGA-BLCA.gistic.tsv"
#' id1 <- "TP53"
#' id2 <- "KRAS"
#' vis_identifier_cor(dataset1, id1, dataset2, id2)
#' }
vis_identifier_cor <- function(dataset1, id1, dataset2, id2, samples = NULL,
                               use_ggstats = FALSE,
                               use_simple_axis_label = TRUE,
                               line_color = "blue", alpha = 0.5, ...) {
  stopifnot(length(id1) == 1, length(id2) == 1)

  if (dataset1 == "custom_feature_dataset") {
    id1_value <- query_custom_feature_value(id1)
  } else {
    id1_value <- query_molecule_value(dataset1, id1)
  }

  if (dataset1 == "custom_feature_dataset") {
    id2_value <- query_custom_feature_value(id2)
  } else {
    id2_value <- query_molecule_value(dataset2, id2)
  }

  df <- dplyr::inner_join(
    dplyr::tibble(
      sample = names(id1_value),
      X = as.numeric(id1_value)
    ),
    dplyr::tibble(
      sample = names(id2_value),
      Y = as.numeric(id2_value)
    ),
    by = "sample"
  )

  if (!is.null(samples)) {
    df <- dplyr::filter(df, .data$sample %in% samples)
  }

  if (!use_ggstats) {
    eval(parse(text = "library(ggpubr)"))
    p <- do.call("ggscatter", list(
      data = df,
      x = "X", y = "Y",
      xlab = if (use_simple_axis_label) id1 else paste0(id1, "(", attr(id1_value, "label"), ")"),
      ylab = if (use_simple_axis_label) id2 else paste0(id2, "(", attr(id2_value, "label"), ")"),
      alpha = alpha,
      add = "reg.line",
      add.params = list(color = line_color, fill = "lightgray"),
      cor.coef = TRUE, ...
    ))
  } else {
    if (!requireNamespace("ggstatsplot")) {
      install.packages("ggstatsplot")
    }
    eval(parse(text = "library(ggstatsplot)"))
    p <- do.call("ggscatterstats", list(
      data = df,
      x = "X", y = "Y",
      xlab = if (use_simple_axis_label) id1 else paste0(id1, "(", attr(id1_value, "label"), ")"),
      ylab = if (use_simple_axis_label) id2 else paste0(id2, "(", attr(id2_value, "label"), ")"),
      ...
    ))
  }
  p
}

#' Visualize Correlation for Multiple Identifiers
#'
#' NOTE: the dataset must be dense matrix in UCSC Xena data hubs.
#'
#' @inheritParams ggstatsplot::ggcorrmat
#' @inheritParams vis_identifier_cor
#' @param ids the molecule identifiers.
#' @param dataset the dataset to obtain identifiers.
#' @param color_low the color code for lower value mapping.
#' @param color_high the color code for higher value mapping.
#' @param ... other parameters passing to [ggstatsplot::ggcorrmat].
#' @export
#' @return a (gg)plot object.
#' @examples
#' \dontrun{
#' dataset <- "TcgaTargetGtex_rsem_isoform_tpm"
#' ids <- c("TP53", "KRAS", "PTEN")
#' vis_identifier_multi_cor(dataset, ids)
#' }
vis_identifier_multi_cor <- function(dataset, ids, samples = NULL,
                                     matrix.type = c("full", "upper", "lower"),
                                     type = c("parametric", "nonparametric", "robust", "bayes"),
                                     partial = FALSE,
                                     sig.level = 0.05,
                                     p.adjust.method = c("holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none"),
                                     color_low = "#E69F00",
                                     color_high = "#009E73",
                                     ...) {
  stopifnot(length(ids) >= 2)
  colors <- c(color_low, "white", color_high)
  matrix.type <- match.arg(matrix.type)
  type <- match.arg(type)
  p.adjust.method <- match.arg(p.adjust.method)

  df <- purrr::map(ids, function(x) {
    message("Querying data of identifier ", x, " from dataset: ", dataset)
    data <- if (dataset == "custom_feature_dataset") {
      query_custom_feature_value(x)
    } else {
      query_molecule_value(dataset, x)
    }
    data <- dplyr::tibble(
      sample = names(data),
      y = as.numeric(data)
    )
    colnames(data)[2] <- x
    data
  }) %>%
    purrr::reduce(dplyr::full_join, by = "sample")

  if (!is.null(samples)) {
    df <- dplyr::filter(df, .data$sample %in% samples)
  }

  if (!requireNamespace("ggstatsplot")) {
    install.packages("ggstatsplot")
  }

  p <- ggstatsplot::ggcorrmat(
    data = df,
    matrix.type = matrix.type,
    type = type,
    partial = partial,
    sig.level = sig.level,
    p.adjust.method = p.adjust.method,
    ...
  )

  p
}

#' Visualize Comparison of an Molecule Identifier between Groups
#'
#' NOTE: the dataset must be dense matrix in UCSC Xena data hubs.
#'
#' @inheritParams ggstatsplot::ggbetweenstats
#' @inheritParams vis_identifier_cor
#' @param id the molecule identifier.
#' @param dataset the dataset to obtain identifiers.
#' @param fun_type select the function to compare groups.
#' @param grp_df When `dataset` and `id` are all not `NULL`, it should be a `data.frame` with 2 or 3 columns.
#' - The first column refers to sample ID.
#' - The second column refers to groups indicated in axis X.
#' - The third column is optional, which indicates facet variable.
#' When any of `dataset` and `id` is `NULL`, it should be a `data.frame` with 3 or 4 columns.
#' - The first column refers to sample ID.
#' - The second column refers to values indicated in axis Y.
#' - The third column refers to groups indicated in axis X.
#' - The fourth column is optional, which indicates facet variable.
#' @param pairwise.comparisons whether pairwise comparison
#' @param ... other parameters passing to [ggstatsplot::ggbetweenstats] or [ggstatsplot::ggwithinstats].
#' @export
#' @return a (gg)plot object.
#' @examples
#' \dontrun{
#' library(UCSCXenaTools)
#' expr_dataset <- "TCGA.LUAD.sampleMap/HiSeqV2_percentile"
#' cli_dataset <- "TCGA.LUAD.sampleMap/LUAD_clinicalMatrix"
#' id <- "TP53"
#' cli_df <- XenaGenerate(
#'   subset = XenaDatasets == "TCGA.LUAD.sampleMap/LUAD_clinicalMatrix"
#' ) %>%
#'   XenaQuery() %>%
#'   XenaDownload() %>%
#'   XenaPrepare()
#'
#' # group data.frame with 2 columns
#' vis_identifier_grp_comparison(expr_dataset, id, cli_df[, c("sampleID", "gender")])
#' # group data.frame with 3 columns
#' vis_identifier_grp_comparison(
#'   expr_dataset, id,
#'   cli_df[, c("sampleID", "pathologic_M", "gender")] %>%
#'     dplyr::filter(pathologic_M %in% c("M0", "MX"))
#' )
#'
#' # When not use the value of `identifier` from `dataset`
#' vis_identifier_grp_comparison(grp_df = cli_df[, c(1, 2, 71)])
#' vis_identifier_grp_comparison(grp_df = cli_df[, c(1, 2, 71, 111)])
#' }
#'
vis_identifier_grp_comparison <- function(dataset = NULL, id = NULL, grp_df, samples = NULL,
                                          fun_type = c("betweenstats", "withinstats"),
                                          type = c("parametric", "nonparametric", "robust", "bayes"),
                                          pairwise.comparisons = TRUE,
                                          p.adjust.method = c("holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none"),
                                          ggtheme = cowplot::theme_cowplot(),
                                          ...) {
  stopifnot(ncol(grp_df) > 1)
  fun_type <- match.arg(fun_type)
  type <- match.arg(type)
  p.adjust.method <- match.arg(p.adjust.method)
  colnames(grp_df)[1] <- "sample"

  if (!is.null(dataset) && !is.null(id)) {
    message("Querying data of identifier ", id, " from dataset ", dataset, " for comparison")
    id_value <- if (dataset == "custom_feature_dataset") {
      query_custom_feature_value(id)
    } else {
      query_molecule_value(dataset, id)
    }
    df <- dplyr::tibble(
      sample = names(id_value),
      X = as.numeric(id_value)
    )
    colnames(df)[2] <- id

    df <- dplyr::inner_join(df, grp_df, by = "sample")

    do_grp <- ncol(grp_df) >= 3
  } else {
    message("Directly use 'grp_df' for comparison analysis.")
    df <- grp_df
    do_grp <- ncol(grp_df) >= 4
  }

  if (!is.null(samples)) {
    df <- dplyr::filter(df, .data$sample %in% samples)
  }

  if (!requireNamespace("ggstatsplot")) {
    install.packages("ggstatsplot")
  }

  if (do_grp) {
    fun <- if (fun_type == "betweenstats") ggstatsplot::grouped_ggbetweenstats else ggstatsplot::grouped_ggwithinstats
    # colnames(df)[4] = "grouping.var"
    p <- fun(
      data = df,
      x = !!colnames(df)[3],
      y = !!colnames(df)[2],
      # grouping.var = grouping.var,
      grouping.var = !!rlang::sym(colnames(df)[4]),
      type = type,
      pairwise.comparisons = pairwise.comparisons,
      p.adjust.method = p.adjust.method,
      ggtheme = ggtheme,
      ...
    )
  } else {
    fun <- if (fun_type == "betweenstats") ggstatsplot::ggbetweenstats else ggstatsplot::ggwithinstats

    p <- fun(
      data = df,
      x = !!colnames(df)[3],
      y = !!colnames(df)[2],
      type = type,
      pairwise.comparisons = pairwise.comparisons,
      p.adjust.method = p.adjust.method,
      ggtheme = ggtheme,
      ...
    )
  }

  p
}

#' Visualize Identifier Group Survival Difference
#'
#' NOTE: the dataset must be dense matrix in UCSC Xena data hubs.
#'
#' @inheritParams tcga_surv_plot
#' @inheritParams vis_identifier_grp_comparison
#' @param surv_df a `data.frame`. The "time" should be in unit of "days".
#' - If there are 3 columns, the names should be "sample", "time", "status".
#' - If there are 4 columns, the names should be "sample", "value", "time", "status".
#' @param cutoff_mode mode for grouping samples, can be "Auto" (default) or "Custom" or "None" (for groups have been prepared).
#' @export
#' @return a (gg)plot object.
#' @examples
#' \dontrun{
#' library(UCSCXenaTools)
#' expr_dataset <- "TCGA.LUAD.sampleMap/HiSeqV2_percentile"
#' cli_dataset <- "TCGA.LUAD.sampleMap/LUAD_clinicalMatrix"
#' id <- "KRAS"
#' cli_df <- XenaGenerate(
#'   subset = XenaDatasets == "TCGA.LUAD.sampleMap/LUAD_clinicalMatrix"
#' ) %>%
#'   XenaQuery() %>%
#'   XenaDownload() %>%
#'   XenaPrepare()
#'
#' # Use individual survival data
#' surv_df1 <- cli_df[, c("sampleID", "ABSOLUTE_Ploidy", "days_to_death", "vital_status")]
#' surv_df1$vital_status <- ifelse(surv_df1$vital_status == "DECEASED", 1, 0)
#' vis_identifier_grp_surv(surv_df = surv_df1)
#'
#' # Use both dataset argument and vis_identifier_grp_surv(surv_df = surv_df1)
#' surv_df2 <- surv_df1[, c(1, 3, 4)]
#' vis_identifier_grp_surv(expr_dataset, id, surv_df = surv_df2)
#' vis_identifier_grp_surv(expr_dataset, id,
#'   surv_df = surv_df2,
#'   cutoff_mode = "Custom", cutpoint = c(25, 75)
#' )
#' }
vis_identifier_grp_surv <- function(dataset = NULL,
                                    id = NULL,
                                    surv_df,
                                    samples = NULL,
                                    cutoff_mode = c("Auto", "Custom", "None"),
                                    cutpoint = c(50, 50),
                                    palette = "aaas",
                                    ...) {
  cutoff_mode <- match.arg(cutoff_mode)

  if (!is.null(dataset) && !is.null(id)) {
    message("Querying data of identifier ", id, " from dataset ", dataset, " for survival analysis")
    id_value <- if (dataset == "custom_feature_dataset") {
      query_custom_feature_value(id)
    } else {
      query_molecule_value(dataset, id)
    }
    df <- dplyr::tibble(
      sample = names(id_value),
      value = as.numeric(id_value)
    )

    if (ncol(surv_df) == 3) {
      colnames(surv_df) <- c("sample", "time", "status")
    } else {
      stop("When only input both 'dataset' and 'surv_df', please make sure that your 'surv_df' have 3 columns with order 'sample', 'time', 'status'")
    }

    df <- dplyr::inner_join(df, surv_df, by = "sample")
  } else {
    message("Directly use 'surv_df' for survival analysis.")
    df <- surv_df
    if (ncol(df) == 4) {
      colnames(df) <- c("sample", "value", "time", "status")
    } else {
      stop("When only input 'surv_df', please make sure that you have 4 columns with order 'sample', 'value', 'time', 'status'")
    }
  }

  if (!is.null(samples)) {
    df <- dplyr::filter(df, .data$sample %in% samples)
  }

  if (cutoff_mode != "None") {
    p <- sur_plot(df, cutoff_mode, cutpoint, palette = palette, ...)
  } else {
    colnames(df)[2] <- "group"
    p <- p_survplot(df, palette = palette, ...)
  }

  p
}

ui.modules_2_pcawg_04 = function(id){
    ns = NS(id)

    main_ui = tagList(
        mol_quick_select_UI(ns("id"), "pcawg", 
                            c("mRNA","miRNA","promoter","fusion","APOBEC")),
        shinyWidgets::actionBttn(
            inputId = ns("search_bttn"),
            label = "Go!",
            style = "gradient",
            icon = icon("search"),
            color = "primary",
            block = TRUE,
            size = "sm"
        )
    )
    out_ui = tagList(
        fluidRow(
            uiOutput(ns("unicox_gene_tree"))
        )
    )
    side_ui = tagList(
        fluidRow(
            column(6,
                h4("1. Visualization parameters"),
                h5("(1) Color palette:"),
                colourpicker::colourInput(inputId = ns("first_col"), "First color", "#6A6F68"),
                colourpicker::colourInput(inputId = ns("second_col"), "Second color", "#E31A1C"),
                colourpicker::colourInput(inputId = ns("third_col"), "Third color", "#377DB8"),

            ),
            column(6,
                h4("2. Download options"),
                h5("(1) Figure:"),
                numericInput(inputId = ns("height"), label = "Height", value = 8),
                numericInput(inputId = ns("width"), label = "Width", value = 8),
                awesomeRadio(ns("device"), label = "Format", 
                    choices = c("pdf", "png"), selected = "pdf", inline = TRUE),
                downloadBttn(
                  outputId = ns("download_1"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                ),
                h5("(2) Data table:"),
                downloadBttn(
                  outputId = ns("download_2"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                )
            )
        )
    )
    fluidPage(
        wellPanel(style = "height:725px",
        box(main_ui,
            width = 5,
            solidHeader = TRUE,
            title = "Quick PCAWG Analysis: Univariate Cox regression survival analysis", 
            status = "info",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Click the bottom button to execute/update the analysis."
        ),
        box(out_ui,
            width = 7,
            solidHeader = TRUE,
            title = "Analytical results:", 
            status = "info",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Pull the sidebar to adjust plot parameters or download results through the top-right widget.",
            sidebar = boxSidebar(
                        id = ns("sidebar"),
                        width = 50,
                        side_ui
            )
        )
        )
    )
}

server.modules_2_pcawg_04 = function(input, output, session){
    ns = session$ns

    mol_info = callModule(mol_quick_select_Server, "id", "pcawg")

    plot_func = eventReactive(input$search_bttn, {
        id <- showNotification(h3("The task is running..."), duration = NULL, closeButton = FALSE, type = "message")
        on.exit(removeNotification(id), add = TRUE)  #reactive语句执行完毕时，运行remove命令
        p <- vis_pcawg_unicox_tree(
            Gene = mol_info$molecule(),
            measure = "OS",
            data_type = mol_info$profile(),
            values = c(input$first_col, input$second_col, input$third_col)
        )
        pdata <- p$data %>% 
            as.data.frame() %>%
            dplyr::select(cancer, measure, n_contrast, n_ref, beta, HR_log, lower_95_log, upper_95_log, Type, p.value)
        return(list(plot = p, data = pdata))
    })

    # Show waiter for plot
    w <- waiter::Waiter$new(id = ns("unicox_gene_tree"), html = waiter::spin_hexdots(), color = "black")
    observeEvent(input$search_bttn,{
        shinyjs::disable("search_bttn")
        # check whether valid out plot
        chect_plot = is.null(plot_func()) 
        if(chect_plot){
            sendSweetAlert(session, title = "Warning", type = "error", text = "Please select a valid molecule.")
            req(chect_plot)
        }
        output$unicox_gene_tree <- renderUI({
            w$show()
            output$plot = renderPlot(plot_func()$plot)
            fluidRow(
                column(8, offset = 2,
                    plotOutput(ns("plot"), height = "580px"),
                )
            )
        }) 
        shinyjs::enable("search_bttn")
    })

    output$download_1 <- downloadHandler(
        filename = function() {
            paste0(mol_info$molecule(), "_", mol_info$profile(), "_pcawg_unicox.", input$device)
        },
        content = function(file) {
            p <- plot_func()$plot
            if (input$device == "pdf") {
                pdf(file, width = input$width, height = input$height)
                print(p)
                dev.off()
            } else {
                png(file, width = input$width, height = input$height, res = 600, units = "in")
                print(p)
                dev.off()
            }
        }
    )

    output$download_2 <- downloadHandler(
        filename = function() {
            paste0(mol_info$molecule(), "_", mol_info$profile(), "_pcawg_unicox.csv")
        },
        content = function(file) {
            data =  plot_func()$data %>%
                dplyr::rename('Project'='cancer', 'Event'='measure','Samples'='n_ref',
                'P.value' = 'p.value') %>%
                dplyr::select(Project,Event,Samples,HR_log,lower_95_log,upper_95_log,Type,P.value)
            write.csv(data, file, row.names = FALSE)
        }
    )
}

ui.modules_pancan_cross_pw_o2m = function(id) {
	ns = NS(id)
	fluidPage(
		fluidRow(
			# 初始设置
			column(
				3,
				wellPanel(
					style = "height:1100px",
					h2("S1: Preset", align = "center"),

					h4(strong("S1.1 Modify datasets"),"[Only for S1.3]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Modify datasets", 
					                   content = "data_origin"),
					mol_origin_UI(ns("mol_origin2cor"), database = "toil"),

					h4(strong("S1.2 Choose cancers")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Cancer types", 
					                   content = "tcga_types"),
					pickerInput(
						ns("choose_cancers"), NULL,
						choices = sort(tcga_names),
						multiple = TRUE,
						selected = sort(tcga_names),
						options = list(`actions-box` = TRUE)
					),

				    br(),

					h4(strong("S1.3 Filter samples"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Filter samples", 
					                   content = "choose_samples"),
					h5("Quick filter:"),
					pickerInput(
						ns("filter_by_code"), NULL,
						choices = NULL, selected =  NULL,
						multiple = TRUE, options = list(`actions-box` = TRUE)
					),
					h5("Exact filter:"),
					filter_samples_UI(ns("filter_samples2cor"), database = "toil"),
					br(),
					verbatimTextOutput(ns("filter_phe_id_info")),
					br()
				)
			),
			# 下载X轴数据
			column(
				3,
				wellPanel(
					style = "height:1100px",
					h2("S2: Get data", align = "center"),
					# 调用下载模块UI
					h4(strong("S2.1 Select one pathway")),
                    virtualSelectInput(
                        inputId = ns("pw_id"), 
                        label = NULL,
                        choices = list(
                            HALLMARK = paste0("HALLMARK_",tcga_id.list[["HM"]]),
                            KEGG = paste0("KEGG_",tcga_id.list[["KEGG"]]),
                            IOBR = paste0("IOBR_",tcga_id.list[["IOBR"]])
                        ),
                        selected = "HALLMARK_ADIPOGENESIS", 
                        width = "100%",
                        search = TRUE,
                        dropboxWidth = "200%"
                    ),
					br(),
				)
			),
			# 分析/绘图/下载
			column(
				6,
				wellPanel(
					# h2("S3: Analyze & Visualize", align = "center"),
					h2("S3: Analyze & Visualize", align = "center") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Analyze & Visualize", 
					                   content = "cross_pw"), 
					style = "height:1100px",
					# h4(strong("S3.1 Set analysis parameters")), 
					shinyWidgets::actionBttn(
						ns("step3_plot"), "Run (Visualize)",
				        style = "gradient",
				        icon = icon("chart-line"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					),
					br(),
					fluidRow(
						column(12, offset = 0,
							   plotOutput({ns("funky_plot")}, height = "700px") 
						)
					),
					br(),
					fluidRow(
						column(3,
							div(shinyjs::hidden(downloadButton(ns("save_plot_bt"), "Figure")), style="display: inline-block;vertical-align:top;"),
							div(style="display: inline-block;vertical-align:top",
								dropMenu(  
									actionBttn(ns("plot_opt"),label = NULL,style = "material-circle", 
												color = "success",icon = icon("gear")),
									div(h3("1. Height:"),style="width:400px;"),
									numericInput(ns("save_plot_H"), NULL ,min = 1, max = 20, value = NULL, step = 0.1),
									div(h3("2. Width:"),style="width:400px;"),
									numericInput(ns("save_plot_W"), NULL ,min = 1, max = 20, value = NULL, step = 0.1),	
									div(h3("3. Format:"),style="width:400px;"),		
									radioGroupButtons(
										inputId = ns("save_plot_F"),
										label = NULL,
										status = "primary",
										choices = c("PDF", "PNG"),
										justified = TRUE
									),
									placement = "top"
								)
							)
						),
						column(3, offset = 0, shinyjs::hidden(downloadButton(ns("save_data_res"), "Statistical data(.csv)")))
					)
				)
			)
		)
	)
}


server.modules_pancan_cross_pw_o2m = function(input, output, session) {
	ns <- session$ns

	# 记录选择癌症
	cancer_choose <- reactiveValues(name = "ACC", phe_primary="",
		filter_phe_id=query_tcga_group(database = "toil", cancer = "BRCA", return_all = T))
	observe({
		cancer_choose$name = input$choose_cancers
		cancer_choose$phe_primary <- query_tcga_group(database = "toil", cancer = cancer_choose$name, return_all = T)
	})
	
	# 数据源设置
	opt_pancan = callModule(mol_origin_Server, "mol_origin2cor", database = "toil")

    custom_meta_sig = reactive(NULL)
    sig_dat = reactive(NULL)

	## 过滤样本
	# exact filter module
	filter_phe_id = callModule(filter_samples_Server, "filter_samples2cor",
					   database = "toil",
					   cancers=reactive(cancer_choose$name),
					   custom_metadata=reactive(custom_meta_sig()),
					   opt_pancan = reactive(opt_pancan()))
	# quick filter widget
	observe({
		code_types_valid = code_types[names(code_types) %in% 
							unique(cancer_choose$phe_primary$Code)]
		code_types_valid = setdiff(code_types_valid, "NT (normal tissue)")
		updatePickerInput(
			session,
			"filter_by_code",
			choices = unlist(code_types_valid,use.names = F),
			selected =  unlist(code_types_valid,use.names = F)
		)
	})

	# 综合上述二者
	observe({
		# quick filter
		choose_codes = names(code_types)[unlist(code_types) %in% input$filter_by_code]
		filter_phe_id2 = cancer_choose$phe_primary %>%
			dplyr::filter(Code %in% choose_codes) %>%
			dplyr::pull("Sample")

		# exact filter
		if(is.null(filter_phe_id())){
			cancer_choose$filter_phe_id = filter_phe_id2
		} else {
			cancer_choose$filter_phe_id = intersect(filter_phe_id2,filter_phe_id())
		}

		output$filter_phe_id_info = renderPrint({
			cat(paste0("Tip: ", length(cancer_choose$filter_phe_id), " samples are retained"))
		})
	})


	plot_func = eventReactive(input$step3_plot,{
        shinyjs::disable("step3_plot")
		res = vis_pathway_cross_omics(input$pw_id,
                                     tumor_projects = cancer_choose$name,
                                     tumor_samples = cancer_choose$filter_phe_id,
							         return_list = TRUE)
        shinyjs::enable("step3_plot")
        res
	})

	output$funky_plot = renderPlot({plot_func()$plot})

	# three download buttons
    observeEvent(input$step3_plot, {
        shinyjs::show("save_plot_bt")
        shinyjs::show("save_data_res")
		updateNumericInput(session, "save_plot_H", value = plot_func()$plot$height)
		updateNumericInput(session, "save_plot_W", value = plot_func()$plot$width)
    })

	output$save_plot_bt = downloadHandler(
		filename = function(){
			paste0("Plot", "_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".",tolower(input$save_plot_F))
		},
		content = function(file){
            p = plot_func()$plot
		    if (input$save_plot_F == "PDF") {
		      pdf(file, width = input$save_plot_W, height = input$save_plot_H, onefile = FALSE)
		      print(p)
		      dev.off()
		    } else if (input$save_plot_F == "PNG"){
		      png(file, width = input$save_plot_W, height = input$save_plot_H, res = 600, units = "in")
		      print(p)
		      dev.off()
		    }
		}
	)
	output$save_data_res = downloadHandler(
		filename = function(){
			paste0("Statdata_",format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".csv")
		},
		content = function(file){
            p_stat = plot_func()$data
			p_stat = apply(p_stat, 2, as.character) #list column
			write.csv(p_stat, file, row.names = FALSE)
		}
	)

}

#' Visualize Pan-cancer TPM (tumor (TCGA) vs Normal (TCGA & GTEx))
#' @import ggplot2 dplyr tibble
#' @param Gene a molecular identifier (e.g., "TP53") or a formula specifying
#' genomic signature (`"TP53 + 2 * KRAS - 1.3 * PTEN"`).
#' @param Mode "Boxplot" or "Violinplot" to represent data
#' @param Show.P.value `TRUE` or `FALSE` whether to count P value
#' @param Method default method is wilcox.test
#' @param Show.P.label `TRUE` or `FALSE` present p value with number or label `*`, `**`, `***` and `****`
#' @param values the color to fill tumor or normal
#' @param TCGA.only include samples only from TCGA dataset
#' @param draw_quantiles draw quantiles for violinplot
#' @param trim whether trim the violin
#' @param data_type choose gene profile type,
#' including "mRNA", "transcript", "protein", "mutation", "cnv", "methylation", "miRNA".
#' @param include.Tumor.only if `TRUE`, include "UVM" and "MESO" these two types
#' with matched normals samples.
#' @param opt_pancan specify one dataset for some molercular profiles
#' @return a `ggplot` object
#' @examples
#' \dontrun{
#' p <- vis_toil_TvsN(Gene = "TP53", Mode = "Violinplot", Show.P.value = FALSE, Show.P.label = FALSE)
#' p <- vis_toil_TvsN(Gene = "TP53", Mode = "Boxplot", Show.P.value = FALSE, Show.P.label = FALSE)
#' }
#' @export
#'
vis_toil_TvsN <- function(Gene = "TP53", Mode = c("Boxplot", "Violinplot"),
                          data_type = "mRNA", Show.P.value = TRUE,
                          Show.P.label = TRUE, Method = c("wilcox.test", "t.test"),
                          values = c("#DF2020", "#DDDF21"),
                          TCGA.only = FALSE, draw_quantiles = c(0.25, 0.5, 0.75),
                          trim = TRUE, include.Tumor.only = FALSE,
                          opt_pancan = .opt_pancan) {
  tcga_gtex <- load_data("tcga_gtex")

  Mode <- match.arg(Mode)
  Method <- match.arg(Method)

  if (!Method %in% c("wilcox.test", "t.test")) {
    stop("only support wilcox.test or t.test")
  }

  if (!Mode %in% c("Boxplot", "Violinplot")) {
    stop("only support Boxplot or Violinplot")
  }

  if (!data_type %in% c("mRNA", "miRNA", "transcript", "methylation")) {
    stop("data_type ", data_type, " does not support in this function!")
  }
  t1 <- query_pancan_value(Gene, data_type = data_type, opt_pancan = opt_pancan)
  if (is.null(t1[[1]])) {
    warning("No data available", immediate. = TRUE)
    return(NULL)
  }
  unit <- switch(data_type,
    cnv = NULL,
    mutation = NULL,
    t1[[2]]
  )
  if (is.list(t1)) t1 <- t1[[1]]

  if (all(is.na(t1))) {
    message("All NAs returned, return NULL instead.")
    return(NULL)
  }

  tcga_gtex <- tcga_gtex %>%
    dplyr::group_by(.data$tissue) %>%
    dplyr::distinct(.data$sample, .keep_all = TRUE)

  t2 <- t1 %>%
    as.data.frame() %>%
    dplyr::rename("value" = ".") %>%
    tibble::rownames_to_column(var = "sample") %>%
    dplyr::inner_join(tcga_gtex, by = "sample")
  tumorlist <- unique(tcga_gtex[tcga_gtex$type2 == "tumor", ]$tissue)
  normallist <- unique(tcga_gtex[tcga_gtex$type2 == "normal", ]$tissue)
  withoutNormal <- setdiff(tumorlist, normallist)
  tcga_gtex <- t2 %>% dplyr::select("value", "tissue", "type2", "sample")
  tcga_gtex$type2 <- factor(tcga_gtex$type2, levels = c("tumor", "normal"))
  tcga_gtex_withNormal <- tcga_gtex[!(tcga_gtex$tissue %in% withoutNormal), ]
  tcga_gtex_withNormal <- tcga_gtex_withNormal %>%
    dplyr::mutate(dataset = ifelse(stringr::str_sub(.data$sample, 1, 4) == "TCGA", "TCGA", "GTEX"))
  if (include.Tumor.only) {
    tcga_gtex_MESO <- tcga_gtex[tcga_gtex$tissue == "MESO", ]
    tcga_gtex_UVM <- tcga_gtex[tcga_gtex$tissue == "UVM", ]
  }
  if (TCGA.only == TRUE) {
    tcga_gtex_withNormal <- tcga_gtex_withNormal %>% dplyr::filter(.data$dataset == "TCGA")
  }
  if (Show.P.value == FALSE) {
    Show.P.label <- FALSE
  }
  if (Show.P.value == TRUE) {
    message("Counting P value")
    pv <- tcga_gtex_withNormal %>%
      ggpubr::compare_means(value ~ type2, data = ., method = Method, group.by = "tissue")
    pv <- pv %>% dplyr::select(c("tissue", "p", "p.signif", "p.adj"))
    message("Counting P value finished")
  }
  if (Mode == "Boxplot") {
    p <- ggplot2::ggplot(tcga_gtex_withNormal, aes_string(x = "tissue", y = "value", fill = "type2")) +
      ggplot2::geom_boxplot() +
      ggplot2::xlab(NULL) +
      ggplot2::theme_set(theme_set(theme_classic(base_size = 20))) +
      ggplot2::theme(axis.text.x = element_text(angle = 45, hjust = .5, vjust = .5)) +
      ggplot2::guides(fill = guide_legend(title = NULL)) +
      ggplot2::theme(
        legend.background = element_blank(),
        legend.position = c(0, 0), legend.justification = c(0, 0)
      ) +
      ggplot2::scale_fill_manual(values = values)
    if (include.Tumor.only) {
      p <- p + ggplot2::geom_boxplot(data = tcga_gtex_MESO) +
        ggplot2::geom_boxplot(data = tcga_gtex_UVM)
    }

    p <- p + ggplot2::ylab(
      if (is.null(unit)) Gene else paste0(Gene, " (", unit, ")")
    )

    if (Show.P.value == TRUE & Show.P.label == TRUE) {
      p <- p + ggplot2::geom_text(
        aes(
          x = .data$tissue,
          y = max(tcga_gtex_withNormal$value) * 1.1,
          label = .data$p.signif
        ),
        data = pv,
        inherit.aes = FALSE
      )
    }
    if (Show.P.value == TRUE & Show.P.label == FALSE) {
      p <- p + ggplot2::geom_text(
        aes(
          x = .data$tissue,
          y = max(tcga_gtex_withNormal$value) * 1.1,
          label = as.character(signif(.data$p, 2))
        ),
        data = pv,
        inherit.aes = FALSE
      )
    }
  }
  if (Mode == "Violinplot") {
    p <- ggplot2::ggplot(
      tcga_gtex_withNormal, aes_string(x = "tissue", y = "value", fill = "type2")
    ) +
      geom_split_violin(
        draw_quantiles = draw_quantiles,
        trim = trim,
        linetype = "solid",
        color = "black",
        size = 0.2,
        na.rm = TRUE,
        position = "identity"
      ) +
      ggplot2::xlab("") +
      ggplot2::scale_fill_manual(values = values) +
      ggplot2::theme_set(ggplot2::theme_classic(base_size = 20)) +
      ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 45, hjust = .5, vjust = .5)) +
      ggplot2::guides(fill = ggplot2::guide_legend(title = NULL)) +
      ggplot2::theme(
        legend.background = ggplot2::element_blank(),
        legend.position = c(0, 0), legend.justification = c(0, 0)
      )

    if (include.Tumor.only) {
      p <- p + geom_split_violin(
        data = tcga_gtex_MESO,
        mapping = aes_string(x = "tissue", y = "value", fill = "type2"),
        draw_quantiles = draw_quantiles,
        trim = trim,
        linetype = "solid",
        color = "black",
        size = 0.2,
        na.rm = TRUE,
        position = "identity"
      ) +
        geom_split_violin(
          data = tcga_gtex_UVM,
          mapping = ggplot2::aes_string(x = "tissue", y = "value", fill = "type2"),
          draw_quantiles = draw_quantiles,
          trim = trim,
          linetype = "solid",
          color = "black",
          size = 0.2,
          na.rm = TRUE,
          position = "identity"
        )
    } #+
    # ggplot2::scale_x_discrete(limits = levels(tcga_gtex$tissue))
    p <- p + ggplot2::ylab(
      if (is.null(unit)) Gene else paste0(Gene, " (", unit, ")")
    )

    if (Show.P.value == TRUE & Show.P.label == TRUE) {
      p <- p + ggplot2::geom_text(
        ggplot2::aes(
          x = .data$tissue,
          y = max(tcga_gtex_withNormal$value) * 1.1,
          label = .data$p.signif
        ),
        data = pv,
        inherit.aes = FALSE
      )
    }
    if (Show.P.value == TRUE & Show.P.label == FALSE) {
      p <- p + ggplot2::geom_text(
        ggplot2::aes(
          x = .data$tissue,
          y = max(tcga_gtex_withNormal$value) * 1.1,
          label = as.character(signif(.data$p, 2))
        ),
        data = pv,
        inherit.aes = FALSE
      )
    }
  }
  return(p)
}

#' Visualize Single Gene Univariable Cox Result from Toil Data Hub
#'
#' @inheritParams vis_toil_TvsN
#' @param measure a survival measure, e.g. "OS".
#' @param data_type choose gene profile type, including "mRNA","transcript","methylation","miRNA","protein","mutation","cnv"
#' @param use_optimal_cutoff use `surv_cutpoint` from survminer package for
#' thresholding samples in each cancer type.
#' @return a `ggplot` object
#' @examples
#' \dontrun{
#' p <- vis_unicox_tree(Gene = "TP53")
#' }
#' @export
vis_unicox_tree <- function(Gene = "TP53", measure = "OS", data_type = "mRNA", use_optimal_cutoff = FALSE,
                            values = c("grey", "#E31A1C", "#377DB8"), opt_pancan = .opt_pancan) {
  tcga_surv <- load_data("tcga_surv")
  tcga_gtex <- load_data("tcga_gtex")

  t1 <- query_pancan_value(Gene, data_type = data_type, opt_pancan = opt_pancan)
  if (is.null(t1[[1]])) {
    warning("No data available", immediate. = TRUE)
    return(NULL)
  }
  if (is.list(t1)) t1 <- t1[[1]]
  if (all(is.na(t1))) {
    message("All NAs returned, return NULL instead.")
    return(NULL)
  }

  # we filter out normal tissue
  tcga_gtex <- tcga_gtex %>% dplyr::filter(.data$type2 != "normal")

  message(paste0("Get data value for ", Gene))
  # NA values could appear, especially in signature calculation:
  # For instance, patient-1 expressed GeneA, but not expressed GeneB, then
  # signature GeneA + GeneB will be NA for patient-1
  s <- data.frame(sample = names(t1), values = t1) %>%
    dplyr::filter(!is.na(.data$values)) 
  ## we use median cutoff here
  ss <- s %>%
    dplyr::filter(!is.na(.data$values)) %>% 
    dplyr::inner_join(tcga_surv, by = "sample") %>%
    dplyr::inner_join(tcga_gtex[, c("tissue", "sample")], by = "sample")
  sss <- split(ss, ss$tissue)
  # discard cancer types with constant values
  sss = sss[sapply(sss, function(x){stats::sd(x$values)!=0})]
  tissues <- names(sss)
  .f <- function(cancer) {
    sss_can <- sss[[cancer]]
    # By default, cox analysis based on continuous molecule values. HR>0: Higher values, More risky. 
    # Set use_optimal_cutoff as TRUE: Divide into 2 groups (Low/High) according the optimal cutoff. Then, 
    ## cox analysis based on discrete molecule groups. HR>0: 'High' Group are more risky.
    if (use_optimal_cutoff) {
      sss_can <- sss_can %>%
        survminer::surv_cutpoint(
          time = paste0(measure, ".time"), event = measure,
          variables = c("values"),
          minprop = 0.25, progressbar = TRUE
        ) %>%
        survminer::surv_categorize(labels = c("Low", "High")) %>%
        data.frame() %>%
        dplyr::mutate(values = factor(values, levels = c("Low", "High")))
    }

    unicox_res_genes <- ezcox::ezcox(
      sss_can %>%
        dplyr::select(c("values", paste0(measure, ".time"), measure)) %>%
        na.omit(),
      covariates = "values",
      time = paste0(measure, ".time"),
      status = measure,
      verbose = FALSE
    )

    unicox_res_genes$cancer <- cancer
    unicox_res_genes$measure <- measure
    return(unicox_res_genes)
  }
  unicox_res_all_cancers <- purrr::map(tissues, purrr::safely(.f)) %>% magrittr::set_names(tissues)

  unicox_res_all_cancers <- unicox_res_all_cancers %>%
    purrr::map(~ .x$result) %>%
    purrr::compact()
  unicox_res_all_cancers_df <- do.call(rbind.data.frame, unicox_res_all_cancers)
  unicox_res_all_cancers_df <- unicox_res_all_cancers_df %>%
    dplyr::mutate(HR_log = log(.data$HR)) %>%
    dplyr::mutate(lower_95_log = log(.data$lower_95)) %>%
    dplyr::mutate(upper_95_log = log(.data$upper_95)) %>%
    dplyr::mutate(Type = ifelse(.data$p.value < 0.05 & .data$HR_log > 0, "Risky", ifelse(.data$p.value < 0.05 & .data$HR_log < 0, "Protective", "NS"))) %>%
    dplyr::mutate(Type = factor(Type, levels = c("NS", "Risky", "Protective")))
  ## visualization
  p <- unicox_res_all_cancers_df %>%
    dplyr::mutate(cancer = factor(.data$cancer, levels = rev(.data$cancer))) %>%
    ggplot2::ggplot(
      aes_string(x = "cancer", y = "HR_log", ymin = "lower_95_log", ymax = "upper_95_log", color = "Type")
    ) +
    ggplot2::theme_bw() +
    ggplot2::geom_pointrange() +
    ggplot2::coord_flip() +
    ggplot2::labs(x = "", y = "ln (Hazard Ratio)") +
    ggtitle(paste0(Gene, if (startsWith(data_type, "mRNA") | startsWith(data_type, "miRNA")) " Expression" else "")) +
    ggplot2::theme(
      axis.text.x = element_text(color = "black"),
      axis.text.y = element_text(color = "black"),
      panel.grid.major = element_blank(),
      panel.grid.minor = element_blank(),
      plot.title = element_text(hjust = 0.5)
    ) +
    ggplot2::scale_color_manual(values = values) +
    ggplot2::geom_hline(yintercept = c(0), linetype = "dashed")
  return(p)
}

#' Visualize Single Gene Expression in Anatomy Location
#'
#' @inheritParams vis_toil_TvsN
#' @inheritParams ggplot2::scale_colour_viridis_d
#' @param Gender a string, "Female" (default) or "Male".
#' @param data_type choose gene profile type, including "mRNA","transcript","methylation","miRNA","protein","cnv"
#' @return a `ggplot` object
#' @importFrom stats complete.cases median
#' @export

vis_pancan_anatomy <- function(Gene = "TP53",
                               Gender = c("Female", "Male"),
                               data_type = "mRNA",
                               option = "D",
                               opt_pancan = .opt_pancan) {
  Gender <- match.arg(Gender)

  # Loading while pass checking
  if (eval(parse(text = "!requireNamespace('gganatogram')"))) {
    stop("Please install 'gganatogram' package firstly!")
  }
  hgMale_key <- eval(parse(text = "gganatogram::hgMale_key"))
  hgFemale_key <- eval(parse(text = "gganatogram::hgFemale_key"))
  gganatogram <- eval(parse(text = "gganatogram::gganatogram"))

  TCGA.organ <- load_data("TCGA.organ")
  tcga_gtex <- load_data("tcga_gtex")
  tcga_gtex <- tcga_gtex %>% dplyr::distinct(sample, .keep_all = TRUE)

  t1 <- query_pancan_value(Gene, data_type = data_type, opt_pancan = opt_pancan)
  if (is.null(t1[[1]])) {
    warning("No data available", immediate. = TRUE)
    return(NULL)
  }
  unit <- switch(data_type,
    cnv = NULL,
    mutation = NULL,
    t1[[2]]
  )
  if (is.list(t1)) t1 <- t1[[1]]

  if (data_type == "cnv") data_type <- "CNV"

  if (all(is.na(t1))) {
    message("All NAs returned, return NULL instead.")
    return(NULL)
  }

  message(paste0("Get data value for ", Gene))
  t2 <- t1 %>%
    as.data.frame() %>%
    dplyr::rename("value_" = ".") %>%
    tibble::rownames_to_column(var = "sample") %>%
    dplyr::inner_join(tcga_gtex, by = "sample")
  t2 <- t2 %>%
    dplyr::left_join(TCGA.organ, by = c("tissue" = "TCGA")) %>%
    mutate(group = paste(.data$tissue, .data$type2, sep = "_"))
  # Male
  Male_input <- t2 %>%
    dplyr::full_join(hgMale_key, by = "organ") %>%
    dplyr::filter(.data$organ != "") %>%
    dplyr::group_by(.data$group) %>%
    dplyr::summarise(
      Median = median(.data$value_),
      tissue = .data$tissue,
      type.x = .data$type.x,
      type = .data$type2,
      organ = .data$organ,
      color = .data$colour
    ) %>%
    dplyr::ungroup() %>%
    dplyr::distinct() %>%
    dplyr::filter(complete.cases(.)) %>%
    dplyr::mutate(value = .data$Median)
  # Female
  Female_input <- t2 %>%
    dplyr::full_join(hgFemale_key, by = "organ") %>%
    dplyr::filter(.data$organ != "") %>%
    dplyr::group_by(.data$group) %>%
    dplyr::summarise(
      Median = median(.data$value_),
      tissue = .data$tissue,
      type.x = .data$type.x,
      type = .data$type2,
      organ = .data$organ,
      color = .data$colour
    ) %>%
    dplyr::ungroup() %>%
    dplyr::distinct() %>%
    dplyr::filter(complete.cases(.)) %>%
    dplyr::mutate(value = .data$Median)
  if (Gender == "Male") {
    p <- gganatogram(
      data = Male_input,
      fillOutline = "white",
      organism = "human",
      sex = "male",
      fill = "value"
    ) +
      facet_wrap(~type) +
      labs(fill = unit) +
      coord_cartesian(ylim = c(-120, 0)) +
      theme_void(base_size = 15) +
      scale_fill_continuous(low = "#3CB371", high = "#DC143C") +
      ggtitle(paste0(data_type, " - ", Gene)) +
      labs(caption = "data source: TCGA + GTEx") +
      theme(plot.title = element_text(hjust = 0.5))

    data_input <- Male_input
  } else {
    p <- gganatogram(
      data = Female_input,
      fillOutline = "white",
      organism = "human",
      sex = "female",
      fill = "value"
    ) +
      facet_wrap(~type) +
      coord_cartesian(ylim = c(-120, 0)) +
      labs(fill = unit) +
      theme_void(base_size = 15) +
      scale_fill_continuous(low = "#3CB371", high = "#DC143C") +
      ggtitle(paste0(data_type, " - ", Gene)) +
      labs(caption = "data source: TCGA + GTEx") +
      theme(plot.title = element_text(hjust = 0.5))

    data_input <- Female_input
  }

  return(list(plot = p, data = data_input))
}

#' Heatmap for Correlation between Gene and Immune Signatures
#'
#' @inheritParams vis_toil_TvsN
#' @param cor_method correlation method
#' @param Immune_sig_type quantification method, default is "Cibersort"
#' @param Plot output the plot directly, default 'TRUE'
#' @examples
#' \dontrun{
#' p <- vis_gene_immune_cor(Gene = "TP53")
#' }
#' @export
vis_gene_immune_cor <- function(
    Gene = "TP53", cor_method = "spearman",
    data_type = "mRNA", Immune_sig_type = "Cibersort", Plot = "TRUE", opt_pancan = .opt_pancan) {
  tcga_pan_immune_signature <- load_data("tcga_pan_immune_signature")
  tcga_gtex <- load_data("tcga_gtex")

  # we filter out normal tissue
  tcga_gtex <- tcga_gtex %>% dplyr::filter(.data$type2 != "normal")

  tcga_pan_immune_signature <- tcga_pan_immune_signature %>%
    tidyr::pivot_longer(3:ncol(.), names_to = "sample", values_to = "score") %>%
    dplyr::mutate(sample = stringr::str_sub(.data$sample, 1, 15))

  t1 <- query_pancan_value(Gene, data_type = data_type, opt_pancan = opt_pancan)
  if (is.null(t1[[1]])) {
    warning("No data available", immediate. = TRUE)
    return(NULL)
  }
  if (is.list(t1)) t1 <- t1[[1]]

  if (all(is.na(t1))) {
    message("All NAs returned, return NULL instead.")
    return(NULL)
  }

  message(paste0("Get data value for ", Gene))
  s <- data.frame(sample = names(t1), values = t1)

  ss <- s %>%
    dplyr::inner_join(tcga_pan_immune_signature, by = "sample") %>%
    dplyr::inner_join(tcga_gtex[, c("tissue", "sample")], by = "sample")

  sss <- split(ss, ss$tissue)
  tissues <- names(sss)
  cor_gene_immune <- purrr::map(tissues, purrr::safely(function(cancer) {
    # cancer = "ACC"
    sss_can <- sss[[cancer]]
    ## filter cibersort data here
    sss_can_class <- sss_can %>% dplyr::filter(.data$Source == Immune_sig_type)
    cells <- unique(sss_can_class$SetName)
    cor_res_class_can <- purrr::map(cells, purrr::safely(function(i) {
      # i = cells[1]
      dd <- sss_can_class[sss_can_class$SetName == i, ]
      suppressWarnings(dd <- stats::cor.test(dd$values, dd$score, method = cor_method))
      ddd <- data.frame(gene = Gene, immune_cells = i, cor = dd$estimate, p.value = dd$p.value, stringsAsFactors = FALSE)
      return(ddd)
    })) %>% magrittr::set_names(cells)
    cor_res_class_can <- cor_res_class_can %>%
      purrr::map(~ .x$result) %>%
      purrr::compact()
    cor_res_class_can_df <- do.call(rbind.data.frame, cor_res_class_can)
    cor_res_class_can_df$cancer <- cancer
    return(cor_res_class_can_df)
  })) %>% magrittr::set_names(tissues)

  cor_gene_immune <- cor_gene_immune %>%
    purrr::map(~ .x$result) %>%
    purrr::compact()
  cor_gene_immune_df <- do.call(rbind.data.frame, cor_gene_immune)
  data <- cor_gene_immune_df
  data$pstar <- ifelse(data$p.value < 0.05,
    ifelse(data$p.value < 0.001, "***", ifelse(data$p.value < 0.01, "**", "*")),
    ""
  )

  p <- ggplot2::ggplot(data, ggplot2::aes_string(x = "cancer", y = "immune_cells")) +
    ggplot2::geom_tile(ggplot2::aes_string(fill = "cor"), colour = "white", size = 1) +
    ggplot2::scale_fill_gradient2(low = "#377DB8", mid = "white", high = "#E31A1C") +
    ggplot2::geom_text(ggplot2::aes_string(label = "pstar"), col = "black", size = 5) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      axis.title.x = ggplot2::element_blank(),
      axis.ticks.x = ggplot2::element_blank(),
      axis.title.y = ggplot2::element_blank(),
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
      axis.text.y = ggplot2::element_text(size = 8)
    ) +
    ggplot2::labs(fill = paste0(" * p < 0.05", "\n\n", "** p < 0.01", "\n\n", "*** p < 0.001", "\n\n", "Correlation")) +
    ggtitle(paste0("The correlation between ", Gene, " ", data_type, " with immune signatures"))

  if (Plot == TRUE) {
    return(p)
  } else {
    pdata <- ss
    plist <- list()
    plist[[1]] <- p
    plist[[2]] <- pdata
    names(plist) <- c("plot", "pdata")
    return(plist)
  }
}


#' Visualize Correlation between Gene and TMB (Tumor Mutation Burden)
#'
#' @inheritParams vis_gene_immune_cor
#' @examples
#' \dontrun{
#' p <- vis_gene_tmb_cor(Gene = "TP53")
#' }
#' @export
vis_gene_tmb_cor <- function(
    Gene = "TP53", cor_method = "spearman",
    data_type = "mRNA", Plot = "TRUE", opt_pancan = .opt_pancan) {
  tcga_tmb <- load_data("tcga_tmb")
  tcga_gtex <- load_data("tcga_gtex")

  t1 <- query_pancan_value(Gene, data_type = data_type, opt_pancan = opt_pancan)
  if (is.null(t1[[1]])) {
    warning("No data available", immediate. = TRUE)
    return(NULL)
  }
  if (is.list(t1)) t1 <- t1[[1]]

  if (all(is.na(t1))) {
    message("All NAs returned, return NULL instead.")
    return(NULL)
  }

  s <- data.frame(sample = names(t1), values = t1)
  ss <- s %>%
    dplyr::inner_join(tcga_tmb, by = c("sample" = "Tumor_Sample_ID")) %>%
    dplyr::inner_join(tcga_gtex[, c("tissue", "sample")], by = "sample")
  sss <- split(ss, ss$tissue)
  tissues <- names(sss)

  cor_gene_tmb <- purrr::map(tissues, purrr::safely(function(cancer) {
    # cancer = "ACC"
    sss_can <- sss[[cancer]]
    dd <- stats::cor.test(sss_can$values, sss_can$Non_silent_per_Mb, type = cor_method)
    ddd <- data.frame(gene = Gene, cor = dd$estimate, p.value = dd$p.value, stringsAsFactors = F)
    ddd$cancer <- cancer
    return(ddd)
  })) %>% magrittr::set_names(tissues)

  cor_gene_tmb <- cor_gene_tmb %>%
    purrr::map(~ .x$result) %>%
    purrr::compact()
  cor_gene_tmb_df <- do.call(rbind.data.frame, cor_gene_tmb)
  data <- cor_gene_tmb_df
  data$pstar <- ifelse(data$p.value < 0.05,
    ifelse(data$p.value < 0.01, "**", "*"),
    ""
  )

  p <- ggplot2::ggplot(data, ggplot2::aes_string(x = "cancer", y = "gene")) +
    ggplot2::geom_tile(ggplot2::aes_string(fill = "cor"), colour = "white", size = 1) +
    ggplot2::scale_fill_gradient2(low = "#2b8cbe", mid = "white", high = "#e41a1c") +
    ggplot2::geom_text(ggplot2::aes_string(label = "pstar"), col = "black", size = 5) +
    ggplot2::theme_minimal() + # 不要背景
    ggplot2::theme(
      axis.title.x = ggplot2::element_blank(), # 不要title
      axis.ticks.x = ggplot2::element_blank(), # 不要x轴
      axis.title.y = ggplot2::element_blank(), # 不要y轴
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1), # 调整x轴文字
      axis.text.y = ggplot2::element_text(size = 8)
    ) + # 调整y轴文字
    # 调整legen
    ggplot2::labs(fill = paste0(" * p < 0.05", "\n\n", "** p < 0.01", "\n\n", "Correlation"))
  if (Plot == TRUE) {
    return(p)
  } else {
    pdata <- ss
    plist <- list()
    plist[[1]] <- p
    plist[[2]] <- pdata
    names(plist) <- c("plot", "pdata")
    return(plist)
  }
}

#' Visualize Correlation between Gene and MSI (Microsatellite instability)
#'
#' @inheritParams vis_gene_immune_cor
#' @examples
#' \dontrun{
#' p <- vis_gene_msi_cor(Gene = "TP53")
#' }
#' @export
vis_gene_msi_cor <- function(Gene = "TP53", cor_method = "spearman",
                             data_type = "mRNA", Plot = "TRUE", opt_pancan = .opt_pancan) {
  tcga_msi <- load_data("tcga_MSI")
  tcga_gtex <- load_data("tcga_gtex")

  t1 <- query_pancan_value(Gene, data_type = data_type, opt_pancan = opt_pancan)
  if (is.null(t1[[1]])) {
    warning("No data available", immediate. = TRUE)
    return(NULL)
  }
  if (is.list(t1)) t1 <- t1[[1]]

  if (all(is.na(t1))) {
    message("All NAs returned, return NULL instead.")
    return(NULL)
  }

  s <- data.frame(sample = names(t1), values = t1)
  ss <- s %>%
    dplyr::mutate(Barcode = stringr::str_sub(sample, 1, 12)) %>%
    dplyr::inner_join(tcga_msi, by = c("Barcode" = "Barcode")) %>%
    dplyr::inner_join(tcga_gtex[, c("tissue", "sample")], by = "sample")
  sss <- split(ss, ss$tissue)
  tissues <- names(sss)

  cor_gene_msi <- purrr::map(tissues, purrr::safely(function(cancer) {
    # cancer = "ACC"
    sss_can <- sss[[cancer]]
    dd <- stats::cor.test(sss_can$values, sss_can$Total_nb_MSI_events, type = cor_method)
    ddd <- data.frame(gene = Gene, cor = dd$estimate, p.value = dd$p.value, stringsAsFactors = F)
    ddd$cancer <- cancer
    return(ddd)
  })) %>% magrittr::set_names(tissues)

  cor_gene_msi <- cor_gene_msi %>%
    purrr::map(~ .x$result) %>%
    purrr::compact()
  cor_gene_msi_df <- do.call(rbind.data.frame, cor_gene_msi)
  data <- cor_gene_msi_df
  data$pstar <- ifelse(data$p.value < 0.05,
    ifelse(data$p.value < 0.01, "**", "*"),
    ""
  )

  p <- ggplot2::ggplot(data, ggplot2::aes_string(x = "cancer", y = "gene")) +
    ggplot2::geom_tile(ggplot2::aes_string(fill = "cor"), colour = "white", size = 1) +
    ggplot2::scale_fill_gradient2(low = "#2b8cbe", mid = "white", high = "#e41a1c") +
    ggplot2::geom_text(ggplot2::aes_string(label = "pstar"), col = "black", size = 5) +
    ggplot2::theme_minimal() + # 不要背景
    ggplot2::theme(
      axis.title.x = ggplot2::element_blank(), # 不要title
      axis.ticks.x = ggplot2::element_blank(), # 不要x轴
      axis.title.y = ggplot2::element_blank(), # 不要y轴
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1), # 调整x轴文字
      axis.text.y = ggplot2::element_text(size = 8)
    ) + # 调整y轴文字
    # 调整legen
    ggplot2::labs(fill = paste0(" * p < 0.05", "\n\n", "** p < 0.01", "\n\n", "Correlation"))
  # p
  if (Plot == TRUE) {
    return(p)
  } else {
    pdata <- ss
    plist <- list()
    plist[[1]] <- p
    plist[[2]] <- pdata
    names(plist) <- c("plot", "pdata")
    return(plist)
  }
}

#' Visualize Correlation between Gene and Tumor Stemness
#'
#' @inheritParams vis_gene_immune_cor
#' @examples
#' \dontrun{
#' p <- vis_gene_stemness_cor(Gene = "TP53")
#' p
#' }
#' ## To generate a radar plot, uncomment the following code
#' # pdata <- p$data %>%
#' #   dplyr::mutate(cor = round(cor, digits = 3), p.value = round(p.value, digits = 3))
#' #
#' # df <- pdata %>%
#' #   select(cor, cancer) %>%
#' #   pivot_wider(names_from = cancer, values_from = cor)
#' #
#' # ggradar::ggradar(
#' #   df[1, ],
#' #   font.radar = "sans",
#' #   values.radar = c("-1", "0", "1"),
#' #   grid.min = -1, grid.mid = 0, grid.max = 1,
#' #   # Background and grid lines
#' #   background.circle.colour = "white",
#' #   gridline.mid.colour = "grey",
#' #   # Polygons
#' #   group.line.width = 1,
#' #   group.point.size = 3,
#' #   group.colours = "#00AFBB") +
#' #   theme(plot.title = element_text(hjust = .5))
#' @export
vis_gene_stemness_cor <- function(Gene = "TP53", cor_method = "spearman",
                                  data_type = "mRNA", Plot = "TRUE", opt_pancan = .opt_pancan) {
  tcga_stemness <- load_data("tcga_stemness")
  tcga_gtex <- load_data("tcga_gtex")

  t1 <- query_pancan_value(Gene, data_type = data_type, opt_pancan = opt_pancan)
  if (is.null(t1[[1]])) {
    warning("No data available", immediate. = TRUE)
    return(NULL)
  }
  if (is.list(t1)) t1 <- t1[[1]]

  if (all(is.na(t1))) {
    message("All NAs returned, return NULL instead.")
    return(NULL)
  }

  s <- data.frame(sample = names(t1), values = t1)
  ss <- s %>%
    dplyr::inner_join(tcga_stemness, by = c("sample")) %>%
    dplyr::inner_join(tcga_gtex[, c("tissue", "sample")], by = "sample")
  sss <- split(ss, ss$tissue)
  tissues <- names(sss)

  cor_gene_stemness <- purrr::map(tissues, purrr::safely(function(cancer) {
    # cancer = "ACC"
    sss_can <- sss[[cancer]]
    dd <- stats::cor.test(sss_can$values, sss_can$RNAss, type = cor_method)
    ddd <- data.frame(gene = Gene, cor = dd$estimate, p.value = dd$p.value, stringsAsFactors = FALSE)
    ddd$cancer <- cancer
    return(ddd)
  })) %>% magrittr::set_names(tissues)

  cor_gene_stemness <- cor_gene_stemness %>%
    purrr::map(~ .x$result) %>%
    purrr::compact()
  cor_gene_stemness_df <- do.call(rbind.data.frame, cor_gene_stemness)
  data <- cor_gene_stemness_df
  data$pstar <- ifelse(data$p.value < 0.05,
    ifelse(data$p.value < 0.01, "**", "*"),
    ""
  )

  p <- ggplot2::ggplot(data, ggplot2::aes_string(x = "cancer", y = "gene")) +
    ggplot2::geom_tile(ggplot2::aes_string(fill = "cor"), colour = "white", size = 1) +
    ggplot2::scale_fill_gradient2(low = "#2b8cbe", mid = "white", high = "#e41a1c") +
    ggplot2::geom_text(ggplot2::aes_string(label = "pstar"), col = "black", size = 5) +
    ggplot2::theme_minimal() + # 不要背景
    ggplot2::theme(
      axis.title.x = ggplot2::element_blank(), # 不要title
      axis.ticks.x = ggplot2::element_blank(), # 不要x轴
      axis.title.y = ggplot2::element_blank(), # 不要y轴
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1), # 调整x轴文字
      axis.text.y = ggplot2::element_text(size = 8)
    ) + # 调整y轴文字
    # 调整legen
    ggplot2::labs(fill = paste0(" * p < 0.05", "\n\n", "** p < 0.01", "\n\n", "Correlation"))

  if (Plot == TRUE) {
    return(p)
  } else {
    pdata <- ss
    plist <- list()
    plist[[1]] <- p
    plist[[2]] <- pdata
    names(plist) <- c("plot", "pdata")
    return(plist)
  }
}

#' Visualize Gene TPM in Single Cancer Type (Tumor (TCGA) vs Normal (TCGA & GTEx))
#' @import ggplot2 dplyr tibble
#' @inheritParams vis_toil_TvsN
#' @param Cancer select cancer cohort(s).
#' @return a `ggplot` object.
#' @export
#'
vis_toil_TvsN_cancer <- function(Gene = "TP53", Mode = c("Violinplot", "Dotplot"),
                                 data_type = "mRNA", Show.P.value = FALSE,
                                 Show.P.label = FALSE, Method = "wilcox.test",
                                 values = c("#DF2020", "#DDDF21"),
                                 TCGA.only = FALSE, Cancer = "ACC", opt_pancan = .opt_pancan) {
  tcga_gtex <- load_data("tcga_gtex")

  Mode <- match.arg(Mode)
  if (!Method %in% c("wilcox.test", "t.test")) {
    stop("only support wilcox.test or t.test")
  }

  t1 <- query_pancan_value(Gene, data_type = data_type, opt_pancan = opt_pancan)
  if (is.null(t1[[1]])) {
    warning("No data available", immediate. = TRUE)
    return(NULL)
  }
  unit <- switch(data_type,
    cnv = NULL,
    mutation = NULL,
    t1[[2]]
  )
  if (is.list(t1)) t1 <- t1[[1]]

  if (all(is.na(t1))) {
    message("All NAs returned, return NULL instead.")
    return(NULL)
  }

  tcga_gtex <- tcga_gtex %>%
    dplyr::group_by(.data$tissue) %>%
    dplyr::distinct(.data$sample, .keep_all = TRUE)

  t2 <- t1 %>%
    as.data.frame() %>%
    dplyr::rename("value" = ".") %>%
    tibble::rownames_to_column(var = "sample") %>%
    dplyr::inner_join(tcga_gtex, by = "sample")
  tumorlist <- unique(tcga_gtex[tcga_gtex$type2 == "tumor", ]$tissue)
  normallist <- unique(tcga_gtex[tcga_gtex$type2 == "normal", ]$tissue)
  withoutNormal <- setdiff(tumorlist, normallist)
  tcga_gtex <- t2 %>% dplyr::select("value", "tissue", "type2", "sample")
  tcga_gtex$type2 <- factor(tcga_gtex$type2, levels = c("tumor", "normal"))
  tcga_gtex_withNormal <- tcga_gtex[!(tcga_gtex$tissue %in% withoutNormal), ]
  tcga_gtex_withNormal <- tcga_gtex_withNormal %>%
    dplyr::mutate(dataset = ifelse(stringr::str_sub(.data$sample, 1, 4) == "TCGA", "TCGA", "GTEX"))
  if (TCGA.only == TRUE) {
    tcga_gtex_withNormal <- tcga_gtex_withNormal %>% dplyr::filter(.data$dataset == "TCGA")
  }
  tcga_gtex_withNormal <- tcga_gtex_withNormal %>% dplyr::filter(.data$tissue %in% Cancer)
  if (Show.P.value == FALSE) {
    Show.P.label <- FALSE
  }
  if (Show.P.value == TRUE) {
    if (TCGA.only == TRUE) stop("Can't compare with only one group")
    message("Counting P value")
    pv <- tcga_gtex_withNormal %>%
      ggpubr::compare_means(value ~ type2, data = ., method = Method)
    pv <- pv %>% dplyr::select(c("p", "p.signif", "p.adj"))
    message("Counting P value finished")
  }
  data <- tcga_gtex_withNormal
  if (Mode == "Dotplot") {
    p <- ggpubr::ggdotplot(
      tcga_gtex_withNormal,
      x = "type2", y = "value", fill = "type2", color = "type2", size = 0.6, binwidth = 0.3
    ) +
      ggplot2::xlab(NULL) +
      theme_set(theme_classic(base_size = 20)) +
      ggplot2::theme(axis.text.x = element_text(angle = 45, hjust = .5, vjust = .5, size = 20)) +
      ggplot2::guides(fill = guide_legend(title = NULL)) +
      ggplot2::theme(
        legend.background = element_blank(),
        legend.position = "none", legend.justification = c(0, 0)
      ) +
      ggplot2::scale_fill_manual(values = values) +
      ggplot2::scale_color_manual(values = values)


    # p <- ggplot2::ggplot(tcga_gtex_withNormal, aes_string(x = "type2", y = "value", fill = "type2")) +
    #   # ggplot2::geom_boxplot() +
    #   ggplot2::geom_dotplot(binaxis = "y", stackdir = "center", position = "identity",size = 0.6) +
    #   ggplot2::xlab(NULL) +
    #   ggplot2::theme_set(theme_set(theme_classic(base_size = 20))) +
    #   ggplot2::theme(axis.text.x = element_text(angle = 45, hjust = .5, vjust = .5)) +
    #   ggplot2::guides(fill = guide_legend(title = NULL)) +
    #   ggplot2::theme(
    #     legend.background = element_blank(),
    #     legend.position = "none", legend.justification = c(0, 0)
    #   ) +
    #   ggplot2::scale_fill_manual(values = values) +
    #   ggplot2::scale_color_manual(values = values)

    p <- p + ggplot2::ylab(
      if (is.null(unit)) Gene else paste0(Gene, " (", unit, ")")
    )

    if (Show.P.value == TRUE & Show.P.label == TRUE) {
      if (TCGA.only == TRUE) stop("Can't compare with only one group")
      p <- p + ggplot2::geom_text(
        aes(
          x = 1.5,
          y = max(data$value) * 1.1,
          label = .data$p.signif
        ),
        data = pv,
        inherit.aes = FALSE
      )
    }
    if (Show.P.value == TRUE & Show.P.label == FALSE) {
      if (TCGA.only == TRUE) stop("Can't compare with only one group")
      p <- p + ggplot2::geom_text(
        aes(
          x = 1.5,
          y = max(data$value) * 1.1,
          label = as.character(signif(.data$p, 2))
        ),
        data = pv,
        inherit.aes = FALSE
      )
    }
  }
  if (Mode == "Violinplot") {
    p <- ggplot2::ggplot(tcga_gtex_withNormal, aes_string(x = "type2", y = "value", fill = "type2")) +
      ggplot2::geom_violin(trim = FALSE) +
      ggplot2::geom_boxplot(width = 0.1, fill = "white") +
      ggplot2::xlab("") +
      ggplot2::scale_fill_manual(values = values) +
      ggplot2::theme_classic(base_size = 20) +
      ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 45, hjust = .5, vjust = .5, size = 20)) +
      ggplot2::guides(fill = ggplot2::guide_legend(title = NULL)) +
      ggplot2::theme(
        legend.background = ggplot2::element_blank(),
        legend.position = c(0, 0), legend.justification = c(0, 0)
      )

    p <- p + ggplot2::ylab(
      if (is.null(unit)) Gene else paste0(Gene, " (", unit, ")")
    )

    if (Show.P.value == TRUE & Show.P.label == TRUE) {
      if (TCGA.only == TRUE) stop("Can't compare with only one group")
      p <- p + ggplot2::geom_text(
        ggplot2::aes(
          x = 1.5,
          y = max(data$value) * 1.1,
          label = .data$p.signif
        ),
        data = pv,
        inherit.aes = FALSE
      )
    }
    if (Show.P.value == TRUE & Show.P.label == FALSE) {
      if (TCGA.only == TRUE) stop("Can't compare with only one group")
      p <- p + ggplot2::geom_text(
        ggplot2::aes(
          x = 1.5,
          y = max(data$value) * 1.1,
          label = as.character(signif(.data$p, 2))
        ),
        data = pv,
        inherit.aes = FALSE
      )
    }
  }

  if (TCGA.only) {
    p <- p + ggplot2::ggtitle(paste0("TCGA: ", paste(Cancer, collapse = "/")))
  } else {
    p <- p + ggplot2::ggtitle(paste0("TCGA/GTEx: ", paste(Cancer, collapse = "/")))
  }

  return(p)
}

#' Visualize Gene-Gene Correlation in TCGA
#'
#' @import ggplot2 dplyr ppcor
#' @param Gene1 a molecular identifier (e.g., "TP53") or a formula specifying
#' genomic signature (`"TP53 + 2 * KRAS - 1.3 * PTEN"`).
#' @param Gene2 a molecular identifier (e.g., "TP53") or a formula specifying
#' genomic signature (`"TP53 + 2 * KRAS - 1.3 * PTEN"`).
#' @param data_type1 choose gene profile type for the first gene, including "mRNA","transcript","methylation","miRNA","protein","cnv_gistic2"
#' @param data_type2 choose gene profile type for the second gene, including "mRNA","transcript","methylation","miRNA","protein","cnv_gistic2"
#' @param purity_adj whether performing partial correlation adjusted by purity
#' @param use_regline if `TRUE`, add regression line.
#' @param filter_tumor whether use tumor sample only, default `TRUE`
#' @param alpha dot alpha.
#' @param color dot color.
#' @param opt_pancan specify one dataset for some molercular profiles
#' @export
vis_gene_cor <- function(Gene1 = "CSF1R",
                         Gene2 = "JAK3",
                         data_type1 = "mRNA",
                         data_type2 = "mRNA",
                         use_regline = TRUE,
                         purity_adj = TRUE,
                         alpha = 0.5,
                         color = "#000000",
                         filter_tumor = TRUE,
                         opt_pancan = .opt_pancan) {
  if (!requireNamespace("cowplot")) {
    install.packages("cowplot")
  }

  if (!data_type1 %in% c("mRNA", "miRNA", "transcript", "methylation", "protein", "cnv")) {
    stop("data_type ", data_type1, " does not support in this function!")
  }

  if (!data_type2 %in% c("mRNA", "miRNA", "transcript", "methylation", "protein", "cnv")) {
    stop("data_type ", data_type2, " does not support in this function!")
  }

  tcga_gtex <- load_data("tcga_gtex")
  tcga_purity <- load_data("tcga_purity")

  tcga_purity$CPE <- as.numeric(tcga_purity$CPE)
  tcga_gtex <- tcga_gtex %>%
    dplyr::group_by(.data$tissue) %>%
    dplyr::distinct(.data$sample, .keep_all = TRUE)

  t1 <- query_pancan_value(Gene1, data_type = data_type1, opt_pancan = opt_pancan)
  if (is.null(t1[[1]])) {
    warning("No data available", immediate. = TRUE)
    return(NULL)
  }
  if (is.list(t1)) t1 <- t1[[1]]
  t3 <- query_pancan_value(Gene2, data_type = data_type2, opt_pancan = opt_pancan)
  if (is.null(t3[[1]])) {
    warning("No data available", immediate. = TRUE)
    return(NULL)
  }
  if (is.list(t3)) t3 <- t3[[1]]

  t2 <- t1 %>%
    as.data.frame() %>%
    dplyr::rename("tpm" = ".") %>%
    tibble::rownames_to_column(var = "sample") %>%
    dplyr::inner_join(tcga_gtex, by = "sample")

  t4 <- t3 %>%
    as.data.frame() %>%
    dplyr::rename("tpm" = ".") %>%
    tibble::rownames_to_column(var = "sample") %>%
    dplyr::inner_join(tcga_gtex, by = "sample")

  df <- dplyr::inner_join(
    data.frame(
      sample = t2$sample,
      tissue = t2$tissue,
      type2 = t2$type2,
      gene1 = t2$tpm,
      stringsAsFactors = FALSE
    ),
    data.frame(
      sample = t4$sample,
      gene2 = t4$tpm,
      stringsAsFactors = FALSE
    ),
    by = "sample"
  )

  df %>%
    dplyr::left_join(tcga_purity, by = "sample") -> df
  if (filter_tumor == TRUE) {
    df %>% dplyr::filter(.data$type2 == "tumor") -> df
  }
  # plot refer to https://drsimonj.svbtle.com/pretty-scatter-plots-with-ggplot2

  if (purity_adj) {
    df %>% dplyr::filter(!is.na(.data$CPE)) -> df
    partial_cor_res <- ezcor_partial_cor(data = df, var1 = "gene1", var2 = "gene2", var3 = "CPE", sig_label = TRUE)
    cor_res <- ezcor(data = df, var1 = "gene1", var2 = "gene2")
    p <- ggplot2::ggplot(df, aes_string(x = "gene1", y = "gene2")) +
      ggplot2::geom_point(shape = 16, size = 1.5, show.legend = FALSE, alpha = alpha, color = color) +
      ggplot2::theme_minimal() +
      ggplot2::scale_color_gradient(low = "#0091ff", high = "#f0650e") +
      ggplot2::labs(x = Gene1, y = Gene2) +
      ggplot2::ggtitle("TCGA PANCAN dataset") +
      ggplot2::annotate(
        "text",
        -Inf, Inf,
        hjust = -0.1, vjust = 1,
        label = paste0(
          "Cor: ", round(cor_res$cor, 2),
          " ", cor_res$pstar, "\n", "Cor_adj: ",
          round(partial_cor_res$cor_partial, 2), " ",
          partial_cor_res$pstar
        ),
        size = 5, colour = "black"
      )
  } else {
    cor_res <- ezcor(data = df, var1 = "gene1", var2 = "gene2")
    p <- ggplot2::ggplot(df, aes_string(x = "gene1", y = "gene2")) +
      ggplot2::geom_point(shape = 16, size = 1.5, show.legend = FALSE, alpha = alpha, color = color) +
      ggplot2::theme_minimal() +
      ggplot2::scale_color_gradient(low = "#0091ff", high = "#f0650e") +
      ggplot2::labs(x = Gene1, y = Gene2) +
      ggplot2::ggtitle("TCGA PANCAN dataset") +
      ggplot2::annotate(
        "text",
        -Inf, Inf,
        hjust = -0.1, vjust = 1,
        label = paste0(
          "Cor: ", round(cor_res$cor, 2), " ",
          cor_res$pstar
        ),
        size = 5, colour = "black"
      )
  }



  if (use_regline) p <- p + ggplot2::geom_smooth(method = stats::lm)

  return(p)
}


#' Visualize Gene-Gene Correlation in a TCGA Cancer Type
#'
#' @import ggplot2 dplyr ppcor
#' @inheritParams vis_gene_cor
#' @param cancer_choose TCGA cohort name, e.g. "ACC".
#' @param cor_method correlation method.
#' @param use_all use all sample, default `FALSE`.
#' @export
vis_gene_cor_cancer <- function(Gene1 = "CSF1R",
                                Gene2 = "JAK3",
                                data_type1 = "mRNA",
                                data_type2 = "mRNA",
                                purity_adj = TRUE,
                                cancer_choose = "GBM",
                                use_regline = TRUE,
                                cor_method = "spearman",
                                use_all = FALSE,
                                alpha = 0.5,
                                color = "#000000",
                                opt_pancan = .opt_pancan) {
  if (!requireNamespace("cowplot")) {
    install.packages("cowplot")
  }

  tcga_gtex <- load_data("tcga_gtex")
  tcga_purity <- load_data("tcga_purity")

  tcga_purity$CPE <- as.numeric(tcga_purity$CPE)
  tcga_gtex <- tcga_gtex %>%
    dplyr::group_by(.data$tissue) %>%
    dplyr::distinct(.data$sample, .keep_all = TRUE)

  t1 <- query_pancan_value(Gene1, data_type = data_type1, opt_pancan = opt_pancan)
  if (length(na.omit(t1[[1]])) == 0) {
    warning("No data available", immediate. = TRUE)
    return(NULL)
  }
  if (is.list(t1)) t1 <- t1[[1]]
  t3 <- query_pancan_value(Gene2, data_type = data_type2, opt_pancan = opt_pancan)
  if (length(na.omit(t1[[3]])) == 0) {
    warning("No data available", immediate. = TRUE)
    return(NULL)
  }
  if (is.list(t3)) t3 <- t3[[1]]

  t2 <- t1 %>%
    as.data.frame() %>%
    dplyr::rename("value" = ".") %>%
    tibble::rownames_to_column(var = "sample") %>%
    dplyr::inner_join(tcga_gtex, by = "sample")

  t4 <- t3 %>%
    as.data.frame() %>%
    dplyr::rename("value" = ".") %>%
    tibble::rownames_to_column(var = "sample") %>%
    dplyr::inner_join(tcga_gtex, by = "sample")

  # merge
  t2 <- t2 %>% inner_join(t4[, c("sample", "value")], by = "sample")

  df <- data.frame(
    sample = t2$sample,
    tissue = t2$tissue,
    type2 = t2$type2,
    gene1 = t2$value.x,
    gene2 = t2$value.y,
    stringsAsFactors = F
  )

  df %>%
    dplyr::left_join(tcga_purity, by = "sample") %>%
    dplyr::filter(.data$type2 == "tumor") -> df

  if (!use_all) {
    df <- df %>% dplyr::filter(.data$tissue %in% cancer_choose)
  }

  # plot refer to https://drsimonj.svbtle.com/pretty-scatter-plots-with-ggplot2

  if (purity_adj) {
    df %>% filter(!is.na(.data$CPE)) -> df
    tryCatch(
      {
        partial_cor_res <- ezcor_partial_cor(data = df, var1 = "gene1", var2 = "gene2", var3 = "CPE", sig_label = TRUE, cor_method = cor_method)
      },
      error = function(e) {
        stop("Error occurs when adjust purity, may be no purity data avaiable in this cohort.")
      }
    )
    cor_res <- ezcor(data = df, var1 = "gene1", var2 = "gene2", cor_method = cor_method)
    # https://drsimonj.svbtle.com/pretty-scatter-plots-with-ggplot2
    p <- ggplot2::ggplot(df, aes_string(x = "gene1", y = "gene2")) +
      ggplot2::geom_point(shape = 16, size = 3, show.legend = FALSE, alpha = alpha, color = color) +
      ggplot2::theme_minimal() +
      ggplot2::labs(x = Gene1, y = Gene2) +
      ggplot2::annotate(
        "text",
        -Inf, Inf,
        hjust = -0.1, vjust = 1,
        label = paste0(
          "Cor: ", round(cor_res$cor, 2), " ",
          cor_res$pstar, "\n", "Cor_adj: ",
          round(partial_cor_res$cor_partial, 2), " ", partial_cor_res$pstar
        ),
        size = 8, colour = "black"
      ) +
      ggplot2::labs(
        x = paste(Gene1, data_type1),
        y = paste(Gene2, data_type2)
      )
  } else {
    cor_res <- ezcor(data = df, var1 = "gene1", var2 = "gene2", cor_method = cor_method)
    p <- ggplot2::ggplot(df, aes_string(x = "gene1", y = "gene2")) +
      ggplot2::geom_point(shape = 16, size = 3, show.legend = FALSE, alpha = alpha, color = color) +
      ggplot2::theme_minimal() +
      ggplot2::labs(x = Gene1, y = Gene2) +
      ggplot2::annotate(
        "text",
        -Inf, Inf,
        hjust = -0.1, vjust = 1,
        label = paste0("Cor: ", round(cor_res$cor, 2), " ", cor_res$pstar),
        size = 8, colour = "black"
      ) +
      ggplot2::labs(
        x = paste(Gene1, data_type1),
        y = paste(Gene2, data_type2)
      )
  }

  if (use_all) {
    p <- p + ggplot2::ggtitle(paste0("TCGA: All data (n=", dim(df)[1], ")"))
  } else {
    p <- p + ggplot2::ggtitle(paste0("TCGA: ", paste(cancer_choose, collapse = "/"), " (n=", dim(df)[1], ")"))
  }

  if (use_regline) {
    p <- p + ggplot2::geom_smooth(method = stats::lm)
  }
  p <- p + ggplot2::theme(legend.position = "none")
  return(p)
}

#' Heatmap for Correlation between Gene and Tumor Immune Infiltration (TIL)
#'
#' @inheritParams vis_toil_TvsN
#' @param cor_method correlation method
#' @param sig Immune Signature, default: result from TIMER
#' @param Plot output the plot directly, default 'TRUE'
#' @examples
#' \dontrun{
#' p <- vis_gene_TIL_cor(Gene = "TP53")
#' }
#' @export
vis_gene_TIL_cor <- function(Gene = "TP53",
                             cor_method = "spearman",
                             data_type = "mRNA",
                             sig = c(
                               "B cell_TIMER",
                               "T cell CD4+_TIMER",
                               "T cell CD8+_TIMER",
                               "Neutrophil_TIMER",
                               "Macrophage_TIMER",
                               "Myeloid dendritic cell_TIMER"
                             ),
                             Plot = "TRUE",
                             opt_pancan = .opt_pancan) {
  tcga_TIL <- load_data("tcga_TIL")
  cell_type <- colnames(tcga_TIL)[-1]
  source <- sapply(stringr::str_split(cell_type, "_"), function(x) x[2])

  tcga_gtex <- load_data("tcga_gtex")

  # we filter out normal tissue
  tcga_gtex <- tcga_gtex %>% dplyr::filter(.data$type2 != "normal")

  t1 <- query_pancan_value(Gene, data_type = data_type, opt_pancan = opt_pancan)
  if (is.null(t1[[1]])) {
    warning("No data available", immediate. = TRUE)
    return(NULL)
  }
  if (is.list(t1)) t1 <- t1[[1]]

  if (all(is.na(t1))) {
    message("All NAs returned, return NULL instead.")
    return(NULL)
  }

  message(paste0("Get data value for ", Gene))
  s <- data.frame(sample = names(t1), values = t1)

  ss <- s %>%
    dplyr::inner_join(tcga_TIL, by = c("sample" = "cell_type")) %>%
    dplyr::inner_join(tcga_gtex[, c("tissue", "sample")], by = "sample")

  sss <- split(ss, ss$tissue)
  tissues <- names(sss)
  cor_gene_immune <- purrr::map(tissues, purrr::safely(function(cancer) {
    # cancer = "ACC"
    sss_can <- sss[[cancer]]
    ## filter cibersort data here
    sss_can_class <- sss_can %>% select(c("sample", "values", sig))
    sss_can_class <- sss_can_class %>% tidyr::pivot_longer(cols = c(3:ncol(.)), names_to = "SetName", values_to = "score")
    sss_can_class <- sss_can_class[complete.cases(sss_can_class), ]
    cells <- unique(sss_can_class$SetName)
    cor_res_class_can <- purrr::map(cells, purrr::safely(function(i) {
      # i = cells[1]
      dd <- sss_can_class[sss_can_class$SetName == i, ]
      dd <- suppressWarnings(stats::cor.test(dd$values, dd$score, method = cor_method))
      ddd <- data.frame(gene = Gene, immune_cells = i, cor = dd$estimate, p.value = dd$p.value, stringsAsFactors = FALSE)
      return(ddd)
    })) %>% magrittr::set_names(cells)
    cor_res_class_can <- cor_res_class_can %>%
      purrr::map(~ .x$result) %>%
      purrr::compact()
    cor_res_class_can_df <- do.call(rbind.data.frame, cor_res_class_can)
    cor_res_class_can_df$cancer <- cancer
    return(cor_res_class_can_df)
  })) %>% magrittr::set_names(tissues)

  cor_gene_immune <- cor_gene_immune %>%
    purrr::map(~ .x$result) %>%
    purrr::compact()
  cor_gene_immune_df <- do.call(rbind.data.frame, cor_gene_immune)
  data <- cor_gene_immune_df
  data$pstar <- ifelse(data$p.value < 0.05,
    ifelse(data$p.value < 0.001, "***", ifelse(data$p.value < 0.01, "**", "*")),
    ""
  )

  p <- ggplot2::ggplot(data, ggplot2::aes_string(x = "cancer", y = "immune_cells")) +
    ggplot2::geom_tile(ggplot2::aes_string(fill = "cor"), colour = "white", size = 1) +
    ggplot2::scale_fill_gradient2(low = "#377DB8", mid = "white", high = "#E31A1C") +
    ggplot2::geom_text(ggplot2::aes_string(label = "pstar"), col = "black", size = 5) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      axis.title.x = ggplot2::element_blank(),
      axis.ticks.x = ggplot2::element_blank(),
      axis.title.y = ggplot2::element_blank(),
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
      axis.text.y = ggplot2::element_text(size = 8)
    ) +
    ggplot2::labs(fill = paste0(" * p < 0.05", "\n\n", "** p < 0.01", "\n\n", "*** p < 0.001", "\n\n", "Correlation")) +
    ggtitle(paste0("The correlation between ", Gene, " ", data_type, " with TIL estimations"))

  if (Plot == TRUE) {
    return(p)
  } else {
    pdata <- ss
    plist <- list()
    plist[[1]] <- p
    plist[[2]] <- pdata
    names(plist) <- c("plot", "pdata")
    return(plist)
  }
}

## scatter plot for individual mode of TPC correlation analysis
plot_cor_o2o = function(data, xlab, ylab, title, cor_method,
            point.args, smooth.line.args, xsidehistogram.args, ysidehistogram.args,
            axis_size, title_size, side_hist, custom_theme){

    names(cor_method) = ifelse(cor_method=="Pearson","parametric","nonparametric")
    
    p = ggscatterstats(
        data, x = !!colnames(data)[6], y = !!colnames(data)[10],
        xlab = xlab, ylab = ylab, title = title, type = names(cor_method),
        point.args = point.args, smooth.line.args = smooth.line.args, 
        xsidehistogram.args = xsidehistogram.args, ysidehistogram.args = ysidehistogram.args,
        bf.message = FALSE
    )
    # change theme
    p = p + custom_theme
    # theme text size
    p = p + 
        theme(text = element_text(size=axis_size),
              plot.title = element_text(size=title_size, hjust = 0.5))
    # refine subtitle
    pval = formatC(extract_stats(p)$subtitle_data$p.value, digits = 3, format = 'e')
    r = round(extract_stats(p)$subtitle_data$estimate,3)
    p$labels$subtitle = bquote(paste(widehat(italic(r))[.(cor_method)] == .(r), ', ' ,italic(p) == .(pval)))
    # remove hist
    if(side_hist=="NO"){
        p = p + theme(#ggside.panel.scale = 1,
                ggside.axis.text = element_blank(),
                ggside.axis.ticks = element_blank())
    }
    p
}


## bar plot for pan-cancer mode of TPC correlation analysis
plot_cor_o2m = function(data, label_size, x_name, title_name, 
        negative_color, positive_color, axis_size, title_size,
        custom_theme){
    p = data %>% 
        dplyr::arrange(estimate) %>% 
        dplyr::mutate(cancer = factor(cancer, levels = cancer)) %>% 
        dplyr::mutate(group = estimate>0) %>% 
        ggplot(aes(x=cancer, y=estimate, fill=group)) + 
        geom_col(color="black") + 
        geom_text(aes(y=0,label=format(round(estimate,2), nsmall =2),
                    hjust = ifelse(estimate >= 0, 1.5, -0.5)),
                size = label_size) +
        xlab("") + ylab(x_name) + #转置
        ggtitle(label = title_name) +
        coord_flip() +
        scale_fill_manual(values = c(negative_color,positive_color))
    # change theme
    p = p + custom_theme
    p = p + 
        theme(legend.position = "none", 
            text = element_text(size=axis_size),
            plot.title = element_text(size=title_size, hjust = 0.5))
    return(p)
}


## violin plot for individual mode of TPC comparison analysis
plot_comb_o2o = function(data, xlab, ylab, title, comp_method, 
        point_size, point_alpha, group_1_color, group_2_color, 
        axis_size, title_size, custom_theme){

    names(comp_method) = ifelse(comp_method=="t-test","parametric","nonparametric")

    p = ggbetweenstats(
        data = data, x = !!colnames(data)[4], y = !!colnames(data)[3],
        xlab = xlab, ylab = ylab, title = title, type = names(comp_method),
        centrality.plotting = FALSE, median.color = 'black',
        bf.message = FALSE,
        point.args = list(size = point_size, alpha = point_alpha,
                          position = ggplot2::position_jitterdodge(dodge.width = 0.6), stroke = 0, na.rm = TRUE))
    p = p + ggplot2::scale_color_manual(values = c(group_1_color, group_2_color))
    # change theme
    p = p + custom_theme
    p = p + theme(text = element_text(size=axis_size),
                  legend.position = "none", 
			      plot.title = element_text(size=title_size, hjust = 0.5),
			      plot.subtitle = element_text(size = 14))
    pval = formatC(extract_stats(p)$subtitle_data$p.value, digits = 3, format = 'e')
    p$labels$subtitle = bquote(paste(.(comp_method),", ",italic(p) == .(pval)))
    return(p)
}

## line plot for pan-cancer mode of TPC comparison analysis
plot_comb_o2m = function(data1, data2, x_name, title_name, 
        group_1_color_2, group_2_color_2, axis_size, title_size,
        significance, label_size, custom_theme){
    ### p1
    data1_sub = data1 %>%
        dplyr::filter(cancer %in% as.character(unique(data2$cancer))) %>%
        dplyr::arrange(desc(cancer)) %>%
        dplyr::mutate(cancer = factor(cancer, levels = unique(cancer)))
		
    p1 = ggplot(data1_sub) +
        stat_summary(aes(x=cancer, y=value, color=group),
                    # colour = "gray",
                    position=position_dodge(width=0.5)) + 
        geom_jitter(aes(x=cancer, y=value, color=group),
                    position=position_dodge(width=0.5), size = 1, 
                    # colour = "gray", 
                    alpha = 0.4) +
        xlab("") + ylab(x_name) + ggtitle(label = title_name) + #转置
		scale_color_manual(values = c(group_1_color_2, group_2_color_2)) +
        coord_flip() + custom_theme
    p1 = p1 + theme(legend.position = "top", plot.margin = margin(0,0,0,0),
                  text = element_text(size=axis_size),
                  plot.title = element_text(size=title_size, hjust = 0.5))

    ### p2
    if(significance=="Value"){
        p2 = ggplot(data2) + 
            geom_text(aes(label=formatC(p.value, format = "e", digits = 2),
                            x=cancer, y=1), size = label_size) +
            coord_flip()
    } else if (significance=="Symbol"){
        p2 = data2 %>%
            dplyr::mutate(p.label=case_when(
                p.value < 0.001 ~ "***",
                p.value < 0.01 ~ "**",
                p.value < 0.05 ~ "*",
                TRUE ~ "ns"
            )) %>% ggplot() + 
            geom_text(aes(label=p.label, x=cancer, y=1), size = label_size) +
            coord_flip()
    }
    p2 = p2 + custom_theme +
        theme(axis.ticks = element_blank(),
            axis.text = element_blank(),
            axis.line = element_blank(),
            axis.title = element_blank(),
            axis.ticks.length.y = unit(0,"pt"),
            plot.margin = margin(0,0,0,0)) +
        theme(panel.grid.major.x = element_blank(),
            panel.grid.minor.x = element_blank()) +
        theme(text = element_text(size=axis_size))

    p = p1 + p2 + patchwork::plot_layout(widths = c(5,0.3))
}


## line plot for individual mode of TPC survival analysis
plot_sur_o20 = function(data, plot_CI, plot_table, plot_ncensor, sur_method,
        one_log_color1, one_log_color2, axis_size, x_name, title_name, title_size,
        text_c1, text_c2, text_c3, axis_size_2, title_name_2){

    if(plot_CI == "NO"){
        conf.int = FALSE
        conf.int.style = "ribbon"
    } else if(plot_CI == "YES(ribbon)"){
        conf.int = TRUE
        conf.int.style = "ribbon"
    } else if(plot_CI == "YES(step)"){
        conf.int = TRUE
        conf.int.style = "step"
    }

    custom_theme <- function(plot_size) {
        theme_classic() %+replace%
        theme(
            plot.title=element_text(hjust=0.5, size = plot_size)
        )
    }

    if(plot_table=="NO" & plot_ncensor=="NO"){
        surv.plot.height = 1
        risk.table.height = 0 
        ncensor.plot.height = 0 
        risk.table = FALSE
        ncensor.plot = FALSE
    } else if (plot_table=="YES" & plot_ncensor=="NO"){
        surv.plot.height = 0.7
        risk.table.height = 0.3 
        ncensor.plot.height = 0
        risk.table = TRUE
        ncensor.plot = FALSE
    } else if (plot_table=="NO" & plot_ncensor=="YES"){
        surv.plot.height = 0.7
        risk.table.height = 0 
        ncensor.plot.height = 0.3 
        risk.table = FALSE
        ncensor.plot = TRUE
    } else if (plot_table=="YES" & plot_ncensor=="YES"){
        surv.plot.height = 0.7
        risk.table.height = 0.15
        ncensor.plot.height = 0.15
        risk.table = TRUE
        ncensor.plot = TRUE
    }

    if(sur_method=="Log-rank test"){	
        fit <- survfit(Surv(time, status) ~ Group, data = data)
        p <- ggsurvplot(fit, data = data,#data = group_sur_final(), 
                        pval = TRUE, pval.method = TRUE, 
                        palette = c(one_log_color1, one_log_color2), 
                        size = 1.2, font.legend = c(14, "black"), 
                        font.x = c(axis_size, "bold", "black"), 
                        font.y = c(axis_size,  "bold", "black"), 
                        font.tickslab = c(12, "bold", "black"), 
                        xlab = x_name,
                        title = title_name,
                        conf.int = conf.int,
                        conf.int.style = conf.int.style,
                        risk.table = risk.table, risk.table.col = "strata", risk.table.y.text = FALSE, 
                        ncensor.plot = ncensor.plot, 
                        surv.plot.height = surv.plot.height, 
                        risk.table.height = risk.table.height, 
                        ncensor.plot.height = ncensor.plot.height, 
                        ggtheme = custom_theme(title_size))
    }  else if (sur_method=="Univariate Cox regression"){
        fit = coxph(Surv(time, status) ~ Group , data = data)
        p = ggforest(fit,data = data,#fontsize = 1,
            cpositions = c(text_c1, text_c2, text_c3),
            fontsize = axis_size_2, main = title_name_2)
    }

}


## bar plot for pan-cancer mode of TPC survival analysis
# data$sur_res
# data$sur_dat
plot_sur_02m = function(data, sur_method, multi_log_color1, multi_log_color2,
        x_name, title_name, multi_log_line, axis_size, title_size, multi_log_label,
        label_size, cox_h_g1_color, cox_h_l1_color, x_name_2, title_name_2, 
        multi_cox_line, axis_size_2, title_size_2, multi_cox_label, label_size_2,
        custom_theme){
    if(sur_method=="Log-rank test"){
        dat = data$sur_res
        # print(head(dat))
        pval_df = dat %>%
            dplyr::select(Cancer, Group, rmean, p.value) %>% 
            dplyr::group_by(Cancer) %>% 
            dplyr::mutate(Risk = ifelse(rmean[1]>rmean[2],
                                        paste0("Low risk(", Group[1],")"), 
                                        paste0("High risk(", Group[1],")"))) %>% 
            dplyr::distinct(Cancer, p.value, Risk) %>% as.data.frame() %>% 
            dplyr::mutate(Cancer = factor(Cancer, levels = rev(sort(Cancer))))
        # print(head(pval_df))
        fill_cols = c(multi_log_color1, multi_log_color2)
        names(fill_cols) = c(
            paste0("Low risk(Group=",levels(data$sur_dat$Group)[1], ")"),
            paste0("High risk(Group=",levels(data$sur_dat$Group)[1], ")"))
        p1 = pval_df %>% 
            dplyr::mutate(pval_log = -log10(p.value)) %>% 
            dplyr::mutate(pval_log = ifelse(pval_log<10,pval_log, 10)) %>% 
            ggplot(aes(x = Cancer, y = pval_log, fill = Risk)) + 
            geom_col() +
            scale_fill_manual(values = fill_cols) + 
            xlab(NULL) + ylab(x_name) +
            ggtitle(label = title_name) +
            geom_hline(yintercept = -log10(multi_log_line), color = "red") +
            coord_flip() +
            custom_theme +
            # theme_minimal() +
            theme(legend.position = "top",
                plot.margin = margin(0,0,0,0),
                text = element_text(size=axis_size),
                plot.title = element_text(size=title_size, hjust = 0.5)
                )  
        if(multi_log_label=="Signif.(value)"){
            p2 =  ggplot(pval_df) +
                geom_text(aes(label=formatC(p.value, format = "e", digits = 2),
                            x=Cancer,y=1),size = label_size) +
                coord_flip()
        } else if (multi_log_label=="Signif.(symbol)"){
            p2 = pval_df %>%
                dplyr::mutate(p.label=case_when(
                p.value  < 0.001 ~ "***",
                p.value  < 0.01 ~ "**",
                p.value  < 0.05 ~ "*",
                TRUE ~ "ns"
                )) %>% ggplot() +
                geom_text(aes(label=p.label,x=Cancer,y=1),size = label_size) + 
                coord_flip()
        }
        p2 = p2 +
            custom_theme +
            # theme_minimal() +
            theme(axis.ticks = element_blank(),
                axis.text = element_blank(),
                axis.line = element_blank(),
                axis.title = element_blank(),
                axis.ticks.length.y = unit(0,"pt"),
                plot.margin = margin(0,0,0,0)) +
            theme(panel.grid.major.x = element_blank(),
                panel.grid.minor.x = element_blank())
        p = p1 + p2 + plot_layout(widths = c(5,1))
        p
    } else if (sur_method=="Univariate Cox regression") {
        dat = data$sur_dat
        sur_res = lapply(sort(unique(dat$Cancer)), function(x){
            sur_dat_sub = subset(dat, Cancer==x)
            fit = coxph(Surv(time, status) ~ Group , data = sur_dat_sub)
            summary(fit)$coefficients %>% as.data.frame() %>% 
            tibble::rownames_to_column("Group") %>% 
            dplyr::mutate(Cancer = x, .before = 1)
        }) %>% do.call(rbind, .)
        fill_cols = c(cox_h_g1_color, cox_h_l1_color)
        names(fill_cols) = c("HR>1","HR<1")
        p1 = sur_res %>%
            dplyr::rename(`p.value`=`Pr(>|z|)`, `HR` = `exp(coef)`) %>% 
            dplyr::select(Cancer, Group, HR, p.value) %>% 
            dplyr::mutate(Cancer = factor(Cancer, levels = rev(sort(Cancer)))) %>%
            dplyr::mutate(HR.label = paste0("HR=",formatC(HR, format = "e", digits = 2))) %>%
            dplyr::mutate(p.value_label = formatC(p.value, format = "e", digits = 2)) %>% 
            dplyr::mutate(p.value_symbol=case_when(
                p.value  < 0.001 ~ "***",
                p.value  < 0.01 ~ "**",
                p.value  < 0.05 ~ "*",
                TRUE ~ "ns"
            )) %>% 
            dplyr::mutate(pval_log = -log10(p.value)) %>% 
            dplyr::mutate(pval_log = ifelse(pval_log<10,pval_log, 10)) %>% 
            dplyr::mutate(Direct = ifelse(HR>1,"HR>1","HR<1")) %>% 
            ggplot(aes(x = Cancer, y = pval_log, fill = Direct)) + 
            geom_col(position="dodge") + 
            scale_fill_manual(values = fill_cols) + 
            xlab(NULL) + ylab(x_name_2) +
            ggtitle(label = title_name_2) +
            guides(fill = guide_legend(title = paste0("Group:",levels(data$sur_dat$Group)[2]))) +
            geom_hline(yintercept = -log10(multi_cox_line), color = "red") +
            coord_flip() +
            custom_theme + 
            # theme_minimal() +
            theme(legend.position = "top",
                text = element_text(size=axis_size_2),
                plot.title = element_text(size=isolate(title_size_2), hjust = 0.5)
                )  
        if(multi_cox_label=="HR value"){
            p = p1 + 
                geom_text(aes(y = max(pval_log),label = HR.label), size = label_size_2,
                        position=position_dodge(width=0.9), hjust = 1)
        } else if (multi_cox_label=="Signif.(symbol)"){
            p = p1 + 
                geom_text(aes(y = max(pval_log),label = p.value_symbol), size = label_size_2,
                        position=position_dodge(width=0.9), hjust = 1)
        } else if (multi_cox_label=="Signif.(value)"){
            p = p1 + 
                geom_text(aes(y = max(pval_log),label = p.value_label), size = label_size_2,
                        position=position_dodge(width=0.9), hjust = 1)
        }
    }
    p
}

#' @describeIn get_pancan_value Fetch copy number value from CCLE dataset
#' @export
get_ccle_cn_value <- function(identifier) {
  host <- "publicHub"
  dataset <- "ccle/CCLE_copynumber_byGene_2013-12-03"

  res <- check_exist_data(identifier, dataset, host)
  if (res$ok) {
    data <- res$data
  } else {
    data <- get_pancan_value(identifier, dataset = dataset, host = host)
    save_data(data, identifier, dataset, host)
  }

  unit <- "log(copy number/2)"
  report_dataset_info(dataset)
  res <- list(data = data, unit = unit)
  res
}




#' @param norm normalization method
#'
#' @describeIn get_pancan_value Fetch gene expression value from CCLE dataset
#' @export
get_ccle_gene_value <- function(identifier, norm = c("rpkm", "nc")) {
  host <- "publicHub"
  if (norm == "rpkm") {
    dataset <- "ccle/CCLE_DepMap_18Q2_RNAseq_RPKM_20180502"
  } else {
    dataset <- "ccle/CCLE_DepMap_18Q2_RNAseq_reads_20180502.log2"
  }
  res <- check_exist_data(identifier, dataset, host)
  if (res$ok) {
    expression <- res$data
  } else {
    expression <- get_pancan_value(identifier, dataset = dataset, host = host)
    save_data(expression, identifier, dataset, host)
  }

  if (norm == "rpkm") {
    unit <- "RPKM"
  } else {
    unit <- "log2(count+1)"
  }

  report_dataset_info(dataset)
  res <- list(expression = expression, unit = unit)
  res
}


#' @describeIn get_pancan_value Fetch gene protein expression value from CCLE dataset
#' @export
get_ccle_protein_value <- function(identifier) {
  ## NOTE: Only ~200 proteins available, so many identifiers will return NAs
  host <- "publicHub"
  dataset <- "ccle/CCLE_RPPA_20180123"

  res <- check_exist_data(identifier, dataset, host)
  if (res$ok) {
    expression <- res$data
  } else {
    expression <- get_pancan_value(identifier, dataset = dataset, host = host)
    save_data(expression, identifier, dataset, host)
  }

  unit <- "norm_value"
  report_dataset_info(dataset)
  res <- list(expression = expression, unit = unit)
  res
}

## .p_dataset_field("https://ucscpublic.xenahubs.net", "ccle/CCLE_RPPA_20180123")
## NOTE: this is different from .all_pancan_proteins
.all_ccle_proteins <- c(
  "14-3-3_beta", "14-3-3_epsilon_Caution", "14-3-3_zeta", "4E-BP1",
  "4E-BP1_pS65", "4E-BP1_pT37_T46", "4E-BP1_pT70", "53BP1", "A-Raf_pS299_Caution",
  "ACC1_Caution", "ACC_pS79", "ACVRL1_Caution", "ADAR1", "AMPK_alpha_Caution",
  "AMPK_pT172", "AR", "ASNS", "ATM", "Acetyl-a-Tubulin (Lys40)_Caution",
  "Akt", "Akt_pS473", "Akt_pT308", "Annexin 1", "Annexin_VII",
  "B-Raf_Caution", "B-Raf_pS445", "BRCA2_Caution", "Bad_pS112",
  "Bak_Caution", "Bap1 c-4", "Bax", "Bcl-2", "Bcl-xL", "Beclin_Caution",
  "Bid_Caution", "Bim(CST2933)", "Bim(EP1036)", "C-Raf(BD610151)_Caution",
  "C-Raf(MP05-739)", "C-Raf_pS338", "CD20 _Caution", "CD31", "CD49b",
  "CDK1", "Caspase-7_cleavedD198_Caution", "Caspase-8_Caution",
  "Caveolin-1", "Chk1_Caution", "Chk1_pS345_Caution", "Chk2", "Chk2_pT68_Caution",
  "Claudin-7", "Collagen_VI", "Cyclin_B1", "Cyclin_D1", "Cyclin_E1",
  "Cyclin_E2_Caution", "DJ-1", "Di-Ras3_Caution", "Dvl3", "E-Cadherin",
  "EGFR", "EGFR_pY1068_Caution", "EGFR_pY1173", "ER-alpha", "ER-alpha_pS118",
  "ERCC1_Caution", "ERK2_Caution", "ETS-1", "FASN", "FOXO3a_Caution",
  "FOXO3a_pS318_S321_Caution", "FRA1_Caution", "Fibronectin", "FoxM1",
  "G6PD", "GAPDH_Caution", "GATA3", "GSK-3-BETA_Caution", "GSK3-alpha-beta",
  "GSK3-alpha-beta_pS21_S9", "GSK3_pS9", "Gab2", "HER2", "HER2_pY1248_Caution",
  "HER3", "HER3_pY1289_Caution", "HSP70_Caution", "Heregulin",
  "IGFBP2", "INPP4B", "IRS1", "JAK2", "JNK2_Caution", "JNK_pT183_Y185",
  "Ku80_Caution", "Lck", "MAPK_pT202_Y204", "MDM2_pS166", "MDMX_MDM4(BetIHC-00108)_Caution",
  "MEK1", "MEK1_pS217_S221", "MIG-6", "MSH2", "MSH6_Caution", "MYH11",
  "Mre11_Caution", "Myosin IIa pS1943", "N-Cadherin", "N-Ras",
  "NDRG1_pT346", "NF-kB-p65_pS536_Caution", "NF2_Caution", "Notch1",
  "P-Cadherin_Caution", "PAI-1", "PARP_cleaved_Caution", "PCNA_Caution",
  "PDCD4_Caution", "PDK1", "PDK1_pS241", "PEA15", "PEA15_pS116",
  "PI3K-p110-alpha_Caution", "PI3K-p85", "PKC-alpha", "PKC-alpha_pS657_Caution",
  "PKC-delta_pS664", "PKC-pan_BetaII_pS660", "PR", "PRAS40_pT246",
  "PRDX1", "PREX1", "PTEN", "Paxillin_Caution", "Porin", "RAD51",
  "RBM15", "RSK1-2-3_Caution", "Rab25", "Rad50", "Raptor", "Rb_Caution",
  "Rb_pS807_S811", "Rictor_Caution", "Rictor_pT1135", "S6_pS235_S236",
  "S6_pS240_S244", "SCD1", "SETD2_Caution", "SF2", "SHP-2_pY542_Caution",
  "STAT3_Caution", "STAT3_pY705", "STAT5-alpha", "Shc_pY317", "Smac_Caution",
  "Smad1", "Smad3", "Smad4", "Snail_Caution", "Src", "Src_pY416_Caution",
  "Src_pY527", "Stathmin", "Syk", "TAZ", "TFRC", "TIGAR", "TSC1_Caution",
  "TTF1", "Transglutaminase", "Tuberin", "Tuberin_pT1462", "VAV1_Caution",
  "VEGFR2", "VHL_Caution", "XBP1_Caution", "XRCC1_Caution", "YAP_Caution",
  "YAP_pS127_Caution", "YB-1", "YB-1_pS102", "alpha-Catenin", "beta Actin_Caution",
  "beta-Catenin", "beta-Catenin_pT41_S45", "c-Jun_pS73", "c-Kit",
  "c-Met_Caution", "c-Met_pY1235", "c-Myc_Caution", "cIAP_Caution",
  "eEF2K", "eEF2_Caution", "eIF4E", "eIF4G_Caution", "mTOR", "mTOR_pS2448_Caution",
  "p14 Arf(BetA300-340A)_Caution", "p21", "p27", "p27_pT157_Caution",
  "p27_pT198", "p38 alpha MAPK", "p38_MAPK", "p38_pT180_Y182",
  "p53_Caution", "p62 Lck ligand_Caution", "p70S6K", "p70S6K_pT389",
  "p90RSK_Caution", "p90RSK_pT359_S363_Caution", "p90RSK_pT573_Caution"
)

# NOTE this result is different from get_pancan_mutation_status!
# It is maf like format, needs further processing.
# See ?read.maf to get nonsync mutation
#' @describeIn get_pancan_value Fetch gene mutation info from CCLE dataset
#' @export
get_ccle_mutation_status <- function(identifier) {
  if (utils::packageVersion("UCSCXenaTools") < "1.3.2") {
    stop("You need to update 'UCSCXenaTools' (>=1.3.2).", call. = FALSE)
  }

  host <- "https://ucscpublic.xenahubs.net"
  dataset <- "ccle/CCLE_DepMap_18Q2_maf_20180502"
  report_dataset_info(dataset)

  res <- check_exist_data(identifier, dataset, host)
  if (res$ok) {
    data <- res$data
  } else {
    query_list <- UCSCXenaTools::fetch_sparse_values(host, dataset, identifier)
    data <- as.data.frame(query_list$rows)
    data <- dplyr::full_join(
      dplyr::tibble(
        sampleID = query_list$samples
      ),
      data,
      by = "sampleID"
    )
    save_data(data, identifier, dataset, host)
  }

  report_dataset_info(dataset)
  data
}

custom_file <- reactiveValues()
custom_file$fData <- NULL
custom_file$pData <- NULL

selected_database_add_url_and_phenotype <- reactive({
  data <- selected_database_add_url()
  
  if (!is.null(data)) {
    # Add phenotype datasets
    # Type == "clinicalMatrix"
    add_datasets <- dplyr::filter(
      XenaData,
      XenaCohorts %in% unique(data$XenaCohorts),
      Type == "clinicalMatrix",
      !XenaDatasets %in% data$XenaDatasets
    )

    if (nrow(add_datasets) > 0) {
      message("Querying extra Phenotype datasets for pre-selected datasets.")
      add_query <- xe_query_url(add_datasets)

      add_datasets$download <- unlist(lapply(add_query$url, function(x) {
        as.character(tags$a(href = x, "download link"))
      }))
      add_datasets$browse <- unlist(lapply(add_query$browse, function(x) {
        as.character(tags$a(href = x, "browse Xena dataset page"))
      }))

      data <- dplyr::bind_rows(data, add_datasets) %>%
        dplyr::arrange(XenaCohorts)
    }
    
    if (!is.null(custom_file$fData)) {
      data <- dplyr::add_row(data, 
                             XenaCohorts = "Custom", 
                             XenaDatasets = "custom_feature_dataset", 
                             Type = "genomicMatrix",
                             SampleCount = ncol(custom_file$fData) - 1L)
    }
    if (!is.null(custom_file$pData)) {
      data <- dplyr::add_row(data, 
                             XenaCohorts = "Custom", 
                             XenaDatasets = "custom_phenotype_dataset", 
                             Type = "clinicalMatrix",
                             SampleCount = nrow(custom_file$pData))
    }
  } else if (!is.null(custom_file$pData) | !is.null(custom_file$fData)) {
    data <- dplyr::tibble()
    if (!is.null(custom_file$fData)) {
      data <- dplyr::bind_rows(
        data, 
        dplyr::tibble(
          XenaCohorts = "Custom", 
          XenaDatasets = "custom_feature_dataset", 
          Type = "genomicMatrix",
          SampleCount = ncol(custom_file$fData) - 1L,
          DataSubtype = NA_character_
        ))
    }
    if (!is.null(custom_file$pData)) {
      data <- dplyr::bind_rows(
        data, 
        dplyr::tibble(
          XenaCohorts = "Custom", 
          XenaDatasets = "custom_phenotype_dataset", 
          Type = "clinicalMatrix",
          SampleCount = nrow(custom_file$pData),
          DataSubtype = NA_character_
        ))
    }
  }
  data
})

selected_database_rm_phenotype <- reactive({
  data <- selected_database_add_url_and_phenotype()
  if (!is.null(data)) {
    # Remove phenotype datasets
    # Type != "clinicalMatrix"
    data <- subset(data, Type != "clinicalMatrix")
  }
  data
})

# Upload custom feature/phenotype data
observeEvent(input$ga_input_feature_file,{
  req(input$ga_input_feature_file)
  
  # ext <- tools::file_ext(input$ga_input_feature_file$name)
  # shiny::validate(need(ext %in% c("csv","tsv", "gz"), "Please upload a csv/tsv file"))
  
  inFile <- input$ga_input_feature_file
  if (is.null(inFile))
    return(NULL)
  df <- data.table::fread(inFile$datapath, header = TRUE)
  message("Saving custom feature data to temp directory.")
  saveRDS(df, file = file.path(tempdir(), "custom_feature_data.rds"))
  
  custom_file$fData <- df
})

observeEvent(input$ga_input_phenotype_file,{
  req(input$ga_input_phenotype_file)
  
  # ext <- tools::file_ext(input$ga_input_phenotype_file$name)
  # shiny::validate(need(ext %in% c("csv","tsv", "gz"), "Please upload a csv/tsv file"))
  
  inFile <- input$ga_input_phenotype_file
  if (is.null(inFile))
    return(NULL)
  df <- data.table::fread(inFile$datapath, header = TRUE)
  message("Saving custom phenotype data to temp directory.")
  saveRDS(df, file = file.path(tempdir(), "custom_phenotype_data.rds"))

  custom_file$pData <- df
})

output$ga_dataset_table <- DT::renderDataTable(
  {
    show_table <- selected_database_add_url_and_phenotype()
    if (!is.null(show_table)) {
      # 同时载入同队列的 phenotype (and probemap?)
      if (!"Label" %in% colnames(show_table)) {
        show_table <- show_table %>%
          dplyr::select(c("XenaCohorts", "XenaDatasets", "SampleCount", "DataSubtype"))
      } else {
        show_table <- show_table %>%
          dplyr::select(c("XenaCohorts", "XenaDatasets", "SampleCount", "DataSubtype", "Label", "Type", "download", "browse"))
      }
      
      colnames(show_table)[1:4] <- c("Cohort", "Dataset", "N", "Subtype")
    } else {
      sendSweetAlert(session,
        title = "Warning!", text = "Please select datasets from Repository page or upload your own data firstly!",
        type = "warning"
      )
    }
    
    show_table
  },
  escape = FALSE
)


# Individual analysis pages -------------------------------------------------------------
observeEvent(req(input$navbar=="General Analysis"),{
  callModule(
    server.modules_ga_scatter_correlation, "module_ga_scatter_correlation",
    selected_database_rm_phenotype, selected_database_add_url_and_phenotype,
    custom_file
  )
  callModule(
    server.modules_ga_matrix_correlation, "module_ga_matrix_correlation",
    selected_database_rm_phenotype, selected_database_add_url_and_phenotype,
    custom_file
  )
  callModule(
    server.modules_ga_group_comparison, "module_ga_group_comparison",
    selected_database_rm_phenotype, selected_database_add_url_and_phenotype,
    custom_file
  )
  callModule(
    server.modules_ga_surv_analysis, "module_ga_surv_analysis",
    selected_database_rm_phenotype, selected_database_add_url_and_phenotype,
    custom_file
  )
  callModule(
    server.modules_ga_dim_distribution, "module_ga_dim_distribution",
    selected_database_rm_phenotype, selected_database_add_url_and_phenotype,
    custom_file
  )
}, once = TRUE)  




# Show use alert ----------------------------------------------------------

observeEvent(input$use_ga_page, {
  shinyalert(
    title = "General Analysis Usage",
    text = paste(
      "Firstly, select datasets from Repository page, the datasets and corresonding clinical datasets will be automatically loaded here (You can also upload your own data with the upload button).",
      "Secondly, use any analysis feature below by clicking the tab.",
      "Lastly, control how to analyze from left panel and filter samples from right panel. The result plot should be shown at the middle.",
      sep = "\n\n"
    ),
    type = "info",
    timer = 0,
    confirmButtonCol = "#202324"
  )
})

observeEvent(input$ga_drop_button, {
  if (is.null(selected_database_add_url_and_phenotype())) {
    sendSweetAlert(session,
      title = "Warning!", text = "Please select datasets from Repository page firstly",
      type = "warning"
    )
  }
})

uiStatAnno <- function(id){
  ns <- NS(id)
  fluidPage(
    column(12,
           navlistPanel(
             tabPanel("Overall Drug Information",
                      tabsetPanel(
                        tabPanel("Drug and Cell Counts",
                                 plotOutput(ns("p_count_drugandcell"))
                        ),
                        tabPanel("Cell Subtypes",
                                 plotly::plotlyOutput(ns("p_count_subtype"))),
                        tabPanel("Drug and Cell Overlap Counts",
                                 plotOutput(ns("p_overlap_drug")),
                                 plotOutput(ns("p_overlap_cell"))
                        )
                      )
             ),
             tabPanel("Annotation",
                      tabsetPanel(
                        tabPanel("Cell",
                                 DT::dataTableOutput(ns("cell_anno"))),
                        tabPanel("Drug",
                                 DT::dataTableOutput(ns("drug_anno"))),
                      )),
           )),
  )
}

serverStatAnno <- function(input, output, session){
  ns <- session$ns
  # Plot ----
  output$p_count_drugandcell <- renderPlot({
    p_count_drugandcell
  })
  output$p_count_subtype <- plotly::renderPlotly({
    p_count_subtype
  })
  output$p_overlap_drug <- renderPlot({
    p_overlap_drug
  })
  output$p_overlap_cell <- renderPlot({
    p_overlap_cell
  })
  # Table ----
  output$drug_anno <- DT::renderDataTable({ 
    drug_anno
  }, options = list(scrollX = TRUE), selection = 'single')
  output$cell_anno <- DT::renderDataTable({ 
    cell_anno
  }, options = list(scrollX = TRUE), selection = 'single')
}

ui.page_PharmacoGenomics <- function() {
  navbarMenu(
    title = "PharmacoGenomics",
    icon = icon("buromobelexperte"),
    ## Drug-Omics Correlation Analysis ----
    tabPanel(
      "Drug-Omics Correlation Analysis",
      uiDrugOmicPair("DrugOmicPair")
    ),
    ## Profiles Display ----
    tabPanel(
      "Feature Abundance Profile in Databases",
      uiFeatureAcrossType("FeatureAcrossType")
    ),
    tabPanel(
      "Dimension Reduction Profile of Cell Drug Sensitivity",
      uiProfileDrugSens("ProfileDrugSens")  
    ),
    ## Features database significant analysis ----
    tabPanel(
      "Feature Scaling Association Analysis",
      uiFeatureDatabaseSig("FeatureDatabaseSig")
    ),
    ## Statistics and Annotations ----
    tabPanel(
      "Statistics and Annotations",
      uiStatAnno("StatAnno")
    ),
    ## Contact ----
    tabPanel("Contact",
             fluidPage(
               strong("Feel free to talk with me if you find any bugs or have any suggestions. :)"),
               p(""),
               p("Email: mugpeng@foxmail.com"),
               p("github: https://github.com/mugpeng"),
               p("You can visit https://github.com/mugpeng/OmicsPharDB to reach the toturial.")
             ))
  )
}

observeEvent(input$hiplot_switch, {
  message("Setting mirror, use hiplot: ", input$hiplot_switch)
  if (input$hiplot_switch) {
    # Check the XenaTools version
    if (utils::packageVersion("UCSCXenaTools") < "1.4.3") {
      show_alert(
        title = "Error !!",
        text = "Cannot work with Hiplot mirror if UCSCXenaTools version< 1.4.3, please upgrade it!",
        type = "error"
      )
    }
    options(use_hiplot = TRUE)
  } else {
    options(use_hiplot = FALSE)
  }
})

add_signature_UI = function(id, database = "toil"){
	ns = NS(id)

	if (!is.null(database)) {
	  if (database == "toil") {
	    choices = c("mRNA Expression","Transcript Expression","DNA Methylation",
	    						"Protein Expression","miRNA Expression","Mutation status","Copy Number Variation")
	  } else if (database == "pcawg") {
	    choices = c("mRNA Expression","Gene Fusion", "Promoter Activity",
	    						"miRNA Expression","APOBEC Mutagenesis")
	  } else if (database == "ccle") {
	    choices = c("mRNA Expression","Protein Expression",
	    						"Copy Number Variation","Mutation status")
	  }
	}
	selected = choices[1]
	tagList(
		dropMenu(
			shinyWidgets::actionBttn(
				ns("sig_edit"), "Edit",
		        style = "gradient",
		        icon = icon("arrows-to-dot"),
		        color = "primary",
		        block = TRUE,
		        size = "sm"
			),
			div(h5("Set your custom signature:"),style="width:800px;"),

			wellPanel(
				style = "background: #f7f7f7",
				h4("1. Basic information"),
				fluidRow(column(6,
					h5("(1) Signature Name"),
					textInput(ns("sig_name"), NULL, value="Signature_X"),
				)),
				fluidRow(column(6,
					h5("(2) Signature Type"),
					selectInput(ns("data_origin"),NULL,
						choices = choices, selected = selected),
				)),
			),

			wellPanel(
				style = "background: #f7f7f7",
				h4("2. Signature molecules"),
				fluidRow(
					column(
						4,
					    actionBttn(
					      inputId = ns("add_bt"),
					      label = "Select&Add molecule",
					      color = "primary",
					      style = "bordered", size = "sm",
					      block = F
					    )
					),
					column(
						4,
						virtualSelectInput(ns("add_mol"), NULL, NULL, 
						                   search = TRUE,
						                   allowNewOption = TRUE,
						                   dropboxWidth = "200%")
					)
				),
				fluidRow(column(8,
					div(style = "border-top: 1px solid #000; margin-top: 0px; margin-bottom: 0px;"),
				)),
			    fluidRow(
			    	column(3, h5("")),
			    	column(2, h5("Direction:")),
			    	column(2, h5("Absolute Coef:")),
			    	column(1, h5("Check:")),
			    ),
				fluidRow(column(8,
					div(style = "border-top: 1px dashed #000; margin-top: 0px; margin-bottom: 5px;"),
				)),
			    uiOutput(ns("multi_condi.ui")),
				fluidRow(column(8,
					div(style = "border-top: 1px solid #000; margin-top: 0px; margin-bottom: 0px;"),
				)),
				uiOutput(ns("del_reset_bt.ui")),
			    br(),
				verbatimTextOutput(ns("print_fm")),
				br(),
			),

			wellPanel(
				style = "background: #f7f7f7",
				fluidRow(column(6,h4("3. Signature data"))),
			    actionBttn(
			      inputId = ns("query_bt"),
			      label = "Query data",
			      color = "primary",
			      style = "bordered", size = "sm",
			      block = F
			    ),
			    actionBttn(
			      inputId = ns("add_custom"),
			      label = "Add/Update metadata",
			      color = "primary",
			      style = "bordered", size = "sm",
			      block = F
			    ),  
			    verbatimTextOutput(ns("sig_dat2custom_tips")),


			    dataTableOutput(ns("sig_dat_table")),
		        shinyjs::hidden(
		          wellPanel(
		            id = ns("save_csv"),
		            downloadButton(ns("downloadTable"), "Save as csv")
		          )
		        ),

			),




			placement = "right-end",
			maxWidth = "1000px"
		)

	)
}



add_signature_Server = function(input, output, session, database = "toil"){
	ns <- session$ns


	id_option = switch(database, 
			"toil"=tcga_id_option,
			"pcawg"=pcawg_id_option,
			"ccle"=ccle_id_option)

	observeEvent(input$data_origin, {
		genomic_profile_choices <- reactive({
		  genomic_profile = ifelse(is.null(input$data_origin),"mRNA Expression",input$data_origin)

		  id_option[["Molecular profile"]][[genomic_profile]]
		})
	    updateVirtualSelect(
	      "add_mol",
	      choices = genomic_profile_choices()$all,
	      selected = genomic_profile_choices()$default
	    )
	})
	dynamic_condi = reactiveValues(mols = NULL, add = 0, del = 0, sum = 0, check = NULL)

	observeEvent(input$add_bt, {
		dynamic_condi$mols = c(dynamic_condi$mols, input$add_mol)
		data_type = switch(input$data_origin,
               `mRNA Expression` = "mRNA",
               `Transcript Expression` = "transcript",
               `DNA Methylation` = "methylation",
               `Protein Expression` = "protein",
               `miRNA Expression` = "miRNA",
               `Mutation status` = "mutation",
               `Copy Number Variation` = "cnv",
               # `mRNA Expression` = "mRNA",
               `Promoter Activity` = "promoter",
               `Gene Fusion` = "fusion",
               # `miRNA Expression` = 'miRNA',
               `APOBEC Mutagenesis` = "APOBEC"
		)
		check_dat = query_pancan_value(input$add_mol, data_type = data_type, database = database)
		check_dat = all(is.na(check_dat[[1]])) #判断是否为NA值
		check_dat = ifelse(check_dat, "×", "√")
		dynamic_condi$check = c(dynamic_condi$check, check_dat)
	})

	observeEvent(input$del_bt, {
		# 删除末尾元素
		if(length(dynamic_condi$mols)>0){
			dynamic_condi$mols = dynamic_condi$mols[-1*length(dynamic_condi$mols)]
		}
	})
	observeEvent(input$reset_bt, {
		# 重置所有元素
		if(length(dynamic_condi$mols)>0){
			dynamic_condi$mols = NULL
		}
	})
	observeEvent(input$data_origin, {
		dynamic_condi$mols = NULL
	})

	output$multi_condi.ui = renderUI({
    	if(length(dynamic_condi$mols) == 0) return()
    	inputTagList <- tagList()
    	lapply(seq(dynamic_condi$mols),function(i){
    		# item1(id) -- direction(label)
    		id_condi_item1 = paste0("item1_",i)
    		# item2(id) -- coef(label)
    		id_condi_item2 = paste0("item2_",i)

    		# 添加新筛选器，不重置旧数据
			new_direc <- "+"
			if (id_condi_item1 %in% names(input)) {
				new_direc <- input[[id_condi_item1]]
			}
			new_coef = 1
			if (id_condi_item2 %in% names(input)) {
				new_coef <- input[[id_condi_item2]]
			}

			dynamic_input = fluidRow(
				column(
					3, 
					p(i,". ",strong(dynamic_condi$mols[i]))
				),
				column(
					2, 
					selectInput(ns(id_condi_item1), NULL,#label_condi_item1, 
	    						c("+","-"), selected=new_direc)),
				column(
					2, 
					numericInput(ns(id_condi_item2),  NULL,#label_condi_item3, 
	    						value = new_coef)),
				column(
					1,
					p(strong(dynamic_condi$check[i]))
				)
			)

			inputTagList <<- tagAppendChild(inputTagList,       
			                              	dynamic_input)
    	})
    	inputTagList
	})

	output$del_reset_bt.ui = renderUI({
		if(length(dynamic_condi$mols)>0){
			tagList(
			    actionBttn(
			      inputId = ns("del_bt"),
			      label = "Del last molecule",
			      color = "primary",
			      style = "bordered", size = "sm",
			      block = F
			    ),
			    actionBttn(
			      inputId = ns("reset_bt"),
			      label = "Reset all molecules",
			      color = "primary",
			      style = "bordered", size = "sm",
			      block = F
			    )
			)
		}
	})


	formula = reactive({
		if(length(dynamic_condi$mols)==0) return(NULL)
		f_left = paste0(input$sig_name, " = ")
		f_right = sapply(seq(dynamic_condi$mols), function(i){
			# i = 1
			paste0(input[[paste0("item1_",i)]],
				  " ",
				  input[[paste0("item2_",i)]],
				  "*",
				  dynamic_condi$mols[i])
		},USE.NAMES = FALSE) %>% paste0(., collapse = " ")
		paste0(f_left, f_right)
	})

	output$print_fm = renderPrint({
		cat(formula())
	})

	sig_dat = eventReactive(input$query_bt,{
		# isolate() 仅点击query/add按钮时才更新signature name/type等
		fm = str_split(isolate(formula()), " = ")[[1]][2]
		data_type = switch(isolate(input$data_origin),
               `mRNA Expression` = "mRNA",
               `Transcript Expression` = "transcript",
               `DNA Methylation` = "methylation",
               `Protein Expression` = "protein",
               `miRNA Expression` = "miRNA",
               `Mutation status` = "mutation",
               `Copy Number Variation` = "cnv",
               # `mRNA Expression` = "mRNA",
               `Promoter Activity` = "promoter",
               `Gene Fusion` = "fusion",
               # `miRNA Expression` = 'miRNA',
               `APOBEC Mutagenesis` = "APOBEC"
		)
		dat = query_pancan_value(fm, data_type = data_type, database = database)
		dat = data.frame(dat[[1]]) %>% 
		  tibble::rownames_to_column("Sample")
		colnames(dat)[2] = str_split(isolate(formula()), " = ")[[1]][1]
		dat
	})

	output$sig_dat_table = renderDataTable({
		datatable(sig_dat(),options = list(pageLength = 3,
			columnDefs = list(list(className = 'dt-center', targets="_all"))))
	})

	observeEvent(input$query_bt, {
	  if (length(dynamic_condi$mols) >= 1) {
	    shinyjs::show(id = "save_csv")
	  } else {
	    shinyjs::hide(id = "save_csv")
	  }
	})

	output$downloadTable <- downloadHandler(
	  filename = function() {
	    paste0(input$sig_name, ".csv")
	  },
	  content = function(file) {
	    write.csv(sig_dat(), file, row.names = FALSE)
	  }
	)

	sig_dat2custom = reactive({
		if(input$add_custom==0){
			NULL
		} else {
			sig_dat()
		}
	})

	sig_dat2custom_tips = eventReactive(input$add_custom,{
		ts = timestamp()
		paste0("Your latest quried signature has been added to the costum metadata.\n",ts)
	})
	output$sig_dat2custom_tips = renderPrint({
		cat(sig_dat2custom_tips())
	})

	return(sig_dat2custom)
}

#' Group TPC samples by build-in or custom phenotype and support filtering or merging operations
#'
#' @param cancer select cancer cohort(s)
#' @param custom upload custom phenotype data
#' @param group target group names
#' @param filter_by filter samples by one or multiple criterion
#' @param merge_by merge the target group for main categories
#' @param return_all return the all phenotype data
#' @param filter_id directly filter samples by provided sample ids
#' @param merge_quantile  whether to merge numerical variable by percentiles
#' @param database one of c("toil","pcawg","ccle")
#'
#' @return a list object with grouping samples and statistics
#' @export
#'
#' @examples
#' \dontrun{
#' query_tcga_group(group = "Age")
#'
#' query_tcga_group(
#'   cancer = "BRCA",
#'   group = "Stage_ajcc"
#' )
#'
#' query_tcga_group(
#'   cancer = "BRCA",
#'   group = "Stage_ajcc",
#'   filter_by = list(
#'     c("Code", c("TP"), "+"),
#'     c("Stage_ajcc", c(NA), "-")
#'   )
#' )
#'
#' query_tcga_group(
#'   cancer = "BRCA",
#'   group = "Stage_ajcc",
#'   filter_by = list(
#'     c("Age", c(0.5), "%>")
#'   )
#' )
#'
#' query_tcga_group(
#'   cancer = "BRCA",
#'   group = "Stage_ajcc",
#'   filter_by = list(
#'     c("Age", c(60), ">")
#'   )
#' )
#'
#' query_tcga_group(
#'   cancer = "BRCA",
#'   group = "Stage_ajcc",
#'   merge_by = list(
#'     "Early" = c("Stage I"),
#'     "Late" = c("Stage II", "Stage III", "Stage IV")
#'   )
#' )
#'
#' query_tcga_group(
#'   cancer = "BRCA",
#'   group = "Age",
#'   merge_by = list(
#'     "Young" = c(20, 60),
#'     "Old" = c(60, NA)
#'   )
#' )
#'
#' query_tcga_group(
#'   cancer = "BRCA",
#'   group = "Age",
#'   merge_quantile = TRUE,
#'   merge_by = list(
#'     "Young" = c(0, 0.5),
#'     "Old" = c(0.5, 1)
#'   )
#' )
#' }
query_tcga_group <- function(database = c("toil", "pcawg", "ccle"),
                             cancer = NULL,
                             custom = NULL,
                             group = "Gender",
                             filter_by = NULL,
                             filter_id = NULL,
                             merge_by = NULL,
                             merge_quantile = FALSE,
                             return_all = FALSE) {
  # step1: load build-in data
  database <- match.arg(database)
  # data("tcga_clinical_fine")
  # data("pcawg_info_fine")
  # data("ccle_info_fine")
  meta_data <- switch(database,
    "toil" = UCSCXenaShiny::tcga_clinical_fine,
    "pcawg" = UCSCXenaShiny::pcawg_info_fine,
    "ccle" = UCSCXenaShiny::ccle_info_fine
  )
  colnames(meta_data)[1:2] <- c("Sample", "Cancer") # CCLE:c("CCLE_name","Primary_Site)



  # Step2: add user-customized data
  if (!is.null(custom)) {
    dup_names <- intersect(colnames(meta_data)[-1:-2], colnames(custom)[-1])
    if (length(dup_names) > 0) {
      meta_data <- meta_data %>%
        dplyr::select(!all_of(dup_names))
    } # 如果有重复列名，则去除原始metadata的重复列
    colnames(custom)[1] <- "Sample"
    meta_data <- dplyr::inner_join(meta_data, custom) # Note: Only consider the intersection
  }

  # step3: filter cancer(s)
  if (is.null(cancer)) {
    cancer <- unique(meta_data$Cancer)
  }
  meta_data_sub <- meta_data %>% dplyr::filter(.data$Cancer %in% cancer)

  # 检查选择的分组参数是否在现有的列名中
  if (!any(colnames(meta_data_sub) == group)) {
    stop(paste0(
      "Please input the right group names:\n",
      paste(colnames(meta_data_sub)[-1:-2], collapse = " ")
    ))
  }

  # step5: filter by specialized conditions
  if (!is.null(filter_by)) {
    ## 按list内条件顺序级联筛选
    for (i in seq(filter_by)) {
      # i = 1
      filter_by_L1 <- trimws(filter_by[[i]][1])
      filter_by_L3 <- trimws(filter_by[[i]][3])
      filter_by_L2 <- sapply(strsplit(filter_by[[i]][2], "|", fixed = T)[[1]],
        trimws,
        USE.NAMES = FALSE
      )
      filter_by_L2[filter_by_L2 %in% "NA"] <- NA

      # 6种过滤方式
      if (filter_by_L3 == "+") { # 保留
        meta_data_sub <- meta_data_sub %>%
          dplyr::filter(.data[[filter_by_L1]] %in% filter_by_L2)
      } else if (filter_by_L3 == "-") { # 剔除
        meta_data_sub <- meta_data_sub %>%
          dplyr::filter(!.data[[filter_by_L1]] %in% filter_by_L2)
      } else if (filter_by_L3 == ">") { # 大于 绝对值
        filter_by_L2 <- as.numeric(filter_by_L2)
        meta_data_sub <- meta_data_sub %>%
          dplyr::filter(.data[[filter_by_L1]] > filter_by_L2)
      } else if (filter_by_L3 == "%>") { # 大于 分位数
        filter_by_L2 <- as.numeric(filter_by_L2)
        meta_data_sub <- meta_data_sub %>%
          dplyr::group_by("Cancer") %>%
          dplyr::filter(.data[[filter_by_L1]] >
            quantile(.data[[filter_by_L1]], filter_by_L2, na.rm = T))
      } else if (filter_by_L3 == "<") { # 小于 绝对值
        filter_by_L2 <- as.numeric(filter_by_L2)
        meta_data_sub <- meta_data_sub %>%
          dplyr::filter(.data[[filter_by_L1]] < filter_by_L2)
      } else if (filter_by_L3 == "%<") { # 小于 分位数
        filter_by_L2 <- as.numeric(filter_by_L2)
        meta_data_sub <- meta_data_sub %>%
          dplyr::group_by("Cancer") %>%
          dplyr::filter(.data[[filter_by_L1]] <
            quantile(.data[[filter_by_L1]], filter_by_L2, na.rm = T))
      }
    }
  }



  # step3-2: filter by sample id
  if (!is.null(filter_id)) {
    meta_data_sub <- meta_data_sub %>%
      dplyr::filter(.data$Sample %in% filter_id)
  }


  dim(meta_data_sub)


  # step3: merge group

  if (!is.null(merge_by)) {
    # if(class(meta_data_sub[,group,drop=T])=="numeric"){
    if (inherits(meta_data_sub[, group, drop = T], "numeric")) {
      # 分位数:每种肿瘤单独的分位数
      if (merge_quantile) {
        for (i in seq(merge_by)) {
          if (is.na(merge_by[[i]][1])) {
            merge_by[[i]][1] <- 0
          }
          if (is.na(merge_by[[i]][2])) {
            merge_by[[i]][2] <- 1
          }
        }
        meta_data_sub_split <- split(meta_data_sub, meta_data_sub$Cancer)
        meta_data_sub <- lapply(meta_data_sub_split, function(meta_data_one) {
          # meta_data_one = meta_data_sub_split[["CESC"]]
          dat_num <- meta_data_one[, group, drop = T]
          meta_data_one[, group, drop = T] <- NA
          merge_by_one <- merge_by
          for (i in seq(merge_by_one)) {
            merge_by_one[[i]] <- quantile(dat_num, merge_by_one[[i]], na.rm = T)
          }
          # 对于较大值组，左右均闭，这样可能导致大组的数目偏多（中位数）
          meta_data_one[, group, drop = T][which(findInterval(dat_num, merge_by_one[[1]]) == 1)] <- names(merge_by_one)[1]
          meta_data_one[, group, drop = T][which(findInterval(dat_num, merge_by_one[[2]], rightmost.closed = TRUE) == 1)] <- names(merge_by_one)[2]
          meta_data_one
        }) %>% do.call(rbind, .)
      } else {
        dat_num <- meta_data_sub[, group, drop = T]
        meta_data_sub[, group, drop = T] <- NA
        for (i in seq(merge_by)) {
          if (is.na(merge_by[[i]][1])) {
            merge_by[[i]][1] <- min(dat_num, na.rm = T)
          }
          if (is.na(merge_by[[i]][2])) {
            merge_by[[i]][2] <- max(dat_num, na.rm = T)
          }
        }
        if (merge_by[[1]][1] < merge_by[[2]][2] && merge_by[[1]][2] > merge_by[[2]][1]) {
          stop("Please provide two independent grouping range")
        }
        meta_data_sub[, group, drop = T][which(findInterval(dat_num, merge_by[[1]]) == 1)] <- names(merge_by)[1]
        meta_data_sub[, group, drop = T][which(findInterval(dat_num, merge_by[[2]], rightmost.closed = TRUE) == 1)] <- names(merge_by)[2]
      }
      # } else if (class(meta_data_sub[,group,drop=T])=="character"){
    } else if (inherits(meta_data_sub[, group, drop = T], "character")) {
      dat_chr <- meta_data_sub[, group, drop = T]
      meta_data_sub[, group, drop = T] <- NA
      if (length(intersect(merge_by[[1]], merge_by[[2]]))) {
        stop("Please provide two independent grouping range")
      }

      meta_data_sub[, group, drop = T][dat_chr %in% merge_by[[1]]] <- names(merge_by)[1]
      meta_data_sub[, group, drop = T][dat_chr %in% merge_by[[2]]] <- names(merge_by)[2]
    }
    meta_data_sub <- meta_data_sub[!is.na(meta_data_sub[, group, drop = T]), ]
  }

  # 对否返回全部值
  if (return_all) {
    return(meta_data_sub)
  }

  meta_data_sub2 <- meta_data_sub[, c("Sample", "Cancer", group)]
  meta_data_sub2 <- meta_data_sub2[, !duplicated(colnames(meta_data_sub2))]

  sub2_group_stat <- meta_data_sub2 %>%
    dplyr::select(last_col()) %>%
    dplyr::mutate(across(where(is.character), as.factor)) %>%
    summary()

  list(data = meta_data_sub2, stat = sub2_group_stat)
}

observeEvent(input$bt03, {
  updateNavbarPage(inputId = "navbar", 
    selected = "Citation")
  shinyjs::runjs("window.scrollTo(0, 0)")
})




output$slick_output <- slickR::renderSlickR({
  # imgs = paste0("slick_img/N",1:6,".png")
  # system.file("shinyapp/www", package = "UCSCXenaShiny")
  imgs = paste0(system.file("shinyapp/www", package = "UCSCXenaShiny"),"/slick_img/N",1:6,".png")
  x = slickR::slickR(imgs, height = 600, width = "80%")  + #, slideType = 'img-lazy'
        slickR::settings(arrows = TRUE, dots = TRUE, autoplay = TRUE, autoplaySpeed = 2500)
  x
})


observeEvent(input$link_to_q1_n, {
  updateNavbarPage(inputId = "navbar", 
    selected = "TCGA (GTEx): Molecular comparison")
  shinyjs::runjs("window.scrollTo(0, 0)")
})

observeEvent(input$link_to_q2_n, {
  updateNavbarPage(inputId = "navbar", 
    selected = "TCGA: Molecular correlation")
  shinyjs::runjs("window.scrollTo(0, 0)")
})

observeEvent(input$link_to_q3_n, {
  updateNavbarPage(inputId = "navbar", 
    selected = "TCGA: Survival analysis")
  shinyjs::runjs("window.scrollTo(0, 0)")
})


observeEvent(input$link_to_p1_n, {
  updateNavbarPage(inputId = "navbar", 
    selected = "TCGA: Comparison Analysis")
  shinyjs::runjs("window.scrollTo(0, 0)")
})

observeEvent(input$link_to_p2_n, {
  updateNavbarPage(inputId = "navbar", 
    selected = "TCGA: Correlation Analysis")
  shinyjs::runjs("window.scrollTo(0, 0)")
})

observeEvent(input$link_to_p3_n, {
  updateNavbarPage(inputId = "navbar", 
    selected = "TCGA: Survival Analysis")
  shinyjs::runjs("window.scrollTo(0, 0)")
})

ui.modules_ga_matrix_correlation <- function(id) {
  ns <- NS(id)
  fluidPage(
    fluidRow(
      column(
        3,
        wellPanel(
          h4("Analysis Controls"),
          uiOutput(ns("ga_data1_id")),
          virtualSelectInput(
            inputId = ns("ga_data1_mid"), # molecule identifier
            label = "Molecule identifiers:",
            choices = NULL,
            multiple = TRUE,
            search = TRUE,
            allowNewOption = TRUE,
            dropboxWidth = "200%"
          ),
          selectInput(ns("ga_matrix_type"), "Matrix Type",
            choices = c("full", "upper", "lower"), selected = "full", multiple = FALSE
          ),
          selectInput(ns("ga_test_type"), "Test Type",
            choices = c("parametric", "nonparametric", "robust", "bayes"),
            selected = "parametric", multiple = FALSE
          ),
          selectInput(ns("ga_test_adjust"), "Adjust Method",
            choices = c(
              "holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr",
              "none"
            ),
            selected = "holm", multiple = FALSE
          ),
          sliderInput(ns("ga_sig_level"), "Signif Level", 0, 1, 0.05, step = 0.01),
          materialSwitch(
            inputId = ns("ga_use_partial"),
            label = "Partial correlation?",
            value = FALSE,
            status = "primary"
          ),
          colourpicker::colourInput(inputId = ns("ga_lower_col"), "Color for negative", "#E69F00"),
          colourpicker::colourInput(inputId = ns("ga_higher_col"), "Color for positive", "#009E73"),
          actionBttn(
            inputId = ns("ga_go"),
            label = "Submit",
            style = "gradient",
            icon = icon("check"),
            color = "default",
            block = TRUE,
            size = "sm"
          )
        )
      ),
      column(
        6,
        plotOutput(ns("ga_output")),
        DT::dataTableOutput(ns("ga_output_data"))
      ),
      column(
        3,
        wellPanel(
          h4("Sample Filters"),
          uiOutput(ns("ga_data_filter1_id")),
          actionBttn(
            inputId = ns("ga_filter_button"),
            label = "Click to filter!",
            color = "primary",
            style = "bordered",
            size = "sm"
          ),
          tags$br(),
          tags$br(),
          numericInput(inputId = ns("height"), label = "Height", value = 8),
          numericInput(inputId = ns("width"), label = "Width", value = 10),
          column(
            width = 12, align = "center",
            prettyRadioButtons(
              inputId = ns("device"),
              label = "Choose plot format",
              choices = c("png", "pdf"),
              selected = "png",
              inline = TRUE,
              icon = icon("check"),
              animation = "jelly",
              fill = TRUE
            )
          ),
          downloadBttn(
            outputId = ns("download"),
            style = "gradient",
            color = "default",
            block = TRUE,
            size = "sm"
          )
        )
      )
    )
  )
}


server.modules_ga_matrix_correlation <- function(
  input, output, session,
  selected_database_rm_phenotype, selected_database_add_url_and_phenotype,
  custom_file) {
  ns <- session$ns

  output$ga_data1_id <- renderUI({
    show_table <- selected_database_rm_phenotype()
    selectInput(
      inputId = ns("ga_data1_id"),
      label = "Select dataset:",
      choices = c("NONE", unique(show_table$XenaDatasets)),
      selected = "NONE",
      multiple = FALSE
    )
  })

  observe({
    updateVirtualSelect(
      "ga_data1_mid",
      choices = if (is.null(custom_file$fData)) all_preload_identifiers else
        unique(c(custom_file$fData[[1]], all_preload_identifiers)),
      selected = c("TP53", "KRAS", "PTEN")
    )
  })

  selected_samps <- reactiveValues(id = NULL)
  p_scatter <- eventReactive(input$ga_go, {
    if (is.null(selected_samps$id)) {
      message("All samples selected for analysis.")
    } else {
      message(length(selected_samps$id), " samples selected for analysis.")
    }
    tryCatch(
      vis_identifier_multi_cor(
        isolate(input$ga_data1_id),
        isolate(input$ga_data1_mid),
        matrix.type = isolate(input$ga_matrix_type),
        type = isolate(input$ga_test_type),
        partial = isolate(input$ga_use_partial),
        sig.level = isolate(input$ga_sig_level),
        p.adjust.method = isolate(input$ga_test_adjust),
        color_low = isolate(input$ga_lower_col),
        color_high = isolate(input$ga_higher_col),
        samples = isolate(selected_samps$id)
      ),
      error = function(e) {
        message("General analysis plot error:")
        print(e$message)
        "Error"
      }
    )
  })
  output$download <- downloadHandler(
    filename = function() {
      paste0(Sys.Date(), "_cormat_heatmap.", input$device)
    },
    content = function(file) {
      ggplot2::ggsave(
        filename = file, plot = p_scatter(), device = input$device,
        units = "cm", width = input$width, height = input$height, dpi = 600
      )
    }
  )

  observeEvent(input$ga_go, {
    # Analyze correlation with 2 input datasets and identifiers
    output$ga_output <- renderPlot(
      if (inherits(p_scatter(), c("ggplot", "grob"))) {
        print(p_scatter())
      } else {
        if (!requireNamespace("ggpubr") | !requireNamespace("ggstatsplot")) {
          sendSweetAlert(
            session,
            title = "Error",
            text = "Error to lack of ggpubr/ggstatsplot package. Please make sure the according package has been installed.",
            type = "error"
          )
        }else {
          sendSweetAlert(
            session,
            title = "Error",
            text = tags$span(
              tags$p("Error to query data and plot. Please make sure dataset is selected and the selected dataset is 'genomicMatrix' type."),
              tags$p("'genomicMatrix' type means the dataset is stored in feature-by-sample format, e.g., gene-by-sample expression matrix."),
              tags$p("The type of datasets can be found at the dataset table by clicking 'Pre-selected Datasets for Analysis' on the 'General Analysis' Page."),
              tags$img(src = "20210708184045.png",
                       alt = "errorImg", width = "98%", height = "98%")
            ),
            type = "error"
          )
        }
      }
    )

    output$ga_output_data <- DT::renderDataTable(server = FALSE, {
      if (inherits(p_scatter(), "ggplot")) {
        dt <- p_scatter()$data
        if (input$ga_test_adjust == "none") {
          names(dt)[4] <- "p"
          caption <- "Note: the p values are not adjusted."
        } else {
          names(dt)[4] <- paste0("p.adj (", input$ga_test_adjust, ")")
          caption <- paste0("Note: the p values are adjusted by ", input$ga_test_adjust, " method.")
        }
        DT::datatable(
          dt,
          rownames = FALSE,
          extensions = c("Buttons"),
          caption = tags$caption(
            style = "caption-side: bottom; text-align: left; margin: 8px 0;",
            caption
          ),
          options = list(
            pageLength = 5,
            dom = "Bfrtip",
            buttons = list(
              list(
                extend = "csv", text = "Download Current Page", filename = "page",
                exportOptions = list(
                  modifier = list(page = "current")
                )
              ),
              list(
                extend = "csv", text = "Download Full Results", filename = "data",
                exportOptions = list(
                  modifier = list(page = "all")
                )
              )
            )
          )
        )
      }
    })
  })

  output$ga_data_filter1_id <- renderUI({
    show_table <- selected_database_add_url_and_phenotype()
    selectInput(
      inputId = ns("ga_data_filter1_id"),
      label = "Select phenotype dataset:",
      choices = c("NONE", unique(show_table$XenaDatasets[show_table$XenaDatasets %in% phenotype_datasets])),
      selected = "NONE",
      multiple = FALSE
    )
  })

  observeEvent(input$ga_data_filter1_id, {
    # If phenotype dataset has been reset, we use all samples for plotting
    if (length(input$ga_data_filter1_id)) {
      if (input$ga_data_filter1_id == "NONE") {
        selected_samps$id <- NULL
      }
    }
  })

  observeEvent(input$ga_filter_button, {
    message("Sample filter button is clicked by user.")
    pdataset <- setdiff(input$ga_data_filter1_id, "NONE")

    if (length(pdataset)) {
      showModal(
        modalDialog(
          title = "Filter samples for analysis",
          size = "l",
          fluidPage(
            wellPanel(
              h4("1. Select available columns on the left to right"),
              uiOutput(ns("ga_col_chooser")),
              tags$br(),
              actionBttn(
                inputId = ns("show_or_update_ptable"),
                label = "Show/Update Phenotype Table",
                color = "primary",
                style = "bordered",
                size = "sm",
                block = F
              )
            ),
            wellPanel(
              h4("2. Filter rows by SearchPanels"),
              DT::dataTableOutput(ns("ga_phenotype_data"))
            ),
            wellPanel(
              h4("3. Specify sample column and hit button"),
              uiOutput(ns("ga_select_samp_col")),
              actionBttn(
                inputId = ns("ga_filter_submit_button"),
                label = "Submit to filter",
                color = "primary",
                style = "bordered", size = "sm",
                block = F
              )
            ),
            wellPanel(
              h4("4. After hitting button, dismiss this page and click submit button")
            )
          )
        )
      )

      phenotype_table <- XenaGenerate(subset = XenaDatasets == pdataset) %>%
        XenaQuery() %>%
        XenaDownload(destdir = XENA_DEST) %>%
        XenaPrepare()

      output$ga_col_chooser <- renderUI({
        all_cols <- colnames(phenotype_table)
        sel_idx <- seq_len(min(length(all_cols), 5))
        chooserInput(ns("ga_col_chooser"), "Available columns", "Selected columns",
          if (length(all_cols) == length(sel_idx)) c() else all_cols[-sel_idx],
          all_cols[sel_idx],
          size = 5, multiple = TRUE
        )
      })

      observeEvent(input$show_or_update_ptable, {
        selected_cols <- isolate(input$ga_col_chooser$right)
        if (length(selected_cols)) {
          message("Following columns selected by users from sample filter window.")
          print(selected_cols)

          output$ga_phenotype_data <- DT::renderDataTable(server = FALSE, {
            DT::datatable(
              phenotype_table %>%
                dplyr::select(all_of(selected_cols)),
              rownames = FALSE,
              extensions = c("Buttons", "Select", "SearchPanes"), # "Scroller" causes bug in searchPanel
              options = list(
                dom = "Bfrtip", # P
                buttons = list(
                  "searchPanes",
                  list(
                    extend = "csv", text = "Download Current Page", filename = "page",
                    exportOptions = list(
                      modifier = list(page = "current")
                    )
                  ),
                  list(
                    extend = "csv", text = "Download Full Results", filename = "data",
                    exportOptions = list(
                      modifier = list(page = "all")
                    )
                  )
                ),
                scrollY = 350,
                scrollX = 300,
                deferRender = TRUE,
                # scroller = TRUE,
                stateSave = FALSE
              ),
              selection = "none"
            )
          })
        } else {
          sendSweetAlert(session, title = "Warning", type = "warn", text = "Please select at least 1 column!")
        }
      })

      output$ga_select_samp_col <- renderUI({
        selectInput(
          inputId = ns("ga_select_samp_col"),
          label = "Select sample column:",
          choices = c("NONE", input$ga_col_chooser$right),
          selected = "NONE",
          multiple = FALSE
        )
      })
      observeEvent(input$ga_filter_submit_button, {
        rows_filtered <- input$ga_phenotype_data_rows_all
        col_sample <- input$ga_select_samp_col
        keep_samples <- phenotype_table[rows_filtered, , drop = FALSE][[col_sample]]
        message(length(keep_samples), " samples left after filtering.") # 为什么该代码块会被多次调用呢？
        if (length(keep_samples) < 1) {
          sendSweetAlert(session, title = "Error", text = "No samples left!", type = "error")
        } else {
          selected_samps$id <- isolate(keep_samples)
        }
      })
    } else {
      sendSweetAlert(session, title = "Warning", type = "warn", text = "Please select a dataset!")
    }
  })
}

ui.modules_1_tcga_04 = function(id){
    ns = NS(id)
    TIL_signatures = lapply(tcga_id_option$`Immune Infiltration`, function(x) {
        x$all
    }) %>% reshape2::melt() %>% 
        dplyr::mutate(x = paste0(value,"_",L1)) %>%
        dplyr::pull(x)
    id2type = data.frame(
        id = TIL_signatures,
        type = str_split(TIL_signatures,"_", simplify = T)[,2])
    id2type.list = split(id2type$id, id2type$type)
    main_ui = tagList(
        mol_quick_select_UI(ns("id"), "tcga", 
            c("mRNA","transcript","methylation","miRNA","protein", "cnv")),
        h4("3. Select TIL cell types"),
        virtualSelectInput(
            ns("immune_sig"), NULL,
            choices = id2type.list,
            multiple = TRUE,
            selected = id2type.list[["TIMER"]]
        ),
        h4("4. Select correlation method"),
        awesomeRadio(
            inputId = ns("cor_method"),
            label = NULL, 
            choices = c("spearman", "pearson"),
            selected = "spearman",
            inline = TRUE, checkbox = TRUE
        ),
        shinyWidgets::actionBttn(
            inputId = ns("search_bttn"),
            label = "Go!",
            style = "gradient",
            icon = icon("search"),
            color = "primary",
            block = TRUE,
            size = "sm"
        )
    )
    out_ui = tagList(
        fluidRow(
            uiOutput(ns("hm_gene_immune_cor")),
        )
    )
    side_ui = tagList(
        fluidRow(
            column(6,
                h4("1. Visualization parameters"),
                h5("(1) Colors:"),
                colourpicker::colourInput(
                    inputId = ns("color_low"), "Low", "#377DB8"),
                colourpicker::colourInput(
                    inputId = ns("color_mid"), "Middle", "white"),
                colourpicker::colourInput(
                    inputId = ns("color_high"), "High", "#E31A1C"),
            ),
            column(6,
                h4("2. Download options"),
                h5("(1) Figure:"),
                numericInput(inputId = ns("height"), label = "Height", value = 6),
                numericInput(inputId = ns("width"), label = "Width", value = 12),
                awesomeRadio(ns("device"), label = "Format", 
                    choices = c("pdf", "png"), selected = "pdf", inline = TRUE),
                downloadBttn(
                  outputId = ns("download_1"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                ),
                h5("(2) Data table:"),
                downloadBttn(
                  outputId = ns("download_2"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                )
            )
        )
    )
    fluidPage(
        wellPanel(style = "height:725px",
        box(main_ui,
            width = 5,
            solidHeader = TRUE,
            title = "Quick TCGA Analysis: Correlation in tumor samples", 
            status = "success",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Click the bottom button to execute/update the analysis."
        ),
        box(out_ui,
            width = 7,
            solidHeader = TRUE,
            title = "Analytical results:", 
            status = "success",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Pull the sidebar to adjust plot parameters or download results through the top-right widget.",
            sidebar = boxSidebar(
                        id = ns("sidebar"),
                        width = 50,
                        side_ui
            )
        )
        )
    )



}


server.modules_1_tcga_04 = function(input, output, session){
    ns = session$ns

    mol_info = callModule(mol_quick_select_Server, "id", "tcga")

    plot_func <- eventReactive(input$search_bttn, {
        id <- showNotification(h3("The task is running..."), duration = NULL, closeButton = FALSE, type = "message")
        on.exit(removeNotification(id), add = TRUE)  #reactive语句执行完毕时，运行remove命令
        p <- vis_gene_TIL_cor(
            Gene = mol_info$molecule(),
            data_type = mol_info$profile(),
            sig = input$immune_sig,
            cor_method = input$cor_method
        )
        p = p + 
            scale_fill_gradient2(low = input$color_low, mid = input$color_mid, high = input$color_high)
        return(p)
    })

    # Show waiter for plot
    w <- waiter::Waiter$new(id = ns("hm_gene_immune_cor"), html = waiter::spin_hexdots(), color = "white")
    observeEvent(input$search_bttn,{
        shinyjs::disable("search_bttn")
        # check whether valid out plot
        chect_plot = is.null(plot_func()) 
        if(chect_plot){
            sendSweetAlert(session, title = "Warning", type = "error", text = "Please select a valid molecule.")
            req(chect_plot)
        }
        output$hm_gene_immune_cor <- renderUI({
            w$show()
            output$plot = renderPlot(plot_func())
            fluidRow(
                column(12,
                    plotOutput(ns("plot"), height = "580px"),
                )
            )
        })    
        shinyjs::enable("search_bttn")
    })

    output$download_1 <- downloadHandler(
        filename = function() {
            paste0(mol_info$molecule(), "_", mol_info$profile(), "_pancan_TIL.", input$device)
        },
        content = function(file) {
            p <- plot_func()
            if (input$device == "pdf") {
                pdf(file, width = input$width, height = input$height)
                print(p)
                dev.off()
            } else {
                png(file, width = input$width, height = input$height, res = 600, units = "in")
                print(p)
                dev.off()
            }
        }
    )

    output$download_2 <- downloadHandler(
        filename = function() {
            paste0(mol_info$molecule(), "_", mol_info$profile(), "_pancan_TIL.csv")
        },
        content = function(file) {
            data = plot_func()$data %>%
                dplyr::rename('Cancer'='cancer','Cell type'='immune_cells',
                'Cor'='cor', 'P.value'='p.value') %>%
                dplyr::select(Cancer, `Cell type`, Cor, P.value) %>%
                tibble::remove_rownames()
        write.csv(data, file, row.names = FALSE)
        }
    )
}

mol_quick_select_UI = function(id, database='tcga', 
        mol_types = c("mRNA","transcript","methylation","miRNA","protein","cnv","mutation")){
    ns = NS(id)
    mol_choices = list(
        tcga = c("mRNA Expression"="mRNA", "Transcript Expression"="transcript", 
                "DNA Methylation"="methylation", "miRNA Expression"="miRNA",
                "Protein Expression"="protein", "Copy Number Variation"="cnv",
                "Mutation status"="mutation"),
        pcawg = c("mRNA Expression"="mRNA", "miRNA Expression"="miRNA",
                  "Promoter Activity"="promoter", "Gene Fusion"="fusion",
                  "APOBEC mutagenesis"="APOBEC"),
        ccle = c("mRNA Expression"="mRNA", "Protein Expression"="protein", 
                "Copy Number Variation"="cnv")
    )

    tagList(
        h4("1. Select omics type"),
        selectInput(
            inputId = ns("profile"), label = NULL,
            choices  = mol_choices[[database]][mol_choices[[database]] %in% mol_types],
            selected = "mRNA"
        ),
        h4("2. Select omics molecule"),
        awesomeRadio(
            inputId = ns("mol_type"),
            label = NULL, 
            choices = c("One molecule", "Multi-molecule formula"),
            selected = "One molecule",
            inline = TRUE, checkbox = TRUE
        ),
        tabsetPanel(
            id = ns("mol_type_tab"),
            type = "hidden",
            tabPanel("One molecule",
                virtualSelectInput(
                    inputId = ns("Pancan_search_1"),
                    label = NULL, choices = NULL,
                    width = "100%", search = TRUE,
                    allowNewOption = TRUE, dropboxWidth = "200%"
                )
            ),
            tabPanel("Multi-molecule formula",
                searchInput(
                    inputId = ns("Pancan_search_2"),
                    label = NULL,
                    value = NULL,
                    btnReset = icon("xmark"),
                    btnSearch = icon("magnifying-glass"),
                    placeholder = 'e.g. "TP53 + 1.3 * PTEN"',
                    resetValue = "TP53 + 1.3 * PTEN"
                ),
                bsTooltip(ns("Pancan_search_2"), "Please check your signature input via the most-right button", 
                         placement = "right", trigger = "hover", options = list(container = "body"))
            )
        )
    )
}



mol_quick_select_Server = function(input, output, session, database="tcga"){
    ns <- session$ns

    observeEvent(input$mol_type, {
        updateTabsetPanel(inputId = "mol_type_tab", selected = input$mol_type)
    })

    profile_choices <- reactive({
        if(database=="tcga"){
            switch(input$profile,
                mRNA = list(all = tcga_id.list[["Gene"]], default = "TP53"),
                methylation = list(all = tcga_id.list[["Gene"]], default = "TP53"),
                protein = list(all = tcga_id.list[["Protein"]], default = "P53"),
                transcript = list(all = tcga_id.list[["Transcript"]], default = "ENST00000000233"),
                miRNA = list(all = tcga_id.list[["miRNA"]], default = "hsa-miR-769-3p"),
                cnv = list(all = tcga_id.list[["Gene"]], default = "TP53"),
                mutation = list(all = tcga_id.list[["Gene"]], default = "TP53"),
                list(all = "NONE", default = "NONE")
            )
        } else if (database=="pcawg"){
            switch(input$profile,
                mRNA = list(all = pcawg_id.list[["Gene"]], default = "TP53"),
                miRNA = list(all = pcawg_id.list[["miRNA"]], default = "hsa-let-7a-2-3p"),
                promoter = list(all = pcawg_id.list[["Promoter"]], default = "prmtr.1"),
                fusion = list(all = pcawg_id.list[["Fusion"]], default = "SAMD11"),
                APOBEC = list(all = pcawg_id.list[["Muta"]], default = "A3A_or_A3B"),
                list(all = "NONE", default = "NONE")
            )
        } else if (database=="ccle"){
            switch(input$profile,
                mRNA = list(all = ccle_id.list[["Gene"]], default = "TP53"),
                protein = list(all = ccle_id.list[["Protein"]], default = "14-3-3_beta"),
                cnv = list(all = ccle_id.list[["Gene"]], default = "TP53"),
                list(all = "NONE", default = "NONE")
            )
        }
    })

    observe({
        updateVirtualSelect(
        "Pancan_search_1",
        choices = profile_choices()$all,
        selected = profile_choices()$default
        )
    })

    observeEvent(input$Pancan_search_2,{
        sendSweetAlert(session, title = "TIP", type = "info", 
            text = paste0("One molecule signature (", input$Pancan_search_2,") was detected.", "\n",
                         'Please make sure that the formula input is valid (e.g. "TP53 + 1.3 * PTEN")'))
    }, ignoreInit = TRUE)


    Pancan_search = reactive({
        if(input$mol_type=="One molecule"){
            input$Pancan_search_1
        } else {
            input$Pancan_search_2
        }
    })

    list(
        molecule=Pancan_search,
        profile = reactive(input$profile)
    )


}

#' Fetch non-omics data of all samples from relevant databases
#'
#' @param database one of "toil" (tcga), "pcawg", "ccle"
#' @param type one of "immune", "pathway", "tumor_index"
#' @param subtype For "immune" type of toil/pcawg database, one of "CIBERSORT", "CIBERSORT-ABS", "EPIC", "MCPCOUNTER", "QUANTISEQ", "TIMER", "XCELL". 
#' For "pathway" type of toil/pcawg database, one of "HALLMARK", "KEGG", "IOBR". 
#' For "tumor_index" type of toil database, one of "Tumor_Purity","Tumor_Stemness","Tumor_Mutation_Burden","Microsatellite_Instability", "Genome_Instability".
#' For "tumor_index" type of pcawg/ccle database, set NULL.
#' If subtype is NULL, return all subtypes.
#'
#' @returns dataframe
#' @export
#' 
#'
#' @examples
#' \dontrun{
#' tcga_immune_xcell = get_nonomics_value(database = "toil", type = "immune", subtype = "XCELL")
#' 
#' tcga_pathway_kegg = get_nonomics_value(database = "toil", type = "pathway", subtype = "KEGG")
#' 
#' # Combined with sample information
#' 
#' head(tcga_clinical_fine)
#' head(tcga_surv)
#' 
#' head(pcawg_info_fine)
#' 
#' head(ccle_info_fine)
#' 
#' }
#' 



get_nonomics_value <- function(database = c("toil", "pcawg", "ccle"),
                               type = c("immune", "pathway", "tumor_index"),
                               subtype = NULL){


    avai_databases = c("toil", "pcawg", "ccle")
    if (!database %in% avai_databases) {
        stop(paste0("Please choose a valid type (", paste(avai_databases, collapse = ", "), ")!"))
    }

    avai_types = c("immune", "pathway", "tumor_index")
    if (!type %in% avai_types) {
        stop(paste0("Please choose a valid type (", paste(avai_types, collapse = ", "), ")!"))
    }

    if (database == "pcawg"){
        if (type == "tumor_index"){
            if(!is.null(subtype)){
                stop("subtype should be NULL when database is pcawg and type is tumor_index")
            }
        }
    }
    if (database == "ccle"){
        if (type %in% c("immune", "pathway")){
            stop("type should not be immune or pathway when database is ccle")
        }
        if (type == "tumor_index"){
            if(!is.null(subtype)){
                stop("subtype should be NULL when database is pcawg and type is tumor_index")
            }
        }
    }

    if (!is.null(subtype)){
        if (type=="immune"){
            avai_subtypes = c("CIBERSORT", "CIBERSORT-ABS", "EPIC", "MCPCOUNTER", "QUANTISEQ", "TIMER", "XCELL")
            if (!subtype %in% avai_subtypes) {
                stop(paste0("Please choose a valid immune subtype (", paste(avai_subtypes, collapse = ", "), ")!"))
            }  
        } else if (type=="pathway"){
            avai_subtypes = c("HALLMARK", "KEGG", "IOBR")
            if (!subtype %in% avai_subtypes) {
                stop(paste0("Please choose a valid pathway subtype (", paste(avai_subtypes, collapse = ", "), ")!"))
            }
        } else if (type=="tumor_index"){
            avai_subtypes = c("Tumor_Purity","Tumor_Stemness","Tumor_Mutation_Burden","Microsatellite_Instability", "Genome_Instability")
            if (!subtype %in% avai_subtypes) {
                stop(paste0("Please choose a valid tumor index subtype (", paste(avai_subtypes, collapse = ", "), ")!"))
            }
        }
    }


    ## toil data
    if (database=="toil" & type=="immune"){
        datas = load_data("tcga_TIL") |> dplyr::rename("Sample"="cell_type")
        if (!is.null(subtype)){
            datas = datas |> 
                dplyr::select("Sample", dplyr::ends_with(subtype))
        }
    } 

    if (database=="toil" & type=="pathway"){
        datas = load_data("tcga_PW") |> 
            tibble::rownames_to_column("Sample") |> 
            tibble::tibble()
        if (!is.null(subtype)){
            datas = datas |> 
                dplyr::select("Sample", dplyr::starts_with(subtype))
        }
    }

    if (database=="toil" & type=="tumor_index"){
        datas_list = list(
            "Tumor_Purity" = load_data("tcga_purity")  |> 
                dplyr::rename("Sample" = "sample") |>  
                dplyr::select(!"cancer_type") |> 
                dplyr::rename_with(.cols = -1,.fn = ~ paste0("Tumor_Purity_", .x)),
            "Tumor_Stemness" = load_data("tcga_stemness")  |> 
                dplyr::rename("Sample" = "sample") |>
                dplyr::rename_with(.cols = -1,.fn = ~ paste0("Tumor_Stemness_", .x)),
            "Tumor_Mutation_Burden" = load_data("tcga_tmb")  |> 
                dplyr::rename("Sample" = "Tumor_Sample_ID") |>
                dplyr::select(!c("Cohort", "Patient_ID")) |> 
                dplyr::rename_with(.cols = -1,.fn = ~ paste0("Tumor_Mutation_Burden_", .x)),
            "Microsatellite_Instability" = load_data("tcga_genome_instability")  |> 
                dplyr::rename("Sample" = "sample") |> 
                dplyr::rename_with(.cols = -1,.fn = ~ paste0("Microsatellite_Instability_", .x))
        )

        if (!is.null(subtype)){
            datas = datas_list[subtype]
        } else {
            datas <- reduce(datas_list, dplyr::full_join, by = "Sample")
        }
    }


    ## pcawg data
    if (database=="pcawg" & type=="immune"){
        datas = load_data("pcawg_TIL") |> dplyr::rename("Sample"="cell_type")
        if (!is.null(subtype)){
            datas = datas |> 
                dplyr::select("Sample", dplyr::ends_with(subtype))
        }
    } 

    if (database=="pcawg" & type=="pathway"){
        datas = load_data("pcawg_PW") |> 
            tibble::rownames_to_column("Sample") |> 
            tibble::tibble()
        if (!is.null(subtype)){
            datas = datas |> 
                dplyr::select("Sample", dplyr::starts_with(subtype))
        }
    }

    if (database=="pcawg" & type=="tumor_index"){
        datas = load_data("pcawg_purity") %>%
            dplyr::rename("Sample" = "icgc_specimen_id")
    }

    ## ccle data
    if (database=="ccle" & type=="tumor_index"){
        datas = load_data("ccle_absolute") %>%
            dplyr::rename("Sample" = "Cell Line")
    }

    return(datas)                 
}

.onAttach <- function(libname, pkgname) {
  version <- utils::packageDescription(pkgname, fields = "Version")

  msg <- paste0("=========================================================================================
", pkgname, " version ", version, "
Project URL: https://github.com/openbiox/UCSCXenaShiny
Usages: https://openbiox.github.io/UCSCXenaShiny/

If you use it in published research, please cite:
  Shensuo Li, Yuzhong Peng, Minjun Chen, Yankun Zhao, Yi Xiong, Jianfeng Li, Peng Luo, 
  Haitao Wang, Fei Zhao, Qi Zhao, Yanru Cui, Sujun Chen, Jian-Guo Zhou, Shixiang Wang,  
  Facilitating integrative and personalized oncology omics analysis with UCSCXenaShiny, 
  Communications Biology, 1200 (2024),  https://doi.org/10.1038/s42003-024-06891-2
=========================================================================================
                              --Enjoy it--")
  base::packageStartupMessage(msg)
}

query_custom_feature_value <- function(id) {
  # 为简单起见，暂不支持 genomic signature 组合多个特征值特性
  message("Reading custom feature data from temp directory.")
  fData <- readRDS(file.path(tempdir(), "custom_feature_data.rds"))
  colnames(fData)[1] <- "sample"
  fData <- fData %>%
    dplyr::filter(.data$sample %in% id)
  value <- fData[, -1]
  names(value) <- colnames(fData)[-1]
  value
}

phenotype_datasets <- UCSCXenaTools::XenaData %>%
  dplyr::filter(Type == "clinicalMatrix") %>%
  dplyr::pull(XenaDatasets)

ui.page_general_analysis <- function() {
  # ns <- NS(id)
  # navbarMenu(
  #   title = "General Analysis",
  #   icon = icon("angle-double-down"),
  #   "result"
  #   # tabPanel("Single Gene Pan-cancer Analysis",
  #   #          ui.sg.pancan.analysis("sg.pancan.analysis"))
  # )
  tabPanel(
    title = "General Analysis",
    icon = icon("angle-double-down"),
    fluidRow(
      column(
        2,
        dropdownButton(
          inputId = "ga_drop_button",
          label = "Pre-selected Datasets for Analysis",
          icon = icon("sliders"),
          status = "primary",
          circle = FALSE,
          DT::dataTableOutput("ga_dataset_table")
        ),
        shinyBS::bsPopover("ga_drop_button",
          title = "Tips",
          content = "Click to show pre-selected datasets including user selected datasets from Repository page and corresponding phenotype datasets.",
          placement = "right", options = list(container = "body")
        )
      ),
      column(
        1, offset = 1,
        dropdownButton(
          inputId = "ga_drop_button_custom_data",
          label = "Upload your data?",
          tags$h3("Upload your data file"),
          br(),
          
          fileInput("ga_input_feature_file", 
                    label = "Feature-by-sample file (*.csv/.tsv/.txt), e.g., a gene expression matrix", 
                    accept = c("text/plain", ".gz")),
          tags$a(href="https://tcga-xena-hub.s3.us-east-1.amazonaws.com/download/TCGA.LAML.sampleMap%2FHiSeqV2.gz", "Click to download example feature data"),
          br(),
          
          fileInput("ga_input_phenotype_file",
                    label = "Phenotype file (*.csv/.tsv/.txt)",
                    accept = c("text/plain", ".gz")),
          tags$a(href="https://tcga-xena-hub.s3.us-east-1.amazonaws.com/download/TCGA.LAML.sampleMap%2FLAML_clinicalMatrix", "Click to download example phenotype data"),
          
          tags$h5("Note: Not both files are required. Prepare your upload files based on your analysis plan.\nBesides, make sure upload files have correct file extensions."),
          
          br(),
          icon = icon("upload"),
          circle = FALSE
        ),
        shinyBS::bsPopover("ga_drop_button_custom_data",
                           title = "Tips",
                           content = "Click to upload custom data for analysis",
                           placement = "right", options = list(container = "body")
        )
      ),
      column(4,
        offset = 4,
        shinyWidgets::actionBttn(
          inputId = "use_ga_page",
          label = "How to use",
          icon = icon("question-circle"),
          style = "bordered",
          color = "primary",
          size = "sm"
        )
      )
    ),
    tags$br(),
    # navlistPanel is an alternative
    tabsetPanel(
      tabPanel(
        "Scatter-Correlation",
        ui.modules_ga_scatter_correlation("module_ga_scatter_correlation")
      ),
      tabPanel(
        "Matrix-Correlation",
        ui.modules_ga_matrix_correlation("module_ga_matrix_correlation")
      ),
      tabPanel(
        "Group-Comparison",
        ui.modules_ga_group_comparison("module_ga_group_comparison")
      ),
      tabPanel(
        "Survival-Analysis",
        ui.modules_ga_surv_analysis("module_ga_surv_analysis")
      ),
      tabPanel(
        "Dimension-Reduction",
        ui.modules_ga_dim_distribution("module_ga_dim_distribution")
      )
    )
  )
}

ui.modules_1_tcga_05 = function(id){
    ns = NS(id)

    main_ui = tagList(
        mol_quick_select_UI(ns("id"), "tcga", 
            c("mRNA","transcript","methylation","miRNA","protein", "cnv")),

        h4("3. Select TIL cell types"),
        selectInput(
            ns("immune_sig"), NULL,
            choices = c("Yasin", "Wolf", "Attractors", "ICR", 
                         "c7atoms", "Bindea", "Cibersort"),
            selected = "Cibersort"
        ),
        h4("4. Select correlation method"),
        awesomeRadio(
            inputId = ns("cor_method"),
            label = NULL, 
            choices = c("spearman", "pearson"),
            selected = "spearman",
            inline = TRUE, checkbox = TRUE
        ),
        shinyWidgets::actionBttn(
            inputId = ns("search_bttn"),
            label = "Go!",
            style = "gradient",
            icon = icon("search"),
            color = "primary",
            block = TRUE,
            size = "sm"
        )
    )
    out_ui = tagList(
        fluidRow(
            uiOutput(ns("hm_gene_immune_cor")),
        )
    )
    side_ui = tagList(
        fluidRow(
            column(6,
                h4("1. Visualization parameters"),
                h5("(1) Colors:"),
                colourpicker::colourInput(
                    inputId = ns("color_low"), "Low", "#377DB8"),
                colourpicker::colourInput(
                    inputId = ns("color_mid"), "Middle", "white"),
                colourpicker::colourInput(
                    inputId = ns("color_high"), "High", "#E31A1C"),
            ),
            column(6,
                h4("2. Download options"),
                h5("(1) Figure:"),
                numericInput(inputId = ns("height"), label = "Height", value = 6),
                numericInput(inputId = ns("width"), label = "Width", value = 12),
                awesomeRadio(ns("device"), label = "Format", 
                    choices = c("pdf", "png"), selected = "pdf", inline = TRUE),
                downloadBttn(
                  outputId = ns("download_1"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                ),
                h5("(2) Data table:"),
                downloadBttn(
                  outputId = ns("download_2"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                )
            )
        )
    )
    fluidPage(
        wellPanel(style = "height:725px",
        box(main_ui,
            width = 5,
            solidHeader = TRUE,
            title = "Quick TCGA Analysis: Correlation in tumor samples", 
            status = "success",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Click the bottom button to execute/update the analysis."
        ),
        box(out_ui,
            width = 7,
            solidHeader = TRUE,
            title = "Analytical results:", 
            status = "success",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Pull the sidebar to adjust plot parameters or download results through the top-right widget.",
            sidebar = boxSidebar(
                        id = ns("sidebar"),
                        width = 50,
                        side_ui
            )
        )
        )
    )
}


server.modules_1_tcga_05 = function(input, output, session){
    ns = session$ns

    mol_info = callModule(mol_quick_select_Server, "id", "tcga")

    plot_func <- eventReactive(input$search_bttn, {
        id <- showNotification(h3("The task is running..."), duration = NULL, closeButton = FALSE, type = "message")
        on.exit(removeNotification(id), add = TRUE)  #reactive语句执行完毕时，运行remove命令
        p <- vis_gene_immune_cor(
            Gene = mol_info$molecule(),
            data_type = mol_info$profile(),
            Immune_sig_type = input$immune_sig,
            cor_method = input$cor_method 
        )
        p = p + 
            scale_fill_gradient2(low = input$color_low, mid = input$color_mid, high = input$color_high)
        return(p)
    })

    # Show waiter for plot
    w <- waiter::Waiter$new(id = ns("hm_gene_immune_cor"), html = waiter::spin_hexdots(), color = "white")
    observeEvent(input$search_bttn,{
        shinyjs::disable("search_bttn")
        # check whether valid out plot
        chect_plot = is.null(plot_func()) 
        if(chect_plot){
            sendSweetAlert(session, title = "Warning", type = "error", text = "Please select a valid molecule.")
            req(chect_plot)
        }
        output$hm_gene_immune_cor <- renderUI({
            w$show()
            output$plot = renderPlot(plot_func())
            fluidRow(
                column(12,
                    plotOutput(ns("plot"), height = "580px"),
                )
            )
        }) 
        shinyjs::enable("search_bttn")
    })

    output$download_1 <- downloadHandler(
        filename = function() {
            paste0(mol_info$molecule(), "_", mol_info$profile(), "_pancan_immune.", input$device)
        },
        content = function(file) {
            p <- plot_func()
            if (input$device == "pdf") {
                pdf(file, width = input$width, height = input$height)
                print(p)
                dev.off()
            } else {
                png(file, width = input$width, height = input$height, res = 600, units = "in")
                print(p)
                dev.off()
            }
        }
    )

    output$download_2 <- downloadHandler(
        filename = function() {
            paste0(mol_info$molecule(), "_", mol_info$profile(), "_pancan_immune.csv")
        },
        content = function(file) {
            data = plot_func()$data %>%
                dplyr::rename('Cancer'='cancer','Cell type'='immune_cells',
                'Cor'='cor', 'P.value'='p.value') %>%
                dplyr::select(Cancer, `Cell type`, Cor, P.value) %>%
                tibble::remove_rownames()
            write.csv(data, file, row.names = FALSE)
        }
    )
}

ui.modules_1_tcga_07 = function(id){
    ns = NS(id)

    main_ui = tagList(
        mol_quick_select_UI(ns("id"), "tcga", 
            c("mRNA","transcript","methylation","miRNA","protein", "cnv")),

        h4("3. Select one pathway"),
        virtualSelectInput(
            inputId = ns("pw_name"), 
            label = NULL,
            choices = list(
                HALLMARK = paste0("HALLMARK_",tcga_id.list[["HM"]]),
                KEGG = paste0("KEGG_",tcga_id.list[["KEGG"]]),
                IOBR = paste0("IOBR_",tcga_id.list[["IOBR"]])
            ),
            selected = "HALLMARK_ADIPOGENESIS", 
            width = "100%",
            search = TRUE,
            dropboxWidth = "200%"
        ),
        h4("3. Select TCGA cancer(s)"),
        virtualSelectInput(
            ns("Cancer"), NULL,
            choices = sort(tcga_names),
            multiple = TRUE,
            selected = "ACC"
        ),
        h4("4. Select correlation method"),
        awesomeRadio(
            inputId = ns("cor_method"),
            label = NULL, 
            choices = c("spearman", "pearson"),
            selected = "spearman",
            inline = TRUE, checkbox = TRUE
        ),
        shinyWidgets::actionBttn(
            inputId = ns("search_bttn"),
            label = "Go!",
            style = "gradient",
            icon = icon("search"),
            color = "primary",
            block = TRUE,
            size = "sm"
        )
    )
    out_ui = tagList(
        fluidRow(
            uiOutput(ns("pw_plot")),
        )
    )
    side_ui = tagList(
        fluidRow(
            column(6,
                h4("1. Visualization parameters"),
                h5("(1) Regression line:"),
                switchInput(
                    inputId = ns("use_regline"),
                    value = TRUE,
                    onLabel = "Yes",
                    offLabel = "No"
                ),
                h5("(2) Point transparent:"),
                sliderTextInput(
                    inputId = ns("alpha"),
                    label = NULL,
                    choices = seq(from = 0, to = 1, by = 0.1),
                    selected = "0.5", grid = TRUE
                ),
                h5("(3) Point color:"),
                colourpicker::colourInput(
                    inputId = ns("color"), NULL, "#000000"),
                h5("(4) ggplot theme:"),
                selectInput(inputId = ns("theme"), label = NULL, 
                            choices = names(themes_list), selected = "Cowplot"),
            ),
            column(6,
                h4("2. Download options"),
                h5("(1) Figure:"),
                numericInput(inputId = ns("height"), label = "Height", value = 6),
                numericInput(inputId = ns("width"), label = "Width", value = 6),
                awesomeRadio(ns("device"), label = "Format", 
                    choices = c("pdf", "png"), selected = "pdf", inline = TRUE),
                downloadBttn(
                  outputId = ns("download_1"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                ),
                h5("(2) Data table:"),
                downloadBttn(
                  outputId = ns("download_2"),
                  style = "gradient",
                  color = "primary",
                  block = TRUE,
                  size = "sm"
                )
            )
        )
    )
    fluidPage(
        wellPanel(style = "height:725px",
        box(main_ui,
            width = 5,
            solidHeader = TRUE,
            title = "Quick TCGA Analysis: Correlation in tumor samples",
            status = "success",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Click the bottom button to execute/update the analysis."
        ),
        box(out_ui,
            width = 7,
            solidHeader = TRUE,
            title = "Analytical results:", 
            status = "success",
            background = "gray",
            collapsible = FALSE,
            style = "height:600px",
            footer = "TIPs: Pull the sidebar to adjust plot parameters or download results through the top-right widget.",
            sidebar = boxSidebar(
                        id = ns("sidebar"),
                        width = 50,
                        side_ui
            )
        )
        )
    )



}


server.modules_1_tcga_07 = function(input, output, session){
    ns = session$ns

    mol_info = callModule(mol_quick_select_Server, "id", "tcga")


	plot_func <- eventReactive(input$search_bttn, {
        id <- showNotification(h3("The task is running..."), duration = NULL, closeButton = FALSE, type = "message")
        on.exit(removeNotification(id), add = TRUE)  #reactive语句执行完毕时，运行remove命令
        p <- vis_gene_pw_cor(
                Gene = mol_info$molecule(),
                data_type = mol_info$profile(),
                pw_name = input$pw_name,
                cancer_choose = input$Cancer,
                cor_method = input$cor_method,
                use_regline = input$use_regline,
                color = input$color,
                alpha = input$alpha)
        p <- p + themes_list[[input$theme]] +
                theme(text = element_text(size = 20),
                    legend.position = "none")
	    return(p)
	})

    # Show waiter for plot
    w <- waiter::Waiter$new(id = ns("pw_plot"), html = waiter::spin_hexdots(), color = "black")
    observeEvent(input$search_bttn,{
        shinyjs::disable("search_bttn")
        # check whether valid out plot
        chect_plot = is.null(plot_func()) 
        if(chect_plot){
            sendSweetAlert(session, title = "Warning", type = "error", text = "Please select a valid molecule.")
            req(chect_plot)
        }
        output$pw_plot <- renderUI({
            w$show()
            output$plot = renderPlot(plot_func())
            fluidRow(
                column(8, offset = 2,
                    plotOutput(ns("plot"), height = "580px"),
                )
            )
        })    
        shinyjs::enable("search_bttn")
    })

    output$download_1 <- downloadHandler(
        filename = function() {
            paste0(mol_info$molecule(), "_", mol_info$profile(), "_pancan_pathway.", input$device)
        },
        content = function(file) {
            p <- plot_func()
            if (input$device == "pdf") {
                pdf(file, width = input$width, height = input$height)
                print(p)
                dev.off()
            } else {
                png(file, width = input$width, height = input$height, res = 600, units = "in")
                print(p)
                dev.off()
            }
        }
    )

    output$download_2 <- downloadHandler(
        filename = function() {
            paste0(mol_info$molecule(), "_", mol_info$profile(), "_pancan_pathway.csv")
        },
        content = function(file) {
            data = plot_func()$data %>%
                    dplyr::select(Cancer, Sample, identifier, values, pw_name, pw_score) %>%
                    dplyr::rename("Molecule"="identifier", "Expression"="values",
                                "Pathway"="pw_name", "ssGSEA"="pw_score")
            write.csv(data, file, row.names = FALSE)
        }
    )
}

ui.link2advanced <- function() {
  navbarMenu(
    title = "Advanced analysis",
    icon = icon("question-circle"),

    tabPanel(a("Whole Functionalities", 
               href="https://shiny.zhoulab.ac.cn/UCSCXenaShiny/",
               target="_blank")),    

    tabPanel("Go to T·P·C Modules", 
              "To be deployed"),
    tabPanel("Go to T·P·C Pipelinnes", 
              "To be deployed"),
    tabPanel("Go to Pharmcogenomics Modules", 
              "To be deployed"),
    # tabPanel(a("T·P·C Pipelines", 
    #            href="https://lishensuo.github.io/UCSCXenaShiny_Book/",
    #            target="_blank")),
    # tabPanel(a("PharmacoGenomics", 
    #            href="https://lishensuo.github.io/UCSCXenaShiny_Book/",
    #            target="_blank")),
  )
}

ui.modules_pancan_sur_o2m = function(id) {
	ns = NS(id)
	fluidPage(
		fluidRow(
			# 初始设置
			column(
				3,
				wellPanel(
					style = "height:1100px",
					h2("S1: Preset", align = "center"),
					h4(strong("S1.1 Modify datasets"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Modify datasets", 
					                   content = "data_origin"),
					mol_origin_UI(ns("mol_origin2sur"), database = "toil"),

					h4(strong("S1.2 Choose cancers")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Cancer types", 
					                   content = "tcga_types"),
					pickerInput(
						ns("choose_cancers"), NULL,
						choices = sort(tcga_names),
						multiple = TRUE,
						selected = sort(tcga_names),
						options = list(`actions-box` = TRUE)
					),
				    br(),

					h4(strong("S1.3 Filter samples"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Filter samples", 
					                   content = "choose_samples"),
					h5("Quick filter:"),
					pickerInput(
						ns("filter_by_code"), NULL,
						choices = NULL, selected =  NULL,
						multiple = TRUE, options = list(`actions-box` = TRUE)
					),
					h5("Exact filter:"),
					filter_samples_UI(ns("filter_samples2sur"), database = "toil"),
					br(),
					verbatimTextOutput(ns("filter_phe_id_info")),
					br(),br(),

					h4(strong("S1.4 Upload metadata"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Upload metadata", 
					                   content = "custom_metadata"),
					shinyFeedback::useShinyFeedback(),
					custom_meta_UI(ns("custom_meta2sur")),
					br(),

					h4(strong("S1.5 Add signature"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Add signature", 
					                   content = "add_signature"),
					add_signature_UI(ns("add_signature2sur"), database = "toil"),
				)
			),
			# 选择生存资料
			column(
				4,
				wellPanel(
					style = "height:1100px",
					h2("S2: Get data", align = "center"),
					h4(strong("S2.1 Select survival endpoint")), 
				    shinyWidgets::prettyRadioButtons(
				        inputId = ns("endpoint_type"), label = NULL,
				        choiceValues = c("OS", "DSS", "DFI", "PFI"),
				        choiceNames = c("OS (Overall Survial)", "DSS (Disease-Specific Survival)", 
				        				"DFI (Disease-Free Interval)", "PFI (Progression-Free Interval)"),
				        selected = "OS"
				    ),
				    br(),br(),
					h4(strong("S2.2 Divide 2 groups by one condition")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Divide 2 groups", 
					                   content = "set_groups"),
				    group_samples_UI(ns("group_samples2sur"), database = "toil") 
				)
			),
			# 分析/可视化/下载
			column(
				5,
				wellPanel(
					style = "height:1100px",
					h2("S3: Analyze & Visualize", align = "center") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Analyze & Visualize", 
					                   content = "analyze_sur_2"),  
					h4(strong("S3.1 Set analysis parameters")), 
					selectInput(ns("sur_method"), "Survival method:",
						choices = c("Log-rank test", "Univariate Cox regression")),
				    materialSwitch(ns("use_origin"), 
				    	"Whether use initial data before grouping?"),
					h4(strong("S3.2 Set visualization parameters")), 
			      	uiOutput(ns("multi_params.ui")),

			      	dropMenu(
						actionBttn(ns("more_visu"), label = "Other options", style = "bordered",color = "success",icon = icon("bars")),		
						placement = "left",
						div(h3("Select ggplot theme:"),style="width:400px;"),
						fluidRow(
							column(6,
								selectInput(inputId = ns("theme"), label = NULL, 
											choices = names(themes_list), selected = "Minimal")
							)
						),
						div(h3(strong("Params for Log-rank test")),style="width:500px;"),
						div(h4("1. Add one vertical line:"),style="width:400px;"),
				      	fluidRow(
					  		column(6, numericInput(ns("multi_log_line"), "Add line(P):", 0.05)),
				      	),
						div(h4("2. Significance display:"),style="width:400px;"),
				      	fluidRow(
					  		column(6, selectInput(ns("multi_log_label"),"Add text:",
					  			choices = c("Signif.(symbol)", "Signif.(value)"),selected = "Signif.(symbol)"))	
				      	),
						div(h4("3. Adjust text size:"),style="width:400px;"),
						fluidRow(
							column(4, numericInput(inputId = ns("axis_size"), label = "Text size:", value = 18, step = 0.5)),
							column(4, numericInput(inputId = ns("title_size"), label = "Title size:", value = 20, step = 0.5)),
							column(4, numericInput(inputId = ns("label_size"), label = "Label size:", value = 5, step = 0.5)),
						),				
						div(h4("4. Adjust lab and title name:"),style="width:400px;"),
						fluidRow(
							column(4, textInput(inputId = ns("x_name"), label = "X-axis name:", value = "-log10(P-value)")),
							column(4, textInput(inputId = ns("title_name"), label = "Title name:",
								value = NULL))
						),

						div(h3(strong("Params for Cox regression")),style="width:500px;"),
						div(h4("1. Add one vertical line:"),style="width:400px;"),
				      	fluidRow(
					  		column(6, numericInput(ns("multi_cox_line"), "Add line(P):", 0.05)),
				      	),
						div(h4("2. Significance display:"),style="width:400px;"),
				      	fluidRow(
					  		column(6, selectInput(ns("multi_cox_label"),"Add text:",
					  			choices = c("Signif.(symbol)", "Signif.(value)", "HR value"),selected = "HR value"))	
				      	),
						div(h4("3. Adjust text size:"),style="width:400px;"),
						fluidRow(
							column(4, numericInput(inputId = ns("axis_size_2"), label = "Text size:", value = 18, step = 0.5)),
							column(4, numericInput(inputId = ns("title_size_2"), label = "Title size:", value = 20, step = 0.5)),
							column(4, numericInput(inputId = ns("label_size_2"), label = "Label size:", value = 5, step = 0.5)),
						),				
						div(h4("4. Adjust lab and title name:"),style="width:400px;"),
						fluidRow(
							column(4, textInput(inputId = ns("x_name_2"), label = "X-axis name:", value = "-log10(P-value)")),
							column(4, textInput(inputId = ns("title_name_2"), label = "Title name:",
								value = NULL))
						),
						div(h5("Note: You can download the raw data and plot in local R environment for more detailed adjustment."))
			      	),
			      	br(),
					shinyWidgets::actionBttn(
						ns("sur_analysis_bt_multi"), "Run",
				        style = "gradient",
				        icon = icon("chart-line"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					),
					br(),
					fluidRow(
						column(10, offset = 1,
							   plotOutput({ns("sur_plot_multi")}, height = "500px") 
						)
					),
					br(),
					h4(strong("S3.3 Download results")), 
				    download_res_UI(ns("download_res2sur"))
				)
			)
		)
	)
}


server.modules_pancan_sur_o2m = function(input, output, session) {
	ns <- session$ns

	# 记录选择癌症
	cancer_choose <- reactiveValues(name = "BRCA", filter_phe_id=NULL,
		phe_primary=query_tcga_group(database = "toil",cancer = "BRCA", return_all = T))
	observe({
		cancer_choose$name = input$choose_cancers
		cancer_choose$phe_primary <- query_tcga_group(database = "toil",cancer = cancer_choose$name, return_all = T)
	})

	# 自定义上传metadata数据
	custom_meta = callModule(custom_meta_Server, "custom_meta2sur",database = "toil")
	# signature
	sig_dat = callModule(add_signature_Server, "add_signature2sur",database = "toil")

	custom_meta_sig = reactive({
		if(is.null(custom_meta())){
			return(sig_dat())
		} else {
			if(is.null(sig_dat())){
				return(custom_meta())
			} else {
				custom_meta_sig = dplyr::inner_join(custom_meta(),sig_dat())
				return(custom_meta_sig)
			}
		}
	})

	# 数据源设置
	opt_pancan = callModule(mol_origin_Server, "mol_origin2sur", database = "toil")


	## 过滤样本
	# exact filter module
	filter_phe_id = callModule(filter_samples_Server, "filter_samples2sur",
					   database = "toil",
					   cancers=reactive(cancer_choose$name),
					   custom_metadata=reactive(custom_meta_sig()),
					   opt_pancan = reactive(opt_pancan()))
	# quick filter widget
	observe({
		code_types_valid = code_types[names(code_types) %in% 
							unique(cancer_choose$phe_primary$Code)]
		updatePickerInput(
			session,
			"filter_by_code",
			choices = unlist(code_types_valid,use.names = F),
			selected =  unlist(code_types_valid,use.names = F)
		)
	})
	# 综合上述二者
	observe({
		# quick filter
		choose_codes = names(code_types)[unlist(code_types) %in% input$filter_by_code]
		filter_phe_id2 = cancer_choose$phe_primary %>%
			dplyr::filter(Code %in% choose_codes) %>%
			dplyr::pull("Sample")
		# exact filter
		if(is.null(filter_phe_id())){
			cancer_choose$filter_phe_id = filter_phe_id2
		} else {
			cancer_choose$filter_phe_id = intersect(filter_phe_id2,filter_phe_id())
		}

		output$filter_phe_id_info = renderPrint({
			cat(paste0("Tip: ", length(cancer_choose$filter_phe_id), " samples are retained"))
		})
	})


	# 生存资料
	sur_dat_v1 = reactive({
		sur_dat_raw = load_data("tcga_surv") %>% dplyr::rename("Sample"="sample")
		cli_dat_raw = load_data("tcga_clinical") %>% dplyr::rename("Sample"="sample")
		sur_dat_sub = sur_dat_raw %>%
			dplyr::filter(Sample %in% cancer_choose$filter_phe_id) %>%
			dplyr::select("Sample",contains(input$endpoint_type)) %>%
			dplyr::mutate(cancer = cli_dat_raw$type[match(Sample,cli_dat_raw$Sample)],.before = 1) %>%
			na.omit()
		colnames(sur_dat_sub)[3:4] = c("status","time")
		sur_dat_sub
	})


	# 设置分组
	group_final = callModule(group_samples_Server, "group_samples2sur",
					   	   database = "toil",
						   cancers=reactive(cancer_choose$name),
						   samples=reactive(sur_dat_v1()$Sample),
						   custom_metadata=reactive(custom_meta_sig()),
						   opt_pancan = reactive(opt_pancan())
						   )

	# 合并分组与生存
	sur_res_multi = reactiveValues(sur_dat = NULL, cutoff=NULL, sur_res = NULL)	

	group_sur_final = reactive({
		dat = dplyr::inner_join(group_final(),sur_dat_v1()[,-1],by=c("Sample"="Sample"))
		## 验证是否只有一组分组的癌症
		dat = dat %>%
			dplyr::filter(Cancer %in% sort(unique(dat$Cancer))[
				apply(table(dat$Cancer,dat$group),1,function(x) {min(x)>=2})])
		dat
	})



	# 根据选项，动态更新绘图参数
	output$multi_params.ui = renderUI(
		if(input$sur_method=="Log-rank test"){
		  	fluidRow(
		  		column(3,colourpicker::colourInput(ns("multi_log_color1"), "Color (Group 1):", "#d53e4f")),
		  		column(3,colourpicker::colourInput(ns("multi_log_color2"), "Color (Group 2):", "#3288bd")),
		  	)
		} else if(input$sur_method=="Univariate Cox regression") {
		  	fluidRow(
		  		column(5,colourpicker::colourInput(ns("cox_h_g1_color"), "Color(HR>1):", "#d53e4f")),
		  		column(5,colourpicker::colourInput(ns("cox_h_l1_color"), "Color(HR<1):", "#3288bd")),
		  	)
		}
	)

	observeEvent(input$sur_analysis_bt_multi, {
		sur_res_multi$sur_dat = group_sur_final()	

		if(input$sur_method=="Log-rank test"){
			if(!input$use_origin){ #是否使用分组前的原始值
				sur_res_multi$sur_dat$Group = sur_res_multi$sur_dat$group
			} else {
				if(class(sur_res_multi$sur_dat$origin) != "character"){ #若原始值为数值型，则寻找最佳阈值
					# res.cut <- surv_cutpoint(sur_res_multi$sur_dat, time = "time", event = "status", variables = "Group")
					# res.cat <- surv_categorize(res.cut)
					sur_res_multi$sur_dat = lapply(sort(unique(sur_res_multi$sur_dat$Cancer)), function(x) {
						sur_dat_sub = subset(sur_res_multi$sur_dat, Cancer==x)
						groups_1_2 = sur_dat_sub %>% 
							  dplyr::group_by(group) %>% 
							  dplyr::summarise(mean = mean(origin)) %>% 
							  dplyr::arrange(mean) %>% 
							  dplyr::pull(group) %>% as.character()
						res.cut <- surv_cutpoint(sur_dat_sub, time = "time", event = "status", variables = "origin")
						sur_dat_sub$Group = ifelse(surv_categorize(res.cut)$origin=="low", groups_1_2[1], groups_1_2[2])
						if(min(table(sur_dat_sub$Group))==0) return(NULL)
						sur_dat_sub$Group = factor(sur_dat_sub$Group, levels=groups_1_2)
						# print(table(sur_dat_sub$Group))
						sur_dat_sub
					}) %>% do.call(rbind, .) %>% as.data.frame()
					# print(table(sur_res_multi$sur_dat$Group, sur_res_multi$sur_dat$Cancer))
				} else {
					# sur_res_multi$sur_dat$Group = sur_res_multi$sur_dat$group
				}
			}
			sur_res_multi$sur_res = lapply(sort(unique(sur_res_multi$sur_dat$Cancer)), function(x){
			  sur_dat_sub = subset(sur_res_multi$sur_dat, Cancer==x)
			  
			  surv_diff <- survdiff(Surv(time, status) ~ Group, data = sur_dat_sub)
			  pval = 1 - pchisq(surv_diff$chisq, length(surv_diff$n) - 1)
			  
			  summary(survfit(Surv(time, status) ~ Group, data = sur_dat_sub))$table %>% 
			    as.data.frame() %>% tibble::rownames_to_column("Group") %>% 
			    dplyr::mutate(Cancer = x, .before = 1) %>% 
			    dplyr::mutate(p.value = pval)
			}) %>% do.call(rbind, .) %>% as.data.frame()

		} else if (input$sur_method=="Univariate Cox regression"){
			if(!input$use_origin){
				sur_res_multi$sur_dat$Group = sur_res_multi$sur_dat$group
			} else {
				if(class(sur_res_multi$sur_dat$origin) != "character"){
					sur_res_multi$sur_dat$Group = sur_res_multi$sur_dat$origin
				} else {
					sur_res_multi$sur_dat$Group = sur_res_multi$sur_dat$group
				}
			}
			sur_res_multi$sur_res = lapply(sort(unique(sur_res_multi$sur_dat$Cancer)), function(x){
			  sur_dat_sub = subset(sur_res_multi$sur_dat, Cancer==x)
			  fit = coxph(Surv(time, status) ~ Group , data = sur_dat_sub)
			  summary(fit)$coefficients %>% as.data.frame() %>% 
			    tibble::rownames_to_column("Group") %>% 
			    dplyr::mutate(Cancer = x, .before = 1)
			}) %>% do.call(rbind, .) %>% as.data.frame()
		}
	})


	sur_plot_multi = eventReactive(input$sur_analysis_bt_multi,{
		shiny::validate(
			need(try(nrow(sur_res_multi$sur_dat)>0), 
				"Please inspect whether to set valid groups in S2 step."),
		)
		p = plot_sur_02m(
			sur_res_multi, sur_method=input$sur_method, 
			multi_log_color1=input$multi_log_color1, multi_log_color2=input$multi_log_color2, 
			x_name=input$x_name, title_name=input$title_name, multi_log_line=input$multi_log_line, 
			axis_size=input$axis_size, title_size=input$title_size, multi_log_label=input$multi_log_label,
        	label_size=input$label_size, cox_h_g1_color=input$cox_h_g1_color, cox_h_l1_color=input$cox_h_l1_color, 
			x_name_2=input$x_name_2, title_name_2=input$title_name_2, multi_cox_line=input$multi_cox_line, 
			axis_size_2=input$axis_size_2, title_size_2=input$title_size_2, 
			multi_cox_label=input$multi_cox_label, label_size_2=input$label_size_2,
			custom_theme=themes_list[[input$theme]]
		)
		p
	})

	# output$sur_plot_one = renderPlot({sur_plot_one()})
	output$sur_plot_multi = renderPlot({sur_plot_multi()})

	# Download results
	observeEvent(input$sur_analysis_bt_multi,{
		res1 = sur_plot_multi()
		res2 = sur_res_multi$sur_dat
		res3 = sur_res_multi$sur_res
		callModule(download_res_Server, "download_res2sur", res1, res2, res3)
	})
}

#' Load Dataset Provided by This Package
#'
#' Load data from builtin or Zenodo.
#' Option `xena.zenodoDir` can be used to set default path for storing
#' extra data from Zenodo, e.g., `options(xena.zenodoDir = "/home/xxx/dataset")`.
#'
#' @param name a dataset name. Could be one of
#'
#' **Builtin datasets**:
#'   - `ccle_absolute`: CCLE ABSOLUTE result.
#'   - `ccle_info`: CCLE information.
#'   - `ccle_info_fine`: cleaned CCLE information for TPC analysis.
#'   - `pcawg_info`: PCAWG information.
#'   - `pcawg_info_fine`: cleaned PCAWG information for TPC analysis.
#'   - `pcawg_purity`: PCAWG tumor purity, ploidy and WGD data.
#'   - `tcga_clinical`: TCGA clinical data.
#'   - `tcga_clinical_fine`: cleaned TCGA information for TPC analysis.
#'   - `tcga_genome_instability`: TCGA genome instability data.
#'   - `tcga_gtex`: TCGA and GTEX sample info.
#'   - `tcga_purity`: TCGA tumor purity data.
#'   - `tcga_subtypes`: TCGA subtypes data.
#'   - `tcga_surv`: TCGA survival data.
#'   - `TCGA.organ`: TCGA organ data.
#'   - `toil_info`: Toil hub information.
#'
#' **Remote datasets stored in [Zenodo](https://zenodo.org/record/12819839/)**:
#'   - `pcawg_promoter_id`: PCAWG promoter identifiers.
#'   - `transcript_identifier`: Common transcript identifiers.
#'   - `ccle_expr_and_drug_response`: CCLE expression and drug response data.
#'   - `ccle_drug_response_extend`: CCLE drug response extended data.
#'   - `pancan_MSI`: Pan-cancer MSI data.
#'   - `tcga_chr_alteration`: TCGA chromosome alteration data.
#'   - `tcga_MSI`: TCGA MSI data.
#'   - `tcga_pan_immune_signature`: TCGA pan-cancer immune signature.
#'   - `tcga_stemness`: TCGA tumor stemness data.
#'   - `tcga_TIL`: TCGA TIL data.
#'   - `tcga_PW`: ssGSEA scores of HALLMARK, KEGG, IOBR terms for TCGA samples.
#'   - `tcga_PW_meta`: metadata annotation for HALLMARK, KEGG, IOBR terms.
#'   - `tcga_tmb`: TCGA TMB data.
#'   - `tcga_armcalls`: TCGA arm alteration calls and Aneuploidy data.
#'   - `tcga_dna_repair`: TCGA DNA repair data.
#'   - `pancancer_conserved_immune_subtype`: Pan-cancer conserved immune subtypes.
#'   - `pcawg_TIL`: PCAWG TIL data.
#'   - `pcawg_PW`: ssGSEA scores of HALLMARK, KEGG, IOBR terms for PCAWG samples.
#'   - ...

#'
#' @return a dataset, typically a `data.frame`.
#' @export
#'
#' @examples
#' data1 <- load_data("tcga_surv")
#' data1
#' \donttest{
#' data2 <- load_data("tcga_armcalls")
#' data2
#' }
load_data <- function(name) {
  stopifnot(length(name) == 1)
  name2 <- paste0(name, ".rda")
  data_path <- file.path(get_zenodo_dir(), name2)

  # builtin datasets
  available_datasets <- c(
    "ccle_absolute", "ccle_info",
    "tcga_clinical", "tcga_genome_instability",
    "tcga_gtex", "tcga_purity",
    "tcga_subtypes", "tcga_surv", "TCGA.organ",
    "toil_info",
    "pcawg_info", "pcawg_purity",
    "tcga_clinical_fine",
    "pcawg_info_fine",
    "ccle_info_fine"
  )
  if (name %in% available_datasets) {
    # The data is builtin
    data(list = name, package = "UCSCXenaShiny", envir = environment())
  } else {
    if (!file.exists(data_path)) {
      # Download it to inst/extdata from zenodo
      # Then load it
      data_url <- file.path("https://zenodo.org/record/12819839/files", name2)
      if (tryCatch(httr::status_code(httr::HEAD(data_url)),
        error = function(e) 404
      ) != 200) {
        # Use an alternative OSS approach
        data_url <- file.path("https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com", name2)
      }
      message("Loading data from remote: ", data_url, ", please wait...")
      name <- FALSE
      tryCatch(
        {
          download.file(data_url, data_path)
          message("Data has been saved to ", data_path)
        },
        error = function(e) {
          message("Data load failed, please check your input and the internet.\n NULL will be returned.")
          if (file.exists(data_path)) unlink(data_path, recursive = TRUE, force = TRUE)
          name <<- TRUE
        }
      )
      if (isTRUE(name)) {
        return(invisible(NULL))
      }
    }
    tryCatch(
      load(data_path, envir = environment()),
      error = function(e) {
        message("Data load failed, probably due to broken download file, please try again.\n This time NULL will be returned.")
        if (file.exists(data_path)) unlink(data_path, recursive = TRUE, force = TRUE)
        name <<- TRUE
      }
    )
    if (isTRUE(name)) {
      return(invisible(NULL))
    }
  }

  return(get(setdiff(ls(), c("name2", "name", "data_path", "data_url", "available_datasets"))))
}

ui.combo.sg.pancan.analysis <- function(id) {
  ns <- NS(id)
  fluidPage(
    titlePanel("Combo: Single Gene Pan-Cancer Analysis"),
    sidebarLayout(
      sidebarPanel = sidebarPanel(
        ## input gene
        shinyWidgets::searchInput(
          inputId = ns("Pancan_search"),
          label = NULL,
          btnSearch = icon("search"),
          btnReset = icon("remove"),
          placeholder = "Enter a gene symbol to show its pan-can distribution, e.g. TP53",
          width = "100%"
        ),
        selectInput(
          inputId = ns("immune_sig"), "Select the immune signature source", selected = "Cibersort",
          choices = c("Yasin", "Wolf", "Attractors", "ICR", "c7atoms", "Bindea", "Cibersort")
        ),
        width = 3
      ),
      mainPanel = mainPanel(
        ## pancan distribution
        plotOutput(ns("gene_pancan_dist")),
        ## uni-cox survival analysis
        plotOutput(ns("unicox_gene_tree")),
        ## heatmap: correlation between gene and immune signatures
        plotOutput(ns("hm_gene_immune_cor")),
        width = 9
      )
    )
  )
}

server.combo.sg.pancan.analysis <- function(input, output, session) {
  observeEvent(input$Pancan_search, {
    if (nchar(input$Pancan_search) >= 1) {
      output$gene_pancan_dist <- renderPlot({
        vis_toil_TvsN(
          Gene = input$Pancan_search,
          Mode = "Boxplot",
          Method = "wilcox.test",
          Show.P.value = TRUE,
          Show.P.label = TRUE,
          include.Tumor.only = TRUE
        )
      })

      output$unicox_gene_tree <- renderPlot({
        vis_unicox_tree(
          Gene = input$Pancan_search
        )
      })

      output$hm_gene_immune_cor <- renderPlot({
        vis_gene_immune_cor(
          Gene = input$Pancan_search,
          Immune_sig_type = input$immune_sig
        )
      })
    }
  })
}

ui.modules_pancan_comp_m2o = function(id) {
	ns = NS(id)
	fluidPage(
		# 第一行：选择肿瘤及样本
		fluidRow(
			# 选择分组依据
			column(
				3,
				wellPanel(
					style = "height:1100px",
					h2("S1: Preset", align = "center"),
					h4(strong("S1.1 Modify datasets"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Modify datasets", 
					                   content = "data_origin"),
					mol_origin_UI(ns("mol_origin2comp"), database = "toil"),

					h4(strong("S1.2 Choose cancer")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Cancer types", 
					                   content = "tcga_types"),
					pickerInput(
						ns("choose_cancer"), NULL,
						choices = sort(tcga_names),
						selected = "BRCA"),
					br(),

					h4(strong("S1.3 Filter samples"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Filter samples", 
					                   content = "choose_samples"),
					h5("Quick filter:"),
					pickerInput(
						ns("filter_by_code"), NULL,
						choices = NULL, selected =  NULL,
						multiple = TRUE, options = list(`actions-box` = TRUE)
					),
					h5("Exact filter:"),
					filter_samples_UI(ns("filter_samples2comp"), database = "toil"),
					br(),
					verbatimTextOutput(ns("filter_phe_id_info")),
					br(),
					
					h4(strong("S1.4 Upload metadata"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Upload metadata", 
					                   content = "custom_metadata"),
					shinyFeedback::useShinyFeedback(),
					custom_meta_UI(ns("custom_meta2comp")),
					br(),	

					h4(strong("S1.5 Add signature"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Add signature", 
					                   content = "add_signature"),
					add_signature_UI(ns("add_signature2comp"), database = "toil"),				
				)
			),
			# 分组设置
			column(
				4,
				wellPanel(
					style = "height:1100px",
					h2("S2: Get data", align = "center"),
					h4(strong("S2.1 Divide 2 groups by one condition")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Divide 2 groups", 
					                   content = "set_groups"),
					# 调用分组模块UI
					group_samples_UI(ns("group_samples2comp"), database = "toil"),
					h4(strong("S2.2 Get batch data for comparison")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Get batch data", 
					                   content = "get_batch_data"),  
					# 批量数据下载
					multi_upload_UI(ns("multi_upload2comp"),"Query", database = "toil"),

				)
			),
			column(
				5,
				wellPanel(
					style = "height:1100px",
					h2("S3: Analyze", align = "center") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Analyze", 
					                   content = "analyze_comp_3"),  
					h4(strong("S3.1 Set analysis parameters")), 
					# br(),br(),
					# h4("1. Set method"),
					selectInput(ns("comp_method"), "Comparison method:",choices = c("wilcox.test", "t.test"),selected="wilcoxon"),
					shinyWidgets::actionBttn(
						ns("cal_batch_comp"), "Run",
				        style = "gradient",
				        icon = icon("table"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					),
					br(),br(),
					fluidRow(
						column(10, offset = 1,
							   div(uiOutput(ns("comp_stat_tb.ui")),style = "height:600px"),
							   )
					),
					h4(strong("S3.2 Download results")), 
					# uiOutput(ns("comp_stat_dw.ui"))
					download_res_UI(ns("download_res2comp"))
				)
			)
		)
	)
}


server.modules_pancan_comp_m2o = function(input, output, session) {
	ns <- session$ns


	# 记录选择癌症
	cancer_choose <- reactiveValues(database = "toil",name = "BRCA", filter_phe_id=NULL,
		phe_primary=query_tcga_group(cancer = "BRCA", return_all = T))
	observe({
		cancer_choose$name = input$choose_cancer
		cancer_choose$phe_primary <- query_tcga_group(database = "toil",cancer = cancer_choose$name, return_all = T)
	})

	# 自定义上传metadata数据
	custom_meta = callModule(custom_meta_Server, "custom_meta2comp", database = "toil")
	# signature
	sig_dat = callModule(add_signature_Server, "add_signature2comp", database = "toil")

	custom_meta_sig = reactive({
		if(is.null(custom_meta())){
			return(sig_dat())
		} else {
			if(is.null(sig_dat())){
				return(custom_meta())
			} else {
				custom_meta_sig = dplyr::inner_join(custom_meta(),sig_dat())
				return(custom_meta_sig)
			}
		}
	})

	# 数据源设置
	opt_pancan = callModule(mol_origin_Server, "mol_origin2comp", database = "toil")

	## 过滤样本
	# exact filter module
	filter_phe_id = callModule(filter_samples_Server, "filter_samples2comp",
					   database = "toil",
					   cancers=reactive(cancer_choose$name),
					   custom_metadata=reactive(custom_meta_sig()),
					   opt_pancan = reactive(opt_pancan()))
	# quick filter widget
	observe({
		code_types_valid = code_types[names(code_types) %in% 
							unique(cancer_choose$phe_primary$Code)]
		updatePickerInput(
			session,
			"filter_by_code",
			choices = unlist(code_types_valid,use.names = F),
			selected =  unlist(code_types_valid,use.names = F)
		)
	})
	# 综合上述二者
	observe({
		# quick filter
		choose_codes = names(code_types)[unlist(code_types) %in% input$filter_by_code]
		filter_phe_id2 = cancer_choose$phe_primary %>%
			dplyr::filter(Code %in% choose_codes) %>%
			dplyr::pull("Sample")

		# exact filter
		if(is.null(filter_phe_id())){
			cancer_choose$filter_phe_id = filter_phe_id2
		} else {
			cancer_choose$filter_phe_id = intersect(filter_phe_id2,filter_phe_id())
		}

		output$filter_phe_id_info = renderPrint({
			cat(paste0("Tip: ", length(cancer_choose$filter_phe_id), " samples are retained"))
		})
	})


	# 设置分组
	group_final = callModule(group_samples_Server, "group_samples2comp",
					   	   database = "toil",
						   cancers=reactive(cancer_choose$name),
						   samples=reactive(cancer_choose$filter_phe_id),
						   custom_metadata=reactive(custom_meta_sig()),
						   opt_pancan = reactive(opt_pancan())
						   )

	# 批量下载数据
	L3s_x_data =  callModule(multi_upload_Server, "multi_upload2comp", 
							 database = "toil",
							 samples=reactive(cancer_choose$filter_phe_id),
							 custom_metadata=reactive(custom_meta_sig()),
						     opt_pancan = reactive(opt_pancan()),
						     table.ui = FALSE
							 )
	L3s_x = reactive({
		unique(L3s_x_data()$id)
	})

	## 比较分析
	comp_stat = eventReactive(input$cal_batch_comp, {
		shiny::validate(
			need(try(nrow(group_final())>0), 
				"Please inspect whether to set groups in S2 step."),
		)
		shinyjs::disable("cal_batch_comp")
		withProgress(message = "Please wait for a while.",{
			comp_stat = lapply(seq(L3s_x()), function(i){
				# i = 1
			    incProgress(1 / length(L3s_x()), detail = paste0("(Finished ",i,"/",length(L3s_x()),")"))

				L3_x = L3s_x()[i]

				y_data = L3s_x_data() %>%
					dplyr::filter(id == L3_x) %>% 
					dplyr::select(Sample, value)

				group_data = group_final()[,c(1,3,4)]
				colnames(group_data) = c("Sample","group","phenotype")
				data = dplyr::inner_join(y_data, group_data) %>%
					dplyr::select(Sample, value, group, everything()) %>% na.omit()
				# 检查数据是否合理
				if(nrow(data)==0 | sd(data$value)==0) return(c(NaN, NaN, NaN))
				if(length(unique(data$group))==1 | min(table(data$group))<3) return(c(NaN, NaN, NaN))
				
				if(input$comp_method == "t.test"){
					comp_obj = t.test(value ~ group, data)
				} else if (input$comp_method == "wilcox.test"){
					comp_obj = wilcox.test(value ~ group, data)
				}
				means = data %>%
				  dplyr::group_by(group) %>% 
				  dplyr::summarise(value=mean(value)) %>% 
				  dplyr::pull(value)

				comp_res = c(means, comp_obj$p.value)
				comp_res
			}) %>% do.call(rbind, .) %>% as.data.frame()

			colnames(comp_stat) = c(levels(group_final()$group),"p.value")
			comp_stat = comp_stat %>% 
			  dplyr::mutate(id = L3s_x(), .before = 1) %>%
			  dplyr::arrange(p.value)
			comp_stat
		})
		shinyjs::enable("cal_batch_comp")
		comp_stat
	})

	output$comp_stat_tb.ui = renderUI({
		output$comp_stat_tb = renderDataTable({
			# comp_stat()
			comp_stat_ = comp_stat() %>%
				dplyr::rename("Batch identifiers"="id")
			comp_stat_$p.value = format(comp_stat_$p.value, scientific=T, digits = 3)
			dt = datatable(comp_stat_,
				# class = "nowrap row-border",
				options = list(pageLength = 10, 
					columnDefs = list(
						list(className = 'dt-center', targets="_all"),
						list(orderable=TRUE, targets = 0)))
			) %>%
				formatRound(columns = levels(group_final()$group), digits = 3)
			dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
			dt
		}) 
	dataTableOutput(ns("comp_stat_tb"))
	})

	# Download results
	observeEvent(input$cal_batch_comp,{
		res1 = NULL
		group_data = group_final()[,c(1,3,4)]
		colnames(group_data) = c("Sample","group","phenotype")
		res2 = L3s_x_data() %>%
			dplyr::inner_join(group_data) %>% na.omit()
		res3 = comp_stat()
		callModule(download_res_Server, "download_res2comp", res1, res2, res3)
	})
}

uiProfileDrugSens <- function(id){
  ns <- NS(id)
  fluidPage(
    sidebarLayout(
      # Select drug sensitivity database ----
      sidebarPanel(
        selectInput(inputId = ns("Select_database"), 
                    h4("Please select a drug sensitivity dataset:"), 
                    choices = c("GDSC1", "GDSC2", 
                                "CTRP1", "CTRP2",
                                "Prism", "gCSI")
        )
      ),
      mainPanel(
        column(6,
               radioButtons(inputId = ns("Select_profile_type"), 
                            strong("Visualization Type:"),
                            choices = list("TSNE" = "TSNE",
                                           "MAD&MEDIAN" = "MAD"), selected = "TSNE")),
        column(6,
               selectizeInput(
                 ns("select_specific_drug"), "Drug Selection:", choices = NULL,
                 options = list(
                   placeholder = 'You can highlight targeted drug',
                   onInitialize = I('function() { this.setValue(""); }'), selected = ""
                 ))
        ),
        column(12,
               plotly::plotlyOutput(ns("p_drug_sens"))
        )
      )
    ))
}

serverProfileDrugSens <- function(input, output, session){
  ns <- session$ns
  # Drugs ----
  drugs_search3 <- reactiveValues()
  observeEvent(input$Select_database, {
    drugs_search3$drugs <- switch(input$Select_database,
                                  "CTRP1" = drugs_search2[drugs_search2$type %in% "CTRP1",]$drugs,
                                  "CTRP2" = drugs_search2[drugs_search2$type %in% "CTRP2",]$drugs,
                                  "GDSC1" = drugs_search2[drugs_search2$type %in% "GDSC1",]$drugs,
                                  "GDSC2" = drugs_search2[drugs_search2$type %in% "GDSC2",]$drugs,
                                  "gCSI" = drugs_search2[drugs_search2$type %in% "gCSI",]$drugs,
                                  "Prism" = drugs_search2[drugs_search2$type %in% "Prism",]$drugs
    )
    updateSelectizeInput(session = session, inputId = 'select_specific_drug',
                         label = 'Drug Selection:', choices = drugs_search3$drugs, server = TRUE,
                         options = list(placeholder = 'You can highlight targeted drug', onInitialize = I('function() { this.setValue(""); }')),
                         selected = ""
    )
  })
  # Plot ----
  p_drug1 <- reactive({switch(input$Select_database,
                              "GDSC1" = list(plotTSNE(gdsc1_tsne, "GDSC1"), 
                                             plotMADandMedian(gdsc1_ms, "GDSC1")
                              ),
                              "GDSC2" = list(plotTSNE(gdsc2_tsne, "GDSC2"), 
                                             plotMADandMedian(gdsc2_ms, "GDSC2")
                              ), 
                              "CTRP1" = list(plotTSNE(ctrp1_tsne, "CTRP1"), 
                                             plotMADandMedian(ctrp1_ms, "CTRP1")
                              ), 
                              "CTRP2" = list(plotTSNE(ctrp2_tsne, "CTRP2"), 
                                             plotMADandMedian(ctrp2_ms, "CTRP2")
                              ),
                              "Prism" = list(plotTSNE(prism_tsne, "PRISM"), 
                                             plotMADandMedian(prism_ms, "PRISM")
                              ), 
                              "gCSI" = list(plotTSNE(gCSI_tsne, "gCSI"), 
                                            plotMADandMedian(gCSI_ms, "gCSI")
                              )
  )})
  p_drug2 <- reactive({switch(input$Select_profile_type, 
                              "TSNE" = p_drug1()[[1]],
                              "MAD" = p_drug1()[[2]])
  })
  # highlight selected drug
  p_drug3 <- reactive({
    if(input$select_specific_drug == ""){
      p_drug2()
    } else{
      p_data1 <- p_drug2()$data
      index <- grepl(input$select_specific_drug, p_data1$Name, ignore.case = T)
      for_label <- p_data1[index,]
      p_drug2() + geom_point(
        data = for_label,
        shape = 1, stroke = 1,
        size = 4, fill = "black") 
      # geom_label_repel(
      #   aes(label = Name),
      #   data = for_label,
      #   color="black")
    }
  })
  output$p_drug_sens <- plotly::renderPlotly({
    p_drug3()
  })
}

# https://shiny.rstudio.com/articles/modules.html
# Another useful link is https://shiny.rstudio.com/articles/scoping.html

library(shiny)

counterButton <- function(id, label = "Counter") {
  ns <- NS(id)
  tagList(
    actionButton(ns("button"), label = label),
    verbatimTextOutput(ns("out"))
  )
}

counter <- function(input, output, session) {
  count <- reactiveVal(0)
  observeEvent(input$button, {
    count(count() + 1)
  })
  output$out <- renderText({
    count()
  })
  count
}

ui <- fluidPage(
  counterButton("counter1", "Counter #1")
)

server <- function(input, output, session) {
  callModule(counter, "counter1")
}

shinyApp(ui, server)

#' Pipe Operator
#'
#' See \code{magrittr::\link[magrittr]{\%>\%}} for details.
#'
#' @name %>%
#' @rdname pipe
#' @keywords internal
#' @export
#' @importFrom magrittr %>%
#' @usage lhs \%>\% rhs
NULL

`%:::%` <- function(pkg, fun, inherits = TRUE) {
  get(fun,
    envir = asNamespace(pkg),
    inherits = inherits
  )
}

ui.page_pancan_tcga <- function() {
  navbarMenu(
    title = "T·P·C Pipelines",
    icon = icon("buromobelexperte"),
    tabPanel(
      "Introduction",
      fluidPage(
        includeMarkdown(set_md_path("intro_personal_pips.md"))
      )
    ),
    #### TCGA
    tabPanel(
      "TCGA: Correlation Analysis",
      h1("TCGA Correlation Analysis", align = "center"),
      tabsetPanel(
        tabPanel(
          "Sole Analysis for Single Cancer",
          br(),
          ui.modules_pancan_cor_o2o("modules_pancan_cor_o2o")
        ),
        tabPanel(
          "Sole Analysis for Multiple Cancers",
          br(),
          ui.modules_pancan_cor_o2m("modules_pancan_cor_o2m")
        ),
        tabPanel(
          # "Multiple items analysis(one cancer)",
          "Batch Analysis for Single Cancer",
          br(),
          ui.modules_pancan_cor_m2o("modules_pancan_cor_m2o")
        )
      )
    ),
    tabPanel(
      "TCGA: Comparison Analysis",
      h1("TCGA Comparison Analysis", align = "center"),
      tabsetPanel(
        tabPanel(
          "Sole Analysis for Single Cancer",
          br(),
          ui.modules_pancan_comp_o2o("modules_pancan_comp_o2o")
        ),
        tabPanel(
          "Sole Analysis for Multiple Cancers",
          br(),
          ui.modules_pancan_comp_o2m("modules_pancan_comp_o2m")
        ),
        tabPanel(
          "Batch Analysis for Single Cancer",
          br(),
          ui.modules_pancan_comp_m2o("modules_pancan_comp_m2o")
        )
      )
    ),
    tabPanel(
      "TCGA: Survival Analysis",
      h1("TCGA Survival Analysis", align = "center"),
      tabsetPanel(
        tabPanel(
          "Sole Analysis for Single Cancer",
          br(),
          ui.modules_pancan_sur_o2o("modules_pancan_sur_o2o")
        ),
        tabPanel(
          "Sole Analysis for Multiple Cancers",
          br(),
          ui.modules_pancan_sur_o2m("modules_pancan_sur_o2m")
        ),
        tabPanel(
          "Batch Analysis for Single Cancer",
          br(),
          ui.modules_pancan_sur_m2o("modules_pancan_sur_m2o")
        )
      )
    ),
    tabPanel(
      "TCGA: Cross-Omics Analysis",
      h1("TCGA Cross-Omics Analysis", align = "center"),
      tabsetPanel(
        tabPanel(
          "Gene Cross-Omics Analysis",
          br(),
          ui.modules_pancan_cross_gene_o2m("modules_pancan_cross_gene_o2m")
        ),
        tabPanel(
          "Pathway Cross-Omics Analysis",
          br(),
          ui.modules_pancan_cross_pw_o2m("modules_pancan_cross_pw_o2m")
        ),
      )
    ),

    #### PCAWG
    tabPanel(
      "PCAWG: Correlation Analysis",
      h1("PCAWG Correlation Analysis", align = "center"),
      tabsetPanel(
        tabPanel(
          "Sole Analysis for Single Cancer",
          br(),
          ui.modules_pcawg_cor_o2o("modules_pcawg_cor_o2o")
        ),
        tabPanel(
          "Sole Analysis for Multiple Cancers",
          br(),
          ui.modules_pcawg_cor_o2m("modules_pcawg_cor_o2m")
        ),
        tabPanel(
          "Batch Analysis for Single Cancer",
          br(),
          ui.modules_pcawg_cor_m2o("modules_pcawg_cor_m2o")
        )
      )
    ),
    tabPanel(
      "PCAWG: Comparison Analysis",
      h1("PCAWG Comparison Analysis", align = "center"),
      tabsetPanel(
        tabPanel(
          "Sole Analysis for Single Cancer",
          br(),
          ui.modules_pcawg_comp_o2o("modules_pcawg_comp_o2o")
        ),
        tabPanel(
          "Sole Analysis for Multiple Cancers",
          br(),
          ui.modules_pcawg_comp_o2m("modules_pcawg_comp_o2m")
        ),
        tabPanel(
          "Batch Analysis for Single Cancer",
          br(),
          ui.modules_pcawg_comp_m2o("modules_pcawg_comp_m2o")
        )
      )
    ),
    tabPanel(
      "PCAWG: Survival Analysis",
      h1("PCAWG Survival Analysis(OS)", align = "center"),
      tabsetPanel(
        tabPanel(
          "Sole Analysis for Single Cancer",
          br(),
          ui.modules_pcawg_sur_o2o("modules_pcawg_sur_o2o")
        ),
        tabPanel(
          "Sole Analysis for Multiple Cancers",
          br(),
          ui.modules_pcawg_sur_o2m("modules_pcawg_sur_o2m")
        ),
        tabPanel(
          "Batch Analysis for Single Cancer",
          br(),
          ui.modules_pcawg_sur_m2o("modules_pcawg_sur_m2o")
        )
      )
    ),
    #### CCLE
    tabPanel(
      "CCLE: Correlation Analysis",
        h1("CCLE Correlation Analysis", align = "center"),
        tabsetPanel(
          tabPanel(
            "Sole Analysis for Cell Lines",
            br(),
            ui.modules_ccle_cor_o2o("modules_ccle_cor_o2o")
          ),
          tabPanel(
            "Batch Analysis for Cell Lines",
            br(),
            ui.modules_ccle_cor_m2o("modules_ccle_cor_m2o")
          )
        ) 
    ),
    tabPanel(
      "CCLE: Comparison Analysis",
        h1("CCLE Comparison Analysis", align = "center"),
        tabsetPanel(
          tabPanel(
            "Sole Analysis for Cell Lines",
            br(),
            ui.modules_ccle_comp_o2o("modules_ccle_comp_o2o")
          ),
          tabPanel(
            "Batch Analysis for Cell Lines",
            br(),
            ui.modules_ccle_comp_m2o("modules_ccle_comp_m2o")
          )
        ) 
    )
  )
}

#' Visualize Correlation between Gene and Pathway signature Score
#' @param Gene 	a molecular identifier (e.g., "TP53") or a formula
#' specifying genomic signature ("TP53 + 2 * KRAS - 1.3 * PTEN").
#' @param data_type choose gene profile type, including
#' "mRNA", "transcript", "protein", "mutation", "cnv", "methylation", "miRNA".
#' @param pw_name the queried Pathway name, see the supported pathway from 'load("toil_sig_score")'default is NULL
#' @param cancer_choose select cancer cohort(s)
#' @param use_regline if TRUE, add regression line.
#' @param cor_method select correlation coefficient (pearson/spearman)
#' @param use_all use all sample, default FALSE.
#' @param alpha dot alpha.
#' @param color dot color.
#' @param filter_tumor whether use tumor sample only, default TRUE
#' @param opt_pancan specify one dataset for some molercular profiles

#' @return a `ggplot` object or  dataframe
#' @examples
#' \dontrun{
#' vis_gene_pw_cor(
#'   Gene = "TP53", data_type = "mRNA",
#'   pw_name = "HALLMARK_ADIPOGENESIS",
#'   cancer_choose = "BRCA"
#' )
#' }

#' @export
#'
vis_gene_pw_cor <- function(Gene = "TP53",
                            data_type = "mRNA",
                            pw_name = "HALLMARK_ADIPOGENESIS",
                            cancer_choose = "GBM",
                            use_regline = TRUE,
                            cor_method = "spearman",
                            use_all = FALSE,
                            alpha = 0.5,
                            color = "#000000",
                            filter_tumor = TRUE,
                            opt_pancan = .opt_pancan) {
  if (!file.exists(file.path(get_zenodo_dir(), "tcga_PW.rda"))) {
    print("This is the first download from zenodo, please wait a few minutes.")
  }
  toil_sig_score <- load_data("tcga_PW")
  toil_sig_meta <- load_data("tcga_PW_meta")
  if (!is.null(pw_name)) {
    if (!(pw_name %in% toil_sig_meta$ID)) {
      stop("You need provide valid pathway name (see load_data('tcga_PW_meta'))")
    }
  }
  # gene expression
  t1 <- query_pancan_value(Gene, data_type = data_type, opt_pancan = opt_pancan)
  if (is.null(t1[[1]])) {
    warning("No data available", immediate. = TRUE)
    return(NULL)
  }
  if (is.list(t1)) t1 <- t1[[1]]
  if (all(is.na(t1))) {
    message("All NAs returned, return NULL instead.")
    return(NULL)
  }
  message(paste0("Get data value for ", Gene))
  s <- data.frame(Sample = names(t1), values = t1)


  # message("Mode-3: one specific pathway correlated to one specific cancer")
  toil_sig_score <- load_data("tcga_PW")

  tcga_gtex <- load_data("tcga_gtex")
  if (use_all) {
    cancer_choose2 <- as.character(unique(tcga_gtex$tissue))
  } else {
    cancer_choose2 <- cancer_choose
  }
  if (filter_tumor) {
    filter_tumor2 <- "tumor"
  } else {
    filter_tumor2 <- c("tumor", "normal")
  }
  tcga_sp <- tcga_gtex %>%
    dplyr::filter(.data$type2 == filter_tumor2) %>%
    dplyr::filter(.data$tissue %in% cancer_choose2) %>%
    dplyr::pull(sample) %>%
    as.character()

  res_pan_spe <- toil_sig_score[rownames(toil_sig_score) %in% tcga_sp, pw_name, drop = F] %>%
    as.data.frame() %>%
    tibble::rownames_to_column("Sample") %>%
    tidyr::pivot_longer(!"Sample", names_to = "pw_name", values_to = "pw_score") %>%
    dplyr::left_join(s) %>%
    dplyr::mutate(identifier = Gene, .before = 1) %>%
    dplyr::mutate(Cancer = tcga_gtex$tissue[match(.data$Sample, tcga_gtex$sample)], .before = 3) %>%
    tibble()

  cor_res <- ezcor(data = as.data.frame(res_pan_spe), var1 = "values", var2 = "pw_score", cor_method = cor_method)
  p <- ggplot2::ggplot(res_pan_spe, aes_string(x = "values", y = "pw_score")) +
    ggplot2::geom_point(shape = 16, size = 3, show.legend = FALSE, alpha = alpha, color = color) +
    ggplot2::theme_minimal() +
    ggplot2::labs(x = paste(Gene, data_type), y = pw_name) +
    ggplot2::annotate(
      "text",
      -Inf, Inf,
      hjust = -0.1, vjust = 1,
      label = paste0("Cor: ", round(cor_res$cor, 2), " ", cor_res$pstar),
      size = 8, colour = "black"
    )

  if (use_regline) p <- p + ggplot2::geom_smooth(method = stats::lm)

  return(p)
}

#' Get Molecule or Signature Data Values from Dense (Genomic) Matrix Dataset of UCSC Xena Data Hubs
#'
#' @param dataset a UCSC Xena dataset in dense matrix format (rows are features
#' (e.g., gene, cell line) and columns are samples).
#' @param molecule a molecular identifier (e.g., "TP53") or a formula specifying
#' genomic signature (`"TP53 + 2 * KRAS - 1.3 * PTEN"`).
#' **NOTE**, when a signature is specified, a space must exist in the input.
#' @param host a UCSC Xena host, default is `NULL`, auto-detect from the dataset.
#'
#' @return a named vector.
#' @export
#'
#' @examples
#' # What does dense matrix mean?
#' table(UCSCXenaTools::XenaData$Type)
#' # It is a the UCSC Xena dataset with "Type" equals to "genomicMatrix"
#' \dontrun{
#' dataset <- "ccle/CCLE_copynumber_byGene_2013-12-03"
#' x <- query_molecule_value(dataset, "TP53")
#' head(x)
#'
#' signature <- "TP53 + 2*KRAS - 1.3*PTEN" # a space must exist in the string
#' y <- query_molecule_value(dataset, signature)
#' head(y)
#' }
query_molecule_value <- function(dataset, molecule, host = NULL) {
  has_signature <- grepl(" ", molecule)
  if (has_signature) {
    fm <- parse(text = molecule)
    ids <- all.vars(fm)
    message("Querying multiple identifiers at the same time for genomic signature...")
    message("IDs include ", paste(ids, collapse = ", "))
    tryCatch(
      {
        values <- purrr::map(ids, ~ get_data(dataset, ., host = host))
        df <- as.data.frame(values %>% purrr::set_names(ids))
        colnames(df) <- ids
        sig_values <- eval(fm, envir = df)
        names(sig_values) <- rownames(df)
        0L
      },
      error = function(e) {
        warning("Query and evaluate failed, bad IDs or formula or data values.", immediate. = TRUE)
        return(NA)
      }
    ) -> return_value
    if (is.na(return_value)) sig_values <- NA
    return(sig_values)
  } else {
    get_data(dataset, molecule, host = host)
  }
}


#' A default setting for pan-cancer studies
#' @export
.opt_pancan <- list(
  # Toil
  toil_mRNA = list(norm = "tpm"),
  toil_transcript = list(),
  toil_protein = list(),
  toil_mutation = list(),
  toil_cnv = list(gistic2 = TRUE, use_thresholded_data = TRUE),
  toil_methylation = list(type = "450K", rule_out = NULL, aggr = "NA"),
  toil_miRNA = list(),
  # PCAWG
  pcawg_mRNA = list(),
  pcawg_fusion = list(),
  pcawg_miRNA = list(norm = "TMM"),
  pcawg_promoter = list(type = "relative"),
  pcawg_APOBEC = list(),
  # CCLE
  ccle_mRNA = list(norm = "rpkm"),
  ccle_protein = list(),
  ccle_mutation = list(),
  ccle_cnv = list()
)




#' Query Single Identifier or Signature Value from Pan-cancer Database
#'
#' @param molecule a molecular identifier (e.g., "TP53") or a formula specifying
#' genomic signature (`"TP53 + 2 * KRAS - 1.3 * PTEN"`).
#' @param data_type data type. Can be one of "mRNA", "transcript", "protein",
#' "mutation", "cnv", "methylation", "miRNA".
#' @param database database, either 'toil' for TCGA TARGET GTEx, or 'ccle' for
#' CCLE.
#' @param reset_id if not `NULL`, set the specified variable at parent frame to "Signature".
#' @param opt_pancan other extra parameters passing to the underlying functions.
#' @return a list.
#' @export

#' @details
#' `query_pancan_value()` provide convenient interface to download multi-omics
#' data from 3 databases by specifying one or several canonical datasets. It is
#' derived from `query_pancan_value()` and support query for genomic signature.
#' To query comprehensive datasets that UCSCXena supports, users can check
#' `UCSCXenaTools::XenaData` and use `get_pancan_value()` directly.
#'
#'  Option `opt_pancan` is a nested list and allow to adjust the downloading details.
#'  For now, only `cnv(toil)`,`methylation(toil)`,`miRNA(toil)`,`miRNA(pcawg)`,`promoter(pcawg)`
#'  support optional parameters. The default set is `.opt_pancan` and we check meanings of sublist(parameters)
#'  through the following relationship.
#'
#' @section "toil" database:
#'  1. mRNA--`get_pancan_gene_value()`
#'  2. transcript--`get_pancan_transcript_value()`
#'  3. protein--`get_pancan_protein_value()`
#'  4. mutation--`get_pancan_mutation_status()`
#'  5. cnv--`get_pancan_cn_value()`
#'  6. methylation--`get_pancan_methylation_value()`
#'  7. miRNA--`get_pancan_miRNA_value()`
#' @section "ccle" database:
#'  1. mRNA--`get_ccle_gene_value()`
#'  2. protein--`get_ccle_protein_value()`
#'  3. mutation--`get_ccle_mutation_status()`
#'  4. cnv--`get_ccle_cn_value()`
#' @section "pcawg" database:
#'  1. mRNA--`get_pcawg_gene_value()`
#'  2. miRNA--`get_pcawg_miRNA_value()`
#'  3. promoter--`get_pcawg_promoter_value()`
#'  4. fusion--`get_pcawg_fusion_value()`
#'  5. APOBEC--`get_pcawg_APOBEC_mutagenesis_value()`
#'
#' @examples
#' \dontrun{
#' query_pancan_value("KRAS")
#' query_pancan_value("KRAS", database = "ccle")
#' query_pancan_value("KRAS", database = "pcawg")
#'
#'
#' query_pancan_value("ENSG00000000419",
#'   database = "pcawg",
#'   data_type = "fusion"
#' ) # gene symbol also work
#'
#' .opt_pancan
#'
#' opt_pancan <- list(toil_cnv = list(use_thresholded_data = FALSE))
#' query_pancan_value("PTEN", data_type = "cnv", database = "toil", opt_pancan = opt_pancan)
#'
#'
#' opt_pancan <- list(toil_methylation = list(type = "450K", rule_out = "cg21115430", aggr = "Q25"))
#' query_pancan_value("PTEN", data_type = "methylation", database = "toil", opt_pancan = opt_pancan)
#' }
query_pancan_value <- function(molecule,
                               data_type = c(
                                 "mRNA", "transcript", "protein", "mutation",
                                 "cnv", "methylation", "miRNA",
                                 "fusion", "promoter", "APOBEC"
                               ),
                               database = c("toil", "ccle", "pcawg"),
                               reset_id = NULL, opt_pancan = .opt_pancan) {
  data_type <- match.arg(data_type)
  database <- match.arg(database)

  # molecule = "TP53 + 2*KRAS - 1.3*PTEN"
  has_signature <- grepl(" ", molecule)
  if (has_signature) {
    fm <- parse(text = molecule)
    ids <- all.vars(fm)
    message("Querying multiple identifiers at the same time for genomic signature...")
    message("IDs include ", paste(ids, collapse = ", "))
    tryCatch(
      {
        values <- purrr::map(ids, function(x) {
          query_value(x, data_type, database, opt_pancan)
        })
        unit <- if (is.list(values[[1]]) && length(values[[1]]) > 1) values[[1]][[2]] else NULL
        if (is.null(unit)) {
          df <- as.data.frame(values %>% purrr::set_names(ids))
        } else {
          df <- purrr::reduce(purrr::map(values, ~ as.data.frame(.[[1]])), mbind) %>% magrittr::set_colnames(ids)
        }
        colnames(df) <- ids
        sig_values <- eval(fm, envir = df)
        names(sig_values) <- rownames(df)
        0L
      },
      error = function(e) {
        warning("Query and evaluate failed, bad IDs or formula or data values.", immediate. = TRUE)
        return(NA)
      }
    ) -> return_value
    if (is.na(return_value)) sig_values <- NA
    if (!exists("unit")) unit <- NULL

    if (!is.null(reset_id)) {
      assign(reset_id, "Signature", envir = parent.frame())
    }

    if (is.null(unit)) {
      return(sig_values)
    } else {
      return(
        list(
          value = sig_values,
          unit = unit
        )
      )
    }
  } else {
    query_value(molecule, data_type, database, opt_pancan)
  }
}


query_value <- function(identifier,
                        data_type = c(
                          "mRNA", "transcript", "protein",
                          "mutation", "cnv",
                          "methylation", "miRNA",
                          "fusion", "promoter", "APOBEC"
                        ),
                        database = c("toil", "ccle", "pcawg"),
                        opt_pancan = .opt_pancan) {
  database <- match.arg(database)
  data_type <- match.arg(data_type)

  if (database == "toil") {
    f <- switch(data_type,
      mRNA = get_pancan_gene_value,
      transcript = get_pancan_transcript_value,
      protein = get_pancan_protein_value,
      mutation = get_pancan_mutation_status,
      cnv = get_pancan_cn_value,
      methylation = get_pancan_methylation_value,
      miRNA = get_pancan_miRNA_value,
      stop("Please choose one of c('mRNA','transcript','protein','mutation','cnv','methylation','miRNA') for Toil database!")
    )
  } else if (database == "ccle") {
    f <- switch(data_type,
      mRNA = get_ccle_gene_value,
      protein = get_ccle_protein_value,
      mutation = get_ccle_mutation_status,
      cnv = get_ccle_cn_value,
      stop("Please choose one of c('mRNA','protein','mutation','cnv') for Toil database!")
    )
  } else if (database == "pcawg") {
    f <- switch(data_type,
      mRNA = get_pcawg_gene_value,
      fusion = get_pcawg_fusion_value,
      miRNA = get_pcawg_miRNA_value,
      promoter = get_pcawg_promoter_value,
      APOBEC = get_pcawg_APOBEC_mutagenesis_value,
      stop("Please choose one of c('mRNA','fusion','miRNA','promoter','APOBEC') for Toil database!")
    )
  }

  do.call(f, c(identifier, opt_pancan[[paste0(database, "_", data_type)]]))
}

#' Visualize CCLE Gene Expression
#' @import ggplot2 dplyr tibble forcats
#' @param Gene a molecular identifier (e.g., "TP53") or a formula specifying
#' genomic signature (`"TP53 + 2 * KRAS - 1.3 * PTEN"`).
#' @param data_type support genomic profile for CCLE, currently "mRNA", "protein","cnv" are supported
#' @param use_log if `TRUE`, log values.
#' @param opt_pancan specify one dataset for some molercular profiles
#' @return a `ggplot` object
#' @export
vis_ccle_tpm <- function(Gene = "TP53", data_type = "mRNA", use_log = FALSE, opt_pancan = .opt_pancan) {
  if (!requireNamespace("cowplot")) {
    install.packages("cowplot")
  }

  ccle_info <- load_data("ccle_info")

  if (!data_type %in% c("mRNA", "protein", "cnv")) {
    stop("data_type ", data_type, " does not support in this function!")
  }
  t1 <- query_pancan_value(Gene, data_type = data_type, database = "ccle", opt_pancan = opt_pancan)
  unit <- switch(data_type,
    cnv = NULL,
    mutation = NULL,
    t1[[2]]
  )
  if (is.list(t1)) t1 <- t1[[1]]

  if (all(is.na(t1))) {
    message("All NAs returned, return NULL instead.")
    return(NULL)
  }

  # if (use_log) t1 <- log2(t1 + 1)

  t2 <- t1 %>%
    as.data.frame() %>%
    dplyr::rename("tpm" = ".") %>%
    tibble::rownames_to_column(var = "cell") %>%
    dplyr::inner_join(ccle_info %>% dplyr::select(c("CCLE_name", "Cell_line_aliases", "Gender", "Site_Primary")),
      by = c("cell" = "CCLE_name")
    )

  p <- t2 %>% ggplot2::ggplot(aes_string(x = "Site_Primary", y = "tpm", fill = "Site_Primary")) +
    ggplot2::geom_boxplot() +
    ggplot2::xlab(NULL) +
    ggplot2::ylab(paste0(
      Gene, " ",
      if (data_type == "cnv") "CNV" else data_type,
      if (data_type == "cnv") "" else paste0(" (", unit, ")")
    )) +
    cowplot::theme_cowplot() +
    ggplot2::theme(
      legend.background = element_blank(),
      legend.position = "none",
      axis.text.x = element_text(angle = 45, color = "black", hjust = 1)
    )
  return(p)
}

#' Visualize CCLE Gene Expression Correlation
#' @import ggplot2 dplyr tibble forcats
#' @import ggplot2 dplyr ppcor
#' @inheritParams vis_gene_cor
#' @param cor_method correlation method
#' @param use_log_x if `TRUE`, log X values.
#' @param use_log_y if `TRUE`, log Y values.
#' @param SitePrimary select cell line origin tissue.
#' @param use_all use all sample, default `FALSE`.
#' @return a `ggplot` object
#' @export

vis_ccle_gene_cor <- function(Gene1 = "CSF1R",
                              Gene2 = "JAK3",
                              data_type1 = "mRNA",
                              data_type2 = "mRNA",
                              cor_method = "spearman",
                              use_log_x = FALSE,
                              use_log_y = FALSE,
                              use_regline = TRUE,
                              SitePrimary = "prostate",
                              use_all = FALSE,
                              alpha = 0.5, color = "#000000",
                              opt_pancan = .opt_pancan) {
  if (!requireNamespace("cowplot")) {
    install.packages("cowplot")
  }
  ccle_info <- load_data("ccle_info")

  if (!data_type1 %in% c("mRNA", "protein", "cnv")) {
    stop("data_type ", data_type1, " does not support in this function!")
  }
  if (!data_type2 %in% c("mRNA", "protein", "cnv")) {
    stop("data_type ", data_type2, " does not support in this function!")
  }

  t1 <- query_pancan_value(Gene1, data_type = data_type1, database = "ccle", opt_pancan = opt_pancan)
  unit1 <- switch(data_type1,
    cnv = NULL,
    mutation = NULL,
    t1[[2]]
  )
  if (is.list(t1)) t1 <- t1[[1]]

  if (all(is.na(t1))) {
    message("All NAs returned, return NULL instead.")
    return(NULL)
  }
  # if (use_log_x) t1 <- log2(t1 + 1)

  t2 <- t1 %>%
    as.data.frame() %>%
    dplyr::rename("tpm" = ".") %>%
    tibble::rownames_to_column(var = "cell") %>%
    dplyr::inner_join(ccle_info, by = c("cell" = "CCLE_name"))

  t3 <- query_pancan_value(Gene2, data_type = data_type2, database = "ccle", opt_pancan = opt_pancan)
  unit2 <- switch(data_type2,
    cnv = NULL,
    mutation = NULL,
    t1[[2]]
  )
  if (is.list(t3)) t3 <- t3[[1]]

  if (data_type1 == "cnv") data_type1 <- "CNV"
  if (data_type2 == "cnv") data_type2 <- "CNV"

  if (all(is.na(t3))) {
    message("All NAs returned, return NULL instead.")
    return(NULL)
  }
  # if (use_log_y) t3 <- log2(t3 + 1)

  t4 <- t3 %>%
    as.data.frame() %>%
    dplyr::rename("tpm" = ".") %>%
    tibble::rownames_to_column(var = "cell") %>%
    dplyr::inner_join(ccle_info, by = c("cell" = "CCLE_name"))

  t2 <- t2 %>% inner_join(t4[, c("cell", "tpm")], by = "cell")

  df <- data.frame(sample = t2$cell, gene1 = t2$tpm.x, gene2 = t2$tpm.y, Site_Primary = t2$Site_Primary, stringsAsFactors = F)
  if (!use_all) {
    df <- df %>% dplyr::filter(.data$Site_Primary %in% SitePrimary)
  }
  cor_res <- ezcor(data = df, var1 = "gene1", var2 = "gene2", cor_method = cor_method)

  p <- ggplot2::ggplot(df, aes_string(x = "gene1", y = "gene2")) +
    ggplot2::geom_point(shape = 16, size = 3, show.legend = FALSE, alpha = alpha, color = color) +
    ggplot2::theme_minimal(base_size = 20) +
    ggplot2::scale_color_gradient(low = "#0091ff", high = "#f0650e") +
    ggplot2::labs(x = Gene1, y = Gene2) +
    ggplot2::annotate(
      "text",
      -Inf, Inf,
      hjust = -0.1, vjust = 1,
      label = paste0("Cor: ", round(cor_res$cor, 2), " ", cor_res$pstar),
      size = 8,
      colour = "black"
    ) +
    ggplot2::labs(
      x = paste(Gene1, data_type1),
      y = paste(Gene2, data_type2)
    )

  if (use_regline) p <- p + ggplot2::geom_smooth(method = stats::lm)

  return(p)
}

#' Analyze Association between Gene (Signature) and Drug Response with CCLE Data
#'
#' Analyze partial correlation of gene-drug association after
#' controlling for tissue average expression.
#'
#' @param gene_list a gene symbol list.
#' @param combine if `TRUE`, combine the expression of gene list as
#' a gene signature.
#'
#' @return a `data.frame`
#' - If `combine` is `TRUE`, genes are combined as `signature`.
#' - `mean.diff` and `median.diff` indicate mean and median of
#' normalized expression difference between High IC50 cells and Low IC50 cells.
#' The cutoff between High and Low are median IC50.
#' @export
#'
#' @examples
#' \dontrun{
#' analyze_gene_drug_response_asso("TP53")
#' analyze_gene_drug_response_asso(c("TP53", "KRAS"))
#' analyze_gene_drug_response_asso(c("TP53", "KRAS"), combine = TRUE)
#'
#' # Visualization
#' vis_gene_drug_response_asso("TP53")
#' }
analyze_gene_drug_response_asso <- function(gene_list, combine = FALSE) {
  stopifnot(length(gene_list) > 0)
  on.exit(invisible(gc()))

  if (any(grepl(" ", gene_list))) {
    stop("Space is detected in your input, it's invalid.\nIf you want to use genomic signature feature, please input a gene list.")
  }

  ccle_data <- load_data("ccle_expr_and_drug_response")

  if (is.null(ccle_data)) {
    stop("Data load failed, try again?")
  }

  valid_genes <- sum(gene_list %in% rownames(ccle_data$expr))
  if (valid_genes > 0 && valid_genes < length(gene_list)) {
    gene_list <- gene_list[gene_list %in% rownames(ccle_data$expr)]
    warning("Some of your input genes do not exist in CCLE data.")
  } else if (valid_genes == 0) {
    stop("None of your input genes exists in CCLE data.")
  }
  expr <- ccle_data$expr[gene_list, , drop = FALSE]


  if (combine && length(gene_list) > 1) {
    expr <- t(apply(expr, 2, gm_mean))
    rownames(expr) <- "signature"
  }

  drug_ic50 <- ccle_data$drug_ic50
  drug_info <- ccle_data$drug_info
  tissues <- unique(drug_info[, c("CCLE Cell Line Name", "Site Primary")])
  expr <- expr[, tissues[["CCLE Cell Line Name"]], drop = FALSE]
  tissues <- tissues[["Site Primary"]]

  rm(ccle_data)

  drugCor <- c()
  for (i in 1:nrow(expr)) {
    gene.exp <- expr[i, ]
    tissues.mean <- stats::aggregate(gene.exp, by = list(tissues), mean)
    rownames(tissues.mean) <- tissues.mean[, 1]
    controls <- tissues.mean[tissues, 2] # control of partial correlation

    for (j in 1:ncol(drug_ic50)) {
      d.IC50 <- drug_ic50[, j]
      # Spearman correlation coefficient, significance and number of cells have IC50 and expression
      # for pair of gene i and drug j
      #
      # partial correlation -- variance-covariance (mat); Spearman's cor (s);Remove NAs(na.rm=T)
      dg.cor <- pcor_test(d.IC50, gene.exp, controls,
        use = "mat", method = "s", na.rm = TRUE
      )
      drugCor <- rbind(drugCor, c(i, j, dg.cor$estimate, dg.cor$p.value, dg.cor$n))
    }
  }

  drugCor <- as.data.frame(drugCor)
  drugCor[, 1] <- rownames(expr)[drugCor[, 1]]
  drugCor[, 2] <- colnames(drug_ic50)[drugCor[, 2]]
  names(drugCor) <- c("genes", "drugs", "cor", "p.value", "num_of_cell_lines")

  drugCor <- dplyr::left_join(drugCor,
    unique(drug_info[, c("Compound", "Target")]),
    by = c("drugs" = "Compound")
  )
  drugCor$fdr <- stats::p.adjust(drugCor$p.value, method = "fdr")

  # Normalized expression of imported gene set (normalized in the same tissue)
  CCLE_mat.sel.n <- expr
  tissues <- unique(tissues)
  for (i in tissues) {
    cells <- unique(drug_info[["CCLE Cell Line Name"]][drug_info[["Site Primary"]] %in% i])
    if (length(cells) == 1) {
      CCLE_mat.sel.n[, cells] <- 0
    } else {
      cells.n <- apply(CCLE_mat.sel.n[, cells, drop = FALSE], 1, Zscore)
      cells.n <- t(cells.n)
      CCLE_mat.sel.n[, cells] <- cells.n
    }
  }

  # Compute the expression diff between high IC50 cells and low IC50 cells for each tissue separately:
  for (i in 1:nrow(drugCor)) {
    g <- drugCor[i, 1]
    d <- drugCor[i, 2]
    d.IC50 <- drug_ic50[, d]
    z.diff <- c()
    for (j in tissues) {
      cells <- unique(drug_info[["CCLE Cell Line Name"]][drug_info[["Site Primary"]] %in% j])
      d.IC50.tissue <- d.IC50[cells]
      if (length(d.IC50.tissue) >= 5) { # at least 5 cell lines in the tissue
        d.IC50.tissue <- sort(d.IC50.tissue)
        l <- length(d.IC50.tissue)
        l <- floor(l / 2) # 按中位分的
        highIC50.cells <- head(names(d.IC50.tissue), l)
        lowIC50.cells <- tail(names(d.IC50.tissue), l)
        z.diff <- rbind(
          z.diff,
          mean(CCLE_mat.sel.n[g, highIC50.cells],
            na.rm = TRUE
          ) - mean(CCLE_mat.sel.n[g, lowIC50.cells],
            na.rm = TRUE
          )
        )
      }
    }
    drugCor[i, 8] <- mean(z.diff)
    drugCor[i, 9] <- median(z.diff)
  }
  colnames(drugCor)[8:9] <- c("mean.diff", "median.diff")


  drugCor %>%
    dplyr::arrange(.data$p.value, .data$fdr) %>%
    dplyr::mutate_if(is.numeric, ~ round(., 3))
}

#' Analyze Difference of Drug Response (IC50 Value (uM)) between Gene (Signature) High and Low Expression with CCLE Data
#'
#' @inheritParams analyze_gene_drug_response_asso
#' @param drug a drug name. Check examples.
#' @param tissue a tissue name. Check examples.
#' @param cutpoint cut point (in percent) for High and Low group, default is `c(50, 50)`.
#'
#' @return a `data.frame`.
#' @export
#'
#' @examples
#' tissue_list <- c(
#'   "prostate", "central_nervous_system", "urinary_tract", "haematopoietic_and_lymphoid_tissue",
#'   "kidney", "thyroid", "soft_tissue", "skin", "salivary_gland",
#'   "ovary", "lung", "bone", "endometrium", "pancreas", "breast",
#'   "large_intestine", "upper_aerodigestive_tract", "autonomic_ganglia",
#'   "stomach", "liver", "biliary_tract", "pleura", "oesophagus"
#' )
#'
#' drug_list <- c(
#'   "AEW541", "Nilotinib", "17-AAG", "PHA-665752", "Lapatinib",
#'   "Nutlin-3", "AZD0530", "PF2341066", "L-685458", "ZD-6474", "Panobinostat",
#'   "Sorafenib", "Irinotecan", "Topotecan", "LBW242", "PD-0325901",
#'   "PD-0332991", "Paclitaxel", "AZD6244", "PLX4720", "RAF265", "TAE684",
#'   "TKI258", "Erlotinib"
#' )
#'
#' target_list <- c(
#'   "IGF1R", "ABL", "HSP90", "c-MET", "EGFR", "MDM2", "GS", "HDAC",
#'   "RTK", "TOP1", "XIAP", "MEK", "CDK4", "TUBB1", "RAF", "ALK", "FGFR"
#' )
#' \dontrun{
#' analyze_gene_drug_response_diff("TP53")
#' analyze_gene_drug_response_diff(c("TP53", "KRAS"), drug = "AEW541")
#' analyze_gene_drug_response_diff(c("TP53", "KRAS"),
#'   tissue = "kidney",
#'   combine = TRUE
#' )
#'
#' # Visualization
#' vis_gene_drug_response_diff("TP53")
#' }
analyze_gene_drug_response_diff <- function(gene_list,
                                            drug = "ALL",
                                            tissue = "ALL",
                                            combine = FALSE,
                                            cutpoint = c(50, 50)) {
  stopifnot(length(gene_list) > 0, length(cutpoint) > 0)
  on.exit(invisible(gc()))
  # 将基因表达按阈值分为高低两组，
  # 然后比较它们的 IC50 差异
  # 这里只要得到表达高低 2 组的 IC50 值即可

  tissue <- unique(tissue)
  drug <- unique(drug)
  if (any(grepl(" ", gene_list))) {
    stop("Space is detected in your input, it's invalid.\nIf you want to use genomic signature feature, please input a gene list.")
  }

  ccle_data <- load_data("ccle_expr_and_drug_response")

  valid_genes <- sum(gene_list %in% rownames(ccle_data$expr))
  if (valid_genes > 0 && valid_genes < length(gene_list)) {
    gene_list <- gene_list[gene_list %in% rownames(ccle_data$expr)]
    warning("Some of your input genes do not exist in CCLE data.")
  } else if (valid_genes == 0) {
    stop("None of your input genes exists in CCLE data.")
  }
  expr <- ccle_data$expr[gene_list, , drop = FALSE]

  # if (any(gene_list %in% rownames(ccle_data$expr))) {
  #   expr <- ccle_data$expr[gene_list, , drop = FALSE]
  # } else {
  #   stop("None of your input genes exists in CCLE data.")
  # }

  if (combine && length(gene_list) > 1) {
    expr <- t(apply(expr, 2, gm_mean))
    rownames(expr) <- paste0("signature (", paste(gene_list, collapse = "&"), ")")
  }

  drug_ic50 <- ccle_data$drug_ic50
  drug_info <- ccle_data$drug_info
  rm(ccle_data)

  df <- expr %>%
    as.data.frame() %>%
    tibble::rownames_to_column("genes") %>%
    tidyr::pivot_longer(-"genes", names_to = "ccle_name", values_to = "expr") %>%
    dplyr::inner_join(
      unique(drug_info[, c("CCLE Cell Line Name", "Site Primary", "Compound", "Target")]),
      by = c("ccle_name" = "CCLE Cell Line Name")
    ) %>%
    dplyr::inner_join(
      drug_ic50 %>%
        as.data.frame() %>%
        tibble::rownames_to_column("ccle_name") %>%
        tidyr::pivot_longer(-"ccle_name", names_to = "drug", values_to = "IC50"),
      by = c("ccle_name" = "ccle_name", "Compound" = "drug")
    ) %>%
    dplyr::mutate(drug_target = paste(.data$Compound, "->", .data$Target)) %>%
    dplyr::select(-c("Target"))
  colnames(df)[1:6] <- c("genes", "ccle_name", "expression", "tissue", "drug", "IC50")

  if (!"ALL" %in% tissue) {
    df <- dplyr::filter(df, .data$tissue %in% .env$tissue)
  }

  if (!"ALL" %in% drug) {
    df <- dplyr::filter(df, .data$drug %in% .env$drug)
  }

  if (length(cutpoint) == 1) {
    cutpoint <- c(cutpoint, cutpoint)
  }
  cutpoint <- cutpoint / 100

  # 分组要保证在不同的组织下进行（因为不同组织表达本身可能有差异）
  # 如果用户选定多组织或全部组织（就不对组织分组）
  if ("ALL" %in% tissue || length(tissue) > 1) {
    df <- df %>%
      dplyr::group_by(.data$genes, .data$drug_target)
  } else {
    df <- df %>%
      dplyr::group_by(.data$genes, .data$drug_target, .data$tissue)
  }
  df <- df %>%
    dplyr::mutate(number_of_cell_lines = dplyr::n()) %>%
    dplyr::filter(.data$number_of_cell_lines >= 3) %>%
    # at least 3 cell lines in a tissue
    dplyr::mutate(group = dplyr::case_when(
      dplyr::percent_rank(.data$expression) > cutpoint[2] ~ "High",
      dplyr::percent_rank(.data$expression) <= cutpoint[1] ~ "Low",
      TRUE ~ NA_character_
    )) %>%
    dplyr::ungroup() %>%
    dplyr::filter(!is.na(.data$group)) %>%
    dplyr::mutate(drug_target = paste0(.data$drug_target, "\n(n = ", .data$number_of_cell_lines, ")")) %>%
    as.data.frame()

  df %>%
    dplyr::mutate_if(is.numeric, ~ round(., 3))
}

# Functions ---------------------------------------------------------------

# https://stackoverflow.com/questions/2602583/geometric-mean-is-there-a-built-in
gm_mean <- function(x, na.rm = TRUE, zero.propagate = FALSE) {
  if (any(x < 0, na.rm = TRUE)) {
    return(NaN)
  }
  if (zero.propagate) {
    if (any(x == 0, na.rm = TRUE)) {
      return(0)
    }
    exp(mean(log(x), na.rm = na.rm))
  } else {
    exp(sum(log(x[x > 0]), na.rm = na.rm) / length(x))
  }
}

# Z-score normalization
Zscore <- function(x) {
  y <- (x - mean(x)) / stats::sd(x)
  return(y)
}


# Source link: https://github.com/Katan5555/Macau_project_1/blob/master/FUNCTIONS/partial_correlation_functions.R
# LICENCE: GPLv3
pcor_test <- function(x, y, z, use = "mat", method = "p", na.rm = TRUE) {
  # The partial correlation coefficient between x and y given z
  #
  # pcor.test is free and comes with ABSOLUTELY NO WARRANTY.
  #
  # x and y should be vectors
  #
  # z can be either a vector or a matrix
  #
  # use: There are two methods to calculate the partial correlation coefficient.
  # 	 One is by using variance-covariance matrix ("mat") and the other is by using recursive formula ("rec").
  # 	 Default is "mat".
  #
  # method: There are three ways to calculate the correlation coefficient,
  # 	    which are Pearson's ("p"), Spearman's ("s"), and Kendall's ("k") methods.
  # 	    The last two methods which are Spearman's and Kendall's coefficient are based on the non-parametric analysis.
  # 	    Default is "p".
  #
  # na.rm: If na.rm is T, then all the missing samples are deleted from the whole dataset, which is (x,y,z).
  #        If not, the missing samples will be removed just when the correlation coefficient is calculated.
  # 	   However, the number of samples for the p-value is the number of samples after removing
  # 	   all the missing samples from the whole dataset.
  # 	   Default is "T".

  x <- c(x)
  y <- c(y)
  z <- as.data.frame(z)

  if (use == "mat") {
    p.use <- "Var-Cov matrix"
    pcor <- pcor.mat(x, y, z, method = method, na.rm = na.rm)
  } else if (use == "rec") {
    p.use <- "Recursive formula"
    pcor <- pcor.rec(x, y, z, method = method, na.rm = na.rm)
  } else {
    stop("\'use\' should be either \"rec\" or \"mat\"!\n")
  }

  # print the method
  if (gregexpr("p", method)[[1]][1] == 1) {
    p.method <- "Pearson"
  } else if (gregexpr("s", method)[[1]][1] == 1) {
    p.method <- "Spearman"
  } else if (gregexpr("k", method)[[1]][1] == 1) {
    p.method <- "Kendall"
  } else {
    stop("\'method\' should be \"pearson\" or \"spearman\" or \"kendall\"!\n")
  }

  # sample number
  n <- dim(stats::na.omit(data.frame(x, y, z)))[1]

  # given variables' number
  gn <- dim(z)[2]

  # p-value
  if (p.method == "Kendall") {
    statistic <- pcor / sqrt(2 * (2 * (n - gn) + 5) / (9 * (n - gn) * (n - 1 - gn)))
    p.value <- 2 * stats::pnorm(-abs(statistic))
  } else {
    statistic <- pcor * sqrt((n - 2 - gn) / (1 - pcor^2))
    p.value <- 2 * stats::pnorm(-abs(statistic))
  }

  data.frame(estimate = pcor, p.value = p.value, statistic = statistic, n = n, gn = gn, Method = p.method, Use = p.use)
}

# By using var-cov matrix
pcor.mat <- function(x, y, z, method = "p", na.rm = T) {
  x <- c(x)
  y <- c(y)
  z <- as.data.frame(z)

  if (dim(z)[2] == 0) {
    stop("There should be given data\n")
  }

  data <- data.frame(x, y, z)

  if (na.rm == T) {
    data <- stats::na.omit(data)
  }

  xdata <- stats::na.omit(data.frame(data[, c(1, 2)]))
  Sxx <- stats::cov(xdata, xdata, m = method)

  xzdata <- stats::na.omit(data)
  xdata <- data.frame(xzdata[, c(1, 2)])
  zdata <- data.frame(xzdata[, -c(1, 2)])
  Sxz <- stats::cov(xdata, zdata, m = method)

  zdata <- stats::na.omit(data.frame(data[, -c(1, 2)]))
  Szz <- stats::cov(zdata, zdata, m = method)

  # is Szz positive definite?
  zz.ev <- eigen(Szz)$values
  if (min(zz.ev)[1] < 0) {
    stop("\'Szz\' is not positive definite!\n")
  }

  # partial correlation
  Sxx.z <- Sxx - Sxz %*% solve(Szz) %*% t(Sxz)

  rxx.z <- stats::cov2cor(Sxx.z)[1, 2]

  rxx.z
}

# By using recursive formula
pcor.rec <- function(x, y, z, method = "p", na.rm = T) {
  #

  x <- c(x)
  y <- c(y)
  z <- as.data.frame(z)

  if (dim(z)[2] == 0) {
    stop("There should be given data\n")
  }

  data <- data.frame(x, y, z)

  if (na.rm == T) {
    data <- stats::na.omit(data)
  }

  # recursive formula
  if (dim(z)[2] == 1) {
    tdata <- stats::na.omit(data.frame(data[, 1], data[, 2]))
    rxy <- stats::cor(tdata[, 1], tdata[, 2], m = method)

    tdata <- stats::na.omit(data.frame(data[, 1], data[, -c(1, 2)]))
    rxz <- stats::cor(tdata[, 1], tdata[, 2], m = method)

    tdata <- stats::na.omit(data.frame(data[, 2], data[, -c(1, 2)]))
    ryz <- stats::cor(tdata[, 1], tdata[, 2], m = method)

    rxy.z <- (rxy - rxz * ryz) / (sqrt(1 - rxz^2) * sqrt(1 - ryz^2))

    return(rxy.z)
  } else {
    x <- c(data[, 1])
    y <- c(data[, 2])
    z0 <- c(data[, 3])
    zc <- as.data.frame(data[, -c(1, 2, 3)])

    rxy.zc <- pcor.rec(x, y, zc, method = method, na.rm = na.rm)
    rxz0.zc <- pcor.rec(x, z0, zc, method = method, na.rm = na.rm)
    ryz0.zc <- pcor.rec(y, z0, zc, method = method, na.rm = na.rm)

    rxy.z <- (rxy.zc - rxz0.zc * ryz0.zc) / (sqrt(1 - rxz0.zc^2) * sqrt(1 - ryz0.zc^2))
    return(rxy.z)
  }
}

#' Keep Only Columns Used for Sample Selection
#'
#' @param x a `data.frame` with many columns.
#' @param keep_sam_cols if `TRUE` (default), keep columns with
#' pattern 'sample', 'patient', etc.
#' @param return_idx if `TRUE` (default), return index of 5 (at most) columns, it is useful in Shiny.
#'
#' @return a `data.frame` or a `list`.
#' @export
keep_cat_cols <- function(x, keep_sam_cols = TRUE, return_idx = TRUE) {
  stopifnot(is.data.frame(x))
  x <- as.data.frame(x)
  samp_pass <- grepl("sample|patient|donor|specimen|submitter", colnames(x), ignore.case = TRUE)
  pass_col <- function(x) {
    if (length(x) < 100) {
      length(unique(x)) < length(x) / 1.5
    } else {
      length(unique(x)) < length(x) / 5
    }
  }
  cols_pass <- sapply(x, pass_col)

  if (keep_sam_cols) {
    x <- x[, samp_pass | cols_pass, drop = FALSE]
  } else {
    x <- x[, cols_pass, drop = FALSE]
  }

  if (return_idx) {
    samp_pass <- grepl("sample|patient|donor|specimen|submitter", colnames(x), ignore.case = TRUE)
    s <- if (any(samp_pass)) which(samp_pass)[1] else c()
    nn <- sum(!samp_pass)
    s <- c(s, sample(which(!samp_pass), min(5, nn) - length(s)))
    return(list(data = x, idx = s))
  } else {
    x
  }
}

get_run_mode <- function() {
  getOption("xena.runMode", default = "client")
}

get_cache_dir <- function() {
  path <- getOption("xena.cacheDir", default = file.path(tempdir(), "UCSCXenaShiny"))
  # message("Path for storing data is ", path, ". You can set it with options(xena.cacheDir = xxx)")
  path
}

get_zenodo_dir <- function() {
  path <- getOption("xena.zenodoDir", default = system.file("extdata", package = "UCSCXenaShiny"))
  # message("Path for storing zenodo extra data is ", path, ". You can set it with options(xena.zenodoDir = xxx)")
  path
}

rm_cache_dir <- function() {
  unlink(get_cache_dir(), recursive = TRUE)
}

# https://stackoverflow.com/questions/6029743/merge-or-combine-by-rownames
mbind <- function(...) {
  Reduce(function(x, y) {
    cbind(x, y[match(row.names(x), row.names(y)), ])
  }, list(...))
}

#' Visualize cross-omics of one gene among pan-cancers
#'
#' @param gene  a gene symbol identifier (e.g., "TP53")
#' @param tumor_samples Select specific tumor samples. Default NULL, indicating all tumor samples.
#' @param tumor_projects Select specific TCGA projects. Default NULL, indicating all TCGA projects.
#' @param n_trans The number of sampling transcripts or specific transcript identifiers.
#' @param n_methy The number of sampling CpG sites or specific CpG identifiers.
#' @param seed The seed of sampling.
#' @param add_mean_trans Add overall column to display the mean values of all gene's transcripts.
#' @param add_mean_methy Add overall column to display the mean values of all gene's cpg sites.
#' @param pval_mrna The P value thresholds
#' @param return_list TRUE returns a list including plot object and data. FALSE just returns plot.
#'
#' @return funkyheatmap
#' @export
#'
vis_gene_cross_omics <- function(gene = "TP53",
                                 tumor_projects = NULL,
                                 tumor_samples = NULL,
                                 n_trans = 5, n_methy = 5, seed = 42,
                                 add_mean_trans = TRUE, add_mean_methy = TRUE, 
                                 pval_mrna = c(0.05, 0.01, 0.001),
                                 return_list = FALSE) {
  # # If input is gene signature, cannot stat and plot transcript&methylation data
  # is_signature <- grepl(" ", molecule)

  tcga_gtex <- load_data("tcga_gtex")
  tcga_clinical_fine <- load_data("tcga_clinical_fine")
  if (is.null(tumor_projects)) {
    tumor_projects <- as.character(sort(unique(tcga_gtex$tissue)))
  } else {
    tumor_projects <- tumor_projects[tumor_projects %in% unique(tcga_gtex$tissue)]
  }

  if (is.null(tumor_samples)) {
    tcga_gtex2 <- tcga_gtex
    tcga_clinical_fine2 <- tcga_clinical_fine
  } else {
    tcga_gtex2 <- tcga_gtex %>%
      dplyr::filter(.data$type2 == "normal" | .data$sample %in% tumor_samples)
    tcga_clinical_fine2 <- tcga_clinical_fine %>%
      dplyr::filter(.data$Code == "NT" | .data$Sample %in% tumor_samples)
  }

  tcga_gtex2 <- tcga_gtex2 %>% dplyr::filter(.data$tissue %in% tumor_projects)
  tcga_clinical_fine2 <- tcga_clinical_fine2 %>% dplyr::filter(.data$Cancer %in% tumor_projects)

  #### Omics--mRNA
  gene_mrna <- query_pancan_value(gene, "mRNA") %>%
    # .[["expression"]] %>%
    .[[1]] %>% # For single gene, the first name is 'expression'; For gene signature, the first name is 'value'
    as.data.frame() %>%
    dplyr::rename("Exp" = ".") %>%
    tibble::rownames_to_column("sample") %>%
    dplyr::inner_join(tcga_gtex2)
  if (all(is.na(gene_mrna$Exp))) stop("Please input a valid gne!")

  # median value, normal samples include GTEx
  gene_mrna_md <- gene_mrna %>%
    dplyr::group_by(.data$tissue, .data$type2) %>%
    dplyr::summarise(Exp = median(.data$Exp)) %>%
    as.data.frame() %>%
    dplyr::mutate(Exp = (.data$Exp - min(.data$Exp)) / (max(.data$Exp) - min(.data$Exp))) %>% # 0~1 scale
    tidyr::pivot_wider(id_cols = .data$tissue, names_from = .data$type2, values_from = .data$Exp)

  gene_mrna_comp <- ggpubr::compare_means(Exp ~ type2,
    data = gene_mrna,
    group.by = "tissue", method = "wilcox.test"
  )

  gene_mrna_plot <- dplyr::left_join(gene_mrna_md, gene_mrna_comp[, c("tissue", "p", "p.adj")]) %>%
    as.data.frame() %>%
    dplyr::rename("id" = "tissue") %>%
    dplyr::mutate(Label = case_when(
      p < pval_mrna[3] ~ "***",
      p < pval_mrna[2] ~ "**",
      p < pval_mrna[1] ~ "*",
      TRUE ~ "-"
    )) %>%
    dplyr::select("id", "normal", "tumor", "Label")

  #### Omics--mutation
  gene_mut <- query_pancan_value(gene, "mutation") %>%
    as.data.frame() %>%
    dplyr::rename("Mut" = ".") %>%
    tibble::rownames_to_column("sample") %>%
    dplyr::inner_join(tcga_clinical_fine2, by = c("sample" = "Sample"))

  gene_mut_plot <- gene_mut %>%
    dplyr::group_by(.data$Cancer) %>%
    dplyr::summarise(Mut = mean(.data$Mut)) %>%
    as.data.frame() %>%
    dplyr::mutate(Ratio = NA)

  for (i in seq(nrow(gene_mut_plot))) {
    # i = 1
    gene_mut_plot$Ratio[i] <- list(c(Mut = gene_mut_plot$Mut[i], Wild = 1 - gene_mut_plot$Mut[i]))
  }

  gene_mut_plot <- gene_mut_plot %>%
    dplyr::mutate(Mut_str = scales::percent(.data$Mut, accuracy = 0.1)) %>%
    dplyr::select("Cancer", "Ratio", "Mut", "Mut_str")

  #### Omics--CNV
  opt_pancan <- .opt_pancan
  opt_pancan$toil_cnv$use_thresholded_data <- T
  gene_cnv <- query_pancan_value(
    gene, "cnv",
    opt_pancan = opt_pancan
  ) %>%
    as.data.frame() %>%
    # dplyr::select("data") %>%
    # dplyr::rename("CNV" = "data") %>%
    dplyr::select(1) %>% 
    dplyr::rename(CNV = 1) %>% 
    tibble::rownames_to_column("sample") %>%
    dplyr::inner_join(tcga_clinical_fine2, by = c("sample" = "Sample"))

  gene_cnv_tb <- table(gene_cnv$Cancer, gene_cnv$CNV)
  gene_cnv_plot <- gene_cnv %>%
    dplyr::count(.data$Cancer, name = "CNV_total") %>%
    dplyr::mutate(CNV_Pie = NA, CNV_Del = NA, CNV_Amp = NA)

  for (i in seq(nrow(gene_cnv_plot))) {
    # i = 1
    Cancer <- gene_cnv_plot$Cancer[i]
    gene_cnv_plot$CNV_Pie[i] <- list(c(
      " -2" = gene_cnv_tb[Cancer, "-2"],
      " -1" = gene_cnv_tb[Cancer, "-1"],
      " 0" = gene_cnv_tb[Cancer, "0"],
      " 1" = gene_cnv_tb[Cancer, "1"],
      " 2" = gene_cnv_tb[Cancer, "2"]
    ))
    gene_cnv_plot$CNV_Del[i] <- gene_cnv_tb[Cancer, "-2"] + gene_cnv_tb[Cancer, "-1"]
    gene_cnv_plot$CNV_Amp[i] <- gene_cnv_tb[Cancer, "2"] + gene_cnv_tb[Cancer, "1"]
  }

  gene_cnv_plot <- gene_cnv_plot %>%
    dplyr::mutate(CNV_Amp = .data$CNV_Amp / .data$CNV_total) %>%
    dplyr::mutate(CNV_Amp_str = scales::percent(.data$CNV_Amp, accuracy = 0.1)) %>%
    dplyr::mutate(CNV_Del = .data$CNV_Del / .data$CNV_total) %>%
    dplyr::mutate(CNV_Del_str = scales::percent(.data$CNV_Del, accuracy = 0.1)) %>%
    dplyr::select("Cancer", "CNV_Pie", "CNV_Amp", "CNV_Amp_str", "CNV_Del", "CNV_Del_str")

  #### Omics--Transcript
  Trans_sub <- load_data("v2_tpc_id_help")$tcga$id_trans %>%
    dplyr::filter(.data$Symbol == gene) %>%
    dplyr::pull(.data$Level3)
  set.seed(seed)
  if (inherits(n_trans, "character")) {
    if (length(n_trans) > 15) stop("Less than 15 transcripts are supported!")
    trans_sle <- n_trans[n_trans %in% Trans_sub]
  } else if (inherits(n_trans, "numeric")) {
    if (n_trans > 15) stop("Less than 15 transcripts are supported!")
    trans_sle <- unique(sample(Trans_sub, n_trans, replace = n_trans > length(Trans_sub)))
  } else if (is.null(n_trans)) {
    trans_sle = NULL
  }

  if (length(trans_sle) == 0 & !add_mean_trans) {
    gene_trans_n <- 0
  } else {
    if (add_mean_trans) {trans_sle = c(trans_sle, gene)}
    gene_trans <- lapply(seq(trans_sle), function(i) {
      # i = 1
      gene_trans_tmp <- query_pancan_value(
        trans_sle[i], "transcript"
      ) %>%
        as.data.frame() %>%
        dplyr::select("expression") %>%
        dplyr::rename("Trans" = "expression") %>%
        tibble::rownames_to_column("sample") %>%
        dplyr::inner_join(tcga_gtex2) %>%
        dplyr::filter(.data$type2 == "tumor") %>%
        dplyr::group_by(.data$tissue) %>%
        dplyr::summarise(Trans = median(.data$Trans))
      colnames(gene_trans_tmp)[2] <- trans_sle[i]
      gene_trans_tmp %>% tibble::column_to_rownames("tissue")
    }) %>% do.call(cbind, .)
    gene_trans_plot <- gene_trans[, apply(gene_trans, 2, stats::sd) != 0, drop = FALSE] %>%
      as.data.frame() %>%
      tibble::rownames_to_column("tissue")
    if (gene %in% colnames(gene_trans_plot)){
      colnames(gene_trans_plot)[which(colnames(gene_trans_plot) %in% gene)] = "ENST_Overall"
    }
    gene_trans_n <- ncol(gene_trans_plot) - 1
    
  }

  #### Omics--Methylation
  Methy450_sub <- load_data("v2_tpc_id_help")$tcga$id_M450 %>%
    dplyr::filter(.data$Level3 == gene) %>%
    dplyr::pull(.data$CpG)
  set.seed(seed)
  if (inherits(n_methy, "character")) {
    if (length(n_methy) > 15) stop("Less than 15 CpG sites are supported!")
    cpg_sites <- n_methy[n_methy %in% Methy450_sub]
  } else if (inherits(n_methy, "numeric")) {
    if (n_methy > 15) stop("Less than 15 CpG sites are supported!")
    cpg_sites <- unique(sample(Methy450_sub, n_methy, replace = n_methy > length(Methy450_sub)))
  } else if (is.null(n_trans)) {
    cpg_sites = NULL
  }
  if (length(cpg_sites) == 0 & !add_mean_methy) {
    gene_methy_n <- 0
  } else {
    if (add_mean_methy) {cpg_sites = c(cpg_sites, gene)}
    gene_methy_cpgs <- lapply(seq(cpg_sites), function(i) {
      # i = 1
      print(i)
      opt_pancan <- .opt_pancan
      if (cpg_sites[i] == gene){
        # Gene level
        opt_pancan$toil_methylation$aggr <- "NA"
        opt_pancan$toil_methylation$rule_out <- NULL
      } else {
        # CpG site level
        opt_pancan$toil_methylation$aggr <- "mean"
        opt_pancan$toil_methylation$rule_out <- setdiff(Methy450_sub, cpg_sites[i])
      }
      gene_methy_tmp <- query_pancan_value(
        gene, "methylation",
        opt_pancan = opt_pancan
      ) %>%
        as.data.frame() %>%
        dplyr::select("data") %>%
        dplyr::rename("Methy" = "data") %>%
        tibble::rownames_to_column("sample") %>%
        # dplyr::inner_join(tcga_gtex2) %>%
        dplyr::inner_join(tcga_clinical_fine2, by = c("sample" = "Sample")) %>%
        dplyr::filter(.data$Code != "NT") %>%
        dplyr::group_by(.data$Cancer) %>%
        dplyr::summarise(Methy = median(.data$Methy))
      colnames(gene_methy_tmp)[2] <- cpg_sites[i]
      gene_methy_tmp %>% tibble::column_to_rownames("Cancer")
    }) %>%
      do.call(cbind, .) %>%
      tibble::rownames_to_column("Cancer")
    gene_methy_plot <- gene_methy_cpgs
    if (gene %in% colnames(gene_methy_plot)){
      colnames(gene_methy_plot)[which(colnames(gene_methy_plot) %in% gene)] = "cg_Overall"
    }
    gene_methy_n <- ncol(gene_methy_plot) - 1
  }

  #### Ready for Plot
  gene_cross_dat <- data.frame(id = tumor_projects) %>%
    dplyr::left_join(gene_mrna_plot) %>%
    dplyr::left_join(gene_mut_plot, by = c("id" = "Cancer")) %>%
    dplyr::left_join(gene_cnv_plot, by = c("id" = "Cancer"))

  idx_1 <- which(sapply(gene_cross_dat$Ratio, is.null))
  for (i in idx_1) {
    gene_cross_dat$Ratio[[i]] <- c(Mut = NA, Wild = NA)
  }
  idx_2 <- which(sapply(gene_cross_dat$CNV_Pie, is.null))
  for (i in idx_2) {
    gene_cross_dat$CNV_Pie[[i]] <- c(` -2` = NA, ` -1` = NA, ` 0` = NA, ` 1` = NA, ` 2` = NA)
  }

  if (gene_trans_n > 0) {
    gene_cross_dat <- dplyr::left_join(gene_cross_dat, gene_trans_plot, by = c("id" = "tissue"))
  }
  if (gene_methy_n > 0) {
    gene_cross_dat <- dplyr::left_join(gene_cross_dat, gene_methy_plot, by = c("id" = "Cancer"))
  }

  label_name <- colnames(gene_cross_dat)
  label_name[1:12] <- c(
    "", "Normal Exp", "Tumor Exp", "T vs. N(Wilcox)",
    "Mutation Dist", "Mutation PCT", "",
    "CNV Dist", "CNV Amp", "", "CNV Del", ""
  )
  column_info <- data.frame(
    # columns to vis
    id = colnames(gene_cross_dat),
    # column name
    name = label_name,
    # group name
    group = c(
      "TCGA", rep("Expression Profile", 3), rep("Mutation Profile", 3), rep("CNV Profile", 5),
      rep("Transcript Profile", gene_trans_n), rep("Methylation Profile", gene_methy_n)
    ),
    # geom type
    geom = c(
      "text", "bar", "bar", "text", "pie", "bar", "text",
      "pie", "bar", "text", "bar", "text",
      rep("funkyrect", gene_trans_n), rep("funkyrect", gene_methy_n)
    ),
    # palette type
    palette = c(
      NA, "Tumor-Exp", "Tumor-Exp", NA, "Mut Status", "Mut-Ratio", NA,
      "CNV Status", "CNV-Amp", NA, "CNV-Del", NA,
      rep("Scale.Transcript.Level", gene_trans_n), rep("Scale.Methylation.Level", gene_methy_n)
    ),
    # column width
    width = c(
      5, 4, 4, 3, 2, 4, 4,
      2, 4, 4, 4, 4, rep(1.1, gene_trans_n), rep(1.1, gene_methy_n)
    )
  )

  column_info$options <- NA
  for (i in seq(nrow(column_info))) {
    column_info$options[i] <- list(list())
  }
  column_info$options[[1]] <- list()
  column_info$options[[2]] <- list(scale = F, hjust = 1)
  column_info$options[[3]] <- list(scale = F)
  column_info$options[[6]] <- list(scale = F)
  column_info$options[[7]] <- list(label = "Mut_str", overlay = T)
  column_info$options[[9]] <- list(scale = F)
  column_info$options[[10]] <- list(label = "CNV_Amp_str", overlay = T)
  column_info$options[[11]] <- list(scale = F)
  column_info$options[[12]] <- list(label = "CNV_Del_str", overlay = T)

  column_groups <- data.frame(
    level1 = c("TCGA", rep(paste("Gene:", gene), 5)),
    level2 = c("", "Expression Profile", "Mutation Profile", "CNV Profile", "Transcript Profile", "Methylation Profile"),
    group = c("TCGA", "Expression Profile", "Mutation Profile", "CNV Profile", "Transcript Profile", "Methylation Profile"),
    palette = "palette1"
  )

  palettes <- list(
    palette1 = c("#66c2a5", "#66c2a5"),
    `Tumor-Exp` = RColorBrewer::brewer.pal(n = 9, name = "Greys")[1:6],
    `Mut-Ratio` = RColorBrewer::brewer.pal(n = 9, name = "YlOrRd"),
    `Mut Status` = c(Mut = "red", Wild = "grey"),
    `CNV Status` = c(" -2" = "#5e3c99", " -1" = "#b2abd2", " 0" = "#f7f7f7", " 1" = "#fdb863", " 2" = "#e66101"),
    `CNV-Amp` = RColorBrewer::brewer.pal(n = 9, name = "Oranges"),
    `CNV-Del` = RColorBrewer::brewer.pal(n = 9, name = "Purples"),
    Scale.Methylation.Level = RColorBrewer::brewer.pal(n = 9, name = "BuPu"),
    Scale.Transcript.Level = RColorBrewer::brewer.pal(n = 9, name = "Greens")
  )


  position_arguments <- funkyheatmap::position_arguments(
    expand_xmax = 6,
    col_annot_angle = 35,
    col_width = 1.1,
    col_space = 0.2,
    col_annot_offset = 4
  )

  g <- funkyheatmap::funky_heatmap(gene_cross_dat,
    column_info = column_info,
    column_groups = column_groups,
    # row_info = row_info,
    palettes = palettes,
    position_args = position_arguments
  )


  if (return_list) {
    return(list(plot = g, data = gene_cross_dat))
  } else {
    return(g)
  }
}



#' Visualize cross-omics of one pathway among pan-cancers
#'
#' @param pw  pathway name
#' @param tumor_samples Select specific tumor samples. Default NULL, indicating all tumor samples.
#' @param tumor_projects Select specific TCGA projects. Default NULL, indicating all TCGA projects.
#' @param pval_mrna The P value thresholds
#' @param return_list TRUE returns a list including plot object and data. FALSE just returns plot.
#'
#' @return funkyheatmap
#' @export
#'
vis_pathway_cross_omics <- function(pw = "HALLMARK_ADIPOGENESIS",
                                    tumor_projects = NULL,
                                    tumor_samples = NULL,
                                    pval_mrna = c(0.05, 0.01, 0.001),
                                    return_list = FALSE) {
  tcga_gtex <- load_data("tcga_gtex")
  tcga_clinical_fine <- load_data("tcga_clinical_fine")
  if (is.null(tumor_projects)) {
    tumor_projects <- as.character(sort(unique(tcga_gtex$tissue)))
  } else {
    tumor_projects <- tumor_projects[tumor_projects %in% unique(tcga_gtex$tissue)]
  }

  if (is.null(tumor_samples)) {
    tcga_gtex2 <- tcga_gtex
    tcga_clinical_fine2 <- tcga_clinical_fine
  } else {
    tcga_gtex2 <- tcga_gtex %>%
      dplyr::filter(.data$type2 == "normal" | .data$sample %in% tumor_samples)
    tcga_clinical_fine2 <- tcga_clinical_fine %>%
      dplyr::filter(.data$Code == "NT" | .data$Sample %in% tumor_samples)
  }

  tcga_gtex2 <- tcga_gtex2 %>% dplyr::filter(.data$tissue %in% tumor_projects)
  tcga_clinical_fine2 <- tcga_clinical_fine2 %>% dplyr::filter(.data$Cancer %in% tumor_projects)

  #### Omics--mRNA
  toil_sig_score <- rbind(load_data("tcga_PW"), load_data("gtex_PW"))
  pw_mrna <- toil_sig_score[, pw, drop = FALSE] %>%
    tibble::rownames_to_column("sample") %>%
    dplyr::inner_join(tcga_gtex2)
  colnames(pw_mrna)[2] <- "Exp"

  pw_mrna_md <- pw_mrna %>%
    dplyr::group_by(.data$tissue, .data$type2) %>%
    dplyr::summarise(Exp = median(.data$Exp)) %>%
    as.data.frame() %>%
    dplyr::mutate(Exp = (.data$Exp - min(.data$Exp)) / (max(.data$Exp) - min(.data$Exp))) %>% # 0~1 scale
    tidyr::pivot_wider(id_cols = .data$tissue, names_from = .data$type2, values_from = .data$Exp)

  pw_mrna_comp <- ggpubr::compare_means(Exp ~ type2,
    data = pw_mrna,
    group.by = "tissue", method = "wilcox.test"
  )

  pw_mrna_plot <- dplyr::left_join(pw_mrna_md, pw_mrna_comp[, c("tissue", "p", "p.adj")]) %>%
    as.data.frame() %>%
    dplyr::rename("id" = "tissue") %>%
    dplyr::mutate(Label = case_when(
      p < pval_mrna[3] ~ "***",
      p < pval_mrna[2] ~ "**",
      p < pval_mrna[1] ~ "*",
      TRUE ~ "-"
    )) %>%
    dplyr::select("id", "normal", "tumor", "Label")

  #### Omics--mutation
  # pw_mut_cnv = load_data("tcga_PW_cross")
  pw_mut_cnv <- calc_pw_mut_cnv(tumor_samples = tumor_samples)

  pw_mut_stat <- dplyr::filter(pw_mut_cnv, Type == "Mut") %>%
    dplyr::filter(.data$PW == pw)

  pw_mut_plot <- pw_mut_stat %>%
    dplyr::select(-"Type", -"PW") %>%
    dplyr::rename("Mut_ratio" = "Ratio", "Mut_me" = "Count") %>%
    dplyr::mutate(Mut_ratio_str = scales::percent(.data$Mut_ratio, accuracy = 0.1), .before = 3) %>%
    dplyr::mutate(Mut_me_str = round(.data$Mut_me, digits = 2), .before = 5) %>%
    dplyr::mutate(Mut_pie = NA, .before = 2)

  for (i in seq(nrow(pw_mut_plot))) {
    # i = 1
    pw_mut_plot$Mut_pie[i] <- list(c(Mut = pw_mut_plot$Mut_ratio[i], Wild = 1 - pw_mut_plot$Mut_ratio[i]))
  }

  #### Omics--CNV
  pw_cnv_stat <- dplyr::filter(pw_mut_cnv, .data$Type != "Mut") %>%
    dplyr::filter(.data$PW == pw)

  pw_cnv_plot <- pw_cnv_stat %>%
    dplyr::select(-"PW", -"Ratio") %>%
    tidyr::pivot_wider(names_from = .data$Type, values_from = .data$Count) %>%
    dplyr::mutate(CNV_Pie = NA)

  for (i in seq(nrow(pw_cnv_plot))) {
    # i = 1
    pw_cnv_plot$CNV_Pie[i] <- list(c(
      "Amp" = pw_cnv_plot$Amp[i],
      "Del" = pw_cnv_plot$Del[i],
      "Non" = pw_cnv_plot$Non[i]
    ))
  }

  pw_cnv_plot <- pw_cnv_plot %>%
    dplyr::mutate(
      CNV_Amp = .data$Amp,
      CNV_Amp_str = round(.data$Amp, 1),
      CNV_Del = .data$Del,
      CNV_Del_str = round(.data$Del, 1)
    ) %>%
    dplyr::select(-"Non", -"Amp", -"Del")


  #### Ready for Plot
  pw_cross_dat <- data.frame(id = tumor_projects) %>%
    dplyr::left_join(pw_mrna_plot) %>%
    dplyr::left_join(pw_mut_plot, by = c("id" = "Cancer")) %>%
    dplyr::left_join(pw_cnv_plot, by = c("id" = "Cancer"))

  idx_1 <- which(sapply(pw_cross_dat$Mut_pie, is.null))
  for (i in idx_1) {
    pw_cross_dat$Mut_pie[[i]] <- c(Mut = NA, Wild = NA)
  }
  idx_2 <- which(sapply(pw_cross_dat$CNV_Pie, is.null))
  for (i in idx_2) {
    pw_cross_dat$CNV_Pie[[i]] <- c(` -2` = NA, ` -1` = NA, ` 0` = NA, ` 1` = NA, ` 2` = NA)
  }

  column_info <- data.frame(
    # columns to vis
    id = colnames(pw_cross_dat),
    # column name
    name = c(
      "", "Normal ssGSEA", "Tumor ssGSEA", "T vs. N(Wilcox)",
      "Mutation Dist Mean", "Mutation PCT Mean", "", "Mut.Genes Mean", "",
      "CNV Dist Mean", "Amp.Genes Mean", "", "Del.Genes Mean", ""
    ),
    # group name
    group = c(
      "TCGA", rep("Expression Profile", 3), rep("Mutation Profile", 5),
      rep("CNV Profile", 5)
    ),
    # geom type
    geom = c(
      "text", "bar", "bar", "text", "pie", "bar", "text", "bar", "text", "pie",
      "bar", "text", "bar", "text"
    ),
    # palette type
    palette = c(
      NA, "Tumor-Exp", "Tumor-Exp", NA, "Mut Status", "Mut-Ratio", NA, "Mut-Count", NA, "CNV Status",
      "CNV-Amp", NA, "CNV-Del", NA
    ), # "CNV-Pie"
    # column width
    width = c(5, 4, 4, 4, 2, 4, 4, 4, 4, 2, 4, 4, 4, 4)
  )

  column_info$options <- NA
  for (i in seq(nrow(column_info))) {
    column_info$options[i] <- list(list())
  }
  column_info$options[[2]] <- list(scale = F, hjust = 1)
  column_info$options[[3]] <- list(scale = F)
  column_info$options[[6]] <- list(scale = F)
  column_info$options[[7]] <- list(label = "Mut_ratio_str", overlay = T)
  column_info$options[[9]] <- list(label = "Mut_me_str", overlay = T)
  column_info$options[[12]] <- list(label = "CNV_Amp_str", overlay = T)
  column_info$options[[14]] <- list(label = "CNV_Del_str", overlay = T)

  column_groups <- data.frame(
    level1 = c("TCGA", rep(paste("Pathway: ", pw), 3)),
    level2 = c("", "Expression Profile", "Mutation Profile", "CNV Profile"),
    group = c("TCGA", "Expression Profile", "Mutation Profile", "CNV Profile"),
    palette = c("palette1")
  )
  palettes <- list(
    palette1 = c("#66c2a5", "#66c2a5"),
    `Tumor-Exp` = RColorBrewer::brewer.pal(n = 9, name = "Greys")[1:6],
    `Mut Status` = c(Mut = "red", Wild = "grey"),
    `Mut-Ratio` = RColorBrewer::brewer.pal(n = 9, name = "YlOrRd"),
    `Mut-mean` = RColorBrewer::brewer.pal(n = 9, name = "RdPu"),
    `CNV Status` = c(Amp = "#d53e4f", Del = "#3288bd", Non = "grey"),
    `CNV-Amp` = c("#fee08b", "#fdae61", "#f46d43", "#d53e4f"),
    `CNV-Del` = c("#e6f598", "#abdda4", "#66c2a5", "#3288bd")
  )

  position_arguments <- funkyheatmap::position_arguments(
    expand_xmax = 6,
    col_annot_angle = 35,
    col_width = 1.1,
    col_space = 0.2,
    col_annot_offset = 4
  )
  g <- funkyheatmap::funky_heatmap(pw_cross_dat,
    column_info = column_info,
    column_groups = column_groups,
    palettes = palettes,
    position_args = position_arguments
  )
  if (return_list) {
    return(list(plot = g, data = pw_cross_dat))
  } else {
    return(g)
  }
}





#' Prepare Mut/CNV data of pathways given specific samples
#'
#' @param tumor_samples Select specific tumor samples. Default NULL, indicating all tumor samples.
#'
#' @export
#'
calc_pw_mut_cnv <- function(tumor_samples = NULL) {
  tcga_clinical_fine <- load_data("tcga_clinical_fine")
  if (is.null(tumor_samples)) {
    return(load_data("tcga_PW_cross"))
  }
  tcga_clinical_fine2 <- tcga_clinical_fine %>%
    dplyr::filter(.data$Code == "NT" | .data$Sample %in% tumor_samples)

  tcga_PW_cross_raw <- load_data("tcga_PW_cross_raw")


  pw_mut_stat <- tcga_PW_cross_raw$pw_mut_one[tcga_clinical_fine2$Sample, ] %>%
    na.omit() %>%
    tibble::rownames_to_column("Sample") %>%
    tidyr::pivot_longer(!.data$Sample, names_to = "PW") %>%
    dplyr::inner_join(tcga_clinical_fine2[, 1:2]) %>%
    dplyr::group_by(.data$Cancer, .data$PW) %>%
    dplyr::summarise(Ratio = mean(.data$value > 0), Count = mean(.data$value)) %>%
    dplyr::ungroup()

  pw_cnv_posi_stat <- tcga_PW_cross_raw$pw_cnv_posi %>%
    as.data.frame() %>%
    tibble::rownames_to_column("Sample") %>%
    dplyr::inner_join(tcga_clinical_fine2[, 1:2], .) %>%
    dplyr::group_by(.data$Cancer) %>%
    dplyr::summarise(across(!.data$Sample, mean)) %>%
    as.data.frame()
  pw_cnv_nega_stat <- tcga_PW_cross_raw$pw_cnv_nega %>%
    as.data.frame() %>%
    tibble::rownames_to_column("Sample") %>%
    dplyr::inner_join(tcga_clinical_fine2[, 1:2], .) %>%
    dplyr::group_by(.data$Cancer) %>%
    dplyr::summarise(across(!.data$Sample, mean)) %>%
    as.data.frame()
  pw_cnv_zero_stat <- tcga_PW_cross_raw$pw_cnv_zero %>%
    as.data.frame() %>%
    tibble::rownames_to_column("Sample") %>%
    dplyr::inner_join(tcga_clinical_fine2[, 1:2], .) %>%
    dplyr::group_by(.data$Cancer) %>%
    dplyr::summarise(across(!.data$Sample, mean)) %>%
    as.data.frame()

  pw_cnv_stat <- rbind(
    pw_cnv_posi_stat %>%
      dplyr::mutate(Type = "Amp", .before = 2),
    pw_cnv_nega_stat %>%
      dplyr::mutate(Type = "Del", .before = 2),
    pw_cnv_zero_stat %>%
      dplyr::mutate(Type = "Non", .before = 2)
  )
  pw_cnv_stat <- pw_cnv_stat %>%
    tidyr::pivot_longer(contains("_"), names_to = c("PW"), values_to = "Count") %>%
    dplyr::group_by(.data$Cancer, .data$PW) %>%
    dplyr::mutate(Ratio = .data$Count / sum(.data$Count))

  pw_mut_cnv <- rbind(
    pw_mut_stat %>% dplyr::mutate(Type = "Mut", .before = 2),
    pw_cnv_stat
  ) %>% dplyr::mutate(PW = as.character(.data$PW))
  return(pw_mut_cnv)
}

## Avoid CRAN checking NOTEs

utils::globalVariables(
  c(
    "XenaHostNames",
    "DataSubtype",
    "XenaDatasets",
    ".",
    "Type",
    "group",
    ".env",
    "icgc_specimen_id",
    "dcc_project_code",
    "OS",
    "OS.time",
    "pval",
    "cor"
  )
)

query_general_id <- function() {
  tcga_clinical_fine <- load_data("tcga_clinical_fine")
  pcawg_info_fine <- load_data("pcawg_info_fine")
  ccle_info_fine <- load_data("ccle_info_fine")
  #### TCGA general value: index/immune/pathway/phenotype
  ## tumor_index
  tcga_index_value <- list(
    `Tumor Purity` = load_data("tcga_purity") %>% dplyr::rename("Sample" = "sample"),
    `Tumor Stemness` = load_data("tcga_stemness") %>% dplyr::rename("Sample" = "sample"),
    `Tumor Mutation Burden` = load_data("tcga_tmb") %>% dplyr::rename("Sample" = "Tumor_Sample_ID"),
    `Microsatellite Instability` = load_data("tcga_MSI"),
    `Genome Instability` = load_data("tcga_genome_instability") %>% dplyr::rename("Sample" = "sample")
  )
  # colnames(tcga_index_value$`Tumor Mutation Burden`)[3] = "sample"
  tcga_index_value$`Microsatellite Instability` <- load_data("tcga_gtex") %>%
    dplyr::rename("Sample" = "sample") %>%
    dplyr::mutate(Barcode = stringr::str_sub(.data$Sample, 1, 12)) %>%
    dplyr::select("Barcode", "Sample") %>%
    dplyr::inner_join(tcga_index_value$`Microsatellite Instability`, by = "Barcode")

  ## tumor_immune
  tcga_TIL <- load_data("tcga_TIL")
  colnames(tcga_TIL)[1] <- "Sample"
  dim(tcga_TIL)
  # [1] 11070   120
  tcga_immune_value <- list(
    `CIBERSORT` = tcga_TIL %>% dplyr::select("Sample", ends_with("CIBERSORT")) %>% dplyr::rename_with(~ gsub("_CIBERSORT", "", .x)),
    `CIBERSORT-ABS` = tcga_TIL %>% dplyr::select("Sample", ends_with("CIBERSORT-ABS")) %>% dplyr::rename_with(~ gsub("_CIBERSORT-ABS", "", .x)),
    `EPIC` = tcga_TIL %>% dplyr::select("Sample", ends_with("EPIC")) %>% dplyr::rename_with(~ gsub("_EPIC", "", .x)),
    `MCPCOUNTER` = tcga_TIL %>% dplyr::select("Sample", ends_with("MCPCOUNTER")) %>% dplyr::rename_with(~ gsub("_MCPCOUNTER", "", .x)),
    `QUANTISEQ` = tcga_TIL %>% dplyr::select("Sample", ends_with("QUANTISEQ")) %>% dplyr::rename_with(~ gsub("_QUANTISEQ", "", .x)),
    `TIMER` = tcga_TIL %>% dplyr::select("Sample", ends_with("TIMER")) %>% dplyr::rename_with(~ gsub("_TIMER", "", .x)),
    `XCELL` = tcga_TIL %>% dplyr::select("Sample", ends_with("XCELL")) %>% dplyr::rename_with(~ gsub("_XCELL", "", .x))
  )

  ## tumor_pathway
  tcga_PW <- load_data("tcga_PW") %>%
    as.data.frame() %>%
    tibble::rownames_to_column("Sample")
  dim(tcga_PW)
  # [1] 10534   501
  tcga_pathway_value <- list(
    `HALLMARK` = tcga_PW %>% dplyr::select("Sample", contains("HALLMARK")) %>% dplyr::rename_with(~ gsub("HALLMARK_", "", .x)),
    `KEGG` = tcga_PW %>% dplyr::select("Sample", contains("KEGG")) %>% dplyr::rename_with(~ gsub("KEGG_", "", .x)),
    `IOBR` = tcga_PW %>% dplyr::select("Sample", contains("IOBR")) %>% dplyr::rename_with(~ gsub("IOBR_", "", .x))
  )

  ## tumor_phenotype (user upload custom metadata)
  tcga_phenotype_value <- list(
    `Clinical Phenotype` = tcga_clinical_fine,
    `Custom metadata` = NULL
  )


  #### PCAWG general value: index/immune/pathway/phenotype
  ## tumor_index
  pcawg_index_value <- list(
    `Tumor Purity` = load_data("pcawg_purity") %>%
      dplyr::filter(.data$icgc_specimen_id %in% pcawg_info_fine$Sample) %>%
      dplyr::rename("Sample" = "icgc_specimen_id")
  )

  ## tumor_immune
  pcawg_TIL <- load_data("pcawg_TIL")
  colnames(pcawg_TIL)[1] <- "Sample"
  dim(pcawg_TIL)
  # [1] 1466   120
  pcawg_immune_value <- list(
    `CIBERSORT` = pcawg_TIL %>% dplyr::select("Sample", ends_with("CIBERSORT")) %>% dplyr::rename_with(~ gsub("_CIBERSORT", "", .x)),
    `CIBERSORT-ABS` = pcawg_TIL %>% dplyr::select("Sample", ends_with("CIBERSORT-ABS")) %>% dplyr::rename_with(~ gsub("_CIBERSORT-ABS", "", .x)),
    `EPIC` = pcawg_TIL %>% dplyr::select("Sample", ends_with("EPIC")) %>% dplyr::rename_with(~ gsub("_EPIC", "", .x)),
    `MCPCOUNTER` = pcawg_TIL %>% dplyr::select("Sample", ends_with("MCPCOUNTER")) %>% dplyr::rename_with(~ gsub("_MCPCOUNTER", "", .x)),
    `QUANTISEQ` = pcawg_TIL %>% dplyr::select("Sample", ends_with("QUANTISEQ")) %>% dplyr::rename_with(~ gsub("_QUANTISEQ", "", .x)),
    `TIMER` = pcawg_TIL %>% dplyr::select("Sample", ends_with("TIMER")) %>% dplyr::rename_with(~ gsub("_TIMER", "", .x)),
    `XCELL` = pcawg_TIL %>% dplyr::select("Sample", ends_with("XCELL")) %>% dplyr::rename_with(~ gsub("_XCELL", "", .x))
  )

  ## tumor_pathway
  pcawg_PW <- load_data("pcawg_PW") %>%
    as.data.frame() %>%
    tibble::rownames_to_column("Sample")
  dim(pcawg_PW)
  # [1] 1466   501
  pcawg_pathway_value <- list(
    `HALLMARK` = pcawg_PW %>% dplyr::select("Sample", contains("HALLMARK")) %>% dplyr::rename_with(~ gsub("HALLMARK_", "", .x)),
    `KEGG` = pcawg_PW %>% dplyr::select("Sample", contains("KEGG")) %>% dplyr::rename_with(~ gsub("KEGG_", "", .x)),
    `IOBR` = pcawg_PW %>% dplyr::select("Sample", contains("IOBR")) %>% dplyr::rename_with(~ gsub("IOBR_", "", .x))
  )

  pcawg_phenotype_value <- list(
    `Clinical Phenotype` = pcawg_info_fine,
    `Custom metadata` = NULL
  )

  #### CCLE general value: index/immune/pathway/phenotype
  ## tumor_index
  ccle_index_value <- list(
    `Tumor Purity` = load_data("ccle_absolute") %>%
      dplyr::rename("Sample" = "Cell Line") %>%
      dplyr::filter(.data$Sample %in% ccle_info_fine$Sample)
  )
  ccle_phenotype_value <- list(
    `Clinical Phenotype` = ccle_info_fine,
    `Custom metadata` = NULL
  )

  tcga_value_option <- list(
    "Tumor index" = tcga_index_value, "Immune Infiltration" = tcga_immune_value,
    "Pathway activity" = tcga_pathway_value, "Phenotype data" = tcga_phenotype_value
  )
  pcawg_value_option <- list(
    "Tumor index" = pcawg_index_value, "Immune Infiltration" = pcawg_immune_value,
    "Pathway activity" = pcawg_pathway_value, "Phenotype data" = pcawg_phenotype_value
  )
  ccle_value_option <- list("Tumor index" = ccle_index_value, "Phenotype data" = ccle_phenotype_value)

  #### TCGA general id: molecule/index/immune/pathway/phenotype
  ## molecule_profile
  # pancan_identifiers <- readRDS(
  #   system.file(
  #     "extdata", "pancan_identifier_list.rds",
  #     package = "UCSCXenaShiny"
  #   )
  # )
  tcga_id <- load_data("v2_tcga_id")
  tcga_id.list <- split(tcga_id$L3, tcga_id$L2)

  tcga_molecule_id <- list(
    `mRNA Expression` = list(all = tcga_id.list[["Gene"]], default = "TP53"),
    # `Transcript Expression` = list(all = load_data("transcript_identifier"), default = "ENST00000000233"),
    `Transcript Expression` = list(all = tcga_id.list[["Transcript"]], default = "ENST00000000233"),
    `DNA Methylation` = list(all = tcga_id.list[["Gene"]], default = "TP53"),
    `Protein Expression` = list(all = tcga_id.list[["Protein"]], default = "P53"),
    `miRNA Expression` = list(all = tcga_id.list[["miRNA"]], default = "hsa-miR-769-3p"),
    `Mutation status` = list(all = tcga_id.list[["Gene"]], default = "TP53"),
    `Copy Number Variation` = list(all = tcga_id.list[["Gene"]], default = "TP53")
  )

  ## tumor_index
  tcga_index_id <- list(
    `Tumor Purity` = list(all = sort(colnames(tcga_index_value$`Tumor Purity`)[3:7]), default = "ESTIMATE"),
    `Tumor Stemness` = list(all = sort(colnames(tcga_index_value$`Tumor Stemness`)[2:6]), default = "RNAss"),
    `Tumor Mutation Burden` = list(all = sort(colnames(tcga_index_value$`Tumor Mutation Burden`)[4:5]), default = "Non_silent_per_Mb"),
    `Microsatellite Instability` = list(
      all = sort(setdiff(
        colnames(tcga_index_value$`Microsatellite Instability`)[3:21],
        c("MSI_intronic", "MSI_intronic_profiled", "MSI_noncoding", "MSI_noncoding_profiled")
      )),
      default = "Total_nb_MSI_events"
    ),
    `Genome Instability` = list(all = sort(colnames(tcga_index_value$`Genome Instability`)[2:6]), default = "ploidy")
  )

  ## tumor_immune
  tcga_immune_id <- lapply(tcga_immune_value, function(x) {
    list(all = sort(colnames(x)[-1]))
  })
  tcga_immune_id$`CIBERSORT`$default <- "Monocyte"
  tcga_immune_id$`CIBERSORT-ABS`$default <- "Monocyte"
  tcga_immune_id$`EPIC`$default <- "Macrophage"
  tcga_immune_id$`MCPCOUNTER`$default <- "Monocyte"
  tcga_immune_id$`QUANTISEQ`$default <- "Monocyte"
  tcga_immune_id$`TIMER`$default <- "Monocyte"
  tcga_immune_id$`XCELL`$default <- "Monocyte"

  ## tumor_pathway
  tcga_pathway_id <- lapply(tcga_pathway_value, function(x) {
    list(all = sort(colnames(x)[-1]))
  })
  tcga_pathway_id$HALLMARK$default <- "APOPTOSIS"
  tcga_pathway_id$KEGG$default <- "CELL_CYCLE"
  tcga_pathway_id$IOBR$default <- "Biotin_Metabolism"

  ## tumor_phenotype
  tcga_phenotype_id <- list(
    `Clinical Phenotype` = list(all = sort(colnames(tcga_phenotype_value$`Clinical Phenotype`[3:8])), default = "Code"),
    `Custom metadata` = list(all = NULL, default = NULL)
  )


  ## merge
  tcga_id_option <- list(
    "Molecular profile" = tcga_molecule_id,
    "Tumor index" = tcga_index_id,
    "Immune Infiltration" = tcga_immune_id,
    "Pathway activity" = tcga_pathway_id,
    "Phenotype data" = tcga_phenotype_id
  )

  #### PCAWG ID
  pcawg_id_option <- tcga_id_option
  # pcawg_id_referrence = load_data("pcawg_identifier")
  pcawg_id_referrence <- load_data("v2_pcawg_id")
  pcawg_id_option$`Molecular profile` <- list(
    `mRNA Expression` = list(all = sort(pcawg_id_referrence$L3[pcawg_id_referrence$L2 == "Gene"]), default = "TP53"),
    `Promoter Activity` = list(all = sort(pcawg_id_referrence$L3[pcawg_id_referrence$L2 == "Promoter"]), default = "prmtr.1"),
    `Gene Fusion` = list(all = sort(pcawg_id_referrence$L3[pcawg_id_referrence$L2 == "Fusion"]), default = "SAMD11"),
    `miRNA Expression` = list(all = sort(pcawg_id_referrence$L3[pcawg_id_referrence$L2 == "miRNA"]), default = "hsa-let-7a-2-3p"),
    `APOBEC Mutagenesis` = list(all = sort(pcawg_id_referrence$L3[pcawg_id_referrence$L2 == "Muta"]), default = "A3A_or_A3B")
  )
  pcawg_id_option$`Tumor index` <- list(
    `Tumor Purity` = list(
      all = c("purity", "ploidy", "purity_conf_mad", "wgd_status", "wgd_uncertain"),
      default = "purity"
    )
  )
  pcawg_id_option$`Phenotype data` <- list(
    `Clinical Phenotype` = list(
      all = c("Age", "Gender", "Type"),
      default = "Age"
    ),
    `Custom metadata` = list(all = NULL, default = NULL)
  )

  #### CCLE ID
  ccle_id_option <- list()
  # ccle_id_referrence = load_data("ccle_identifier")
  ccle_id_referrence <- load_data("v2_ccle_id")
  ccle_id_option$`Molecular profile` <- list(
    `mRNA Expression` = list(all = sort(ccle_id_referrence$L3[ccle_id_referrence$L2 == "Gene"]), default = "TP53"),
    `Protein Expression` = list(all = sort(ccle_id_referrence$L3[ccle_id_referrence$L2 == "Protein"]), default = "14-3-3_beta"),
    `Copy Number Variation` = list(all = sort(ccle_id_referrence$L3[ccle_id_referrence$L2 == "Gene"]), default = "TP53"),
    `Mutation status` = list(all = sort(ccle_id_referrence$L3[ccle_id_referrence$L2 == "Gene"]), default = "TP53")
  )
  ccle_id_option$`Tumor index` <- list(
    `Tumor Purity` = list(
      all = c("Purity", "Ploidy", "Genome Doublings", "Lineage"),
      default = "Purity"
    )
  )
  ccle_id_option$`Immune Infiltration` <- list(NULL)
  ccle_id_option$`Pathway activity` <- list(NULL)
  ccle_id_option$`Phenotype data` <- list(
    `Clinical Phenotype` = list(
      all = c("Gender", "Histology", "Type"),
      default = "Gender"
    ),
    `Custom metadata` = list(all = NULL, default = NULL)
  )

  res_merge <- list(
    value = list(tcga_value_option, pcawg_value_option, ccle_value_option),
    id = list(tcga_id_option, pcawg_id_option, ccle_id_option)
  )

  return(res_merge)
}







#' Download data for shiny general analysis
#'
#' @param L1 level 1  main datatype
#' @param L2 level 2  sub datatype
#' @param L3 level 3  identifier
#' @param opt_pancan      molecular datasets parameters
#' @param custom_metadata user customized metadata
#' @param database        one of c("toil","pcawg","ccle")
#' @param tpc_value_nonomics     non-omics matrix data of one database
#'
#' @examples
#' \dontrun{
#' general_value_id <- UCSCXenaShiny:::query_general_id()
#' tcga_value_option <- general_value_id[["value"]][[1]]
#' tcga_index_value <- tcga_value_option[["Tumor index"]]
#' tcga_immune_value <- tcga_value_option[["Immune Infiltration"]]
#' tcga_pathway_value <- tcga_value_option[["Pathway activity"]]
#' tcga_phenotype_value <- tcga_value_option[["Phenotype data"]]
#' 
#' clinical_phe = tcga_phenotype_value[["Clinical Phenotype"]]
#' x_data = UCSCXenaShiny:::query_general_value(
#'            "Molecular profile", "mRNA Expression", "TP53", "toil",
#'            tcga_index_value, tcga_immune_value, tcga_pathway_value,
#'            clinical_phe)
#'
#' y_data = UCSCXenaShiny:::query_general_value(
#'            "Immune Infiltration", "CIBERSORT", "Monocyte", "toil",
#'            tcga_index_value, tcga_immune_value, tcga_pathway_value,
#'            clinical_phe)
#' }


query_general_value <- function(L1, L2, L3,
                                database = c("toil", "pcawg", "ccle"),
                                tpc_value_nonomics = NULL,
                                opt_pancan = NULL, custom_metadata = NULL) {
  database <- match.arg(database)

  if (L1 == "Molecular profile") {
    # L2 = "mRNA Expression"
    # L3 = "TP53"
    L2_label <- switch(L2,
      `mRNA Expression` = "mRNA",
      `Transcript Expression` = "transcript",
      `DNA Methylation` = "methylation",
      `Protein Expression` = "protein",
      `miRNA Expression` = "miRNA",
      `Mutation status` = "mutation",
      `Copy Number Variation` = "cnv",
      `Promoter Activity` = "promoter",
      `Gene Fusion` = "fusion",
      `APOBEC Mutagenesis` = "APOBEC"
    )
    if (is.null(opt_pancan)) {
      opt_pancan <- .opt_pancan
    }
    x_data <- query_pancan_value(
      molecule = L3,
      data_type = L2_label,
      database = database,
      opt_pancan = opt_pancan
    )
    if (database == "ccle" & L2 == "Mutation status") {
      x_data <- x_data[, c("sampleID", "genes")]
      x_data <- x_data %>%
        dplyr::select("sampleID", "genes") %>%
        dplyr::mutate(genes = ifelse(is.na(.data$genes), 0, 1)) %>%
        dplyr::arrange(.data$sampleID, desc(.data$genes)) %>%
        dplyr::distinct(.data$sampleID, .keep_all = T) %>%
        dplyr::mutate(id = L3, .before = 1) %>%
        dplyr::rename("Sample" = "sampleID", "value" = "genes") %>%
        dplyr::mutate(level2 = "Mutation status")
    } else {
      if (is.list(x_data)) x_data <- x_data[[1]]
      x_data <- data.frame(
        "id" = L3,
        "Sample" = names(x_data), "value" = as.numeric(x_data),
        "level2" = L2
      )
    }
  } else {
    if (is.null(tpc_value_nonomics)) {
      if(database=="toil"){
        tpc_value_nonomics <- load_data(paste0("v2_tcga_value_nonomics"))
      } else {
        tpc_value_nonomics <- load_data(paste0("v2_", database, "_value_nonomics"))
      }
    }
    if (L2 != "Custom metadata") {
      L1_label <- switch(L1,
        `Tumor index` = "Index",
        `Immune Infiltration` = "Immune",
        `Pathway activity` = "Pathway",
        `Phenotype data` = "Phenotype"
      )
      L2_label <- switch(L2,
        # "Tumor index"
        `Tumor Purity` = "Purity",
        `Tumor Stemness` = "Stem",
        `Tumor Mutation Burden` = "TMB",
        `Microsatellite Instability` = "MSI",
        `Genome Instability` = "GI",
        # "Immune Infiltration"
        `CIBERSORT` = "CIB",
        `CIBERSORT-ABS` = "CIB.ABS",
        `EPIC` = "EPIC",
        `MCPCOUNTER` = "MCP",
        `QUANTISEQ` = "Quant",
        `TIMER` = "TIMER",
        `XCELL` = "XCELL",
        # "Pathway activity"
        `HALLMARK` = "HM",
        `KEGG` = "KEGG",
        `IOBR` = "IOBR",
        # "Phenotype data"
        `Clinical Phenotype` = "Clinical"
      )
      nonomics_id <- paste(L1_label, L2_label, L3, sep = "+")
      stopifnot(nonomics_id %in% colnames(tpc_value_nonomics))
      x_data <- tpc_value_nonomics[, c("Sample", nonomics_id)]
    } else {
      stopifnot(!is.null(custom_metadata))
      x_data <- custom_metadata[, c("Sample", L3)] %>%
        dplyr::filter(.data$Sample %in% tpc_value_nonomics$Sample)
    }

    colnames(x_data) <- c("Sample", "value")
    x_data <- x_data %>%
      dplyr::mutate(id = L3, .before = 1) %>%
      dplyr::mutate(level2 = L2)
  }
  x_data <- as.data.frame(x_data)
  # 检查重复样本
  if (is.numeric(x_data$value)) {
    x_data <- x_data %>%
      # dplyr::group_by(.data$Sample, .data$id, .data$level2) %>%
      # dplyr::summarise(value = mean(.data$value)) %>%
      dplyr::filter(!is.na(.data$value)) %>%
      as.data.frame()
  }
  x_data
  # id                  Sample value          level2
  # 1 TP53 GTEX-S4Q7-0003-SM-3NM8M 4.785 mRNA Expression
  # 2 TP53         TCGA-19-1787-01 5.887 mRNA Expression
  # 3 TP53         TCGA-S9-A7J2-01 5.517 mRNA Expression
  # 4 TP53 GTEX-QV31-1626-SM-2S1QC 4.431 mRNA Expression
  # 5 TP53         TCGA-G3-A3CH-11 2.382 mRNA Expression
  # 6 TP53         TCGA-B5-A5OE-01 5.765 mRNA Expression
}









#' Quick molecule analysis and report generation
#'
#' @inheritParams query_pancan_value
#' @param out_dir path to save analysis result and report, default is '.'
#' @param out_report logical value wheather to generate html report
#'
#' @return a list.
#' @export
mol_quick_analysis <- function(molecule, data_type, out_dir = ".", out_report = FALSE) {
  if (dir.exists(out_dir)) {
    cat("Warning: The file path already exists. Existing file may be overwritten.\n")
  } else {
    dir.create(out_dir)
  }

  print("##### Step1: Query the moleluce value... #####")
  mol_data <- query_pancan_value(molecule, data_type = data_type, database = "toil")
  if (is.list(mol_data)) mol_data <- mol_data[[1]]
  
  if (all(is.na(mol_data))) {
    message("All NAs returned, return NULL instead.")
    return(NULL)
  }

  print(paste0("=== ", "Clinical phenotype"))
  mol_data_df <- suppressMessages(data.frame(
    id = molecule,
    value = mol_data
  ) %>%
    tibble::rownames_to_column("Sample") %>%
    dplyr::inner_join(load_data("tcga_gtex"), by = c("Sample" = "sample")))

  print(paste0("---> ", "Tumor&Normal"))
  comp_TN <- suppressMessages(mol_data_df %>%
    dplyr::filter(!.data$tissue %in% c("MESO", "UVM")) %>%
    dplyr::group_by(.data$tissue) %>%
    dplyr::summarise(wilcox_p = stats::wilcox.test(value ~ type2)$p.value))
  comp_TN <- suppressMessages(mol_data_df %>%
    dplyr::filter(!.data$tissue %in% c("MESO", "UVM")) %>%
    dplyr::group_by(.data$tissue, .data$type2) %>%
    dplyr::summarise(value = mean(.data$value)) %>%
    tidyr::pivot_wider(names_from = "type2", values_from = "value") %>%
    dplyr::inner_join(comp_TN))

  print(paste0("---> ", "Age"))
  mol_data_Age <- suppressMessages(mol_data_df[, 1:4] %>%
    dplyr::inner_join(load_data("tcga_clinical_fine")[, c("Sample", "Age")]) %>%
    dplyr::filter(!is.na(.data$Age)) %>%
    dplyr::mutate(Age = ifelse(.data$Age >= 60, "Old", "Young")))
  valid_types <- suppressMessages(mol_data_Age %>%
    dplyr::group_by(.data$tissue) %>%
    dplyr::count(.data$Age) %>% dplyr::count(.data$tissue) %>%
    dplyr::filter(n == 2) %>% dplyr::pull(.data$tissue))
  mol_data_Age <- mol_data_Age[mol_data_Age$tissue %in% valid_types, ]
  comp_Age <- suppressMessages(mol_data_Age %>%
    dplyr::group_by(.data$tissue) %>%
    dplyr::summarise(wilcox_p = stats::wilcox.test(value ~ Age)$p.value))
  comp_Age <- suppressMessages(mol_data_Age %>%
    dplyr::group_by(.data$tissue, .data$Age) %>%
    dplyr::summarise(value = mean(.data$value)) %>%
    tidyr::pivot_wider(names_from = "Age", values_from = "value") %>%
    dplyr::inner_join(comp_Age))

  print(paste0("---> ", "Gender"))
  mol_data_Gender <- suppressMessages(mol_data_df[, 1:4] %>%
    dplyr::inner_join(load_data("tcga_clinical_fine")[, c("Sample", "Gender")]) %>%
    dplyr::filter(!is.na(.data$Gender)))
  valid_types <- suppressMessages(mol_data_Gender %>%
    dplyr::group_by(.data$tissue) %>%
    dplyr::count(.data$Gender) %>% dplyr::count(.data$tissue) %>%
    dplyr::filter(n == 2) %>% dplyr::pull(.data$tissue))
  mol_data_Gender <- mol_data_Gender[mol_data_Gender$tissue %in% valid_types, ]
  comp_Gender <- suppressMessages(mol_data_Gender %>%
    dplyr::group_by(.data$tissue) %>%
    dplyr::summarise(wilcox_p = stats::wilcox.test(value ~ Gender)$p.value))
  comp_Gender <- suppressMessages(mol_data_Gender %>%
    dplyr::group_by(.data$tissue, .data$Gender) %>%
    dplyr::summarise(value = mean(.data$value)) %>%
    tidyr::pivot_wider(names_from = "Gender", values_from = "value") %>%
    dplyr::inner_join(comp_Gender))

  print(paste0("---> ", "Stage"))
  mol_data_Stage <- suppressMessages(mol_data_df[, 1:4] %>%
    dplyr::inner_join(load_data("tcga_clinical_fine")[, c("Sample", "Stage_ajcc")]) %>%
    dplyr::filter(!is.na(.data$Stage_ajcc)))
  valid_types <- suppressMessages(mol_data_Stage %>%
    dplyr::group_by(.data$tissue) %>%
    dplyr::count(.data$Stage_ajcc) %>% dplyr::count(.data$tissue) %>%
    dplyr::filter(n >= 2) %>% dplyr::pull(.data$tissue))
  mol_data_Stage <- mol_data_Stage[mol_data_Stage$tissue %in% valid_types, ]
  comp_Stage <- suppressMessages(mol_data_Stage %>%
    dplyr::group_by(.data$tissue) %>%
    dplyr::summarise(aov_p = summary(stats::aov(value ~ Stage_ajcc))[[1]][1, 5]))
  comp_Stage <- suppressMessages(mol_data_Stage %>%
    dplyr::group_by(.data$tissue, .data$Stage_ajcc) %>%
    dplyr::summarise(value = mean(.data$value)) %>%
    tidyr::pivot_wider(names_from = "Stage_ajcc", values_from = "value") %>%
    dplyr::inner_join(comp_Stage))

  phe_res <- list(
    comp_TN = comp_TN %>%
      tidyr::pivot_longer(c("normal", "tumor")) %>% dplyr::rename("P.value" = "wilcox_p"),
    comp_Age = comp_Age %>%
      tidyr::pivot_longer(c("Old", "Young")) %>% dplyr::rename("P.value" = "wilcox_p"),
    comp_Gender = comp_Gender %>%
      tidyr::pivot_longer(c("FEMALE", "MALE")) %>% dplyr::rename("P.value" = "wilcox_p"),
    comp_Stage = comp_Stage %>%
      tidyr::pivot_longer(starts_with("Stage")) %>% dplyr::rename("P.value" = "aov_p")
  ) %>%
    do.call(rbind, .) %>%
    as.data.frame()


  tcga_clinical <- load_data("tcga_clinical")
  mol_data_df <- data.frame(
    id = molecule,
    value = mol_data
  ) %>%
    tibble::rownames_to_column("Sample") %>%
    dplyr::filter(.data$Sample %in% tcga_clinical$sample) %>%
    dplyr::mutate(type = tcga_clinical$type[match(.data$Sample, tcga_clinical$sample)]) %>%
    dplyr::filter(grepl("01$", .data$Sample)) %>% # -01 primary tumor
    dplyr::arrange(.data$type, .data$Sample)

  # 依据每种癌症的中位数进行分组
  if (data_type == "mutation") {
    mol_data_df <- mol_data_df %>%
      dplyr::mutate(group = ifelse(.data$value == 1, "Mutate", "Wild")) %>%
      as.data.frame()
  } else {
    mol_data_df <- mol_data_df %>%
      dplyr::group_by(.data$type) %>%
      dplyr::mutate(group = ifelse(.data$value > median(.data$value), "Higher", "Lower")) %>%
      as.data.frame()
  }

  general_value_id <- query_general_id()
  tcga_value_option <- general_value_id[["value"]][[1]]
  tcga_id_option <- general_value_id[["id"]][[1]]

  ## 相关性分析结果
  print("##### Step2: Execute correlation analysis...  #####")
  cor_res <- lapply(c("Tumor index", "Immune Infiltration", "Pathway activity"), function(L1_type) {
    # L1_type = "Pathway activity"
    print(paste0("=== ", L1_type))
    tcga_id <- tcga_id_option[[L1_type]]
    tcga_value <- tcga_value_option[[L1_type]]
    L2_types <- names(tcga_id)

    cor_L1 <- lapply(L2_types, function(L2_type) {
      # L2_type = L2_types[3]
      print(paste0("---> ", L2_type))
      L3_ids <- tcga_id[[L2_type]]$all
      value_df <- tcga_value[[L2_type]] %>% as.data.frame()
      L3_ids <- L3_ids[sapply(L3_ids, function(id) {
        class(value_df[, id])
      }) != "character"]

      value_df_long <- value_df[, c("Sample", L3_ids)] %>%
        tidyr::pivot_longer(-.data$Sample, names_to = "L3", values_to = "target")

      data_merge <- suppressMessages(dplyr::inner_join(mol_data_df, value_df_long) %>%
        dplyr::filter(!is.na(.data$target)))

      valid_L3 <- data_merge %>%
        dplyr::distinct(.data$group, .data$L3) %>%
        dplyr::count(.data$L3) %>%
        dplyr::filter(n == 2) %>%
        dplyr::pull(.data$L3)

      data_merge <- data_merge %>%
        dplyr::filter(.data$L3 %in% valid_L3)

      cor_L2 <- suppressMessages(data_merge %>%
        dplyr::group_by(.data$L3, .data$type) %>%
        dplyr::summarise(
          spearman_r = stats::cor.test(.data$value, .data$target, method = "spearman")$estimate,
          spearman_p = stats::cor.test(.data$value, .data$target, method = "spearman")$p.value
        ) %>%
        dplyr::mutate(L1 = L1_type, .before = 1) %>%
        dplyr::mutate(L2 = L2_type, .before = 2))
      return(cor_L2)
    }) %>% do.call(rbind, .)
    cor_L1
  }) %>% do.call(rbind, .)


  ## 生存分析结果
  print("##### Step4: Execute survival analysis...  #####")
  sur_res <- lapply(c("OS", "DSS", "DFI", "PFI"), function(event) {
    # event = "OS"
    # print(event)
    print(paste0("=== ", event))
    tcga_surv_sub <- load_data("tcga_surv") %>%
      dplyr::select(sample, contains(event)) %>%
      na.omit()
    colnames(tcga_surv_sub) <- c("Sample", "status", "time")

    data_merge <- suppressMessages(dplyr::inner_join(mol_data_df, tcga_surv_sub))


    sur_merge <- lapply(unique(data_merge$type), function(type) {
      # type = "BRCA"
      # print(type)
      sur_dat <- data_merge[data_merge$type == type, ]
      if (length(unique(sur_dat$group)) == 1) {
        return(NA)
      }
      surv_diff <- survival::survdiff(survival::Surv(time, status) ~ group, data = sur_dat)
      p.val <- 1 - stats::pchisq(surv_diff$chisq, length(surv_diff$n) - 1)
      hr <- exp(survival::coxph(survival::Surv(time, status) ~ group, data = sur_dat)$coefficients)
      c(p.val, hr)
    }) %>%
      do.call(rbind, .) %>%
      as.data.frame() %>%
      dplyr::mutate(type = unique(data_merge$type), .before = 1) %>%
      # tibble::rownames_to_column("type") %>%
      dplyr::rename("logrank_p" = "V1") %>%
      dplyr::mutate(P.direction = ifelse(.data$groupLower < 1, "Higher Risk", "Lower Risk")) %>%
      dplyr::mutate("event" = event, .before = 1) %>%
      dplyr::select(!.data$groupLower)
    sur_merge
  }) %>% do.call(rbind, .)


  utils::write.csv(phe_res,
    row.names = FALSE,
    file = file.path(out_dir, "molecule_clinical_result.csv")
  )
  utils::write.csv(cor_res,
    row.names = FALSE,
    file = file.path(out_dir, "molecule_correlation_result.csv")
  )
  utils::write.csv(sur_res,
    row.names = FALSE,
    file = file.path(out_dir, "molecule_survival_result.csv")
  )

  # phe_res = read.csv(file.path(tempdir(),"molecule_clinical_result.csv"))
  # sur_res = read.csv(file.path(tempdir(),"molecule_survival_result.csv"))
  # cor_res = read.csv(file.path(tempdir(),"molecule_correlation_result.csv"))




  if (out_report) {
    params <- list(
      "sur_res" = sur_res,
      "cor_res" = cor_res,
      "phe_res" = phe_res,
      "id_name" = molecule,
      "id_type" = data_type
    )
    print("##### Step5: Render html report...  #####")
    file.copy(
      from = system.file("rmd", "report_template.Rmd", package = "UCSCXenaShiny"),
      to = file.path(out_dir, "report_template.Rmd"), overwrite = TRUE
    )
    rmarkdown::render(file.path(out_dir, "report_template.Rmd"), "html_document",
      file.path(out_dir, "report_result.html"),
      params = params,
      envir = new.env(parent = globalenv())
    )
    file.remove(file.path(out_dir, "report_template.Rmd"))
  }
  print("All analysis is done!")

  return(list(sur_res = sur_res, cor_res = cor_res, phe_res = phe_res))
}

#' Run Correlation between Two Variables and Support Group by a Variable
#'
#' @param data a `data.frame` containing variables
#' @param split whether perform correlation grouped by a variable, default is 'FALSE'
#' @param split_var a `character`, the group variable
#' @param var1 a character, the first variable in correlation
#' @param var2 a character, the second variable in correlation
#' @param cor_method method="pearson" is the default value. The alternatives to be passed to cor are "spearman" and "kendall"
#' @param adjust_method What adjustment for multiple tests should be used? ("holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none")
#' @param use	use="pairwise" will do pairwise deletion of cases. use="complete" will select just complete cases
#' @param sig_label whether add symbal of significance. P < 0.001,`***`; P < 0.01,`**`; P < 0.05,`*`; P >=0.05,""
#' @param verbose if `TRUE`, print extra info.
#' @return a `data.frame`
#' @author Yi Xiong
#' @export
ezcor <- function(data = NULL,
                  split = FALSE,
                  split_var = NULL,
                  var1 = NULL,
                  var2 = NULL,
                  cor_method = "pearson",
                  adjust_method = "none",
                  use = "complete",
                  sig_label = TRUE,
                  verbose = TRUE) {
  stopifnot(is.data.frame(data))
  ss <- data
  if (!var1 %in% colnames(ss)) {
    stop("the first variable is unavailable in the dataset!")
  }
  if (!var2 %in% colnames(ss)) {
    stop("the second variable is unavailable in the dataset!")
  }
  if (length(var1) != 1) {
    stop("only one element is needed in the first variable!")
  }
  if (length(var2) != 1) {
    stop("only one element is needed in the second variable!")
  }
  if (split == TRUE) {
    if (!split_var %in% colnames(ss)) {
      stop("split variable is unavailable in the dataset!")
    }
    # index
    n <- which(colnames(ss) %in% split_var)
    sss <- with(ss, split(ss, ss[, n]))
    s <- names(sss)
    ## calculate correlation
    cor2var <- purrr::map(s, purrr::safely(function(x) {
      # x = s[1]
      sss_sub <- sss[[x]]
      dd <-
        psych::corr.test(
          as.numeric(sss_sub[, var1]),
          as.numeric(sss_sub[, var2]),
          method = cor_method,
          adjust = adjust_method,
          use =  use
        )
      # dd <- stats::cor.test(as.numeric(sss_can[,var1]),as.numeric(sss_can[,var2]), type = cor_method)
      ddd <-
        data.frame(
          cor = dd$r,
          p.value = dd$p,
          method = cor_method,
          adjust = adjust_method,
          v1 = var1,
          v2 = var2,
          stringsAsFactors = F
        )
      ddd$group <- x
      return(ddd)
    })) %>% purrr::set_names(s)

    cor2var <- cor2var %>%
      purrr::map(~ .x$result) %>%
      purrr::compact()
    cor2var_df <- do.call(rbind.data.frame, cor2var)

    if (sig_label == TRUE) {
      cor2var_df$pstar <- ifelse(
        cor2var_df$p.value < 0.05,
        ifelse(
          cor2var_df$p.value < 0.001,
          "***",
          ifelse(cor2var_df$p.value < 0.01, "**", "*")
        ),
        ""
      )
    }
    return(cor2var_df)
  }

  if (split == FALSE) {
    sss <- ss
    dd <-
      psych::corr.test(
        as.numeric(sss[, var1]),
        as.numeric(sss[, var2]),
        method = cor_method,
        adjust = adjust_method,
        use =  use
      )
    ddd <-
      data.frame(
        cor = dd$r,
        p.value = dd$p,
        method = cor_method,
        adjust = adjust_method,
        v1 = var1,
        v2 = var2,
        stringsAsFactors = F
      )

    cor2var_df <- ddd

    if (sig_label == TRUE) {
      cor2var_df$pstar <- ifelse(
        cor2var_df$p.value < 0.05,
        ifelse(
          cor2var_df$p.value < 0.001,
          "***",
          ifelse(cor2var_df$p.value < 0.01, "**", "*")
        ),
        ""
      )
    }
    return(cor2var_df)
  }
}

#' Run correlation between two variables in a batch mode and support group by a variable
#' @inheritParams ezcor
#' @param parallel if `TRUE`, do parallel computation by **furrr** package.
#' @return a `data.frame`
#' @author Yi Xiong, Shixiang Wang
#' @export

ezcor_batch <- function(data,
                        var1,
                        var2,
                        split = FALSE,
                        split_var = NULL,
                        cor_method = "pearson",
                        adjust_method = "none",
                        use = "complete",
                        sig_label = TRUE,
                        parallel = FALSE,
                        verbose = FALSE) {
  stopifnot(is.data.frame(data))

  ss <- data

  if (!var1 %in% colnames(ss)) {
    stop("the first variable is unavailable in the dataset!")
  }
  if (!var2 %in% colnames(ss)) {
    stop("the second variable is unavailable in the dataset!")
  }
  if (length(var1) != 1) {
    stop("only one element is needed in the var1 variable!")
  }
  if (split == TRUE) {
    if (!split_var %in% colnames(ss)) {
      stop("split variable is unavailable in the dataset!")
    }
    all_cols <- unique(c(var1, var2, split_var))

    ss <- ss[, all_cols]
    if (parallel) {
      if (!requireNamespace("furrr")) {
        stop("Please install 'furrr' package firstly!")
      }
      if (length(var2) < 200) {
        if (verbose) {
          message("Warning: variable < 200, parallel computation is not recommended!")
        }
      }

      oplan <- future::plan()
      future::plan("multiprocess")
      on.exit(future::plan(oplan), add = TRUE)
      res <- furrr::future_map(
        var2,
        ezcor_caller,
        data = ss,
        split_var = split_var,
        split = split,
        var1 = var1,
        cor_method = cor_method,
        adjust_method = adjust_method,
        use = use,
        sig_label = sig_label,
        verbose = verbose
      )
    } else {
      res <- purrr::map(
        var2,
        ezcor_caller,
        data = ss,
        split_var = split_var,
        split = split,
        var1 = var1,
        cor_method = cor_method,
        adjust_method = adjust_method,
        use = use,
        sig_label = sig_label,
        verbose = verbose
      ) %>% purrr::set_names(var2)
    }
    res2 <- dplyr::bind_rows(res)
    return(res2)
  } else {
    all_cols <- unique(c(var1, var2))
    ss <- ss[, all_cols]
    res <- purrr::map(
      var2,
      ezcor_caller,
      data = ss,
      split_var = split_var,
      split = split,
      var1 = var1,
      cor_method = cor_method,
      adjust_method = adjust_method,
      use = use,
      sig_label = sig_label,
      verbose = verbose
    ) %>% purrr::set_names(var2)
    res2 <- dplyr::bind_rows(res)
    return(res2)
  }
}

ezcor_caller <- function(var2,
                         data,
                         split_var = split_var,
                         split = split,
                         var1 = var1,
                         cor_method = cor_method,
                         adjust_method = adjust_method,
                         use = use,
                         sig_label = sig_label,
                         verbose = verbose) {
  ezcor(
    data = data,
    split_var = split_var,
    split = split,
    var1 = var1,
    var2 = var2,
    cor_method = cor_method,
    adjust_method = adjust_method,
    use = use,
    sig_label = sig_label,
    verbose = verbose
  )
}

#' Run partial correlation
#'
#' @seealso [ppcor::pcor.test()] which this function wraps.
#' @param data a `data.frame` containing variables
#' @param split whether perform correlation grouped by a variable, default is 'FALSE'
#' @param split_var a `character`, the group variable
#' @param var1 a `character`, the first variable in correlation
#' @param var2 a `character`, the second variable in correlation
#' @param var3 a `character` or `character vector`, the third variable in correlation
#' @param cor_method method="pearson" is the default value. The alternatives to be passed to cor are "spearman" and "kendall"
#' @param sig_label whether add symbal of significance. P < 0.001,"***"; P < 0.01,"**"; P < 0.05,"*"; P >=0.05,""
#' @param ... other arguments passed to methods
#' @import purrr
#' @export
#' @return a `data.frame`
#' @author Yi Xiong
#'
ezcor_partial_cor <- function(data = NULL,
                              split = FALSE,
                              split_var = NULL,
                              var1 = NULL,
                              var2 = NULL,
                              var3 = NULL,
                              cor_method = "pearson",
                              sig_label = TRUE,
                              ...) {
  if (!requireNamespace("ppcor")) {
    stop("Please install 'ppcor' package firstly!")
  }
  stopifnot(is.data.frame(data))
  ss <- data
  # var1 = x; var2 = y; var3 = z
  if (!var1 %in% colnames(ss)) {
    stop("the first variable is unavailable in the dataset!")
  }
  if (!var2 %in% colnames(ss)) {
    stop("the second variable is unavailable in the dataset!")
  }
  if (length(var1) != 1) {
    stop("only one element is needed in the first variable!")
  }
  if (length(var2) != 1) {
    stop("only one element is needed in the second variable!")
  }
  if (split == TRUE) {
    if (!split_var %in% colnames(ss)) {
      stop("split variable is unavailable in the dataset!")
    }
    # index
    n <- which(colnames(ss) %in% split_var)
    sss <- with(ss, split(ss, ss[, n]))
    s <- names(sss)
    ## calculate correlation
    cor2var <- purrr::map(s, purrr::safely(function(x) {
      # x = s[1]
      sss_sub <- sss[[x]]
      if (length(var3) > 1) {
        dd <-
          ppcor::pcor.test(
            as.numeric(sss_sub[, var1]),
            as.numeric(sss_sub[, var2]),
            as.matrix(sss_sub[, var3]),
            method = cor_method,
            ...
          )
        ddd <-
          data.frame(
            cor_partial = dd$estimate,
            p.value = dd$p.value,
            method = cor_method,
            x = var1,
            y = var2,
            z = paste(var3, collapse = ","),
            stringsAsFactors = F
          )
        ddd$group <- x
        return(ddd)
      } else {
        dd <-
          ppcor::pcor.test(as.numeric(sss_sub[, var1]),
            as.numeric(sss_sub[, var2]),
            as.numeric(sss_sub[, var3]),
            method = cor_method
          )
        ddd <-
          data.frame(
            cor_partial = dd$estimate,
            p.value = dd$p.value,
            method = cor_method,
            x = var1,
            y = var2,
            z = var3,
            stringsAsFactors = F
          )
        ddd$group <- x
        return(ddd)
      }
    })) %>% purrr::set_names(s)

    cor2var <- cor2var %>%
      purrr::map(~ .x$result) %>%
      purrr::compact()
    cor2var_df <- do.call(rbind.data.frame, cor2var)

    if (sig_label == TRUE) {
      cor2var_df$pstar <- ifelse(
        cor2var_df$p.value < 0.05,
        ifelse(
          cor2var_df$p.value < 0.001,
          "***",
          ifelse(cor2var_df$p.value < 0.01, "**", "*")
        ),
        ""
      )
    }
    return(cor2var_df)
  }

  if (split == FALSE) {
    sss <- ss
    if (length(var3) > 1) {
      dd <-
        ppcor::pcor.test(as.numeric(sss[, var1]),
          as.numeric(sss[, var2]),
          as.matrix(sss[, var3]),
          method = cor_method,
          ...
        )
      ddd <-
        data.frame(
          cor_partial = dd$estimate,
          p.value = dd$p.value,
          method = cor_method,
          x = var1,
          y = var2,
          z = paste(var3, collapse = ","),
          stringsAsFactors = F
        )
      # ddd$group <- x
      # return(ddd)
    } else {
      dd <-
        ppcor::pcor.test(as.numeric(sss[, var1]),
          as.numeric(sss[, var2]),
          as.numeric(sss[, var3]),
          method = cor_method
        )
      ddd <-
        data.frame(
          cor_partial = dd$estimate,
          p.value = dd$p.value,
          method = cor_method,
          x = var1,
          y = var2,
          z = var3,
          stringsAsFactors = F
        )
      # ddd$group <- x
      # return(ddd)
    }

    cor2var_df <- ddd

    if (sig_label == TRUE) {
      cor2var_df$pstar <- ifelse(
        cor2var_df$p.value < 0.05,
        ifelse(
          cor2var_df$p.value < 0.001,
          "***",
          ifelse(cor2var_df$p.value < 0.01, "**", "*")
        ),
        ""
      )
    }
    return(cor2var_df)
  }
}

#' Show Available Hosts
#'
#' @return hosts
#' @export
#'
#' @examples
#' available_hosts()
available_hosts <- function() {
  .xena_hosts <- "UCSCXenaTools" %:::% ".xena_hosts"
  .xena_hosts %>% as.character()
}


#' Fetch Identifier Value from Pan-cancer Dataset
#'
#' Identifier includes gene/probe etc.
#'
#' @param identifier a length-1 character representing a gene symbol, ensembl gene id, or probe id.
#' Gene symbol is highly recommended.
#' @param subtype a length-1 chracter representing a regular expression for matching
#' `DataSubtype` column of [UCSCXenaTools::XenaData].
#' @param dataset a length-1 chracter representing a regular expression for matching
#' `XenaDatasets` of [UCSCXenaTools::XenaData].
#' @param host a character vector representing host name(s), e.g. "toilHub".
#' @param samples a character vector representing samples want to be returned.
#' @param ... other parameters.
#' @return a named vector or `list`.
#'
#' @examples
#' \dontrun{
#' # Fetch TP53 expression value from pan-cancer dataset
#' t1 <- get_pancan_value("TP53",
#'   dataset = "TcgaTargetGtex_rsem_isoform_tpm",
#'   host = "toilHub"
#' )
#' t2 <- get_pancan_gene_value("TP53")
#' t3 <- get_pancan_protein_value("AKT")
#' t4 <- get_pancan_mutation_status("TP53")
#' t5 <- get_pancan_cn_value("TP53")
#' }
#'
#' @export
#' @describeIn get_pancan_value Fetch identifier value from pan-cancer dataset
get_pancan_value <- function(identifier, subtype = NULL, dataset = NULL, host = available_hosts(),
                             samples = NULL, ...) {
  stopifnot(length(identifier) == 1, !all(is.null(subtype), is.null(dataset)))
  if (!requireNamespace("UCSCXenaTools", quietly = TRUE)) {
    stop("Package 'UCSCXenaTools' is not installed.")
  }
  if (!"UCSCXenaTools" %in% .packages()) {
    attachNamespace("UCSCXenaTools")
  }
  host <- match.arg(host, choices = available_hosts(), several.ok = TRUE)

  data <- UCSCXenaTools::XenaData
  if (!is.null(subtype)) {
    data <- data %>%
      dplyr::filter(XenaHostNames %in% host, grepl(subtype, DataSubtype))
  }
  if (!is.null(dataset)) {
    data <- data %>%
      dplyr::filter(XenaHostNames %in% host, grepl(dataset, XenaDatasets))
  }

  if (nrow(data) == 0) {
    stop("No dataset is determined by input")
  }

  if (nrow(data) > 1) {
    data <- dplyr::filter(data, .data$XenaDatasets == dataset)
  }

  res <- try_query_value(data[["XenaHosts"]], data[["XenaDatasets"]],
    identifiers = identifier, samples = samples,
    check = FALSE, use_probeMap = TRUE, ...
  )
  res2 <- res[1, ]
  names(res2) <- colnames(res)
  res2
}

## try solving internet connection problem
try_query_value <- function(host, dataset,
                            identifiers, samples,
                            check = FALSE, use_probeMap = TRUE,
                            rule_out = NULL,
                            aggr = c("NA", "mean", "Q0", "Q25", "Q50", "Q75", "Q100"),
                            max_try = 5L) {
  aggr <- match.arg(aggr)
  Sys.sleep(0.05)

  tryCatch(
    {
      message("Try querying data #", abs(max_try - 6L))

      if (aggr != "NA") {
        # Designed for methylation data
        # be careful!
        message("Fetching all ids, take patience...")
        # ids = UCSCXenaTools::fetch_dataset_identifiers(host, dataset)
        ## Download probeMap time-cost
        # xe = UCSCXenaTools::XenaQueryProbeMap(UCSCXenaTools::XenaGenerate(subset = XenaDatasets == dataset))
        # xd = UCSCXenaTools::XenaPrepare(UCSCXenaTools::XenaDownload(xe), col_names = FALSE)[, c(1, 2)]
        # xd = tidyr::separate_rows(xd, "X2", sep = ",")

        ## Use the prepared ID referrence data
        id_ref <- load_data("pancan_identifier_help")[["id_molecule"]]
        if (grepl("450", dataset)) {
          xd <- id_ref[["id_M450"]]
        } else {
          xd <- id_ref[["id_M27K"]]
        }
        # xd = ifelse(grepl("450", dataset), id_ref[["id_M450"]], id_ref[["id_M27K"]])
        xd <- xd %>%
          dplyr::select("CpG", "Level3") %>%
          dplyr::rename("X1" = "CpG", "X2" = "Level3")

        xd <- dplyr::filter(xd, .data$X2 %in% identifiers)

        if (!is.null(rule_out)) {
          xd <- dplyr::filter(xd, !.data$X1 %in% rule_out) # X2 → X1
        }
        ids <- xd$X1

        d <- UCSCXenaTools::fetch_dense_values(host, dataset,
          identifiers = ids, samples = samples,
          check = FALSE, use_probeMap = FALSE
        )
        f <- switch(aggr,
          mean = function(x) mean(x, na.rm = TRUE),
          Q0 = function(x) quantile(x, 0, na.rm = TRUE),
          Q25 = function(x) quantile(x, 0.25, na.rm = TRUE),
          Q50 = function(x) quantile(x, 0.5, na.rm = TRUE),
          Q75 = function(x) quantile(x, 0.75, na.rm = TRUE),
          Q100 = function(x) quantile(x, 1, na.rm = TRUE)
        )

        z <- apply(d, 2, f)
        matrix(z, nrow = 1, dimnames = list("aggr_methy_value", names(z)))
      } else {
        UCSCXenaTools::fetch_dense_values(host, dataset,
          identifiers = identifiers, samples = samples,
          check = check, use_probeMap = use_probeMap
        )
      }
    },
    error = function(e) {
      if (max_try == 1) {
        warning("Tried 5 times but failed, please check URL or your internet connection or try it later!", immediate. = TRUE)
        return(NULL)
      } else {
        try_query_value(host, dataset,
          identifiers, samples,
          check = check, use_probeMap = use_probeMap,
          max_try = max_try - 1L,
          rule_out = rule_out,
          aggr = aggr
        )
      }
    }
  )
}


# Toil/TCGA ---------------------------------------------------------------

#' @describeIn get_pancan_value Fetch gene expression value from pan-cancer dataset
#' @export
get_pancan_gene_value <- function(identifier, norm = c("tpm", "fpkm", "nc")) {
  host <- "toilHub"

  norm <- match.arg(norm)
  dataset <- switch(norm,
    "tpm" = "TcgaTargetGtex_rsem_gene_tpm",
    "fpkm" = "TcgaTargetGtex_rsem_gene_fpkm",
    "nc" = "TcgaTargetGtex_RSEM_Hugo_norm_count"
  )
  # dataset <- "TcgaTargetGtex_rsem_gene_tpm"

  expression <- get_data(dataset, identifier, host)
  unit <- switch(norm,
    "tpm" = "log2(tpm+0.001)",
    "fpkm" = "log2(fpkm+0.001)",
    "nc" = "log2(norm_count+1)"
  )
  report_dataset_info(dataset)
  res <- list(expression = expression, unit = unit)
  res
}


#' @describeIn get_pancan_value Fetch gene transcript expression value from pan-cancer dataset
#' @export
get_pancan_transcript_value <- function(identifier, norm = c("tpm", "fpkm", "isopct")) {
  # ENST00000000233
  id <- identifier
  host <- "toilHub"

  norm <- match.arg(norm)

  dataset <- switch(norm,
    "tpm" = "TcgaTargetGtex_rsem_isoform_tpm",
    "fpkm" = "TcgaTargetGtex_RSEM_isoform_fpkm",
    "isopct" = "TcgaTargetGtex_rsem_isopct"
  )

  res_p <- check_exist_data("mp", dataset, host)
  if (res_p$ok) {
    ids <- res_p$data
  } else {
    ids <- UCSCXenaTools::fetch_dataset_identifiers("https://toil.xenahubs.net", dataset)
    names(ids) <- sub("\\.[0-9]+", "", ids)
    save_data(ids, "mp", dataset, host)
  }

  identifier <- as.character(ids[identifier])
  if (is.na(identifier)) identifier <- id # roll back

  expression <- get_data(dataset, identifier, host)
  unit <- switch(norm,
    "tpm" = "log2(tpm+0.001)",
    "fpkm" = "log2(fpkm+0.001)",
    "isopct" = "IsoPct"
  )
  report_dataset_info(dataset)
  res <- list(expression = expression, unit = unit)
  res
}

#' @describeIn get_pancan_value Fetch protein expression value from pan-cancer dataset
#' @export
get_pancan_protein_value <- function(identifier) {
  ## NOTE: Only ~200 proteins available, so many identifiers will return NAs
  host <- "pancanAtlasHub"
  dataset <- "TCGA-RPPA-pancan-clean.xena"

  expression <- get_data(dataset, identifier, host)

  unit <- "norm_value"
  report_dataset_info(dataset)
  res <- list(expression = expression, unit = unit)
  res
}

## .p_dataset_field("https://pancanatlas.xenahubs.net", "TCGA-RPPA-pancan-clean.xena")
.all_pancan_proteins <- c(
  "ACC1", "ACC_pS79", "ACETYLATUBULINLYS40", "ACVRL1", "ADAR1",
  "AKT", "AKT_pS473", "AKT_pT308", "ALPHACATENIN", "AMPKALPHA",
  "AMPKALPHA_pT172", "ANNEXIN1", "ANNEXINVII", "AR", "ARAF", "ARAF_pS299",
  "ARID1A", "ASNS", "ATM", "AXL", "BAD_pS112", "BAK", "BAP1C4",
  "BAX", "BCL2", "BCL2A1", "BCLXL", "BECLIN", "BETACATENIN", "BID",
  "BIM", "BRAF", "BRAF_pS445", "BRCA2", "BRD4", "CA9", "CABL",
  "CASPASE3", "CASPASE7CLEAVEDD198", "CASPASE8", "CASPASE9", "CAVEOLIN1",
  "CD20", "CD26", "CD31", "CD49B", "CDK1", "CDK1_pY15", "CHK1",
  "CHK1_pS296", "CHK1_pS345", "CHK2", "CHK2_pT68", "CHROMOGRANINANTERM",
  "CIAP", "CJUN_pS73", "CK5", "CKIT", "CLAUDIN7", "CMET", "CMET_pY1235",
  "CMYC", "COG3", "COLLAGENVI", "COMPLEXIISUBUNIT30", "CRAF", "CRAF_pS338",
  "CTLA4", "CYCLINB1", "CYCLIND1", "CYCLINE1", "CYCLINE2", "DIRAS3",
  "DJ1", "DUSP4", "DVL3", "E2F1", "ECADHERIN", "EEF2", "EEF2K",
  "EGFR", "EGFR_pY1068", "EGFR_pY1173", "EIF4E", "EIF4G", "ENY2",
  "EPPK1", "ERALPHA", "ERALPHA_pS118", "ERCC1", "ERCC5", "ERK2",
  "ETS1", "EZH2", "FASN", "FIBRONECTIN", "FOXM1", "FOXO3A", "FOXO3A_pS318S321",
  "G6PD", "GAB2", "GAPDH", "GATA3", "GATA6", "GCN5L2", "GSK3ALPHABETA",
  "GSK3ALPHABETA_pS21S9", "GSK3_pS9", "GYGGLYCOGENIN1", "GYS",
  "GYS_pS641", "HER2", "HER2_pY1248", "HER3", "HER3_pY1289", "HEREGULIN",
  "HIF1ALPHA", "HSP70", "IGF1R_pY1135Y1136", "IGFBP2", "INPP4B",
  "IRF1", "IRS1", "JAB1", "JAK2", "JNK2", "JNK_pT183Y185", "KEAP1",
  "KU80", "LCK", "LCN2A", "LDHA", "LDHB", "LKB1", "MACC1", "MAPK_pT202Y204",
  "MEK1", "MEK1_pS217S221", "MIG6", "MITOCHONDRIA", "MRE11", "MSH2",
  "MSH6", "MTOR", "MTOR_pS2448", "MYH11", "MYOSINIIA", "MYOSINIIA_pS1943",
  "NAPSINA", "NCADHERIN", "NDRG1_pT346", "NF2", "NFKBP65_pS536",
  "NOTCH1", "NRAS", "NRF2", "OXPHOSCOMPLEXVSUBUNITB", "P16INK4A",
  "P21", "P27", "P27_pT157", "P27_pT198", "P38MAPK", "P38_pT180Y182",
  "P53", "P62LCKLIGAND", "P63", "P70S6K1", "P70S6K_pT389", "P90RSK",
  "P90RSK_pT359S363", "PAI1", "PARP1", "PARPAB3", "PARPCLEAVED",
  "PAXILLIN", "PCADHERIN", "PCNA", "PDCD1", "PDCD4", "PDK1", "PDK1_pS241",
  "PDL1", "PEA15", "PEA15_pS116", "PI3KP110ALPHA", "PI3KP85", "PKCALPHA",
  "PKCALPHA_pS657", "PKCDELTA_pS664", "PKCPANBETAII_pS660", "PKM2",
  "PR", "PRAS40_pT246", "PRDX1", "PREX1", "PTEN", "PYGB", "PYGBAB2",
  "PYGL", "PYGM", "RAB11", "RAB25", "RAD50", "RAD51", "RAPTOR",
  "RB", "RBM15", "RB_pS807S811", "RET_pY905", "RICTOR", "RICTOR_pT1135",
  "S6", "S6_pS235S236", "S6_pS240S244", "SCD1", "SETD2", "SF2",
  "SHC_pY317", "SHP2_pY542", "SLC1A5", "SMAC", "SMAD1", "SMAD3",
  "SMAD4", "SNAIL", "SRC", "SRC_pY416", "SRC_pY527", "STAT3_pY705",
  "STAT5ALPHA", "STATHMIN", "SYK", "SYNAPTOPHYSIN", "TAZ", "TFRC",
  "THYMIDILATESYNTHASE", "TIGAR", "TRANSGLUTAMINASE", "TSC1", "TTF1",
  "TUBERIN", "TUBERIN_pT1462", "VEGFR2", "X1433BETA", "X1433EPSILON",
  "X1433ZETA", "X4EBP1", "X4EBP1_pS65", "X4EBP1_pT37T46", "X4EBP1_pT70",
  "X53BP1", "XBP1", "XRCC1", "YAP", "YAP_pS127", "YB1", "YB1_pS102"
)

#' @describeIn get_pancan_value Fetch mutation status value from pan-cancer dataset
#' @export
get_pancan_mutation_status <- function(identifier) {
  if (utils::packageVersion("UCSCXenaTools") < "1.3.2") {
    stop("You need to update 'UCSCXenaTools' (>=1.3.2).", call. = FALSE)
  }

  host <- "pancanAtlasHub"
  dataset <- "mc3.v0.2.8.PUBLIC.nonsilentGene.xena"
  report_dataset_info(dataset)

  data <- get_data(dataset, identifier, host)

  return(data)
}

#' @param gistic2 if `TRUE` (default), use GISTIC2 data.
#' @param use_thresholded_data if `TRUE`, use GISTIC2-thresholded value.
#' @describeIn get_pancan_value Fetch gene copy number value from pan-cancer dataset processed by GISTIC 2.0
#' @export
get_pancan_cn_value <- function(identifier, gistic2 = TRUE, use_thresholded_data = FALSE) {
  if (gistic2 & use_thresholded_data) {
    host <- "tcgaHub"
    dataset <- "TCGA.PANCAN.sampleMap/Gistic2_CopyNumber_Gistic2_all_thresholded.by_genes"
    unit <- "-2,-1,0,1,2: 2 copy del,1 copy del,no change,amplification,high-amplification"
  } else if (gistic2 & !use_thresholded_data) {
    host <- "tcgaHub"
    dataset <- "TCGA.PANCAN.sampleMap/Gistic2_CopyNumber_Gistic2_all_data_by_genes"
    unit <- "Gistic2 copy number"
  } else if (!gistic2) {
    host <- "pancanAtlasHub"
    dataset <- "broad.mit.edu_PANCAN_Genome_Wide_SNP_6_whitelisted.gene.xena"
    unit <- "log(tumor/normal)"
  }

  data <- get_data(dataset, identifier, host)
  report_dataset_info(dataset)
  res <- list(data = data, unit = unit)
  res
}


#' @describeIn get_pancan_value Fetch gene expression value from CCLE dataset
#' @param type methylation type, one of "450K" and "27K".
#' for function `get_pcawg_promoter_value`, it can be one of
#' "raw", "relative", "outlier".
#' @param rule_out methylation sites to rule out before analyzing.
#' @param aggr apporaches to aggregate the methylation data, default is 'NA',
#' in such case, a mean value is obtained for gene-level methylation.
#' Allowed value is one of `c("NA", "mean", "Q0", "Q25", "Q50", "Q75", "Q100")`.
#' Here, `Q50` is median.
#' @export
get_pancan_methylation_value <- function(identifier, type = c("450K", "27K"),
                                         rule_out = NULL,
                                         aggr = c("NA", "mean", "Q0", "Q25", "Q50", "Q75", "Q100")) {
  type <- match.arg(type)
  aggr <- match.arg(aggr)

  if (type == "450K") {
    host <- "pancanAtlasHub"
    dataset <- "jhu-usc.edu_PANCAN_HumanMethylation450.betaValue_whitelisted.tsv.synapse_download_5096262.xena"
    # host <- "gdcHub"
    # dataset <- "GDC-PANCAN.methylation450.tsv"
  } else {
    host <- "tcgaHub"
    dataset <- "TCGA.PANCAN.sampleMap/HumanMethylation27"
    # host <- "gdcHub"
    # dataset <- "GDC-PANCAN.methylation27.tsv"
  }

  data <- get_data(dataset, identifier, host, rule_out = rule_out, aggr = aggr)
  ## 去重，优先取No.16位字符为A的情况
  data <- sort(data)
  names(data) <- substr(names(data), 1, 15)
  data <- data[!duplicated(names(data))]

  unit <- "beta value"
  report_dataset_info(dataset)
  res <- list(data = data, unit = unit)
  res
}

#' @describeIn get_pancan_value Fetch miRNA expression value from pan-cancer dataset
#' @export
get_pancan_miRNA_value <- function(identifier) {
  host <- "pancanAtlasHub"
  dataset <- "pancanMiRs_EBadjOnProtocolPlatformWithoutRepsWithUnCorrectMiRs_08_04_16.xena"

  expression <- get_data(dataset, identifier, host)

  unit <- "log2(norm_value+1)"
  report_dataset_info(dataset)
  res <- list(expression = expression, unit = unit)
  res
}

# Utils -------------------------------------------------------------------

report_dataset_info <- function(dataset) {
  msg <- paste0(
    "More info about dataset please run following commands:\n",
    "  library(UCSCXenaTools)\n",
    "  XenaGenerate(subset = XenaDatasets == \"", dataset, "\") %>% XenaBrowse()"
  )
  message(msg)
}

check_file <- function(id, dataset, host, ...) {
  f <- file.path(
    get_cache_dir(),
    # paste0(
    #   host, "_",
    #   gsub("[/]", "_", dataset),
    #   "_", id, ".rds"
    # ),
    paste0(digest::digest(list(id, dataset, host, ...)), ".rds")
  )
  return(f)
}

check_exist_data <- function(id, dataset, host, ...) {
  f <- check_file(id, dataset, host, ...)
  if (file.exists(f)) {
    message("Reading cache data ", f)
    data <- tryCatch(
      readRDS(f),
      error = function(e) {
        message("Read cache data failed.")
        return(NULL)
      }
    )
    if (!is.null(data)) {
      return(list(
        ok = TRUE,
        data = data
      ))
    } else {
      return(
        list(
          ok = FALSE,
          data = NULL
        )
      )
    }
  } else {
    return(
      list(
        ok = FALSE,
        data = NULL
      )
    )
  }
}

save_data <- function(data, id, dataset, host, ...) {
  f <- check_file(id, dataset, host, ...)
  if (!dir.exists(dirname(f))) {
    dir.create(dirname(f), recursive = TRUE)
  }

  message("Saving data to file ", f)

  tryCatch(
    saveRDS(data, file = f),
    error = function(e) {
      message("Save data to cache directory failed.")
    }
  )
}

get_data <- function(dataset, identifier, host = NULL, ...) {
  stopifnot(length(dataset) == 1)

  if (is.null(host)) {
    host <- UCSCXenaTools::XenaData %>%
      dplyr::filter(.data$XenaDatasets == dataset) %>%
      dplyr::pull(.data$XenaHostNames)
  }
  res <- check_exist_data(identifier, dataset, host, ...)
  if (res$ok) {
    value <- res$data
  } else {
    value <- get_pancan_value(identifier, dataset = dataset, host = host, ...)
    label <- UCSCXenaTools::XenaData %>%
      dplyr::filter(.data$XenaDatasets == dataset) %>%
      dplyr::pull(.data$DataSubtype)
    if (!is.null(value)) attr(value, "label") <- label
    save_data(value, identifier, dataset, host, ...)
  }
  value
}

#' Visualize molecular profile difference between mutation and wild status of queried gene
#'
#' @param mut_Gene the queried gene to determine grouping based on mutation and wild status
#' @param Gene a molecular identifier (e.g., "TP53") or a formula specifying
#' genomic signature (`"TP53 + 2 * KRAS - 1.3 * PTEN"`).
#' @param data_type choose gene profile type,
#' including "mRNA", "transcript", "methylation", "miRNA".
#' @param Mode choose one visualize mode to represent data
#' @param Show.P.value  `TRUE` or `FALSE` whether to count P value
#' @param Show.P.label `TRUE` or `FALSE` present p value with number or label `*`, `**`, `***` and `****`
#' @param Method default method is wilcox.test
#' @param values the color to fill mutation or wild status
#' @param draw_quantiles draw quantiles for violinplot
#' @param trim whether to trim the violin
#' @param opt_pancan specify one dataset for some molercular profiles

#' @return a `ggplot` object or a tibble data.frame

#'
#' @examples
#' \dontrun{
#' p <- vis_toil_Mut(mut_Gene = "TP53")
#' p <- vis_toil_Mut(mut_Gene = "TP53", Gene = "TNF")
#' p <- vis_toil_Mut(mut_Gene = "TP53", Gene = "hsa-let-7d-3p", data_type = "miRNA")
#' }
#' @export

vis_toil_Mut <- function(mut_Gene = "TP53", Gene = NULL, data_type = NULL,
                         Mode = c("Boxplot", "Violinplot"),
                         Show.P.value = TRUE, Show.P.label = TRUE,
                         Method = c("wilcox.test", "t.test"),
                         values = c("#DF2020", "#DDDF21"),
                         draw_quantiles = c(0.25, 0.5, 0.75),
                         trim = TRUE, opt_pancan = .opt_pancan) {
  Mode <- match.arg(Mode)
  Method <- match.arg(Method)

  # tumor samples grouping based on mutation
  mut_dat_raw <- query_pancan_value(mut_Gene, data_type = "mutation")
  if (all(is.na(mut_dat_raw))) {
    stop("For the gene(", mut_Gene, ") mutation, all NAs returned.")
  }
  tcga_gtex <- load_data("tcga_gtex")
  mut_dat <- mut_dat_raw %>%
    as.data.frame() %>%
    tibble::rownames_to_column("sample") %>%
    dplyr::rename("mut" = ".") %>%
    dplyr::inner_join(tcga_gtex, .) %>%
    dplyr::filter(.data$type2 == "tumor") %>%
    dplyr::select("sample", "tissue", "mut")
  # affected molecular profile
  if (is.null(Gene)) Gene <- mut_Gene
  if (is.null(data_type)) data_type <- "mRNA"
  if (!data_type %in% c("mRNA", "miRNA", "transcript", "methylation")) {
    stop("data_type ", data_type, " does not support in this function!")
  }
  exp_dat_raw <- query_pancan_value(Gene, data_type = data_type, opt_pancan = opt_pancan)
  if (all(is.na(exp_dat_raw$expression))) {
    message("All NAs returned, return NULL instead.")
    return(NULL)
  }
  unit <- exp_dat_raw$unit
  exp_dat <- exp_dat_raw$expression %>%
    as.data.frame() %>%
    tibble::rownames_to_column("sample") %>%
    dplyr::rename("expression" = ".") %>%
    dplyr::inner_join(tcga_gtex, .) %>%
    dplyr::select("sample", "expression")

  # merge data
  merge_dat <- dplyr::inner_join(mut_dat, exp_dat) %>%
    dplyr::mutate(
      tissue = as.character(.data$tissue),
      mut = factor(.data$mut, levels = c(1, 0), labels = c("Mutation", "Wild"))
    )
  group_stat <- table(merge_dat$tissue, merge_dat$mut)
  tcga_discard <- rownames(group_stat)[apply(group_stat, 1, min) < 3]
  if (length(tcga_discard) == length(group_stat)) {
    stop("No one valid cancer type has enough grouping samples (<3) for comparsion.")
  } else {
    message(
      paste(tcga_discard, collapse = " "), " was/were discarded due to less than ",
      3, " samples in one group."
    )
  }
  merge_dat <- merge_dat %>%
    dplyr::filter(!(.data$tissue %in% tcga_discard)) %>%
    dplyr::mutate(mut = paste0(.data$mut, "(", mut_Gene, ")"))


  # calculate p value significantce
  tmp_dat <- merge_dat %>%
    dplyr::group_by(.data$tissue, .data$mut) %>%
    dplyr::summarise(expression = mean(.data$expression)) %>%
    tidyr::pivot_wider(names_from = "mut", values_from = "expression")

  pv <- merge_dat %>%
    ggpubr::compare_means(expression ~ mut, data = ., method = Method, group.by = "tissue") %>%
    dplyr::select(c("tissue", "p", "p.signif", "p.adj")) %>%
    dplyr::mutate(Gene = Gene, .before = 1) %>%
    dplyr::arrange(p) %>%
    dplyr::inner_join(tmp_dat)

  # if (!plot) {
  #   return(pv)
  # }

  # visualization
  if (Mode == "Boxplot") {
    p <- ggplot2::ggplot(merge_dat, aes_string(x = "tissue", y = "expression", fill = "mut")) +
      ggplot2::geom_boxplot()
  } else if (Mode == "Violinplot") {
    p <- ggplot2::ggplot(
      merge_dat, aes_string(x = "tissue", y = "expression", fill = "mut")
    ) +
      geom_split_violin(
        draw_quantiles = draw_quantiles,
        trim = trim,
        linetype = "solid",
        color = "black",
        size = 0.2,
        na.rm = TRUE,
        position = "identity"
      )
  }

  p <- p +
    ggplot2::xlab("") +
    ggplot2::ylab(if (is.null(unit)) Gene else paste0(Gene, " (", unit, ")")) +
    ggplot2::scale_fill_manual(values = values) +
    ggplot2::theme_set(ggplot2::theme_classic(base_size = 20)) +
    ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 45, hjust = .5, vjust = .5)) +
    ggplot2::guides(fill = ggplot2::guide_legend(title = NULL)) +
    ggplot2::theme(
      legend.background = ggplot2::element_blank(),
      legend.position = c(0, 0), legend.justification = c(0, 0)
    )

  if (Show.P.value == TRUE & Show.P.label == TRUE) {
    p <- p + ggplot2::geom_text(
      aes(
        x = .data$tissue,
        y = max(merge_dat$expression) * 1.1,
        label = .data$p.signif
      ),
      data = pv,
      inherit.aes = FALSE
    )
  }

  if (Show.P.value == TRUE & Show.P.label == FALSE) {
    p <- p + ggplot2::geom_text(
      aes(
        x = .data$tissue,
        y = max(merge_dat$expression) * 1.1,
        label = as.character(signif(.data$p, 2))
      ),
      data = pv,
      inherit.aes = FALSE
    )
  }

  return(p)
}




#' Visualize molecular profile difference between mutation and wild status of queried gene in Single Cancer Type
#' @inheritParams vis_toil_Mut
#' @param Cancer select cancer cohort(s).
#' @return a `ggplot` object or a tibble data.frame.
#' @export
#'
vis_toil_Mut_cancer <- function(mut_Gene = "TP53", Gene = NULL, data_type = NULL,
                                Mode = c("Dotplot", "Violinplot"),
                                Show.P.value = TRUE, Show.P.label = TRUE,
                                Method = c("wilcox.test", "t.test"),
                                values = c("#DF2020", "#DDDF21"),
                                draw_quantiles = c(0.25, 0.5, 0.75),
                                trim = TRUE, Cancer = "ACC", opt_pancan = .opt_pancan) {
  Mode <- match.arg(Mode)
  Method <- match.arg(Method)

  # tumor samples grouping based on mutation
  mut_dat_raw <- query_pancan_value(mut_Gene, data_type = "mutation")
  if (all(is.na(mut_dat_raw))) {
    stop("For the gene(", mut_Gene, ") mutation, all NAs returned.")
  }
  tcga_gtex <- load_data("tcga_gtex")
  mut_dat <- mut_dat_raw %>%
    as.data.frame() %>%
    tibble::rownames_to_column("sample") %>%
    dplyr::rename("mut" = ".") %>%
    dplyr::inner_join(tcga_gtex, .) %>%
    dplyr::filter(.data$type2 == "tumor") %>%
    dplyr::select("sample", "tissue", "mut")
  # affected molecular profile
  if (is.null(Gene)) Gene <- mut_Gene
  if (is.null(data_type)) data_type <- "mRNA"
  if (!data_type %in% c("mRNA", "miRNA", "transcript", "methylation")) {
    stop("data_type ", data_type, " does not support in this function!")
  }
  exp_dat_raw <- query_pancan_value(Gene, data_type = data_type)
  if (all(is.na(exp_dat_raw$expression))) {
    message("All NAs returned, return NULL instead.")
    return(NULL)
  }
  unit <- exp_dat_raw$unit
  exp_dat <- exp_dat_raw$expression %>%
    as.data.frame() %>%
    tibble::rownames_to_column("sample") %>%
    dplyr::rename("expression" = ".") %>%
    dplyr::inner_join(tcga_gtex, .) %>%
    dplyr::select("sample", "expression")

  # merge data
  merge_dat <- dplyr::inner_join(mut_dat, exp_dat) %>%
    dplyr::mutate(
      tissue = as.character(.data$tissue),
      mut = factor(.data$mut, levels = c(1, 0), labels = c("Mutation", "Wild"))
    ) %>%
    dplyr::filter(.data$tissue == Cancer) %>%
    dplyr::mutate(mut = paste0(.data$mut, "(", mut_Gene, ")"))


  group_stat <- table(merge_dat$mut)
  if (min(group_stat) < 3) {
    stop("The cancer type does not has enough grouping samples (<3) for comparsion.")
  }


  # calculate p value significantce
  tmp_dat <- merge_dat %>%
    dplyr::group_by(.data$tissue, .data$mut) %>%
    dplyr::summarise(expression = mean(.data$expression)) %>%
    tidyr::pivot_wider(names_from = "mut", values_from = "expression")

  pv <- merge_dat %>%
    ggpubr::compare_means(expression ~ mut, data = ., method = Method, group.by = "tissue") %>%
    dplyr::select(c("tissue", "p", "p.signif")) %>%
    dplyr::mutate(Gene = Gene, .before = 1) %>%
    dplyr::arrange(p) %>%
    dplyr::inner_join(tmp_dat)

  # if (!plot) {
  #   return(pv)
  # }

  if (Mode == "Dotplot") {
    p <- ggpubr::ggdotplot(
      merge_dat,
      x = "mut", y = "expression", fill = "mut", color = "mut", size = 0.6, binwidth = 0.2
    )
  } else if (Mode == "Violinplot") {
    p <- ggplot2::ggplot(
      merge_dat,
      aes_string(x = "mut", y = "expression", fill = "mut")
    ) +
      ggplot2::geom_violin(trim = FALSE) +
      ggplot2::geom_boxplot(width = 0.1, fill = "white")
  }
  p <- p +
    ggplot2::xlab(NULL) +
    ggplot2::ylab(if (is.null(unit)) Gene else paste0(Gene, " (", unit, ")")) +
    ggplot2::ggtitle(paste0("TCGA: ", Cancer)) +
    theme_set(theme_classic(base_size = 20)) +
    ggplot2::theme(axis.text.x = element_text(angle = 45, hjust = .5, vjust = .5, size = 20)) +
    ggplot2::theme(legend.position = "none") +
    ggplot2::scale_fill_manual(values = values) +
    ggplot2::scale_color_manual(values = values)

  if (Show.P.value == TRUE & Show.P.label == TRUE) {
    p <- p + ggplot2::geom_text(
      aes(
        x = 1.5,
        y = max(merge_dat$expression) * 1.1,
        label = .data$p.signif
      ),
      data = pv,
      inherit.aes = FALSE
    )
  }

  if (Show.P.value == TRUE & Show.P.label == FALSE) {
    p <- p + ggplot2::geom_text(
      aes(
        x = 1.5,
        y = max(merge_dat$expression) * 1.1,
        label = as.character(signif(.data$p, 2))
      ),
      data = pv,
      inherit.aes = FALSE
    )
  }
  return(p)
}

ui.modules_pancan_cor_m2o = function(id) {
	ns = NS(id)
	fluidPage(
		# 第一行：选择肿瘤及样本
		fluidRow(
			# 选择分组依据
			column(
				3,
				wellPanel(
					style = "height:1100px",
					h2("S1: Preset", align = "center"),

					h4(strong("S1.1 Modify datasets"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Modify datasets", 
					                   content = "data_origin"),
					mol_origin_UI(ns("mol_origin2cor"), database = "toil"),

					h4(strong("S1.2 Choose cancer")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Cancer types", 
					                   content = "tcga_types"),
					pickerInput(
						ns("choose_cancer"), NULL,
						choices = sort(tcga_names)),
					br(),

					h4(strong("S1.3 Filter samples"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Filter samples", 
					                   content = "choose_samples"),
					h5("Quick filter:"),
					pickerInput(
						ns("filter_by_code"), NULL,
						choices = NULL, selected =  NULL,
						multiple = TRUE, options = list(`actions-box` = TRUE)
					),
					h5("Exact filter:"),
					filter_samples_UI(ns("filter_samples2cor"), database = "toil"),
					br(),
					verbatimTextOutput(ns("filter_phe_id_info")),
					br(),
					
					h4(strong("S1.4 Upload metadata"),"[opt]") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Upload metadata", 
					                   content = "custom_metadata"),
					shinyFeedback::useShinyFeedback(),
					custom_meta_UI(ns("custom_meta2cor_batch")),
					br(),

					h4(strong("S1.5 Add signature"),"[opt]") %>% 
						helper(type = "markdown", size = "m", fade = TRUE, 
					                   title = "Add signature", 
					                   content = "add_signature"),
					add_signature_UI(ns("add_signature2cor"), database = "toil"),
				)
			),
			column(
				4,
				wellPanel(
					style = "height:1100px",
					h2("S2: Get data", align = "center"),
					# 批量数据下载
					h4(strong("S2.1 Get batch data for X-axis")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Get batch data", 
					                   content = "get_batch_data"), 
					multi_upload_UI(ns("multi_upload2cor"),button_name = "Query", database = "toil"),
					# br(),br(),
					# 单项数据下载
					h4(strong("S2.2 Get data for Y-axis")) %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Get one data", 
					                   content = "get_one_data"), 
					download_feat_UI(ns("download_y_axis"), button_name="Query", database = "toil")
				)
			),
			column(
				5,
				wellPanel(
					style = "height:1100px",
					h2("S3: Analyze", align = "center") %>% 
						helper(type = "markdown", size = "l", fade = TRUE, 
					                   title = "Analyze", 
					                   content = "analyze_cor_3"),  

					h4(strong("S3.1 Set analysis parameters")), 
					selectInput(ns("cor_method"),"Correlation method:",choices = c("spearman","pearson"), selected="Spearman"),
					shinyWidgets::actionBttn(
						ns("cal_batch_cor"), "Run",
				        style = "gradient",
				        icon = icon("table"),
				        color = "primary",
				        block = TRUE,
				        size = "sm"
					),
					br(),br(),
					fluidRow(
						column(10, offset = 1,
							   div(uiOutput(ns("cor_stat_tb.ui")),style = "height:600px"),
							   )
					),
					h4(strong("S3.2 Download results")), 
					download_res_UI(ns("download_res2cor"))
				)
			)
		)
	)
}


server.modules_pancan_cor_m2o = function(input, output, session) {
	ns <- session$ns

	# 记录选择癌症
	cancer_choose <- reactiveValues(name = "BRCA", filter_phe_id=NULL,
		phe_primary=query_tcga_group(database = "toil", cancer = "BRCA", return_all = T))
	observe({
		cancer_choose$name = input$choose_cancer
		cancer_choose$phe_primary <- query_tcga_group(database = "toil", cancer = cancer_choose$name, return_all = T)
	})

	# 自定义上传metadata数据
	custom_meta = callModule(custom_meta_Server, "custom_meta2cor_batch", database = "toil")
	# signature
	sig_dat = callModule(add_signature_Server, "add_signature2cor", database = "toil")

	custom_meta_sig = reactive({
		if(is.null(custom_meta())){
			return(sig_dat())
		} else {
			if(is.null(sig_dat())){
				return(custom_meta())
			} else {
				custom_meta_sig = dplyr::inner_join(custom_meta(),sig_dat())
				return(custom_meta_sig)
			}
		}
	})

	# 数据源设置
	opt_pancan = callModule(mol_origin_Server, "mol_origin2cor", database = "toil")

	## 过滤样本
	# exact filter module
	filter_phe_id = callModule(filter_samples_Server, "filter_samples2cor",
					   database = "toil",
					   cancers=reactive(cancer_choose$name),
					   custom_metadata=reactive(custom_meta_sig()),
					   opt_pancan = reactive(opt_pancan()))
	# quick filter widget
	observe({
		code_types_valid = code_types[names(code_types) %in% 
							unique(cancer_choose$phe_primary$Code)]
		updatePickerInput(
			session,
			"filter_by_code",
			choices = unlist(code_types_valid,use.names = F),
			selected =  unlist(code_types_valid,use.names = F)
		)
	})
	# 综合上述二者
	observe({
		# quick filter
		choose_codes = names(code_types)[unlist(code_types) %in% input$filter_by_code]
		filter_phe_id2 = cancer_choose$phe_primary %>%
			dplyr::filter(Code %in% choose_codes) %>%
			dplyr::pull("Sample")

		# exact filter
		if(is.null(filter_phe_id())){
			cancer_choose$filter_phe_id = filter_phe_id2
		} else {
			cancer_choose$filter_phe_id = intersect(filter_phe_id2,filter_phe_id())
		}

		output$filter_phe_id_info = renderPrint({
			cat(paste0("Tip: ", length(cancer_choose$filter_phe_id), " samples are retained"))
		})
	})


	# 批量下载数据
	L3s_x_data =  callModule(multi_upload_Server, "multi_upload2cor", 
							 database = "toil",
							 samples=reactive(cancer_choose$filter_phe_id),
							 custom_metadata=reactive(custom_meta_sig()),
						     opt_pancan = reactive(opt_pancan())
							 )
	L3s_x = reactive({
		unique(L3s_x_data()$id)
	})

	L3_y_data = callModule(download_feat_Server, "download_y_axis", 
							 database = "toil",
							 samples=reactive(cancer_choose$filter_phe_id),
							 custom_metadata=reactive(custom_meta_sig()),
						     opt_pancan = reactive(opt_pancan()),
						     check_numeric=TRUE							 )

	# 相关性分析
	cor_stat = eventReactive(input$cal_batch_cor,{
		shinyjs::disable("cal_batch_cor")
		x_datas = L3s_x_data()[,c("id","Sample","value")]
		colnames(x_datas)[c(1,3)] = paste0("x_",colnames(x_datas)[c(1,3)])
		y_data = L3_y_data()[,c("id","Sample","value")]
		colnames(y_data)[c(1,3)] = paste0("y_",colnames(y_data)[c(1,3)])

		withProgress(message = "Please wait for a while.",{
			cor_stat = lapply(seq(L3s_x()), function(i) {	
				L3_x = L3s_x()[i]
				xy_data = x_datas %>%
					dplyr::filter(x_id == L3_x) %>%
					dplyr::inner_join(y_data) %>% as.data.frame()
				if(nrow(na.omit(xy_data))<3){return(c(NaN, NaN))}
			    cor_obj = cor.test(xy_data[,"x_value"],xy_data[,"y_value"],
			                       method = input$cor_method)
			    incProgress(1 / length(L3s_x()), detail = paste0("(Finished ",i,"/",length(L3s_x()),")"))
			   	return(c(cor_obj$p.value, cor_obj$estimate))
			}) %>% do.call(rbind, .) %>% as.data.frame()
			colnames(cor_stat) = c("p.value","R")
			cor_stat2 = cor_stat %>% 
			  dplyr::select(R, p.value) %>% 
			  dplyr::mutate(id.x = L3s_x(), .before = 1) %>% 
			  dplyr::mutate(id.y = input$L3_y, .before = 2) %>%
			  dplyr::arrange(p.value)
			cor_stat2
		})
		shinyjs::enable("cal_batch_cor")
		cor_stat2
	})


	output$cor_stat_tb.ui = renderUI({
		output$cor_stat_tb = renderDataTable({
			cor_stat_ = cor_stat() %>%
				dplyr::rename("Batch identifiers"="id.x")
			cor_stat_$p.value = format(cor_stat_$p.value, scientific=T, digits = 3)
			dt = datatable(cor_stat_,
				# class = "nowrap row-border",
				options = list(pageLength = 10, 
					columnDefs = list(
						list(className = 'dt-center', targets="_all"),
						list(orderable=TRUE, targets = 0)))
			) %>%
				formatRound(columns = c("R"), digits = 3)
			dt$x$data[[1]] <- as.numeric(dt$x$data[[1]]) 
			dt
		}) 
	dataTableOutput(ns("cor_stat_tb"))
	})

	# Download results
	observeEvent(input$cal_batch_cor,{
		res1 = NULL
		x_datas = L3s_x_data() %>%
			dplyr::mutate(axis = "X")
		y_data = L3_y_data() %>%
			dplyr::select(id, Sample, value) %>%
			dplyr::mutate(axis = "Y")
		res2 = rbind(x_datas, y_data)
		res3 = cor_stat()
		callModule(download_res_Server, "download_res2cor", res1, res2, res3)
	})
}

#' TCGA Survival Analysis
#'
#' - Firstly, get merged data of one molecular profile value and associated clinical data from TCGA Pan-Cancer dataset.
#' - Secondly, filter data as your wish.
#' - Finally, show K-M plot.
#'
#' @name tcga survival analysis
#' @rdname tcga_surv_analysis
#' @param item a molecular identifier, can be gene symbol (common cases), protein symbol, etc.
#' @param TCGA_cohort a TCGA cohort, e.g. "LUAD" (default), "LUSC", "ACC".
#' @param profile a molecular profile. Option can be one of "mRNA" (default), "miRNA", "methylation", "transcript", "protein", "mutation", "cnv".
#' @param TCGA_cli_data a `data.frame` containing TCGA clinical data. Default use pre-compiled TCGA clinical data in
#' this package.
#' @param time the column name for "time".
#' @param status the column name for "status".
#' @param cutoff_mode mode for grouping samples, can be "Auto" (default) or "Custom".
#' @param cutpoint cut point (in percent) for "Custom" mode, default is `c(50, 50)`.
#' @param cnv_type only used when profile is "cnv", can select from `c("Duplicated", "Normal", "Deleted")`.
#' @param data a subset of result from `tcga_surv_get()`.
#' @param opt_pancan specify one dataset for some molercular profiles
#'
#' @return a `data.frame` or a plot.
#' @export
#' @examples
#' \dontrun{
#' # 1. get data
#' data <- tcga_surv_get("TP53")
#' # 2. filter data (optional)
#'
#' # 3. show K-M plot
#' tcga_surv_plot(data, time = "DSS.time", status = "DSS")
#' }
tcga_surv_get <- function(item,
                          TCGA_cohort = "LUAD",
                          profile = c("mRNA", "miRNA", "methylation", "transcript", "protein", "mutation", "cnv"),
                          TCGA_cli_data = dplyr::full_join(
                            load_data("tcga_clinical"),
                            load_data("tcga_surv"),
                            by = "sample"
                          ),
                          opt_pancan = .opt_pancan) {
  stopifnot(length(item) == 1)
  profile <- match.arg(profile)
  if (!requireNamespace("stringr")) {
    install.packages("stringr")
  }

  # print(head(TCGA_cli_data))
  # type 有几种额外的 Xena 做过聚合的结果
  # "COADREAD" "FPPP"     "GBMLGG"   "LUNG"     "PANCAN"
  # FPPP 好像不是正常的样本，不推荐使用，已去除
  if (TCGA_cohort == "PANCAN") {
    cliMat <- TCGA_cli_data
  } else {
    .type <- switch(TCGA_cohort,
      COADREAD = c("COAD", "READ"),
      GBMLGG = c("GBM", "LGG"),
      LUNG = c("LUAD", "LUSC"),
      TCGA_cohort
    )
    cliMat <- TCGA_cli_data %>% dplyr::filter(.data$type %in% .type)
  }

  message("Querying data of molecule(s) ", item, " for survival analysis in TCGA cohort ", TCGA_cohort, ".")

  gd <- query_pancan_value(item, data_type = profile, opt_pancan = opt_pancan)
  if (is.list(gd)) gd <- gd[[1]]

  if (all(is.na(gd))) {
    return(NULL)
  }
  gd <- gd[nchar(names(gd)) == 15]
  merged_data <- dplyr::tibble(
    sampleID = names(gd),
    value = as.numeric(gd)
  ) %>%
    dplyr::filter(as.numeric(substr(.data$sampleID, 14, 15)) < 10) %>%
    dplyr::inner_join(cliMat, by = c("sampleID" = "sample")) %>%
    dplyr::select(c(
      "sampleID", "value", "OS", "OS.time", "DSS", "DSS.time", "DFI", "DFI.time", "PFI", "PFI.time",
      "gender", "age_at_initial_pathologic_diagnosis", "ajcc_pathologic_tumor_stage"
    )) %>%
    dplyr::rename(
      age = .data$age_at_initial_pathologic_diagnosis,
      stage = .data$ajcc_pathologic_tumor_stage
    ) %>%
    dplyr::mutate(stage = stringr::str_match(.data$stage, "Stage\\s+(.*?)[ABC]?$")[, 2]) %>%
    dplyr::mutate(
      stage = ifelse(is.na(.data$stage), "Unknown", .data$stage),
      gender = ifelse(is.na(.data$gender), "Unknown", .data$gender)
    )
  return(merged_data)
}

#' @rdname tcga_surv_analysis
#' @param palette color palette, can be "hue", "grey", "RdBu", "Blues", "npg", "aaas", etc.
#' More see `?survminer::ggsurvplot`.
#' @param ... other parameters passing to `survminer::ggsurvplot`
#' @export
tcga_surv_plot <- function(data,
                           time = "time",
                           status = "status",
                           cutoff_mode = c("Auto", "Custom"),
                           cutpoint = c(50, 50),
                           cnv_type = c("Duplicated", "Normal", "Deleted"),
                           profile = c("mRNA", "miRNA", "methylation", "transcript", "protein", "mutation", "cnv"),
                           palette = "aaas",
                           ...) {
  cutoff_mode <- match.arg(cutoff_mode)
  profile <- match.arg(profile)
  kept_cols <- c("value", time, status)
  data <- dplyr::select(data, dplyr::all_of(kept_cols))
  if (time != "time") colnames(data)[2] <- "time"
  if (status != "status") colnames(data)[3] <- "status"

  if (!requireNamespace("survminer")) {
    install.packages("survminer")
  }

  if (profile %in% c("mRNA", "miRNA", "methylation", "transcript", "protein")) {
    sur_plot(data, cutoff_mode, cutpoint, palette = palette, ...)
  } else if (profile == "mutation") {
    sur_plot_mut(data, palette = palette, ...)
  } else {
    sur_plot_cnv(data, cnv_type, palette = palette, ...)
  }
}

## Survival analysis for mRNA and protein expression
sur_plot <- function(data, cutoff_mode, cutpoint, palette = "aaas", ...) {
  if (cutoff_mode == "Auto") {
    data2 <- data %>%
      survminer::surv_cutpoint(
        time = "time", event = "status",
        variables = c("value"),
        minprop = 0.25, progressbar = TRUE
      ) %>%
      survminer::surv_categorize(labels = c("Low", "High")) %>%
      data.frame()
    data$group <- data2$value
  } else {
    if (length(cutpoint) == 1) {
      cutpoint <- c(cutpoint, cutpoint)
    }
    data <- data %>%
      dplyr::arrange(.data$value) %>%
      dplyr::mutate(per_rank = 100 / nrow(.) * (1:nrow(.)))
    data <- data %>%
      dplyr::mutate(group = dplyr::case_when(
        .data$per_rank > !!cutpoint[2] ~ "High",
        .data$per_rank <= !!cutpoint[1] ~ "Low",
        TRUE ~ NA_character_
      ))
  }
  p_survplot(data %>% dplyr::mutate(group = factor(group, c("Low", "High"))), palette = palette, ...)
}

## Survival analysis for mutation DNA
sur_plot_mut <- function(data, palette = "aaas", ...) {
  data <- data %>%
    dplyr::rename(mut = .data$value) %>%
    dplyr::mutate(group = ifelse(.data$mut == 1, "MT", "WT"))
  if (length(table(data$group)) < 2) {
    return(NULL)
  }
  p_survplot(data %>% dplyr::mutate(group = factor(group, c("WT", "MT"))), palette = palette, ...)
}

## Survival analysis for CNV
sur_plot_cnv <- function(data, cnv_type = c("Duplicated", "Normal", "Deleted"), palette = "aaas", ...) {
  data <- data %>%
    dplyr::rename(mut = .data$value) %>%
    dplyr::mutate(group = ifelse(.data$mut == 0, "Normal",
      ifelse(.data$mut > 0, "Duplicated", "Deleted")
    )) %>%
    dplyr::filter(.data$group %in% cnv_type)
  if (length(table(data$group)) < 2) {
    return(NULL)
  }
  p_survplot(data %>% dplyr::mutate(group = factor(group, c("Deleted", "Duplicated", "Normal"))), palette = palette, ...)
}

## ggsurvplot
p_survplot <- function(data, palette = "aaas", ...) {
  fit <- survival::survfit(survival::Surv(time, status) ~ group, data = data)
  p <- survminer::ggsurvplot(fit,
    data = data, pval = TRUE, pval.method = TRUE,
    palette = palette,
    size = 1.2, # change line size
    font.legend = c(14, "black"),
    font.x = c(14, "bold", "black"),
    font.y = c(14, "bold", "black"),
    font.tickslab = c(12, "bold", "black"),
    xlab = "Duration overall survival (days)",
    risk.table = TRUE,
    risk.table.col = "strata", # Risk table color by groups
    risk.table.y.text = FALSE, # show bars instead of names in text annotations
    ncensor.plot = TRUE, # plot the number of censored subjects at time t
    surv.plot.height = 0.7,
    risk.table.height = 0.15,
    ncensor.plot.height = 0.15,
    ggtheme = ggplot2::theme_classic(), # Change ggplot2 theme
    ...
  ) # %+% ggplot2::guides(color = ggplot2::guide_legend(ncol = 3))
  p$plot <- p$plot %+% ggplot2::guides(color = ggplot2::guide_legend(ncol = 3))

  attr(p, "data") <- na.omit(data[, c("time", "status", "group")])
  p
}

#' Obtain ToilHub Info for Single Molecule
#'
#' @inheritParams get_pancan_value
#' @importFrom utils data
#' @return a `tibble`
#' @export
#'
#' @examples
#' \dontrun{
#' t <- query_toil_value_df()
#' t
#' }
query_toil_value_df <- function(identifier = "TP53") {
  df <- get_pancan_gene_value(identifier = identifier)
  data("toil_info", package = "UCSCXenaShiny", envir = environment())
  toil_info <- toil_info %>%
    dplyr::left_join(dplyr::tibble(
      sample = names(df$expression),
      expression = df$expression
    ), by = "sample") %>%
    dplyr::filter(!is.na(expression)) %>%
    dplyr::rename(
      primary_site = `_primary_site`,
      sample_type = `_sample_type`,
      gender = `_gender`,
      study = `_study`
    ) %>%
    dplyr::select(sample, primary_site, detailed_category, sample_type, gender, study, expression) %>%
    dplyr::mutate(unit = df$unit) %>%
    dplyr::as_tibble() %>%
    dplyr::filter(sample_type %in% c(
      "Primary Tumor", "Normal Tissue", "Primary Solid Tumor", "Solid Tissue Normal",
      "Metastatic", "Cell Line", "Primary Blood Derived Cancer - Peripheral Blood",
      "Primary Blood Derived Cancer - Bone Marrow", "Recurrent Blood Derived Cancer - Bone Marrow"
    )) %>%
    # Only keep samples with count > 100
    dplyr::mutate(
      sample_type = factor(sample_type, levels = c(
        "Primary Tumor", "Normal Tissue", "Primary Solid Tumor", "Solid Tissue Normal",
        "Metastatic", "Cell Line", "Primary Blood Derived Cancer - Peripheral Blood",
        "Primary Blood Derived Cancer - Bone Marrow", "Recurrent Blood Derived Cancer - Bone Marrow"
      ))
    )

  toil_info
}

utils::globalVariables(
  c(
    "_primary_site",
    "_sample_type",
    "_gender",
    "_study",
    "primary_site",
    "sample_type",
    "gender",
    "study",
    "expression",
    "detailed_category"
  )
)

#' Visualize the distribution difference of samples after dimensionality reduction analysis
#'
#' @param ids molecular identifiers (>=3)
#' @param data_type molecular types, refer to query_pancan_value() function
#' @param group_info two-column grouping information with names 'Sample','Group'
#' @param DR_method the dimensionality reduction method
#' @param palette the color setting of RColorBrewer
#' @param add_margin the marginal plot (NULL, "density", "boxplot")
#' @param opt_pancan specify one dataset for some molercular profiles

#' @return a ggplot object or rawdata list
#' @export
#'
#' @examples
#' \dontrun{
#' group_info <- tcga_clinical_fine %>%
#'   dplyr::filter(Cancer == "BRCA") %>%
#'   dplyr::select(Sample, Code) %>%
#'   dplyr::rename(Group = Code)
#'
#' vis_dim_dist(
#'   ids = c("TP53", "KRAS", "PTEN", "MDM2", "CDKN1A"),
#'   group_info = group_info
#' )
#' }
#'
vis_dim_dist <- function(ids = c("TP53", "KRAS", "PTEN", "MDM2", "CDKN1A"),
                         data_type = "mRNA",
                         group_info = NULL,
                         DR_method = c("PCA", "UMAP", "tSNE"),
                         palette = "Set1",
                         add_margin = NULL,
                         opt_pancan = .opt_pancan) {
  # Mode <- match.arg(Mode)
  DR_method <- match.arg(DR_method)

  if (length(ids) < 3) {
    stop("The number of valid ids is less than three. Please inspect the input ids and data_type(?query_pancan_value)")
  }

  exp_raw <- tryCatch(
    {
      purrr::map(ids, function(x) {
        # x = ids[1]
        data <- query_pancan_value(x, data_type = data_type, opt_pancan = opt_pancan)
        data <- data[[1]]
        data <- dplyr::tibble(Sample = names(data), y = as.numeric(data))
        colnames(data)[2] <- x
        data
      }) %>% purrr::reduce(dplyr::full_join, by = "Sample")
    },
    error = function(e) {
      rlang::inform("access data failed, the message is provided below")
      rlang::warn(conditionMessage(e))
      NULL
    }
  )
  if (is.null(exp_raw)) {
    return(NULL)
  }


  # meta_raw = query_tcga_group(...)$data
  if (is.null(group_info)) {
    stop("Please input valid grouping information for `group_info` parameter.")
  }
  if (!all(colnames(group_info) == c("Sample", "Group"))) {
    stop("The group_info should have two colnames named `Sample` and `Group`.")
  }
  meta_raw <- group_info
  meta_data <- meta_raw %>% dplyr::filter(.data$Sample %in% exp_raw$Sample)

  if (nrow(meta_data) == 0) {
    stop("No intersected samples are detected for the group_info.")
  }
  if (length(unique(meta_data$Group)) < 2) {
    stop("Less two valid groups are detected for the group_info.")
  }


  exp_data <- exp_raw[match(meta_data$Sample, exp_raw$Sample), ]
  ids_NAN <- colnames(exp_data[, -1])[apply(exp_data[, -1], 2, function(x) all(is.na(x)))]
  ids_SD0 <- colnames(exp_data[, -1])[apply(exp_data[, -1], 2, function(x) stats::sd(x) == 0)] %>% na.omit()
  ids_OK <- setdiff(ids, c(ids_NAN, ids_SD0))
  # message(paste0((length(ids_OK)/length(ids))*100, "%"), " of input ids were obtained")
  exp_data <- exp_data[, which(!colnames(exp_data) %in% c(ids_NAN, ids_SD0))]


  if (DR_method == "PCA") {
    pca_obj <- prcomp(exp_data[, ids_OK], center = TRUE, scale = TRUE)
    res_dims <- pca_obj$x[, 1:2] %>%
      as.data.frame() %>%
      dplyr::rename("PC_1" = "PC1", "PC_2" = "PC2")
  } else if (DR_method == "tSNE") {
    if (!requireNamespace("Rtsne", quietly = TRUE)) {
      stop(
        "Package \"Rtsne\" must be installed to use this method.",
        call. = FALSE
      )
    }
    set.seed(123)
    tsne_obj <- Rtsne::Rtsne(exp_data[, ids_OK])
    res_dims <- tsne_obj$Y %>%
      as.data.frame() %>%
      dplyr::rename("tSNE_1" = "V1", "tSNE_2" = "V2")
  } else if (DR_method == "UMAP") {
    if (!requireNamespace("umap", quietly = TRUE)) {
      stop(
        "Package \"umap\" must be installed to use this method.",
        call. = FALSE
      )
    }
    umap_obj <- umap::umap(exp_data[, ids_OK])
    res_dims <- umap_obj$layout %>%
      as.data.frame() %>%
      dplyr::rename("UMAP_1" = "V1", "UMAP_2" = "V2")
  }

  res_dims <- cbind(res_dims, meta_data) %>%
    dplyr::inner_join(exp_data)


  ## Step5: ggplot scatter plot


  group_levels <- unique(res_dims$Group)


  if (length(group_levels) > 6) {
    colors <- grDevices::hcl(
      h = seq(15, 375, length = length(group_levels) + 1),
      l = 65, c = 100
    )[seq(length(group_levels))]
    shapes <- rep(16, length(group_levels))
  } else {
    colors <- RColorBrewer::brewer.pal(n = 6, name = palette)[seq(group_levels)]
    shapes <- c(15:20)[seq(group_levels)]
  }


  p <- ggplot2::ggplot(res_dims, aes_string(colnames(res_dims)[1], colnames(res_dims)[2], color = "Group", shape = "Group")) +
    ggplot2::geom_point() +
    ggplot2::stat_ellipse() +
    ggplot2::theme_bw(base_size = 20) +
    ggplot2::guides(
      color = guide_legend(title = NULL),
      shape = guide_legend(title = NULL)
    ) +
    ggplot2::theme(
      # legend.background = element_blank(),
      # legend.position = c(0, 0),
      # legend.justification = c(0, 0)
      legend.position = "bottom"
    ) +
    ggplot2::scale_color_manual(values = colors) +
    ggplot2::scale_shape_manual(values = shapes)

  if (!is.null(add_margin)) {
    geom_type <- switch(add_margin,
      "density" = geom_density,
      "boxplot" = geom_boxplot,
      stop("Please choose one of density/boxplot marginal type")
    )

    p_right <- cowplot::axis_canvas(p, axis = "x") +
      geom_type(
        data = p$data, aes_string(x = colnames(p$data)[1], fill = "Group"),
        alpha = 0.8, linewidth = 0.3
      ) +
      ggplot2::scale_fill_manual(values = colors)
    p_top <- cowplot::axis_canvas(p, axis = "y", coord_flip = TRUE) +
      geom_type(
        data = p$data, aes_string(x = colnames(p$data)[2], fill = "Group"),
        alpha = 0.8, linewidth = 0.3
      ) +
      coord_flip() +
      ggplot2::scale_fill_manual(values = colors)
    p_tmp <- p %>%
      cowplot::insert_xaxis_grob(p_right, grid::unit(.2, "null"), position = "top") %>%
      cowplot::insert_yaxis_grob(p_top, grid::unit(.2, "null"), position = "right")
    p_tmp2 <- cowplot::ggdraw(p_tmp)
    p_tmp2$data <- p$data
    p <- p_tmp2
  }
  return(p)
}

# Visualize Violin plot
GeomSplitViolin <- ggproto("GeomSplitViolin", GeomViolin,
  draw_group = function(self, data, ..., draw_quantiles = NULL) {
    # Original function by Jan Gleixner (@jan-glx)
    # Adjustments by Wouter van der Bijl (@Axeman)
    data <- transform(data, xminv = x - violinwidth * (x - xmin), xmaxv = x + violinwidth * (xmax - x))
    grp <- data[1, "group"]
    newdata <- plyr::arrange(transform(data, x = if (grp %% 2 == 1) xminv else xmaxv), if (grp %% 2 == 1) y else -y)
    newdata <- rbind(newdata[1, ], newdata, newdata[nrow(newdata), ], newdata[1, ])
    newdata[c(1, nrow(newdata) - 1, nrow(newdata)), "x"] <- round(newdata[1, "x"])
    if (length(draw_quantiles) > 0 & !scales::zero_range(range(data$y))) {
      stopifnot(all(draw_quantiles >= 0), all(draw_quantiles <= 1))
      quantiles <- create_quantile_segment_frame(data, draw_quantiles, split = TRUE, grp = grp)
      aesthetics <- data[rep(1, nrow(quantiles)), setdiff(names(data), c("x", "y")), drop = FALSE]
      aesthetics$alpha <- rep(1, nrow(quantiles))
      both <- cbind(quantiles, aesthetics)
      quantile_grob <- GeomPath$draw_panel(both, ...)
      ggplot2:::ggname("geom_split_violin", grid::grobTree(GeomPolygon$draw_panel(newdata, ...), quantile_grob))
    } else {
      ggplot2:::ggname("geom_split_violin", GeomPolygon$draw_panel(newdata, ...))
    }
  }
)
# Visualize Violin plot
create_quantile_segment_frame <- function(data, draw_quantiles, split = FALSE, grp = NULL) {
  interleave <- "vctrs" %:::% "vec_interleave"
  dens <- cumsum(data$density) / sum(data$density)
  suppressWarnings(ecdf <- stats::approxfun(dens, data$y))
  ys <- ecdf(draw_quantiles)
  violin.xminvs <- (stats::approxfun(data$y, data$xminv))(ys)
  violin.xmaxvs <- (stats::approxfun(data$y, data$xmaxv))(ys)
  violin.xs <- (stats::approxfun(data$y, data$x))(ys)
  if (grp %% 2 == 0) {
    data.frame(
      x = interleave(violin.xs, violin.xmaxvs),
      y = rep(ys, each = 2), group = rep(ys, each = 2)
    )
  } else {
    data.frame(
      x = interleave(violin.xminvs, violin.xs),
      y = rep(ys, each = 2), group = rep(ys, each = 2)
    )
  }
}
# Visualize Violin plot
geom_split_violin <- function(mapping = NULL, data = NULL, stat = "ydensity", position = "identity", ...,
                              draw_quantiles = NULL, trim = TRUE, scale = "area", na.rm = FALSE,
                              show.legend = NA, inherit.aes = TRUE) {
  layer(
    data = data, mapping = mapping, stat = stat, geom = GeomSplitViolin, position = position,
    show.legend = show.legend, inherit.aes = inherit.aes,
    params = list(trim = trim, scale = scale, draw_quantiles = draw_quantiles, na.rm = na.rm, ...)
  )
}

#' Visualize the distribution difference of samples after Molecule Identifier dimensionality reduction analysis
#'
#' NOTE: the dataset must be dense matrix in UCSC Xena data hubs.
#'
#'
#' @param dataset the dataset to obtain identifiers.
#' @param ids the molecule identifiers.
#' @param grp_df When `dataset` and `id` are all not `NULL`, it should be a `data.frame` with 2 columns.
#' - The first column refers to sample ID.
#' - The second column refers to groups indicated in axis X.
#' @param samples default is `NULL`, can be common sample names for two datasets.
#' @param return.data whether to reture the raw meta/matrix data (list) instead of plot
#' @param DR_method the dimensionality reduction method
#' @param add_margin the marginal plot (NULL, "density", "boxplot")
#' @param palette the color setting of RColorBrewer
#'
#' @return  a `ggplot` object.
#' @export
#'
#' @examples
#' # vis_identifier_dim_dist(expr_dataset, ids, grp_df, DR_method="PCA")
#' @export
vis_identifier_dim_dist <- function(dataset = NULL, ids = NULL, grp_df, samples = NULL,
                                    return.data = FALSE,
                                    DR_method = c("PCA", "UMAP", "tSNE"), add_margin = NULL,
                                    palette = "Set1") {
  stopifnot(ncol(grp_df) > 1)
  DR_method <- match.arg(DR_method)
  colnames(grp_df)[1] <- "sample"

  if (!is.null(dataset) && !is.null(ids)) {
    df <- tryCatch(
      {
        purrr::map(ids, function(x) {
          message("Querying data of identifier ", x, " from dataset ", dataset, " for DR")
          data <- if (dataset == "custom_feature_dataset") {
            query_custom_feature_value(x)
          } else {
            query_molecule_value(dataset, x)
          }
          data <- dplyr::tibble(
            sample = names(data),
            y = as.numeric(data)
          )
          colnames(data)[2] <- x
          data
        }) %>%
          purrr::reduce(dplyr::full_join, by = "sample")
      },
      error = function(e) {
        rlang::inform("access data failed, the message is provided below")
        rlang::warn(conditionMessage(e))
        NULL
      }
    )
    if (is.null(df)) {
      return(NULL)
    }

    df <- dplyr::inner_join(grp_df, df, by = "sample")
  } else {
    message("Directly use 'grp_df' for comparison analysis.")
    df <- grp_df
    ids <- colnames(grp_df)[-1:-2]
  }

  if (!is.null(samples)) {
    df <- dplyr::filter(df, .data$sample %in% samples)
  }

  ids_NAN <- colnames(df[, ids])[apply(df[, ids], 2, function(x) all(is.na(x)))]
  ids_SD0 <- colnames(df[, ids])[apply(df[, ids], 2, function(x) stats::sd(x) == 0)] %>% na.omit()
  ids_OK <- setdiff(ids, c(ids_NAN, ids_SD0))

  if (length(ids_OK) < 3) {
    stop("The number of valid ids is less than three. Please inspect the input ids and data_type(?query_pancan_value)")
  }

  if (return.data) {
    return(list(meta = grp_df, exp = df))
  }

  if (DR_method == "PCA") {
    pca_obj <- prcomp(df[, ids_OK], center = TRUE, scale = TRUE)
    res_dims <- pca_obj$x[, 1:2] %>%
      as.data.frame() %>%
      dplyr::rename("PC_1" = "PC1", "PC_2" = "PC2")
  } else if (DR_method == "tSNE") {
    set.seed(123)
    tsne_obj <- Rtsne::Rtsne(df[, ids_OK])
    res_dims <- tsne_obj$Y %>%
      as.data.frame() %>%
      dplyr::rename("tSNE_1" = "V1", "tSNE_2" = "V2")
  } else if (DR_method == "UMAP") {
    umap_obj <- umap::umap(df[, ids_OK])
    res_dims <- umap_obj$layout %>%
      as.data.frame() %>%
      dplyr::rename("UMAP_1" = "V1", "UMAP_2" = "V2")
  }

  res_dims <- cbind(df[1:2], res_dims)

  groups <- length(unique(res_dims[, colnames(grp_df)[2], drop = TRUE]))
  if (groups < 2) {
    stop("The number of valid group levels is less than two")
  }

  if (groups > 6) {
    color <- grDevices::hcl(
      h = seq(15, 375, length = groups + 1),
      l = 65, c = 100
    )[seq(groups)]
    shape <- rep(16, groups)
  } else {
    color <- RColorBrewer::brewer.pal(n = 6, name = palette)[seq(groups)]
    shape <- c(15:20)[seq(groups)]
  }

  p <- ggplot2::ggplot(
    res_dims,
    aes_string(colnames(res_dims)[3], colnames(res_dims)[4], color = colnames(res_dims)[2], shape = colnames(res_dims)[2])
  ) +
    ggplot2::geom_point() +
    ggplot2::stat_ellipse() +
    ggplot2::theme_classic(base_size = 20) +
    ggplot2::guides(
      color = guide_legend(title = NULL),
      shape = guide_legend(title = NULL)
    ) +
    ggplot2::theme(
      legend.background = element_blank(),
      legend.position = c(0, 0),
      legend.justification = c(0, 0)
    ) +
    ggplot2::scale_color_manual(values = color) +
    ggplot2::scale_shape_manual(values = shape)

  if (!is.null(add_margin)) {
    geom_type <- switch(add_margin,
      "density" = geom_density,
      "boxplot" = geom_boxplot,
      stop("Please choose one of density/boxplot marginal type")
    )

    p_right <- cowplot::axis_canvas(p, axis = "x") +
      geom_type(
        data = p$data, aes_string(x = colnames(p$data)[3], fill = colnames(grp_df)[2]),
        alpha = 0.8, linewidth = 0.3
      ) +
      ggplot2::scale_fill_manual(values = color)
    p_top <- cowplot::axis_canvas(p, axis = "y", coord_flip = TRUE) +
      geom_type(
        data = p$data, aes_string(x = colnames(p$data)[4], fill = colnames(grp_df)[2]),
        alpha = 0.8, linewidth = 0.3
      ) +
      coord_flip() +
      ggplot2::scale_fill_manual(values = color)
    p_tmp <- p %>%
      cowplot::insert_xaxis_grob(p_right, grid::unit(.2, "null"), position = "top") %>%
      cowplot::insert_yaxis_grob(p_top, grid::unit(.2, "null"), position = "right")
    p_tmp2 <- cowplot::ggdraw(p_tmp)
    p_tmp2$data <- p$data
    p <- p_tmp2
  }
  return(p)
}

#' Visualize Gene and Drug-Target Association with CCLE Data
#'
#' See [analyze_gene_drug_response_asso] for examples.
#'
#' @export
#' @param output_form `plotly` or `ggplot2`.
#' @inheritParams vis_toil_TvsN
#' @param x_axis_type set the value type for X axis.
#' @return `plotly` or `ggplot2` object.
vis_gene_drug_response_asso <- function(Gene = "TP53",
                                        x_axis_type = c("mean.diff", "median.diff"),
                                        output_form = c("plotly", "ggplot2")) {
  x_axis_type <- match.arg(x_axis_type)
  output_form <- match.arg(output_form)

  if (!requireNamespace("plotly")) install.packages("plotly")
  if (!requireNamespace("ggrepel")) install.packages("ggrepel")

  df <- analyze_gene_drug_response_asso(Gene, combine = TRUE) # Combine as a signature if more than 1 gene

  df$p_log <- -log10(df$p.value)
  df$text <- paste(
    "Gene(s): ", paste(Gene, collapse = "/"),
    "<br>Drug: ", df$drugs,
    "<br>Target: ", df$Target,
    "<br>Correlation: ", round(df$cor, digits = 3),
    "<br><i>P</i> value: ", round(df$p.value, digits = 3),
    "<br>FDR: ", round(df$fdr, digits = 3),
    "<br>Number of Cell Lines: ", df$num_of_cell_lines
  )

  p <- ggplot(data = df, aes_string(
    x = x_axis_type,
    y = "p_log",
    color = "cor",
    text = "text"
  )) +
    geom_point() +
    ggtitle(paste0(
      if (length(Gene) > 1) {
        paste0("Signature (", paste(Gene, collapse = "&"), ")")
      } else {
        Gene
      },
      " and Drug-Target Response Association"
    )) +
    labs(x = if (x_axis_type == "mean.diff") {
      "Mean of expression difference between high and low IC50 cell lines"
    } else {
      "Median of expression difference between high and low IC50 cell lines"
    }, y = "-log10(P-value)") +
    cowplot::theme_cowplot() +
    scale_color_gradient2(low = scales::muted("blue"), high = scales::muted("red"), midpoint = 0) +
    theme(
      plot.title = element_text(hjust = 0.5)
    ) +
    geom_hline(yintercept = -log10(0.05), linetype = 2, size = 0.5, alpha = 0.5)
  if (output_form == "plotly") p <- plotly::ggplotly(p, tooltip = "text")
  return(p)
}


#' Visualize Gene and Drug Response Difference with CCLE Data
#'
#' See [analyze_gene_drug_response_diff] for examples.
#'
#' @export
#' @inheritParams vis_toil_TvsN
#' @param tissue select cell line origin tissue.
#' @param alpha set alpha for dots.
#' @return a `ggplot` object.
vis_gene_drug_response_diff <- function(Gene = "TP53", tissue = "lung",
                                        Show.P.label = TRUE,
                                        Method = "wilcox.test",
                                        values = c("#DF2020", "#DDDF21"),
                                        alpha = 0.5) {
  df <- analyze_gene_drug_response_diff(Gene, tissue = tissue, combine = TRUE) # Combine as a signature if more than 1 gene

  if (Show.P.label) {
    message("Counting P value")
    pv <- ggpubr::compare_means(IC50 ~ group, data = df, method = Method, group.by = "drug_target")
    pv <- pv %>%
      dplyr::arrange(.data$p)
    pv$drug_target <- factor(pv$drug_target, levels = unique(pv$drug_target))
    df$drug_target <- factor(df$drug_target, levels = unique(pv$drug_target))
    message("Counting P value finished")
    pv$y.position <- 8.5
  }

  p <- ggpubr::ggdotplot(
    df,
    x = "group", y = "IC50", color = "group", fill = "group",
    add = "mean_sd", facet.by = "drug_target", alpha = alpha, size = 0.6
  ) +
    labs(x = "Drug -> Target", y = "IC50 (uM)")

  if (Show.P.label) {
    p <- p + ggpubr::stat_pvalue_manual(pv, label = "p.signif", tip.length = 0.01) +
      ggplot2::scale_y_continuous(limits = c(0, 10))
  }

  p <- p +
    ggplot2::scale_color_manual(values = values) +
    ggplot2::scale_fill_manual(values = values) +
    cowplot::theme_cowplot() +
    theme(
      axis.text.x = element_blank(),
      axis.ticks.x = element_blank(),
      legend.position = c(0.85, 0.1)
    )
  return(p)
}

## how to document datasets: you need to specify @docType and @name; do not
## forget NULL in the end

#' Purity Data of PCAWG
#' @docType data
#' @name pcawg_purity
#' @format A `data.frame`
#' @source UCSC Xena.
#' @examples
#' data("pcawg_purity")
NULL

#' Phenotype Info of CCLE Database
#' @docType data
#' @name ccle_info
#' @format A `data.frame`
#' @source UCSC Xena.
#' @examples
#' data("ccle_info")
NULL

#' Cleaned Phenotype Info of CCLE Database for grouping
#' @docType data
#' @name ccle_info_fine
#' @format A `data.frame`
#' @source UCSC Xena.
#' @examples
#' data("ccle_info_fine")
NULL


#' Phenotype Info of PCAWG Database
#' @docType data
#' @name pcawg_info
#' @format A `data.frame`
#' @source UCSC Xena.
#' @examples
#' data("pcawg_info")
NULL


#' Cleaned Phenotype Info of PCAWG Database for grouping
#' @docType data
#' @name pcawg_info_fine
#' @format A `data.frame`
#' @source UCSC Xena.
#' @examples
#' data("pcawg_info_fine")
NULL



#' ABSOLUTE Result of CCLE Database
#' @docType data
#' @name ccle_absolute
#' @format A `data.frame`
#' @source see "data_source" attribute.
#' @examples
#' data("ccle_absolute")
NULL

#' Toil Hub: TCGA TARGET GTEX Selected Phenotype
#' @docType data
#' @name toil_info
#' @format A [data.frame]
#' @source Generate from data-raw
#' @examples
#' data("toil_info")
NULL


#' Toil Hub: TCGA Clinical Data
#'
#' See `tcga_surv` for TCGA survival data.
#' @docType data
#' @name tcga_clinical
#' @format A [data.frame]
#' @source Generate from data-raw
#' @examples
#' data("tcga_clinical")
NULL


#' Toil Hub: Cleaned TCGA Clinical Data for grouping
#'
#' See `tcga_surv` for TCGA survival data.
#' @docType data
#' @name tcga_clinical_fine
#' @format A [data.frame]
#' @source Generate from data-raw
#' @examples
#' data("tcga_clinical_fine")
NULL


#' Toil Hub: TCGA Survival Data
#' @docType data
#' @name tcga_surv
#' @format A [data.frame]
#' @source Generate from data-raw
#' @examples
#' data("tcga_surv")
NULL


#' Toil Hub: Merged TCGA GTEx Selected Phenotype
#' @docType data
#' @name tcga_gtex
#' @format A [data.frame]
#' @examples
#' data("tcga_gtex")
NULL

#' TCGA: TMB (Tumor Mutation Burden) Data
#' @docType data
#' @name  tcga_tmb
#' @format A [data.frame]
#' @examples
#' data("tcga_tmb")
#' @source <https://gdc.cancer.gov/about-data/publications/panimmune>
NULL


#' TCGA: Genome Instability Data
#' @docType data
#' @name  tcga_genome_instability
#' @format A [data.frame]
#' @examples
#' data("tcga_genome_instability")
#' @source <https://gdc.cancer.gov/about-data/publications/PanCanStemness-2018>
NULL

#' TCGA: Purity Data
#' @docType data
#' @name  tcga_purity
#' @format A [data.frame]
#' @examples
#' data("tcga_purity")
#' @source <https://www.nature.com/articles/ncomms9971#Sec14>
NULL

#' TCGA: Organ Data
#' @docType data
#' @name  TCGA.organ
#' @format A [data.frame]
#' @examples
#' data("TCGA.organ")
NULL

#' TCGA Subtype Data
#' @docType data
#' @name  tcga_subtypes
#' @format A [data.frame]
#' @source UCSC Xena.
#' @examples
#' data("tcga_subtypes")
NULL

#' @title Xena Shiny App
#' @description A Shiny App for UCSC Xena Data Hubs. See <https://github.com/openbiox/UCSCXenaShiny> for details.
#' @keywords internal
"_PACKAGE"

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
#' @import ggplot2
#' @importFrom psych corr.test
#' @importFrom utils download.file head install.packages tail
#' @importFrom stats prcomp predict quantile na.omit
## usethis namespace: end
NULL

#########################################
#### A Shiny App for UCSC Xena ##########
#########################################
##### LICENSE: GPLv3 @Openbiox   ##########
#########################################


#' Run UCSC Xena Shiny App
#'
#' @importFrom shiny shinyAppFile
#' @inheritParams shiny::runApp
#' @param runMode default is 'client' for personal user, set it to 'server' for running on server.
#' @return NULL
#' @export
#'
#' @examples
#' \dontrun{
#' app_run()
#' }
app_run <- function(runMode = "client", port = getOption("shiny.port")) {
  runMode <- match.arg(runMode, choices = c("client", "server"))
  options(xena.runMode = runMode)
  shiny::shinyAppFile(system.file("shinyapp", "App.R", package = "UCSCXenaShiny"),
    options = list(port = port)
  )
}




#' Run UCSC Xena Shiny App with specifc content
#'
#' @importFrom shiny shinyAppFile
#' @inheritParams shiny::runApp
#' @param runMode default is 'client' for personal user, set it to 'server' for running on server.
#' @param content Modules to enable. 
#' - `a`: all modules
#' - `s`: only loading basic modules; 
#' - `q`: add tpc (TCGA, PCAWG, CCLE) modules
#' - `p`: add tpc pipelines
#' - `d`: add pharmcogenomics modules
#'
#' @return NULL
#' @export
#'
#' @examples
#' \dontrun{
#' app_run2(content = "s")
#' }
app_run2 <- function(runMode = "client", port = getOption("shiny.port"),
                     content = c("a", "s", "q", "p", "d")) {
  runMode <- match.arg(runMode, choices = c("client", "server"))
  options(xena.runMode = runMode)
  content <- match.arg(content)
  if (content == "a") {
    cm1 <- ""
    cm2 <- ""
    cm3 <- ""
    cm0 <- ""
    cm_link <- "#"
  } else if (content == "s") {
    cm1 <- "#"
    cm2 <- "#"
    cm3 <- "#"
    cm0 <- ""
    cm_link <- "#"
    if (runMode == "server") {
      cm_link <- ""
    } # server application with link2advanced tabpanel
  } else if (content == "q") {
    cm1 <- ""
    cm2 <- "#"
    cm3 <- "#"
    cm0 <- "#"
    cm_link <- "#"
  } else if (content == "p") {
    cm1 <- "#"
    cm2 <- ""
    cm3 <- "#"
    cm0 <- "#"
    cm_link <- "#"
  } else if (content == "d") {
    cm1 <- "#"
    cm2 <- ""
    cm3 <- ""
    cm0 <- "#"
    cm_link <- "#"
  }
  codes <- glue::glue(
    'source(system.file("shinyapp/app_preset.R", package = "UCSCXenaShiny")) \n',
    "\n",
    "ui <- tagList( \n",
    "  tags$head( \n",
    '    tags$title("XenaShiny"), \n',
    '    # tags$link(rel = "icon", type="image/svg+xml", href = "./icon.svg"), \n',
    '    tags$link(rel = "icon", type="image/png", href = "./icon.png"), \n',
    '    tags$link(rel = "stylesheet", type = "text/css", href = "./css/global.css") \n',
    "  ), \n",
    "  useWaiter(), \n",
    "  waiterPreloader(html = tagList( \n",
    "    spin_fading_circles(), \n",
    "    br(), br(), \n",
    '    h1(strong("Welcome to use UCSCXenaShiny v2 application!")), \n',
    "    br(), \n",
    '    p("An interactive web tool with general and personalized modules to explore UCSC Xena datasets" \n',
    '      ,style = "font-size: 25px;"), \n',
    "    br(),br(), \n",
    '    p("Notes:", "(1) The initiation could take about 10 seconds. (2) Please zoom in or up screen for better representation.", \n',
    '      style = "font-size: 16px;") \n',
    '  ), color = "#2C3E50"), \n',
    "  shinyjs::useShinyjs(), \n",
    "  autoWaiter(html = spin_loader(), color = transparent(0.5)), # change style https://shiny.john-coene.com/waiter/ \n",
    "  navbarPage( \n",
    '    id = "navbar", \n',
    '    title = "UCSCXenaShiny v2", \n',
    '    windowTitle = "UCSCXenaShiny", \n',
    "    # inst/shinyapp/ui \n",
    "    ui.page_home(), \n",
    "    {cm0}ui.page_repository(), \n",
    "    {cm0}ui.page_general_analysis(), \n",
    "    {cm_link}ui.link2advanced(), \n",
    "    {cm1}ui.page_pancan_quick(), \n",
    "    {cm2}ui.page_pancan_tcga(), \n",
    "    {cm3}ui.page_PharmacoGenomics(), \n",
    "    {cm0}ui.page_download(), \n",
    "    ui.page_help(), \n",
    "    ui.page_developers(), \n",
    "    footer = ui.footer(), \n",
    "    collapsible = TRUE, \n",
    '    theme = tryCatch(shinythemes::shinytheme("flatly"), \n',
    "                     error = function(e) {{ \n",
    '                       "Theme \'flatly\' is not available, use default." \n',
    "                       NULL \n",
    "                     }}) \n",
    "  ) \n",
    ") \n",
    "\n",
    "server <- function(input, output, session) {{ \n",
    '  message("Shiny app run successfully! Enjoy it!\n") \n',
    '  message("               --  Xena shiny team\n") \n',
    "  # Stop warn \n",
    '  storeWarn <- getOption("warn") \n',
    "  options(warn = -1) \n",
    "  # observe(print(input$navbar)) \n",
    "  # inst/shinyapp/server \n",
    '  source(server_file("home.R"), local = TRUE) \n',
    '  source(server_file("repository.R"), local = TRUE) \n',
    '  source(server_file("modules.R"), local = TRUE) \n',
    '  source(server_file("general-analysis.R"), local = TRUE) \n',
    '  observe_helpers(help_dir = system.file("shinyapp", "helper", package = "UCSCXenaShiny")) \n',
    "}} \n",
    "\n",
    "shiny::shinyApp( \n",
    "  ui = ui, \n",
    "  server = server \n",
    ") \n"
  )


  app_file <- paste0(get_cache_dir(), "/App_", content, ".R")
  dirname(app_file)

  if (!dir.exists(dirname(app_file))) {
    dir.create(dirname(app_file), recursive = TRUE)
  }

  readr::write_file(codes, file = app_file)

  shiny::shinyAppFile(app_file,
    options = list(port = port)
  )
}

# PCAWG -------------------------------------------------------------------

# Not all gene data available
# query_pancan_value("KRAS", database = "pcawg")
# query_pancan_value("hsa-let-7c-3p", database = "pcawg", data_type = "miRNA")
# query_pancan_value("hsa-let-7c-3p", database = "pcawg", data_type = "miRNA", norm = "UQ")
# query_pancan_value("ENSG00000000419", database = "pcawg", data_type = "fusion") # gene symbol also work
# query_pancan_value("tCa_MutLoad_MinEstimate", database = "pcawg", data_type = "APOBEC")
# query_pancan_value("prmtr.10000", database = "pcawg", data_type = "promoter")
# query_pancan_value("X:99891803:TSPAN6", database = "pcawg", data_type = "promoter")

#' @describeIn get_pancan_value Fetch specimen-level gene expression value from PCAWG cohort
#' @export
get_pcawg_gene_value <- function(identifier) {
  host <- "pcawgHub"
  dataset <- "tophat_star_fpkm_uq.v2_aliquot_gl.sp.log"

  expression <- get_data(dataset, identifier, host)
  unit <- "log2(fpkm-uq+0.001)"
  report_dataset_info(dataset)
  res <- list(data = expression, unit = unit)
  res
}

#' @describeIn get_pancan_value Fetch specimen-level gene fusion value from PCAWG cohort
#' @export
get_pcawg_fusion_value <- function(identifier) {
  host <- "pcawgHub"
  dataset <- "pcawg3_fusions_PKU_EBI.gene_centric.sp.xena"

  expression <- get_data(dataset, identifier, host)
  unit <- "binary fusion call, 1 fusion, 0 otherwise"
  report_dataset_info(dataset)
  res <- list(data = expression, unit = unit)
  res
}

#' @describeIn get_pancan_value Fetch specimen-level gene promoter activity value from PCAWG cohort
#' @export
get_pcawg_promoter_value <- function(identifier, type = c("raw", "relative", "outlier")) {
  # promoter identifier prmtr.10000 or symbol are supported
  # but the latter seems meaningless
  host <- "pcawgHub"
  type <- match.arg(type)

  if (type == "raw") {
    dataset <- "rawPromoterActivity.sp"
    unit <- "raw promoter activity"
  } else if (type == "relative") {
    dataset <- "relativePromoterActivity.sp"
    unit <- "portion of transcription activity of the gene driven by the promoter"
  } else {
    dataset <- "promoterCentricTable_0.2_1.0.sp"
    unit <- "-1 (low expression), 0 (normal), 1 (high expression)"
  }

  if (!startsWith(identifier, "prmtr")) {
    # Try parsing from location:symbol map
    map <- load_data("pcawg_promoter_id")
    id_map <- map[names(map) == identifier]
    if (length(id_map) > 1) {
      # query_pancan_value("19:12203078:ZNF788", database = "pcawg", data_type = "promoter")
      # 存在极少数有多 id 情况，直接求和
      expression <- purrr::reduce(
        purrr::map(
          as.character(id_map),
          ~ get_data(dataset, ., host)
        ), `+`
      )
    } else {
      expression <- get_data(dataset, as.character(id_map), host)
    }
  } else {
    expression <- get_data(dataset, identifier, host)
  }


  report_dataset_info(dataset)
  res <- list(data = expression, unit = unit)
  res
}

#' @param norm the normalization method.
#' @describeIn get_pancan_value Fetch specimen-level miRNA value from PCAWG cohort
#' @export
get_pcawg_miRNA_value <- function(identifier, norm = c("TMM", "UQ")) {
  host <- "pcawgHub"
  norm <- match.arg(norm)

  if (norm == "TMM") {
    dataset <- "x3t2m1.mature.TMM.mirna.matrix.log"
    unit <- "log2(cpm-TMM+0.1)"
  } else {
    dataset <- "x3t2m1.mature.UQ.mirna.matrix.log"
    unit <- "log2(cpm-uq+0.1)"
  }

  expression <- get_data(dataset, identifier, host)
  report_dataset_info(dataset)
  res <- list(data = expression, unit = unit)
  res
}


#' @describeIn get_pancan_value Fetch specimen-level gene fusion value from PCAWG cohort
#' @export
get_pcawg_APOBEC_mutagenesis_value <- function(identifier = c(
                                                 "tCa_MutLoad_MinEstimate", "APOBECtCa_enrich",
                                                 "A3A_or_A3B", "APOBEC_tCa_enrich_quartile", "APOBECrtCa_enrich",
                                                 "APOBECytCa_enrich", "APOBECytCa_enrich-APOBECrtCa_enrich",
                                                 "BH_Fisher_p-value_tCa", "ntca+tgan", "rtCa_to_G+rtCa_to_T",
                                                 "rtca+tgay", "tCa_to_G+tCa_to_T",
                                                 "ytCa_rtCa_BH_Fisher_p-value", "ytCa_rtCa_Fisher_p-value", "ytCa_to_G+ytCa_to_T",
                                                 "ytca+tgar"
                                               )) {
  identifier <- match.arg(identifier)
  host <- "pcawgHub"
  dataset <- "MAF_Aug31_2016_sorted_A3A_A3B_comparePlus.sp"

  expression <- get_data(dataset, identifier, host)
  unit <- ""
  report_dataset_info(dataset)
  res <- list(data = expression, unit = unit)
  res
}

## 突变数据直接用表格展示??

parse_pacwg_args <- function(data_type) {
  if (!data_type %in% c(
    "mRNA", "miRNA_TMM", "miRNA_UQ",
    "promoter_raw", "promoter_relative", "promoter_outlier",
    "fusion", "APOBEC"
  )) {
    stop('Valid data_type contains "mRNA" "miRNA_TMM" "miRNA_UQ" "promoter_raw" "promoter_relative" "promoter_outlier" "fusion" "APOBEC"')
  }

  # 用 _ 标志额外的参数
  if (grepl("_", data_type)) {
    y <- unlist(strsplit(data_type, "_"))
    out <- list(
      data_type = y[1],
      args = y[2]
    )

    z <- switch(y[1],
      miRNA = "norm_method",
      promoter = "type",
      stop("Don't support such data type!")
    )
    names(out)[2] <- z
    out
  } else {
    list(data_type = data_type)
  }
}

# e.g.,
# test <- query_pcawg_pancan_value("hsa-miR-769-3p", data_type = "miRNA_UQ")
# str(test)
query_pcawg_pancan_value <- function(id, data_type) {
  args <- c(
    list(molecule = id, database = "pcawg"),
    parse_pacwg_args(data_type)
  )
  do.call("query_pancan_value", args = args)
}

#' Visualize molecular profile in PCAWG
#' @inheritParams vis_toil_TvsN
#' @return a `ggplot` object
#' @examples
#' \dontrun{
#' p <- vis_pcawg_dist(Gene = "TP53")
#' }
#' @export
#'
vis_pcawg_dist <- function(Gene = "TP53",
                           Mode = c("Boxplot", "Violinplot"),
                           data_type = "mRNA", Show.P.value = TRUE,
                           Show.P.label = TRUE, Method = c("wilcox.test", "t.test"),
                           values = c("#DF2020", "#DDDF21"),
                           draw_quantiles = c(0.25, 0.5, 0.75),
                           trim = TRUE, opt_pancan = .opt_pancan) {
  Mode <- match.arg(Mode)
  Method <- match.arg(Method)

  if (!Method %in% c("wilcox.test", "t.test")) {
    stop("only support wilcox.test or t.test")
  }

  if (!Mode %in% c("Boxplot", "Violinplot")) {
    stop("only support Boxplot or Violinplot")
  }

  pcawg_info <- load_data("pcawg_info")

  # t1 <- query_pcawg_pancan_value(Gene, data_type)
  t1 <- query_pancan_value(Gene, data_type, database = "pcawg", opt_pancan = opt_pancan)


  unit <- switch(data_type,
    cnv = NULL,
    mutation = NULL,
    t1[[2]]
  )
  if (is.list(t1)) t1 <- t1[[1]]
  if (all(is.na(t1))) {
    message("All NAs returned, return NULL instead.")
    return(NULL)
  }

  t2 <- t1 %>%
    as.data.frame() %>%
    dplyr::rename("tpm" = ".") %>%
    tibble::rownames_to_column(var = "icgc_specimen_id") %>%
    dplyr::inner_join(pcawg_info, by = "icgc_specimen_id")

  # table(pcawg_info$dcc_specimen_type)
  pcawg_data <- t2 %>%
    dplyr::select("tpm", "dcc_project_code", "type2", "icgc_specimen_id")
  pcawg_data$type2 <- factor(pcawg_data$type2, c("tumor", "normal"))


  if (Show.P.value == FALSE) {
    Show.P.label <- FALSE
  }
  if (Show.P.value == TRUE) {
    message("Counting P value")
    pv <- pcawg_data %>%
      ggpubr::compare_means(tpm ~ type2, data = ., method = Method, group.by = "dcc_project_code")
    pv <- pv %>% dplyr::select(c("dcc_project_code", "p", "p.signif", "p.adj"))
    message("Counting P value finished")
  }

  if (Mode == "Boxplot") {
    p <- ggplot2::ggplot(pcawg_data, aes_string(x = "dcc_project_code", y = "tpm", fill = "type2")) +
      ggplot2::geom_boxplot() +
      ggplot2::xlab(NULL) +
      ggplot2::theme_set(theme_set(theme_classic(base_size = 20))) +
      ggplot2::theme(axis.text.x = element_text(angle = 45, hjust = .5, vjust = .5)) +
      ggplot2::guides(fill = guide_legend(title = NULL)) +
      ggplot2::theme(
        legend.background = element_blank(),
        legend.position = c(0, 0), legend.justification = c(0, 0)
      ) +
      ggplot2::scale_fill_manual(values = values)

    p <- p + ggplot2::ylab(
      if (is.null(unit)) {
        Gene
      } else {
        paste0(Gene, if (nchar(unit) < 30) {
          paste0(" (", unit, ")")
        } else {
          paste0("\n(", unit, ")")
        })
      }
    )

    if (Show.P.value == TRUE & Show.P.label == TRUE) {
      p <- p + ggplot2::geom_text(
        aes(
          x = .data$dcc_project_code,
          y = max(pcawg_data$tpm) * 1.1,
          label = .data$p.signif
        ),
        data = pv,
        inherit.aes = FALSE
      )
    }
    if (Show.P.value == TRUE & Show.P.label == FALSE) {
      p <- p + ggplot2::geom_text(
        aes(
          x = .data$dcc_project_code,
          y = max(pcawg_data$tpm) * 1.1,
          label = as.character(signif(.data$p, 2))
        ),
        data = pv,
        inherit.aes = FALSE
      )
    }
  }

  if (Mode == "Violinplot") {
    p <- ggplot2::ggplot(pcawg_data, aes_string(x = "dcc_project_code", y = "tpm", fill = "type2")) +
      geom_split_violin(
        draw_quantiles = draw_quantiles,
        trim = trim,
        linetype = "solid",
        color = "black",
        size = 0.2,
        na.rm = TRUE,
        position = "identity"
      ) +
      ggplot2::xlab("") +
      ggplot2::scale_fill_manual(values = values) +
      ggplot2::theme_set(ggplot2::theme_classic(base_size = 20)) +
      ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 45, hjust = .5, vjust = .5)) +
      ggplot2::guides(fill = ggplot2::guide_legend(title = NULL)) +
      ggplot2::theme(
        legend.background = ggplot2::element_blank(),
        legend.position = c(0, 0), legend.justification = c(0, 0)
      )

    p <- p + ggplot2::ylab(
      if (is.null(unit)) {
        Gene
      } else {
        paste0(Gene, if (nchar(unit) < 30) {
          paste0(" (", unit, ")")
        } else {
          paste0("\n(", unit, ")")
        })
      }
    )

    if (Show.P.value == TRUE & Show.P.label == TRUE) {
      p <- p + ggplot2::geom_text(
        ggplot2::aes(
          x = .data$dcc_project_code,
          y = max(pcawg_data$tpm) * 1.1,
          label = .data$p.signif
        ),
        data = pv,
        inherit.aes = FALSE
      )
    }
    if (Show.P.value == TRUE & Show.P.label == FALSE) {
      p <- p + ggplot2::geom_text(
        ggplot2::aes(
          x = .data$dcc_project_code,
          y = max(pcawg_data$tpm) * 1.1,
          label = as.character(signif(.data$p, 2))
        ),
        data = pv,
        inherit.aes = FALSE
      )
    }
  }
  return(p)
}

#' Visualize Single Gene Univariable Cox Result in PCAWG
#'
#' @inheritParams vis_toil_TvsN
#' @param measure a survival measure, e.g. "OS".
#' @param data_type choose gene profile type, including "mRNA","transcript","methylation","miRNA","protein","cnv"
#' @param use_optimal_cutoff use `surv_cutpoint` from survminer package for
#' thresholding samples in each cancer type.
#' @return a `ggplot` object
#' @examples
#' \dontrun{
#' p <- vis_pcawg_unicox_tree(Gene = "TP53")
#' }
#' @export

vis_pcawg_unicox_tree <- function(
    Gene = "TP53", measure = "OS", data_type = "mRNA", use_optimal_cutoff = FALSE,
    values = c("grey", "#E31A1C", "#377DB8"), opt_pancan = .opt_pancan) {
  pcawg_info <- load_data("pcawg_info")

  # t1 <- query_pcawg_pancan_value(Gene, data_type)
  t1 <- query_pancan_value(Gene, data_type, database = "pcawg", opt_pancan = opt_pancan)

  unit <- switch(data_type,
    cnv = NULL,
    mutation = NULL,
    t1[[2]]
  )
  if (is.list(t1)) t1 <- t1[[1]]
  if (all(is.na(t1))) {
    message("All NAs returned, return NULL instead.")
    return(NULL)
  }

  # we filter out normal tissue
  pcawg_info <- pcawg_info %>% dplyr::filter(.data$type2 != "normal")
  s <- data.frame(icgc_specimen_id = names(t1), values = t1)
  ss <- s %>%
    dplyr::inner_join(pcawg_info, by = "icgc_specimen_id") %>%
    dplyr::select(values, icgc_specimen_id, dcc_project_code, OS, OS.time)

  sss <- split(ss, ss$dcc_project_code)
  tissues <- names(sss)
  .f <- function(cancer) {
    sss_can <- sss[[cancer]]

    if (use_optimal_cutoff) {
      sss_can <- sss_can %>%
        survminer::surv_cutpoint(
          time = paste0(measure, ".time"), event = measure,
          variables = c("values"),
          minprop = 0.25, progressbar = TRUE
        ) %>%
        survminer::surv_categorize(labels = c("Low", "High")) %>%
        data.frame() %>%
        dplyr::mutate(values = factor(values, levels = c("Low", "High")))
    }

    unicox_res_genes <- ezcox::ezcox(
      sss_can,
      covariates = "values",
      time = paste0(measure, ".time"),
      status = measure,
      verbose = FALSE
    )

    unicox_res_genes$cancer <- cancer
    unicox_res_genes$measure <- measure
    return(unicox_res_genes)
  }
  unicox_res_all_cancers <- purrr::map(tissues, purrr::safely(.f)) %>% magrittr::set_names(tissues)

  unicox_res_all_cancers <- unicox_res_all_cancers %>%
    purrr::map(~ .x$result) %>%
    purrr::compact()
  unicox_res_all_cancers_df <- do.call(rbind.data.frame, unicox_res_all_cancers)
  unicox_res_all_cancers_df <- unicox_res_all_cancers_df %>%
    dplyr::mutate(HR_log = log(.data$HR)) %>%
    dplyr::mutate(lower_95_log = log(.data$lower_95)) %>%
    dplyr::mutate(upper_95_log = log(.data$upper_95)) %>%
    dplyr::mutate(Type = ifelse(.data$p.value < 0.05 & .data$HR_log > 0, "Risky", ifelse(.data$p.value < 0.05 & .data$HR_log < 0, "Protective", "NS"))) %>%
    dplyr::mutate(Type = factor(Type, levels = c("NS", "Risky", "Protective")))


  unicox_res_all_cancers_df <- unicox_res_all_cancers_df[complete.cases(unicox_res_all_cancers_df), ]
  ## visualization
  p <- unicox_res_all_cancers_df %>%
    dplyr::mutate(cancer = factor(.data$cancer, levels = rev(.data$cancer))) %>%
    ggplot2::ggplot(aes_string(x = "cancer", y = "HR_log", ymin = "lower_95_log", ymax = "upper_95_log", color = "Type")) +
    ggplot2::theme_bw() +
    ggplot2::geom_pointrange() +
    ggplot2::coord_flip() +
    ggplot2::labs(x = "", y = "log (Hazard Ratio)") +
    ggtitle(paste0(Gene, if (startsWith(data_type, "mRNA") | startsWith(data_type, "miRNA")) " Expression" else "")) +
    ggplot2::theme(
      axis.text.x = element_text(color = "black"),
      axis.text.y = element_text(color = "black"),
      panel.grid.major = element_blank(),
      panel.grid.minor = element_blank(),
      plot.title = element_text(hjust = 0.5)
    ) +
    ggplot2::scale_color_manual(values = values) +
    ggplot2::geom_hline(yintercept = c(0), linetype = "dashed")

  return(p)
}

#' Visualize Gene-Gene Correlation in TCGA
#' @import ggplot2 dplyr tibble forcats
#' @import ggplot2 dplyr ppcor
#' @inheritParams vis_gene_cor
#' @param cor_method correlation method
#' @param use_log_x if `TRUE`, log X values.
#' @param use_log_y if `TRUE`, log Y values.
#' @param dcc_project_code_choose select project code.
#' @param use_all use all sample, default `FALSE`.
#' @param filter_tumor whether use tumor sample only, default `TRUE`
#' @return a `ggplot` object
#' @export

vis_pcawg_gene_cor <- function(Gene1 = "CSF1R",
                               Gene2 = "JAK3",
                               data_type1 = "mRNA",
                               data_type2 = "mRNA",
                               cor_method = "spearman",
                               purity_adj = TRUE,
                               use_log_x = FALSE,
                               use_log_y = FALSE,
                               use_regline = TRUE,
                               dcc_project_code_choose = "BLCA-US",
                               use_all = FALSE,
                               filter_tumor = TRUE,
                               alpha = 0.5, color = "#000000",
                               opt_pancan = .opt_pancan) {
  if (!requireNamespace("cowplot")) {
    install.packages("cowplot")
  }

  pcawg_info <- load_data("pcawg_info")
  pcawg_purity <- load_data("pcawg_purity")

  # t1 <- query_pcawg_pancan_value(Gene1, data_type1)
  t1 <- query_pancan_value(Gene1, data_type1, database = "pcawg", opt_pancan = opt_pancan)

  unit1 <- switch(data_type1,
    cnv = NULL,
    mutation = NULL,
    t1[[2]]
  )
  if (is.list(t1)) t1 <- t1[[1]]

  if (all(is.na(t1))) {
    message("All NAs returned, return NULL instead.")
    return(NULL)
  }
  if (use_log_x) t1 <- log2(t1 + 1)

  t2 <- t1 %>%
    as.data.frame() %>%
    dplyr::rename("tpm" = ".") %>%
    tibble::rownames_to_column(var = "icgc_specimen_id") %>%
    dplyr::inner_join(pcawg_info, by = c("icgc_specimen_id"))

  # t3 <- query_pcawg_pancan_value(Gene2, data_type2)
  t3 <- query_pancan_value(Gene2, data_type2, database = "pcawg", opt_pancan = opt_pancan)

  unit2 <- switch(data_type2,
    cnv = NULL,
    mutation = NULL,
    t1[[2]]
  )
  if (is.list(t3)) t3 <- t3[[1]]

  # if (data_type1 == "cnv") data_type1 <- "CNV"
  # if (data_type2 == "cnv") data_type2 <- "CNV"

  if (all(is.na(t3))) {
    message("All NAs returned, return NULL instead.")
    return(NULL)
  }
  if (use_log_y) t3 <- log2(t3 + 1)

  t4 <- t3 %>%
    as.data.frame() %>%
    dplyr::rename("tpm" = ".") %>%
    tibble::rownames_to_column(var = "icgc_specimen_id") %>%
    dplyr::inner_join(pcawg_info, by = c("icgc_specimen_id"))

  t2 <- t2 %>% inner_join(t4[, c("icgc_specimen_id", "tpm")], by = "icgc_specimen_id")

  df <- data.frame(
    icgc_specimen_id = t2$icgc_specimen_id,
    gene1 = t2$tpm.x, gene2 = t2$tpm.y,
    dcc_project_code = t2$dcc_project_code,
    type2 = t2$type2,
    stringsAsFactors = F
  )

  if (!use_all) {
    df <- df %>% dplyr::filter(.data$dcc_project_code %in% dcc_project_code_choose)
  }

  df %>%
    dplyr::left_join(pcawg_purity, by = "icgc_specimen_id") %>%
    dplyr::filter(!is.na(.data$purity)) -> df

  if (filter_tumor) {
    df %>% dplyr::filter(.data$type2 == "tumor") -> df
  }

  if (nrow(df) == 0) {
    warning("No intersected samples for the two identifiers in the cohort")
    return(NULL)
  }

  # print(dim(df))

  if (purity_adj) {
    partial_cor_res <- ezcor_partial_cor(data = df, var1 = "gene1", var2 = "gene2", var3 = "purity", sig_label = TRUE)
    cor_res <- ezcor(data = df, var1 = "gene1", var2 = "gene2", cor_method = cor_method)

    p <- ggplot2::ggplot(df, aes_string(x = "gene1", y = "gene2")) +
      ggplot2::geom_point(shape = 16, size = 1.5, show.legend = FALSE, alpha = alpha, color = color) +
      ggplot2::theme_minimal() +
      ggplot2::scale_color_gradient(low = "#0091ff", high = "#f0650e") +
      ggplot2::labs(x = Gene1, y = Gene2) +
      # ggplot2::ggtitle(paste0("PCAWG dataset (n=",dim(df)[1],")")) +
      ggplot2::annotate(
        "text",
        -Inf, Inf,
        hjust = -0.1, vjust = 1,
        label = paste0(
          "Cor: ", round(cor_res$cor, 2),
          " ", cor_res$pstar, "\n", "Cor_adj: ",
          round(partial_cor_res$cor_partial, 2), " ",
          partial_cor_res$pstar
        ),
        size = 5, colour = "black"
      )
  } else {
    cor_res <- ezcor(data = df, var1 = "gene1", var2 = "gene2", cor_method = cor_method)

    p <- ggplot2::ggplot(df, aes_string(x = "gene1", y = "gene2")) +
      ggplot2::geom_point(shape = 16, size = 1.5, show.legend = FALSE, alpha = alpha, color = color) +
      ggplot2::theme_minimal() +
      ggplot2::scale_color_gradient(low = "#0091ff", high = "#f0650e") +
      ggplot2::labs(x = Gene1, y = Gene2) +
      # ggplot2::ggtitle(paste0("PCAWG dataset (n=",dim(df)[1],")")) +
      ggplot2::annotate(
        "text",
        -Inf, Inf,
        hjust = -0.1, vjust = 1,
        label = paste0(
          "Cor: ", round(cor_res$cor, 2), " ",
          cor_res$pstar
        ),
        size = 5, colour = "black"
      )
  }

  if (use_regline) p <- p + ggplot2::geom_smooth(method = stats::lm)

  if (use_all == TRUE) {
    p <- p + ggplot2::ggtitle(paste0("PCAWG: All data (n=", dim(df)[1], ")"))
  } else {
    p <- p + ggplot2::ggtitle(paste0("PCAWG: ", paste(dcc_project_code_choose, collapse = "/"), " (n=", dim(df)[1], ")"))
  }

  return(p)
}

- <font size="4">In **S3** Step, **survival analysis  across multiple cancers** can now be performed after data preparation;</font>
- <font size="4">Firstly, you can adjust the default analysis and visualization parameters, respectively;</font>
- <font size="4">Noteworthily, you can try whether to use **initial data** before grouping if the data type is **numerical**;</font>

> 1. For log-rank test, it will decide the optimal cutoff;
> 2. For univariate cox regression, it will directly analyze based on the numerical data. 

- <font size="4">Then, you will obtain the bar (log-rank test & univariate cox regression) plot labeled with statistical result after clicking the "**Run**" button;</font>
- <font size="4">Finally, you can download the 3 types of results.</font>

> 1. "**Figure**": visualization result;
> 2. "**Raw data(.csv)**": queried data for X-axis and Y-axis;
> 3. "**Analyzed data(.csv)**": detailed correlation result.

<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_analyze_sur_2.png" alt="helper_analyze_sur_2" width="700"/>
</p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 

<font size="4">**Visualize multi-omics profile of one pathway among TCGA cancer types** </font>

<font size="4">1. Expression Profile</font>

- <font size="4">Normal ssGSEA : The median ssGSEA  gene expression in normal tissues (Including GTEx samples).</font>
- <font size="4">Tumor ssGSEA :  The median ssGSEA  gene expression in tumor tissues.</font>
- <font size="4">T vs. N(Wilcox) : The Wilcoxon test between tumor and normal samples (P<0.001, "\*\*\*";  P<0.01, "\*\*";  P<0.05, "\*";  P>0.05, "-").</font>

> Bar plot: The 0~1 normalization on TPM expression data.



<font size="4">2. Mutation Profile</font>

- <font size="4">Mutation Dist Mean:  The distribution of mutation or wild pathway gene status in tumor tissues. If one of pathway genes is mutated, it is considered as mutation. </font>

- <font size="4">Mutation PCT Mean:  The percentage of pathway mutation status in tumor tissues.</font>

- <font size="4">Mut.Genes Mean:  The mean counts of mutated pathway genes in tumor tissues.</font>



<font size="4">3. CNV Profile</font>

- <font size="4">CNV Dist Mean: The distribution of pathway gene copy number variation status in tumor tissues.</font>
  - Amp, homozygous deletion or single copy deletion; 
  - Non, diploid normal copy; 
  - Del, Copy number amplification.


- <font size="4">Amp.Genes Mean : The copy number amplification percentage of pathway genes  in tumor tissues.</font>
- <font size="4">Del.Genes Mean : The copy number deletion percentage of pathway gene in tumor tissues.</font>



<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_cross_pw.png" alt="helper_cross_pw"  width="700"/>



---

**Data source—**

- mRNA: [TcgaTargetGtex_rsem_gene_tpm](https://xenabrowser.net/datapages/?dataset=TcgaTargetGtex_rsem_gene_tpm&host=https://toil.xenahubs.net)
- Mutation: [mc3.v0.2.8.PUBLIC.nonsilentGene.xena](https://xenabrowser.net/datapages/?dataset=mc3.v0.2.8.PUBLIC.nonsilentGene.xena&host=https://pancanatlas.xenahubs.net)
- CNV: [TCGA.PANCAN.sampleMap/Gistic2_CopyNumber_Gistic2_all_thresholded.by_genes](https://xenabrowser.net/datapages/?dataset=TCGA.PANCAN.sampleMap/Gistic2_CopyNumber_Gistic2_all_thresholded.by_genes&host=https://tcga.xenahubs.net)

<font size="4">**Visualize multi-omics profile of one gene among TCGA cancer types** </font>

<font size="4">1. Expression Profile</font>

- <font size="4">Normal Exp : The median gene expression in normal tissues (Including GTEx samples).</font>
- <font size="4">Tumor Exp :  The median gene expression in tumor tissues.</font>
- <font size="4">T vs. N(Wilcox): The Wilcoxon test between tumor and normal samples (P<0.001, "\*\*\*";  P<0.01, "\*\*";  P<0.05, "\*";  P>0.05, "-").</font>

> Bar plot: The 0~1 normalization on TPM expression data.



<font size="4">2. Mutation Profile</font>

- <font size="4">Mutation Dist :  The distribution of gene mutation or wild status in tumor tissues.</font>

- <font size="4">Mutation PCT :  The percentage of gene mutation status in tumor tissues.</font>



<font size="4">3. CNV Profile</font>

- <font size="4">CNV Dist : The distribution of gene copy number variation status in tumor tissues.</font>

  - -2, homozygous deletion; 
  - -1, single copy deletion; 
  - 0, diploid normal copy; 
  - 1: low-level copy number amplification;
  - 2: high-level copy number amplification


- <font size="4">CNV Amp : The percentage of gene copy number amplification (1,2) in tumor tissues.</font>
- <font size="4">CNV Del : The percentage of gene copy number deletion (-1, -2) in tumor tissues.</font>



<font size="4">4. Transcript Profile</font>

- <font size="4">Selected transcript(s) of one gene : The median transcript expression in tumor tissues.</font>



<font size="4">5. Methylation Profile</font>

- <font size="4">Selected CpG sites of one gene : The median  beta value  of CpG sites in tumor tissues.</font>



<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_cross_gene.png" alt="helper_cross_gene"   width="700"/>



---

**Data source—**

- mRNA: [TcgaTargetGtex_rsem_gene_tpm](https://xenabrowser.net/datapages/?dataset=TcgaTargetGtex_rsem_gene_tpm&host=https://toil.xenahubs.net)
- Mutation: [mc3.v0.2.8.PUBLIC.nonsilentGene.xena](https://xenabrowser.net/datapages/?dataset=mc3.v0.2.8.PUBLIC.nonsilentGene.xena&host=https://pancanatlas.xenahubs.net)
- CNV: [TCGA.PANCAN.sampleMap/Gistic2_CopyNumber_Gistic2_all_thresholded.by_genes](https://xenabrowser.net/datapages/?dataset=TCGA.PANCAN.sampleMap/Gistic2_CopyNumber_Gistic2_all_thresholded.by_genes&host=https://tcga.xenahubs.net)
- Transcript: [TcgaTargetGtex_rsem_isoform_tpm](https://xenabrowser.net/datapages/?dataset=TcgaTargetGtex_rsem_isoform_tpm&host=https://toil.xenahubs.net)
- Methylation: [GDC-PANCAN.methylation450.tsv](https://xenabrowser.net/datapages/?dataset=GDC-PANCAN.methylation450.tsv&host=https://gdc.xenahubs.net)

- <font size="4">In this step, you can query multiple identifiers of tumor data under one specific "**Data type**"--"**Data subtype**;</font> 
- <font size="4">Generally, the following 3 way can be performed to select batch IDs;</font></font>

> 1. In the "**Selection**" way, choose multiple IDs one by one;
> 2. In the "**All**" way, directly choose all IDs under the data subtype. For molecular profile, you need further choose one pathway to limit the ID size;
> 3. In the "**File**" way, choose IDs by uploading an one-column file. You can download the example data to check the format requirements.

<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_get_batch_data.png" alt="helper_get_batch_data"   width="700"/>
</p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 

---
title: "Examples of Accessible Functions"
output: 
  rmarkdown::html_document:
    toc: true
    toc_float: true
    toc_depth: 3
author: ["Shixiang Wang, Yi Xiong, Longfei Zhao, Kai Gu, Yin Li, Fei Zhao"]
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{Accessible Functions}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.align = "center"
)
```

Firstly, assume you have already installed **UCSCXenaShiny** package.

```{r setup}
library(UCSCXenaShiny)
```

## Tutorial Book

From UCSCXenaShiny v2, we provide a comprehensive [tutorial book](https://lishensuo.github.io/UCSCXenaShiny_Book/) for introducing how to use the datasets, functions, and the Shiny application.

## Fetch Data

We provide function to retrieve multi-dimensional data including genomic, epigenomic, transcriptomic, and proteomic data from TCGA (note, this actually contains data from TCGA/TARGET/GTEx databases) and CCLE Pan-Cancer dataset for single identifier (e.g., gene, protein).

Check parameters:

```{r}
args(query_pancan_value)
```

For TCGA gene expression data, we use Xena dataset with ID `TcgaTargetGtex_rsem_gene_tpm` which includes 19131 samples with tumor tissue samples and normal tissue samples. The expression value unit is `log2(tpm+0.001)`.

Let's check several examples.

### Fetch Gene Expression

```{r}
gene_expr <- query_pancan_value("TP53")
```

```{r}
str(gene_expr)
```

### Fetch Transcript Expression

```{r eval=FALSE}
transcript_expr <- query_pancan_value("ENST00000000233", data_type = "transcript")
```

### Fetch Gene CNV

```{r eval=FALSE}
gene_cnv <- query_pancan_value("TP53", data_type = "cnv")
```

### Fetch Gene Mutation

```{r eval=FALSE}
gene_mut <- query_pancan_value("TP53", data_type = "mutation")
```

### Fetch miRNA mature strand expression

```{r eval=FALSE}
miRNA_expr <- query_pancan_value("hsa-let-7a-2-3p", data_type = "miRNA")
```

## Data Visualization

### Visualize Single Gene Expression in PANCAN Dataset

```{r fig.width=12}
vis_toil_TvsN(Gene = "TP53", Mode = "Violinplot", Show.P.value = FALSE, Show.P.label = FALSE)
```

### Compare Gene Expression Level in Single Cancer Type

```{r fig.width=5}
vis_toil_TvsN_cancer(
  Gene = "TP53",
  Mode = "Violinplot",
  Show.P.value = TRUE,
  Show.P.label = TRUE,
  Method = "wilcox.test",
  values = c("#DF2020", "#DDDF21"),
  TCGA.only = FALSE,
  Cancer = "ACC"
)
```

### Compare Gene Expression Level in Different Anatomic Regions

This function needs [gganatogram](https://github.com/jespermaag/gganatogram) package, which is not on CRAN. Please install it before using this function.


```
if (require("gganatogram")) {
  vis_pancan_anatomy(Gene = "TP53", Gender = c("Female", "Male"), option = "D")
}
```

[Click to see the output](https://gitee.com/ShixiangWang/ImageCollection/raw/master/png/20210322130516.png)

### Visualize Relationship between Gene Expression and Prognosis in the PANCAN Dataset

```{r fig.width=5, fig.height=6}
vis_unicox_tree(
  Gene = "TP53",
  measure = "OS",
  values = c("grey", "#E31A1C", "#377DB8")
)
```

### Other Functions and Data

All exported data and functions are organized at [here](https://openbiox.github.io/UCSCXenaShiny/reference/index.html).

- <font size="4"> In this step, you can upload custom TCGA/PCAWG/CCLE sample data for joint analysis. The example data can be downloded to check the relevant format requirements;</font> 
- <font size="4"> The uploaded data will be placed with **Custom metadata** (Data subtype) of Phenotype data (Data type) for analysis;</font>


<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_costom_metadata.png" alt="helper_costom_metadata"  width="700"/>
</p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 

<font size="4">In **S3** Step, **batch correlation analysis** can now be performed after data preparation:</font>

- <font size="4">Firstly, you can adjust the default analysis parameters.</font>
  - <font size="4">Use **Spearman correlation** (based on the ranks of the data rather than the actual values) when input data is ordinal, not normally distributed, or you expect a monotonic but not necessarily linear relationship .</font>
  - <font size="4">Use **Pearson correlation** (based on covariance and standard deviation) when input data is continuous, normally distributed, and you expect a linear relationship.</font>
- <font size="4">Then, you will obtain the table arranged by absolute correlation coefficients after clicking the "**Run**" button.</font>
- <font size="4">Finally, you can download the 2 types of results.</font>

> 1. "**Raw data(.csv)**": queried data for X-axis and Y-axis;
> 2. "**Analyzed data(.csv)**": detailed correlation result.


<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_analyze_cor_3.png" alt="helper_analyze_cor_3"    width="700"/>
</p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 

<center>
<h1>TPC modules</h1>
</center>


<font size="5">Here, we introduced multiple custom  modules for quick TPC molecule exploration with few steps.  In general, 10 panels for common analytical scenario are available, as follows:</font>


<center>
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_intro_quick_mods_1.png" alt="helper_intro_quick_mods_1" style="zoom:50%;" />
</center>




## 1. Box Layout

- <font size="5">The [dashboard](https://github.com/RinteRface/shinydashboardPlus)-based **box** layouts are adopted for simple and intuitive representation. The **left box** is usually for data selection and analytical parameters adjustment. The **right box** is usually for display of result plot and one **sidebar panel** can be pulled to adjust  visualization options and download results.</font>

<center>
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_intro_quick_mods_2.png" alt="helper_intro_quick_mods_2" />
</center>

## 2. Molecular database

<font size="5">We designate one specific UCSC Xena dataset for each molecular type of TPC databases. You can check the detailed information through the links below.</font>

### 2.1 TCGA

- <font size="5">mRNA Expression —— [TcgaTargetGtex_rsem_gene_tpm](https://xenabrowser.net/datapages/?dataset=TcgaTargetGtex_rsem_gene_tpm&host=https://toil.xenahubs.net)</font>
- <font size="5">Transcript Expression —— [TcgaTargetGtex_rsem_isoform_tpm](https://xenabrowser.net/datapages/?dataset=TcgaTargetGtex_rsem_isoform_tpm&host=https://toil.xenahubs.net)</font>
- <font size="5">DNA Methylation —— [GDC-PANCAN.methylation450.tsv](https://xenabrowser.net/datapages/?dataset=GDC-PANCAN.methylation450.tsv&host=https://gdc.xenahubs.net)</font>
- <font size="5">miRNA Expression —— [pancanMiRs_EBadjOnProtocolPlatformWithoutRepsWithUnCorrectMiRs_08_04_16.xena](https://xenabrowser.net/datapages/?dataset=pancanMiRs_EBadjOnProtocolPlatformWithoutRepsWithUnCorrectMiRs_08_04_16.xena&host=https://pancanatlas.xenahubs.net)</font>
- <font size="5">Protein Expression —— [TCGA-RPPA-pancan-clean.xena](https://xenabrowser.net/datapages/?dataset=TCGA-RPPA-pancan-clean.xena&host=https://pancanatlas.xenahubs.net)</font>
- <font size="5">Copy Number Variation —— [Gistic2_CopyNumber_Gistic2_all_data_by_genes](https://xenabrowser.net/datapages/?dataset=TCGA.PANCAN.sampleMap/Gistic2_CopyNumber_Gistic2_all_data_by_genes&host=https://tcga.xenahubs.net)</font>
- <font size="5">Mutation status —— [mc3.v0.2.8.PUBLIC.nonsilentGene.xena](https://xenabrowser.net/datapages/?dataset=mc3.v0.2.8.PUBLIC.nonsilentGene.xena&host=https://pancanatlas.xenahubs.net)</font>


### PCAWG

- <font size="5">mRNA Expression —— [tophat_star_fpkm_uq.v2_aliquot_gl.sp.log](https://xenabrowser.net/datapages/?dataset=tophat_star_fpkm_uq.v2_aliquot_gl.sp.log&host=https://pcawg.xenahubs.net)</font>
- <font size="5">miRNA Expression —— [x3t2m1.mature.TMM.mirna.matrix.log](https://xenabrowser.net/datapages/?dataset=x3t2m1.mature.TMM.mirna.matrix.log&host=https://pcawg.xenahubs.net)</font>
- <font size="5">Promoter Activity —— [rawPromoterActivity.sp](https://xenabrowser.net/datapages/?dataset=rawPromoterActivity.sp&host=https://pcawg.xenahubs.net)</font>
- <font size="5">Gene Fusion —— [pcawg3_fusions_PKU_EBI.gene_centric.sp.xena](https://xenabrowser.net/datapages/?dataset=pcawg3_fusions_PKU_EBI.gene_centric.sp.xena&host=https://pcawg.xenahubs.net)</font>
- <font size="5">APOBEC mutagenesis —— [MAF_Aug31_2016_sorted_A3A_A3B_comparePlus.sp](https://xenabrowser.net/datapages/?dataset=MAF_Aug31_2016_sorted_A3A_A3B_comparePlus.sp&host=https://pcawg.xenahubs.net)</font>



### CCLE

- <font size="5">mRNA Expression —— [ccle/CCLE_DepMap_18Q2_RNAseq_RPKM_20180502](https://xenabrowser.net/datapages/?dataset=ccle/CCLE_DepMap_18Q2_RNAseq_RPKM_20180502&host=https://ucscpublic.xenahubs.net)</font>
- <font size="5">Protein Expression —— [ccle/CCLE_RPPA_20180123](https://xenabrowser.net/datapages/?dataset=ccle/CCLE_RPPA_20180123&host=https://ucscpublic.xenahubs.net)</font>
- <font size="5">Copy Number Variation —— [ccle/CCLE_copynumber_byGene_2013-12-03](https://xenabrowser.net/datapages/?dataset=ccle/CCLE_copynumber_byGene_2013-12-03&host=https://ucscpublic.xenahubs.net)</font>



<br></br>

<br></br>



> <font size="5">Tips: The menu of `TPC Pipelins` supports more general and personalized TPC molecular analysis, including alternative datasets, precise data preparation and versatile modes.</font>

#### [PCAWG project types](https://dcc.icgc.org/releases/release_28/Projects)
<br>

| Study Abbreviation | Study Name                                                 |
| ------------------ | ---------------------------------------------------------- |
| BLCA-US            | Bladder Urothelial Cancer - TCGA, US                       |
| BRCA-US            | Breast Cancer - TCGA, US                                   |
| CESC-US            | Cervical Squamous Cell Carcinoma - TCGA, US                |
| CLLE-ES            | Chronic Lymphocytic Leukemia - ES                          |
| COAD-US            | Colon Adenocarcinoma - TCGA, US                            |
| DLBC-US            | Lymphoid Neoplasm Diffuse Large B-cell Lymphoma - TCGA, US |
| ESAD-UK            | Esophageal Adenocarcinoma - UK                             |
| GBM-US             | Brain Glioblastoma Multiforme - TCGA, US                   |
| HNSC-US            | Head and Neck Squamous Cell Carcinoma - TCGA, US           |
| KICH-US            | Kidney Chromophobe - TCGA, US                              |
| KIRC-US            | Kidney Renal Clear Cell Carcinoma - TCGA, US               |
| KIRP-US            | Kidney Renal Papillary Cell Carcinoma - TCGA, US           |
| LAML-US            | Acute Myeloid Leukemia - TCGA, US                          |
| LGG-US             | Brain Lower Grade Glioma - TCGA, US                        |
| LIHC-US            | Liver Hepatocellular carcinoma - TCGA, US                  |
| LIRI-JP            | Liver Cancer - RIKEN, JP                                   |
| LUAD-US            | Lung Adenocarcinoma - TCGA, US                             |
| LUSC-US            | Lung Squamous Cell Carcinoma - TCGA, US                    |
| MALY-DE            | Malignant Lymphoma - DE                                    |
| OV-AU              | Ovarian Cancer - AU                                        |
| OV-US              | Ovarian Serous Cystadenocarcinoma - TCGA, US               |
| PACA-AU            | Pancreatic Cancer - AU                                     |
| PRAD-US            | Prostate Adenocarcinoma - TCGA, US                         |
| READ-US            | Rectum Adenocarcinoma - TCGA, US                           |
| RECA-EU            | Renal Cell Cancer - EU/FR                                  |
| SARC-US            | Sarcoma - TCGA, US                                         |
| SKCM-US            | Skin Cutaneous melanoma - TCGA, US                         |
| STAD-US            | Gastric Adenocarcinoma - TCGA, US                          |
| THCA-US            | Head and Neck Thyroid Carcinoma - TCGA, US                 |
| UCEC-US            | Uterine Corpus Endometrial Carcinoma- TCGA, US             |

<font size="4">In **S3** Step, **individual survival analysis** can now be performed after data preparation:</font>

- <font size="4">Firstly, you can adjust the default analysis and visualization parameters, respectively.</font>
  - <font size="4">Use **Log-Rank Test** to compare survival distributions between two or more groups. Suitable for categorical variables (e.g., high vs. low gene expression groups).</font>
  - <font size="4">Use **Univariate Cox Regression** to estimate the effect (Hazard Ratio) of a single predictor on survival time. Suitable for continuous or binary predictor variables.</font>

- <font size="4">Noteworthily, you can try whether to use **initial data** before grouping if the data type is **numerical**.</font>

> 1. For log-rank test, it will decide the optimal cutoff;
> 2. For univariate cox regression, it will directly analyze based on the numerical data. 

- <font size="4">Then, you will obtain the curve (log-rank test) or forest (univariate cox regression) plot labeled with statistical result after clicking the "**Run**" button.</font>
- <font size="4">Finally, you can download the 3 types of results.</font>

> 1. "**Figure**": visualization result;
> 2. "**Raw data(.csv)**": queried data for X-axis and Y-axis;
> 3. "**Analyzed data(.csv)**": detailed correlation result.


<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures//helper_analyze_sur_1.png" alt="helper_analyze_sur_1" width="700"/>
</p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 

<center> <h1> Usage </h1> </center>

- [Home Page](#home-page)
  - [Overview](#overview)
  - [Quick search](#quick-search)
  - [Video](#video)
- [Repository Page](#repository-page)
  - [Overview](#overview-1)
  - [Dataset filter](#dataset-filter)
  - [Dataset selection](#dataset-selection)
  - [Dataset check](#dataset-check)
  - [Dataset download](#dataset-download)
  - [Video](#video-1)
- [General Analysis Page](#general-analysis-page)
  - [Overview](#overview-2)
  - [Video](#video-2)
- [Quick PanCan Analysis Page](#quick-pancan-analysis-page)
  - [Overview](#overview-3)
  - [Notes](#notes)
  - [Videos](#videos)
- [Global Setting Page](#global-setting-page)
- [Feature: Genomic Signature](#feature-genomic-signature)
  - [Overview](#overview-4)
  - [Video](#video-3)
- [Use UCSCXenaShiny R package](#use-ucscxenashiny-r-package)

## Home Page
### Overview

Home page gives you a quick overview of this tool, which includes

- basic information and statistics about supported data hubs.
- current version of this tool and its key dependent package "UCSCXenaTools".
- where the datasets come from.

If you use this tool locally, sometimes the version number can help you check if there is a update of this tool and track any bug when you report to the developers.

### Quick search

The web GUI is mainly designed and developed for cancer researchers without programming experience or want to quickly explore the flash of inspiration. We provide a search bar on the top of this page, you can type the gene symbol (e.g. *TP53*) and a window should pop up to show the pan-cancer expression distribution of this gene.

If you want to keep this data, your either right click to save this image or please go to module "TCGA: Molecular Profile Distribution Across Cancer Types (Tumor VS Normal)" under "Quick PanCan Analysis" page. The analysis module is more powerful and provides many features to custom this figure for academic use.

NOTE: *Although we label the module with TCGA, the data come from [toil data hub](https://toil.xenahubs.net), it contains samples from TCGA, TARGET and GTEx databases*.

### Video

[Click watch operations on bilibili](https://www.bilibili.com/video/bv1R84y1F7WT)

## Repository Page

### Overview

This page provides many features related to datasets of UCSC Xena and the entry to "General Analysis". This section we will only focus on the former part, for "General Analysis", please refer to the next section.

Repository page shows multiple filters on the left panels, a dataset table on the main panel and 3 buttons on the bottom.
These widgets are combined to create a full-feature page for dataset filtering, checking and downloading.

### Dataset filter

At default, when you open this page, you will find a table showing hundreds of datasets. To quickly locate what you are looking for, you can use filters from different aspects:

1. Data hub (when you don't select any data hub, all datasets will be shown in the right data table).
2. Cohort name. A cohort contains multiple datasets for same samples. So it is the common target for a analysis.
3. Data type.
4. Data subtype.

Besides, you can use the "Filter with keyword" search bar on the top-right of the dataset table to filter the datasets with any keyword.

### Dataset selection

**The filtered dataset table are not selected, you have to use your mouse to click one or more data rows to select corresponding datasets.**
After your selection, the tool will repond you selection to query the extra information (e.g., download link, the corresponding UCSC Xena page) of the datasets from UCSC Xena server.

If you just want to download datasets, a simple way is to click the download link one by one.

After dataset selection, you can use the 3 buttons below the table to

1. Check the metadata of the datasets.
2. Request (download) datasets by multiple ways.
3. Analyze datasets (when you click this button, it will automatically jump to "General Analysis" page).

### Dataset check

After you click the "Show Metadata" button, a windown will pop up to show the detail metadata of your selected datasets.
### Dataset download

After you click the "Request Data" button, a windown will pop up to show the key information of the datasets and 3 ways to download the data:

1. "Download data directly" will download datasets from UCSC Xena data hubs and zip the datasets to provides download by your browser.
2. "Batch download" will provide a `.sh` script to you, so you can run the download process in a Unix (Linux/MacOS) environment (`wget` is required in the environment). This is recommended if you selected many big datasets.
3. "Copy R download code" will show the R code to download the datasets, so you can integerate it with your following data analysis code and enhance data analysis reproducibility.

### Video

[Click watch operations on bilibili](https://www.bilibili.com/video/bv1Mq4y1J73o)

## General Analysis Page

### Overview

The 3rd button below the dataset table in "Repository" page is "Analyze Data". When you click the button, the tool will be automatically switched to this "General Analysis" page.

(To enhance the analysis, all phenotype datasets in the same cohort will be automatically queried and loaded.)

4 analysis modules are provided for solving common analysis tasks.

1. Scatter-Correlation: analyze and visualize correlation for two variables.
2. Matrix-Correlation: analyze and visualize correlation for two or more variables.
3. Group-Comparison: analyze and visualize numeric difference of a continuous variable between multiple groups of a variable.
4. Survival-Analysis: analyze and visualize survival curves between multiple groups of a variable.

On the right panel of each analysis module, we provide a sample filter. If you want to use a sample subset of the dataset for analysis, you should click the "Click to filter!" button and follow the hint step by step.

### Video

[Click watch operations on bilibili](https://www.bilibili.com/video/bv1p64y1U7iU)

## Quick PanCan Analysis Page

### Overview

This page provides many indepedent analysis modules for TCGA and CCLE related data. They are very simple, useful and easy to get started.

For most of modules provided here, multiple types of molecules including mRNA, transcript, mutation, etc. are supported. In most cases, you can just put **gene symbol** (e.g., *TP53*). When you choose to explore the transcipt and miRNA expression, you need iuput the **Ensembl ID** (e.g., **ENST00000000233**) and **miRNA ID** (e.g., **hsa-miR-128-3p**).

### Notes

1. The DNA methylation data is gene level, it is obtained from the average of all probes mapping to a gene.
2. The copy number varation value is obtained from the GISTIC2 software, if the numeric value is used, `0` indicates the normal copy.
3. The definition and data of four major clinical outcome endpoints are from reference [***doi: 10.1016/j.cell.2018.02.052***](https://pubmed.ncbi.nlm.nih.gov/29625055/).
4. For TCGA survival analysis, when you choose **Auto** for **cutoff mode**, the function of **surv_cutpoint {survminer package}** will be called. The method is described as *Determine the optimal cutpoint for one or multiple continuous variables at once, using the maximally selected rank statistics from the 'maxstat' R package. This is an outcome-oriented methods providing a value of a cutpoint that correspond to the most significant relation with outcome (here for survival data)*.

### Video

[Click watch operations on bilibili](https://www.bilibili.com/video/bv1o64y1y7jW)

## Global Setting Page

This page is used to display widgets to control global settings of the Shiny tool.

Currently, only the mirror setting is supported.

## Feature: Genomic Signature

### Overview

Inspired by UCSC Xena browser, feature "genomic signature" is also supported in this tool. For most of analysis modules, this feature is deployed and work well.

To use this feature, you just type a formula instead of a molecule identifier.

For example, if you want to see TCGA mRNA expression distribution of a signature generated by `TP53 + 1.5 * KRAS`. Firstly, delete the default symbol in the input bar "Input a gene or formula (as signature)", then input `TP53 + 1.5 * KRAS`, click the shown "Add TP53 + 1.5 * KRAS", run analysis as usual. **When you use this feature, you must input a space in the formula, so we can understand that you are inputting a signature instead of a normal symbol like gene name**.

NOTE: the signature defined in two CCLE drug related analysis modules is different from others shown above. In the CCLE drug related analysis modules, you just select multiple genes, a signature is constructed from geometrical mean of mRNA expression of your input gene list.

### Video

[Click watch operations on bilibili](https://www.bilibili.com/video/bv1zB4y1c7eB)

## Use UCSCXenaShiny R package

For most data query and analysis, we implemented them as R functions available in our R package [`{UCSCXenaShiny}`](https://github.com/openbiox/UCSCXenaShiny).
To use the data and functions provided by us, please check the [reference list](https://openbiox.github.io/UCSCXenaShiny/reference/index.html).

---
title: "Exploratory Molecule Analysis of UCSCXenashiny v2.0"
subtitle: "TCGA Pan-Cancer Report for `r params$id_type` `r params$id_name`"
author: 
  - name: "Shensuo Li, UCSCXenashiny developer team"
    email: lishensuo@163.com
date: "`r Sys.Date()`"
params:
  sur_res: NA
  cor_res: NA
  phe_res: NA
  id_name: NA
  id_type: NA
  
output: 
  html_document:
    theme: journal
    toc: yes
    toc_float:
      collapsed: true
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)

cor_res = params$cor_res
sur_res = params$sur_res
phe_res = params$phe_res
id_name = params$id_name
id_type = params$id_type

colnames(phe_res)[1] = "Cancer"
phe_res = phe_res %>% 
  dplyr::mutate(value = round(value, 3)) %>% 
  dplyr::mutate(P.format = format(P.value, digits = 3, scientific=TRUE)) %>% 
  dplyr::mutate(P.label = case_when(
    P.value<0.001 ~ "***",
    P.value<0.01 ~ "**",
    P.value<0.05 ~ "*",
    TRUE ~ " "
  ))

colnames(cor_res)[4:6] = c("Cancer", "R", "P.value")
cor_res = cor_res %>%
  dplyr::mutate(Cancer = factor(Cancer, levels = sort(unique(Cancer),decreasing = T))) %>% 
  dplyr::mutate(R = round(R, 3)) %>% 
  dplyr::mutate(P.format = format(P.value, digits = 3, scientific=TRUE)) %>% 
  dplyr::mutate(P.label = case_when(
    P.value<0.001 ~ "***",
    P.value<0.01 ~ "**",
    P.value<0.05 ~ "*",
    TRUE ~ " "
  ))

colnames(sur_res)[1:3] = c("Event", "Cancer", "P.value")
sur_res = sur_res %>% 
  dplyr::mutate(P.format = format(P.value, digits = 3, scientific=TRUE)) 


link_set = switch(id_type,
                  "mRNA"="https://xenabrowser.net/datapages/?dataset=tcga_RSEM_gene_tpm&host=https://toil.xenahubs.net",
                  "transcript"="https://xenabrowser.net/datapages/?dataset=tcga_rsem_isoform_tpm&host=https://toil.xenahubs.net",
                  "protein"="https://xenabrowser.net/datapages/?dataset=TCGA-RPPA-pancan-clean.xena&host=https://pancanatlas.xenahubs.net",
                  "mutation"="https://xenabrowser.net/datapages/?dataset=mc3.v0.2.8.PUBLIC.nonsilentGene.xena&host=https://pancanatlas.xenahubs.net",
                  "cnv"="https://xenabrowser.net/datapages/?dataset=broad.mit.edu_PANCAN_Genome_Wide_SNP_6_whitelisted.gene.xena&host=https://pancanatlas.xenahubs.net",
                  "methylation"="https://xenabrowser.net/datapages/?dataset=GDC-PANCAN.methylation450.tsv&host=https://gdc.xenahubs.net",
                  "miRNA"="https://xenabrowser.net/datapages/?dataset=pancanMiRs_EBadjOnProtocolPlatformWithoutRepsWithUnCorrectMiRs_08_04_16.xena&host=https://pancanatlas.xenahubs.net"
                  )

```




# Introduction
This report is generated using the results obtained from the `UCSCXenaShiny::mol_quick_analysis` function. The analysis encompasses 5 sections, each executed for `r id_type` ``r id_name`` sourced from the [dataset](`r link_set`) in UCSC Xena TOIL(TCGA) hub. [All TCGA projects](https://gdc.cancer.gov/resources-tcga-users/tcga-code-tables/tcga-study-abbreviations) are analyzed individually and some cancer projects may be missing due to few samples with valid data for some analysis. The notation for statistical significance analysis is as follows: * indicates a P-value less than 0.05, ** indicates a P-value less than 0.01, and *** indicates a P-value less than 0.001. 

Please note that our [Shiny app](https://shiny.hiplot.cn/ucsc-xena-shiny/) offers more comprehensive exploration possibilities, including alternative datasets for the same molecule, alternative analysis methods, molecule-molecule analysis, personalized sample filtering and grouping, result visualization, and download options.


# 1. Clinical phenotype analysis
This part compares the distribution of `r id_type` ``r id_name`` in Normal group samples using across all TCGA projects. For mRNA and transcript data type, samples from GTEx project are also included in Normal groups.


## 1.1 Normal tissues
```{r, phe_0, echo=FALSE}
normal_dat = phe_res %>% 
  dplyr::filter(name=="normal")

p = normal_dat %>%
  dplyr::arrange(value) %>%
  dplyr::mutate(Cancer = factor(Cancer, levels=Cancer)) %>%
  ggplot(aes(x = value, y = Cancer, fill = Cancer)) +
  geom_col() +
  xlab("Molecule value in normal tissues") + ylab(NULL) +
  theme_bw() +
  theme(legend.position = "none")

ggplotly(p)
```





## 1.2 Tumor&Normal 
This part compares the distribution of `r id_type` ``r id_name`` between Tumor and Normal group samples using the Wilcoxon test across all TCGA projects. For mRNA and transcript data type, samples from GTEx project are also included in Normal groups.

"Higher Exp." indicates higher value in Tumor groups, whereas "Lower Exp." indicates lower value in Tumor groups. The red vertical line indicates that P equals 0.05.


```{r, phe_1, echo=FALSE}
phe_res_TN = subset(phe_res, name %in% c("normal","tumor"))
phe_res_TN$name = ifelse(phe_res_TN$name=="normal","Normal", "Tumor")
phe_res_TN = phe_res_TN %>% 
  tidyr::pivot_wider(names_from = "name", values_from = "value")
colnames(phe_res_TN)[1] = "Cancer"

phe_res_TN[,c(1,5,6,3,4)] %>%
  dplyr::rename(P.value=P.format) %>%
  dplyr::mutate(P.direction = ifelse(Tumor>Normal,"Higher Exp.","Lower Exp.")) %>% 
  datatable(.,filter = 'top',
            options = list(pageLength = 5,
						columnDefs = list(list(className = 'dt-center', targets="_all"))))
```

<center>
```{r, phe_1_plot, echo=FALSE, fig.height=4.5, fig.width=7}
p = phe_res_TN %>% 
  dplyr::mutate(P.direction = ifelse(Tumor>Normal,"Higher Exp.","Lower Exp.")) %>% 
  dplyr::mutate(P.log = -log10(P.value)) %>%
  dplyr::arrange(P.log) %>%
  dplyr::mutate(Cancer = factor(Cancer, levels=Cancer)) %>%
  ggplot(aes(x = P.log, y = Cancer, fill = P.direction, text = paste0("P.value: ", P.format))) +
  geom_col() +
  scale_fill_manual(values = c("Higher Exp."="#fc8d59","Lower Exp."="#99d594")) +
  geom_vline(xintercept = -log10(0.05), 
           # linetype="dashed", 
           color="red", linewidth=0.3) +
  xlab("-log10(P.value)") + ylab(NULL) +
  theme_bw()
ggplotly(p)
```
</center>

## 1.3 Age  
This part compares the distribution of `r id_type` ``r id_name`` between Old and Young group samples using the Wilcoxon test across all TCGA projects. Samples aged 60 and above are categorized into the Old group, while those below 60 are grouped into the Young group.

"Higher Exp." indicates higher value in Old groups, whereas "Lower Exp." indicates lower value in Old groups. The red vertical line indicates that P equals 0.05.

```{r, phe_2, echo=FALSE}
phe_res_Age = subset(phe_res, name %in% c('Old','Young'))
phe_res_Age = phe_res_Age %>% 
  tidyr::pivot_wider(names_from = "name", values_from = "value")
colnames(phe_res_Age)[1] = "Cancer"

phe_res_Age[,c(1,6,5,3,4)] %>%
  dplyr::rename(P.value=P.format) %>%
  dplyr::mutate(P.direction = ifelse(Old>Young,"Higher Exp.","Lower Exp.")) %>% 
  datatable(.,filter = 'top',
            options = list(pageLength = 5,
						columnDefs = list(list(className = 'dt-center', targets="_all"))))
```

<center>
```{r, phe_2_plot, echo=FALSE, fig.height=4.5, fig.width=7}
p = phe_res_Age %>% 
  dplyr::mutate(P.direction = ifelse(Old>Young,"Higher Exp.","Lower Exp.")) %>% 
  dplyr::mutate(P.log = -log10(P.value)) %>%
  dplyr::arrange(P.log) %>%
  dplyr::mutate(Cancer = factor(Cancer, levels=Cancer)) %>%
  ggplot(aes(x = P.log, y = Cancer, fill = P.direction, text = paste0("P.value: ", P.format))) +
  geom_col() +
  scale_fill_manual(values = c("Higher Exp."="#fc8d59","Lower Exp."="#99d594")) +
  geom_vline(xintercept = -log10(0.05), 
         # linetype="dashed", 
         color="red", linewidth=0.3) +
  xlab("-log10(P.value)") + ylab(NULL) +
  theme_bw()
ggplotly(p)
```
</center>


## 1.4 Gender 
This part compares the distribution of `r id_type` ``r id_name`` between Male and Female group samples using the Wilcoxon test across all TCGA projects. 

"Higher Exp." indicates higher value in Male groups, whereas "Lower Exp." indicates lower value in Male groups. The red vertical line indicates that P equals 0.05.

```{r, phe_3, echo=FALSE}
phe_res_Gender = subset(phe_res, name %in% c("FEMALE","MALE"))
phe_res_Gender$name = ifelse(phe_res_Gender$name=="FEMALE","Female", "Male")
phe_res_Gender = phe_res_Gender %>% 
  tidyr::pivot_wider(names_from = "name", values_from = "value")

phe_res_Gender[,c(1,5,6,3,4)] %>%
  dplyr::rename(P.value=P.format) %>%
  dplyr::mutate(P.direction = ifelse(Male>Female,"Higher Exp.","Lower Exp.")) %>% 
  datatable(.,filter = 'top',
            options = list(pageLength = 5,
						columnDefs = list(list(className = 'dt-center', targets="_all"))))
```

<center>
```{r, phe_3_plot, echo=FALSE, fig.height=4.5, fig.width=7}
p = phe_res_Gender %>% 
  dplyr::mutate(P.direction = ifelse(Male>Female,"Higher Exp.","Lower Exp.")) %>% 
  dplyr::mutate(P.log = -log10(P.value)) %>%
  dplyr::arrange(P.log) %>%
  dplyr::mutate(Cancer = factor(Cancer, levels=Cancer)) %>%
  ggplot(aes(x = P.log, y = Cancer, fill = P.direction, text = paste0("P.value: ", P.format))) +
  geom_col() +
  scale_fill_manual(values = c("Higher Exp."="#fc8d59","Lower Exp."="#99d594")) +
  geom_vline(xintercept = -log10(0.05), 
         # linetype="dashed", 
         color="red", linewidth=0.3) +
  xlab("-log10(P.value)") + ylab(NULL) +
  theme_bw()
ggplotly(p)
```
</center>


## 1.5 Stage  
This part compares the distribution of `r id_type` ``r id_name`` among samples in different AJCC Stage using the ANOVA analysis across all TCGA projects.

The red vertical line indicates that P equals 0.05.
```{r, phe_4, echo=FALSE}
phe_res_Stage = subset(phe_res, grepl("Stage",name))
phe_res_Stage = phe_res_Stage %>% 
  tidyr::pivot_wider(names_from = "name", values_from = "value")
colnames(phe_res_Stage)[1] = "Cancer"

phe_res_Stage[,c(1,5,6,7,8, 3,4)] %>%
  dplyr::rename(P.value=P.format) %>%
  datatable(.,filter = 'top',
            options = list(pageLength = 5,
						columnDefs = list(list(className = 'dt-center', targets="_all"))))
```

<center>
```{r, phe_4_plot, echo=FALSE, fig.height=4.5, fig.width=7}
p = phe_res_Stage %>% 
  dplyr::mutate(P.log = -log10(P.value)) %>%
  dplyr::arrange(P.log) %>%
  dplyr::mutate(Cancer = factor(Cancer, levels=Cancer)) %>%
  ggplot(aes(x = P.log, y = Cancer, text = paste0("P.value: ", P.format))) +
  geom_col(fill = "#80b1d3") +
  xlab("-log10(P.value)") + ylab(NULL) +
  geom_vline(xintercept = -log10(0.05), 
         # linetype="dashed", 
         color="red", linewidth=0.3) +
  theme_bw()
ggplotly(p)
```
</center>




# 2. Survival analysis
## 2.1 OS
This part includes the log-rank test between higher and lower `r id_type` ``r id_name`` groups based on median value in primary tumor samples with valid Overall Survival (OS) data across all TCGA projects.

"Higher risk" indicates higher risk in the group with elevated molecular expression, whereas "Lower risk" indicates lower risk in the group with elevated molecular expression. The red vertical line indicates that P equals 0.05.

```{r sur_1, echo = FALSE}
sur_res_sub = subset(sur_res, Event == "OS") %>%
  tibble::remove_rownames()

sur_res_sub[,c(1,2,5,4)] %>%
  dplyr::rename(P.value=P.format) %>%
  datatable(.,filter = 'top',
           options = list(pageLength = 5,
						columnDefs = list(list(className = 'dt-center', targets="_all"))))

```

<center>
```{r, sur_1_plot, echo = FALSE, fig.width=6, fig.height=4.5}
p = sur_res_sub %>% na.omit() %>% 
  dplyr::mutate(P.log = -log10(P.value)) %>%
  dplyr::arrange(P.log) %>%
  dplyr::mutate(Cancer = factor(Cancer, levels=Cancer)) %>%
  ggplot(aes(x = P.log, y = Cancer, fill = P.direction, text = paste0("P.value: ", P.format))) +
  geom_col() +
  scale_fill_manual(values = c("Higher Risk"="#fc8d59","Lower Risk"="#99d594")) +
  geom_vline(xintercept = -log10(0.05), 
         # linetype="dashed", 
         color="red", linewidth=0.3) +
  xlab("-log10(P.value)") + ylab(NULL) +
  theme_bw()

ggplotly(p)
```
</center>


## 2.2 PFI
This part includes the log-rank test between higher and lower `r id_type` ``r id_name`` groups based on median value in primary tumor samples with valid Progression-Free Interval (PFI) data across all TCGA projects.

"Higher risk" indicates higher risk in the group with elevated molecular expression, whereas "Lower risk" indicates lower risk in the group with elevated molecular expression. The red vertical line indicates that P equals 0.05.

```{r sur_2, echo = FALSE}
sur_res_sub = subset(sur_res, Event == "PFI") %>%
  tibble::remove_rownames()

sur_res_sub[,c(1,2,5,4)] %>%
  dplyr::rename(P.value=P.format) %>%
  datatable(.,filter = 'top',
           options = list(pageLength = 5,
						columnDefs = list(list(className = 'dt-center', targets="_all"))))

```

<center>
```{r, sur_2_plot, echo = FALSE, fig.width=6, fig.height=4.5}
p = sur_res_sub %>% na.omit() %>% 
  dplyr::mutate(P.log = -log10(P.value)) %>%
  dplyr::arrange(P.log) %>%
  dplyr::mutate(Cancer = factor(Cancer, levels=Cancer)) %>%
  ggplot(aes(x = P.log, y = Cancer, fill = P.direction, text = paste0("P.value: ", P.format))) +
  geom_col() +
  scale_fill_manual(values = c("Higher Risk"="#fc8d59","Lower Risk"="#99d594")) +
  geom_vline(xintercept = -log10(0.05), 
         # linetype="dashed", 
         color="red", linewidth=0.3) +
  xlab("-log10(P.value)") + ylab(NULL) +
  theme_bw()

ggplotly(p)
```
</center>

## 2.3 DFI
This part includes the log-rank test between higher and lower `r id_type` ``r id_name`` groups based on median value in primary tumor samples with valid Disease-Free Interval (DFI) data across all TCGA projects.

"Higher risk" indicates higher risk in the group with elevated molecular expression, whereas "Lower risk" indicates lower risk in the group with elevated molecular expression. The red vertical line indicates that P equals 0.05.

```{r sur_3, echo = FALSE}
sur_res_sub = subset(sur_res, Event == "DFI") %>%
  tibble::remove_rownames()

sur_res_sub[,c(1,2,5,4)] %>%
  dplyr::rename(P.value=P.format) %>%
  datatable(.,filter = 'top',
           options = list(pageLength = 5,
						columnDefs = list(list(className = 'dt-center', targets="_all"))))

```

<center>
```{r, sur_3_plot, echo = FALSE, fig.width=6, fig.height=4.5}
p = sur_res_sub %>% na.omit() %>% 
  dplyr::mutate(P.log = -log10(P.value)) %>%
  dplyr::arrange(P.log) %>%
  dplyr::mutate(Cancer = factor(Cancer, levels=Cancer)) %>%
  ggplot(aes(x = P.log, y = Cancer, fill = P.direction, text = paste0("P.value: ", P.format))) +
  geom_col() +
  scale_fill_manual(values = c("Higher Risk"="#fc8d59","Lower Risk"="#99d594")) +
  geom_vline(xintercept = -log10(0.05), 
         # linetype="dashed", 
         color="red", linewidth=0.3) +
  xlab("-log10(P.value)") + ylab(NULL) +
  theme_bw()

ggplotly(p)
```
</center>


## 2.4 DSS
This part includes the log-rank test between higher and lower `r id_type` ``r id_name`` groups based on median value in primary tumor samples with valid Disease-Specific Survival (DSS) survival data across all TCGA projects.

"Higher risk" indicates higher risk in the group with elevated molecular expression, whereas "Lower risk" indicates lower risk in the group with elevated molecular expression. The red vertical line indicates that P equals 0.05.

```{r sur_4, echo = FALSE}
sur_res_sub = subset(sur_res, Event == "DSS") %>%
  tibble::remove_rownames()

sur_res_sub[,c(1,2,5,4)] %>%
  dplyr::rename(P.value=P.format) %>%
  datatable(.,filter = 'top',
           options = list(pageLength = 5,
						columnDefs = list(list(className = 'dt-center', targets="_all"))))

```

<center>
```{r, sur_4_plot, echo = FALSE, fig.width=6, fig.height=4.5}
p = sur_res_sub %>% na.omit() %>% 
  dplyr::mutate(P.log = -log10(P.value)) %>%
  dplyr::arrange(P.log) %>%
  dplyr::mutate(Cancer = factor(Cancer, levels=Cancer)) %>%
  ggplot(aes(x = P.log, y = Cancer, fill = P.direction, text = paste0("P.value: ", P.format))) +
  geom_col() +
  scale_fill_manual(values = c("Higher Risk"="#fc8d59","Lower Risk"="#99d594")) +
  geom_vline(xintercept = -log10(0.05), 
         # linetype="dashed", 
         color="red", linewidth=0.3) +
  xlab("-log10(P.value)") + ylab(NULL) +
  theme_bw()

ggplotly(p)
```
</center>




# 3. Tumor index association
## 3.1 Tumor Stemness
This part includes the spearman correlation of `r id_type` ``r id_name`` with **`r length(unique(cor_res$L3[cor_res$L2=="Tumor Stemness"]))`** common Tumor Stemness related items in primary tumor samples across all TCGA projects. They are `r sort(unique(cor_res$L3[cor_res$L2=="Tumor Stemness"]))`.



```{r idx_1, echo = FALSE}
cor_res_sub = cor_res %>%
  dplyr::filter(L1 == "Tumor index") %>%
  dplyr::filter(L2 == "Tumor Stemness") %>%
  dplyr::rename(`Tumor Stemness` = L3)


cor_res_sub[,c(-1, -2, -6)] %>%
  dplyr::rename(P.value=P.format) %>%
  datatable(.,filter = 'top',
            options = list(pageLength = 5,
						columnDefs = list(list(className = 'dt-center', targets="_all"))))
```


<center>
```{r idx_1_plot, echo = FALSE, fig.width=6, fig.height=5}
p = cor_res_sub %>%
  ggplot(aes(x = `Tumor Stemness`, y = Cancer, text = paste0("R: ", R))) +
  geom_tile(aes(fill = R)) +
  geom_text(aes(label=P.label), nudge_y = -0.2, size = 3)+
  xlab(NULL) + ylab(NULL) +
  scale_fill_gradient2(low = "green", mid = "white", high = "red",
                        breaks = c(-0.4,0,0.4)) +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 45, hjust=1))
# p
ggplotly(p)
```
</center>


## 3.2 Tumor Mutation Burden
This part includes the spearman correlation of `r id_type` ``r id_name`` with **`r length(unique(cor_res$L3[cor_res$L2=="Tumor Mutation Burden"]))`** common Tumor Mutation Burden related items in primary tumor samples across all TCGA projects. They are `r sort(unique(cor_res$L3[cor_res$L2=="Tumor Mutation Burden"]))`.


```{r idx_2, echo = FALSE}
cor_res_sub = cor_res %>%
  dplyr::filter(L1 == "Tumor index") %>%
  dplyr::filter(L2 == "Tumor Mutation Burden") %>%
  dplyr::rename(`Tumor Mutation Burden` = L3)


cor_res_sub[,c(-1, -2, -6)] %>%
  dplyr::rename(P.value=P.format) %>%
  datatable(.,filter = 'top',
            options = list(pageLength = 5,
						columnDefs = list(list(className = 'dt-center', targets="_all"))))
```


<center>
```{r idx_2_plot, echo = FALSE, fig.width=6, fig.height=5}
p = cor_res_sub %>%
  ggplot(aes(x = `Tumor Mutation Burden`, y = Cancer, text = paste0("R: ", R))) +
  geom_tile(aes(fill = R)) +
  geom_text(aes(label=P.label), nudge_y = -0.2, size = 3)+
  xlab(NULL) + ylab(NULL) +
  scale_fill_gradient2(low = "green", mid = "white", high = "red",
                        breaks = c(-0.4,0,0.4)) +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 45, hjust=1))
# p
ggplotly(p)
```
</center>



## 3.3 Genome Instability
This part includes the spearman correlation of `r id_type` ``r id_name`` with **`r length(unique(cor_res$L3[cor_res$L2=="Genome Instability"]))`** common Genome Instability related items in primary tumor samples across all TCGA projects. They are `r sort(unique(cor_res$L3[cor_res$L2=="Genome Instability"]))`.



```{r idx_3, echo = FALSE}
cor_res_sub = cor_res %>%
  dplyr::filter(L1 == "Tumor index") %>%
  dplyr::filter(L2 == "Genome Instability") %>%
  dplyr::rename(`Genome Instability` = L3)


cor_res_sub[,c(-1, -2, -6)] %>%
  dplyr::rename(P.value=P.format) %>%
  datatable(.,filter = 'top',
            options = list(pageLength = 5,
						columnDefs = list(list(className = 'dt-center', targets="_all"))))
```


<center>
```{r idx_3_plot, echo = FALSE, fig.width=6, fig.height=5}
p = cor_res_sub %>%
  ggplot(aes(x = `Genome Instability`, y = Cancer, text = paste0("R: ", R))) +
  geom_tile(aes(fill = R)) +
  geom_text(aes(label=P.label), nudge_y = -0.2, size = 3)+
  xlab(NULL) + ylab(NULL) +
  scale_fill_gradient2(low = "green", mid = "white", high = "red",
                        breaks = c(-0.4,0,0.4)) +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 45, hjust=1))
# p
ggplotly(p)
```
</center>




# 4. Immune infiltration association 
## 4.1 CIBERSORT
This part includes the spearman correlation of `r id_type` ``r id_name`` with **`r length(unique(cor_res$L3[cor_res$L2=="CIBERSORT"]))`** CIBERSORT based items in primary tumor samples across all TCGA projects. They are `r sort(unique(cor_res$L3[cor_res$L2=="CIBERSORT"]))`.



```{r immu_1, echo = FALSE}
cor_res_sub = cor_res %>%
  dplyr::filter(L1 == "Immune Infiltration") %>%
  dplyr::filter(L2 == "CIBERSORT") %>%
  dplyr::rename(`CIBERSORT` = L3)


cor_res_sub[,c(-1, -2, -6)] %>%
  dplyr::rename(P.value=P.format) %>%
  datatable(.,filter = 'top',
            options = list(pageLength = 5,
						columnDefs = list(list(className = 'dt-center', targets="_all"))))
```


<center>
```{r immu_1_plot, echo = FALSE, fig.width=7, fig.height=6}
p = cor_res_sub %>%
  ggplot(aes(x = `CIBERSORT`, y = Cancer, text = paste0("R: ", R))) +
  geom_tile(aes(fill = R)) +
  geom_text(aes(label=P.label), nudge_y = -0.2, size = 3)+
  xlab(NULL) + ylab(NULL) +
  scale_fill_gradient2(low = "green", mid = "white", high = "red",
                        breaks = c(-0.4,0,0.4)) +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 45, hjust=1))
# p
ggplotly(p)
```
</center>



## 4.2 TIMER
This part includes the spearman correlation of `r id_type` ``r id_name`` with **`r length(unique(cor_res$L3[cor_res$L2=="TIMER"]))`** TIMER based items in primary tumor samples across all TCGA projects. They are `r sort(unique(cor_res$L3[cor_res$L2=="TIMER"]))`.

```{r immu_2, echo = FALSE}
cor_res_sub = cor_res %>%
  dplyr::filter(L1 == "Immune Infiltration") %>%
  dplyr::filter(L2 == "TIMER") %>%
  dplyr::rename(`TIMER` = L3)


cor_res_sub[,c(-1, -2, -6)] %>%
  dplyr::rename(P.value=P.format) %>%
  datatable(.,filter = 'top',
            options = list(pageLength = 5,
						columnDefs = list(list(className = 'dt-center', targets="_all"))))
```


<center>
```{r immu_2_plot, echo = FALSE, fig.width=7, fig.height=6}
p = cor_res_sub %>%
  ggplot(aes(x = `TIMER`, y = Cancer, text = paste0("R: ", R))) +
  geom_tile(aes(fill = R)) +
  geom_text(aes(label=P.label), nudge_y = -0.2, size = 3)+
  xlab(NULL) + ylab(NULL) +
  scale_fill_gradient2(low = "green", mid = "white", high = "red",
                        breaks = c(-0.4,0,0.4)) +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 45, hjust=1))
# p
ggplotly(p)
```
</center>


## 4.3 XCELL
This part includes the spearman correlation of `r id_type` ``r id_name`` with **`r length(unique(cor_res$L3[cor_res$L2=="XCELL"]))`** XCELL based items in primary tumor samples across all TCGA projects. They are `r sort(unique(cor_res$L3[cor_res$L2=="XCELL"]))`.

```{r immu_3, echo = FALSE}
cor_res_sub = cor_res %>%
  dplyr::filter(L1 == "Immune Infiltration") %>%
  dplyr::filter(L2 == "XCELL") %>%
  dplyr::rename(`XCELL` = L3)


cor_res_sub[,c(-1, -2, -6)] %>%
  dplyr::rename(P.value=P.format) %>%
  datatable(.,filter = 'top',
            options = list(pageLength = 5,
						columnDefs = list(list(className = 'dt-center', targets="_all"))))
```


<center>
```{r immu_3_plot, echo = FALSE, fig.width=9, fig.height=6}
p = cor_res_sub %>%
  ggplot(aes(x = `XCELL`, y = Cancer, text = paste0("R: ", R))) +
  geom_tile(aes(fill = R)) +
  geom_text(aes(label=P.label), nudge_y = -0.2, size = 2)+
  xlab(NULL) + ylab(NULL) +
  scale_fill_gradient2(low = "green", mid = "white", high = "red",
                        breaks = c(-0.4,0,0.4)) +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 45, hjust=1))
# p
ggplotly(p)
```
</center>



# 5. Pathway activity association

## 5.1 HALLMARK
This part includes the spearman correlation of `r id_type` ``r id_name`` with **`r length(unique(cor_res$L3[cor_res$L2=="HALLMARK"]))`** HALLMARK originated items in primary tumor samples across all TCGA projects. See all the items  [here](https://www.gsea-msigdb.org/gsea/msigdb/human/genesets.jsp?collection=H).

```{r pw_1, echo = FALSE}
cor_res_sub = cor_res %>%
  dplyr::filter(L1 == "Pathway activity") %>%
  dplyr::filter(L2 == "HALLMARK") %>%
  dplyr::rename(`HALLMARK` = L3)


cor_res_sub[,c(-1, -2, -6)] %>%
  dplyr::rename(P.value=P.format) %>%
  datatable(.,filter = 'top',
            options = list(pageLength = 5,
						columnDefs = list(list(className = 'dt-center', targets="_all"))))
```


<center>
```{r pw_1_plot, echo = FALSE, fig.width=10, fig.height=6.5}
p = cor_res_sub %>%
  ggplot(aes(x = `HALLMARK`, y = Cancer, text = paste0("R: ", R))) +
  geom_tile(aes(fill = R)) +
  geom_text(aes(label=P.label), nudge_y = -0.2, size = 1.7)+
  xlab(NULL) + ylab(NULL) +
  scale_fill_gradient2(low = "green", mid = "white", high = "red",
                        breaks = c(-0.4,0,0.4)) +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 45, hjust=1, size = 7))
# p
ggplotly(p)
```
</center>



## 5.2 KEGG
This part includes the spearman correlation of `r id_type` ``r id_name`` with **`r length(unique(cor_res$L3[cor_res$L2=="KEGG"]))`** KEGG originated items in primary tumor samples across all TCGA projects. See all the items  [here](https://www.gsea-msigdb.org/gsea/msigdb/human/genesets.jsp?collection=CP:KEGG_LEGACY). Top 50 most significant terms are selected for visualization.

```{r pw_2, echo = FALSE}
cor_res_sub = cor_res %>%
  dplyr::filter(L1 == "Pathway activity") %>%
  dplyr::filter(L2 == "KEGG") %>%
  dplyr::rename(`KEGG` = L3)


cor_res_sub[,c(-1, -2, -6)] %>%
  dplyr::rename(P.value=P.format) %>%
  datatable(.,filter = 'top',
            options = list(pageLength = 5,
						columnDefs = list(list(className = 'dt-center', targets="_all"))))
```


<center>
```{r pw_2_plot, echo = FALSE, fig.width=10, fig.height=6.5}
# 选择50条显著通路
KEGG_sle = cor_res_sub %>%
  dplyr::group_by(KEGG) %>%
  dplyr:::summarise(n = sum(P.label==" ")) %>%
  dplyr::arrange(n) %>% head(50) %>%
  dplyr::pull(KEGG)

p = cor_res_sub %>%
  dplyr::filter(KEGG %in% KEGG_sle) %>%
  ggplot(aes(x = `KEGG`, y = Cancer, text = paste0("R: ", R))) +
  geom_tile(aes(fill = R)) +
  geom_text(aes(label=P.label), nudge_y = -0.2, size = 1.7)+
  xlab(NULL) + ylab(NULL) +
  scale_fill_gradient2(low = "green", mid = "white", high = "red",
                        breaks = c(-0.4,0,0.4)) +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 45, hjust=1, size = 7))
# p
ggplotly(p)
```
</center>



## 5.3 IOBR
This part includes the spearman correlation of `r id_type` ``r id_name`` with  **`r length(unique(cor_res$L3[cor_res$L2=="IOBR"]))`** [IOBR](https://doi.org/10.3389/fimmu.2021.687975) originated items in primary tumor samples across all TCGA projects. See all the items here through IOBR R package `IOBR::signature_collection` or [here](https://github.com/lishensuo/utils/blob/main/IOBR_signature_collection.csv). Top 50 most significant terms are selected for visualization.

```{r pw_3, echo = FALSE}
cor_res_sub = cor_res %>%
  dplyr::filter(L1 == "Pathway activity") %>%
  dplyr::filter(L2 == "IOBR") %>%
  dplyr::rename(`IOBR` = L3)


cor_res_sub[,c(-1, -2, -6)] %>%
  dplyr::rename(P.value=P.format) %>%
  datatable(.,filter = 'top',
            options = list(pageLength = 5,
						columnDefs = list(list(className = 'dt-center', targets="_all"))))
```


<center>
```{r pw_3_plot, echo = FALSE, fig.width=10, fig.height=6.5}
# 选择50条显著通路
IOBR_sle = cor_res_sub %>%
  dplyr::group_by(IOBR) %>%
  dplyr:::summarise(n = sum(P.label==" ")) %>%
  dplyr::arrange(n) %>% head(50) %>%
  dplyr::pull(IOBR)

p = cor_res_sub %>%
  dplyr::filter(IOBR %in% IOBR_sle) %>%
  ggplot(aes(x = `IOBR`, y = Cancer, text = paste0("R: ", R))) +
  geom_tile(aes(fill = R)) +
  geom_text(aes(label=P.label), nudge_y = -0.2, size = 1.7)+
  xlab(NULL) + ylab(NULL) +
  scale_fill_gradient2(low = "green", mid = "white", high = "red",
                        breaks = c(-0.4,0,0.4)) +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 45, hjust=1, size = 7))
# p
ggplotly(p)
```
</center>

- <font size="4"> In this step, you can check which UCSC Xena hub and dataset each type of **Molecular profile** comes from for downstream analysis. You can click on the provided hub and dataset links at the bottom to learn more details about them；</font>
- <font size="4"> For some data types, there are **multiple datasets** available based on factors such as standardization methods, sequencing platforms, and others； </font>
- <font size="4"> Noteworthily, we provide more personlized options for **DNA methylation** type (Only TCGA); </font>

>1. Select one methylation platform;
>2. Select one gene;
>3. Select CpG sites through chromosomal coordinates.  By default, it is the range of all sites for that gene;
>4. Further manually select CpG sites based on above coordinates. By default, all sites will be selected
>5. Select the aggregation method for calculating gene-level methylation. By default, the mean of the selected CpG sites will be calculated.

<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_data_origin.png" alt="helper_data_origin"  width="450" />
</p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 

- In this step, you can choose one tumor data to divide the samples into two groups; - First, select an identifier and observe its distribution among samples; - Then, set the ranges for the two groups as the grouping criterion; > 1. For **character** class, you should assign one or multiple categories for either group; > 2. For **numeric** class, you can set two ranges by the absolute and percentile intervals.

--- - See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/).

- <font size="4">In this step, you can choose one tumor data to divide the samples into two groups;</font> 
- <font size="4">First, select an identifier and observe its distribution among samples;</font> 
- <font size="4">Then, set the ranges for the two groups as the grouping criterion;</font> 

> 1. For **character** class, you should assign one or multiple categories for either group;
> 2. For **numeric** class, you can set two ranges by the absolute and percentile intervals.


<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_set_groups.png" alt="helper_set_groups"  width="700"/>
</p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 

<font size="4">In **S3** Step, **individual correlation analysis** can now be performed after data preparation:</font>

- <font size="4">Firstly, you can adjust the default analysis and visualization parameters, respectively.</font>
  - <font size="4">Use **Spearman correlation** (based on the ranks of the data rather than the actual values) when input data is ordinal, not normally distributed, or you expect a monotonic but not necessarily linear relationship .</font>
  - <font size="4">Use **Pearson correlation** (based on covariance and standard deviation) when input data is continuous, normally distributed, and you expect a linear relationship.</font>

- <font size="4">Then, you will obtain the scatter plot labeled with correlation result after clicking the "**Run**" button.</font>
- <font size="4">Finally, you can download the 3 types of results.</font>

> 1. "**Figure**": visualization result;
> 2. "**Raw data(.csv)**": queried data for X-axis and Y-axis;
> 3. "**Analyzed data(.csv)**": detailed correlation result.


<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_analyze_cor_1.png" alt="helper_analyze_cor_1"   width="700"/>
</p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 

- <font size="4">In this step, you can filter suitable samples through two modules based on personalized analysis requirements;</font>
- <font size="4"> In "**quick filter**" module, you can quickly filter samples based on the sample's code type such as normal/tumor (TCGA,PCAWG) or tissue type such as lung, liver (CCLE); </font>


<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_choose_samples_1.png" alt="helper_choose_samples_1"   width="450"  />
</p>

- <font size="4"> In "**exact filter**" module, you can execute detailed filter using multiple conditions for different phenotypes. </font>



> 1. Firstly, you need to add one or multiple tumor data of interest as candidate filtering conditions;
>
> 2. Then, you can get an overview of the distribution of each candidate condition;
>
> 3. Finally, you can select one or more conditions from the candidate list for sample filtering. Different methods are provided for of different data types. 
>
>    - For **character** class, set "+" or "-" to retain or discard, respectively;
>
>    - For **numeric**  class, set ">" or "<" based on absolute values (min, max) or "%>" or "%<" based on percentile values (0,1).

<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_choose_samples_2.png" alt="helper_choose_samples_2"   width="700"  />
</p>


- <font size="4"> See more details in UCSCXenaShiny v2 book site (todo list). </font> 

#### [TCGA cancer types](https://gdc.cancer.gov/resources-tcga-users/tcga-code-tables/tcga-study-abbreviations)

<br>

| Study Abbreviation | Study Name                                                   |
| :----------------- | :----------------------------------------------------------- |
| LAML               | Acute Myeloid Leukemia                                       |
| ACC                | Adrenocortical carcinoma                                     |
| BLCA               | Bladder Urothelial Carcinoma                                 |
| LGG                | Brain Lower Grade Glioma                                     |
| BRCA               | Breast invasive carcinoma                                    |
| CESC               | Cervical squamous cell carcinoma and endocervical adenocarcinoma |
| CHOL               | Cholangiocarcinoma                                           |
| LCML               | Chronic Myelogenous Leukemia                                 |
| COAD               | Colon adenocarcinoma                                         |
| CNTL               | Controls                                                     |
| ESCA               | Esophageal carcinoma                                         |
| FPPP               | FFPE Pilot Phase II                                          |
| GBM                | Glioblastoma multiforme                                      |
| HNSC               | Head and Neck squamous cell carcinoma                        |
| KICH               | Kidney Chromophobe                                           |
| KIRC               | Kidney renal clear cell carcinoma                            |
| KIRP               | Kidney renal papillary cell carcinoma                        |
| LIHC               | Liver hepatocellular carcinoma                               |
| LUAD               | Lung adenocarcinoma                                          |
| LUSC               | Lung squamous cell carcinoma                                 |
| DLBC               | Lymphoid Neoplasm Diffuse Large B-cell Lymphoma              |
| MESO               | Mesothelioma                                                 |
| MISC               | Miscellaneous                                                |
| OV                 | Ovarian serous cystadenocarcinoma                            |
| PAAD               | Pancreatic adenocarcinoma                                    |
| PCPG               | Pheochromocytoma and Paraganglioma                           |
| PRAD               | Prostate adenocarcinoma                                      |
| READ               | Rectum adenocarcinoma                                        |
| SARC               | Sarcoma                                                      |
| SKCM               | Skin Cutaneous Melanoma                                      |
| STAD               | Stomach adenocarcinoma                                       |
| TGCT               | Testicular Germ Cell Tumors                                  |
| THYM               | Thymoma                                                      |
| THCA               | Thyroid carcinoma                                            |
| UCS                | Uterine Carcinosarcoma                                       |
| UCEC               | Uterine Corpus Endometrial Carcinoma                         |
| UVM                | Uveal Melanoma                                               |

<center> <h2> Extra Datasets </h2> </center>

Extra datasets cleaned from UCSCXena hubs or references are deposited in [Zenodo repo](https://zenodo.org/record/12819839).

You can download them directly from the Zenodo repo or using `get_data()` function provided in UCSCXenaShiny R package.

If you want to know the dataset source, you can check the "data_source" attribute.

e.g.,

```r
> str(pancan_MSI)
tibble [11,139 x 3] (S3: tbl_df/tbl/data.frame)
 $ case_id     : chr [1:11139] "10.1038_ng.2273_B085" "10.1038_ng.2273_B099" "10.1038_ng.2273_R104" "10.1038_ng.2273_T26" ...
 $ cancer_type : chr [1:11139] "CHOL_10.1038_ng.2273" "CHOL_10.1038_ng.2273" "CHOL_10.1038_ng.2273" "CHOL_10.1038_ng.2273" ...
 $ MANTIS_Score: num [1:11139] 0.285 0.257 0.275 0.292 0.281 ...
 - attr(*, "data_source")= chr "DOI:https://doi.org/10.1200/PO.17.00073"
```

<font size="4">In **S3** Step, **correlation analysis across multiple cancers** can now be performed after data preparation:</font>

- <font size="4">Firstly, you can adjust the default analysis and visualization parameters, respectively.</font>
  - <font size="4">Use **Spearman correlation** (based on the ranks of the data rather than the actual values) when input data is ordinal, not normally distributed, or you expect a monotonic but not necessarily linear relationship .</font>
  - <font size="4">Use **Pearson correlation** (based on covariance and standard deviation) when input data is continuous, normally distributed, and you expect a linear relationship.</font>
- <font size="4">Then, you will obtain the bar plot labeled with correlation coefficients after clicking the "**Run**" button.</font>
- <font size="4">Finally, you can download the 3 types of results.</font>

> 1. "**Figure**": visualization result;
> 2. "**Raw data(.csv)**": queried data for X-axis and Y-axis;
> 3. "**Analyzed data(.csv)**": detailed correlation result.

<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_analyze_cor_2.png" alt="helper_analyze_cor_2"   width="700" />
</p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 

### Abbreviations

#### TCGA cancer types

| Study Abbreviation | Study Name                                                   |
| :----------------- | :----------------------------------------------------------- |
| LAML               | Acute Myeloid Leukemia                                       |
| ACC                | Adrenocortical carcinoma                                     |
| BLCA               | Bladder Urothelial Carcinoma                                 |
| LGG                | Brain Lower Grade Glioma                                     |
| BRCA               | Breast invasive carcinoma                                    |
| CESC               | Cervical squamous cell carcinoma and endocervical adenocarcinoma |
| CHOL               | Cholangiocarcinoma                                           |
| LCML               | Chronic Myelogenous Leukemia                                 |
| COAD               | Colon adenocarcinoma                                         |
| CNTL               | Controls                                                     |
| ESCA               | Esophageal carcinoma                                         |
| FPPP               | FFPE Pilot Phase II                                          |
| GBM                | Glioblastoma multiforme                                      |
| HNSC               | Head and Neck squamous cell carcinoma                        |
| KICH               | Kidney Chromophobe                                           |
| KIRC               | Kidney renal clear cell carcinoma                            |
| KIRP               | Kidney renal papillary cell carcinoma                        |
| LIHC               | Liver hepatocellular carcinoma                               |
| LUAD               | Lung adenocarcinoma                                          |
| LUSC               | Lung squamous cell carcinoma                                 |
| DLBC               | Lymphoid Neoplasm Diffuse Large B-cell Lymphoma              |
| MESO               | Mesothelioma                                                 |
| MISC               | Miscellaneous                                                |
| OV                 | Ovarian serous cystadenocarcinoma                            |
| PAAD               | Pancreatic adenocarcinoma                                    |
| PCPG               | Pheochromocytoma and Paraganglioma                           |
| PRAD               | Prostate adenocarcinoma                                      |
| READ               | Rectum adenocarcinoma                                        |
| SARC               | Sarcoma                                                      |
| SKCM               | Skin Cutaneous Melanoma                                      |
| STAD               | Stomach adenocarcinoma                                       |
| TGCT               | Testicular Germ Cell Tumors                                  |
| THYM               | Thymoma                                                      |
| THCA               | Thyroid carcinoma                                            |
| UCS                | Uterine Carcinosarcoma                                       |
| UCEC               | Uterine Corpus Endometrial Carcinoma                         |
| UVM                | Uveal Melanoma                                               |

#### TCGA sample type codes

| Code | Definition                                        | Short Letter Code |
| :--- | :------------------------------------------------ | :---------------- |
| 01   | Primary Solid Tumor                               | TP                |
| 02   | Recurrent Solid Tumor                             | TR                |
| 03   | Primary Blood Derived Cancer - Peripheral Blood   | TB                |
| 04   | Recurrent Blood Derived Cancer - Bone Marrow      | TRBM              |
| 05   | Additional - New Primary                          | TAP               |
| 06   | Metastatic                                        | TM                |
| 07   | Additional Metastatic                             | TAM               |
| 08   | Human Tumor Original Cells                        | THOC              |
| 09   | Primary Blood Derived Cancer - Bone Marrow        | TBM               |
| 10   | Blood Derived Normal                              | NB                |
| 11   | Solid Tissue Normal                               | NT                |
| 12   | Buccal Cell Normal                                | NBC               |
| 13   | EBV Immortalized Normal                           | NEBV              |
| 14   | Bone Marrow Normal                                | NBM               |
| 15   | sample type 15                                    | 15SH              |
| 16   | sample type 16                                    | 16SH              |
| 20   | Control Analyte                                   | CELLC             |
| 40   | Recurrent Blood Derived Cancer - Peripheral Blood | TRB               |
| 50   | Cell Lines                                        | CELL              |
| 60   | Primary Xenograft Tissue                          | XP                |
| 61   | Cell Line Derived Xenograft Tissue                | XCL               |
| 99   | sample type 99                                    | 99SH              |

#### TCGA platform codes

| Platform Code                         | Platform Alias                     | Platform Name                                                |
| :------------------------------------ | :--------------------------------- | :----------------------------------------------------------- |
| HT_HG-U133A                           | HT_HG-U133A                        | Affymetrix HT Human Genome U133 Array Plate Set              |
| HuEx-1_0-st-v2                        | HuEx-1_0-st-v2                     | Affymetrix Human Exon 1.0 ST Array                           |
| Genome_Wide_SNP_6                     | Genome_Wide_SNP_6                  | Affymetrix Genome-Wide Human SNP Array 6.0                   |
| HG-CGH-415K_G4124A                    | HG-CGH-415K_G4124A                 | Agilent Human Genome CGH Custom Microarray 2x415K            |
| WHG-CGH_4x44B                         | WHG-CGH_4x44B                      | Agilent Human Genome CGH Microarray 44K                      |
| HG-CGH-244A                           | HG-CGH-244A                        | Agilent Human Genome CGH Microarray 244A                     |
| WHG-1x44K_G4112A                      | 1 x 44K                            | Agilent Whole Human Genome                                   |
| WHG-4x44K_G4112F                      | 4 x 44K                            | Agilent Whole Human Genome Microarray Kit                    |
| AgilentG4502A_07_1                    | AgilentG4502A_07                   | Agilent 244K Custom Gene Expression G4502A-07-1              |
| H-miRNA_G4470A                        | H-miRNA_G4470A                     | Agilent Human miRNA Microarray                               |
| AgilentG4502A_07_2                    | AgilentG4502A_07                   | Agilent 244K Custom Gene Expression G4502A-07-2              |
| H-miRNA_8x15Kv2                       | H-miRNA_8x15K                      | Agilent Human miRNA Microarray Rel12.0                       |
| AgilentG4502A_07_3                    | AgilentG4502A_07                   | Agilent 244K Custom Gene Expression G4502A-07-3              |
| H-miRNA_8x15K                         | H-miRNA_8x15K                      | Agilent 8 x 15K Human miRNA-specific microarray              |
| CGH-1x1M_G4447A                       | CGH-1x1M_G4447A                    | Agilent SurePrint G3 Human CGH Microarray Kit 1x1M           |
| H-miRNA_EarlyAccess                   | H-miRNA_EarlyAccess                | Agilent Human miRNA Early Access Array                       |
| IlluminaGG                            | IlluminaGG                         | Illumina GoldenGate                                          |
| HumanMethylation27                    | HumanMethylation27                 | Illumina Infinium Human DNA Methylation 27                   |
| IlluminaDNAMethylation_OMA003_CPI     | IlluminaDNAMethylation             | Illumina DNA Methylation OMA003 Cancer Panel I               |
| Human1MDuo                            | Human1MDuo                         | Illumina Human1M-Duo BeadChip                                |
| IlluminaDNAMethylation_OMA002_CPI     | IlluminaDNAMethylation             | Illumina DNA Methylation OMA002 Cancer Panel I               |
| HumanHap550                           | HumanHap550                        | Illumina 550K Infinium HumanHap550 SNP Chip                  |
| AgilentG4502A_07                      | AgilentG4502A_07                   | Agilent 244K Custom Gene Expression G4502A-07                |
| bio                                   | bio                                | Biospecimen Metadata - Complete Set                          |
| biotab                                | biotab                             | Biospecimen Metadata - Complete Set - All Samples - Tab-delimited |
| minbio                                | minbio                             | Biospecimen Metadata - Minimal Set                           |
| minbiotab                             | minbiotab                          | Biospecimen Metadata - Minimal Set - All Samples - Tab-delimited |
| ABI                                   | ABI                                | Applied Biosystems Sequence data                             |
| IlluminaHiSeq_DNASeq                  | Mutation Calling                   | Illumina HiSeq 2000 DNA Sequencing                           |
| SOLiD_DNASeq                          | Mutation Calling                   | ABI SOLiD DNA Sequencing                                     |
| IlluminaGA_DNASeq                     | Mutation Calling                   | Illumina Genome Analyzer DNA Sequencing                      |
| IlluminaGA_mRNA_DGE                   | IlluminaGA_mRNA_DGE                | Illumina Genome Analyzer mRNA Digital Gene Expression        |
| 454                                   | 454                                | 454 Life Sciences Genome Sequence data                       |
| HG-U133A_2                            | HG-U133A_2                         | Affymetrix Human Genome U133A 2.0 Array                      |
| HG-U133_Plus_2                        | HG-U133_Plus_2                     | Affymetrix Human Genome U133 Plus 2.0 Array                  |
| Mapping250K_Nsp                       | Mapping250K_Nsp                    | Affymetrix Human Mapping 250K Nsp Array                      |
| Mapping250K_Sty                       | Mapping250K_Sty                    | Affymetrix Human Mapping 250K Sty Array                      |
| GenomeWideSNP_5                       | GenomeWideSNP_5                    | Affymetrix Genome-Wide Human SNP Array 5.0                   |
| tissue_images                         | tissue_images                      | Tissue Images                                                |
| IlluminaGA_RNASeq                     | IlluminaGA_RNASeq                  | Illumina Genome Analyzer RNA Sequencing                      |
| IlluminaGA_miRNASeq                   | IlluminaGA_miRNASeq                | Illumina Genome Analyzer miRNA Sequencing                    |
| diagnostic_images                     | diagnostic_images                  | Diagnostic Images                                            |
| pathology_reports                     | pathology_reports                  | Pathology Reports                                            |
| MDA_RPPA_Core                         | MDA_RPPA_Core                      | M.D. Anderson Reverse Phase Protein Array Core               |
| microsat_i                            | microsat_i                         | Microsatellite Instability Analysis                          |
| HumanMethylation450                   | HumanMethylation450                | Illumina Infinium Human DNA Methylation 450                  |
| IlluminaHiSeq_mRNA_DGE                | IlluminaHiSeq_mRNA_DGE             | Illumina HiSeq 2000 mRNA Digital Gene Expression             |
| IlluminaHiSeq_miRNASeq                | IlluminaHiSeq_miRNASeq             | Illumina HiSeq 2000 miRNA Sequencing                         |
| IlluminaHiSeq_RNASeq                  | IlluminaHiSeq_RNASeq               | Illumina HiSeq 2000 RNA Sequencing                           |
| IlluminaHiSeq_DNASeqC                 | IlluminaHiSeq_DNASeqC              | Illumina HiSeq for Copy Number Variation                     |
| fh_analyses                           | fh_analyses                        | Firehose Analyses                                            |
| fh_stddata                            | fh_stddata                         | Firehose Standardized Data                                   |
| fh_reports                            | fh_reports                         | Firehose Reports                                             |
| IlluminaGA_RNASeqV2                   | IlluminaGA_RNASeqV2                | Illumina Genome Analyzer RNA Sequencing Version 2 analysis   |
| IlluminaHiSeq_RNASeqV2                | IlluminaHiSeq_RNASeqV2             | Illumina HiSeq 2000 RNA Sequencing Version 2 analysis        |
| IlluminaHiSeq_DNASeq_Cont             | Mutation Calling                   | Illumina HiSeq 2000 DNA Sequencing - Controlled              |
| IlluminaGA_DNASeq_Cont                | Mutation Calling                   | Illumina Genome Analyzer DNA Sequencing - Controlled         |
| IlluminaHiSeq_TotalRNASeqV2           | IlluminaHiSeq_TotalRNASeqV2        | Illumina HiSeq 2000 Total RNA Sequencing Version 2 analysis  |
| SOLiD_DNASeq_Cont                     | Mutation Calling                   | ABI SOLiD DNA Sequencing - Controlled                        |
| Mixed_DNASeq                          | Mutation Calling                   | Mixed DNA Sequencing                                         |
| Mixed_DNASeq_Cont                     | Mutation Calling                   | Mixed DNA Sequencing - Controlled                            |
| IlluminaHiSeq_WGBS                    | IlluminaHiSeq_WGBS                 | Illumina HiSeq 2000 Bisulfite-converted DNA Sequencing       |
| IlluminaHiSeq_DNASeq_automated        | Automated Mutation Calling         | IlluminaHiSeq automated DNA sequencing                       |
| IlluminaHiSeq_DNASeq_curated          | Curated Mutation Calling           | IlluminaHiSeq curated DNA sequencing                         |
| IlluminaGA_DNASeq_automated           | Automated Mutation Calling         | IlluminaGA automated DNA sequencing                          |
| IlluminaGA_DNASeq_curated             | Curated Mutation Calling           | IlluminaGA curated DNA sequencing                            |
| SOLiD_DNASeq_automated                | Automated Mutation Calling         | SOLiD automated DNA sequencing                               |
| SOLiD_DNASeq_curated                  | Curated Mutation Calling           | SOLiD curated DNA sequencing                                 |
| Mixed_DNASeq_automated                | Automated Mutation Calling         | Mixed automated DNA sequencing                               |
| Mixed_DNASeq_curated                  | Curated Mutation Calling           | Mixed curated DNA sequencing                                 |
| IlluminaHiSeq_DNASeq_Cont_automated   | Automated Mutation Calling         | IlluminaHiSeq automated DNA sequencing - controlled          |
| IlluminaHiSeq_DNASeq_Cont_curated     | Curated Mutation Calling           | IlluminaHiSeq curated DNA sequencing - controlled            |
| IlluminaGA_DNASeq_Cont_automated      | Automated Mutation Calling         | IlluminaGA automated DNA sequencing - controlled             |
| IlluminaGA_DNASeq_Cont_curated        | Curated Mutation Calling           | IlluminaGA curated DNA sequencing - controlled               |
| SOLiD_DNASeq_Cont_automated           | Automated Mutation Calling         | SOLiD automated DNA sequencing - controlled                  |
| SOLiD_DNASeq_Cont_curated             | Curated Mutation Calling           | SOLiD curated DNA sequencing - controlled                    |
| Mixed_DNASeq_Cont_automated           | Automated Mutation Calling         | Mixed automated DNA sequencing - controlled                  |
| Mixed_DNASeq_Cont_curated             | Curated Mutation Calling           | Mixed curated DNA sequencing - controlled                    |
| supplemental_clinical                 | supplemental_clinical              | supplemental_clinical                                        |
| Multicenter_mutation_calling_MC3      | Multicenter Mutation Calling (MC3) | Multi-Center Mutations                                       |
| Multicenter_mutation_calling_MC3_Cont | Multicenter Mutation Calling (MC3) | Multi-Center Mutations - Controlled                          |

<br>

All TCGA code tables are available at <https://gdc.cancer.gov/resources-tcga-users/tcga-code-tables/>

<center> <h2> UCSCXenaShiny v2 </h2> </center>

- <font size="5">Shensuo Li, Yuzhong Peng, Minjun Chen, Yankun Zhao, Yi Xiong, Jianfeng Li, Peng Luo, Haitao Wang, Fei Zhao, Qi Zhao, Yanru Cui, Sujun Chen, Jian-Guo Zhou, Shixiang Wang,  Facilitating integrative and personalized oncology omics analysis with UCSCXenaShiny, Communications Biology, 1200 (2024),  https://doi.org/10.1038/s42003-024-06891-2</font>

- <font size="5">PMID: [39341906](https://pubmed.ncbi.nlm.nih.gov/39341906/)</font>
- <font size="5">Bibtex format: </font>

```
@ARTICLE{Li2024-yd,
  title    = "Facilitating integrative and personalized oncology omics analysis with UCSCXenaShiny",
  author   = "Li, Shensuo and Peng, Yuzhong and Chen, Minjun and Zhao, Yankun and Xiong, Yi and Li, Jianfeng and Luo, Peng and Wang, Haitao and Zhao, Fei and Zhao, Qi and Cui, Yanru and Chen, Sujun and Zhou, Jian-Guo and Wang, Shixiang",
  abstract = "The continuous generation of multi-omics and phenotype data is propelling advancements in precision oncology. UCSCXenaShiny was developed as an interactive tool for exploring thousands of cancer datasets available on UCSC Xena. However, its capacity for comprehensive and personalized pan-cancer data analysis is being challenged by the growing demands. Here, we introduce UCSCXenaShiny v2, a milestone update through a variety of improvements. Firstly, by integrating multidimensional data and implementing adaptable sample settings, we create a suite of robust TPC (TCGA, PCAWG, CCLE) analysis pipelines. These pipelines empower users to conduct in-depth analyses of correlation, comparison, and survival in three modes: Individual, Pan-cancer and Batch screen. Additionally, the tool includes download interfaces that enable users to access diverse data and outcomes, several features also facilitate the joint analysis of drug sensitivity and multi-omics of cancer cell lines. UCSCXenaShiny v2 is an open-source R package and a web application, freely accessible at https://github.com/openbiox/UCSCXenaShiny.",
  journal  = "Commun. Biol.",
  volume   =  7,
  number   =  1,
  pages    = "1200",
  month    =  sep,
  year     =  2024,
  language = "en"
}
```









<br>

<center> <h2> UCSCXenaShiny v1 </h2> </center>

- <font size="5">Shixiang Wang<sup>\#</sup>, Yi Xiong<sup>\#</sup>, Longfei Zhao<sup>\#</sup>, Kai Gu<sup>\#</sup>, Yin Li, Fei Zhao, Jianfeng Li, Mingjie Wang, Haitao Wang, Ziyu Tao, Tao Wu, Yichao Zheng, Xuejun Li, Xue-Song Liu, UCSCXenaShiny: An R/CRAN Package for Interactive Analysis of UCSC Xena Data, Bioinformatics, 2021;, btab561, https://doi.org/10.1093/bioinformatics/btab561.</font>



- <font size="5">PubMed ID: [34323947](https://pubmed.ncbi.nlm.nih.gov/34323947/)</font>
- <font size="5">Bibtex format:</font>

```
@article{10.1093/bioinformatics/btab561,
    author = {Wang, Shixiang and Xiong, Yi and Zhao, Longfei and Gu, Kai and Li, Yin and Zhao, Fei and Li, Jianfeng and Wang, Mingjie and Wang, Haitao and Tao, Ziyu and Wu, Tao and Zheng, Yichao and Li, Xuejun and Liu, Xue-Song},
    title = "{UCSCXenaShiny: An R/CRAN Package for Interactive Analysis of UCSC Xena Data}",
    journal = {Bioinformatics},
    year = {2021},
    month = {07},
    abstract = "{UCSC Xena platform provides huge amounts of processed cancer omics data from large cancer research projects (e.g. TCGA, CCLE and PCAWG) or individual research groups and enables unprecedented research opportunities. However, a graphical user interface (GUI) based tool for interactively analyzing UCSC Xena data and generating elegant plots is still lacking, especially for cancer researchers and clinicians with limited programming experience. Here, we present UCSCXenaShiny, an R Shiny package for quickly searching, downloading, exploring, analyzing and visualizing data from UCSC Xena data hubs. This tool could effectively promote the practical use of public data, and can serve as an important complement to the current Xena genomics explorer.UCSCXenaShiny is an open source R package under GPLv3 license and it is freely available at https://github.com/openbiox/UCSCXenaShiny or https://cran.r-project.org/package=UCSCXenaShiny. The docker image is available at https://hub.docker.com/r/shixiangwang/ucscxenashiny.Supplementary data are available at Bioinformatics online.}",
    issn = {1367-4803},
    doi = {10.1093/bioinformatics/btab561},
    url = {https://doi.org/10.1093/bioinformatics/btab561},
    note = {btab561},
    eprint = {https://academic.oup.com/bioinformatics/advance-article-pdf/doi/10.1093/bioinformatics/btab561/39456169/btab561.pdf},
}
```
<br>

<center> <h2> UCSC Xena </h2> </center>


<font size="5">When you use UCSC Xena data, you should also cite it with:</font>

- <font size="5">Goldman, M. J., Craft, B., Hastie, M., Repečka, K., McDade, F., Kamath, A., ... & Haussler, D. (2020). Visualizing and interpreting cancer genomics data via the Xena platform. Nature biotechnology, 38(6), 675-678. https://doi:10.1038/s41587-020-0546-8</font>

- <font size="5">PubMed ID: [32444850](https://pubmed.ncbi.nlm.nih.gov/32444850/)</font>
- <font size="5">Bibtex format:</font>

```
@article{goldman2020visualizing,
  title={Visualizing and interpreting cancer genomics data via the Xena platform},
  author={Goldman, Mary J and Craft, Brian and Hastie, Mim and Repe{\v{c}}ka, Kristupas and McDade, Fran and Kamath, Akhil and Banerjee, Ayan and Luo, Yunhai and Rogers, Dave and Brooks, Angela N and others},
  journal={Nature biotechnology},
  volume={38},
  number={6},
  pages={675--678},
  year={2020},
  publisher={Nature Publishing Group}
}
```

- <font size="4">In this step, you can query specific tumor data according to the "**Data type**"--"**Data subtype**"--"**Identifier**" search sequence;</font> 
- <font size="4">As "**Molecular profile**" data type needs to be obtained from the UCSC Xena site, the caching time depends on the network conditions;</font> 


<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_get_one_data.png" alt="helper_get_one_data" width="700"/>
</p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 

<font size="4">In **S3** Step, **batch comparison analysis** can now be performed after data preparation:</font>

- <font size="4">Firstly, you can adjust the default analysis parameters;</font>
  - <font size="4">Use **t.test ** method,  if input data is normally distributed (especially for small sample sizes) and meet homogeneity of variances (variances should be similar in both groups).</font>
  - <font size="4">Use **Wilcoxon test **method, if input data that does not meet the assumptions of the t-test by comparing the ranks of values between two groups.</font>
- <font size="4">Then, you will obtain the table arranged by p.value after clicking the "**Run**" button;</font>
- <font size="4">Finally, you can download the 2 types of results.</font>

> 1. "**Raw data(.csv)**": queried data for comparison between groups;
> 2. "**Analyzed data(.csv)**": detailed correlation result.


<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures//helper_analyze_comp_3.png" alt="helper_analyze_comp_3"    width="700"/>
</p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 

<center>
<h1>TPC pipelines</h1>
</center>


<font size="5">Here, we introduced a series of personalized TPC pipelines for comprehensive and precise TPC molecule exploration with various operations.  In general, 9 panels for common analytical scenario are available, as follows:</font>


<center>
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_intro_personal_pips_1.png" alt="helper_intro_personal_pips_1"  style="zoom:50%";/>
</center>


<font size="5">Generally, each analytical pipeline comprises of three main subpanels (S1, S2, S3).</font>

- <font size="5">In S1 subpanel, you can prepare and select the expected data and sample ranges;</font>
- <font size="5">In S2 subpanel, you can fetch the specific tumor data for downstream analysis;</font>
- <font size="5">In S3 subpanel, you can execute the analysis, visualize and download results. </font>



<center>
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_intro_personal_pips_2.png" alt="helper_intro_personal_pips_2" style="zoom:80%;" />
</center>


<br></br>

<br></br>



> <font size="5">Tips: The menu of `TPC modules` enable specific and quick  TPC molecular analysis, such as tumor and tomor comparison, molecule and molecule correlation.</font>

<font size="5">In this step, you should perform two types of operations</font>





<font size="4">**In first step**, you can query multiple identifiers of tumor data under one specific "**Data type**"--"**Data subtype**;</font> 

- <font size="4">Generally, the following 3 way can be performed to select and cache batch IDs;</font></font>

> 1. In the "**Selection**" way, choose multiple IDs one by one;
> 2. In the "**All**" way, directly choose all IDs under the data subtype. For molecular profile, you need further choose one pathway to limit the ID size;
> 3. In the "**File**" way, choose IDs by uploading an one-column file. You can download the example data to check the format requirements.

<font size="4">**In second step**, you should use above tumor data to divide the samples into two groups;</font> 

- <font size="4">Set the ranges for the two groups as the grouping criterion;</font> 
> 1. For **character** class, you should assign one or multiple categories for either group;
> 2. For **numeric** class, you can set two ranges by the absolute and percentile intervals.


<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_get_batch_data_set_groups.png" alt="helper_get_batch_data_set_groups" width="700"/>
<p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 

- <font size="4">In **S3** Step, **batch survival analysis** can now be performed after data preparation;</font>
- <font size="4">Firstly, you can adjust the default analysis and visualization parameters, respectively;</font>
- <font size="4">Noteworthily, you can try whether to use **initial data** before grouping if the data type is **numerical**;</font>

> 1. For log-rank test, it will decide the optimal cutoff;
> 2. For univariate cox regression, it will directly analyze based on the numerical data. 

- <font size="4">Then, you will obtain the table arranged by p.value  after clicking the "**Run**" button;</font>
- <font size="4">Finally, you can download the 2 types of results.</font>

> 1. "**Raw data(.csv)**": queried data for X-axis and Y-axis;
> 2. "**Analyzed data(.csv)**": detailed correlation result.




<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_analyze_sur_3.png" alt="helper_analyze_sur_3" width="700"/>
</p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 

<center> <h2> Citation </h2> </center>

Please cite the following article when you used **UCSCXenaShiny** in your study:

Shixiang Wang<sup>\#</sup>, Yi Xiong<sup>\#</sup>, Longfei Zhao<sup>\#</sup>, Kai Gu<sup>\#</sup>, Yin Li, Fei Zhao, Jianfeng Li, Mingjie Wang, Haitao Wang, Ziyu Tao, Tao Wu, Yichao Zheng, Xuejun Li, Xue-Song Liu, UCSCXenaShiny: An R/CRAN Package for Interactive Analysis of UCSC Xena Data, Bioinformatics, 2021;, btab561, https://doi.org/10.1093/bioinformatics/btab561.

Bibtex format:

```
@article{10.1093/bioinformatics/btab561,
    author = {Wang, Shixiang and Xiong, Yi and Zhao, Longfei and Gu, Kai and Li, Yin and Zhao, Fei and Li, Jianfeng and Wang, Mingjie and Wang, Haitao and Tao, Ziyu and Wu, Tao and Zheng, Yichao and Li, Xuejun and Liu, Xue-Song},
    title = "{UCSCXenaShiny: An R/CRAN Package for Interactive Analysis of UCSC Xena Data}",
    journal = {Bioinformatics},
    year = {2021},
    month = {07},
    abstract = "{UCSC Xena platform provides huge amounts of processed cancer omics data from large cancer research projects (e.g. TCGA, CCLE and PCAWG) or individual research groups and enables unprecedented research opportunities. However, a graphical user interface (GUI) based tool for interactively analyzing UCSC Xena data and generating elegant plots is still lacking, especially for cancer researchers and clinicians with limited programming experience. Here, we present UCSCXenaShiny, an R Shiny package for quickly searching, downloading, exploring, analyzing and visualizing data from UCSC Xena data hubs. This tool could effectively promote the practical use of public data, and can serve as an important complement to the current Xena genomics explorer.UCSCXenaShiny is an open source R package under GPLv3 license and it is freely available at https://github.com/openbiox/UCSCXenaShiny or https://cran.r-project.org/package=UCSCXenaShiny. The docker image is available at https://hub.docker.com/r/shixiangwang/ucscxenashiny.Supplementary data are available at Bioinformatics online.}",
    issn = {1367-4803},
    doi = {10.1093/bioinformatics/btab561},
    url = {https://doi.org/10.1093/bioinformatics/btab561},
    note = {btab561},
    eprint = {https://academic.oup.com/bioinformatics/advance-article-pdf/doi/10.1093/bioinformatics/btab561/39456169/btab561.pdf},
}
```

When you use UCSC Xena data, you should also cite it with:

Goldman, M. J., Craft, B., Hastie, M., Repečka, K., McDade, F., Kamath, A., ... & Haussler, D. (2020). Visualizing and interpreting cancer genomics data via the Xena platform. Nature biotechnology, 38(6), 675-678.

Bibtex format:

```
@article{goldman2020visualizing,
  title={Visualizing and interpreting cancer genomics data via the Xena platform},
  author={Goldman, Mary J and Craft, Brian and Hastie, Mim and Repe{\v{c}}ka, Kristupas and McDade, Fran and Kamath, Akhil and Banerjee, Ayan and Luo, Yunhai and Rogers, Dave and Brooks, Angela N and others},
  journal={Nature biotechnology},
  volume={38},
  number={6},
  pages={675--678},
  year={2020},
  publisher={Nature Publishing Group}
}
```

<center> <h2> Contact </h2> </center>

### Bug report or feature request

If you find any bug or have any feature to request, please report it at GitHub <https://github.com/openbiox/UCSCXenaShiny/issues> or email to me <mailto:w_shixiang@163.com>.

If you love our work, please give use a star at <https://github.com/openbiox/UCSCXenaShiny>.

### Discussion

To discuss ideas or suggestions, you can contact any of us via information shown in "Developers" page.

- <font size="4">In this step, you can set custom signature based on molecular profile of TCGA/PCAWG/CCLE sample data for joint analysis;</font>
- <font size="4">You can (1) set the basic signature information and (2) molecular formula, (3) calculate the signature value and add into **Custom metadata** (Data subtype) of Phenotype data (Data type) for analysis;</font>

<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_add_signature.png" alt="helper_add_signature"    width="700"  />
</p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 

- In this step, you can filter suitable samples through two modules based on personalized analysis requirements; - In "**quick filter**" module, you can quickly filter samples based on the sample's code type such as normal/tumor (TCGA,PCAWG) or tissue type such as lung, liver (CCLE);

- In "**exact filter**" module, you can execute detailed filter using multiple conditions for different phenotypes. > 1. Firstly, you need to add one or multiple tumor data of interest as candidate filtering conditions; > > 2. Then, you can get an overview of the distribution of each candidate condition; > > 3. Finally, you can select one or more conditions from the candidate list for sample filtering. Different methods are provided for of different data types. > > - For **character** class, set "+" or "-" to retain or discard, respectively; > > - For **numeric** class, set ">" or "<" based on absolute values (min, max) or "%>" or "%<" based on percentile values (0,1).

- See more details in UCSCXenaShiny v2 book site (todo list).

# TPC pipelines

Here, we introduced a series of personalized TPC pipelines for comprehensive and precise TPC molecule exploration with various operations. In general, 9 panels for common analytical scenario are available, as follows:
Generally, each analytical pipeline comprises of three main subpanels (S1, S2, S3). - In S1 subpanel, you can prepare and select the expected data and sample ranges; - In S2 subpanel, you can fetch the specific tumor data for downstream analysis; - In S3 subpanel, you can execute the analysis, visualize and download results.
> Tips: The menu of `TPC modules` enable specific and quick TPC molecular analysis, such as tumor and tomor comparison, molecule and molecule correlation.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_pancan_value.R
\name{available_hosts}
\alias{available_hosts}
\title{Show Available Hosts}
\usage{
available_hosts()
}
\value{
hosts
}
\description{
Show Available Hosts
}
\examples{
available_hosts()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tcga_subtypes}
\alias{tcga_subtypes}
\title{TCGA Subtype Data}
\format{
A \link{data.frame}
}
\source{
UCSC Xena.
}
\description{
TCGA Subtype Data
}
\examples{
data("tcga_subtypes")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{keep_cat_cols}
\alias{keep_cat_cols}
\title{Keep Only Columns Used for Sample Selection}
\usage{
keep_cat_cols(x, keep_sam_cols = TRUE, return_idx = TRUE)
}
\arguments{
\item{x}{a \code{data.frame} with many columns.}

\item{keep_sam_cols}{if \code{TRUE} (default), keep columns with
pattern 'sample', 'patient', etc.}

\item{return_idx}{if \code{TRUE} (default), return index of 5 (at most) columns, it is useful in Shiny.}
}
\value{
a \code{data.frame} or a \code{list}.
}
\description{
Keep Only Columns Used for Sample Selection
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_value.R
\name{query_molecule_value}
\alias{query_molecule_value}
\title{Get Molecule or Signature Data Values from Dense (Genomic) Matrix Dataset of UCSC Xena Data Hubs}
\usage{
query_molecule_value(dataset, molecule, host = NULL)
}
\arguments{
\item{dataset}{a UCSC Xena dataset in dense matrix format (rows are features
(e.g., gene, cell line) and columns are samples).}

\item{molecule}{a molecular identifier (e.g., "TP53") or a formula specifying
genomic signature (\code{"TP53 + 2 * KRAS - 1.3 * PTEN"}).
\strong{NOTE}, when a signature is specified, a space must exist in the input.}

\item{host}{a UCSC Xena host, default is \code{NULL}, auto-detect from the dataset.}
}
\value{
a named vector.
}
\description{
Get Molecule or Signature Data Values from Dense (Genomic) Matrix Dataset of UCSC Xena Data Hubs
}
\examples{
# What does dense matrix mean?
table(UCSCXenaTools::XenaData$Type)
# It is a the UCSC Xena dataset with "Type" equals to "genomicMatrix"
\dontrun{
dataset <- "ccle/CCLE_copynumber_byGene_2013-12-03"
x <- query_molecule_value(dataset, "TP53")
head(x)

signature <- "TP53 + 2*KRAS - 1.3*PTEN" # a space must exist in the string
y <- query_molecule_value(dataset, signature)
head(y)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_analyze_gene_drug_response.R
\name{vis_gene_drug_response_diff}
\alias{vis_gene_drug_response_diff}
\title{Visualize Gene and Drug Response Difference with CCLE Data}
\usage{
vis_gene_drug_response_diff(
  Gene = "TP53",
  tissue = "lung",
  Show.P.label = TRUE,
  Method = "wilcox.test",
  values = c("#DF2020", "#DDDF21"),
  alpha = 0.5
)
}
\arguments{
\item{Gene}{a molecular identifier (e.g., "TP53") or a formula specifying
genomic signature (\code{"TP53 + 2 * KRAS - 1.3 * PTEN"}).}

\item{tissue}{select cell line origin tissue.}

\item{Show.P.label}{\code{TRUE} or \code{FALSE} present p value with number or label \code{*}, \verb{**}, \verb{***} and \verb{****}}

\item{Method}{default method is wilcox.test}

\item{values}{the color to fill tumor or normal}

\item{alpha}{set alpha for dots.}
}
\value{
a \code{ggplot} object.
}
\description{
See \link{analyze_gene_drug_response_diff} for examples.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_toil_Mut.R
\name{vis_toil_Mut_cancer}
\alias{vis_toil_Mut_cancer}
\title{Visualize molecular profile difference between mutation and wild status of queried gene in Single Cancer Type}
\usage{
vis_toil_Mut_cancer(
  mut_Gene = "TP53",
  Gene = NULL,
  data_type = NULL,
  Mode = c("Dotplot", "Violinplot"),
  Show.P.value = TRUE,
  Show.P.label = TRUE,
  Method = c("wilcox.test", "t.test"),
  values = c("#DF2020", "#DDDF21"),
  draw_quantiles = c(0.25, 0.5, 0.75),
  trim = TRUE,
  Cancer = "ACC",
  opt_pancan = .opt_pancan
)
}
\arguments{
\item{mut_Gene}{the queried gene to determine grouping based on mutation and wild status}

\item{Gene}{a molecular identifier (e.g., "TP53") or a formula specifying
genomic signature (\code{"TP53 + 2 * KRAS - 1.3 * PTEN"}).}

\item{data_type}{choose gene profile type,
including "mRNA", "transcript", "methylation", "miRNA".}

\item{Mode}{choose one visualize mode to represent data}

\item{Show.P.value}{\code{TRUE} or \code{FALSE} whether to count P value}

\item{Show.P.label}{\code{TRUE} or \code{FALSE} present p value with number or label \code{*}, \verb{**}, \verb{***} and \verb{****}}

\item{Method}{default method is wilcox.test}

\item{values}{the color to fill mutation or wild status}

\item{draw_quantiles}{draw quantiles for violinplot}

\item{trim}{whether to trim the violin}

\item{Cancer}{select cancer cohort(s).}

\item{opt_pancan}{specify one dataset for some molercular profiles}
}
\value{
a \code{ggplot} object or a tibble data.frame.
}
\description{
Visualize molecular profile difference between mutation and wild status of queried gene in Single Cancer Type
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_identifier_dim_dist.R
\name{vis_identifier_dim_dist}
\alias{vis_identifier_dim_dist}
\title{Visualize the distribution difference of samples after Molecule Identifier dimensionality reduction analysis}
\usage{
vis_identifier_dim_dist(
  dataset = NULL,
  ids = NULL,
  grp_df,
  samples = NULL,
  return.data = FALSE,
  DR_method = c("PCA", "UMAP", "tSNE"),
  add_margin = NULL,
  palette = "Set1"
)
}
\arguments{
\item{dataset}{the dataset to obtain identifiers.}

\item{ids}{the molecule identifiers.}

\item{grp_df}{When \code{dataset} and \code{id} are all not \code{NULL}, it should be a \code{data.frame} with 2 columns.
\itemize{
\item The first column refers to sample ID.
\item The second column refers to groups indicated in axis X.
}}

\item{samples}{default is \code{NULL}, can be common sample names for two datasets.}

\item{return.data}{whether to reture the raw meta/matrix data (list) instead of plot}

\item{DR_method}{the dimensionality reduction method}

\item{add_margin}{the marginal plot (NULL, "density", "boxplot")}

\item{palette}{the color setting of RColorBrewer}
}
\value{
a \code{ggplot} object.
}
\description{
NOTE: the dataset must be dense matrix in UCSC Xena data hubs.
}
\examples{
# vis_identifier_dim_dist(expr_dataset, ids, grp_df, DR_method="PCA")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tcga_purity}
\alias{tcga_purity}
\title{TCGA: Purity Data}
\format{
A \link{data.frame}
}
\source{
\url{https://www.nature.com/articles/ncomms9971#Sec14}
}
\description{
TCGA: Purity Data
}
\examples{
data("tcga_purity")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tcga_tmb}
\alias{tcga_tmb}
\title{TCGA: TMB (Tumor Mutation Burden) Data}
\format{
A \link{data.frame}
}
\source{
\url{https://gdc.cancer.gov/about-data/publications/panimmune}
}
\description{
TCGA: TMB (Tumor Mutation Burden) Data
}
\examples{
data("tcga_tmb")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_cross.R
\name{calc_pw_mut_cnv}
\alias{calc_pw_mut_cnv}
\title{Prepare Mut/CNV data of pathways given specific samples}
\usage{
calc_pw_mut_cnv(tumor_samples = NULL)
}
\arguments{
\item{tumor_samples}{Select specific tumor samples. Default NULL, indicating all tumor samples.}
}
\description{
Prepare Mut/CNV data of pathways given specific samples
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{TCGA.organ}
\alias{TCGA.organ}
\title{TCGA: Organ Data}
\format{
A \link{data.frame}
}
\description{
TCGA: Organ Data
}
\examples{
data("TCGA.organ")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_cross.R
\name{vis_gene_cross_omics}
\alias{vis_gene_cross_omics}
\title{Visualize cross-omics of one gene among pan-cancers}
\usage{
vis_gene_cross_omics(
  gene = "TP53",
  tumor_projects = NULL,
  tumor_samples = NULL,
  n_trans = 5,
  n_methy = 5,
  seed = 42,
  add_mean_trans = TRUE,
  add_mean_methy = TRUE,
  pval_mrna = c(0.05, 0.01, 0.001),
  return_list = FALSE
)
}
\arguments{
\item{gene}{a gene symbol identifier (e.g., "TP53")}

\item{tumor_projects}{Select specific TCGA projects. Default NULL, indicating all TCGA projects.}

\item{tumor_samples}{Select specific tumor samples. Default NULL, indicating all tumor samples.}

\item{n_trans}{The number of sampling transcripts or specific transcript identifiers.}

\item{n_methy}{The number of sampling CpG sites or specific CpG identifiers.}

\item{seed}{The seed of sampling.}

\item{add_mean_trans}{Add overall column to display the mean values of all gene's transcripts.}

\item{add_mean_methy}{Add overall column to display the mean values of all gene's cpg sites.}

\item{pval_mrna}{The P value thresholds}

\item{return_list}{TRUE returns a list including plot object and data. FALSE just returns plot.}
}
\value{
funkyheatmap
}
\description{
Visualize cross-omics of one gene among pan-cancers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ope_pancan_value.R, R/query_toil_value_df.R
\name{query_toil_value_df}
\alias{query_toil_value_df}
\title{Obtain ToilHub Info for Single Molecule}
\usage{
query_toil_value_df(identifier = "TP53")

query_toil_value_df(identifier = "TP53")
}
\arguments{
\item{identifier}{a length-1 character representing a gene symbol, ensembl gene id, or probe id.
Gene symbol is highly recommended.}
}
\value{
a \code{tibble}

a \code{tibble}
}
\description{
Obtain ToilHub Info for Single Molecule

Obtain ToilHub Info for Single Gene
}
\examples{
\dontrun{
t <- query_toil_value_df()
t
}
\dontrun{
t <- query_toil_value_df()
t
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ezcor.R
\name{ezcor}
\alias{ezcor}
\title{Run Correlation between Two Variables and Support Group by a Variable}
\usage{
ezcor(
  data = NULL,
  split = FALSE,
  split_var = NULL,
  var1 = NULL,
  var2 = NULL,
  cor_method = "pearson",
  adjust_method = "none",
  use = "complete",
  sig_label = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{data}{a \code{data.frame} containing variables}

\item{split}{whether perform correlation grouped by a variable, default is 'FALSE'}

\item{split_var}{a \code{character}, the group variable}

\item{var1}{a character, the first variable in correlation}

\item{var2}{a character, the second variable in correlation}

\item{cor_method}{method="pearson" is the default value. The alternatives to be passed to cor are "spearman" and "kendall"}

\item{adjust_method}{What adjustment for multiple tests should be used? ("holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none")}

\item{use}{use="pairwise" will do pairwise deletion of cases. use="complete" will select just complete cases}

\item{sig_label}{whether add symbal of significance. P < 0.001,\verb{***}; P < 0.01,\verb{**}; P < 0.05,\code{*}; P >=0.05,""}

\item{verbose}{if \code{TRUE}, print extra info.}
}
\value{
a \code{data.frame}
}
\description{
Run Correlation between Two Variables and Support Group by a Variable
}
\author{
Yi Xiong
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tcga_surv.R
\name{tcga survival analysis}
\alias{tcga survival analysis}
\alias{tcga_surv_get}
\alias{tcga_surv_plot}
\title{TCGA Survival Analysis}
\usage{
tcga_surv_get(
  item,
  TCGA_cohort = "LUAD",
  profile = c("mRNA", "miRNA", "methylation", "transcript", "protein", "mutation", "cnv"),
  TCGA_cli_data = dplyr::full_join(load_data("tcga_clinical"), load_data("tcga_surv"), by
    = "sample"),
  opt_pancan = .opt_pancan
)

tcga_surv_plot(
  data,
  time = "time",
  status = "status",
  cutoff_mode = c("Auto", "Custom"),
  cutpoint = c(50, 50),
  cnv_type = c("Duplicated", "Normal", "Deleted"),
  profile = c("mRNA", "miRNA", "methylation", "transcript", "protein", "mutation", "cnv"),
  palette = "aaas",
  ...
)
}
\arguments{
\item{item}{a molecular identifier, can be gene symbol (common cases), protein symbol, etc.}

\item{TCGA_cohort}{a TCGA cohort, e.g. "LUAD" (default), "LUSC", "ACC".}

\item{profile}{a molecular profile. Option can be one of "mRNA" (default), "miRNA", "methylation", "transcript", "protein", "mutation", "cnv".}

\item{TCGA_cli_data}{a \code{data.frame} containing TCGA clinical data. Default use pre-compiled TCGA clinical data in
this package.}

\item{opt_pancan}{specify one dataset for some molercular profiles}

\item{data}{a subset of result from \code{tcga_surv_get()}.}

\item{time}{the column name for "time".}

\item{status}{the column name for "status".}

\item{cutoff_mode}{mode for grouping samples, can be "Auto" (default) or "Custom".}

\item{cutpoint}{cut point (in percent) for "Custom" mode, default is \code{c(50, 50)}.}

\item{cnv_type}{only used when profile is "cnv", can select from \code{c("Duplicated", "Normal", "Deleted")}.}

\item{palette}{color palette, can be "hue", "grey", "RdBu", "Blues", "npg", "aaas", etc.
More see \code{?survminer::ggsurvplot}.}

\item{...}{other parameters passing to \code{survminer::ggsurvplot}}
}
\value{
a \code{data.frame} or a plot.
}
\description{
\itemize{
\item Firstly, get merged data of one molecular profile value and associated clinical data from TCGA Pan-Cancer dataset.
\item Secondly, filter data as your wish.
\item Finally, show K-M plot.
}
}
\examples{
\dontrun{
# 1. get data
data <- tcga_surv_get("TP53")
# 2. filter data (optional)

# 3. show K-M plot
tcga_surv_plot(data, time = "DSS.time", status = "DSS")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ccle_absolute}
\alias{ccle_absolute}
\title{ABSOLUTE Result of CCLE Database}
\format{
A \code{data.frame}
}
\source{
see "data_source" attribute.
}
\description{
ABSOLUTE Result of CCLE Database
}
\examples{
data("ccle_absolute")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{pcawg_purity}
\alias{pcawg_purity}
\title{Purity Data of PCAWG}
\format{
A \code{data.frame}
}
\source{
UCSC Xena.
}
\description{
Purity Data of PCAWG
}
\examples{
data("pcawg_purity")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run.R
\name{app_run2}
\alias{app_run2}
\title{Run UCSC Xena Shiny App with specifc content}
\usage{
app_run2(
  runMode = "client",
  port = getOption("shiny.port"),
  content = c("a", "s", "q", "p", "d")
)
}
\arguments{
\item{runMode}{default is 'client' for personal user, set it to 'server' for running on server.}

\item{port}{The TCP port that the application should listen on. If the
\code{port} is not specified, and the \code{shiny.port} option is set (with
\code{options(shiny.port = XX)}), then that port will be used. Otherwise,
use a random port between 3000:8000, excluding ports that are blocked
by Google Chrome for being considered unsafe: 3659, 4045, 5060,
5061, 6000, 6566, 6665:6669 and 6697. Up to twenty random
ports will be tried.}

\item{content}{Modules to enable.
\itemize{
\item \code{a}: all modules
\item \code{s}: only loading basic modules;
\item \code{q}: add tpc (TCGA, PCAWG, CCLE) modules
\item \code{p}: add tpc pipelines
\item \code{d}: add pharmcogenomics modules
}}
}
\description{
Run UCSC Xena Shiny App with specifc content
}
\examples{
\dontrun{
app_run2(content = "s")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xenashiny.R
\docType{package}
\name{UCSCXenaShiny-package}
\alias{UCSCXenaShiny}
\alias{UCSCXenaShiny-package}
\title{Xena Shiny App}
\description{
A Shiny App for UCSC Xena Data Hubs. See \url{https://github.com/openbiox/UCSCXenaShiny} for details.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/openbiox/UCSCXenaShiny}
  \item \url{https://openbiox.github.io/UCSCXenaShiny/}
  \item Report bugs at \url{https://github.com/openbiox/UCSCXenaShiny/issues}
}

}
\author{
\strong{Maintainer}: Shixiang Wang \email{w_shixiang@163.com} (\href{https://orcid.org/0000-0001-9855-7357}{ORCID})

Authors:
\itemize{
  \item Shensuo Li \email{lishensuo@163.com}
  \item Yi Xiong \email{neuroxiong@openbiox.com} (\href{https://orcid.org/0000-0002-4370-9824}{ORCID})
  \item Longfei Zhao \email{longfei8533@126.com} (\href{https://orcid.org/0000-0002-6277-0137}{ORCID})
  \item Kai Gu \email{gukai1212@163.com} (\href{https://orcid.org/0000-0002-0177-0774}{ORCID})
  \item Yin Li \email{yinli18@fudan.edu.cn}
  \item Fei Zhao \email{zhaofei415@mails.ucas.edu.cn}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyze_gene_drug_response.R
\name{analyze_gene_drug_response_diff}
\alias{analyze_gene_drug_response_diff}
\title{Analyze Difference of Drug Response (IC50 Value (uM)) between Gene (Signature) High and Low Expression with CCLE Data}
\usage{
analyze_gene_drug_response_diff(
  gene_list,
  drug = "ALL",
  tissue = "ALL",
  combine = FALSE,
  cutpoint = c(50, 50)
)
}
\arguments{
\item{gene_list}{a gene symbol list.}

\item{drug}{a drug name. Check examples.}

\item{tissue}{a tissue name. Check examples.}

\item{combine}{if \code{TRUE}, combine the expression of gene list as
a gene signature.}

\item{cutpoint}{cut point (in percent) for High and Low group, default is \code{c(50, 50)}.}
}
\value{
a \code{data.frame}.
}
\description{
Analyze Difference of Drug Response (IC50 Value (uM)) between Gene (Signature) High and Low Expression with CCLE Data
}
\examples{
tissue_list <- c(
  "prostate", "central_nervous_system", "urinary_tract", "haematopoietic_and_lymphoid_tissue",
  "kidney", "thyroid", "soft_tissue", "skin", "salivary_gland",
  "ovary", "lung", "bone", "endometrium", "pancreas", "breast",
  "large_intestine", "upper_aerodigestive_tract", "autonomic_ganglia",
  "stomach", "liver", "biliary_tract", "pleura", "oesophagus"
)

drug_list <- c(
  "AEW541", "Nilotinib", "17-AAG", "PHA-665752", "Lapatinib",
  "Nutlin-3", "AZD0530", "PF2341066", "L-685458", "ZD-6474", "Panobinostat",
  "Sorafenib", "Irinotecan", "Topotecan", "LBW242", "PD-0325901",
  "PD-0332991", "Paclitaxel", "AZD6244", "PLX4720", "RAF265", "TAE684",
  "TKI258", "Erlotinib"
)

target_list <- c(
  "IGF1R", "ABL", "HSP90", "c-MET", "EGFR", "MDM2", "GS", "HDAC",
  "RTK", "TOP1", "XIAP", "MEK", "CDK4", "TUBB1", "RAF", "ALK", "FGFR"
)
\dontrun{
analyze_gene_drug_response_diff("TP53")
analyze_gene_drug_response_diff(c("TP53", "KRAS"), drug = "AEW541")
analyze_gene_drug_response_diff(c("TP53", "KRAS"),
  tissue = "kidney",
  combine = TRUE
)

# Visualization
vis_gene_drug_response_diff("TP53")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ccle.R, R/get_pancan_value.R,
%   R/get_pcawg_value.R
\name{get_ccle_cn_value}
\alias{get_ccle_cn_value}
\alias{get_ccle_gene_value}
\alias{get_ccle_protein_value}
\alias{get_ccle_mutation_status}
\alias{get_pancan_value}
\alias{get_pancan_gene_value}
\alias{get_pancan_transcript_value}
\alias{get_pancan_protein_value}
\alias{get_pancan_mutation_status}
\alias{get_pancan_cn_value}
\alias{get_pancan_methylation_value}
\alias{get_pancan_miRNA_value}
\alias{get_pcawg_gene_value}
\alias{get_pcawg_fusion_value}
\alias{get_pcawg_promoter_value}
\alias{get_pcawg_miRNA_value}
\alias{get_pcawg_APOBEC_mutagenesis_value}
\title{Fetch Identifier Value from Pan-cancer Dataset}
\usage{
get_ccle_cn_value(identifier)

get_ccle_gene_value(identifier, norm = c("rpkm", "nc"))

get_ccle_protein_value(identifier)

get_ccle_mutation_status(identifier)

get_pancan_value(
  identifier,
  subtype = NULL,
  dataset = NULL,
  host = available_hosts(),
  samples = NULL,
  ...
)

get_pancan_gene_value(identifier, norm = c("tpm", "fpkm", "nc"))

get_pancan_transcript_value(identifier, norm = c("tpm", "fpkm", "isopct"))

get_pancan_protein_value(identifier)

get_pancan_mutation_status(identifier)

get_pancan_cn_value(identifier, gistic2 = TRUE, use_thresholded_data = FALSE)

get_pancan_methylation_value(
  identifier,
  type = c("450K", "27K"),
  rule_out = NULL,
  aggr = c("NA", "mean", "Q0", "Q25", "Q50", "Q75", "Q100")
)

get_pancan_miRNA_value(identifier)

get_pcawg_gene_value(identifier)

get_pcawg_fusion_value(identifier)

get_pcawg_promoter_value(identifier, type = c("raw", "relative", "outlier"))

get_pcawg_miRNA_value(identifier, norm = c("TMM", "UQ"))

get_pcawg_APOBEC_mutagenesis_value(
  identifier = c("tCa_MutLoad_MinEstimate", "APOBECtCa_enrich", "A3A_or_A3B",
    "APOBEC_tCa_enrich_quartile", "APOBECrtCa_enrich", "APOBECytCa_enrich",
    "APOBECytCa_enrich-APOBECrtCa_enrich", "BH_Fisher_p-value_tCa", "ntca+tgan",
    "rtCa_to_G+rtCa_to_T", "rtca+tgay", "tCa_to_G+tCa_to_T",
    "ytCa_rtCa_BH_Fisher_p-value", "ytCa_rtCa_Fisher_p-value", "ytCa_to_G+ytCa_to_T",
    "ytca+tgar")
)
}
\arguments{
\item{identifier}{a length-1 character representing a gene symbol, ensembl gene id, or probe id.
Gene symbol is highly recommended.}

\item{norm}{the normalization method.}

\item{subtype}{a length-1 chracter representing a regular expression for matching
\code{DataSubtype} column of \link[UCSCXenaTools:XenaData]{UCSCXenaTools::XenaData}.}

\item{dataset}{a length-1 chracter representing a regular expression for matching
\code{XenaDatasets} of \link[UCSCXenaTools:XenaData]{UCSCXenaTools::XenaData}.}

\item{host}{a character vector representing host name(s), e.g. "toilHub".}

\item{samples}{a character vector representing samples want to be returned.}

\item{...}{other parameters.}

\item{gistic2}{if \code{TRUE} (default), use GISTIC2 data.}

\item{use_thresholded_data}{if \code{TRUE}, use GISTIC2-thresholded value.}

\item{type}{methylation type, one of "450K" and "27K".
for function \code{get_pcawg_promoter_value}, it can be one of
"raw", "relative", "outlier".}

\item{rule_out}{methylation sites to rule out before analyzing.}

\item{aggr}{apporaches to aggregate the methylation data, default is 'NA',
in such case, a mean value is obtained for gene-level methylation.
Allowed value is one of \code{c("NA", "mean", "Q0", "Q25", "Q50", "Q75", "Q100")}.
Here, \code{Q50} is median.}
}
\value{
a named vector or \code{list}.
}
\description{
Identifier includes gene/probe etc.
}
\section{Functions}{
\itemize{
\item \code{get_ccle_cn_value()}: Fetch copy number value from CCLE dataset

\item \code{get_ccle_gene_value()}: Fetch gene expression value from CCLE dataset

\item \code{get_ccle_protein_value()}: Fetch gene protein expression value from CCLE dataset

\item \code{get_ccle_mutation_status()}: Fetch gene mutation info from CCLE dataset

\item \code{get_pancan_value()}: Fetch identifier value from pan-cancer dataset

\item \code{get_pancan_gene_value()}: Fetch gene expression value from pan-cancer dataset

\item \code{get_pancan_transcript_value()}: Fetch gene transcript expression value from pan-cancer dataset

\item \code{get_pancan_protein_value()}: Fetch protein expression value from pan-cancer dataset

\item \code{get_pancan_mutation_status()}: Fetch mutation status value from pan-cancer dataset

\item \code{get_pancan_cn_value()}: Fetch gene copy number value from pan-cancer dataset processed by GISTIC 2.0

\item \code{get_pancan_methylation_value()}: Fetch gene expression value from CCLE dataset

\item \code{get_pancan_miRNA_value()}: Fetch miRNA expression value from pan-cancer dataset

\item \code{get_pcawg_gene_value()}: Fetch specimen-level gene expression value from PCAWG cohort

\item \code{get_pcawg_fusion_value()}: Fetch specimen-level gene fusion value from PCAWG cohort

\item \code{get_pcawg_promoter_value()}: Fetch specimen-level gene promoter activity value from PCAWG cohort

\item \code{get_pcawg_miRNA_value()}: Fetch specimen-level miRNA value from PCAWG cohort

\item \code{get_pcawg_APOBEC_mutagenesis_value()}: Fetch specimen-level gene fusion value from PCAWG cohort

}}
\examples{
\dontrun{
# Fetch TP53 expression value from pan-cancer dataset
t1 <- get_pancan_value("TP53",
  dataset = "TcgaTargetGtex_rsem_isoform_tpm",
  host = "toilHub"
)
t2 <- get_pancan_gene_value("TP53")
t3 <- get_pancan_protein_value("AKT")
t4 <- get_pancan_mutation_status("TP53")
t5 <- get_pancan_cn_value("TP53")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_identifier.R
\name{vis_identifier_multi_cor}
\alias{vis_identifier_multi_cor}
\title{Visualize Correlation for Multiple Identifiers}
\usage{
vis_identifier_multi_cor(
  dataset,
  ids,
  samples = NULL,
  matrix.type = c("full", "upper", "lower"),
  type = c("parametric", "nonparametric", "robust", "bayes"),
  partial = FALSE,
  sig.level = 0.05,
  p.adjust.method = c("holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr",
    "none"),
  color_low = "#E69F00",
  color_high = "#009E73",
  ...
)
}
\arguments{
\item{dataset}{the dataset to obtain identifiers.}

\item{ids}{the molecule identifiers.}

\item{samples}{default is \code{NULL}, can be common sample names for two datasets.}

\item{matrix.type}{Character, \code{"upper"} (default), \code{"lower"}, or \code{"full"},
display full matrix, lower triangular or upper triangular matrix.}

\item{type}{A character specifying the type of statistical approach:
\itemize{
\item \code{"parametric"}
\item \code{"nonparametric"}
\item \code{"robust"}
\item \code{"bayes"}
}

You can specify just the initial letter.}

\item{partial}{Can be \code{TRUE} for partial correlations. For Bayesian partial
correlations, "full" instead of pseudo-Bayesian partial correlations (i.e.,
Bayesian correlation based on frequentist partialization) are returned.}

\item{sig.level}{Significance level (Default: \code{0.05}). If the \emph{p}-value in
\emph{p}-value matrix is bigger than \code{sig.level}, then the corresponding
correlation coefficient is regarded as insignificant and flagged as such in
the plot.}

\item{p.adjust.method}{Adjustment method for \emph{p}-values for multiple
comparisons. Possible methods are: \code{"holm"} (default), \code{"hochberg"},
\code{"hommel"}, \code{"bonferroni"}, \code{"BH"}, \code{"BY"}, \code{"fdr"}, \code{"none"}.}

\item{color_low}{the color code for lower value mapping.}

\item{color_high}{the color code for higher value mapping.}

\item{...}{other parameters passing to \link[ggstatsplot:ggcorrmat]{ggstatsplot::ggcorrmat}.}
}
\value{
a (gg)plot object.
}
\description{
NOTE: the dataset must be dense matrix in UCSC Xena data hubs.
}
\examples{
\dontrun{
dataset <- "TcgaTargetGtex_rsem_isoform_tpm"
ids <- c("TP53", "KRAS", "PTEN")
vis_identifier_multi_cor(dataset, ids)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{pcawg_info}
\alias{pcawg_info}
\title{Phenotype Info of PCAWG Database}
\format{
A \code{data.frame}
}
\source{
UCSC Xena.
}
\description{
Phenotype Info of PCAWG Database
}
\examples{
data("pcawg_info")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_gene_pw_cor.R
\name{vis_gene_pw_cor}
\alias{vis_gene_pw_cor}
\title{Visualize Correlation between Gene and Pathway signature Score}
\usage{
vis_gene_pw_cor(
  Gene = "TP53",
  data_type = "mRNA",
  pw_name = "HALLMARK_ADIPOGENESIS",
  cancer_choose = "GBM",
  use_regline = TRUE,
  cor_method = "spearman",
  use_all = FALSE,
  alpha = 0.5,
  color = "#000000",
  filter_tumor = TRUE,
  opt_pancan = .opt_pancan
)
}
\arguments{
\item{Gene}{a molecular identifier (e.g., "TP53") or a formula
specifying genomic signature ("TP53 + 2 * KRAS - 1.3 * PTEN").}

\item{data_type}{choose gene profile type, including
"mRNA", "transcript", "protein", "mutation", "cnv", "methylation", "miRNA".}

\item{pw_name}{the queried Pathway name, see the supported pathway from 'load("toil_sig_score")'default is NULL}

\item{cancer_choose}{select cancer cohort(s)}

\item{use_regline}{if TRUE, add regression line.}

\item{cor_method}{select correlation coefficient (pearson/spearman)}

\item{use_all}{use all sample, default FALSE.}

\item{alpha}{dot alpha.}

\item{color}{dot color.}

\item{filter_tumor}{whether use tumor sample only, default TRUE}

\item{opt_pancan}{specify one dataset for some molercular profiles}
}
\value{
a \code{ggplot} object or  dataframe
}
\description{
Visualize Correlation between Gene and Pathway signature Score
}
\examples{
\dontrun{
vis_gene_pw_cor(
  Gene = "TP53", data_type = "mRNA",
  pw_name = "HALLMARK_ADIPOGENESIS",
  cancer_choose = "BRCA"
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{pcawg_info_fine}
\alias{pcawg_info_fine}
\title{Cleaned Phenotype Info of PCAWG Database for grouping}
\format{
A \code{data.frame}
}
\source{
UCSC Xena.
}
\description{
Cleaned Phenotype Info of PCAWG Database for grouping
}
\examples{
data("pcawg_info_fine")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_general.R
\name{query_general_value}
\alias{query_general_value}
\title{Download data for shiny general analysis}
\usage{
query_general_value(
  L1,
  L2,
  L3,
  database = c("toil", "pcawg", "ccle"),
  tpc_value_nonomics = NULL,
  opt_pancan = NULL,
  custom_metadata = NULL
)
}
\arguments{
\item{L1}{level 1  main datatype}

\item{L2}{level 2  sub datatype}

\item{L3}{level 3  identifier}

\item{database}{one of c("toil","pcawg","ccle")}

\item{tpc_value_nonomics}{non-omics matrix data of one database}

\item{opt_pancan}{molecular datasets parameters}

\item{custom_metadata}{user customized metadata}
}
\description{
Download data for shiny general analysis
}
\examples{
\dontrun{
general_value_id <- UCSCXenaShiny:::query_general_id()
tcga_value_option <- general_value_id[["value"]][[1]]
tcga_index_value <- tcga_value_option[["Tumor index"]]
tcga_immune_value <- tcga_value_option[["Immune Infiltration"]]
tcga_pathway_value <- tcga_value_option[["Pathway activity"]]
tcga_phenotype_value <- tcga_value_option[["Phenotype data"]]

clinical_phe = tcga_phenotype_value[["Clinical Phenotype"]]
x_data = UCSCXenaShiny:::query_general_value(
           "Molecular profile", "mRNA Expression", "TP53", "toil",
           tcga_index_value, tcga_immune_value, tcga_pathway_value,
           clinical_phe)

y_data = UCSCXenaShiny:::query_general_value(
           "Immune Infiltration", "CIBERSORT", "Monocyte", "toil",
           tcga_index_value, tcga_immune_value, tcga_pathway_value,
           clinical_phe)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_identifier.R
\name{vis_identifier_cor}
\alias{vis_identifier_cor}
\title{Visualize Identifier-Identifier Correlation}
\usage{
vis_identifier_cor(
  dataset1,
  id1,
  dataset2,
  id2,
  samples = NULL,
  use_ggstats = FALSE,
  use_simple_axis_label = TRUE,
  line_color = "blue",
  alpha = 0.5,
  ...
)
}
\arguments{
\item{dataset1}{the dataset to obtain \code{id1}.}

\item{id1}{the first molecule identifier.}

\item{dataset2}{the dataset to obtain \code{id2}.}

\item{id2}{the second molecule identifier.}

\item{samples}{default is \code{NULL}, can be common sample names for two datasets.}

\item{use_ggstats}{if \code{TRUE}, use \href{https://github.com/IndrajeetPatil/ggstatsplot}{ggstatsplot} package for plotting.}

\item{use_simple_axis_label}{if \code{TRUE} (default), use simple axis labels.
Otherwise, data subtype will be labeled.}

\item{line_color}{set the color for regression line.}

\item{alpha}{set the alpha for dots.}

\item{...}{other parameters passing to \href{http://rpkgs.datanovia.com/ggpubr/reference/ggscatter.html}{ggscatter}.}
}
\value{
a (gg)plot object.
}
\description{
NOTE: the dataset must be dense matrix in UCSC Xena data hubs.
}
\examples{
\dontrun{
dataset <- "TcgaTargetGtex_rsem_isoform_tpm"
id1 <- "TP53"
id2 <- "KRAS"
vis_identifier_cor(dataset, id1, dataset, id2)

samples <- c(
  "TCGA-D5-5538-01", "TCGA-VM-A8C8-01",
  "TCGA-ZN-A9VQ-01", "TCGA-EE-A17X-06",
  "TCGA-05-4420-01"
)
vis_identifier_cor(dataset, id1, dataset, id2, samples)

dataset1 <- "TCGA-BLCA.htseq_counts.tsv"
dataset2 <- "TCGA-BLCA.gistic.tsv"
id1 <- "TP53"
id2 <- "KRAS"
vis_identifier_cor(dataset1, id1, dataset2, id2)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_nonomics_value.R
\name{get_nonomics_value}
\alias{get_nonomics_value}
\title{Fetch non-omics data of all samples from relevant databases}
\usage{
get_nonomics_value(
  database = c("toil", "pcawg", "ccle"),
  type = c("immune", "pathway", "tumor_index"),
  subtype = NULL
)
}
\arguments{
\item{database}{one of "toil" (tcga), "pcawg", "ccle"}

\item{type}{one of "immune", "pathway", "tumor_index"}

\item{subtype}{For "immune" type of toil/pcawg database, one of "CIBERSORT", "CIBERSORT-ABS", "EPIC", "MCPCOUNTER", "QUANTISEQ", "TIMER", "XCELL".
For "pathway" type of toil/pcawg database, one of "HALLMARK", "KEGG", "IOBR".
For "tumor_index" type of toil database, one of "Tumor_Purity","Tumor_Stemness","Tumor_Mutation_Burden","Microsatellite_Instability", "Genome_Instability".
For "tumor_index" type of pcawg/ccle database, set NULL.
If subtype is NULL, return all subtypes.}
}
\value{
dataframe
}
\description{
Fetch non-omics data of all samples from relevant databases
}
\examples{
\dontrun{
tcga_immune_xcell = get_nonomics_value(database = "toil", type = "immune", subtype = "XCELL")

tcga_pathway_kegg = get_nonomics_value(database = "toil", type = "pathway", subtype = "KEGG")

# Combined with sample information

head(tcga_clinical_fine)
head(tcga_surv)

head(pcawg_info_fine)

head(ccle_info_fine)

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_pancan_value.R
\name{vis_gene_tmb_cor}
\alias{vis_gene_tmb_cor}
\title{Visualize Correlation between Gene and TMB (Tumor Mutation Burden)}
\usage{
vis_gene_tmb_cor(
  Gene = "TP53",
  cor_method = "spearman",
  data_type = "mRNA",
  Plot = "TRUE",
  opt_pancan = .opt_pancan
)
}
\arguments{
\item{Gene}{a molecular identifier (e.g., "TP53") or a formula specifying
genomic signature (\code{"TP53 + 2 * KRAS - 1.3 * PTEN"}).}

\item{cor_method}{correlation method}

\item{data_type}{choose gene profile type,
including "mRNA", "transcript", "protein", "mutation", "cnv", "methylation", "miRNA".}

\item{Plot}{output the plot directly, default 'TRUE'}

\item{opt_pancan}{specify one dataset for some molercular profiles}
}
\description{
Visualize Correlation between Gene and TMB (Tumor Mutation Burden)
}
\examples{
\dontrun{
p <- vis_gene_tmb_cor(Gene = "TP53")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_identifier.R
\name{vis_identifier_grp_surv}
\alias{vis_identifier_grp_surv}
\title{Visualize Identifier Group Survival Difference}
\usage{
vis_identifier_grp_surv(
  dataset = NULL,
  id = NULL,
  surv_df,
  samples = NULL,
  cutoff_mode = c("Auto", "Custom", "None"),
  cutpoint = c(50, 50),
  palette = "aaas",
  ...
)
}
\arguments{
\item{dataset}{the dataset to obtain identifiers.}

\item{id}{the molecule identifier.}

\item{surv_df}{a \code{data.frame}. The "time" should be in unit of "days".
\itemize{
\item If there are 3 columns, the names should be "sample", "time", "status".
\item If there are 4 columns, the names should be "sample", "value", "time", "status".
}}

\item{samples}{default is \code{NULL}, can be common sample names for two datasets.}

\item{cutoff_mode}{mode for grouping samples, can be "Auto" (default) or "Custom" or "None" (for groups have been prepared).}

\item{cutpoint}{cut point (in percent) for "Custom" mode, default is \code{c(50, 50)}.}

\item{palette}{color palette, can be "hue", "grey", "RdBu", "Blues", "npg", "aaas", etc.
More see \code{?survminer::ggsurvplot}.}

\item{...}{other parameters passing to \code{survminer::ggsurvplot}}
}
\value{
a (gg)plot object.
}
\description{
NOTE: the dataset must be dense matrix in UCSC Xena data hubs.
}
\examples{
\dontrun{
library(UCSCXenaTools)
expr_dataset <- "TCGA.LUAD.sampleMap/HiSeqV2_percentile"
cli_dataset <- "TCGA.LUAD.sampleMap/LUAD_clinicalMatrix"
id <- "KRAS"
cli_df <- XenaGenerate(
  subset = XenaDatasets == "TCGA.LUAD.sampleMap/LUAD_clinicalMatrix"
) \%>\%
  XenaQuery() \%>\%
  XenaDownload() \%>\%
  XenaPrepare()

# Use individual survival data
surv_df1 <- cli_df[, c("sampleID", "ABSOLUTE_Ploidy", "days_to_death", "vital_status")]
surv_df1$vital_status <- ifelse(surv_df1$vital_status == "DECEASED", 1, 0)
vis_identifier_grp_surv(surv_df = surv_df1)

# Use both dataset argument and vis_identifier_grp_surv(surv_df = surv_df1)
surv_df2 <- surv_df1[, c(1, 3, 4)]
vis_identifier_grp_surv(expr_dataset, id, surv_df = surv_df2)
vis_identifier_grp_surv(expr_dataset, id,
  surv_df = surv_df2,
  cutoff_mode = "Custom", cutpoint = c(25, 75)
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_dim_dist.R
\name{vis_dim_dist}
\alias{vis_dim_dist}
\title{Visualize the distribution difference of samples after dimensionality reduction analysis}
\usage{
vis_dim_dist(
  ids = c("TP53", "KRAS", "PTEN", "MDM2", "CDKN1A"),
  data_type = "mRNA",
  group_info = NULL,
  DR_method = c("PCA", "UMAP", "tSNE"),
  palette = "Set1",
  add_margin = NULL,
  opt_pancan = .opt_pancan
)
}
\arguments{
\item{ids}{molecular identifiers (>=3)}

\item{data_type}{molecular types, refer to query_pancan_value() function}

\item{group_info}{two-column grouping information with names 'Sample','Group'}

\item{DR_method}{the dimensionality reduction method}

\item{palette}{the color setting of RColorBrewer}

\item{add_margin}{the marginal plot (NULL, "density", "boxplot")}

\item{opt_pancan}{specify one dataset for some molercular profiles}
}
\value{
a ggplot object or rawdata list
}
\description{
Visualize the distribution difference of samples after dimensionality reduction analysis
}
\examples{
\dontrun{
group_info <- tcga_clinical_fine \%>\%
  dplyr::filter(Cancer == "BRCA") \%>\%
  dplyr::select(Sample, Code) \%>\%
  dplyr::rename(Group = Code)

vis_dim_dist(
  ids = c("TP53", "KRAS", "PTEN", "MDM2", "CDKN1A"),
  group_info = group_info
)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_value.R
\name{query_pancan_value}
\alias{query_pancan_value}
\title{Query Single Identifier or Signature Value from Pan-cancer Database}
\usage{
query_pancan_value(
  molecule,
  data_type = c("mRNA", "transcript", "protein", "mutation", "cnv", "methylation",
    "miRNA", "fusion", "promoter", "APOBEC"),
  database = c("toil", "ccle", "pcawg"),
  reset_id = NULL,
  opt_pancan = .opt_pancan
)
}
\arguments{
\item{molecule}{a molecular identifier (e.g., "TP53") or a formula specifying
genomic signature (\code{"TP53 + 2 * KRAS - 1.3 * PTEN"}).}

\item{data_type}{data type. Can be one of "mRNA", "transcript", "protein",
"mutation", "cnv", "methylation", "miRNA".}

\item{database}{database, either 'toil' for TCGA TARGET GTEx, or 'ccle' for
CCLE.}

\item{reset_id}{if not \code{NULL}, set the specified variable at parent frame to "Signature".}

\item{opt_pancan}{other extra parameters passing to the underlying functions.}
}
\value{
a list.
}
\description{
Query Single Identifier or Signature Value from Pan-cancer Database
}
\details{
\code{query_pancan_value()} provide convenient interface to download multi-omics
data from 3 databases by specifying one or several canonical datasets. It is
derived from \code{query_pancan_value()} and support query for genomic signature.
To query comprehensive datasets that UCSCXena supports, users can check
\code{UCSCXenaTools::XenaData} and use \code{get_pancan_value()} directly.

Option \code{opt_pancan} is a nested list and allow to adjust the downloading details.
For now, only \code{cnv(toil)},\code{methylation(toil)},\code{miRNA(toil)},\code{miRNA(pcawg)},\code{promoter(pcawg)}
support optional parameters. The default set is \code{.opt_pancan} and we check meanings of sublist(parameters)
through the following relationship.
}
\section{"toil" database}{

\enumerate{
\item mRNA--\code{get_pancan_gene_value()}
\item transcript--\code{get_pancan_transcript_value()}
\item protein--\code{get_pancan_protein_value()}
\item mutation--\code{get_pancan_mutation_status()}
\item cnv--\code{get_pancan_cn_value()}
\item methylation--\code{get_pancan_methylation_value()}
\item miRNA--\code{get_pancan_miRNA_value()}
}
}

\section{"ccle" database}{

\enumerate{
\item mRNA--\code{get_ccle_gene_value()}
\item protein--\code{get_ccle_protein_value()}
\item mutation--\code{get_ccle_mutation_status()}
\item cnv--\code{get_ccle_cn_value()}
}
}

\section{"pcawg" database}{

\enumerate{
\item mRNA--\code{get_pcawg_gene_value()}
\item miRNA--\code{get_pcawg_miRNA_value()}
\item promoter--\code{get_pcawg_promoter_value()}
\item fusion--\code{get_pcawg_fusion_value()}
\item APOBEC--\code{get_pcawg_APOBEC_mutagenesis_value()}
}
}

\examples{
\dontrun{
query_pancan_value("KRAS")
query_pancan_value("KRAS", database = "ccle")
query_pancan_value("KRAS", database = "pcawg")


query_pancan_value("ENSG00000000419",
  database = "pcawg",
  data_type = "fusion"
) # gene symbol also work

.opt_pancan

opt_pancan <- list(toil_cnv = list(use_thresholded_data = FALSE))
query_pancan_value("PTEN", data_type = "cnv", database = "toil", opt_pancan = opt_pancan)


opt_pancan <- list(toil_methylation = list(type = "450K", rule_out = "cg21115430", aggr = "Q25"))
query_pancan_value("PTEN", data_type = "methylation", database = "toil", opt_pancan = opt_pancan)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_general.R
\name{mol_quick_analysis}
\alias{mol_quick_analysis}
\title{Quick molecule analysis and report generation}
\usage{
mol_quick_analysis(molecule, data_type, out_dir = ".", out_report = FALSE)
}
\arguments{
\item{molecule}{a molecular identifier (e.g., "TP53") or a formula specifying
genomic signature (\code{"TP53 + 2 * KRAS - 1.3 * PTEN"}).}

\item{data_type}{data type. Can be one of "mRNA", "transcript", "protein",
"mutation", "cnv", "methylation", "miRNA".}

\item{out_dir}{path to save analysis result and report, default is '.'}

\item{out_report}{logical value wheather to generate html report}
}
\value{
a list.
}
\description{
Quick molecule analysis and report generation
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/load_data.R
\name{load_data}
\alias{load_data}
\title{Load Dataset Provided by This Package}
\usage{
load_data(name)
}
\arguments{
\item{name}{a dataset name. Could be one of

\strong{Builtin datasets}:
\itemize{
\item \code{ccle_absolute}: CCLE ABSOLUTE result.
\item \code{ccle_info}: CCLE information.
\item \code{ccle_info_fine}: cleaned CCLE information for TPC analysis.
\item \code{pcawg_info}: PCAWG information.
\item \code{pcawg_info_fine}: cleaned PCAWG information for TPC analysis.
\item \code{pcawg_purity}: PCAWG tumor purity, ploidy and WGD data.
\item \code{tcga_clinical}: TCGA clinical data.
\item \code{tcga_clinical_fine}: cleaned TCGA information for TPC analysis.
\item \code{tcga_genome_instability}: TCGA genome instability data.
\item \code{tcga_gtex}: TCGA and GTEX sample info.
\item \code{tcga_purity}: TCGA tumor purity data.
\item \code{tcga_subtypes}: TCGA subtypes data.
\item \code{tcga_surv}: TCGA survival data.
\item \code{TCGA.organ}: TCGA organ data.
\item \code{toil_info}: Toil hub information.
}

\strong{Remote datasets stored in \href{https://zenodo.org/record/12819839/}{Zenodo}}:
\itemize{
\item \code{pcawg_promoter_id}: PCAWG promoter identifiers.
\item \code{transcript_identifier}: Common transcript identifiers.
\item \code{ccle_expr_and_drug_response}: CCLE expression and drug response data.
\item \code{ccle_drug_response_extend}: CCLE drug response extended data.
\item \code{pancan_MSI}: Pan-cancer MSI data.
\item \code{tcga_chr_alteration}: TCGA chromosome alteration data.
\item \code{tcga_MSI}: TCGA MSI data.
\item \code{tcga_pan_immune_signature}: TCGA pan-cancer immune signature.
\item \code{tcga_stemness}: TCGA tumor stemness data.
\item \code{tcga_TIL}: TCGA TIL data.
\item \code{tcga_PW}: ssGSEA scores of HALLMARK, KEGG, IOBR terms for TCGA samples.
\item \code{tcga_PW_meta}: metadata annotation for HALLMARK, KEGG, IOBR terms.
\item \code{tcga_tmb}: TCGA TMB data.
\item \code{tcga_armcalls}: TCGA arm alteration calls and Aneuploidy data.
\item \code{tcga_dna_repair}: TCGA DNA repair data.
\item \code{pancancer_conserved_immune_subtype}: Pan-cancer conserved immune subtypes.
\item \code{pcawg_TIL}: PCAWG TIL data.
\item \code{pcawg_PW}: ssGSEA scores of HALLMARK, KEGG, IOBR terms for PCAWG samples.
\item ...
}}
}
\value{
a dataset, typically a \code{data.frame}.
}
\description{
Load data from builtin or Zenodo.
Option \code{xena.zenodoDir} can be used to set default path for storing
extra data from Zenodo, e.g., \code{options(xena.zenodoDir = "/home/xxx/dataset")}.
}
\examples{
data1 <- load_data("tcga_surv")
data1
\donttest{
data2 <- load_data("tcga_armcalls")
data2
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tcga_clinical_fine}
\alias{tcga_clinical_fine}
\title{Toil Hub: Cleaned TCGA Clinical Data for grouping}
\format{
A \link{data.frame}
}
\source{
Generate from data-raw
}
\description{
See \code{tcga_surv} for TCGA survival data.
}
\examples{
data("tcga_clinical_fine")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_pancan_value.R
\name{vis_gene_cor}
\alias{vis_gene_cor}
\title{Visualize Gene-Gene Correlation in TCGA}
\usage{
vis_gene_cor(
  Gene1 = "CSF1R",
  Gene2 = "JAK3",
  data_type1 = "mRNA",
  data_type2 = "mRNA",
  use_regline = TRUE,
  purity_adj = TRUE,
  alpha = 0.5,
  color = "#000000",
  filter_tumor = TRUE,
  opt_pancan = .opt_pancan
)
}
\arguments{
\item{Gene1}{a molecular identifier (e.g., "TP53") or a formula specifying
genomic signature (\code{"TP53 + 2 * KRAS - 1.3 * PTEN"}).}

\item{Gene2}{a molecular identifier (e.g., "TP53") or a formula specifying
genomic signature (\code{"TP53 + 2 * KRAS - 1.3 * PTEN"}).}

\item{data_type1}{choose gene profile type for the first gene, including "mRNA","transcript","methylation","miRNA","protein","cnv_gistic2"}

\item{data_type2}{choose gene profile type for the second gene, including "mRNA","transcript","methylation","miRNA","protein","cnv_gistic2"}

\item{use_regline}{if \code{TRUE}, add regression line.}

\item{purity_adj}{whether performing partial correlation adjusted by purity}

\item{alpha}{dot alpha.}

\item{color}{dot color.}

\item{filter_tumor}{whether use tumor sample only, default \code{TRUE}}

\item{opt_pancan}{specify one dataset for some molercular profiles}
}
\description{
Visualize Gene-Gene Correlation in TCGA
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_identifier.R
\name{vis_identifier_grp_comparison}
\alias{vis_identifier_grp_comparison}
\title{Visualize Comparison of an Molecule Identifier between Groups}
\usage{
vis_identifier_grp_comparison(
  dataset = NULL,
  id = NULL,
  grp_df,
  samples = NULL,
  fun_type = c("betweenstats", "withinstats"),
  type = c("parametric", "nonparametric", "robust", "bayes"),
  pairwise.comparisons = TRUE,
  p.adjust.method = c("holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr",
    "none"),
  ggtheme = cowplot::theme_cowplot(),
  ...
)
}
\arguments{
\item{dataset}{the dataset to obtain identifiers.}

\item{id}{the molecule identifier.}

\item{grp_df}{When \code{dataset} and \code{id} are all not \code{NULL}, it should be a \code{data.frame} with 2 or 3 columns.
\itemize{
\item The first column refers to sample ID.
\item The second column refers to groups indicated in axis X.
\item The third column is optional, which indicates facet variable.
When any of \code{dataset} and \code{id} is \code{NULL}, it should be a \code{data.frame} with 3 or 4 columns.
\item The first column refers to sample ID.
\item The second column refers to values indicated in axis Y.
\item The third column refers to groups indicated in axis X.
\item The fourth column is optional, which indicates facet variable.
}}

\item{samples}{default is \code{NULL}, can be common sample names for two datasets.}

\item{fun_type}{select the function to compare groups.}

\item{type}{A character specifying the type of statistical approach:
\itemize{
\item \code{"parametric"}
\item \code{"nonparametric"}
\item \code{"robust"}
\item \code{"bayes"}
}

You can specify just the initial letter.}

\item{pairwise.comparisons}{whether pairwise comparison}

\item{p.adjust.method}{Adjustment method for \emph{p}-values for multiple
comparisons. Possible methods are: \code{"holm"} (default), \code{"hochberg"},
\code{"hommel"}, \code{"bonferroni"}, \code{"BH"}, \code{"BY"}, \code{"fdr"}, \code{"none"}.}

\item{ggtheme}{A \code{{ggplot2}} theme. Default value is
\code{\link[ggstatsplot:theme_ggstatsplot]{theme_ggstatsplot()}}. Any of the \code{{ggplot2}} themes (e.g.,
\code{\link[ggplot2:ggtheme]{ggplot2::theme_bw()}}), or themes from extension packages are allowed
(e.g., \code{ggthemes::theme_fivethirtyeight()}, \code{hrbrthemes::theme_ipsum_ps()},
etc.). But note that sometimes these themes will remove some of the details
that \code{{ggstatsplot}} plots typically contains. For example, if relevant,
\code{\link[ggstatsplot:ggbetweenstats]{ggbetweenstats()}} shows details about multiple comparison test as a
label on the secondary Y-axis. Some themes (e.g.
\code{ggthemes::theme_fivethirtyeight()}) will remove the secondary Y-axis and
thus the details as well.}

\item{...}{other parameters passing to \link[ggstatsplot:ggbetweenstats]{ggstatsplot::ggbetweenstats} or \link[ggstatsplot:ggwithinstats]{ggstatsplot::ggwithinstats}.}
}
\value{
a (gg)plot object.
}
\description{
NOTE: the dataset must be dense matrix in UCSC Xena data hubs.
}
\examples{
\dontrun{
library(UCSCXenaTools)
expr_dataset <- "TCGA.LUAD.sampleMap/HiSeqV2_percentile"
cli_dataset <- "TCGA.LUAD.sampleMap/LUAD_clinicalMatrix"
id <- "TP53"
cli_df <- XenaGenerate(
  subset = XenaDatasets == "TCGA.LUAD.sampleMap/LUAD_clinicalMatrix"
) \%>\%
  XenaQuery() \%>\%
  XenaDownload() \%>\%
  XenaPrepare()

# group data.frame with 2 columns
vis_identifier_grp_comparison(expr_dataset, id, cli_df[, c("sampleID", "gender")])
# group data.frame with 3 columns
vis_identifier_grp_comparison(
  expr_dataset, id,
  cli_df[, c("sampleID", "pathologic_M", "gender")] \%>\%
    dplyr::filter(pathologic_M \%in\% c("M0", "MX"))
)

# When not use the value of `identifier` from `dataset`
vis_identifier_grp_comparison(grp_df = cli_df[, c(1, 2, 71)])
vis_identifier_grp_comparison(grp_df = cli_df[, c(1, 2, 71, 111)])
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_analyze_gene_drug_response.R
\name{vis_gene_drug_response_asso}
\alias{vis_gene_drug_response_asso}
\title{Visualize Gene and Drug-Target Association with CCLE Data}
\usage{
vis_gene_drug_response_asso(
  Gene = "TP53",
  x_axis_type = c("mean.diff", "median.diff"),
  output_form = c("plotly", "ggplot2")
)
}
\arguments{
\item{Gene}{a molecular identifier (e.g., "TP53") or a formula specifying
genomic signature (\code{"TP53 + 2 * KRAS - 1.3 * PTEN"}).}

\item{x_axis_type}{set the value type for X axis.}

\item{output_form}{\code{plotly} or \code{ggplot2}.}
}
\value{
\code{plotly} or \code{ggplot2} object.
}
\description{
See \link{analyze_gene_drug_response_asso} for examples.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tcga_gtex}
\alias{tcga_gtex}
\title{Toil Hub: Merged TCGA GTEx Selected Phenotype}
\format{
A \link{data.frame}
}
\description{
Toil Hub: Merged TCGA GTEx Selected Phenotype
}
\examples{
data("tcga_gtex")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_ccle_value.R
\name{vis_ccle_tpm}
\alias{vis_ccle_tpm}
\title{Visualize CCLE Gene Expression}
\usage{
vis_ccle_tpm(
  Gene = "TP53",
  data_type = "mRNA",
  use_log = FALSE,
  opt_pancan = .opt_pancan
)
}
\arguments{
\item{Gene}{a molecular identifier (e.g., "TP53") or a formula specifying
genomic signature (\code{"TP53 + 2 * KRAS - 1.3 * PTEN"}).}

\item{data_type}{support genomic profile for CCLE, currently "mRNA", "protein","cnv" are supported}

\item{use_log}{if \code{TRUE}, log values.}

\item{opt_pancan}{specify one dataset for some molercular profiles}
}
\value{
a \code{ggplot} object
}
\description{
Visualize CCLE Gene Expression
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_value.R
\docType{data}
\name{.opt_pancan}
\alias{.opt_pancan}
\title{A default setting for pan-cancer studies}
\format{
An object of class \code{list} of length 16.
}
\usage{
.opt_pancan
}
\description{
A default setting for pan-cancer studies
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_pancan_value.R
\name{vis_gene_stemness_cor}
\alias{vis_gene_stemness_cor}
\title{Visualize Correlation between Gene and Tumor Stemness}
\usage{
vis_gene_stemness_cor(
  Gene = "TP53",
  cor_method = "spearman",
  data_type = "mRNA",
  Plot = "TRUE",
  opt_pancan = .opt_pancan
)
}
\arguments{
\item{Gene}{a molecular identifier (e.g., "TP53") or a formula specifying
genomic signature (\code{"TP53 + 2 * KRAS - 1.3 * PTEN"}).}

\item{cor_method}{correlation method}

\item{data_type}{choose gene profile type,
including "mRNA", "transcript", "protein", "mutation", "cnv", "methylation", "miRNA".}

\item{Plot}{output the plot directly, default 'TRUE'}

\item{opt_pancan}{specify one dataset for some molercular profiles}
}
\description{
Visualize Correlation between Gene and Tumor Stemness
}
\examples{
\dontrun{
p <- vis_gene_stemness_cor(Gene = "TP53")
p
}
## To generate a radar plot, uncomment the following code
# pdata <- p$data \%>\%
#   dplyr::mutate(cor = round(cor, digits = 3), p.value = round(p.value, digits = 3))
#
# df <- pdata \%>\%
#   select(cor, cancer) \%>\%
#   pivot_wider(names_from = cancer, values_from = cor)
#
# ggradar::ggradar(
#   df[1, ],
#   font.radar = "sans",
#   values.radar = c("-1", "0", "1"),
#   grid.min = -1, grid.mid = 0, grid.max = 1,
#   # Background and grid lines
#   background.circle.colour = "white",
#   gridline.mid.colour = "grey",
#   # Polygons
#   group.line.width = 1,
#   group.point.size = 3,
#   group.colours = "#00AFBB") +
#   theme(plot.title = element_text(hjust = .5))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_pancan_value.R
\name{vis_pancan_anatomy}
\alias{vis_pancan_anatomy}
\title{Visualize Single Gene Expression in Anatomy Location}
\usage{
vis_pancan_anatomy(
  Gene = "TP53",
  Gender = c("Female", "Male"),
  data_type = "mRNA",
  option = "D",
  opt_pancan = .opt_pancan
)
}
\arguments{
\item{Gene}{a molecular identifier (e.g., "TP53") or a formula specifying
genomic signature (\code{"TP53 + 2 * KRAS - 1.3 * PTEN"}).}

\item{Gender}{a string, "Female" (default) or "Male".}

\item{data_type}{choose gene profile type, including "mRNA","transcript","methylation","miRNA","protein","cnv"}

\item{option}{A character string indicating the color map option to use.
Eight options are available:
\itemize{
\item \code{"magma"} (or \code{"A"})
\item \code{"inferno"} (or \code{"B"})
\item \code{"plasma"} (or \code{"C"})
\item \code{"viridis"} (or \code{"D"})
\item \code{"cividis"} (or \code{"E"})
\item \code{"rocket"} (or \code{"F"})
\item \code{"mako"} (or \code{"G"})
\item \code{"turbo"} (or \code{"H"})
}}

\item{opt_pancan}{specify one dataset for some molercular profiles}
}
\value{
a \code{ggplot} object
}
\description{
Visualize Single Gene Expression in Anatomy Location
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ezcor.R
\name{ezcor_partial_cor}
\alias{ezcor_partial_cor}
\title{Run partial correlation}
\usage{
ezcor_partial_cor(
  data = NULL,
  split = FALSE,
  split_var = NULL,
  var1 = NULL,
  var2 = NULL,
  var3 = NULL,
  cor_method = "pearson",
  sig_label = TRUE,
  ...
)
}
\arguments{
\item{data}{a \code{data.frame} containing variables}

\item{split}{whether perform correlation grouped by a variable, default is 'FALSE'}

\item{split_var}{a \code{character}, the group variable}

\item{var1}{a \code{character}, the first variable in correlation}

\item{var2}{a \code{character}, the second variable in correlation}

\item{var3}{a \code{character} or \verb{character vector}, the third variable in correlation}

\item{cor_method}{method="pearson" is the default value. The alternatives to be passed to cor are "spearman" and "kendall"}

\item{sig_label}{whether add symbal of significance. P < 0.001,"\emph{\strong{"; P < 0.01,"}"; P < 0.05,"}"; P >=0.05,""}

\item{...}{other arguments passed to methods}
}
\value{
a \code{data.frame}
}
\description{
Run partial correlation
}
\seealso{
\code{\link[ppcor:pcor.test]{ppcor::pcor.test()}} which this function wraps.
}
\author{
Yi Xiong
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_pancan_value.R
\name{vis_toil_TvsN_cancer}
\alias{vis_toil_TvsN_cancer}
\title{Visualize Gene TPM in Single Cancer Type (Tumor (TCGA) vs Normal (TCGA & GTEx))}
\usage{
vis_toil_TvsN_cancer(
  Gene = "TP53",
  Mode = c("Violinplot", "Dotplot"),
  data_type = "mRNA",
  Show.P.value = FALSE,
  Show.P.label = FALSE,
  Method = "wilcox.test",
  values = c("#DF2020", "#DDDF21"),
  TCGA.only = FALSE,
  Cancer = "ACC",
  opt_pancan = .opt_pancan
)
}
\arguments{
\item{Gene}{a molecular identifier (e.g., "TP53") or a formula specifying
genomic signature (\code{"TP53 + 2 * KRAS - 1.3 * PTEN"}).}

\item{Mode}{"Boxplot" or "Violinplot" to represent data}

\item{data_type}{choose gene profile type,
including "mRNA", "transcript", "protein", "mutation", "cnv", "methylation", "miRNA".}

\item{Show.P.value}{\code{TRUE} or \code{FALSE} whether to count P value}

\item{Show.P.label}{\code{TRUE} or \code{FALSE} present p value with number or label \code{*}, \verb{**}, \verb{***} and \verb{****}}

\item{Method}{default method is wilcox.test}

\item{values}{the color to fill tumor or normal}

\item{TCGA.only}{include samples only from TCGA dataset}

\item{Cancer}{select cancer cohort(s).}

\item{opt_pancan}{specify one dataset for some molercular profiles}
}
\value{
a \code{ggplot} object.
}
\description{
Visualize Gene TPM in Single Cancer Type (Tumor (TCGA) vs Normal (TCGA & GTEx))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_tcga_group.R
\name{query_tcga_group}
\alias{query_tcga_group}
\title{Group TPC samples by build-in or custom phenotype and support filtering or merging operations}
\usage{
query_tcga_group(
  database = c("toil", "pcawg", "ccle"),
  cancer = NULL,
  custom = NULL,
  group = "Gender",
  filter_by = NULL,
  filter_id = NULL,
  merge_by = NULL,
  merge_quantile = FALSE,
  return_all = FALSE
)
}
\arguments{
\item{database}{one of c("toil","pcawg","ccle")}

\item{cancer}{select cancer cohort(s)}

\item{custom}{upload custom phenotype data}

\item{group}{target group names}

\item{filter_by}{filter samples by one or multiple criterion}

\item{filter_id}{directly filter samples by provided sample ids}

\item{merge_by}{merge the target group for main categories}

\item{merge_quantile}{whether to merge numerical variable by percentiles}

\item{return_all}{return the all phenotype data}
}
\value{
a list object with grouping samples and statistics
}
\description{
Group TPC samples by build-in or custom phenotype and support filtering or merging operations
}
\examples{
\dontrun{
query_tcga_group(group = "Age")

query_tcga_group(
  cancer = "BRCA",
  group = "Stage_ajcc"
)

query_tcga_group(
  cancer = "BRCA",
  group = "Stage_ajcc",
  filter_by = list(
    c("Code", c("TP"), "+"),
    c("Stage_ajcc", c(NA), "-")
  )
)

query_tcga_group(
  cancer = "BRCA",
  group = "Stage_ajcc",
  filter_by = list(
    c("Age", c(0.5), "\%>")
  )
)

query_tcga_group(
  cancer = "BRCA",
  group = "Stage_ajcc",
  filter_by = list(
    c("Age", c(60), ">")
  )
)

query_tcga_group(
  cancer = "BRCA",
  group = "Stage_ajcc",
  merge_by = list(
    "Early" = c("Stage I"),
    "Late" = c("Stage II", "Stage III", "Stage IV")
  )
)

query_tcga_group(
  cancer = "BRCA",
  group = "Age",
  merge_by = list(
    "Young" = c(20, 60),
    "Old" = c(60, NA)
  )
)

query_tcga_group(
  cancer = "BRCA",
  group = "Age",
  merge_quantile = TRUE,
  merge_by = list(
    "Young" = c(0, 0.5),
    "Old" = c(0.5, 1)
  )
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyze_gene_drug_response.R
\name{analyze_gene_drug_response_asso}
\alias{analyze_gene_drug_response_asso}
\title{Analyze Association between Gene (Signature) and Drug Response with CCLE Data}
\usage{
analyze_gene_drug_response_asso(gene_list, combine = FALSE)
}
\arguments{
\item{gene_list}{a gene symbol list.}

\item{combine}{if \code{TRUE}, combine the expression of gene list as
a gene signature.}
}
\value{
a \code{data.frame}
\itemize{
\item If \code{combine} is \code{TRUE}, genes are combined as \code{signature}.
\item \code{mean.diff} and \code{median.diff} indicate mean and median of
normalized expression difference between High IC50 cells and Low IC50 cells.
The cutoff between High and Low are median IC50.
}
}
\description{
Analyze partial correlation of gene-drug association after
controlling for tissue average expression.
}
\examples{
\dontrun{
analyze_gene_drug_response_asso("TP53")
analyze_gene_drug_response_asso(c("TP53", "KRAS"))
analyze_gene_drug_response_asso(c("TP53", "KRAS"), combine = TRUE)

# Visualization
vis_gene_drug_response_asso("TP53")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_ccle_value.R
\name{vis_ccle_gene_cor}
\alias{vis_ccle_gene_cor}
\title{Visualize CCLE Gene Expression Correlation}
\usage{
vis_ccle_gene_cor(
  Gene1 = "CSF1R",
  Gene2 = "JAK3",
  data_type1 = "mRNA",
  data_type2 = "mRNA",
  cor_method = "spearman",
  use_log_x = FALSE,
  use_log_y = FALSE,
  use_regline = TRUE,
  SitePrimary = "prostate",
  use_all = FALSE,
  alpha = 0.5,
  color = "#000000",
  opt_pancan = .opt_pancan
)
}
\arguments{
\item{Gene1}{a molecular identifier (e.g., "TP53") or a formula specifying
genomic signature (\code{"TP53 + 2 * KRAS - 1.3 * PTEN"}).}

\item{Gene2}{a molecular identifier (e.g., "TP53") or a formula specifying
genomic signature (\code{"TP53 + 2 * KRAS - 1.3 * PTEN"}).}

\item{data_type1}{choose gene profile type for the first gene, including "mRNA","transcript","methylation","miRNA","protein","cnv_gistic2"}

\item{data_type2}{choose gene profile type for the second gene, including "mRNA","transcript","methylation","miRNA","protein","cnv_gistic2"}

\item{cor_method}{correlation method}

\item{use_log_x}{if \code{TRUE}, log X values.}

\item{use_log_y}{if \code{TRUE}, log Y values.}

\item{use_regline}{if \code{TRUE}, add regression line.}

\item{SitePrimary}{select cell line origin tissue.}

\item{use_all}{use all sample, default \code{FALSE}.}

\item{alpha}{dot alpha.}

\item{color}{dot color.}

\item{opt_pancan}{specify one dataset for some molercular profiles}
}
\value{
a \code{ggplot} object
}
\description{
Visualize CCLE Gene Expression Correlation
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_cross.R
\name{vis_pathway_cross_omics}
\alias{vis_pathway_cross_omics}
\title{Visualize cross-omics of one pathway among pan-cancers}
\usage{
vis_pathway_cross_omics(
  pw = "HALLMARK_ADIPOGENESIS",
  tumor_projects = NULL,
  tumor_samples = NULL,
  pval_mrna = c(0.05, 0.01, 0.001),
  return_list = FALSE
)
}
\arguments{
\item{pw}{pathway name}

\item{tumor_projects}{Select specific TCGA projects. Default NULL, indicating all TCGA projects.}

\item{tumor_samples}{Select specific tumor samples. Default NULL, indicating all tumor samples.}

\item{pval_mrna}{The P value thresholds}

\item{return_list}{TRUE returns a list including plot object and data. FALSE just returns plot.}
}
\value{
funkyheatmap
}
\description{
Visualize cross-omics of one pathway among pan-cancers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_pcawg_value.R
\name{vis_pcawg_unicox_tree}
\alias{vis_pcawg_unicox_tree}
\title{Visualize Single Gene Univariable Cox Result in PCAWG}
\usage{
vis_pcawg_unicox_tree(
  Gene = "TP53",
  measure = "OS",
  data_type = "mRNA",
  use_optimal_cutoff = FALSE,
  values = c("grey", "#E31A1C", "#377DB8"),
  opt_pancan = .opt_pancan
)
}
\arguments{
\item{Gene}{a molecular identifier (e.g., "TP53") or a formula specifying
genomic signature (\code{"TP53 + 2 * KRAS - 1.3 * PTEN"}).}

\item{measure}{a survival measure, e.g. "OS".}

\item{data_type}{choose gene profile type, including "mRNA","transcript","methylation","miRNA","protein","cnv"}

\item{use_optimal_cutoff}{use \code{surv_cutpoint} from survminer package for
thresholding samples in each cancer type.}

\item{values}{the color to fill tumor or normal}

\item{opt_pancan}{specify one dataset for some molercular profiles}
}
\value{
a \code{ggplot} object
}
\description{
Visualize Single Gene Univariable Cox Result in PCAWG
}
\examples{
\dontrun{
p <- vis_pcawg_unicox_tree(Gene = "TP53")
}
}
